function [Spatches] = i0100_IniPatches ...
    (Spatches, Npa, tFace, ccTp, Xtol, DUtol, Ks)

% RETURNS:
% Spatches object with first Refinement instance (Refin) defined
% rEskin: refined Eskins to capture shape and with GP's

pudim = 3 ;
xdim  = 3 ;

for pR = 1:1:Npa
    
    disp(  strcat( '03.3.', num2str(pR) ,' -  Patch ',  num2str(pR)) );


	% 1.- PATCH TRIMMING STATE, NGP IN EACH DIRECTION and GPT (if applies):
    
    trpF = Spatches(pR,1).trpF; % trimmed patch flag
    
    % No. Gauss points required in each direction:
    pKnotV  = Spatches(pR,1).INS(1).mKnotV;
    pUsize  = Spatches(pR,1).INS(1).mUsize;
    pDegree = Spatches(pR,1).INS(1).mDegree;
    
    [NGd] = rm0162_NRgp (pKnotV, pUsize, pDegree, pudim);
    
    if trpF == 0
        GPT = 0;
    elseif trpF == 1
        GPT = im0158_GPT (Spatches, pR, NGd);
    end
    
    %%%%%%%%%%%%%%%%%%% 
    % plots GPT
    %{

    
    fr = 99 ;
    pudim = 3 ;
    
    for e = 1:1:GPT.NE
        
        Anod = zeros(4,pudim);

        for i = 1:1:4
            nR = GPT.EVcon(e,i);
            Anod(i,:) = GPT.Av(nR,:);
        end
        
         pt1362_pTet (Anod, pudim, fr, 'k', 0.95, 1.2, 0);
        
    end
    
    if hh==0
    end

    %}
    

    
    % 2.- TRIANGULATION OF TRIMMING SURFACES => rTskin:

    if trpF == 0
        
        rTskin = 0 ;
        Nts = 0 ;
        tsR = 0 ;
        
        disp(  strcat( '        untrimmed patch '));
        
    elseif trpF == 1
        
        [rTskin, Nts, tsR] = st0301_rTskin (Spatches, pR, GPT, tFace, ccTp, Xtol, DUtol, Ks);
        
        disp(  strcat( '        trimming surfaces approximated '));
        
    end
    

    
    % 3.- SOLID MESH: tetrahedral or hexahedral mesh: 
    % almost complete, needs only to include icp's features for Gp's

    [IMesh] = im0141_IMesh (Spatches, pR, GPT, NGd, rTskin, ccTp, Nts, tsR, pudim, xdim, trpF, Xtol, DUtol);


    
    % 4.- GAUSS POINTS AND VERTEXES:
    % (excep J1 and dRx, that are updated in each loop of analysis)

    tol = 0.0001 ; % tolerance to neglect one cp influence
    
    % Gauss points:
    
    Ngp = IMesh.mNgp ;
    GP =  IMesh.mGP  ;

    [Spatches, tNon, tCPref, GP] = a0111_CPGon (Spatches, pR, Ngp, GP, tol);

    IMesh.mGP = GP ;

    
    % Skin vertexes: icp's and parameter space variables:
    
    [Spatches] = a0118_VPicp (Spatches, pR, tol);
    
    
    % 5.- ALLOCATION TO PATCHES:
    
    Spatches(pR,1).INS(1).mIM   = IMesh;
    
    Spatches(pR,1).INS(1).mNon  = tNon; 
    Spatches(pR,1).INS(1).mCPref = tCPref ;


end



% Patch Icp's in each B.E. skin integration point:
%[Eskin] = su2918_Pcp2Egp (Spatches, Npa, 1, Eskin);


end

