function [IN] = i0102_pIN (pr, tdm, P, MLT, dim, typ, SP, nd)

% RETURNS:
% IN = 1 if P is inside the domain (may use parameter or physical)

% INPUTS:
% pr: point reference (used for checking purposes)
% tdm: threshold distance to discriminate
% P(dim,1): point to evaluate

% next INPUTS come from MLT:
% NV:           No. vertexes
% TVcon, VTcon: conn. matrices
% Pv(NV,dim):   vertexes coordinates
% PNv(NV,dim), PNe(NE,dim): 
%               normals to vertexes and to elements (points towards outside the domain)
% typ:          type 'closed' Pr=0 is IN, 'open' Pr=0 is out


NV    = MLT.tNV;
    
LVcon = MLT.tLVcon;
VLcon = MLT.tVLcon;
    
TVcon = MLT.tTVcon;
VTcon = MLT.tVTcon;

LTcon = MLT.tLTcon;


if SP == "A" 

	Pv    = MLT.tAv;
	PNv   = MLT.tANv;
	PNe   = MLT.tANe;
    
elseif SP == "X" 
    
    Pv = MLT.tXv ;
    PNv = MLT.tXNv ;
    PNe = MLT.tXNe ;
    
end


% 1.- FIND CLOSEST LOCATION OF MLT & MEASURES DISTANCE, NORMAL AND PROJECTED POINT:

% Pp is the projected point on the triangle

[tve, tveR, dmin, Pp, nv] = i0104_closestP (pr, P, NV, LVcon, VLcon, TVcon, VTcon, PNv, PNe, Pv, dim, nd);

%{
if pr == 409

    P
    
    tve
    tveR
    dmin
    Pp
    nv
    
    fr = 6 ;

    figure(fr)
    
    lrF = 0 ;
    vrF = 0 ;
    erF = 1 ;

    pm0650_plotMLT (MLT, SP, fr, 'k', 1, 0, 0, 0, 0, lrF, vrF, erF, 0)

    scatter3(P(1), P(2), P(3),'r'); hold on
    
    %if gg==0
    %end

end

%}


% 2.- DETERMINES THRESHOLD DISTANCE:

if tve == "tri"
    
    D = MLT.tDtet(tveR,1);
    
elseif tve == "edg"
    
    D = 0 ;
    conta = 0 ;
    
    for i = 1:1:2
        
        r = LTcon(tveR,i);
        
        if r > 0 
            conta = conta + 1 ;
            D = D + MLT.tDtet(r,1);
        end
        
    end
    
    D = D/conta;
    
elseif tve == "ver"
    
    D = 0 ;
    conta = 0 ;
    
    for i = 1:1:size(VTcon,2)
        
        r = VTcon(tveR,i);
        
        if r > 0 
            conta = conta + 1 ;
            D = D + MLT.tDtet(r,1);
        end
        
    end
    
    D = D/conta;
    
end

td = D * tdm ;


% 3.- DETERMINES IF IS 'IN':

% 3.1.- Distance lower than thresold => removes it
% Except if the point is at one corner of the patch (prev: it said:
% "the point is at any limit of the patch")

P = round(P,3);

edF = 0 ; % 'edge' flag = No. dims in limits. Eg. 1 side, 2 edge, 3 corner

for di = 1:1:dim
    if P(di)==0 || P(di)==1
        edF = edF+1;
    end
end

if (dmin < td) && edF < 3  % edF <= 1
    
    IN = 0 ;
    return
    
end


% 3.2.- Other points further than td:
% Works out projection of d to nv ('Pr vector)

d = Pp-P; % vector from projected point to point

if norm(d) < 1*10^(-6)
    
    Pr = 0;
    
else
    
    d = d / norm(d);
    Pr = round(dot(d,nv),6);
    
end

   

if Pr > 0
    
    IN = 1 ;
    
elseif Pr==0
    
    if strcmp(typ,'closed')==1
        
        IN = 1 ;
        
    elseif strcmp(typ,'open')==1
        
        IN = 0 ;
        
    end
    
else
    IN = 0 ;
end

end

