function [tve, tveR, sdmin, Pp, nv] = i0104_closestP (pr, P, NV, LV, VL, TV, VT, PNv, PNe, Pv, dim, nd)

% RETURNS: closest location of skin to a point P
% tve: 'tri' or 'ver' or 'edg' if closest location to P lies in triangule, vertex or edge
% tveR: ref. of triangle, vert or edge
% sdmin: minimum distance measured
% Pp: point projected onto skin (might be vertex,on triangle or on edge)
% nv: normal vector at Pp, pointing outwards the domain

% INPUTS:
% pr: point reference (used for checking purposes)
% P(dim,1): coordinates of point to check
% NV: No. vertexes
% TV, VT: conn. matrices
% Nv: normals to vertexes
% Ne: normals to elems (triangles)
% Pv: surfaces vertexes coordinates


dminv = 9999999999999999 ;
dmine = 9999999999999999 ;
dmint = 9999999999999999 ;

% 1.- DETECTS THREE CLOSEST vtx 'cv':
% Already stores the closes vertex

vdist = zeros(NV,2); % vtx distance to P and reference

for v = 1:1:NV
    
    a = zeros(dim,1);
    a(:,1) = Pv(v,:);
    d = norm(a-P);
  
    vdist(v,1) = d;
    vdist(v,2) = v;
end

vdist = sortrows(vdist,1); % first vtxs are the closests ones
 
Rv = vdist(1,2) ;
dminv = vdist(1,1);
Ppv = zeros(dim,1);
Ppv(:,1) = Pv(Rv,:);
nvv(:,1) = PNv(Rv,:);

cv = zeros(3,1);
for i = 1:1:3
    cv(i,1) = vdist(i,2);
end



% 2.- PRE-SELECTS TRIANGLES & EDGES ATTACHED CLOSEST VTXs:

[Nt, ptR] = GEN_29_SelAtt (cv, VT);

cv = cv(1:2,:);  
[Nl, plR] = GEN_29_SelAtt (cv, VL);


% 3.- FINDS CLOSEST TRIANGLE:
% Measures perp. distance to each pre-triangle and if projections lies
% within the triangle or not:

nv = zeros(dim,1);
xp = zeros(dim,1);

Nti = 0 ; % No. triangles where Pp lies in
dwt = zeros(1,3); % distance / proj'd in triang (0-no 1-yes) / triag. ref
mPp = zeros(1,dim); % proj'd point coord's

for t = 1:1:Nt
    
    tR = ptR(t); % triangle ref.
    
    nv(:,1) = PNe(tR,:); % normal & point of triangle
    xp(:,1) = Pv(TV(tR,1),:);
    
    [Ppt] = go2518_POntoP(P, nv, xp);
   
    IN = i0105_PwithinT (Ppt, TV, tR, Pv, nv, dim);
    
    if IN > 0
        Nti = Nti + 1 ;
        
        dwt(Nti,1) = norm(Ppt-P);
        dwt(Nti,2) = IN;
        dwt(Nti,3) = tR;

        for di = 1:1:dim
            mPp(Nti,di) = Ppt(di);
        end
    end
    
end


if Nti > 0 % stores the closest triangle features

    edwt = [dwt mPp];

    edwt = sortrows(edwt,1); % sort by distance

    dwt = edwt(:,1:3);
    mPp = edwt(:,4:6);

    t = 1 ;
    tR = dwt(t,3);
    Ppt = zeros(dim,1);
    nvt = zeros(dim,1);
    
    Ppt(:,1) = mPp(t,:);     % projected point
    
    dmint = dwt(t,1);   % min. distance
    Rt    = dwt(t,3);   % triangle ref
    nvt(:,1) = PNe(tR,:);  % normal vertx
    

end

% 4.- FINDS CLOSEST EDGE:
 
Pr1 = zeros(dim,1);
Pr2 = zeros(dim,1);
vr = zeros(dim,1);
    
dmine = 999999 ;
    
slr = 0 ; % selected line ref
    
for i = 1:1:Nl
        
	r = plR(i);
        
	Pr1(:,1) = Pv(LV(r,1),:);
 	Pr2(:,1) = Pv(LV(r,2),:);

	vr(:,1) = Pr2(:,1)-Pr1(:,1);

	[rdis,d] = go2527_distPtoRecta (P, Pr1, vr, dim);

    if rdis < dmine
    
        rPp = P+d;
        
        L = norm(Pr2-Pr1);
        L1 = norm(rPp-Pr1);
        L2 = norm(rPp-Pr2);

        ON = 0 ;
        if L1<=L && L2<=L
            ON = 1;
        end

        if  ON == 1   
            dmine = rdis;
            slr = r ;
            Ppe = rPp;
        end
        
    end
        
end
    
    
% fills responses in case finds edge:
    
if slr > 0

	vr1 = LV(slr,1);
	vr2 = LV(slr,2);
        
	Pr1(:,1) = Pv(vr1,:);
 	Pr2(:,1) = Pv(vr2,:);

 	dTotal = norm(Pr1-Pr2);
 	N1 = norm(Ppe-Pr2)/dTotal;
  	N2 = norm(Ppe-Pr1)/dTotal;

 	nv(:,1) = N1*PNv(vr1,:) + N2*PNv(vr2,:); 

	Ppe = Ppe;      % projected point
    
    dmine = dmine; 	% min. distance
    Re    = slr;  	% line ref
    nve   = nv;     % normal vertx
        
end
    
 
    
% 5.- COMPARES:

if dminv <= dmine && dminv <= dmint

    tve = 'ver';
    tveR = Rv ;

    sdmin = dminv;
    Pp = Ppv;
    nv = nvv;

elseif dmine <= dminv && dmine <= dmint
       
	tve = 'edg';
    tveR = Re ;

    sdmin = dmine;
    Pp = Ppe;
    nv = nve;
    
elseif dmint <= dmine && dmint <= dminv
    
	tve = 'tri';
    tveR = Rt ;

    sdmin = dmint;
    Pp = Ppt;
    nv = nvt; 
    
end



end

