function [tve, tveR, sdmin, Pp, nv] = i0104_closestP (pr, P, NV, LV, VL, TV, VT, PNv, PNe, Pv, dim)

% RETURNS: closest location of skin to a point P
% tve: 'tri' or 'ver' or 'edg' if closest location to P lies in triangule, vertex or edge
% tveR: ref. of triangle, vert or edge
% sdmin: minimum distance measured
% Pp: point projected onto skin (might be vertex,on triangle or on edge)
% nv: normal vector at Pp, pointing outwards the domain

% INPUTS:
% pr: point reference (used for checking purposes)
% P(dim,1): coordinates of point to check
% NV: No. vertexes
% TV, VT: conn. matrices
% Nv: normals to vertexes
% Ne: normals to elems (triangles)
% Pv: surfaces vertexes coordinates


% 1.- DETECTS TWO CLOSEST vtx 'cv':

% 1.1.- Fist analysis by choosing just the three closest MLT nodes:

vdist = zeros(NV,2); % vtx distance to P and reference

for v = 1:1:NV
    
    a = zeros(dim,1);
    a(:,1) = Pv(v,:);
    d = norm(a-P);
  
    vdist(v,1) = d;
    vdist(v,2) = v;
end

vdist = sortrows(vdist,1); % first vtxs are the closests ones

cv1 = vdist(1,2);
cv2 = vdist(2,2);
cv3 = vdist(2,2);

tp1 = zeros(dim,1);
tp2 = zeros(dim,1);
tp3 = zeros(dim,1);

tp1(:,1) = Pv(cv1,:);
tp2(:,1) = Pv(cv2,:);
tp3(:,1) = Pv(cv3,:);

[CO1] = go2542_coincidentP (tp1, tp2, dim, 4);
[CO2] = go2542_coincidentP (tp2, tp3, dim, 4);

if CO1 == 1 || CO2 == 1 % for duplicate nodes, i.e. cv1=cv2 that may happen in closed surfaces, selects the next ones

    cv2 = vdist(3,2);
    cv3 = vdist(4,2);
    
end



% 2.- PRE-SELECTS TRIANGLES WITH CLOSEST VTXs:

ptR1 = zeros(1,1);
Nt1 = 0 ;

ptR2 = zeros(1,1);
Nt2 = 0 ;

ptR3 = zeros(1,1);
Nt3 = 0 ;

for i = 1:1:size(VT,2) % loop on vertexes
    
    if VT(cv1,i)>0 
        Nt1 = Nt1+1;
        ptR1(Nt1,1) = VT(cv1,i);
    end
    
    if VT(cv2,i)>0 
        Nt2 = Nt2+1;
        ptR2(Nt2,1) = VT(cv2,i);
    end
    
 	if VT(cv3,i)>0 
        Nt3 = Nt3+1;
        ptR3(Nt3,1) = VT(cv3,i);
    end
    
end

ptR = [ptR1 ; ptR2 ; ptR3];  % pre-selected triangles refs
[ptR,~] = GEN_16_RemoveRepeatedRows (ptR, 1, 0.1);
Nt = size(ptR,1);



% 3.- FINDS CLOSEST POINT (Pp)

% 3.1.- Measures perp. distance to each pre-triangle and if projections lies
% within the triangle or not:

dwt = zeros(Nt,3); % distance / proj'd in triang (0-no 1-yes) / triag. ref
mPp = zeros(Nt,dim); % proj'd point coord's

nv = zeros(dim,1);
xp = zeros(dim,1);

for t = 1:1:Nt
    
    tR = ptR(t); % triangle ref.
    
    nv(:,1) = PNe(tR,:); % normal & point of triangle
    xp(:,1) = Pv(TV(tR,1),:);
    
    [Ppt] = go2518_POntoP(P, nv, xp);
   
    IN = i0105_PwithinT (Ppt, TV, tR, Pv, nv, dim);
    
    dwt(t,1) = norm(Ppt-P);
    dwt(t,2) = IN;
    dwt(t,3) = tR;
    
    for di = 1:1:dim
        mPp(t,di) = Ppt(di);
    end
    
end


edwt = [dwt mPp];

edwt = sortrows(edwt,1); % sort by distance

dwt = edwt(:,1:3);
mPp = edwt(:,4:6);


% 3.2.- Check if Pp lies in any triangle. If so, that is the closest:

flagT = 0 ;
Pp = zeros(dim,1);

for t = 1:1:Nt
    
    tR = dwt(t,3);
    
    if dwt(t,2) == 1
        
        flagT = 1 ;
        
        tve = 'tri';
        tveR = dwt(t,3);
        
        sdmin = dwt(t,1);
        Pp(:,1) = mPp(t,:);
        nv(:,1) = PNe(tR,:); 
        
        break
        
    end 
    
end


% 3.3.- If not a triangle checks if is edge:

% Note: only checks edges contained within the VL matrix, i.e. existing
% edges within the MLT. This is to avoid considering non-existing edges.


if pr==79
    
    flagT 
    
end

flagE = 0 ;

if flagT == 0
    
    % Checks lines connected to cv1 and cv2
    
    plR = zeros(1,1); % pre-selects lines
    conta = 0 ;
    
    for i = 1:1:size(VL,2)
        
        r1 = VL(cv1,i);
        
        if r1 > 0
            conta = conta + 1 ;
            plR(conta,1) = r1;
        end
        
        r2 = VL(cv2,i);
        
        if r2 > 0
            conta = conta + 1 ;
            plR(conta,1) = r2;
        end
        
    end
    
    [plR,~] = GEN_16_RemoveRepeatedRows (plR, 1, 0.1);
    Nl = size(plR,1);
    
    
    % Finds the shortest distance edge:
    
   	Pp = zeros(dim,1);

   	Pr1 = zeros(dim,1);
   	Pr2 = zeros(dim,1);
   	vr = zeros(dim,1);
    
    dmin = 999999 ;
    
    slr = 0 ; % selected line ref
    
    for i = 1:1:Nl
        
        r = plR(i);
        
        Pr1(:,1) = Pv(LV(r,1),:);
        Pr2(:,1) = Pv(LV(r,2),:);

        vr(:,1) = Pr2(:,1)-Pr1(:,1);

        [rdis,d] = go2527_distPtoRecta (P, Pr1, vr, dim);

        rPp = P+d;

        ON = go2520_PonSegment (rPp, Pr1, Pr2, dim, 0.5);
        
        if rdis < dmin && ON == 1
            
            dmin = rdis;
            slr = r ;
            Pp = rPp;
        end
        
    end
    
    
    % fills responses in case finds edge:
    
 	if slr > 0
        
        flagE = 1 ;

      	tve = 'edg';
       	tveR = slr ;
            
     	sdmin = dmin;

        vr1 = LV(slr,1);
        vr2 = LV(slr,2);
        
       	Pr1(:,1) = Pv(vr1,:);
        Pr2(:,1) = Pv(vr2,:);

        dTotal = norm(Pr1-Pr2);
    	N1 = norm(Pp-Pr2)/dTotal;
     	N2 = norm(Pp-Pr1)/dTotal;

    	nv(:,1) = N1*PNv(vr1,:) + N2*PNv(vr2,:); 

	end
    
 
    
    %{
    % checks if both vtx are connected by one existing line:

    conF = 0 ; % connected flag
    
    
    % lines that start at cv1:
    
    conta = 0;
    LR = zeros(1,1);
    
    for i = 1:1:size(VL,2)
        
        if VL(cv1,i)>0
            conta = conta + 1 ;
            LR(conta,1) = VL(cv1,i);
        end
        
    end
    
    
    % checks in cv2 is at the other end:
    
    for i = 1:1:conta
        
        r = LR(i,1);
        
        if cv2 == LV(r,1) || cv2 == LV(r,2)
            
            conF = 1 ; % turns connected flag
            break
        end
        
    end
    

    
    if conF == 1
    
        Pp = zeros(dim,1);

        Pr1 = zeros(dim,1);
        Pr2 = zeros(dim,1);
        vr = zeros(dim,1);

        Pr1(:,1) = Pv(cv1,:);
        Pr2(:,1) = Pv(cv2,:);
        vr(:,1) = Pr2(:,1)-Pr1(:,1);

        [rdis,d] = go2527_distPtoRecta (P, Pr1, vr, dim);

        rPp = P+d;

        ON = go2520_PonSegment (rPp, Pr1, Pr2, dim, 0.5);

        if ON == 1

            flagE = 1 ;

            tve = 'edg';
            tveR = r ;
            
            sdmin = rdis;
            Pp(:,1) = rPp(:,1);

            dTotal = norm(Pr1-Pr2);
            N1 = norm(rPp-Pr2)/dTotal;
            N2 = norm(rPp-Pr1)/dTotal;

            nv(:,1) = N1*PNv(cv1,:) + N2*PNv(cv2,:); 

        end
    
    end
    
    %}
    
end




% 3.4.- If is not triangle and edge => is the vtx:

if flagT == 0 && flagE == 0
    
    Pp = zeros(dim,1);
     
   	tve = 'ver';
    tveR = cv1 ;
    
 	sdmin = vdist(1,1);
  	Pp(:,1) = Pv(cv1,:);
    nv(:,1) = PNv(cv1,:);
    
end



end

