function [Spatches, Non, CPref, GP] = a0111_CPGon (Spatches, pR, Ngp, GP, tol)

% RETURNS:
% control points related variables, including updated GPs of Tet-mesh with icps information

% 1.- PRELIMINARIES:

pudim = 3 ;

pNcp     = Spatches(pR,1).INS(1,1).mNcp ;
pDegree  = Spatches(pR,1).INS(1,1).mDegree ;
pKnotV   = Spatches(pR,1).INS(1,1).mKnotV ;
pw       = Spatches(pR,1).INS(1,1).mw ;

Nacp     = Spatches(pR,1).INS(1,1).mNacp ;
CP       = Spatches(pR,1).INS(1,1).mCP; 


% 2.- CPs STATE, GPs THEY HAVE INFLUENCE ON, DERIVATIVES wrt PARAM.:

gNicp = zeros(Ngp,1);
gRicp = zeros(Ngp,1); % "1" will grow

gBF   = zeros(Ngp,1); % influential B-spline basis functions value at each GP

gBFdir  = zeros(Ngp,1,pudim); % influential B-splines basis function values at each GP, but separated by pram. direction
gdBFdir = zeros(Ngp,1,pudim); % and 1st derivatives in each direction

for c = 1:1:Nacp
    
    aULims = zeros(pudim,2);
    aULims(:,:) = CP(c,1).mAcoLi(:,:);
    
    ON = 0 ;
    
    % conditions to be active: GP lies inside aULims and its [b.funct OR any derivative] are not 0 

    for g = 1:1:Ngp
        
        gAcoor = GP(g,1).mAcoor;

        IN = GEN_08_Orthodom_Pin_ndec (gAcoor, aULims, pudim, 'closed', 6);
        
        % lies inside:
        if IN == 1
            
            br = CP(c,1).mIX;

            % evaluates b. function and its derivatives:
            [d1Ni, Ni] = z2702_d1NML (gAcoor, br, pNcp, pDegree, pKnotV, pudim);
            
            % accounts only if any derivative Or fucntion > 0
            if sum(abs(d1Ni)) > tol || abs(prod(Ni)) > tol
                
                ON = 1 ;
                
                gNicp(g,1) = gNicp(g,1) + 1 ;
                gRicp(g,gNicp(g,1)) = c ;
                
                gBF(g,gNicp(g,1)) = prod(Ni) ;
                
                for di = 1:1:pudim
                    gdBFdir(g,gNicp(g,1),di) = d1Ni(di);
                    gBFdir (g,gNicp(g,1),di) = Ni(di);
                end
                
            end
            
        end
         
    end
    
    
    % allocates ON flag to cp:
    
    Spatches(pR,1).INS(1,1).mCP(c,1).ON = ON; 
    
    
end



% NURBS basis functions and their derivatives w.r.t. parameters at Gauss
% points. Also allocates values to GP:

for g = 1:1:Ngp
    
    % Extracts GP info:
    
    Nicp = gNicp(g,1);
    Ricp = gRicp(g,1:Nicp);
    BF   = gBF(g,1:Nicp);
    
    dBFdir = zeros(Nicp,pudim);
    BFdir = zeros(Nicp,pudim);
           
	for c = 1:1:Nicp
        dBFdir(c,:) = gdBFdir(g,c,:);
        BFdir(c,:)  = gBFdir(g,c,:);
	end
    
    % Computes Bf's and derivatives:
    
    [R, dRpa] = a0112_RdRgp (Nicp, Ricp, BF, BFdir, dBFdir, pw, pudim);

        
    % Allocates:
        
    GP(g,1).mNicp = gNicp(g,1);
    
    for i = 1:1:gNicp(g,1)
        
        GP(g,1).mRicp(i,1) = gRicp(g,i);
        GP(g,1).Rrel(i,1) = R(i);
        
        for di = 1:1:pudim
            GP(g,1).dRpa(i,di) = dRpa(i,di);
        end
    end

    
end




% 4.- NO. ACTIVE ANCHORS & POSITION FOR STIFFNESS MATRIX:

Non = 0 ;
CPref = zeros(Nacp,1);

for c = 1:1:Nacp
if Spatches(pR,1).INS(1,1).mCP(c,1).ON == 1
    
    Non = Non + 1 ;
    CPref(c,1) = Non ;
    
end
end




end