function [IMesh] = im0141_IMesh (Spatches, pR, GPT, NGd, rTskin, ccTp, Nts, tsR, pudim, pxdim, trpF, Xtol, DUtol)


% RETURNS:
% IMesh: integration mesh (with tetrahedrals) with all features except icp's on Gauss points

% INPUTS:
% trpF = 1 if patch is trimmed, 0 otherwise

% 0.- DECLARES IMESH AND EXTRACTS PATCHES FEATURES:

IMesh = cIMesh ;

pNcp     = Spatches(pR,1).INS(1).mNcp ;
pDegree  = Spatches(pR,1).INS(1).mDegree ;
pUsize   = Spatches(pR,1).INS(1).mUsize ;
pKnotV   = Spatches(pR,1).INS(1).mKnotV ;

pEP      = Spatches(pR,1).INS(1).mEP ;
pEw      = Spatches(pR,1).INS(1).mEw ;


% 1.- UN-TRIMMED SOLIDS: 'standard Gauss points arrangement'

if trpF == 0
    
    NE	= 0 ;
    
    connT = struct('LV',[],'EL',[],'SV',[],'ES',[],'VL',[],'VE',[],'EV',[]);
    THC = struct('Xv',[], 'Av',[], 'Uv',[], 'Bv',[], 'NCS',[], 'rm', []);
    
    connT.EV = 0 ;
    connT.LV = 0 ;
    connT.EL = 0 ;
    Nve   = 0 ;

    NV	= 0 ;
    NVm	= 0 ;
    THC.Av	= 0 ;
    THC.Xv	= 0 ;

    Eiem	= 0 ;
    
    [Ngp, GP] = rm0161_Rmesh (NGd, pDegree, pNcp, pKnotV, pUsize, pEw, pEP);

end


% 2.- TRIMMED SOLIDS

if trpF == 1

    nd = 3 ;
    
	% 2.1.- MLT: merged linear triangulations features
    % contains triangles of all trimming rTskin in a single structure.
    
    [MLT] = mt0601_MLT (pR, Spatches, rTskin, Nts, tsR); % refined skin for generating tetrahedrals
 
    %{
    
    fr = 4 ;
    pm0650_plotMLT (MLT, 'A', fr, 'k', 1, 1, 1, 0.1, 'r', 0, 0, 0, 0);

    x1 = [0 0 0 ; 0 0 1 ; 0 1 1 ; 0 1 0 ; 0 0 0 ];
    x2 = [1 0 0 ; 1 0 1 ; 1 1 1 ; 1 1 0 ; 1 0 0 ];
    x3 = [0 0 0 ; 1 0 0 ; 1 0 1 ; 0 0 1 ] ;
    x4 = [0 1 0 ; 1 1 0 ; 1 1 1 ; 0 1 1 ] ;
    
    plot3(x1(:,1), x1(:,2), x1(:,3), 'g'); hold on
    plot3(x2(:,1), x2(:,2), x2(:,3), 'g'); hold on
    plot3(x3(:,1), x3(:,2), x3(:,3), 'g'); hold on
    plot3(x4(:,1), x4(:,2), x4(:,3), 'g'); hold on
    
    if gg==0
    end
    
    %} 

    % 2.2.- DELETE PRELIM. NODES THAT ARE OUTSIDE TTR & ADDS FROM TTR:

    tdm = 1.33 ; % multiplier for threshold distance to discriminate

    [THC, THCea] = im0146_SetAllNods (trpF, GPT, MLT, tdm);


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % nodes and MLT:
    %{

    if pR == 1
    
        fr = 3 ;
    
        figure(fr)
        
        
        % all nodes from GPT:
        
        gAv = GPT.Av ;
        
        for i = 1:1:GPT.Nn
            % scatter3(gAv(i,1), gAv(i,2), gAv(i,3), 'k'); hold on
        end
        
        
        % nodes IN:
        
        mAv = THC.Av;
        
        for i = 1:1:size(mAv,1)
            scatter3(mAv(i,1), mAv(i,2), mAv(i,3), 'k', 'filled'); hold on
        end
    
        
        % external auxiliary: NOT USED
        %{
        mAea = THCea.Av;
        
        for i = 1:1:size(mAea,1)
            %scatter3(mAea(i,1), mAea(i,2), mAea(i,3), 'm', 'filled'); hold on
        end
        %}
        
        
        % MLT:
        pm0650_plotMLT (MLT, 'A', fr, 'g', 1.1, 0, 0, 0, 0, 0, 0, 0, 0)
      

        axis equal

        if gg==0
        end
        
    end
        
    %}
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 	

    % 2.3.- TETRAHEDRAL MESH:

    [NE, NV, connT, Nve, THC, NVm, Eiem] = im0147_TetMesh ...
  	(pR, Spatches, rTskin, THC, THCea, MLT, ccTp, Xtol, DUtol, nd);



    % 2.4.- GAUSS POINTS (linear for 4 nodes, quadratic for more).
    % Still needs icp's, that is done alongside with anchors.

    [Ngp, GP] = im0144_patchGPs (NE, connT.EV, Nve, Eiem, THC.Av, pKnotV, pNcp, pDegree, pUsize, pEw, pEP, pudim, pxdim);


    
end


% 3.- ALLOCATES TO IMESH:

IMesh.mNE       = NE ;
IMesh.mEVcon    = connT.EV ;
IMesh.mLVcon    = connT.LV ;
IMesh.mELcon    = connT.EL ;
IMesh.mNve      = Nve ;

IMesh.mNV       = NV ;
IMesh.mNVm      = NVm ;
IMesh.mAcoor    = THC.Av ;
IMesh.mXcoor    = THC.Xv ;

IMesh.iem       = Eiem ;

IMesh.mNgp      = Ngp ;
IMesh.mGP       = GP ;




end

