function [Ncn, Xcn] = im0143_DivideCell (Rn, vx, xdim, ref)

% RETURNS:
% Xnc(Ncn,xdim): coordinates of nodes inside the cell

% INPUTS:
% Rn: nominal distance between nodes
% vx(8,xdim): vtx coord's, hierarchically

% 1.- FIND LOCAL BASIS OF THAT CELL:
% orthogonal basis rotated to adjust directions at mid point of the cell:

% 1.1.- Mid-points of pairs of faces 1, 2 and 3:

mp = zeros(3,2,xdim);

% face-vtx connection:
FVcon = [ 1 3 5 7 ; 1 2 5 6 ; 1 2 3 4 ; 2 4 6 8 ; 3 4 7 8 ; 5 6 7 8 ];

for f = 1:1:3
    
    xacc1 = zeros(xdim,1);
    xacc2 = zeros(xdim,1);
    
    for i = 1:1:4  
        for di = 1:1:xdim
            xacc1(di,1) = xacc1(di,1) + vx(FVcon(f,i),di);
            xacc2(di,1) = xacc2(di,1) + vx(FVcon(f+3,i),di); 
        end
    end
    
    mp(f,1,:) = xacc1(:,1)/4;
    mp(f,2,:) = xacc2(:,1)/4;
end


% 1.2.- Three versors of cell directions ("cell-base") 
% & direction of basis ("diagonal of the paralelogram") or bisector:

cv = zeros(3,xdim); % one vector per row
db = zeros(xdim,1);

for di = 1:1:xdim
   
    v = zeros(xdim,1);
    v(:,1) = mp(di,2,:) - mp(di,1,:);
    v = v / norm(v);
    
    cv(di,:) = v(:,1);
    
    db(:,1) = db(:,1) + v(:,1);
end

db = db/norm(db);



% 1.3.- Octaedral plane for local basis:

np = db; % normal to plane
xp = 0.5774*np; % plane point and intersection with bisector


% 1.4.- Projection of cell-base onto plane and vectors projected on this plane:

P = zeros(xdim,1);
cvp = zeros(3,xdim); % one vector per row

for di = 1:1:3
    
    P(:,1) = cv(di,:);
    Pp = go2518_POntoP(P, np, xp);
    
    P(:,1) = Pp-xp;
    
    P = 0.8165 * P/norm(P); % lengh that must have to coincide with O-base circumference
    cvp(di,:) = P(:,1);
end


% 1.5.- Find out projected angles
% (the first vector is assumed to be 0 radians)

v1 = zeros(xdim,1);
v2 = zeros(xdim,1);

cpa =zeros(3,1); % cell-base projected angles

v1(:,1) = cvp(1,:);
v2(:,1) = cvp(2,:);
cpa(2,1) = acos(dot(v1,v2)/(norm(v1)*norm(v2)));

v1(:,1) = cvp(2,:);
v2(:,1) = cvp(3,:);
ag = acos(dot(v1,v2)/(norm(v1)*norm(v2)));
cpa(3,1) = cpa(2,1) + ag;


% 1.6.- Computes angle of adjusted ortho-base:

opa = zeros(3,1); % O-base projected angle

opa(1,1) = sum(cpa)/3;
opa(2,1) = opa(1) + 120/57.298;
opa(3,1) = opa(2) + 120/57.298;


% 1.7.- Location of points rotated with R radius and opa angles w.r.t.
% cpa(1st vector) on circumference on oct-plane

C = xp ;
Rc = 0.8165 ;

v1 = zeros(xdim,1);
v1(:,1) = cvp(1,:);
v1 = v1/norm(v1);
v2 =cross(np,v1);
v2 = v2/norm(v2);

ovp = zeros(3,xdim);

for i = 1:1:3
    
    ag = opa(i);
    
    for di = 1:1:xdim
        ovp(i,di) = C(di) + Rc*cos(ag)*v1(di) + Rc*sin(ag)*v2(di) ;
    end
    
end

% 1.8.- O-base vectors of that cell are given directly by those points:

% each row of Ov is the O-base versor, i.e. Ov is the rotation matrix
Ov = ovp;




% 2.- GENERATES A NET OF R-REGULAR SPACED NODES ORIENTATED AS Ov 

% 2.1.- Rotates cell vtxs acc to Ov:

rvx = zeros(8,xdim);
T = Ov;

for i = 1:1:8
    
    v1(:,1) = vx(i,:);
    vr1 = T*v1;
    rvx(i,:) = vr1(:,1);
end


% onwards works with rotated coordinates

% 2.2.- Planes and contours:

PL(6,1) = struct('n',[], 'P', [], 'xc', []); % normal, point of plane, contour coord's leaving valid plane at LHS

vR = [ 1 3 7 5 1 ; 1 5 6 2 1 ; 1 2 4 3 1 ; 2 6 8 4 2 ; 3 4 8 7 3 ; 5 7 8 6 5 ]; % vtx for contour leaving at LHS plane
nvR = [ 1 3 1 5 ; 1 5 1 2 ; 1 2 1 3 ; 8 4 8 6 ; 8 7 8 4 ; 8 6 8 7]; % vtx for v1 and v2 for nv = v1 x v2

v1 = zeros(xdim,1);
v2 = zeros(xdim,1);
Pt = zeros(xdim,1);


for i = 1:1:6
    
    v1(:,1) = rvx(nvR(i,2),:)-rvx(nvR(i,1),:);
    v2(:,1) = rvx(nvR(i,4),:)-rvx(nvR(i,3),:);
    
    nt = cross(v1,v2);
    nt = nt/norm(nt);
    
    Pt(:,1) = rvx(vR(i,1),:);
    
    xct = zeros(5,xdim);
    for v = 1:1:5 
        xct(v,:) = rvx(vR(i,v),:);
    end
    
    PL(i,1).n = nt;
    PL(i,1).P = Pt;
    PL(i,1).xc = xct;
end




% 2.2.- Find envelope coordinates and segments in each direction:

XLims = zeros(xdim,2);
sg = zeros(3,1); % one will increase
Nps = zeros(xdim,1); % No. points per segment
Re = zeros(3,1); % step in each direction

for di=1:1:3
    
    x = rvx(:,di);
    
    XLims(di,1) = min(x);
    XLims(di,2) = max(x);
    
    x1 = min(x);
    x2 = max(x);
    Nmin = 2 ;
    [N, sd] = go2531_DivSegment (x1, x2, Nmin, Rn, 1, 1);
    
    Nps(di) = N;
    
    for i = 1:1:N
        sg(di,i) = sd(i);
    end
    
    Re(di) = sd(2)-sd(1);
    
end


Dtol = min(Re)*0.05; % to reject nodes if they are very close

% 2.3.- Computes envelope net nodes coordinates & those ones IN:

Xen = zeros(2,xdim);
Xin = zeros(2,xdim);

P = zeros(xdim,1);

conta = 0 ;
contaIN = 0;

for i = 1:1:Nps(1)-1
for j = 1:1:Nps(2)-1
for k = 1:1:Nps(3)-1
    
    conta = conta + 1 ;
    
    Xen(conta,1) = sg(1,i)+Re(1)/2; % note shifts half L
    Xen(conta,2) = sg(2,j)+Re(2)/2;
    Xen(conta,3) = sg(3,k)+Re(3)/2;
    
    % checks if lies within the cell, uses closest point and closest plane:
    
    IN = 0 ;
    
    P(:,1) = Xen(conta,:);
    
    dmin = 999999999;
    for f = 1:1:6
        
        n = PL(f).n;
        xp = PL(f).P;

        [Pp] = go2518_POntoP(P, n, xp);
        
        r = Pp-P;
        d = norm(r);
        
        if d < dmin
            dmin = d ;
            cpR = f ;
            cr = r ;
            cPp = Pp;
        end
    end
    
    % distance check
    if Dtol < dmin

        cr = cr/norm(cr);
        ag1 = dot(cr,n);

        % normal check
        if ag1 < 90/57.3

            % contour check
            v1 = zeros(xdim,1);
            v2 = zeros(xdim,1);
            c = 1;

            v1(:,1) = PL(cpR).xc(c+1,:)- PL(cpR).xc(c,:);
            
            for di = 1:1:xdim
                v2(di) = cPp(di,1)- PL(cpR).xc(c,di);
            end

          	nt = cross(v1,v2);
            nt = nt/norm(nt);

            ag2 = dot(nt,n);
            
            % contour check
            if ag2 < 90/57.3
                IN = 1 ;
            end

        end
    end


    % add to nodes inside if IN=1, rotates to global axis
    
    if IN == 1   
        contaIN = contaIN+1;
        
        P = T'*P;
        Xin(contaIN,:) = P(:,1);
        
    end
    
    
end 
end
end


Ncn = contaIN;
Xcn = Xin;


%{

if ref == 1 || ref == 12

    figure(ref)

    %scatter3(vx(:,1),vx(:,2),vx(:,3),'k'); hold on
    %scatter3(rvx(:,1),rvx(:,2),rvx(:,3),'r'); hold on

    r = [1 2 4 3 1];
    for i = 1:1:4
        plot3( [ rvx(r(i),1) rvx(r(i+1),1)], [ rvx(r(i),2) rvx(r(i+1),2)], [ rvx(r(i),3) rvx(r(i+1),3)], 'b'); hold on
    end

  	r = [5 6 8 7 5];
    for i = 1:1:4
        plot3( [ rvx(r(i),1) rvx(r(i+1),1)], [ rvx(r(i),2) rvx(r(i+1),2)], [ rvx(r(i),3) rvx(r(i+1),3)], 'b'); hold on
    end

 	r = [2 6 8 4 2];
    for i = 1:1:4
        plot3( [ rvx(r(i),1) rvx(r(i+1),1)], [ rvx(r(i),2) rvx(r(i+1),2)], [ rvx(r(i),3) rvx(r(i+1),3)], 'b'); hold on
    end
    
 	r = [1 5 7 3 1];
    for i = 1:1:4
        plot3( [ rvx(r(i),1) rvx(r(i+1),1)], [ rvx(r(i),2) rvx(r(i+1),2)], [ rvx(r(i),3) rvx(r(i+1),3)], 'b'); hold on
    end
    
    scatter3(Xin(:,1), Xin(:,2), Xin(:,3), 'r'); hold on
    
    axis equal

end


if GG==2
    
end

%}


% 2.1.- Envelope 






end

