function [THC, THCea] = im0146_SetAllNods (trpF, GPT, MLT, tdm)

% RETURNS:
% coordinates of nodes that are inside the MLT, i.e. nodes valid for tetrahedralization
% THC.Xv, Av, Uv, Bv: physical, patch-param, surf-param, curve-param

% THCea are the external auxiliary nodes used to force the
% tetrahedralization to comply with MLT triangles.


THC = struct('Xv',[], 'Av',[], 'Uv',[], 'Bv',[], 'NCS',[], 'rm', [], 'ANv', []);
THCea = struct('Av',[]);

% rm(NV,1) is the corresponding node ref in the MLT

pNn = GPT.Nn ;
pXv = GPT.Xv ;
pAv = GPT.Av ;
pLv = GPT.Lv ; % average lengths attached to each node (para. and physical spaces)

xdim = 3 ;
pudim = 3 ;
sudim = 2 ;
cudim = 1 ;

nd = 3 ;

pULims = [ 0 1 ; 0 1 ; 0 1 ] ;


if trpF == 0
    
    % 1.- UN-TRIMMED PATCHES:
    
    NV = size(pXv,1);
    
    THC.Xv = pXv;
    THC.Av = pAv;
    THC.Uv = zeros(NV,sudim);
    THC.Bv = zeros(NV,cudim);
    
    THC.NCS = zeros(NV,2);
    
    THC.rm  = zeros(NV,1);
    
    THC.ANv = zeros(NV,pudim);
    
    
elseif trpF == 1

    
    % 2.- FOR TRIMMED PATCHES:
    
    % 2.1.- Delete the GPT nodes outside MLT:
    
    Bnod = zeros(1,cudim);
    Unod = zeros(1,sudim);
    Anod = zeros(1,pudim);
    Xnod = zeros(1,xdim);
    NCSi = zeros(1,2); % curve and surfaces refs
    ANvi = zeros(1,pudim);
    
    Nn1 = 0 ;

    Acoor = zeros(pudim,1);
    Xcoor = zeros(xdim,1);
    
    for i = 1:1:pNn
    
        Acoor(:,1) = pAv(i,:); % candidate
        Xcoor(:,1) = pXv(i,:);

        % Examines if is IN:
        IN = i0102_pIN (i, tdm, Acoor, MLT, pudim, 'open', 'A', nd);

        % Allocates:
        if IN==1
            
            Nn1 = Nn1 + 1 ;
            
            Bnod(Nn1,1) = 0;
            Unod(Nn1,1) = 0; Unod(Nn1,2) = 0;
            Anod(Nn1,:) = Acoor(:,1);
            Xnod(Nn1,:) = Xcoor(:,1);
            
            NCSi(Nn1,1) = 0 ;
            NCSi(Nn1,2) = 0 ;
            
            ANvi(Nn1,:) = [0 0 0];
            
        end
   
    end


    
    % 2.2.- Delete the AtetIN nodes outside MLT:
    
  	ATBv = zeros(1,cudim);
    ATUv = zeros(1,sudim);
    ATAv = zeros(1,pudim);
    ATXv = zeros(1,xdim);
    ATNCS = zeros(1,2); % curve and surfaces refs
    ATANv = zeros(1,pudim);
    
    Nn2 = 0 ;
    
    for e = 1:1:MLT.tNE
        
    	Acoor(:,1) = MLT.tAtetIN(e,:); % candidate
        Xcoor(:,1) = MLT.tXtetIN(e,:);
        
        % Examines if is IN:
        IN1 = i0102_pIN (e, 0, Acoor, MLT, pudim, 'open', 'A', nd);

        % Examines if is within patch limits:
        IN2 = GEN_08_Orthodom_Pin_ndec (Acoor, pULims, pudim, 'opened', nd);
        
        % Allocates:
        if IN1 == 1 && IN2 == 1
            
            Nn2 = Nn2 + 1 ;

            ATBv(Nn2,1) = 0;
            ATUv(Nn2,1) = 0; ATUv(Nn2,2) = 0;
            ATAv(Nn2,:) = Acoor(:,1);
            ATXv(Nn2,:) = Xcoor(:,1);
            
            ATNCS(Nn2,1) = 0 ;
            ATNCS(Nn2,2) = 0 ;
            
            ATANv(Nn2,:) = [0 0 0];
            
        end
    end
    

    
    
    % 2.3.- Add MLT nodes at the beggining, GPT in between, and AtetIN nodes at the end
    
    Bv	= MLT.tBv;
	Uv  = MLT.tUv;
	Av	= MLT.tAv;
 	Xv  = MLT.tXv;
        
	NCS = MLT.tNCS ; 
    rm = [1:1:MLT.tNV]';
	ANv = MLT.tANv ;
    
    
    if Nn1 > 0
        
        Bv = [ Bv ; Bnod ];
        Uv = [ Uv ; Unod ];
        Av = [ Av ; Anod ];
        Xv = [ Xv ; Xnod ];
        
        NCS = [NCS ; NCSi ];
        rm  = [rm ; zeros(Nn1,1)];
        ANv = [ANv ; ANvi ];
    end
    
% Nn2 = 0 ; %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if Nn2 > 0
        
        Bv = [ Bv ; ATBv ];
        Uv = [ Uv ; ATUv ];
        Av = [ Av ; ATAv ];
        Xv = [ Xv ; ATXv ];
        
        NCS = [NCS ; ATNCS ];
        rm  = [rm ; zeros(Nn2,1)];
        ANv = [ANv ; ATANv ];
    end
    

    % 2.4.- External Auxiliary nodes: NOT USED
    % They must lie outside!
    %{
    Aea = zeros(1,pudim);
    Nnea = 0 ;
    
    for e = 1:1:MLT.tNE
        
    	Acoor(:,1) = MLT.tAtetOU(e,:); % candidate
        
        Nnea = Nnea + 1 ;
      	Aea(Nnea,:) = Acoor(:,1);
        
        
        % Examines if is OUT:
        IN = i0102_pIN (e, 0, Acoor, MLT, pudim, 'open', 'A');

        % Allocates:
        if IN == 0
            Nnea = Nnea + 1 ;
            Aea(Nnea,:) = Acoor(:,1);
        end
        
    end
    %}
    

    % 2.5.- Allocates:
    
	THC.Xv = Xv;
    THC.Av = Av;
    THC.Uv = Uv;
    THC.Bv = Bv;
    
    THC.NCS = NCS;
    THC.rm  = rm;
    THC.ANv = ANv;
    
    %THCea.Av = Aea;
    THCea = 0 ;
    
end






end