function [NE, NV, connT, Nve, THC, NVm, Eiem] = im0147_TetMesh ...
            (pR, Spatches, rTskin, THC, THCea, MLT, ccTp, Xtol, DUtol, nd)

% RETURNS:
% NE: No. elements
% EVcon: conn. matrix element => vertex
% Nve(NE,1): No. vertexes per element
% THC: tets coordinates
% Nvm: No. vertexes considering inserted mid-nodes
% Eiem matrices


agCurv = ccTp(pR).agCurv;

pudim = 3 ;
xdim  = 3 ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
figure(81)

NV = size(THC.Av,1);
n = zeros(pudim,1);
sc = 0.2 ;

for i = 1:1:NV
    
    if THC.NCS(i,2)>0
        
        n(:,1) = sc*THC.ANv(i,:);
        
        scatter3( THC.Av(i,1) , THC.Av(i,2) ,THC.Av(i,3) ,'r'); hold on
        quiver3( THC.Av(i,1) , THC.Av(i,2) ,THC.Av(i,3) , n(1),n(2),n(3),'b'); hold on
    end
    
    
end

axis equal

if gg==8
end
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% 1.- PRELIMINARY TRIANGULATION:

%mAv = [THC.Av ; THCea.Av]; 
mAv = THC.Av;

DT = delaunayTriangulation(mAv);

pEVcon  = DT.ConnectivityList; % conn. matrix element => vertex
pNE     = size(pEVcon,1);      % No. elements


NV = size(mAv,1);
connT = th1333_oConnL (pNE, NV, pEVcon); 



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plots linear tets:
%{
if pR ==1
    fr = 15 ; %%%%%%%%%%%%%%%%%%%%%%%%%

    %pm0650_plotMLT (MLT, 'A', fr, 'g', 4.2, 0, 0, 0, 0, 0, 0, 0, 0);
    
    Lthk = 1 ;
    transp = 0.33 ;
    lrF = 0 ; % line ref. flag

    Cv = THC.Av ;
    NLI = size(connT.LV,1);
    LVcon = connT.LV;

    pt1368_pTetLinMesh (Cv, NLI, LVcon, fr, Lthk, transp, lrF)

    axis equal
end
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 2.- ENSURES THAT MLT lines are MATCH IN THE TETRAHEDRALIZATION:

pL = 2.5 ; % percentage of av. line length of tet. to consider a new point too close to one opposite side

[pNE, connT, THC, MLT] = th1336_surLines ...
(pR, Spatches, rTskin, pNE, connT, THC, MLT, agCurv, pL, Xtol, DUtol);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Triangulation after MLT lines fitting
%{
if pR == 1
    fr = 700 ; %%%%%%%%%%%%%%%%%%%%%%%%%

    pm0650_plotMLT (MLT, 'A', fr, 'g', 4.2, 0, 0, 0, 0, 0, 0, 0, 0);
    
    Lthk = 1 ;
    transp = 0.33 ;
    lrF = 0 ; % line ref. flag

    Cv = mAv ;
    NLI = size(connT.LV,1);
    LVcon = connT.LV;

    pt1368_pTetLinMesh (Cv, NLI, LVcon, fr, Lthk, transp, lrF)
    
    if kk==0
    end

end
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NV = size(THC.Av,1);


% 3.- DELETE TRETRAHEDRONS THAT ARE QUASI-PLANE (in parameter space):

% Compares its current volume 'Vc' against the volume of the regular tetrahedron
% whose side length a = average(sides) 'Va'. If Va/Vc< treshold disregard that tet.
% Works in parameter space.
% Also, returns tet's centre in physical and parameter spaces.

EVcon = connT.EV;
NE = size(EVcon,1);

Vthr = 1/50 ;  % 50 times smaller volume
[NE, EVcon] = th1331_delPlanes (Vthr, NE, EVcon, THC);

[xCt, aCt] = tm1314_Ctets (NE, EVcon, THC);


%%%%%%%%%%%%%%%%%%%
% MLT and lines from tets
%{
    fr = 14 ; %%%%%%%%%%%%%%%%%%%%%%%%%

    figure(fr)
    
    pm0650_plotMLT (MLT, 'A', fr, 'g', 4.2, 1, 1, 0.25, 0, 0, 0, 0, 0);
    
    Cv = THC.Av ;
    nodON = 0 ;
    
    dim = size(Cv,2);
    NLI = size(connT.LV,1);
    
    for L = 1:1:NLI
        
        n1 = connT.LV(L,1);
        n2 = connT.LV(L,2);
        
        Cnod = zeros(2,dim);
        
        if THC.rm(n1) > 0 && THC.rm(n2) > 0

            Cnod(1,:) = Cv(n1,:);
            Cnod(2,:) = Cv(n2,:);
            
            pt1364_pLin (Cnod, 1, 2, fr, 'k', nodON, dim);
            
        end
        
    end

    if gg==0
    end
    
%}
%%%%%%%%%%%%%%%%%%%


% 4.- REMOVE TETS OUTSIDE MTL:

[NE, EVcon,~,~] = im0152_delMLTout (MLT, NE, EVcon, THC.Av, THC.Xv, xCt, aCt, nd);

NV = size(THC.Xv,1);
[connT] = th1333_oConnL (NE, NV, EVcon);




%%%%%%%%%%%%%%%%%%%
% MLT and lines from tets
%{
    fr = 17 ; %%%%%%%%%%%%%%%%%%%%%%%%%
    figure(fr)
    
    for i = 1:1:size(connT.LE,2)
    
        r = connT.LE(819,i);
        if r > 0
            % pt1373_pTetConn (r, connT, THC.Av, fr,'r', 1, 1, 0, 0)
        end
    end
    

    pm0650_plotMLT (MLT, 'A', fr, 'g', 4.2, 0, 0, 0.22, 0, 0, 0, 0, 0);
    
    Cv = THC.Av ;
    nodON = 0 ;
    
    dim = size(Cv,2);
    NLI = size(connT.LV,1);
    
    for L = 1:1:NLI
        
        n1 = connT.LV(L,1);
        n2 = connT.LV(L,2);
        
        Cnod = zeros(2,dim);
        Ca = zeros(1,pudim);
        
        if THC.rm(n1) > 0 && THC.rm(n2) > 0
            
            Cnod(1,:) = Cv(n1,:);
            Cnod(2,:) = Cv(n2,:);
            
            pt1364_pLin (Cnod, 1, 2, fr, 'k', nodON, dim);
            
            Ca(1,:) = (Cv(n1,:) + Cv(n2,:))/2;
            
            %text( Ca(1), Ca(2), Ca(3), num2str(L)); hold on
            
        end
        
        
        
    end

    % at im0152_delMLTout looses the edges !! CHECK IT OUT
    
    if gg==0
    end
    
%}
%%%%%%%%%%%%%%%%%%%



% 5.- UPDATES INSERTING MID-VTX TO CONSIDER SHAPING SURFACES:
% Recovers coordinates from rTskin

NVo = size(THC.Av,1);

[connT, THC, NVm, Eiem, Nve] = th1337_upQuad ...
(connT, NE, THC, Spatches, pR, MLT, rTskin, ccTp, Xtol, DUtol);


%%%%%%%%%%%%%%%%%%%
% initial MQT
%{
    fr = 22 ; %%%%%%%%%%%%%%%%%%%%%%%%%

    figure(fr)
    
    Cv = THC.Xv ;
    nodON = 0 ;
    
    dim = size(Cv,2);
    NLI = size(connT.LV,1);
    
    for L = 1:1:NLI
        
        nR = connT.LV(L,:);
        
        Cnod = zeros(3,dim);
        
        if nR(3) > 0
            
            for i = 1:1:3
                Cnod(i,:) = Cv(nR(i),:);
            end
            
            pt1364_pLin (Cnod, 0.25, 3, fr, 'k', nodON, dim);
            
        end
        
    end
    
    
    % nodes refs:
    
    %{
    
    NV = size(THC.Av,1);
    nr = zeros(1,dim+1); % coords of the node and its ref

    conta = 0;
    for i = 1:1:NVo
        
        if THC.rm(i,1) > 0
            
            conta = conta+1;
            
            for di = 1:1:dim
                nr(conta,di) = Cv(i,di);
            end
            
            nr(conta,di+1) = i;

        end
    end
    
    
    for i = NVo+1:1:NV
        
        conta = conta + 1 ;
        
      	for di = 1:1:dim
        	nr(conta,di) = Cv(i,di);
        end
            
    	nr(conta,di+1) = i;
        
    end
    
    
    
    for i = 1:1:size(nr,1)
        if dim == 2

            scatter ( nr(i,1), nr(i,2), 'r', 'Filled'); hold on
            text ( nr(i,1), nr(i,2), num2str(nr(i,dim+1)) ); hold on

        elseif dim == 3

            scatter3 ( nr(i,1), nr(i,2), nr(i,3), 'r', 'Filled'); hold on
            text ( nr(i,1), nr(i,2), nr(i,3), num2str(nr(i,dim+1)) ); hold on

        end
    end
    
    %}

    

%}
%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initial quadratic tets:
%{


    fr = 11 ; %%%%%%%%%%%%%%%%%%%%%%%%%

    figure(fr)
    % scatter3(THC.Av(:,1),THC.Av(:,2),THC.Av(:,3),'r'); hold on
    %scatter3(THC.Av(79,1),THC.Av(79,2),THC.Av(79,3),'r'); hold on
    
    sp = 'a';
    stL = 0.1 ;
    
    Mcolr = 'k';

    Cv = THC.Av ;
    NLI = size(connT.LV,1);
    LVcon = connT.LV;

    lrF = 0 ;
    
    pKnotV = Spatches(pR).INS(1).mKnotV;
    pNcp   = Spatches(pR).INS(1).mNcp;
    pDegree = Spatches(pR).INS(1).mDegree;
    pUsize  = Spatches(pR).INS(1).mUsize;
    pEw  = Spatches(pR).INS(1).mEw;
    pEP  = Spatches(pR).INS(1).mEP;
    
    pt1370_pTetMDMesh (sp, stL, Cv, NLI, LVcon, pKnotV, pNcp, pDegree, pUsize, pEw, pEP, fr, Mcolr, lrF);
    
    sp = 'x';
    fr = fr+1 ;
    pt1370_pTetMDMesh (sp, stL, Cv, NLI, LVcon, pKnotV, pNcp, pDegree, pUsize, pEw, pEP, fr, Mcolr, lrF);
    
    if gg==0
    end

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plots lines on T-surface
%{

figure(33)

NL = size(connT.LV,1);

for L = 1:1:NL
    
    if connT.LV(L,3) > 0
        
        x = zeros(2,pudim);
        
        for di = 1:1:pudim
            
            x(1,di) = THC.Av(connT.LV(L,1),di);
            x(2,di) = THC.Av(connT.LV(L,3),di);
            
        end
        
        
        plot3(x(:,1), x(:,2), x(:,3), 'b'); hold on
        
        
        
    end
    
    
end


axis equal

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 6.- CORRECTS THOSE LINES THAT INTERSECTS WITH rTskins ('self-intersection'):


NLI = size(connT.LV,1);

[THC, connT] = th1380_tSc (pR, Spatches, NLI, NE, THC, connT, NVm, Eiem, Nve, MLT);

%if gg==0
%end



end


