function [Nn, NE, Cn, EVcon, DT] = im0155_untrimTets (NR, ULims, V, Vsize)

udim = 3 ;

% RETURNS: tetrahedralization of untrimmed patch
% NN, NE: No. nodes and No. elements
% Cn(Nv,udim): Nodes coordinates
% EVcon(NE,4): connectivity matrix

% INPUTS:
% NR(udim,1); No. required GPs in each direction
% ULIms(di,2): parameter limits in each direction
% V(udim,Vsize): non-repeated knot vectors
% Vsize(udim,1): No. non-repeaterd knot vectors

D = 1.80; % No. GPs in each direction provided by one cell

L = zeros(udim,1);
for di = 1:1:udim
    L(di,1) = ULims(di,2) - ULims(di,1);
end

% 1.- REQUIRED No. WEDGES IN EACH DIRECTION:
% Also stores No. directions with odd wedges and their refs

nwa = zeros(udim,1);    % No. wedges to add
sw  = zeros(udim,1);    % step length for wedges

ndo = 0 ;
doR = zeros(1,1);

for di = 1:1:udim
    
    nwa(di) = round ((NR(di)-D)/(D/2)) ; % each wedge adds half D
    
    if nwa(di)<0
        nwa(di)=0;
    end
    
    if mod(nwa(di),2)==1 % odd No. wedges to add
        
        sw(di) = L(di)/((nwa(di)+1));
        
        ndo = ndo + 1 ;
        doR(ndo,1) = di ;
        
    elseif  mod(nwa(di),2)==0 % even No. nwa
        
        sw(di) = L(di)/((nwa(di)+1));
        
    end
end




% 2.- ESTABLISHES DIRECTION TO ALTERNATE 'adi' AND ITS PERP. DIRECTIONS 'pdi':

if ndo==1
    
    adi = doR(1);
    
elseif ndo ==0
    
    Mnwa = max(nwa);
    for di = 1:1:udim
        if Mnwa==nwa(di)
            adi = di ;
            break
        end
    end
    
else
    
    Mnwa = 0 ;
    for i = 1:1:ndo
        if nwa(doR(i))>Mnwa
            Mnwa = nwa(doR(i));
            adi = doR(i);
        end
    end
    
end

pdi = zeros(2,1); % perp. directions to alternated diretion

conta = 0 ;
for di = 1:1:udim
            
    if di ~= adi
        conta = conta + 1;
        pdi(conta,1) = di ;
    end
end




% 3.- COMPUTES COORD'S IN PLANE OF 'pdi' TO ACHIEVE No. GPs IN THOSE DRECTIONS:

% 3.1.- No. cells to add and step of each cell:

sc  = zeros(udim,1);    % step length for cells
nca = zeros(udim,1);    % No. cells to add

for di = 1:1:udim
    if di~=adi
        nca(di,1) = round ((NR(di)-D)/D);
        sc(di) = L(di)/(nca(di)+1);
    end
end




% 3.2.- Introduces the bias due to knots distribution:

Ngm = zeros(udim,1); % No. knot of cells (gamma-space)
gm = zeros(udim,1);

for di = 1:1:2
    
    dr = pdi(di);
     
    % non-repeated knots:
    Nknd = Vsize(dr);
    knd = V(dr,1:Nknd);
 
    % equally spaced nodes in gamma-space:
    Ngm(dr,1) = nca(dr)+2;
    
    gmd  = zeros(1,Ngm(dr,1));
    
	stp = L(dr)/(Ngm(dr)-1); % equally spaced step (gamma space)
    
    acc = 0 ;
    for i = 1:1:Ngm(dr,1)
        
        if i==Ngm(dr,1)
           acc = 1 ; 
        end
        gmd(1,i) = acc;
        
        acc = acc + stp;
    end 
    
    
    % introduces bias and allocates:
    [uk] = im0156_knotBias (gmd, Ngm(dr,1), knd, Nknd);

    for i = 1:1:Ngm(dr,1)
        gm(dr,i) = uk(1,i);
    end
    
end



% 4.- ADD NODES IN THE 'adi' DIRECTION:
% Recall here alternates positions from line to line.

[Cn, Nn] = im0157_adiRows (V, Vsize, sw, nwa, nca, gm, adi, pdi);

DT = delaunayTriangulation(Cn);

EVcon  = DT.ConnectivityList;   % conn. matrix element => vertex
NE     = size(EVcon,1);         % No. elements


% 5.- PLOTTING:

% plot the whole tet-mesh:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
fr = 21 ;

figure(fr)

for e = 1:1:NE
   
    unod = zeros(4,udim);
    for i = 1:1:4
        unod(i,:) = Cn(EVcon(e,i),:);
    end
    
   	pt1362_pTet (unod, udim, fr, 0, 1);

end

axis equal
%}



% perpendicular directions nodes (drawn on the plane):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
fr = 3 ;

figure(fr)

%scatter3(Cn(:,1), Cn(:,2), Cn(:,3), 35 , 'k'); hold on


for i = 1:1:Nn
    
    x = Cn(i,:);
    
    if x(adi)==0
        scatter3(x(1),x(2),x(3), 40 ,'filled', 'k'); hold on
    end    
    
end


plot3([0 1], [0 0] , [1 1], '--k', 'LineWidth', 1); hold on

axis equal

xlim([0 1])
ylim([0 1])
zlim([0 1])

if jj==0
end

%}



% nodes along alternating direction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{

fr = 9 ;

%plot3([ 1 1] , [1 1], [0 1], 'k'); hold on

axis equal

% tets edges, drawn under restrictions:


for e = 1:1:NE
   
    unod = zeros(4,udim);
    for i = 1:1:4
        
        unod(i,:) = Cn(EVcon(e,i),:);
        
    end
    
    r = [1 2 ; 2 3 ; 3 1 ; 4 1 ; 4 2 ; 4 3 ];
  
    
    for i = 1:1:6

        x1 = unod(r(i,1),:);
        x2 = unod(r(i,2),:);

        if x1(2)==0 && x2(2)==0
            
            plot3( [x1(1) x2(1)],  [x1(2) x2(2)],  [x1(3) x2(3)],'k'); hold on 
            
        elseif x1(1)==0 && x2(1)==0
            
            plot3( [x1(1) x2(1)],  [x1(2) x2(2)],  [x1(3) x2(3)],'k'); hold on 
        
        elseif x1(3)==1 && x2(3)==1
            
            plot3( [x1(1) x2(1)],  [x1(2) x2(2)],  [x1(3) x2(3)],'k'); hold on 

        end
        
        
        
        
    end
    
    
    pt1362_pTet (unod, udim, fr, 0, 1);
      
end

axis equal

if jj==0
end

%}


end

