function [uk] = im0156_knotBias (gmd, Ngm, knd, Nknd)

% RETURNS:
% uk(1,Ngm): ararnged nodes positions with knot spans bias

% works in a single direction, that must be allocated to the inputs before
% calling the function

% INPUTS:
% gmd(1,Ngm):   nodes positions in gamma space, equally spaced
% Ngm:          No. nodes
% knd(1,Knnd):  non-repeared knots position in the relevant direction
% Nknd:         No. non-repeated knots

uk = zeros(1,Ngm);

GKt = zeros(Nknd-1,2);  % gamma => u transformation, 
                        % for each non-void knot span includes both end values


% 1.- Populates matrix with gamma => u transformation: GKt:
    
stp = 1 / (Nknd-1);
    
acc1 = 0 ;
acc2 = stp ;
for i = 1:1:Nknd-1
        
    if i == Nknd-1
        acc1 = 1-stp;
        acc2 = 1 ;
    end
        
	GKt(i,1) = acc1 ;
	GKt(i,2) = acc2 ;
        
	acc1 = acc1 + stp;
	acc2 = acc2 + stp ;
end

GKt(i,2) = 1 ;


% 2.- Computes u-coord's, that accounts for the knot spans bias:
    
for i = 1:1:Ngm
        
	g = gmd(1,i); % gamma coords

	% finds segment of GKt where a lies:
        
    for j = 1:1:Nknd-1       
        if GKt(j,1)<= g && g <= GKt(j,2)
            sg = j;
            break
        end
    end
        
    
	% interpolares within sg:
    
	gL = GKt(sg,2) - GKt(sg,1); % g-segment lenght
	Dg = g - GKt(sg,1);         % g-increment
 	uL = knd(1,sg+1) - knd(1,sg); % u-segment increment
        
	Du = Dg/gL * uL ;
        
	% allocates:
	uk(1,i) = knd(1,sg) + Du ;

end


end

