function [Cn, Nn] = im0157_adiRows (V, Vsize, sw, nwa, nca, gm, adi, pdi)

% RETURNS:
% Cn(Nn,udim): Coordinates of nodes of the tetrahedral mesh

% INPUTS:
% nca(udim,1): No. cells to add in each direction
% nwa(udim,1): No. wedges to add in each direction
% sw(udim): step for wedges
% gm(udim, max(nr,nc)): coordinates of nodes of plane pdi
% adi: direction to alternate coordinates
% pdi(2,1): perpendicular directions to adi

% Alternates positions from line to line and use plane pdi coordiantes

udim = 3 ;

Cn = zeros(1,udim); % to stores tets coordiantes
Nn = 0 ;

% 1.- BASIC PARAMETERS OF THE NET OF POINTS:

st = 2*sw(adi); % step for each cell

% 1.1.- No. rows and cols, given by the plane of coords of pdi

nr = nca(pdi(1))+2; 
nc = nca(pdi(2))+2;


% 1.2.- No. intermediate nodes to add

if mod(nwa(adi),2)==1
    
    Ni = (nwa(adi)-1)/2 ; % odd case => computes the lower value (add +1 in alternative rows)
    
elseif mod(nwa(adi),1)==0
    
    Ni = nwa(adi)/2 ;
    
end


% 1.3.- Non-void knot spans used for knot bias calculation:

Nknd = Vsize(adi,1);

knd = zeros(1,Nknd);
for i = 1:1:Nknd 
    knd(1,i) = V(adi,i);
end


% 2.- RUNS OVER THE WHOLE NET OF POINTS IN THE pdi PLANE:

for i = 1:1:nr

    % alternates offset flag from one row to the next:
    
    if mod(i,2)==0 
        x = 1;
    else
        x = 0 ;
    end
    

    for j = 1:1:nc

        % offset flag:
        
        oF = 0;
        if mod(j,2)==x
            oF = 1 ;
        end
        
        
        % GENERATES THE ROW OF NODES (& THEN APPLIES THE BIAS):
        
        conta = 0 ;
        tr   = zeros(1,2); % 'temporary row'
         
        % 4.1.1.- initial point:
        conta = conta + 1 ;
        tr(1,conta) = 0;
        
        % 4.1.2.- intermediate points:
        acc = 0 ;
        for k = 1:1:Ni
            
            conta = conta + 1 ;
            acc = acc + st ; 
            tr(1,conta) = acc; 
            
            % applies offset
            if oF == 1
                tr(1,conta) = acc-st/2; 
            end
        end
        
        % 4.1.3.- for alt. rows and nwa odd => adds extra node:
        
        if oF==1 && mod(nwa(adi),2)==1
            
            conta = conta + 1  ;
            
            if nwa(adi) == 1
                tr(1,conta) = tr(1,conta-1)+st/2;
            elseif nwa(adi) > 1
                tr(1,conta) = tr(1,conta-1)+st;
            end
            
            
            
        end
        
        % 4.1.4.- final point:
        conta = conta + 1  ;
     	tr(1,conta) = 1;
        


        % 4.2.- APPLIES BIAS AND ALLOCATES:
        
        % knd = [ 0 0.25 0.50 0.75 1 ]; %%%%%%%%%%%%%%%%%%%%%%%%%
        
        [uk] = im0156_knotBias (tr, conta, knd, Nknd);
        
        
        
        for c = 1:1:conta
            
            Nn = Nn+1;
            
            Cn(Nn,pdi(1)) = gm(pdi(1),i);
            Cn(Nn,pdi(2)) = gm(pdi(2),j); 
            Cn(Nn,adi) = uk(1,c); 
            
        end
        

    end
    
end





end

