function [GPT] = im0158_GPT (Spatches, pR, NGd)

% RETURNS:
% GTP structure for patch pR



GPT = struct('Nn', 0, 'NE', 0, 'Xv', [], 'Av', [], 'EVcon', [],'DT', 0, 'Lax', [], 'Lv', [] );
% Lax(NE,2): average length of tetrahedron edge in param. and phys. space
% Lv (Nn,2): average length of edge attached to each node in param. and phys. space

pudim = 3 ;
xdim = 3 ;

pKnotV  = Spatches(pR,1).INS(1).mKnotV;
pNcp    = Spatches(pR,1).INS(1).mNcp;
pDegree = Spatches(pR,1).INS(1).mDegree;
pUsize  = Spatches(pR,1).INS(1).mUsize;
pEw     = Spatches(pR,1).INS(1).mEw;
pEP     = Spatches(pR,1).INS(1).mEP;
    
[pNn, pNE, pXv, pAv, pEVcon, pDT] = im0145_tetGross (NGd, pKnotV, pNcp, pDegree, pUsize, pEw, pEP, xdim, pudim);
     
GPT.Nn = pNn;
GPT.NE = pNE;
GPT.Xv = pXv;
GPT.Av = pAv;
GPT.EVcon = pEVcon;
GPT.DT = pDT;
    
% average length of tet. edges:
  
Anod = zeros(4,xdim);
Xnod = zeros(4,xdim);
pLax = zeros(pNE,2);
    
for e = 1:1:pNE
        
    for i = 1:1:4
        
        Anod(i,:) = pAv(pEVcon(e,i),:);
        Xnod(i,:) = pXv(pEVcon(e,i),:);
        
    end
    
    La = tm1309_avEL (Anod, 4, xdim);
	Lx = tm1309_avEL (Xnod, 4, xdim);
      
    pLax(e,1) = La ;
	pLax(e,2) = Lx ;    
end
    
GPT.Lax = pLax;


% average of lengths attached to nodes:

% first stores nodes linked to each node

NLn = zeros(pNn,1); % No. lines per node
nRn = zeros(pNn,1); % nodes refs linked to each node

for e = 1:1:pNE
for i = 1:1:4
        
        bn = pEVcon(e,i);  % base node
                            % checks to the other 3 nodes
        for j = 1:1:4
            
            on = pEVcon(e,j); % opposite node
            
            if on~=bn
                
                addF = 1 ; % add opposite node flag
                
                % searches if is already stored
                
                for k = 1:1:NLn(bn)
                    if nRn(bn,k)==on
                        addF = 0;
                        break
                    end
                end
                
                % adds node if rquired
                if addF==1
                    NLn(bn) = NLn(bn) + 1 ;
                    nRn(bn,NLn(bn)) = on ;
                end   
            end
            
        end
end
end

% then computes distances and average:

pLv = zeros(pNn,2);

xb = zeros(xdim,1);
xo = zeros(pudim,1);
ab = zeros(xdim,1);
ao = zeros(pudim,1);

for k = 1:1:pNn
    
    Aacc = 0 ;
    Xacc = 0 ;
    
    xb(:,1) = pXv(k,:);
    ab(:,1) = pAv(k,:);
    
    for i = 1:1:NLn(k)
        
        on = nRn(k,i);
        
      	xo(:,1) = pXv(on,:);
        ao(:,1) = pAv(on,:);
        
        Aacc = Aacc + norm(ao-ab);
        Xacc = Xacc + norm(xo-xb);
    end
    
    pLv(k,1) = Aacc / NLn(k);
    pLv(k,2) = Xacc / NLn(k);
end

GPT.Lv = pLv;

end



