function [sRt, pRt] = st0302_skinRt (Spatches, Npa, Eskin, Nbe, DUtol)

% RETURNS:
% sRt(Nbe,1):  target distance for initial triangulation of skins.
% pRt(Npa,,1): target distance for each patch, computed as equilateral triangle side lenght
% Takes it from relevant patch.

pudim = 3 ;
xdim = 3 ;


% 1.- COMPUTES TARGET DISTANCE FOR EACH PATCH:

% This is obtained as the edge length of one equilateral triangle whose
% area is the same as the average area of patch triangles at boundary.

pRt = zeros(Npa,1); % R-phys distance for each patch , comes from average triangle area

for pR = 1:1:Npa
    
    % patch information:
	pNcp     = Spatches(pR,1).INS(1).mNcp ;
    pDegree  = Spatches(pR,1).INS(1).mDegree ;
    pUsize   = Spatches(pR,1).INS(1).mUsize ;
    pKnotV   = Spatches(pR,1).INS(1).mKnotV ;
    pEP      = Spatches(pR,1).INS(1).mEP ;
    pEw      = Spatches(pR,1).INS(1).mEw ;

    [NGd] = rm0162_NRgp (pKnotV, pUsize, pDegree, pudim); % No. Gps in each param. direction
    

    % nodes from Cp's: preliminary nodes (ignoring Eskins):
    [pNn, pXv, pAv] = im0145_tetGross (NGd, pKnotV, pNcp, pDegree, pUsize, pEw, pEP, xdim, pudim);
       

    % tetrahedrons with preliminary nodes (the whole untrimmed patch)
    [NE, NV, EVcon, LVcon, ELcon] = im0147c_unTrTetMesh (Spatches, pR, pXv, pAv, xdim, pudim);
    
    % boundary triangles: number, vxt conn. and surface conn. (6 surfaces, as boundary faces)
    [NBT, BTVcon, BTScon] = im0150_TetbTri (NE, NV, pXv, pAv, EVcon, DUtol);
    
    % extracts the 6 boundary surfaces NURBS features of solid patch:
    [sfs] = a02_surfFromSolid (pNcp, pDegree, pKnotV, pUsize, pEP, pEw);
    
    % average area in phys. space (Aavg) of boundary triangles:
    [At, Aavg] = im0151_AvgTriArea (NBT, BTVcon, BTScon, pAv, sfs);

    
    % computes triangle side length and allocates:
    pRt(pR,1) = 1.52*Aavg^0.5;

end


% 2.- TARGET DISTANCE FOR EACH SKIN (TAKEN FROM PATCH WHERE IT BELONGS):

sRt = zeros(Nbe,1);
for b = 1:1:Nbe
    
    if Eskin(b).mCou==0
        
        pR = Eskin(b).mPa(1);
        sRt(b,1) = pRt(pR,1);
        
    else
        pR1 = Eskin(b).mPa(1);
        pR2 = Eskin(b).mPa(2);
        sRt(b,1) = (pRt(pR1,1) + pRt(pR2,1)) / 2 ;
    end
 
end


end

