function [rTskin, NcvE, cLUvE, cLXvE, Ntc, tcR, Nts, tsR] = st0303_iniTriang ...
    (GPT, Spatches, Eskin, agCurvE, Nbe, CPcon, nloE, SCconE, ncuE, tolRc, tolRs, Xtol, DUtol, ndig, tolD, DerC)

% RETURNS:
% rTskin(Nbe,1); triangulation with initial Rt, prepared to be refined to quadratic approximation
% tsR(Nts,2): trimming surface ref. and patch used for its initial triangulation

pudim = 3 ;
eudim = 2 ;
cudim = 1 ;
xdim = 3 ;

TNcuE = size(agCurvE,1);




% 1.- IDENTIFIES TRIMMING SURFACES AND THE PATCH ATTACHED TO:
% For coupling surfaces uses the master patch.

[Nts, tsR] = st0305_tsR (Nbe, Eskin);


% 2.- PRELIM. DISCRETIZATION OF CURVES:

[NcvE, cLXvE, cLUvE, Ntc, tcR] = st0306_cPrelim ...
(Spatches, GPT, Nts, tsR, TNcuE, agCurvE, nloE, ncuE, SCconE, xdim, cudim, Xtol, DUtol, tolRc);



% 2.- PRELIM. DISCRETIZATION OF TRIMMING SURFACES:

[sRt] = st0307_sTarget (GPT, Eskin, Nts, tsR);

Nvx = 2 ;

rTskin(Nts,1) = cEskin ;

for b = 1:1:Nbe
    rTskin(b,1) = Eskin(b,1);
end


for s = 1:1:Nts

    sR = tsR(s,1);
    
    % 3.1.- Surface features:
    
    sNcp    = Eskin(sR,1).mNcp;
    sDegree = Eskin(sR,1).mDegree;
    sUsize  = Eskin(sR,1).mUsize;
    sKnotV  = Eskin(sR,1).mKnotV;
    CS     = Eskin(sR,1).mCS;
    sP     = Eskin(sR,1).mP;
    sw     = Eskin(sR,1).mw;
    [sEw,sEP] = GEN_12_ExpandBw (sw, sP,xdim, eudim, sNcp);

    sULims = [ sKnotV(1,1) sKnotV(1,sUsize(1))  ;  sKnotV(2,1) sKnotV(2,sUsize(2))];
    
    NL = nloE(sR); % No. loops
    
    % 3.2.- Target distance of that surface:
    
    R = sRt(s,1);
    
 	eswP = 30 ;
    esw = eswP/100 * R ;
    
	% 3.2.- Contours coord's in terms of surface: Note only selects curves
    % relevant to that surface
    
    % Surface contour edges: starts by 0,0 and moves counter-clock-wise:
    % they are numbered as 1 to 4 edges with that order

    [edCurv] = ts3313_EdgesContours (sNcp, sDegree, sUsize, sP,sw, sKnotV);

    % edCurv(4) = struct ('Ncp',0,  'Degree',0, 'Usize',0, 'KnotV',[], 'w',[], 'P',[], 'conF', 0, 'CD', 0);


    
    % 2.- LOOPS ON SURFACE LOOP TO COMPUTE CONTOUR VERTEXES
    
    [cXv, cUv, Nlv, Nlcv] = rs3336b_loops ...
    (edCurv, cLXvE, cLUvE, NcvE, SCconE, ncuE, nloE, agCurvE, s, sDegree, sNcp, sKnotV, sUsize, sP, sw, sEP, sEw, CS, eudim, xdim, Xtol, DUtol);



    % 3.- CLOSED SURFACE CHECK:

    if sum(CS) > 0
        [cUv] = su2914_CSuLims (cUv, sKnotV, sUsize, nloE(s), Nlv, CS, eudim);
    end

    
	% 3.1.- Edge contour loop for that surface alone: 'unique surface' = usf
    
    usf = struct('Ncp',[], 'Degree',[], 'Usize',[], 'KnotV',[], 'EP',[], 'Ew',[], 'P',[],'w',[], 'CS',[0 0]); % un-trimmed surface feat.
    
	usf.Ncp     = sNcp;
    usf.Degree  = sDegree;
    usf.KnotV   = sKnotV;
    usf.Usize   = sUsize;
    usf.P       = sP;
    usf.w       = sw;
	usf.EP      = sEP;
    usf.Ew      = sEw;
    usf.CS      = CS;

    [ecCurv, eTNcu, eSCcon, encu, enlo, ~, ~] = ts3311_ECcurves (1, usf, xdim);
    
    
    [ecLXv, ecLUv, eNcv, ecRc] = ts3312_Cvertexes (ecCurv, eTNcu, R, Nvx, tolRc, xdim);
    

    
    [ecXv, ecUv, eNlv, eNlcv] = rs3336_cUvSurface (ecLXv, ecLUv, eNcv, eSCcon, encu, enlo, ecCurv, 1, usf, xdim);
    

	% 3.2.- Detects if it is strip or not:
    
    pt = 15 ; % percentage of deviation of var. coeff. to consider strip
    [strF, Ld] = sv3346_Strip (sNcp, sEP, R, pt, xdim, eudim);
    

    %{
    if s == 1
        
        figure(s)
        scatter3(sP(:,1), sP(:,2),sP(:,3),'r'); hold on
        
        for i = 1:1:prod(sNcp)
            text(sP(i,1), sP(i,2),sP(i,3),num2str(i)); hold on
        end

        axis equal
        
        if gg==0
        end
        
    end
    %}
    

	% 3.4.- Surface vertexes for un-trimmed surface:
    % separates vertexes inside surf. param. limits (sUv) from a bigger group of all of them (sUvt):

    [NCT, cnR, sRvt, sUvt, sXvt, inF, coF, dSmesh, sRv, sUv, sXv, Nv] = sv3340_Svertexes ...
    (s, strF, Ld, pt , sDegree, sNcp, sKnotV, sUsize, sP,sw, sEw,sEP, sULims, R, esw, xdim, eudim, ndig, tolRs,tolD, ecUv, eNlv, eNlcv, DerC);

   
    % 3.5.- Estimates pattern coordinates of contours:
    % cRv = zeros(NL,max(Nlv),udim) ;

    [cRv] = rs3337_coRcoor (sRvt, sUvt, coF, cUv, NL, Nlv, eudim, s);
    

    % 3.6.- Selects only surface coordinates that are within the loops
    % from sUv selectes them and stores in sUvi


    
    [sRvi, sUvi, sXvi, Nvi] = sv3344_SvforT (s, eswP, sRv, sUv, sXv, Nv, cUv, cXv, NL, Nlv, xdim, eudim);
    

    % 3.7.- Merge all coordinates and triangulates:
    
    [NE, TVcon, VTcon, LVcon, VLcon, TLcon, Rv, Uv, Xv, XNe, XNv, Ae] = rs3334_Rtriang ...
    (cRv, sRvi, cUv, sUvi, cXv, sXvi, Nvi, NL, Nlv, Nlcv, ncuE, SCconE, s, xdim);


    % 3.8.- Allocates to response:

    rTskin(s,1).mNE = NE ;
    rTskin(s,1).mTVcon = TVcon ;
    rTskin(s,1).mVTcon = VTcon ;
    rTskin(s,1).mLVcon = LVcon ;
    rTskin(s,1).mVLcon = VLcon ;
    rTskin(s,1).mTLcon = TLcon ;
    rTskin(s,1).mRv = Rv ;
    rTskin(s,1).mUv = Uv ;
    rTskin(s,1).mXv = Xv ;
    
    rTskin(s,1).mXNe = XNe ;
    rTskin(s,1).mXNv = XNv ;
    rTskin(s,1).mArea = Ae ;

    
    rTskin(s,1).mNV  = size(Xv,1);
    rTskin(s,1).mNlv = Nlv ;
    
    
    %{
    %if s == 2
    fr = 1 ;
    
    NLI = size(LVcon,1);
    
    NV = size(Xv,1);
    
    dim = xdim ;
    Cv = Xv;
    
    pt1366_pTriLinMesh_nD ...
    (Cv, NLI, LVcon, TVcon, dim, fr, 'k', 0, NV, 0, 0, NE, 0, 0, 0, 0, 0, 1)
    
    %end
    %}
    
    
    
end





end

