function [NcvE, cLXvE, cLUvE, Ntc, tcR] = st0306_cPrelim ...
    (Spatches, GPT, Nts, tsR, TNcuE, agCurvE, nloE, ncuE, SCconE, xdim, cudim, Xtol, DUtol, tolRc)

% RETURNS: preliminary discretization of the trimming edge curves


cLXvE = zeros(TNcuE,2,xdim); 
cLUvE = zeros(TNcuE,2,cudim);  
%cAvE  = zeros(TNcuE,2,pudim); 
NcvE  = zeros(TNcuE,1);


% 1.- DETECTS EDGE CURVES TO DISCRETIZE and COMPUTES TARGET LENGTH IN PHYSICAL SPACE:
% Those ones attached to trimming surfaces

Ntc = 0 ;
tcR = zeros(1,3); % trimming curves, patch ref and target phys. distance 

% 1.1.- Curves and patches refs.

for i = 1:1:Nts
    
    sR = tsR(i,1);
    pR = tsR(i,2);
    
    for l = 1:1:nloE(sR,1)
        
        for c = 1:1:ncuE(sR,1)
            
            cR = abs(SCconE(sR,l,c)); % curve ref.
            
            % stores the ref. if not stored previously
            
            ncF = 1 ; % new curve flag
            for j = 1:1:Ntc
                
                if tcR(j,1)==cR
                    ncF = 0;
                    break
                end
            end
            
            if ncF==1
                Ntc = Ntc+1;
                tcR(Ntc,1) = cR;
                tcR(Ntc,2) = pR;
            end
            
        end
        
    end
    
end


% 1.2.- Target distances:
% Averate of N tets. Lax where the curve pass through:

st = 0.5;

for i = 1:1:Ntc
    
    % curve and attached patch:
    
    cR = tcR(i,1);
    pR = tcR(i,2);

    
    % tets and average of edges length:
    
    pDT  = GPT(pR,1).DT;
    pLax = GPT(pR,1).Lax;

    
    % curve and patch features:
    
    cNcp = agCurvE(cR,1).Ncp;
    cDegree = agCurvE(cR,1).Degree;
    cKnotV = agCurvE(cR,1).KnotV;
    cUsize = agCurvE(cR,1).Usize;
    cw = agCurvE(cR,1).w;
    cP = agCurvE(cR,1).P;

    pNcp = Spatches(pR,1).INS(1,1).mNcp;
    pDegree = Spatches(pR,1).INS(1,1).mDegree;
    pKnotV = Spatches(pR,1).INS(1,1).mKnotV;
    pUsize = Spatches(pR,1).INS(1,1).mUsize;
    pw = Spatches(pR,1).INS(1,1).mw;
    pP = Spatches(pR,1).INS(1,1).mP;
    pCD = [0 0 0];
    
    % loop on curve sample points:
    
    cX = zeros(1,xdim);
    conta = 0 ;
    for u = 0:st:1
        
        conta = conta + 1 ;
        
        % X location:
        [X] = b2601_BNURBS_Xcoord (u, cKnotV, cNcp,cDegree, cUsize, cw,cP, 1,xdim);
        
        cX(conta,:) = X(:,1);
        
    end

    % A location:
    cA = v2861b_PPvol (cX, conta, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, xdim, Xtol, DUtol);
    

    % detects where each point lies and stores tet. ID
    tID = pointLocation(pDT,cA);

    
    % computes average of Lax and allocates to R:
    R = 0 ;
    for j = 1:1:conta
        R = R + pLax(tID(j))/conta;
    end
    
    tcR(i,3) = R ;

    
end




% 2.- CONTOUR CURVES DISCRETIZATION:

Nvx = 2 ; % sets min. number of divisions in the curves to two

for i = 1:1:Ntc
 
    cR = tcR(i,1);
    R  = tcR(i,3);
    
    
	% 2.1.- Extracts NURBS features of curve:

    cNcp    = agCurvE(cR,1).Ncp ;
	cDegree = agCurvE(cR,1).Degree ;
    cUsize  = agCurvE(cR,1).Usize ;
  	cKnotV  = agCurvE(cR,1).KnotV ;
  	cB      = agCurvE(cR,1).P ;
 	cw      = agCurvE(cR,1).w ;

    
	% knot vector has to be row vector:
    if size(cKnotV,2)==1
        cKnotV = cKnotV';
    end
    
	% 2.2.- Physical, param. (curve space) and param. (patch space) coord's:
       
    [cU,cX,~,~] = ts3315_oneCvtxs (R, Nvx, cNcp, cDegree, cUsize, cKnotV, cB, cw, xdim, cudim, tolRc);

    ccv = size(cX,1); % No. vtxs of the contour curve
    

	%[cA] = v2861b_PPvol (cX, ccv, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, xdim, Xtol, DUtol);


    % 2.3.- Allocates to response:
    
    NcvE(cR,1) = ccv ;
    
    for v = 1:1:ccv 
        cLXvE(cR,v,:) = cX(v,:);
        cLUvE(cR,v,:) = cU(v);
        %cAvE(c,v,:) = cA(v,:);
    end
 
end


end

