function [NEA] = gf0510_LSM (xs, Ncp, Degree, xdim, udim)


% RETURNS:
% NEA: NURBS entity approximated to a set of points by LSM

% INPUTS:
% xs(ns,xdim): sample points
% nr: No. control points used for NEA

% done only for curves, extend to surfaces and solids when I have chance

NEA = struct('Ncp',[],'Degree',[],'KnotV',[], 'Usize',[], 'P', [], 'w', [] );



% 1.- MAIN DATA:

ns = size(xs,1);
nr = Ncp ;      % for curves
p = Degree ;    % for curves


% 2.- PARAM. COORDS OF SAMPLE POINTS:

% 2.1.- Chords length:

dn = zeros(ns-1,1);

for i = 1:1:ns-1
    
    for di = 1:1:xdim 
        dn(i,1) = dn(i,1) + (xs(i,di) - xs(i+1,di))^2;
    end
    dn(i,1) = dn(i,1)^0.5 ;
end
D = sum(dn);


% 2.2.- U coords of sample points:

us = zeros(ns,udim);
us(1,1)  = 0 ;
us(ns,1) = 1 ;

for i = 2:1:ns-1 
    us(i,1) = us(i-1,1) + dn(i-1) / D ;
end



% 3.- DEFINITION OF KNOTS FOR KNOT SPAN:
% distribution according to sample points (us) spacing

nik = nr-p-1 ; % No. internal knots of the knot span

U = zeros(udim,nr+p+1);

% G-space: ns points uniformily distribuited
dg = 1 / (ns-1);
G = [0:dg:1];


% internal knots in G space:
dg = 1 / (nik+1);
Gu = [dg:dg:1-dg];


% finds interval of each Gu and then interpolates

for i = 1:1:nik
    
    g = Gu(i);
    
    % interval:
    for j = 1:1:ns-1
        if G(j) <= g && g <= G(j+1)  
            k = j ; % interval reference
            break
        end
    end 
    
    % interpolates:
    u = us(k,1) + (us(k+1,1) - us(k,1)) / (G(k+1) - G(k)) * (g-G(k));
    
    % allocates:
    U(1,p+1+i) = u ;
    
end

for i = nik+1 : 1 : nik+p+1
    U(1,p+1+i) = 1 ; 
end



% 4.- MATRIX of SHAPE FUNCTION and R vector:

m = nr + p + 1 ;

% 4.1.- Shape matrix:

N = zeros(ns-2,nr-2);

for i = 1:1:ns-2
    
    u = us(i+1,1);
    
    for b = 1:1:nr-2
        N(i,b) = b2604_MNL (u, b+1, U, nr, p, m, udim);
    end

end


% 4.2.- Rk vector:

Rk = zeros(ns-2,xdim);

for i = 1:1:ns-2
    
    u = us(i+1,1);
    
    N1 = b2604_MNL (u, 1,  U, nr, p, m, udim);
    Ne = b2604_MNL (u, nr, U, nr, p, m, udim);
    
    Rk(i,:) = xs(i+1,:) - N1*xs(1,:) - Ne*xs(ns,:);

end


% 4.3.- R vector:

R = N'*Rk;


% 5.- COMPUTES THE CONTROL POINTS:

Pu = (N'*N)\R;

P = [ xs(1,:) ; Pu ; xs(ns,:) ] ;


% 6.- ALLOCATES:

NEA.Ncp = nr ;
NEA.Degree = Degree ;
NEA.KnotV = U ;
NEA.Usize = nr+p+1 ;
NEA.P = P ;
NEA.w = ones(1,nr) ;


end

