function [NER] = gf0511_removeLSM (NEO, r, ns1, xdim, udim)

% RETURNS:
% NER: NURBS entity reduced: one cp less

% INPUTS:
% NEO: NURBS entity original
% r: ref of cp to be removed
% ns1: No. sample points within the affected knot spans

NER = struct('Ncp',[],'Degree',[],'KnotV',[], 'Usize',[], 'P', [], 'w', [] );


% 1.- EXTRACTS NURBS DATA:

n   = NEO.Ncp;
p   = NEO.Degree;
U   = NEO.KnotV;
P   = NEO.P;
w   = NEO.w;
m   = NEO.Usize;


% 1.- FIRST AND LAST CP's SUSTECTIBLE TO BE REMOVED:

Fc = ceil(p/2)+1;
Lc = ceil(p/2)+n-p-1;


% 2.- LOCALIZES LIMITS OF CPS: 
% All in updated arrangement, i.e. one cp less:

% 1 to a:       prev. to affected
% a+1 to b-1:   affected 
% b to n:       post affected

nL = floor((p-1)/2);

nR = floor(p/2);

Nc = nL + 1 + nR;

IC  = [r-nL : 1 : r+nR ]';
ICr = [r-nL : 1 : r+nR-1 ]';

% Associated knots refs to IC in the original arrangement:

IK = zeros(Nc,1);
for i = 1:1:Nc
    IK(i,1) = IC(i) - Fc + p + 2 ;
end

kr = r - Fc + p + 2 ; % knot ref to remove


% 3.- SAMPLE POINTS:
% parameter and physical coordinates
% they are just the average within each knot span + affected knots spans

us1 = zeros(ns1,udim);

u1 = U(IK(1));
u2 = U(IK(end));
du = (u2-u1)/ns1 ;

uacc = u1 - du/2;

for i = 1:1:ns1  
    uacc = uacc + du;
    us1(i,1) = uacc ;
end


% adds at least one sample point at each mid-span and initial and last:

us2 = zeros(1,udim);
ns2 = 0 ;
for k = 1:1:m-1
    
    sF = 0 ; 
    if k < IK(1) 
        sF = 1 ; 
    elseif IK(end) < k
        sF = 1 ;
    end
    
    if sF==1 
        if U(k) ~= U(k+1)
            ns2 = ns2 + 1 ;
            us2(ns2,1) = (U(k) + U(k+1))/2 ;
        end
    end 
end



us2 = [U(1) ; us2 ; U(end)];
us = [us1 ; us2 ];
us = sortrows(us,1);
ns = ns1 + ns2 + 2 ;


% physical coordinates:

xs = zeros(ns,xdim);
for i = 1:1:ns 
    
    x = b2601_BNURBS_Xcoord (us(i), U, n, p, m, w, P, udim, xdim);
    xs(i,:) = x(:,1);
end


% 5.- REDUCED VALUES:

% 5.1.- Knot vector & Npcs:

Ur = zeros(1,m-1);
nr = n - 1 ;

mr = m-1;

j = 0 ;
for i = 1:1:m
    if i~=kr 
        j = j+1;
        Ur(j) = U(i);
    end
end


% 5.2.- Temporary values of control points (reduced):

wr = [w(1:r-1) , w(r+1:n)] ;
T  = [P(1:r-1,:) ; P(r+1:n,:)]; % temporaty P with no r cp


% 6.- NN MATRIX:


% 6.1.- Shape matrix:

N = zeros(ns,nr);

for i = 1:1:ns
    
    u = us(i,1);
    
    for b = 1:1:nr
        N(i,b) = b2604_MNL (u, b, Ur, nr, p, mr, udim);
    end

end


% limits of knowns:

a = ICr(1);
b = ICr(end);

a = a - 1;
b = b + 1;


% extracts matrices / vectors:

M = N'*N;

Mk1 = M(a+1:b-1,1:a);
Mu  = M(a+1:b-1,a+1:b-1);
Mk2 = M(a+1:b-1,b:nr);

Pk1 = T(1:a,:);
Pk2 = T(b:nr,:);

R = N'*xs ;

Ru = R(a+1:b-1,:);

Rue = Ru - Mk1*Pk1 - Mk2*Pk2 ;

% 7.- SOLVES AND ALLOCATES:

Pu = Mu\Rue ;

Pr = [Pk1 ; Pu ; Pk2 ];


NER.Ncp     = nr ;
NER.Degree  = p ;
NER.KnotV   = Ur ;
NER.Usize   = mr ;
NER.P       = Pr ;
NER.w       = wr ;



end

