function [MLT] = mt0601_MLT (pR, Spatches, rTskin, Nts, tsR)

% RETURNS:
% MLT: merged linear triangulation features

% MLT contains triangles of all rTskin in a single structure.
% ASSUMES TRIANGLES ARE LINEAR, i.e. DOES NOT CONSIDER MID-VTX

xdim  = 3 ;
pudim = 3 ;
sudim = 2 ;
cudim = 1 ;

MLT = struct('tNE',0, 'tNV',0, 'tNL', 0 , ...
            'tTVcon', [], 'tLVcon',[], 'tVLcon',[], 'tTLcon',[], 'tVTcon',[], 'tLTcon',[], ...
            'tBv',[], 'tUv',[], 'tAv',[], 'tXv',[], 'tANv',[], 'tANe',[], 'tXNv',[], 'tXNe',[], 'tNCS',[], ...
            'tDtet', [], 'tAtetIN', [], 'tAtetOU', [], 'tXtetIN', []);

% tNCS(tNV,2): T-curve and t-surf where the node belongs (0 in case not applied)

Acn = zeros(xdim,1); % candidate
Aex = zeros(xdim,1); % existing

tAv = zeros(1,pudim);
tXv = zeros(1,xdim);
tUv = zeros(1,sudim);
tBv = zeros(1,cudim);

tANv = zeros(1,pudim);
tANe = zeros(1,xdim);

tXNv = zeros(1,pudim);
tXNe = zeros(1,xdim);

tNE = 0 ;
tNV = 0 ;

tTVcon = zeros(1,3);

tNCS = zeros(1,2);





% conn. matrix: local node (sR,nR)=> TTR node ref (for repeated nodes and
% their relation with TTR elements).
% 'Local to global' node connectivity for nodes

LGcon = zeros(Nts,1);

for s = 1:1:Nts  
    
    sR = tsR(s); % surface ref.
    
    TVcon = rTskin(sR,1).mTVcon;

	% 1.- NODES:

        % 1.1.- First node:

        if tNV == 0 % (no elements added coz it is the first trimming surface)

            tNV = tNV + 1 ;

            [tBv, tUv, tAv, tXv, tANv, tXNv, tNCS] = mt0602_NNttr (sR, tNV, 1, tBv, tUv, tAv, tXv, tANv, tXNv, tNCS, rTskin(sR));

            LGcon(sR,1) = tNV;

        end



        % 1.2.- Checks the rest of the nodes (if not checked previously) repeated:

        for i = 1:1:rTskin(sR).mNV

            % candidate:
            
            Acn(:,1) = rTskin(sR).mAv(i,:);
           
            flagRN = 0; % 'repeated node' flag


            % compares with already stored:

            for j = 1:1:tNV

                Aex(:,1) = tAv(j,:);
                CO = go2542_coincidentP (Acn, Aex, xdim, 3);

                if CO==1
                    flagRN = 1;
                    break
                end
            end


            % case it is new node:

            if flagRN == 0

                tNV = tNV + 1 ;

                [tBv, tUv, tAv, tXv, tANv, tXNv, tNCS] = mt0602_NNttr (sR, tNV, i, tBv, tUv, tAv, tXv,tANv, tXNv, tNCS, rTskin(sR));

                LGcon(sR,i) = tNV;


            % case it is repeated node:  ith node of rEskin coincides with jth stored node:

            elseif flagRN == 1

                [tANv, tXNv] = mt0603_ENttr (rTskin(sR), tANv, tXNv, i,j);
                
                % node local to global conn. matrix:
                LGcon(sR,i) = j;

            end

        end


	% 2.- ELEMENTS: they are directly passed

        for e = 1:1:rTskin(sR).mNE

            eR = tNE + e;

            tANe(eR,:) = rTskin(sR).mANe(e,:);
            tXNe(eR,:) = rTskin(sR).mXNe(e,:);

            for i = 1:1:3
                nlr = TVcon(e,i);
                ngr = LGcon(sR,nlr);
                tTVcon(eR,i) = ngr ;
            end

        end

        tNE = tNE + rTskin(sR).mNE;
      
end


% 3.- OTHER CONN. MATRICES:

[tLVcon, tLTcon, tVLcon, tTLcon, tVTcon] = tr1347_otherConn (0, 0, tNE, tNV, tTVcon, tAv);

tNL = size(tLVcon,1);


% 4.- NODES TO FORM OPTIMAL TETS:

tAtetIN = zeros(tNE,pudim);
tAtetOU = zeros(tNE,pudim);

tXtetIN = zeros(tNE,xdim);

tDtet = zeros(tNE,1);

rIN = 0.40 ;
rOU = 0.0 ;


for e = 1:1:tNE

    [AtetIN, AtetOU, XtetIN, Dtet] = mt0605_opTetPos ...
    (e, Spatches, pR, tTVcon, tAv, rIN, rOU);
    
    tAtetIN(e,:) = AtetIN(:,1);
    tAtetOU(e,:) = AtetOU(:,1);
    
    tXtetIN(e,:) = XtetIN(:,1);
    
    tDtet(e,:)   = Dtet;

end



% 5.- ALLOCATES:

MLT.tNE = tNE;
MLT.tNV = tNV;
MLT.tNL = tNL;

MLT.tTVcon = tTVcon;
MLT.tLVcon = tLVcon;
MLT.tVLcon = tVLcon;
MLT.tTLcon = tTLcon;
MLT.tVTcon = tVTcon;
MLT.tLTcon = tLTcon;

MLT.tBv = tBv;
MLT.tUv = tUv;
MLT.tAv = tAv;
MLT.tXv = tXv;

MLT.tANv = tANv;
MLT.tANe = tANe;
MLT.tXNv = tXNv;
MLT.tXNe = tXNe;

MLT.tNCS = tNCS;

MLT.tAtetIN = tAtetIN;
MLT.tAtetOU = tAtetOU;

MLT.tXtetIN = tXtetIN;

MLT.tDtet = tDtet;


%{

if pR==1

    fr = 54 ;
    pm0650_plotMLT (MLT, fr, 'k', 1.1, 0, 0, 0, 0, 0, 0, 0, 1);

    if gg==0
    end

end
%}

end

