function [MLT] = um0627_nnMLT (MLT, eTHC, etR)

% RETURNS:
% MLT with new node inserted

% nodes coordinates and NCS:


% 1.- PRELIMINARIES:

% extracts arrays to facilitate coding:

tNE = MLT.tNE ;
tNV = MLT.tNV ;
tNL = MLT.tNL ;
tTVcon = MLT.tTVcon ;
tLVcon = MLT.tLVcon ;
tVLcon = MLT.tVLcon ;
tTLcon = MLT.tTLcon ;
tVTcon = MLT.tVTcon ;

tBv  = MLT.tBv ;
tUv  = MLT.tUv ;
tAv  = MLT.tAv ;
tXv  = MLT.tXv ;

tANv = MLT.tANv ;
tXNv = MLT.tXNv ;
tANe = MLT.tANe ;
tXNe = MLT.tXNe ;
tNCS = MLT.tNCS ;


% 2.- INVOLVED:

inr = [ tTVcon(etR,1), tTVcon(etR,2), tTVcon(etR,3), tNV+1 ] ; 
itr = [ etR , tNE+1, tNE+2 ] ;        
ilr = [ tNL+1, tNL+2, tNL+3] ;

Nni = 4 ; % No. involved nodes, considering the new inserted
Nti = 3 ;  % No. involved triangles 
Nli = 3 ;  % No. lines involved
Nnli = 3 ; % No. new lines involved


% 3.- UPDATES COORDINATES AND NUMBERS:
    
nr = tNV + 1; % node to insert
 
tNE = tNE + 2 ;
tNV = tNV + 1 ;
tNL = tNL + 3 ;
    
% node is added at the end

tBv = [ tBv ; eTHC.Bv ] ;
tUv = [ tUv ; eTHC.Uv ] ;
tAv = [ tAv ; eTHC.Av ] ;
tXv = [ tXv ; eTHC.Xv ] ;

tNCS = [ tNCS ; zeros(1,2) ] ;

tNCS(tNV,1) = eTHC.NCS(1); 
tNCS(tNV,2) = eTHC.NCS(2);



% 4.- CONNECTIVITY MATRICES:

% 4.1.- TVcon :

tTVcon = [tTVcon ; zeros(2,3) ];

TVe = [ tTVcon(etR,1) , tTVcon(etR,2) , tTVcon(etR,3) , tTVcon(etR,1) ] ;

for t = 1:1:Nti
    
    tR = itr(t);
    
    tTVcon(tR,1) = nr ; 
    
    tTVcon(tR,2) = TVe(t) ;
    tTVcon(tR,3) = TVe(t+1) ;
 
end


% 4.2.- LVcon:

tLVcon = [ tLVcon ; zeros(3,2) ] ;

for k = 1:1:3
    
    lR = ilr(k);
    
    tLVcon(lR,1) = TVe(k);
    tLVcon(lR,2) = nr;
    
end


% 4.3.- VLcon:
[tVLcon] = um0634_upVLcon (tVLcon, tLVcon, Nni, inr, ilr);


% 4.4.- TLcon :
[tTLcon] = um0633_upTLcon (tTLcon, tTVcon, tLVcon, etR, tNL, Nnli, Nti, itr);


% 4.5.- VTcon:
[tVTcon] = um0632_upTVcon (tVTcon, tTVcon, etR, Nti, itr, Nni, inr);


% 5.- VECTORS:
[tANv, tANe, tXNv, tXNe] = um0631_upNvects (tANv, tANe, tXNv, tXNe, tTVcon, tAv, tXv, Nti, itr);

 

% 6.- RE-ALLOCATES:

MLT.tNE = tNE;
MLT.tNV = tNV;
MLT.tNL = tNL;
MLT.tTVcon = tTVcon;
MLT.tLVcon = tLVcon;
MLT.tVLcon = tVLcon;
MLT.tTLcon = tTLcon;
MLT.tVTcon = tVTcon;

MLT.tBv = tBv;
MLT.tUv = tUv;
MLT.tAv = tAv;
MLT.tXv = tXv;

MLT.tANv = tANv;
MLT.tXNv = tXNv;
MLT.tANe = tANe;
MLT.tXNe = tXNe;
MLT.tNCS = tNCS ;




end

