function [Na, ate, cLR] = um0661_affTL (TR, eTHC, tTVcon, tTLcon, tLVcon, tLTcon, tAv)

% RETURNS:
% Na: No. affected triangles (1 or 2)
% ate(1,2): MLT triangles ref (the latter may be 0 if is interior)
% cLR: common MLT line ref in case there is a line to split

% INPUTS:
% TR: ref. of triangle of MLT
% MLT features

% FOUR SCENARIOS: inside TR and at each edge of TR
% Selects the one that has the best - worst triangle

pudim = 3 ;

a = zeros(pudim,1);
b = zeros(pudim,1);

ate = zeros(4,2);   % attached triangles refs
Na  = zeros(4,1) ;  % No. attached triangles
mro = zeros(4,1) ;  % min. ro number for each scenario

iAv = eTHC.Av'; % node to insert (already projected)


oAnod = zeros(3,pudim); % MLT triangle nodes original coords

for i = 1:1:3
    nR = tTVcon(TR,i);
    oAnod(i,:) = tAv(nR,:);
end


% 1.- SCENARIO 1: within triangle

Tro = zeros(3,1);

for k = 1:1:3
    
    Anod = oAnod ;
    
    Anod(k,:) = iAv(1,:);
    
	%{
    if round(eTHC.Av(2),4) == 0.7996 
        
	fr = 1;
  	pt1372_pTri_iem (oAnod, zeros(3,3), 1, fr, 'm', 0.25, 3, 0, pudim);
	pt1372_pTri_iem (Anod, zeros(3,3), 1, fr, 'k', 1, 1.2, 0, pudim);
 	axis equal
    
    end
	%}
    
    
    ro = tm1304_Ronumber (Anod, pudim); % ro = 1 ideal. The lower ro the worse.

    Tro(k) = ro ;
    
end

Na(1,1)  = 1 ;
ate(1,1) = TR; ate(1,2) = 0;
cLR(1,1) = 0 ;
mro(1,1) = min(Tro);


% 2.- SCENARIO 2 to 4: combined to each edge element:

for s = 1:1:3
    
    LR = tTLcon(TR,s);
    
    cLR(s+1,1) = LR ;
    
    % second triangle attached:
    
    TR2 = 0 ;
    for i = 1:1:2
       
        if tLTcon(LR,i) > 0 && tLTcon(LR,i) ~= TR
            TR2 = tLTcon(LR,i);
        end
            
    end
    
    
    % no triangle attached, divides TR into 2:

    if TR2 == 0
        
        nr = zeros(3,1);
        
        nr(1) = tLVcon(LR,1);
        nr(3) = tLVcon(LR,2);

        for i = 1:1:3
            
            n1 = tTVcon(TR,i);

            if n1~=nr(1) && n1~=nr(3)
                nr(2) = n1;
            end
        end
        
        Tro = zeros(2,1);
        for k = 1:1:2
            
            Anod = zeros(3,pudim) ;

            Anod(1,:) = tAv(nr(k),:);
            Anod(2,:) = tAv(nr(k+1),:);
            Anod(3,:) = iAv(1,:);

            %{
            if round(eTHC.Av(2),4) == 0.7996  
                
            fr = s+1 ;
            pt1372_pTri_iem (oAnod, zeros(3,3), 1, fr, 'm', 0.25, 3, 0, pudim);
            pt1372_pTri_iem (Anod, zeros(3,3), 1, fr, 'k', 1, 1.2, 0, pudim);
            axis equal
            
            end
            %}
            
            ro = tm1304_Ronumber (Anod, pudim); % ro = 1 ideal. The lower ro the worse.
            Tro(k) = ro ;
            
        end
        
        
        Na(s+1,1)  = 1 ;
        ate(s+1,1) = TR; ate(1,2) = 0;
        mro(s+1,1) = min(Tro);
 
    end
    
    
    % one triagle attached at the side => works out the ro number with both triangles split
    
    if TR2 > 0
        
        nr = zeros(5,1);
        
        nr(1) = tLVcon(LR,1);
        nr(3) = tLVcon(LR,2);
        nr(5) = tLVcon(LR,1);
        
        for i = 1:1:3
            
            n1 = tTVcon(TR,i);
            n2 = tTVcon(TR2,i);
            
            if n1~=nr(1) && n1~=nr(3)
                nr(2) = n1;
            end
            
            if n2~=nr(1) && n2~=nr(3)
                nr(4) = n2;
            end
        end
        
        
        Tro = zeros(4,1);
        for k = 1:1:4
            
            Anod = zeros(3,pudim) ;

            Anod(1,:) = tAv(nr(k),:);
            Anod(2,:) = tAv(nr(k+1),:);
            Anod(3,:) = iAv(1,:);

            %{
            if round(eTHC.Av(2),4) == 0.7996  
                
            fr = s+1 ;
            pt1372_pTri_iem (oAnod, zeros(3,3), 1, fr, 'm', 0.25, 3, 0, pudim);
            pt1372_pTri_iem (Anod, zeros(3,3), 1, fr, 'k', 1, 1.2, 0, pudim);
            axis equal
            
            end
            %}
            
            ro = tm1304_Ronumber (Anod, pudim); % ro = 1 ideal. The lower ro the worse.
            Tro(k) = ro ;
            
        end
        
        Na(s+1,1)  = 2 ;
        ate(s+1,1) = TR; ate(s+1,2) = TR2;
        mro(s+1,1) = min(Tro);
        
    end
     
end




% 3.- SELECTS SCENARIO WITH THE HIGHEST mro:

agg = [Na ate cLR mro];
agg = sortrows(agg,5);

Na = agg(4,1);
ate = [agg(4,2) agg(4,3)] ;
cLR = agg(4,4);




%{

tAv(tLVcon(cLR,1),:)
tAv(tLVcon(cLR,2),:)


if jj==0
end



for t = 1:1:tNE
    
    tv = zeros(3,pudim);
    
    for i = 1:1:3
        
        vR = tTVcon(t,i);
        tv(i,:) = tAv(vR,:);
        
    end
    
    IN = go2546_PinTriang (tv, eAv, tolL, tolA, nd);
    
    
    %{
    if round(eAv(3),4)==0.3943 && IN > 0

    	iem = zeros(3,3);

        Anod = zeros(3,pudim);

        for i = 1:1:3
            nR = tTVcon(t,i);
            Anod(i,:) = tAv(nR,:);
        end

        fr = 88 ;

        figure(fr)

        scatter3 ( eAv(1),  eAv(2),  eAv(3), 'r'); hold on

        pt1372_pTri_iem (Anod, iem, 1, fr, 'k', 1, 1.2, 0, pudim);

    end
    %}
    

    if IN == 1
        
        % within triangle:
        
        Na = Na + 1 ;
        
        ate(Na,1) = t ;
        ate(Na,2) = 0 ;

    elseif IN == 2 
        
        % at edge: localizes the edge:
        
        for i = 1:1:3
            
            lR = tTLcon(t,i);
            
            n1 = tLVcon(lR,1);
            n2 = tLVcon(lR,2);
            
            a(:,1) = tAv(n1,:);
            b(:,1) = tAv(n2,:);
            
            ON = go2520_PonSegment (eAv, a,b, pudim, tolL);
            
            if ON == 1
                break
            end
        end
        
    	Na = Na + 1 ;
        
        ate(Na,1) = t ;
        ate(Na,2) = lR ;

    end
    
end

%}

end

