function [tTVcon, tLVcon] = um0663_TVLVcon (tTVcon, tLVcon, tNE, tNL, Nti, itr, Nnli, ilr, Na, ate, cLR, nr)

% RETURNS:
% updated TV/LV conn matrices

% INPUTS:
% original conn matrices
% aet(Na,2): affected triangles and egdes refs
% nr: ref of new node to insert

if Na == 1 && cLR==0
    
    % 1.- INTERIOR:
    
    
    
    % 1.1.- TVcon :

    tTVcon = [tTVcon ; zeros(2,3) ];

    tR = ate(1,1);
    TVe = [ tTVcon(tR,1) , tTVcon(tR,2) , tTVcon(tR,3) , tTVcon(tR,1) ] ;

    for t = 1:1:Nti

        tR = itr(t);

        tTVcon(tR,1) = nr ; 

        tTVcon(tR,2) = TVe(t) ;
        tTVcon(tR,3) = TVe(t+1) ;

    end


    % 1.2.- LVcon:

    tLVcon = [ tLVcon ; zeros(3,2) ] ;

    for k = 1:1:Nnli

        lR = ilr(k);

        tLVcon(lR,1) = TVe(k);
        tLVcon(lR,2) = nr;

    end

    
    
elseif Na == 1 && cLR > 0
    
    % 2.- AT LINE, and line is at edge of MLT:
    
	% 2.1.- TV:
    
    tR = ate(1,1); % triangle ref.
    lR = cLR; % line ref.
    lnR = [ tLVcon(lR,1)  tLVcon(lR,2) ]; % nodes refs of inserted line
    
    TVe = [tTVcon(tR,1) , tTVcon(tR,2) , tTVcon(tR,3) , tTVcon(tR,1) ];

    % selects index of nodes of tR just before the insertion

    for i = 1:1:4

        if (TVe(i) == lnR(1) && TVe(i+1) == lnR(2)) ...
        || (TVe(i) == lnR(2) && TVe(i+1) == lnR(1))

            isd = i ; % index selected
            break
        end

    end

    if isd == 1
        a = [3 1];
        b = [2 3];
    elseif isd == 2
        a = [1 2];
        b = [3 1];
    elseif isd == 3
        a = [2 3];
        b = [1 2];
    end

    % temporal nodes order for elements

    tmp1 = [ tTVcon(tR,a(1)) , tTVcon(tR,a(2)) , nr ] ;
    tmp2 = [ nr ,  tTVcon(tR,b(1)) , tTVcon(tR,b(2)) ] ;

    tTVcon = [tTVcon ; zeros(1,3) ];

    for i = 1:1:3
        tTVcon(tR,i) = tmp1(i);
        tTVcon(tNE,i) = tmp2(i);
    end


    % 2.2.- LV:

    tLVcon = [ tLVcon ; zeros(1,2) ] ;

    tLVcon(lR,1) = tmp1(2);
    tLVcon(lR,2) = nr;

    tLVcon(tNL-1,1) = nr;
    tLVcon(tNL-1,2) = tmp2(2);

    tLVcon(tNL,1) = nr;
    tLVcon(tNL,2) = tmp1(1);
    
    
elseif Na == 2 && cLR > 0

    
    % 3.- AT LINE, and the line is interior to MLT:
    
    lR = cLR;                               % line ref.
  	lnR = [ tLVcon(lR,1)  tLVcon(lR,2) ];   % original nodes refs of inserted line
    
    
    % local refs for allocating to elements and lines
    LER = [ tNE-1 , tNE ] ;
    LLR = [ tNL-2, tNL-1, tNL ] ;
    
    for k = 1:1:2 % loop on elements:
        
        % 3.1.- TV:

        tR = ate(1,k); % triangle ref.

        TVe = [tTVcon(tR,1) , tTVcon(tR,2) , tTVcon(tR,3) , tTVcon(tR,1) ];
        
        % selects index of nodes of tR just before the insertion

        for i = 1:1:4

            if (TVe(i) == lnR(1) && TVe(i+1) == lnR(2)) ...
            || (TVe(i) == lnR(2) && TVe(i+1) == lnR(1))

                isd = i ; % index selected
                break
            end

        end

        if isd == 1
            a = [3 1];
            b = [2 3];
        elseif isd == 2
            a = [1 2];
            b = [3 1];
        elseif isd == 3
            a = [2 3];
            b = [1 2];
        end

        % temporal nodes order for elements

        tmp1 = [ tTVcon(tR,a(1)) , tTVcon(tR,a(2)) , nr ] ;
        tmp2 = [ nr ,  tTVcon(tR,b(1)) , tTVcon(tR,b(2)) ] ;

        tTVcon = [tTVcon ; zeros(1,3) ];

        for i = 1:1:3
            tTVcon(tR,i)     = tmp1(i);
            tTVcon(LER(k),i) = tmp2(i);
        end

        

        % 3.2.- LV:

        if k == 1
            
            tLVcon = [ tLVcon ; zeros(2,2) ] ;

            tLVcon(lR,1) = tmp1(2);
            tLVcon(lR,2) = nr;

            tLVcon(LLR(1),1) = nr;
            tLVcon(LLR(1),2) = tmp2(2);

            tLVcon(LLR(2),1) = nr;
            tLVcon(LLR(2),2) = tmp1(1);

        elseif k == 2
            
            tLVcon = [ tLVcon ; zeros(1,2) ] ;
            
            tLVcon(LLR(3),1) = nr;
            tLVcon(LLR(3),2) = tmp1(1);
            
        end
        
    end
    

    
end
    


end

