function [NFC, uNFC] = fs0710_fit2conto ...
    (Spatches, pR, GPT, tFace, ccTp, Ntc, tcR, Ns, tolSt, Xtol, DUtol)


% RETURNS:
% Nodes of contour curves of trimming surface. with this nodes the
% quadratic interpolation has an error less than the tolerance

% structure NFC 'nodes fitted to contours'
% Nc(TNcu,1): No. nodes per curve, include initial and final, and mid-nodes for each curve !
% Bn, Xn, An: coordinates in curve param, physical and patch param.
% Note: in these arrays, those curves that are not trimming are just filled with zeros

% INPUTS:
% TNcu: total No.curves
% Ns: No. sample points

agCurv = ccTp(pR,1).agCurv;
TNcu = size(agCurv,1); % total Number of curves


NFC = struct('Nc',[], 'Bn',[], 'Xn',[], 'An',[]);

uNFC(TNcu,1) = struct('Nsr',0 , 'sr',[], 'Un',[]);

sudim = 2 ;
pudim = 3 ;
xdim  = 3 ;

mNc = zeros(TNcu,1);
mBn = zeros(TNcu,1);
mXn = zeros(TNcu,1, xdim);
mAn = zeros(TNcu,1, pudim);

h = 1 /(Ns-1); % uses same step for sampling and for derivative estimation


% 1.- COMPUTES POINTS FOR THE CURVES IN CURVE PARAM. SPACE:

cDuc = zeros(TNcu,Ns); % stores addmisible spacing for each trimming curve (used for printing the examples exclusively)

for j = 1:1:Ntc

    % 1.1.- Curve and patch references:
    
    cR = tcR(j,1);
    
    % 1.2.- Sample points:
    
    cUs = [0:h:1]';
    cUs(end) = 1;


    % 1.3.- Max. allowable step:

    [cDu, cN3d] = fs0711_SreqdC (Spatches, pR, agCurv, cR, Ns, cUs, tolSt, Xtol, DUtol, h);
      

    % 1.4.- Computes nodes, including mid-nodes of the quadratic interpolation:

    [iNv, iUv] = fs0721_iNodes (cUs, Ns, cDu, cR);
    

    % 1.5.- Allocates:
    
    mNc(cR,1) = iNv;
    
    for i = 1:1:iNv
       mBn(cR,i) = iUv(i); 
    end
    
    for i = 1:1:Ns
        cDuc(cR,i) = cDu(i);
    end
    
end






% 3.- COMPUTES X AND A COORDS:

for j = 1:1:Ntc
    
    cR = tcR(j,1);
    
	cDegree = agCurv(cR,1).Degree ;
    cNcp    = agCurv(cR,1).Ncp ;
    cKnotV  = agCurv(cR,1).KnotV ;
    cP      = agCurv(cR,1).P ;
    cw      = agCurv(cR,1).w ;
    
	pDegree = Spatches(pR,1).INS(1).mDegree ;
    pNcp    = Spatches(pR,1).INS(1).mNcp ;
    pKnotV  = Spatches(pR,1).INS(1).mKnotV ;
    pUsize  = Spatches(pR,1).INS(1).mUsize;
    pP      = Spatches(pR,1).INS(1).mP ;
    pw      = Spatches(pR,1).INS(1).mw ;
    pCD     = zeros(pudim,1);

    n = mNc(cR,1);
    
    cX = zeros(n,xdim);

    
    % physical coordinates:
    
    for v = 1:1:n
        
        [X] = b2601_BNURBS_Xcoord (mBn(cR,v), cKnotV, cNcp,cDegree, cNcp+cDegree+1 , cw,cP, 1, xdim);
        
        cX(v,:) = X(:,1);
        mXn(cR,v,:) = X(:,1);
    end
    
    
    % patch param. coordinates:
    
    [cA] = v2861b_PPvol (cX, n, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, xdim, Xtol, DUtol);

    for v = 1:1:n
        mAn(cR,v,:) = cA(v,:);
    end
    
    
    
    % surface param. coords:
    
    CScon = ccTp(pR,1).CScon;
   
    Nsr = 0 ;
    sr = zeros(1,1);
    
    for i = 1:1:size(CScon,2)
        
        if CScon(cR,i) > 0
            Nsr = Nsr + 1 ;
            sr(Nsr,1) = CScon(cR,i);
        end
    end
    
    Un = zeros(Nsr, n, sudim);
    

    for k = 1:1:Nsr
        
        sDegree = tFace(1,pR).mSur(sr(k),1).Degree;
        sNcp    = tFace(1,pR).mSur(sr(k),1).Ncp;
        sKnotV  = tFace(1,pR).mSur(sr(k),1).KnotV;
        sUsize  = tFace(1,pR).mSur(sr(k),1).Usize;
        sP      = tFace(1,pR).mSur(sr(k),1).P;
        sw      = tFace(1,pR).mSur(sr(k),1).w;
        sCD     = tFace(1,pR).mSur(sr(k),1).CS;
        
        [cU] = s2831b_PPsurf (cX, n, sDegree, sNcp, sKnotV, sUsize, sP, sw, sCD, xdim, Xtol, DUtol);
        
        for i = 1:1:n
            Un(k,i,:)= cU(i,:);
        end
        
    end
    
  	uNFC(cR,1).Nsr = Nsr ;
    uNFC(cR,1).sr  = sr ;
    uNFC(cR,1).Un  = Un ;
    

end



% plots curves (for the example of the thesis):
%{

cR = 3 ;

figure(89)
   
% Profile of admissible spacing x 2 :

for i = 1:1:Ns
    plot([cUs(i) cUs(i)],[0 2*cDuc(cR,i)],'b'); hold on
end
        
plot(cUs,2*cDuc(cR,:),'k', 'LineWidth',2.1); hold on

n = mNc(cR,1);

ne = (n+1)/2;
Bne = zeros(ne,1);
Ane = zeros(ne,3);
        
conta = 0 ;
for i = 1:2:n
    conta = conta+1;
    Bne(conta,1) = mBn(cR,i);
    Ane(conta,:) = mAn(cR,i,:);
end
        
scatter(Bne(:,1),zeros(ne,1),'k', 'Filled'); hold on
  
% axis equal

% curve in A-space:

figure(90)

scatter3(Ane(:,1),Ane(:,2),Ane(:,3),'k', 'Filled'); hold on
plot3(Ane(:,1),Ane(:,2),Ane(:,3),'k','lineWidth',2.2); hold on

% axis equal

if gg==0
end

%}



% 4.- CONSIDERS GPT TETRAHEDRONS SIZE TO REFINE:
% In case the curves nodes spacing is greater that the  
% tets representative edge length, insertion happens.

%ACTIVATE_THIS = 1 
% {

rcaL = 1.67 ; % limit for ratio Lc/La, if Lc/La > rcaL => divides segment

for j = 1:1:Ntc
    
    cR = tcR(j,1);
    

    [Ncn, cBn, cAn, cXn, uNFC] = fs0725_tetDis ...
        (pR, cR, Spatches, tFace, agCurv, rcaL, mNc, mBn, mXn, mAn, uNFC, GPT, Xtol, DUtol);

    % transfers:
    
    mNc(cR,1) = Ncn ;
    
    for i = 1:1:Ncn 
        mBn(cR,i)   = cBn(i,1);
        mAn(cR,i,:) = cAn(i,:);
        mXn(cR,i,:) = cXn(i,:);
    end
    

end

%}

% 5.- ALLOCATES TO STRUCTURE:

NFC.Nc = mNc ;
NFC.Bn = mBn ;
NFC.Xn = mXn ;
NFC.An = mAn ;





end

