function [cDu, cN3d] = fs0711_SreqdC (Spatches, pR, agCurv, cR, Ns, cUs, tolSt, Xtol, DUtol, h)

% RETURNS: 
% cDu(Ns,1): required distance in curve param space to reduce below tolerance

% INPUTS:

cudim = 1 ;
pudim = 3 ;
xdim = 3 ;

cDu  = zeros(Ns,1);
cN3d = zeros(Ns,1);


% 1.- EXTRACTS DATA FROM RELEVANT PATCH:
    
pDegree = Spatches(pR,1).INS(1).mDegree ;
pNcp    = Spatches(pR,1).INS(1).mNcp ;
pKnotV  = Spatches(pR,1).INS(1).mKnotV ;
pUsize  = Spatches(pR,1).INS(1).mUsize;
pP      = Spatches(pR,1).INS(1).mP ;
pw      = Spatches(pR,1).INS(1).mw ;
pCD     = zeros(pudim,1);
    
% param. limits and admisible error in absolute value
    
pULims = zeros(xdim,2);
Udiag = 0 ;
    
for di = 1:1:pudim
        
    pULims(di,1) = pKnotV(di,1);
	pULims(di,2) = pKnotV(di,pUsize(di));
        
	Udiag = Udiag + (pULims(di,2) - pULims(di,1))^2 ;
        
end
Udiag = Udiag^0.5;
    
Eadm = Udiag * tolSt/100;
    
    
% 2.- EXTRACTS DATA FROM CURVE:
 
% Us = zeros(Ns,cudim);  

cDegree = agCurv(cR,1).Degree ;
cNcp    = agCurv(cR,1).Ncp ;
cKnotV  = agCurv(cR,1).KnotV ;
cUsize  = agCurv(cR,1).Usize ;
cP      = agCurv(cR,1).P ;
cw      = agCurv(cR,1).w ;
cULims = [ cKnotV(1,1)  cKnotV(1,cUsize)]; 



% 3.- COMPUTES DISTANCE to 
% reduce error < tolS in patch param. space.


% 3.1.- Sample points of the curve in patch parameter space:

cXi = zeros(Ns,xdim);

for i = 1:1:Ns
	[X] = b2601_BNURBS_Xcoord (cUs(i), cKnotV, cNcp, cDegree, cNcp+cDegree+1, cw,cP, 1, xdim);
    cXi(i,:) = X(:,1);
end

f = 1 ; % factor to improve accuracy, e.g. f = 0.01 ;
[cAi] = v2861b_PPvol (cXi, Ns, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, xdim, f*Xtol, f*DUtol);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
if cR == 1
     
    figure(88)
    scatter3(cAi(:,1), cAi(:,2),cAi(:,3),'r'); hold on
    axis equal
      
end
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i = 1:1:Ns
   
	% 3.2.- Computes points for derivative:
    [Ai1, Ai2] = fs0793_Cdi (cAi, Ns, i);
 
    
	% 3.3.- Req'd distance bt'n nodes to reduce it to tolS:
	[smax, nC3d] = fs0794_SmaxC (Ai1, Ai2, h, Eadm, xdim, cR);


    if smax > 1
        smax = 1 ; 
    end
     
	cDu(i,1)  = smax ;
	cN3d(i,1) = nC3d ;
      
end




end
