function [NFS] = fs0730_fit2inner ...
    (GPT, Spatches, pR, tFace, Nts, tsR, Nsd, eNFC, NFC, ccTp, tolSt, Xtol, DUtol)

NFS = struct('Ns',[] , 'Un',[] , 'Xn',[] , 'An',[], 'Bn',[] ) ;

sudim = 2 ;
pudim = 3 ;
xdim  = 3 ;

Nf = tFace(pR).Nfa;

NFS.Ns = zeros(Nf,1);
NFS.Bn = zeros(Nf,1);
NFS.Un = zeros(Nf,1, sudim);
NFS.Xn = zeros(Nf,1, xdim);
NFS.An = zeros(Nf,1, pudim);

ncuT    = ccTp(pR,1).ncu;
nloT    = ccTp(pR,1).nlo;
SCconT  = ccTp(pR,1).SCcon;

% 1.- COMPUTES POINTS FOR THE SURFACES IN SURF. PARAM. SPACE:

for j = 1:1:Nts
    
    % 1.1.- Surface reference:
    
    sR = tsR(j,1);

    NL = nloT(sR,1); % No. loops of the surface
    
    sDegree = tFace(pR).mSur(sR).Degree ;
    sNcp    = tFace(pR).mSur(sR).Ncp ;
    sKnotV  = tFace(pR).mSur(sR).KnotV ;
    sUsize  = tFace(pR).mSur(sR).Usize ;
    sP     = tFace(pR).mSur(sR).P ;
    sw     = tFace(pR).mSur(sR).w ;
    [sEw,sEP] = GEN_12_ExpandBw (sw, sP, xdim, sudim, sNcp);   
    
	pDegree = Spatches(pR,1).INS(1).mDegree ;
    pNcp    = Spatches(pR,1).INS(1).mNcp ;
    pKnotV  = Spatches(pR,1).INS(1).mKnotV ;
    pUsize  = Spatches(pR,1).INS(1).mUsize;
    pP      = Spatches(pR,1).INS(1).mP ;
    pw      = Spatches(pR,1).INS(1).mw ;
    pCD     = zeros(pudim,1);
    

    % 1.2.- Constructs the contours:

    [CNT] = fs0792_Sconto (pR, sR, tFace, NFC, NL, ncuT, SCconT, Xtol, DUtol);
    [eCNT] = fs0792_Sconto (pR, sR, tFace, eNFC, NL, ncuT, SCconT, Xtol, DUtol);

    
    %{
    if sR == 4
        
        figure(45)
        plot(CNT.cU(1,:,1), CNT.cU(1,:,2), 'r'); hold on
        axis equal
        
        
        figure(46)
        plot3(CNT.cA(1,:,1), CNT.cA(1,:,2), CNT.cA(1,:,3), 'r'); hold on
        axis equal
        
        
        
        if gg==4
        end
        
    end
    %}
 
    
    % 1.3.- Max. allowable step:

    [sDu, Us, Ns] = fs0731_SreqdS (Spatches, pR, tFace, sR, Nsd, CNT.cU, CNT.Nc, tolSt, Xtol, DUtol);
     
    %%%%%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%
    % max allowable step
    %{
    
        figure(84)
        for k = 1:1:Ns
            
            scatter3 ( Us(k,1), Us(k,2),sDu(k,2),'r'); hold on
            
        end
        
        axis equal
        
        if hh==0
        end
    %}
    %%%%%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%
 
    % 1.4.- Computes nodes, WITH NO mid-nodes of the quadratic
    % interpolation, i.e. uses 2 x sDu distance:

    [iNn, iUn] = fs0741_iSnodes (Us, Ns, sDu, CNT.cU, CNT.Nc, NL, sR);


    % 1.5.- Computes coords in other spaces:
    
    % stores in sX and sA
    % deletes those too close
    % stores in structure
    
    if iNn > 0
        
      	% Computes X and A coordinates:

        iXn = zeros(iNn,xdim);
        u = zeros(sudim,1);

        for i = 1:1:iNn 

            u(:,1) = iUn(i,:);
            [X] = b2601_BNURBS_Xcoord (u, sKnotV, sNcp, sDegree, sUsize , sEw, sEP, sudim, xdim);
            iXn(i,:) = X(:,1);
        end

        iAn = v2861b_PPvol (iXn, iNn, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, xdim, Xtol, DUtol);

        
        % Deletes those points closer than thresold distance to the contours in patch parameters:
        
        pc = 50 ; % percentaje of closest contour segment, to remove one node
        nd = 3;

        [Un, An, Xn, Nn] = fs0746_DelNclo (iUn, iAn, iXn, iNn, NL, eCNT.Nc, eCNT.cA, pc, nd);
        
        
        % Allocates:
        
        NFS.Ns(sR,1) = Nn;

        for i = 1:1:Nn
            NFS.Bn(sR,i,:) = 0; % no curve is attached to these nodes
            NFS.Un(sR,i,:) = Un(i,:);
            NFS.An(sR,i,:) = An(i,:);
            NFS.Xn(sR,i,:) = Xn(i,:);
        end

    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % plots retained nodes in patch param space:
    %{

    if sR==4
    

        figure(62+sR)

        scatter3(NFS.An(sR,:,1), NFS.An(sR,:,2), NFS.An(sR,:,3),'r'); hold on

        x = zeros(CNT.Nc(1),pudim);
        
        for i = 1:1:CNT.Nc(1) 
            x(i,:) = CNT.cA(1,i,:); 
        end
        plot3( x(:,1),  x(:,2), x(:,3), 'b'); hold on

        axis equal

        if gg==0
        end
        
    end
    %}
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


	% 1.6.- Compares with involved GPT-tets size:
    
    rcaL = 1.0 ;
    endF = 0 ;

    contaL = 0 ;

    % Activate this!
    % {
    while endF == 0
    
        contaL = contaL + 1 ;

        [NFS, endF] = fs0745_StetDis ...
        (contaL, GPT, pR, sR, Spatches, tFace, NL, NFS, eCNT, ccTp, Xtol, DUtol, rcaL);


    end
%}

end




end

