function [iNv, iUv] = fs0741_iSnodes (Us, Ns, sDu, cU, Nc, NL, sR)

% RETURNS:
% iNv: No. iserted nodes
% iUv(iNv,sudim): their coordinates in surface parameter space

sudim = 2 ;
Du = 2*sDu ; % max. allowable distance

% 1.- MESH REFINEMENT OVER THE WHOLE SURFACE:
% applies 2D-octree division

% Initial mesh:

NE = 1 ;                % No. elements
Nn = 4 ;                % No. nodes
Cm = [ 0 0 ; 0 1 ; 1 0 ; 1 1 ];   % mesh nodes coordinates
ENc = [ 1 2 3 4 ];      % connectivity matrix

u = zeros(sudim,1);

endF = 0 ;

% 'virgin' flags, turn to 0 when the the insertion happens in each direction
viF = ones(2,1) ; 

while endF == 0

    endF = 1 ; % changes to 0 if at least one refinement is done:
    
    for i = 1:1:Ns
        
        % find element where sample point lies in:
        
        u(:,1) = Us(i,:);
        
        for e = 1:1:NE
            
            IN = fs0742_Ein (u, e, ENc, Cm);
            
            if IN==1
               break 
            end
            
        end
        

        % checks element distances and refines if required:
        
        Dui = [Du(i,1), Du(i,2)];
        
        [NE, Nn, ENc, Cm, rF, viF] = fs0743_Mref (e, NE, Nn, ENc, Cm, Dui, viF);
        
        if rF == 1
     	endF = 0 ;
        end
    end
 
    
end



% Removes repeated nodes:

[Cm ,~] = GEN_16_RemoveRepeatedRows (Cm, sudim, 0.00001);

Nn = size(Cm,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plots Cm (all of them) and contour
%{
if sR==4
    
    figure(42+sR)

    scatter(Cm(:,1), Cm(:,2),'r'); hold on

    for i =1:1:Nn
        text( Cm(i,1), Cm(i,2), num2str(i)); hold on
    end
    
    
    x = zeros(Nc(1),2);
    for i = 1:1:Nc(1) 
        x(i,:) = cU(1,i,:); 
    end
    plot( x(:,1),  x(:,2), 'b'); hold on
    
    axis equal
    
end

if gg==0
end

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% 2.- REMOVES NODES OUTSIDE THE CONTOURS & CLOSER THAN pc TO EDGES, and ALLOCATES:

Ncm = size(Cm,1);

[iNv, iUv] = fs0744_nodesIN2 (Ncm, Cm, Nc, cU, NL);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plots retained nodes
%{

    figure(52+sR)

    scatter(iUv(:,1), iUv(:,2),'r'); hold on
    
    x = zeros(Nc(1),2);
    for i = 1:1:Nc(1) 
        x(i,:) = cU(1,i,:); 
    end
    plot( x(:,1),  x(:,2), 'b'); hold on
    
    axis equal

    if gg==0
    end
    
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


end

