function [rskin] = fs0750_triangSkin ...
    (pR, sR, Spatches, tFace, NFC, eNFC, NFS, ccTp, Xtol, DUtol)

pudim = 3 ;
sudim = 2 ;
xdim  = 3 ;

ncuT    = ccTp(pR,1).ncu;
NL      = ccTp(pR,1).nlo(sR,1);
SCconT  = ccTp(pR,1).SCcon;


% 1.- CONTOURS WITH ALL NODES AND WITH ONLY END-NODES:  

[CNT]   = fs0792_Sconto (pR, sR, tFace, NFC,  NL, ncuT, SCconT, Xtol, DUtol);
[eCNT]  = fs0792_Sconto (pR, sR, tFace, eNFC, NL, ncuT, SCconT, Xtol, DUtol);

% plots nodes in Q^space
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{

fr = 100 ;

figure(fr) 

for L = 1:1:NL
    
    Uc = zeros(eCNT.Nc(L),sudim);
    
    for i = 1:1:eCNT.Nc(L)
   	Uc(i,:) = eCNT.cU(L,i,:); 
    end
    
    plot( Uc(:,1), Uc(:,2),'r'); hold on
    scatter( Uc(:,1), Uc(:,2),'r','filled'); hold on
    
end

Ui = zeros(NFS.Ns(sR),sudim);

for i = 1:1:NFS.Ns(sR)
    Ui(i,:) = NFS.Un(sR,i,:);
end

scatter( Ui(:,1), Ui(:,2),'b','filled'); hold on

axis equal

xlim([0 1])
ylim([0 1])

if gg==0
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}



% 2.- LINEAR TRIANGULATION:

[NV, NE, TRC, conn, NL, Nlv] = fs0751_LinTrin (sR, eCNT, NFS, NL);
% included previously CNT



% plots intiial linear triangulation (without improvement)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{

fr = 5 ;
Cv = TRC.Av ;

NLI = size(conn.LV,1);
dim = size(Cv,2);

LVcon = conn.LV ;
TVcon = conn.TV ;

Mthk = 1 ;

nvF = 0 ;
Nvx = 0 ;
neF = 0 ;
Nel = 0 ;
sc = 1 ;
Ncolr = 'r' ;
lrF = 0 ;
vrF = 0 ;
erF = 0 ;

pt1366_pTriLinMesh_nD ...
(Cv, NLI, LVcon, TVcon, dim, fr, 'k', Mthk, nvF, NV, Nvx, neF, NE, Nel, sc, Ncolr, lrF, vrF, erF);

for i = 1:1:size(Cv,1)
    
    %scatter(Cv(:,1),Cv(:,2),'k','Filled'); hold on
    
end

if gg==0
end

%}

% preliminary improvement:

pL = 60 ;
[NV, NE, TRC, conn, NL, Nlv] = fs0754_IMPRVtri1 ...
(NV, NE, TRC, conn, NL, Nlv, ccTp, Spatches, pR, tFace, sR, Xtol, DUtol, pL, 0);



% plots improved linear triangulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{

fr = 6 ;
Cv = TRC.Av ;

NLI = size(conn.LV,1);
dim = size(Cv,2);

LVcon = conn.LV ;
TVcon = conn.TV ;

Mthk = 1 ;

nvF = 0 ;
Nvx = 0 ;
neF = 0 ;
Nel = 0 ;
sc = 1 ;
Ncolr = 'r' ;
lrF = 0 ;
vrF = 0 ;
erF = 0 ;

pt1366_pTriLinMesh_nD ...
(Cv, NLI, LVcon, TVcon, dim, fr, 'k', Mthk, nvF, NV, Nvx, neF, NE, Nel, sc, Ncolr, lrF, vrF, erF);

if gg==0
end

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 3.- INSERTS MID-NODES & IMPROVES:
% At contours it retrieves from contour curves, at surface adds just at the U-middle


%{

[NVm, Bvm, Uvm, Avm, Xvm, econn, mNC] = fs0761_finalizeT ...
(NV,  TRC.Bv, TRC.Uv, TRC.Av, TRC.Xv, conn,  TRC.NC, ccTp, NE, NL, CNT, tFace, sR, Spatches, pR, Xtol, DUtol);

TVcon = econn.TV ;
VTcon = econn.VT ;
LVcon = econn.LV ;
LTcon = econn.LT ;
VLcon = econn.VL ;
TLcon = econn.TL ;

%}

% x.- PLOTTING (only for explanations):

%{

ssR = 1 ; % selected surface reference

Cvm = Avm ;

Cv = Cvm(1:NV,:) ;

fr = [ 9 10 20 30 ] ;

dim = size(Cv,2);
NLI = size(LVcon,1);

NE = size(TVcon,1);

if sR == ssR
    
    % 7.5.- Triangulation: mid-nodes inserted
    
    stL = 0.1 ;
    stT = 0.1 ;
    
	eCol = 'k';

    eF = 1;
    sF = 0;
    nF = 0;

    lrF = 0;
    vrF = 0;
    erF = 0;
    
    
    pt1367_pTriQuaMesh ...
    (stL, stT, Cvm, NLI, econn.LV, econn.TV, fr(4), NE, eF, sF, nF, lrF, vrF, erF, dim, eCol);



if gg==0
end
    
    
end


%}




% 4.- NORMALS AND TRIANGLE AREAS:

spF = 0 ; % master patch

[Ae, XNe, XNv] = tm1308_TmeshNA (spF, NE, NV, conn.TV, conn.VT, TRC.Xv, xdim);
[~,  ANe, ANv] = tm1308_TmeshNA (spF, NE, NV, conn.TV, conn.VT, TRC.Av, pudim);
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
    % plot surface
    
    %if sR==1
        
        fr = 3 ;
        
        Cv = Avm;

        dim = size(Cv,2);
        
        eCol = 'k';
        
        NLI = size(LVcon,1);
        
        neF = 1;
        nvF = 1;

        lrF = 0;
        vrF = 0;
        erF = 0;
       
        sc = 10 ;
        
        pt1366_pTriLinMesh_nD ...
        (Cv, NLI, LVcon, TVcon, dim, fr, 'k', 1.2, nvF, NV, XNv, neF, NE, XNe, sc, 'r', lrF, vrF, erF)

        % pt1367_pTriQuaMesh (0.1, 0.1, Cv, NLI, LVcon, TVcon, fr, NE, eF, sF, nF, lrF, vrF, erF, dim, eCol)
    
        if kk==0
        end
        
    %end
    
    
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    

% 5.- COORDINATES IN TERMS OF CURVE PARAMETER SPACE:



    
% 6.- ALLOCATES:
    
rskin = cTskin;

    for k = 1:1:1 % (this loop is only to being able to contract the allocation code)
    
    rskin.mDegree = tFace(pR).mSur(sR).Degree ;
    rskin.mKnotV = tFace(pR).mSur(sR).KnotV ;
    rskin.mUsize = tFace(pR).mSur(sR).Usize ;
    rskin.mNcp   = tFace(pR).mSur(sR).Ncp ;
    rskin.mP     = tFace(pR).mSur(sR).P ;
    rskin.mw     = tFace(pR).mSur(sR).w ;

    rskin.mPN = tFace(pR).mSur(sR).PL ;
    rskin.mCS = tFace(pR).mSur(sR).CS ;
    
    rskin.mNE = NE ;
    rskin.mNV = NV ;
    rskin.mNVm = NV ;
    %rskin.mNVm = NVm ;
    
    rskin.mBv = TRC.Bv ;
    rskin.mUv = TRC.Uv ;
    rskin.mAv = TRC.Av ;
    rskin.mXv = TRC.Xv ;

    rskin.mTVcon = conn.TV ;
    rskin.mVTcon = conn.VT ;
    rskin.mLVcon = conn.LV ;
    rskin.mVLcon = conn.VL ;
    rskin.mLTcon = conn.LT ;
    rskin.mTLcon = conn.TL ;

    rskin.mArea = Ae ;

    rskin.mANv = ANv ;
    rskin.mANe = ANe ;

    rskin.mXNv = XNv ;
    rskin.mXNe = XNe ;


    rskin.mNlo = NL ;
    
    Nlv = CNT.Nc - ones(NL,1);
    rskin.mNlv = Nlv ;
    
    rskin.mNC = TRC.NC ;
    
    end

end

