function [NV, NE, TRC, conn, NL, Nlv] = fs0754_IMPRVtri1 ...
         (NV, NE, TRC, conn, NL, Nlv, ccTp, Spatches, pR, tFace, sR, Xtol, DUtol, pL, contaL)


% RETURNS: improved triangulation in patch parameter space

% INPUTS:
% pL  percentaje to establish the threshold length for small lines removal

% Swap edge-line between 2 triangles if:
% - the alternative line is shorter
% and
% - the resultant triangles improve the regularity (condition number)

% Removes 'small lines'


pudim = 3 ;
sudim = 2 ;


% 0.- PRELIMINARIES:

endF = 0 ;  % ends flag turns to 0 if changes happen

% No. lines and verteces:
NLI = size(conn.LT,1);


Ltol  = 20 ; % demasas admited in %
Rtol = 20 ;


omNC = TRC.NC ;


% 1.- LOOP ON LINES TO ANALYZE PAIRS OF TRIANGLES:

contaF = 0 ;

while endF == 0

    endF = 1 ;

    contaF = contaF + 1 ;
    
    LRc = 0 ; % stores temporarily the lastest changed line ref.

    for LR = 1:1:NLI

	% triangles refs and nodes involved:
 	[tR, nR] = fs0755_NTrefs (conn, LR);

	if tR(2) > 0 && LR ~= LRc % It does not consider: edge-lines and LR = previously changed LR

            
        % Flips de line if :
        
     	% - Shape is improved by Lc/La ratio and triang, regularity AND
        % - in U space both lines are not coincident            AND
     	% - alternative mid-point is not at contour (coF = 1)   AND
        % - no node can lie within one of the resultant triangles
    	
        % 1.1.- Diagonals lengths and regularity:

        Lc = 0 ;
        La = 0 ;
            
        for di = 1:1:pudim
            Lc = Lc + (TRC.Av(nR(1),di) - TRC.Av(nR(3),di))^2 ;
        	La = La + (TRC.Av(nR(2),di) - TRC.Av(nR(4),di))^2 ;
        end
            
    	Lc = Lc^0.5 ;
     	La = La^0.5 ;

        % Regularity
        [roc, roa] = fs0756_regTri (nR, TRC.Av);

        [fpF] = fs0763_evaFlip (Lc, La, roc, roa, Ltol, Rtol);
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %{
        if LR == 73

            La
            Lc
            
        end
        %}
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
       
        if fpF == 1

            % 1.2.- Checks if there are coincident lines in surf. parameter space:
            
            dg = 2.5 ;
            ucF = fs0759_UlinCoin (nR, TRC.Uv, dg);
            
            
            if ucF == 0

                % 1.3.- All alternative nodes not at contour curves:

                coF = fs0757_alterContr(nR, TRC.NC, TRC.Bv, TRC.Xv, ccTp, pR) ; % contour flag

                if coF==0

                    % 1.4.- There is no node inside triangle in surface 
                	%       param. space in the alternative arrangement:
                        
                    [niF] = fs0760_NodIN (nR, TRC.Uv);
                        
                 	if niF == 0
                        
                        %{
                            if LR == 234

                                At = zeros(5,3);
                                Ut = zeros(5,2);
                                r = [1 2 3 4 1];

                                for i = 1:1:5
                                    At(i,:) = Avm(nR(r(i)),:);
                                    Ut(i,:) = Uvm(nR(r(i)),:);
                                end

                                Ac = [ Avm(nR(1),:) ; Avm(nR(3),:) ] ;
                                Aa = [ Avm(nR(2),:) ; Avm(nR(4),:) ] ;

                                Uc = [ Uvm(nR(1),:) ; Uvm(nR(3),:) ] ;
                                Ua = [ Uvm(nR(2),:) ; Uvm(nR(4),:) ] ;

                                figure(2)
                                plot3(At(:,1), At(:,2), At(:,3), 'k'); hold on
                                plot3(Ac(:,1), Ac(:,2),Ac(:,3),'r'); hold on
                                axis equal

                                figure(3)
                                plot3(At(:,1), At(:,2), At(:,3), 'k'); hold on
                                plot3(Aa(:,1), Aa(:,2),Aa(:,3),'r'); hold on
                                axis equal

                                figure(20)
                                plot(Ut(:,1), Ut(:,2), 'k'); hold on
                                plot(Uc(:,1), Uc(:,2), 'r'); hold on
                                axis equal

                                figure(30)
                                plot(Ut(:,1), Ut(:,2), 'k'); hold on
                                plot(Ua(:,1), Ua(:,2), 'r'); hold on
                                axis equal

                            end
                            %}

                        % Updates conn. matrices to flip the common-line:
                       	[conn] = fs0758_upconnLin (TRC.Av, NV, conn, tR, nR, LR);

                     	%{
                            if contaL==2 && (nR(1)==191 || nR(2)==191 || nR(3)==191 || nR(4)==191)

                                cc = cc + 1 ;

                                %%%%%%%%%%%%%%%%%   %%%%%%%%%%%%%%
                                % plots triangulation
                                %{

                                if pR == 1

                                    fr = 100+cc ;

                                    figure(fr)

                                    Cv = Avm;

                                    NLI = size(conn.LV,1);
                                    NV = size(Cv,1);
                                    dim = size(Cv,2);

                                    if dim == 2
                                        scatter( Cv(:,1), Cv(:,2),'r'); hold on
                                    elseif dim == 3
                                        scatter3( Cv(:,1), Cv(:,2), Cv(:,3),'r'); hold on
                                    end

                                    lrF = 0;
                                    vrF = 0;
                                    erF = 0;

                                    pt1366_pTriLinMesh_nD ...
                                    (Cv, NLI, conn.LV, conn.TV, dim, fr, 'k', 1.3, 0, NV, 0, 0, NE, 0, 0, 0, lrF, vrF, erF);

                                end

                                %}
                                %%%%%%%%%%%%%%%%%   %%%%%%%%%%%%%%



                            end
                            %}

                        endF = 0 ; % ending flag turns to 0 
                        LRc = LR; % line ref. changed
                      	break
                        
                    end
                        
                end
                     
            end
        end
	end
    end

    
    %%%%%%%%%%%%%%%%%   %%%%%%%%%%%%%%
    % plots triangulation
    %{

    if pR == 1 && contaL == 3 && (contaF == 4 || contaF == 5)

        
        
        fr = 300 + contaF ;

        figure(fr)

        Cv = TRC.Av;

        NLI = size(conn.LV,1);
        dim = size(Cv,2);

        if dim == 2
            scatter( Cv(:,1), Cv(:,2),'r'); hold on
        elseif dim == 3
            %scatter3( Cv(:,1), Cv(:,2), Cv(:,3),'r'); hold on
        end

        lrF = 0;
        vrF = 0;
        erF = 0;

        pt1366_pTriLinMesh_nD ...
        (Cv, NLI, conn.LV, conn.TV, dim, fr, 'r', 1.3, 0, 0, 0, 0, NE, 0, 0, 0, lrF, vrF, erF);

    end

    %}
    %%%%%%%%%%%%%%%%%   %%%%%%%%%%%%%%

end


%%%%%%%%%%%%%%%%%   %%%%%%%%%%%%%%
% plots triangulation
%{

if pR == 1 && contaL == 3

    fr = 200 + contaL ;
    
    figure(fr)

    Cv = TRC.Av;

    NLI = size(conn.LV,1);
    dim = size(Cv,2);

    if dim == 2
        scatter( Cv(:,1), Cv(:,2),'r'); hold on
    elseif dim == 3
        %scatter3( Cv(:,1), Cv(:,2), Cv(:,3),'r'); hold on
    end
    
    lrF = 0;
    vrF = 0;
    erF = 0;

    pt1366_pTriLinMesh_nD ...
    (Cv, NLI, conn.LV, conn.TV, dim, fr, 'r', 1.3, 0, 0, 0, 0, NE, 0, 0, 0, lrF, vrF, erF);


    if gg==0
    end

end

%}
%%%%%%%%%%%%%%%%%   %%%%%%%%%%%%%%




% 2.- DELETES SMALL LINES:
% Removes those nodes that are too close by merging them (and deleting the triangles):

[RLT] = uc1906_RLtri (TRC.Av, conn.TV, pudim); % representative lenght of triangles


cLoop = 0 ;
endF = 0;
while endF == 0

    cLoop = cLoop + 1 ;
    
    endF = 1 ;
    
    %[contaL, cLoop]
    
    % Searches one line shorter than threshold
    
    for LR = 1:1:NLI
    if conn.LT(LR,2) > 0 % only interior lines (with 2 triangles attached)
        
        
        % Line length:
        
        LL = uc1907_lineL (conn.LV, LR, TRC.Av);  
        
        
        % Threshold length:
        
        r1 = conn.LT(LR,1);
        r2 = conn.LT(LR,2);
        
        tL = (RLT(r1) + RLT(r2)) / 2 * pL/100 ;
        
        
     	% Evaluates and corrects:

        if LL < tL

            oAvm = TRC.Av;
            oNLI = NLI ;
            oconn = conn ;
            oNE = NE ;
            
            [NV, NE, NLI, TRC, conn] = uc1900_upconnLR ...
            (NV, NE, NLI, TRC, conn, NL, Nlv, LR, Spatches, pR, tFace, sR, Xtol, DUtol, contaL, cLoop);

            %%%%%%%%%%%%%%%%%%%%%%%%
            %{
            if contaL == 1 && cLoop == 1
                fr = 200 + cLoop ;
                pt1366_pTriLinMesh_nD ...
                (oAvm, oNLI, oconn.LV, oconn.TV, pudim, fr, 'r', 0.9, 0, 0, 0, 0, oNE, 0, 0, 0, 0, 0, 0);

                pt1366_pTriLinMesh_nD ...
                (Avm, NLI, conn.LV, conn.TV, pudim, fr, 'k', 0.9, 0, 0, 0, 0, NE, 0, 0, 0, 0, 0, 0);
            end
            %}
            %%%%%%%%%%%%%%%%%%%%%%%%
            
            [RLT] = uc1906_RLtri (TRC.Av, conn.TV, pudim);
            endF = 0;
            
            break
        end

    end    
    end
   
end


% after = [omNC(1:NV,1) mNC]




end

