function [conn, Bvm, Uvm, Avm, Xvm, endF] = fs0770_IMPRVtri2 ...
         (Bvm, Uvm, Avm, Xvm, conn, mNC, ccTp, Nne, tFace, sR, Spatches, pR, Xtol, DUtol)

% RETURNS: improved triangulation in patch parameter space

% Swap edge-line between 2 triangles if:
% - the alternative line is shorter
% and
% - the resultant triangles improve the regularity (condition number)

pudim = 3 ;
sudim = 2 ;

% 1.- PRELIMINARIES:

endF = 1 ;  % ends flag turns to 0 if changes happen

% No. lines and verteces:
NLI = size(conn.LT,1);
NV =  size(conn.VL,1); 

Ltol  = 20 ; % demasas admited in %
CVtol = 20 ;


% 2.- LOOP ON LINES TO ANALYZE PAIRS OF TRIANGLES:

LRc = 0 ; % stores temporarily the lastest changed line ref.

for LR = 1:1:NLI
  
% triangles refs and nodes involved:
[tR, nR] = fs0771_NTrefs (conn, LR);

if tR(2) > 0 && LR~=LRc % It does not consider: edge-lines and LR = previously changed LR


    % mid-node of alternative line:
    [Ua, Aa, Xa, coF] = fs0776_MnodeAlte ...
    (LR, [nR(2) nR(4)], Bvm, Uvm, Avm, Xvm, mNC, ccTp, tFace, sR, Spatches, pR, Xtol, DUtol);

    
	% 2.1.- CONDITION A : about diagonal lengths
        
    if coF == 0
    
        [Lc, La] = fs0772_condA (sR, LR, nR, Avm, Aa', Nne);
        caR = Lc / La ;
    
    elseif coF == 1
        
        caR = 0 ;
        
    end


    % carries on to second condition if :
    % - Lc/La ratio is 1.00 or larger  AND
    % - alternative mid-point is not at contour (coF = 1)
    
	if caR >= 1.00 && coF == 0

        % 2.2.- CONDITION B: about regularity of the triangles

        [ncv, ncM , nav, naM] = fs0773_condB (nR, Avm, Aa, Nne, LR);

        
        % 3.- EVALUATES and FLIPS IF IMPROVES THE MESH (if conditions are met):
        
        [swF] = fs0774_evaSwap (caR, ncv, ncM , nav, naM, Ltol, CVtol) ;

        if swF == 1 

            % updates the mid-node coordinates:
            
            for di = 1:1:pudim
                Xvm(nR(9),di) = Xa(di);
                Avm(nR(9),di) = Aa(di);
            end

            for di = 1:1:sudim
                Uvm(nR(9),di) = Ua(di);
            end
            
            
            % Updates conn. matrices:
            [conn] = fs0775_upconn (NV, conn, tR, nR, LR);

            endF = 0 ; % ending flag turns to 0 
            LRc = LR; % line ref. changed
            break
            
        end
    
	end
          
end  
end




end

