function [tR, nR] = fs0771_NTrefs (conn, LR)

% RETURNS:
% tR(2,1): triangles refs attached to that line ref (LR)
% nR(9,1): nodes invovled arranged as follows:
    % 1,3:      LR end-nodes
    % 2,4:      vrtx of triangles tR(1) and tR(2)
    % 5,6,7,8:  mid-nodes after 1,2,3,4
    % 9:        mid-node of LR
    
% 1.- TRIANGLES INVOLVED:

tR = [conn.LT(LR,1); conn.LT(LR,2)]; 
 
nR = zeros(9,1); 

if tR(2) > 0  % only for internal lines

    
    % 2.- NODES:
    
    % 2.1.- Nodes at LR:

    nR(1) = conn.LV(LR,1); % ends
    nR(3) = conn.LV(LR,3);

    nR(9) = conn.LV(LR,2); % mid


    % 2.2.- Nodes at triangles 'free' vtx:

    for i = 1:1:3

        r1 = conn.TV(tR(1),i);
        r2 = conn.TV(tR(2),i);

        if r1~=nR(1) && r1~=nR(3)
            nR(2) = r1 ;
        end
        if r2~=nR(1) && r2~=nR(3)
            nR(4) = r2 ;
        end
    end

    % 2.3.- Nodes at triangle 1 'free' edges:

    for i = 1:1:3

        eR = conn.TL(tR(1),i);

        if eR~=LR

            % compares end-nodes of the eR line

            if (conn.LV(eR,1)==nR(1) &&  conn.LV(eR,3)==nR(2)) || ...
               (conn.LV(eR,1)==nR(2) &&  conn.LV(eR,3)==nR(1))

                nR(5) = conn.LV(eR,2);
            end

            if (conn.LV(eR,1)==nR(2) &&  conn.LV(eR,3)==nR(3)) || ...
               (conn.LV(eR,1)==nR(3) &&  conn.LV(eR,3)==nR(2))

                nR(6) = conn.LV(eR,2);
            end

        end

    end

    % 2.4.- Nodes at triangle 2 'free' edges:

    for i = 1:1:3

        eR = conn.TL(tR(2),i);

        if eR~=LR

            % compares end-nodes of the eR line

            if (conn.LV(eR,1)==nR(3) &&  conn.LV(eR,3)==nR(4)) || ...
               (conn.LV(eR,1)==nR(4) &&  conn.LV(eR,3)==nR(3))

                nR(7) = conn.LV(eR,2);
            end

            if (conn.LV(eR,1)==nR(4) &&  conn.LV(eR,3)==nR(1)) || ...
               (conn.LV(eR,1)==nR(1) &&  conn.LV(eR,3)==nR(4))

                nR(8) = conn.LV(eR,2);
            end

        end

    end

end

end

