function [econn] = fs0775_upconn (NV, conn, tR, nR, LR)

% RETURNS:
% updated conn struct


% 1.- PRELIMINARIES:

eTV = conn.TV ;
eLV = conn.LV ;
eLT = conn.LT ;
eVL = conn.VL ;
eTL = conn.TL ;
eVT = conn.VT  ;



% 1.1.- Stores all involved edges within one matrix:
% These are the 5 lines enclosing both triangles 'tR'.

ilR = zeros(6,1);
                
conta = 0;
for i = 1:1:2 
    for j = 1:1:3 
        conta = conta + 1 ;
        ilR(conta,1) = eTL(tR(i),j);  
    end
end
                   
[ilR,~] = GEN_16_RemoveRepeatedRows (ilR, 1, 0.1);
  

% 1.2.- Same with triangles:
% These are all the triangles adjacent to the 5 lines, not only the 'tR'.         

itR = zeros(2,1);
            
conta = 0 ;
for i = 1:1:5
                    
    conta = conta+1;
   	itR(conta,1) = eLT(ilR(i),1);
                    
    if eLT(ilR(i),2) > 0
        conta = conta+1;
     	itR(conta,1) = eLT(ilR(i),2);
    end
end
                
[itR,~] = GEN_16_RemoveRepeatedRows (itR, 1, 0.1);
                


% 2.- UPDATES TRIANGULATION:
% TVcon, VTcon, LVcon, LTcon, VLcon, TLcon

% 2.1.- TV: _____________________________________________________________________________________
   
% triangle 1:

if (eTV(tR(1),1)==nR(1) && eTV(tR(1),2)==nR(2)) || ...
   (eTV(tR(1),2)==nR(1) && eTV(tR(1),3)==nR(2)) || ...
   (eTV(tR(1),3)==nR(1) && eTV(tR(1),1)==nR(2))

    % same sense as nR
    eTV(tR(1),1) = nR(1); eTV(tR(1),2) = nR(2); eTV(tR(1),3) = nR(4);
    eTV(tR(1),4) = nR(5); eTV(tR(1),5) = nR(9); eTV(tR(1),6) = nR(8);
    
else

	% opposite sense to nR
    eTV(tR(1),1) = nR(1); eTV(tR(1),2) = nR(4); eTV(tR(1),3) = nR(2);
    eTV(tR(1),4) = nR(8); eTV(tR(1),5) = nR(9); eTV(tR(1),6) = nR(5);
end


% triangle 2:

if (eTV(tR(2),1)==nR(3) && eTV(tR(2),2)==nR(4)) || ...
   (eTV(tR(2),2)==nR(3) && eTV(tR(2),3)==nR(4)) || ...
   (eTV(tR(2),3)==nR(3) && eTV(tR(2),1)==nR(4))

    % same sense as nR
    eTV(tR(2),1) = nR(3); eTV(tR(2),2) = nR(4); eTV(tR(2),3) = nR(2);
    eTV(tR(2),4) = nR(7); eTV(tR(2),5) = nR(9); eTV(tR(2),6) = nR(6);
    
else
	% opposite sense to nR
    eTV(tR(2),1) = nR(3); eTV(tR(2),2) = nR(2); eTV(tR(2),3) = nR(4);
    eTV(tR(2),4) = nR(6); eTV(tR(2),5) = nR(9); eTV(tR(2),6) = nR(7);
end



% 2.2.- VT: _____________________________________________________________________________________

% nR 2 and 4 gain one triangle
   
taR = [ tR(2) tR(1) ]; % ref. of triangle to add to nR2 and nR4

for i = 1:1:2
             
    r = i*2; % nR ref

	% current triangles of nR 2-4

	ta = zeros(1,1);
	conta = 0 ;
                    
    for j = 1:1:size(eVT,2)
                        
        cnF = 1 ; % new col. flag
                        
        if eVT(nR(r),j)>0
                            
            conta = conta+1;
            ta(conta,1) = eVT(nR(r),j);
                            
        elseif eVT(nR(r),j)==0
                            
            cnF = 0 ;
            break             
        end
    end

    
    % adds the triangle ref: taR(i)              
	if cnF == 0
                        
        eVT(nR(r),conta+1) = taR(i);
                        
	elseif cnF==1
                        
        % extends one column and adds:              
        eVT = [eVT zeros(size(eVT,1),1) ];
       	eVT(nR(r),size(eVT,2)) = taR(i);
    end

end
      
  
% nR 1 and 3 lose one triangle
  
trR = [ tR(2) tR(1) ]; % triangle to remove ref.

for i = 1:1:2
  
    r = i*2-1; % nR ref

    % remove trR(i):
                    
	t = zeros(size(eVT,2),1);
	conta = 0 ;
                    
    for j = 1:1:size(eVT,2) % temporary storage
 
        if eVT(nR(r),j)~=trR(i)
            conta = conta + 1 ;
            t(conta,1) = eVT(nR(r),j);
        end
        
    end
                    
    for j = 1:1:size(eVT,2)
        eVT(nR(r),j) = t(j,1);
    end               
end
                

% 2.3.- LV: _____________________________________________________________________________________
 
eLV(LR,1) = nR(2);
eLV(LR,2) = nR(9);
eLV(LR,3) = nR(4);            
                


% 2.4.- LT: _____________________________________________________________________________________
                
% checks involved lines: ilR and itR are created previously

for i = 1:1:5
           
    r = ilR(i);

    
	% nodes refs:
    
	n1 = eLV(r,1);
	n2 = eLV(r,3);
       
    
    % re-start eLT for that line:
    
    for k = 1:1:size(eLT,2)
        eLT(r,k) = 0 ;
    end

    
    % searches triangles with those nodes and allocates
                    
	conta = 0 ;
                    
    for j = 1:1:size(itR,1)
                        
        itn = [ eTV(itR(j),1) eTV(itR(j),2) eTV(itR(j),3) eTV(itR(j),1)];

        for k = 1:1:3              
            if (itn(k)==n1 && itn(k+1)==n2) || (itn(k)==n2 && itn(k+1)==n1)
                            
                conta = conta+1;
                eLT(r,conta) = itR(j);             
                break                
            end
        end             
                        
        if conta==2
            break 
        end                
    end  

    
    
end
    


       
% 2.5.- VL: _____________________________________________________________________________________


c = size(eVL,2);

% nR 1 - 3 loose LR line

for i = 1:1:2
         
    r = i*2-1; % nR ref
 
	t = zeros(1,1); % temporary

    conta = 0 ;
    for j = 1:1:c
                        
        if (eVL(nR(r),j)>0 && eVL(nR(r),j)~= LR)
                            
            conta = conta + 1 ;
            t(conta,1) = eVL(nR(r),j);
                            
        end
        eVL(nR(r),j)=0; % resets
    end
                    
	% re-allocates
    for j = 1:1:conta  
    	eVL(nR(r),j) = t(j);
    end
    
    
end


% nR 2 - 4 gain LR line

for i = 1:1:2
    
    r = i*2; % nR ref

 	ncF = 1 ; % new column flag
                    
    for j = 1:1:c
                        
        if eVL(nR(r),j)==0               
            eVL(nR(r),j) = LR;
            ncF = 0 ;             
            break
        end
    end
                    
	if ncF==1 
        eVL = [eVL zeros(NV,1)];
        c = c+1;
    	eVL(nR(r),c) = LR ;
    end   

end
                
                
% 2.6.- TL: _____________________________________________________________________________________
                    
t = [ 1 2 ; 2 3 ; 3 1 ]; % temp. refs for nodes
                
for i = 1:1:2
      
    % re-sets
    for k = 1:1:size(eTL,2)
        eTL(tR(i),k) = 0;
    end
    

    conta = 0 ;
    for j = 1:1:3
                    
        tn = [ eTV(tR(i),t(j,1))  eTV(tR(i),t(j,2)) ]; % pair of nodes refs

        % searches this pair in lines in the updated LV:
                        
        for k = 1:1:5
                            
            lr = ilR(k);

            if (eLV(lr,1)==tn(1) && eLV(lr,3)==tn(2)) || (eLV(lr,1)==tn(2) && eLV(lr,3)==tn(1))     
                
                conta = conta + 1 ;
                eTL(tR(i),conta) = lr;
                break  
                
            end 
            
        end
        
    end
 
end
     

% 3.- ALLOCATES:

econn = struct ('TV',[], 'LV',[], 'LT',[], 'VL',[], 'TL',[],'VT',[]);

econn.TV = eTV;
econn.LV = eLV;
econn.LT = eLT;
econn.VL = eVL;
econn.TL = eTL;
econn.VT = eVT;

                
end

