function [CNT] = fs0792_Sconto (pR, sR, tFace, NFC, NL, ncuE, SCconE, Xtol, DUtol)

% RETURNS: points for contour loops
% CNT structure:
%   Nc(NL,1): No. points if each loop, 1st and last are repeated
%   cX, cA, cU, cB,(NL,Nc,udim): coordiantes in physical, P-param, S-param and C-param
%   cRef(NL,Nc): curve ref per node in each loop

CNT = struct('Nc',[], 'cX',[], 'cA',[], 'cU',[], 'cB',[], 'cRef',[]);

xdim  = 3 ;
pudim = 3 ;
sudim = 2 ;
cudim = 1 ;

sDegree = tFace(pR).mSur(sR).Degree ;
sNcp    = tFace(pR).mSur(sR).Ncp ;
sKnotV  = tFace(pR).mSur(sR).KnotV ;
sUsize  = tFace(pR).mSur(sR).Usize ;
sP     = tFace(pR).mSur(sR).P ;
sw     = tFace(pR).mSur(sR).w ;
sCD    = tFace(pR).mSur(sR).CS ;


mNc = zeros(NL,1);
mcX = zeros(NL,1, xdim);  % the first point is repeated at the end, it is a closed contour
mcA = zeros(NL,1, pudim);
mcB = zeros(NL,1);
mcU = zeros(NL,1, sudim);

mcRef = zeros(NL,1);

for L = 1:1:NL

    xL = zeros(1,xdim);
    aL = zeros(1,pudim);
    bL = zeros(1,cudim);
    n = 0 ;
    
    % 1.- CONTOUR OF Xcoords WITH POINTS FROM CURVES:

    for c = 1:1:ncuE(sR,L)

        cR = SCconE(sR,L,c);

        sg = sign(cR);
        cR = abs(cR);

        xt = zeros(NFC.Nc(cR),xdim);
        at = zeros(NFC.Nc(cR),pudim);
        bt = zeros(NFC.Nc(cR),cudim);
        
        for i = 1:1:NFC.Nc(cR)
            xt(i,:) = NFC.Xn(cR,i,:);
            at(i,:) = NFC.An(cR,i,:);
            bt(i,1) = NFC.Bn(cR,i);
        end

        if sg == -1
            [xt] = GEN_40_invertRowsOrder (xt);
            [at] = GEN_40_invertRowsOrder (at);
            [bt] = GEN_40_invertRowsOrder (bt);
        end

        for i = 1:1:NFC.Nc(cR)-1
            
            n = n+1;
            
            xL(n,:) = xt(i,:);
            aL(n,:) = at(i,:);
            bL(n,1) = bt(i,1);
            
            mcRef(L,n) = cR ;
        end

        
        
    end


    % 2.- CONTOUR PARAMETER COORDINATES IN SURFACE PARAMETER SPACE:

    [uL] = s2831b_PPsurf (xL, n, sDegree, sNcp, sKnotV, sUsize, sP, sw, sCD, xdim, Xtol, DUtol);


    % 3.- LAST POINTS, that coincide with first:
    
    n = n + 1 ;
    xL(n,:) = xL(1,:);
    aL(n,:) = aL(1,:);
    uL(n,:) = uL(1,:);
    bL(n,1) = bL(1,1);
    
    mcRef(L,n) = cR ;
    
    % 4.- ALLOCATES:
    
    mNc(L,1) = n ;

    for i = 1:1:n
        mcB(L,i) = bL(i,1);
        mcU(L,i,:) = uL(i,:);
        mcX(L,i,:) = xL(i,:);
        mcA(L,i,:) = aL(i,:);
    end
    
end



CNT.Nc = mNc;
CNT.cX = mcX;
CNT.cA = mcA;
CNT.cU = mcU;
CNT.cB = mcB;
CNT.cRef = mcRef;



end

