function [bf, bk] = b0822_addDirichlet ...
    (NicpE, RicpE, pfPos, Aref, Bentities, bR, rdv, thk, pxdim, DoF, insR)

% RETURNS:
% bf, bk: vector / matrix of Dirichlet for B.E. 'b' (local ref.)

% INPUTS:
% NicpE:     No. influential CP's on B.E. 'b' 
% RicpE(NicpE): their references
% pfPos:    first position in global K of patch
% b / bR:   BE local ref (in terms of patch) and global ref.

       
% Computes constrained degrees of freedom 'cDoF' for that DBC and construct restr(pxdim) array

cDoF = 0;
restr = zeros(pxdim,1);

for di=1:1:pxdim
    if isnan(rdv(di)) == 0 
        cDoF = cDoF + 1 ;
        restr(di,1) = 1 ;
    end
end
            

% No. rows to add to K : Nrows = Constrained DoF x Nicp on that B.E.
% and relative position of each anchor within RicpE:

Nrows   = cDoF * NicpE;
[sorA] = b0841_AnchorRelDir (NicpE, RicpE);


% prepares matrix and vector to add to K and F

bf = zeros (Nrows,1);
bk = zeros (Nrows,DoF); 


% loop on GP's:

Ngp = Bentities(bR).INS(insR,1).skin.mNgp;
GP = Bentities(bR).INS(insR,1).skin.mGP;
         
for g = 1:1:Ngp
    
    gNicp = GP(g).mNicp;
    gRicp = GP(g).mRicp;
    gRrel  = GP(g).Rrel ;

    %{
    if bR==2
        g
        gRicp
    end
    %}
    
    dJ1 = GP(g).dJ1 ;
    dJ2a = GP(g).dJ2a ;

    for Lr1 = 1:1:gNicp
     
        % rows ref's to add in bf & bk
        [filR] = b0842_rowsRef (gRicp, Lr1, sorA, cDoF, restr, pxdim);

        % vector bf:

        R1 = gRrel(Lr1);

        cte1 = dJ1 *  dJ2a * R1 * thk ;

        for di = 1:1:pxdim
            if isnan(rdv(di))==0 
                bf(filR(di),1) = bf(filR(di),1) + cte1 * rdv(di); 
            end
        end

    
        % matrix bk:

        for Lr2 = 1:1:gNicp

            R2 = gRrel(Lr2);
            aR = gRicp(Lr2);

            % First col position with in the whole system (K)
            col = c0952_AbsFrow (pfPos, Aref, aR, pxdim); 

            cte12 = dJ1 *  dJ2a * R1 * R2 * thk ;

            for di=1:1:pxdim
                if isnan(rdv(di)) == 0 
                    bk(filR(di),(col+(di-1))) = bk(filR(di),(col+(di-1))) + cte12 ;
                end
            end
        end


    end
end


       
            
end
