function [Tfe, Dfe] = c0904_FextTstep ...
    (MTL, Spatches, Bentities, Npa, PFpos, insR, thk, lambda, DoF, xdim)

% RETURNS:

% tFe:   total external force applied at certain time step (insR)
% DFe:   increment of that step

% This function inserts in F vector Neuman's BC's.
% mF is object attribute, and can be accessed from here directly
   
Tfe  = zeros (DoF,1) ; % total force vector for that step
                
for pa = 1:1:Npa % Loop on patches:
    

    % 1.- NEUMANN BC:
    
 	% Loop on B.E. of the patch to detect which one has BC's:
                
	pnbe = Spatches(pa,1).nbe; 
    pRbe = Spatches(pa,1).Rbe; 
    mCPref = Spatches(pa,1).INS(insR,1).mCPref; % position of active anchors
    

	for b = 1:1:pnbe
                    
      	bR = pRbe(b);
 
      	% Neumann conditions attached to that B.E., if=0 => no condition
      	NBC = Bentities(bR,1).mNEU;
        
       	if NBC > 0

           	NeuF = zeros (DoF,1); % prepares vector to add to F
            
            Ngp = Bentities(bR,1).INS(insR,1).skin.mNgp;
            GP  = Bentities(bR,1).INS(insR,1).skin.mGP;
            
            for g = 1:1:Ngp
                
                Nicp  = GP(g).mNicp;
                Ricp  = GP(g).mRicp;
                Rrel  = GP(g).Rrel;
                dJ1   = GP(g).dJ1 ;
                dJ2a  = GP(g).dJ2a ;
                 
                for a = 1:1:Nicp
                    
                    fag = zeros(xdim,1); % force at anchor due to one GP contribution
                    
                    R1  = Rrel(a);

                    aR = Ricp(a);
                    rou = c0952_AbsFrow (PFpos(pa), mCPref, aR, xdim); % First row position within the whole system (K)

                    % computes contribution:
                    cte = dJ1 *  dJ2a * R1 * thk * lambda ;
                    
                    for di=1:1:xdim
                        fag(di) = cte * Bentities(bR,1).INS(insR,1).mt(di) ;
                    end
                    
                    % add to NeuF:
                    for di=1:1:xdim
                        NeuF(rou+(di-1),1) = NeuF(rou+(di-1),1) + fag(di) ;
                    end

                end
                
            end
            

            % add contribution from that B.E.:
            Tfe = Tfe + NeuF ;
            
        end % there is Neumann BC
        
	end % loop on B.E.
    
    
    
    % 2.- BODY FORCES:
    
    if MTL.swF == 1
        
        BodF = zeros (DoF,1); % prepares vector to add to F
        
        Ngp = Spatches(pa,1).INS(insR,1).mIM.mNgp ;      
        GP  = Spatches(pa,1).INS(insR,1).mIM.mGP ;
        
        for g = 1:1:Ngp
            
            Nicp  = GP(g).mNicp;
        	Ricp  = GP(g).mRicp;
         	Rrel  = GP(g).Rrel;
           	dJ1   = GP(g).dJ1 ;
          	dJ2a  = GP(g).dJ2a ;
            
            
            for a = 1:1:Nicp
                    
                fag = zeros(xdim,1); % force at anchor due to one GP contribution
                    
             	R1  = Rrel(a);

                aR = Ricp(a);
                rou = c0952_AbsFrow (PFpos(pa), mCPref, aR, xdim); % First row position within the whole system (K)

                % computes contribution:
            	cte = dJ1 *  dJ2a * R1 * thk * lambda ;
                    
                fag(3) = cte * (-MTL.sw); % specific weight is only considered in z direction
             
                % add to BodF:
                for di = 1:1:xdim
                    BodF(rou+(di-1),1) = BodF(rou+(di-1),1) + fag(di) ;
                end

            end
            
        end
        
        
        % add contribution from that patch body forces:
     	Tfe = Tfe + BodF ;
        
    end
    

end % loop on patches




% increment in that step.
Dfe = Tfe / insR ;




end

