function [Spatches, Bentities] = c0911_DS ...
         (BCk, BCf, MTL, yModel, Spatches, Npa, PFpos, insR, ...
          DoF, DoFr, Anty, thk, rtol, Dfe, Tfe, Bentities, Nbe, xdim)

      
% RETURNS:
% Spatches with displacements and stresses


% INPUTS:
% rtol:     torelance in the error of residual in percentage of initial residual
% DFext:    increment in Fext at that step
% FextTstep:total external force applied at that step


% 1.- LOOPS TO MINIMIZE THE RESIDUAL:

% 1.1.- Set initial error and accumulated displacement increments:
%{
if norm(Dfe) < 0.0000001
    rerror = 0 ;
else 
    rerror = norm(Dfe)/norm(Dfe)*100 ;
end
%}
rerror = rtol + 1 ;

Duacc =  zeros(DoF,1); 

NLmax = 1 ; % max. No. loops to converge
NL = 1 ;


while (rerror > rtol && NL <= NLmax)
    
    stage = [insR NL];
    % stage
    
    
    % 1.2.- er : Extend the increment of force (or residual) with BCs:
    
    Df = [  Dfe ;
            BCf ] ;
    

    % 1.3.- Computes consistent tangent operator: uses mPar at Gp's:

    [Spatches] = c0912_CT_Operator (Spatches, Npa, yModel, Anty, MTL, insR);


    %[Spatches] = c0917_XbfGPskins (Spatches, Npa, insR);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %{
    % Computes integrals of functions f = dRi,a * dRj,b 
    
    pa = 1 ;
    
    % works out all the BFs combinations (only for checking discretization):
    [Npr, bfr, bfd] = c0956_allBFcomb (Spatches, pa);
    
    
    % computs integrals and plots:
    [iBF] = c0955_integrBFS (Spatches, pa, Npr, bfr, bfd) ;
    
    if hh==0
    end
    
    %}
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    % 1.5.- Constructs K with no BC's and extends with BCs: Coupling and Dirichlet:

    [K] = s1011_KfromGP (Npa, Spatches, PFpos, insR, Anty, thk, DoF, xdim);

    assignin ('base','K', K);
    
    
    %{
    K_11  =  zeros(3,3);
    K_209 =  zeros(3,3);
    
    a1 = 31;
    a2 = 207;
    
    for i = 0:1:2
    for j = 0:1:2
        
        K_11(i+1,j+1) =  K( a1+i, a1+j );
        K_209(i+1,j+1) = K( a2+i, a2+j );
    end
    end
    
    K_11
    K_209
    
    %}
    
    nc = size(BCk,1);
    
    Kbc = [K     BCk' ;
          BCk   zeros(nc,nc)] ;
   
      
      
    % 1.6.- Computes displacements increment: returns du without Lagrange multipliers
    
    Dul = Kbc\Df;       % extended increment of displacement, with lambdas
    Du = Dul(1:DoF);    % increment of displacement, removes lambdas
   

    Duacc = Duacc + Du;
    
    
    %{
    
    % 1.7.- CHECKS FOR PLASTIC REGIME:
    
    % a.- update at each point:
    
    [Spatches] = c0903_uP...
        (GrevN, GrevP, UsInse,yModel, Spatches, PatchPos, du, Npa, insR, Anty, ...
        yLims, OrtElas, H, ty0, Gf ,dfract, NGPp, pxdim, pudim, NL);

    
    % b.- Interior forces, computed from stresses - Fint:
    Fint = f0907_Fint (DoF, Spatches, Npa, insR, NGPp, PatchPos, pxdim);
    
    % Fint
    
    % c.- Stores iteration parameters for anchors:
    Spatches = f0912_mParAnchors (Spatches, Npa, pxdim, tFe, Fint, du, NL, insR);
    
    
    % d.- Residual vector, note that the constrained DoF (DoFr) are set to zero:
    j = 1 ;
    r = zeros(DoF,1);
    for i=1:1:DoF
        
        if i==DoFr(j)
            r(i) = 0 ;
            j=j+1;
        else
            r(i) = tFe(i) - Fint(i) ;
        end
        
        if j  > size(DoFr,1)
        j = size(DoFr,1); 
        end
    end

   %[FextTstep Fint r]
    
    rerror = norm(r)/norm(DFe)*100

    %}
    
    NL = NL+1 ;
    %assignin ('base','Fint', Fint);

end


% 2.- NEXT INSTANCES:


% 2.1.- Copy features that do not depend on the physical coordinates:

[Spatches] = n1401_CopyINSnoX (Spatches, Npa, insR);
[Bentities] = n1411_CbeINSnoX (Bentities, Nbe, insR);


% 2.2.- Computes features dependant on the physical coordinates:

[Spatches] = n1402_Du2INS (Spatches, Npa, insR, Duacc, xdim);
[Bentities] = n1412_Du2beINS (Bentities, Spatches, Nbe, insR, xdim);




% 2.3.- Computes stresses:

[Spatches] = c0916_ESt (Spatches, PFpos, Duacc, Npa, insR, xdim);






% 4.7.- Post-processor: OJO AQUI , ACTIVARLO !!
    
%   [Spatches] = p1101_LinTri (Spatches, Npa, insR);


% 6.- Allocates displacements to ACTUAL instance (insR):
% wires of both meshes 
% nodes & GP's for Analysis mesh:

% [Spatches] = f0909_XdispTransfer (IntRects, Spatches, Npa, insR, pudim, pxdim);
 
% 4.5.- Allocates anchors acc. displacements to the ACTUAL instance (insR):
    
% [Spatches] = c0914_u2anchors (Spatches, Npa, Duacc, Du, insR) ;
    


end

