function [iBF] = c0955_integrBFS (Spatches, pa, Npr, bfr, bfd)


% RETURNS:
% iBF(Npair,1): integrals of product of derivatives of basis functions

% INPUTS:
% Spatches: patches object
% pa: patch ref. to operate
% Npr: No. pairs of functions to derivate
% bfr(Npr,2): basis functions refs
% bfd(Npr,2): direction of derivatives: 1, 2 o 3

% e.g.: Npair = 3 ; bfr = [2 4; 4 4 ; 4 6]; bfd = [1 1 ; 1 2 ; 3 2];
% corresponds to: dR2,x * dR4,x ; dR4,x * dR4,y ; dR4,z * dR6,y ; 

% 1.- PRELIMINARIES:

xdim = 3 ;

iBF = zeros(Npr,1);

Ngp = Spatches(pa,1).INS(1,1).mIM.mNgp;
GP = Spatches(pa,1).INS(1,1).mIM.mGP;
CP = Spatches(pa,1).INS(1,1).mCP;

bfv = zeros(2,1); % basis funct. values (dRi,j) at computable Gp's



% 2.- COMPUTES INTEGRALS:
tic

L1 = 1 ;
L2 = 200 ;

%L1 = 1 ;
%L2 = Npr ;

for p = L1:1:L2 % loop on pairs     
    
    iBFt = 0 ; % temporary value of the integral
    
    % only carries on if both bf's intersect:
    Acoor = CP(bfr(p,1),1).mAcoor;
    AcoLi = CP(bfr(p,2),1).mAcoLi;
    
    IN = GEN_08_Orthodom_Pin_ndec (Acoor, AcoLi, xdim, 'closed', 6);
    
    if IN==1

        % Loop on Gps
        % checks each GP, if it has both bfr it is computable for the integral
        for g = 1:1:Ngp

            % Gp info:
            Nicp = GP(g,1).mNicp ;
            Ricp = GP(g,1).mRicp ;
            dJ1 = GP(g,1).dJ1;
            dJ2a = GP(g,1).dJ2a;

            dRx = GP(g,1).dRfi ;

            % makes sure dRx is (Nicp,xdim)
            if size(dRx,2) ~= xdim
               dRx = dRx'; 
            end

            % computable GP flag:
            % it has to be = 2 to be a computable GP for the integration of those bfr
            cgpF = 0 ;  

            % first function:
            j = 1 ;
            for i = 1:1:Nicp

                if Ricp(i)==bfr(p,j)

                    bfv(j,1) = dRx(i,bfd(p,j));
                    cgpF = cgpF + 1 ;

                    ip = i ;
                    break

                end
            end

            % second function:
            if cgpF==1

                j = 2 ;
                for i = ip:1:Nicp

                    if Ricp(i)==bfr(p,j)

                        bfv(j,1) = dRx(i,bfd(p,j));
                        cgpF = cgpF + 1 ;

                        break

                    end
                end

            end

            % adds to integral in case is computable:
            if cgpF==2
                iBFt = iBFt + bfv(1,1)*bfv(2,1)*dJ1*dJ2a;
            end

        end

    
    end
    
    % transfers integral value:
    iBF(p,1) = iBFt ;
    
end

toc

% 3.- PLOTS:

fr = 1 ;

figure(fr)

plot([L1:1:L2],iBF(L1:L2,1),'b'); hold on

iBF(L1:L2,1)

%axis equal





end

