classdef cGPpa < handle 
    

    properties
        
        % Coord's:
        mAcoor;     % (pudim) 
        mXcoor;     % (xdim) 

        % Inf. anchors:
        mNicp;     %           
        mRicp;     % (mNicp)   
        
        % Basis functions & deriv.:     
        Rrel;       % (mNicp)
        dRpa;      % (mNicp, udim)
        dRfi;      % (mNicp, xdim), for singular points can be LHS, RHS or average
        dJ1;        %
        dJ2a;       % aggregated [det(J2)*weight]
        
        % Const. matrix
        mD;

     	% Stresses:
        mSv;        % (6,1): sx, sy, sz, txy, tyz, txz
        mVM;        % Von Mises
        mPst;       % (3,1): ppal. stresses
        mPdi;       % (3,3): ppal. directions, each in a column
        
        %{
        % Point variables (scalars and vector) in each iteration and No. iterations:
        nIT  ; % No. iterations
        nVsc ; % No. vbles. - scalars
        nVve ; % No. vbles. - vectors
        mVsc ; % (nVsc, nIT)
        mVve ; % (nVsc, nIT, 6)
        
        % internal variables
        mDeet;      % trial increment of strain (all elastic)
        mDee;       % increment of elastic strain
        mDep;       % increment of plastic strain
        ma;         % internal variables
        %}
        
    end
    
    methods
        
        
        %{
        
        function InitiatesVbles(obj, yModel)
            
            % initiates ma, nPA, mPar
            % it is called in initiation and copy of instance
            
            % INPUTS:
            % model
            % some params
            
            % 1.- Vbles. acc. to selected model:
            
            %{
            % Vsc: 1-Dp, 2-H, 3-d, 4-Fy, 5-Hs, 6-Plas, 7-epa
            % Vve: 1-ee(6), 2-Sv(6), 3-ep(6)
            
            if strcmp(yModel , 'Ho') == true  
                
                obj.nVsc = 7 ; 
                obj.nVve = 3 ; 
                
                nIter = 1 ; % No. iterations (will increase if plastic)
                
                obj.mVsc = zeros(obj.nVsc, nIter );
                obj.mVsc(4) = 1 ;
                obj.mVve = zeros(obj.nVve, nIter, 6 );
            end
            %}
            
            % 2.- Stress tensor:
            
            obj.mSv = zeros(6,1);
            obj.mVM = 0;
            obj.mPst = zeros(3,1);
            obj.mPdi = zeros(3,3);
            
        end
         
        %}
        
        %{
        
        function kL = LocalStiffnessSubK (obj,a,b, Anty, xdim, udim ,thk, dfract, addNL)

            % calculates sub-matrix (xdim,xdim relating cp's a-b)

            % INPUTS:
            % a,b:  basis functions references, correspondant to dRgp_ph references
            % xdim:
            % NGPe: No. Gauss points
            % we:   weight of Gp's (array)
            % thk:  thickness, for 3D cases assume = 1
            % addNL:    if 'yes' adds KNL stiffness matrix
            
            
            % 1.- DEFORMATION MATRICES PRE-SIZING:
            
            if xdim==2
                Bfil =  3 ;
            elseif xdim==3
                Bfil =  6 ;
            end

            kL  = zeros(xdim,xdim); % to answer
            kLg = zeros(xdim,xdim); % for storing kL at each GP and add to kL
            kLgNL = zeros(xdim,xdim);
            
            % deformation matrices for each cp:
            
            Ba = zeros(Bfil,xdim); % standards
            Bb = zeros(Bfil,xdim);
            
            if strcmp(addNL,'yes')==true
                BaNL = zeros( 2*xdim, xdim ); % with NL
                BbNL = zeros( 2*xdim, xdim );
            end
            
            
            % 2.- CONTRIBUTION OF THAT GP:
            
            dRx = zeros(obj.mNicp,xdim);
         	d = obj.mVsc(3);
                
            % 3.1.- For fully damaged point:
            
            if d > dfract
                    
                kLg = zeros(xdim,xdim);
                cte = 0 ;
            
                
            % 3.2.- For 'working' point:
            
            else
                
                % extract dRx and J1 for that Gp:
                
                dRx(:,:)= obj.dR_fi(:,:);
                detJ1 = obj.dJ1;
                
                
                
                % Jacobian 2: considers as many as elements where the Gp's
                % is located in:
                
                nEg = obj.nE;
                detJ2 = 0 ;
                for e=1:1:nEg
                    detJ2 = detJ2 + obj.dJ2(e);
                end
                


                
                % Computes kLg for that point:
                
                Ba = GEN_1001_Bmatrix ( 1, a , dRx, xdim); % B matrices for cp's 'a' and 'b'
                Bb = GEN_1001_Bmatrix ( 1, b , dRx, xdim);
                
                D = dimsDmatrix(obj, Anty, udim); % see function below

            	kLg = Ba' * D ;
              	kLg = kLg * Bb ;

                    
               	% Coputes kLgNL for that point (if selected): 
                
                if strcmp(addNL,'yes')==true
                        
                    BaNL = GEN_1010_BNLmatrix ( 1, a , dRx, xdim);
                    BbNL = GEN_1010_BNLmatrix ( 1, b , dRx, xdim);
                    
                	tm = GEN_1011_tMatrix (obj, xdim );

                    kLgNL = BaNL' * tm ;
                    kLgNL = kLgNL * BbNL ;
                end
                  
                
                % Constant, depending on the GP:
             	cte = (detJ1 * detJ2 * obj.mw * thk) ;
                
            end
                

        	% 3.3.- Computes kL:
            
            kL = (kLg + kLgNL).* cte ; 
            
        end
        
        %}

        
    end
    
    
    
end

