function [dRpa, dRpaRHS] = p1212_mdRpa (Usingu, Nicp, KnotV, Ricp, mA, Degree, Nabs, udim)

% RETURNS:
% dRpa:   deriv. about param. (dR/dxi) at location u. RHS for right hand side
% location u is given implicitly by Ni

% INPUTS:
% Nicp:     No. icp on that element
% Nabs:      basis functions in each direction at location 'u'

% NOTE: if non-singular dRpaRHS = dRpa
%       if is singular, dRpaRHS is computed from dNaRHS and dWRHS


% 1.- Sizes and initiates the matrices:

dRpa    = zeros(Nicp,udim); 
dRpaRHS = zeros(Nicp,udim);



% 2.- Calculates derivatives at 'u' for each bf's (ref's a), the actual 
% basis functions values and the total weight derivatives dW(2) and the weight W:
   
W = 0 ;
dW = zeros(udim, 1); % derivatives about u(1), u(2), u(3)
dWRHS = zeros(udim, 1);

N   = zeros(Nicp, udim);  

dNa     = zeros(Nicp, udim);  
dNaRHS  = zeros(Nicp, udim);  

dNfactor = zeros(Nicp, udim);  
dNfactorRHS = zeros(Nicp, udim); 
Nfactor = zeros(Nicp,1);  


for a = 1:1:Nicp % loop on influential anchors

    aR = Ricp(a); % anchor abs. reference
    
    
    % 2.1.- B-spline derivatives:
    
    for di=1:1:udim
        
        % degree and LKV:
        
        p = Degree(di);
        ix = mA(aR).mIX(di);
        
        Dk = 0.5 + p*0.5;
        kmid = Dk + ix;
        
        k1 = kmid-Dk;
        k2 = kmid+Dk;
        
        U = zeros(p+2,1);
        
        k = k1 ;
        for i=1:1:p+2

            U(i) = KnotV(di,k);
            k = k+1;
        end
        
        
        % actual spline function value
        
        N(a,di) = Nabs(di,aR,p+1,1); 
        
        singu = Usingu(a,di); % indicates is if singular ('peaky')
        
        % computes derivative
        [dNa(a,di), dNaRHS(a,di)] = z2799_BsplineDeriv (singu, p, U, Nabs, di, aR);

    end
    

    % 2.2.- Nfactor, dNfactor, total weight(W) and dW respect u1, u2 or u3; and 
    
    Nfactor(a) = 1 ; % it is just product of b-spline basis functions: N*M*L
    
    for di=1:1:udim

        Nfactor(a) = Nfactor(a) * N(a,di); 
        
        dNfactor(a,di) = 1 ; % similar as Nfactor, but relevant direction is derived
        dNfactorRHS(a,di) = 1 ;
        
        for di2=1:1:udim
           
            if di2==di
                dNfactor(a,di) = dNfactor(a,di) * dNa(a,di2);
                dNfactorRHS(a,di) = dNfactorRHS(a,di) * dNaRHS(a,di2);
            else
                dNfactor(a,di) = dNfactor(a,di) * N(a,di2);
                dNfactorRHS(a,di) = dNfactorRHS(a,di) * N(a,di2);
            end
        end
        
        dW(di) = dW(di) + dNfactor (a,di)* mA(aR).mw;
        dWRHS(di) = dWRHS(di) + dNfactorRHS (a,di)* mA(aR).mw;
    end

    W = W + Nfactor(a) * mA(aR).mw;   
    
end





% 3.- Calculates the 1st derivatives and the basis functions
%     values for each basis functions references:


for a=1:1:Nicp 
    
    aR = Ricp(a); % anchor abs. reference
    
    for di=1:1:udim
        
        dRpa(a,di) =  mA(aR).mw * ( dNfactor(a,di) * W - Nfactor(a)*dW(di) ) / W^2 ;
        dRpaRHS(a,di) = dRpa(a,di) ;
        
        if Usingu(a,di)~=0
            dRpaRHS(a,di) =  mA(aR).mw * ( dNfactorRHS(a,di) * W - Nfactor(a)*dWRHS(di) ) / W^2 ;
        end
        
    end
 
end





end



