function [ sp, dp, VM ] = s1235_StPrin_vm (t)

% RETURNS:
% sp(3):    principal stresses vector
% dp(3,3):  principal directions, each column is one direction
% VM:       Von Mises stress

% INPUTS:
% t(6): stresses tensor, but in vector form: sx, sy, sz, txy, tyz, txz


% 1.-Initiates responses:
sp = zeros(3,1);
dp = zeros(3,3);
VM = 0 ;


% 2.- Assemblies tensor:

T = [   t(1) t(4) t(6) ; 
        t(4) t(2) t(5) ;
        t(6) t(5) t(3) ];


    
    
% 3.- Computes eigen values and eigen vectors (dp):
[dp,e] = eig(T);



% 4.- Tranfers e to sp:
for i=1:1:3
    sp(i) = e(i,i);
end


% 5.- Von Mises:
VM = (((sp(1)-sp(2))^2 + (sp(1)-sp(3))^2 + (sp(2)-sp(3))^2 )/2 )^0.5 ;


end

