function [z, outF] = tm1302_interpTri (DT, TVcon, Vc, Vz, c, zdim)

% RETURNS:
% z(zdim,1): interpolated values for c location
% outF: if c is out of DT domain, outF=1

% INPUTS:
% DT: Delaunay triangulation object
% TVcon(NT,3): conn. matrix triangles VS vertexes
% Vc(NV,2): coordiantes of vertexes
% Vz(NV,zdim): image vaues of vertexex
% c(2): coordinates of point where we want values to be computed
% zdim: No. dims of the imate (No. cols of Vz)




% 1.- TRIANGLE REF. WHERE x BELONGS TO:

tr = pointLocation(DT,c);


% 2.- COMPUTATION:

if isnan(tr)
    
    % 2.1.- z = 0 if x does not belong to DT:
    
    z = zeros(zdim,1) ;
    outF = 1 ;
else
    
    % 2.2.- z = 0 if x does not belong to DT:
    
    z = zeros(zdim,1) ;
    outF = 0 ;
    
    % extracts triangle nodes x-coor's:
    
    xt = zeros(3,2);
    for i = 1:1:3
        nR = TVcon(tr,i);
        xt(i,1) = Vc(nR,1);
        xt(i,2) = Vc(nR,2);
    end
    
    
    % total triangle area:

    x = [xt(1,1) ; xt(2,1) ;xt(3,1)];
    y = [xt(1,2) ; xt(2,2) ;xt(3,2)];
    At = polyarea(x,y);
    
    
    % Finds basis functions values using areas
    
    ov = [2 3 ; 3 1 ; 1 2]; % opposite vertex local ref.
    N = zeros(3,1);
    
    for i = 1:1:3
        
        x = [ xt(ov(i,1),1) ; xt(ov(i,2),1) ; c(1) ];
        y = [ xt(ov(i,1),2) ; xt(ov(i,2),2) ; c(2) ];
        Ai = polyarea(x,y);
        
        N(i)= Ai/At;
        
    end
    
     
    % Computes response:
    
    for i = 1:1:3
        
        nR = TVcon(tr,i);
        for di = 1:1:zdim
            z(di) = z(di) + N(i)*Vz(nR,di);
        end

    end
    
  
    
end



end

