function [ro] = tm1304_Ronumber (Unod, ndim)

% RETURNS:
% ro = rA/0.606

% where:    0606 is rA for equilateral one
%           rA is the ratio Ac/A of the current triangle, with:
%               Ac: area of inscribed circle
%               A: area of triangle

% the higher ro the more quality (closer to equilateral)

% INPUTS:
% Unod(3,ndim): coordinates of triangle nodes

% 1.- EXTRACT VECTORS AND POINTS:

nd = 6 ;

a1 = zeros(3,1); 
a2 = zeros(3,1); 

v12 = zeros(3,1);
v13 = zeros(3,1);
v23 = zeros(3,1);

for di = 1:1:ndim
    
    a1(di,1) = Unod(1,di);
    a2(di,1) = Unod(2,di);
    
	v12(di,1) = (Unod(2,di) - Unod(1,di));
	v13(di,1) = (Unod(3,di) - Unod(1,di));
    v23(di,1) = (Unod(3,di) - Unod(2,di));
    
end
    
v21 = -v12;


% 2.- AREA OF TRIANGLE:

nv = cross(v12,v13);
A = norm(nv)/2;



% 3.- AREA OF INSCRIBED CIRCUMFERENCE:

% Use vectors 1-2 and 1-3, and 2-1 and 2-3, to compute bisectors:

[b1] = go2536_bisector (v12, v13, ndim);
[b2] = go2536_bisector (v21, v23, ndim);


[IP, fNI] = go2530_LinInters_nD (a1,b1, a2,b2, ndim, nd);

while fNI==1
    
    nd = nd-1;
    [IP, fNI] = go2530_LinInters_nD (a1,b1, a2,b2, ndim, nd);
    
    if nd==1
        fNI = 1 ;
        break
    end

end



% radius is distance to any edge, uses two of them and averages:

[d1, ~] = go2527_distPtoRecta (IP, a1, v12, ndim);
[d2, ~] = go2527_distPtoRecta (IP, a2, v23, ndim);

R = (d1 + d2)/2;


% area of circle and ratio Ac/A:

Ac = 3.14159 * R^2 ;

rA = Ac / A ;


% 4.- Ro NUMBER:

ro = rA / 0.606  ;



% plots for checking:
%{

eUnod =  [Unod ; Unod(1,:)];

figure(1)

quiver3(a1(1),a1(2),a1(3),b1(1),b1(2),b1(3),'r');
quiver3(a2(1),a2(2),a2(3),b2(1),b2(2),b2(3),'r');

scatter3(Unod(:,1), Unod(:,2), Unod(:,3), 'r'); hold on
plot3(eUnod(:,1), eUnod(:,2), eUnod(:,3), 'k'); hold on

scatter3(IP(1),IP(2),IP(3),'b'); hold on

axis equal


if gg==0
end

%}


end

