function [P, GP] = tm1310_opTet (Snod)

% RETURNS:
% P(udim,1): coords of 4th node to achieve highest R value in a tet
% GP(pudim,1): vector from centroid to P

% INPUTS:
% Tnod(3,pudim): coords of one triangle of the tet

% Uses N-R

dim = 3 ;


% 1.- BASE VECTORS: u,v,x AND CONSTANTS:

u = zeros(dim,1);
v = zeros(dim,1);
x = zeros(dim,1);

for di = 1:1:dim
    u(di,1) = Snod(2,di) - Snod(1,di);
    v(di,1) = Snod(3,di) - Snod(1,di);
    x(di,1) = Snod(3,di) - Snod(2,di);
end

C = cross(u,v);
K = norm(u) + norm(v) + norm(x);


% 2.- LINE PARAMETERS: point (CoG) and direction (perpendicular)

G = zeros(dim,1);
a = zeros(dim,1);
b = zeros(dim,1);
c = zeros(dim,1);

for di = 1:1:dim
    
    G(di,1) = (Snod(1,di) + Snod(2,di) + Snod(3,di)) / 3 ;
    
    a(di,1) = Snod(1,di);
    b(di,1) = Snod(2,di);
    c(di,1) = Snod(3,di);
    
end

[p1,p2,p3,~] = go2506_Plane3P (a,b,c);

d = [p1;p2;p3];
d = d/norm(d);


% 3.- N-R:

tol = 1.0 ;
err = tol + 1 ;

% initial points: 50 % of average length of triangle sides

t = 0.5 * K/3 ; %%%%%%%%%%%%%


contaL = 0 ;

while (err > tol)
    
    contaL = contaL + 1 ;
    
    % ingredients factors:
    [R, dR1, dR2] = tm1311_otFactors (t, d, a, b, c, G, C, K);

  
    % next point calculaton:
    t2 = t - dR1/dR2;
    
    % error:
    err = 100 * abs((t2-t) / t) ;
    
    % updates:
    t = t2 ;  %%%%%%%%%%%%%
    
    if contaL == 0
        err = 0 ;
    end
    
end


P = G + t*d ;
GP = P - G;

% 4.- PLOTS (only for checking purposes):


%{
s0 = zeros(1,2);
s1 = zeros(1,2);
s2 = zeros(1,2);
contaL = 0 ;

for t = 0:0.25:20
    
    contaL = contaL + 1 ;
    
    [R, dR1, dR2] = tm1311_otFactors (t, d, a, b, c, G, C, K);
    
    s0(contaL,1) = t ;
    s0(contaL,2) = R ;
    
    s1(contaL,1) = t ;
    s1(contaL,2) = dR1 ;
    
	s2(contaL,1) = t ;
    s2(contaL,2) = dR2 ;
    
end

figure(7)
plot( s0(:,1) ,  s0(:,2) ,'k'); hold on

figure(8)
plot( s1(:,1) ,  s1(:,2) ,'r'); hold on

figure(9)
plot( s2(:,1) ,  s2(:,2) ,'b'); hold on

%}

% triangle and tet:
%{
fr = 10 ;
figure(fr);

pt1372_pTri_iem (Snod, zeros(3,3), 1, fr, 'k', 1, 1, 1, dim);

scatter3(G(1),G(2),G(3),'b'); hold on

quiver3(G(1),G(2),G(3),d(1),d(2),d(3),'r'); hold on

scatter3(P(1),P(2),P(3),'m'); hold on


Tnod = [Snod ; P'];

[R] = th1319_tetR (Tnod);

pt1362_pTet (Tnod, dim, fr, 'r', 0.25, 3, 0);
%}

end

