function [R, dR1, dR2] = tm1311_otFactors (t, d, a, b, c, G, C, K)

% RETURNS:
% R,dR,dR2: regularity and its derivatives


dim = 3 ;


% w norms:

wa = G + t*d - a ;
nwa = norm(wa);

wb = G + t*d - b ;
nwb = norm(wb);

wc = G + t*d - c ;
nwc = norm(wc);


Wi = nwa + nwb + nwc ;


sa = 0 ;
sb = 0 ;
sc = 0 ;

r = 0 ;

for i = 1:1:dim
    
    sa = sa + d(i) * ( G(i) + d(i)*t - a(i) ) ;
    sb = sb + d(i) * ( G(i) + d(i)*t - b(i) ) ;
    sc = sc + d(i) * ( G(i) + d(i)*t - c(i) ) ;
    
    r = r + d(i)^2 ;
    
end

dnwa1 = sa / nwa;
dnwa2 = (r - dnwa1^2) / nwa;

dnwb1 = sb / nwb;
dnwb2 = (r - dnwb1^2) / nwb;

dnwc1 = sc / nwc;
dnwc2 = (r - dnwc1^2) / nwc;

dWi1  = dnwa1 + dnwb1 + dnwc1 ;
dWi2 = dnwa2 + dnwb2 + dnwc2 ;


% A factors & B factors:

A = 0 ;
dA1 = 0 ;

for i = 1:1:dim
    A = A + C(i) * ( G(i) + d(i)*t - a(i) );
    dA1 = dA1 + C(i)*d(i);
end

B = (K + Wi)^3 ;
dB1 = 3*(K + Wi)^2 * dWi1 ;
dB2 = 6*(K + Wi) * dWi1^2 + 3*(K + Wi)^2 * dWi2 ;



% N factors:

N = dA1*B - A*dB1 ;
dN1 = -A * dB2 ;



% Responses:

R = 305.5 * A/B ;

dR1 = 305.5 * N/B^2;

dR2 = 305.5 * (dN1*B^2 - 2*N*B*dB1) / B^4;


end

