function [eTHC, mltF] = th1321_iCoords (pR, Spatches, rTskin, agCurv, ILn, THC, Xtol, DUtol)

% RETURNS:
% coordinates of new end-nodes to be inserted
% mltF = 1 in case the new node lies on MLT, otherwise is 0

% 1.- EXTRACTION AND INITIATION OF VARIABLES:

eTHC = struct('Xv',[], 'Av',[], 'Uv',[], 'Bv',[], 'NCS',[], 'ANv',[]);

cudim = 1 ;
sudim = 2 ;
pudim = 3 ;
xdim  = 3 ;

pDegree = Spatches(pR).INS(1).mDegree;
pNcp    = Spatches(pR).INS(1).mNcp;
pKnotV  = Spatches(pR).INS(1).mKnotV;
pP      = Spatches(pR).INS(1).mP;
pw      = Spatches(pR).INS(1).mw;
pUsize  = Spatches(pR).INS(1).mUsize;
pULims  = [0 1 ; 0 1 ; 0 1 ];
pEP     = Spatches(pR).INS(1).mEP;
pEw     = Spatches(pR).INS(1).mEw;
pCD     = [0 0 0 ];

U0 = zeros(1,sudim);
A0 = zeros(1,pudim);

eBv = zeros(1,cudim); % new end-node
eUv = zeros(1,sudim); 
eAv = zeros(1,pudim); 
eXv = zeros(1,xdim);

eNCS = zeros(1,2); % T-curve and T-surface where it lies

eANv = zeros(1,pudim);

tn = [ ILn(1,1) ; ILn(2,1) ] ; % nodes involved, in tets refs

cs = zeros(2,2); % T-curves and T-surfaces attached to these tet nodes

for v = 1:1:2
    cs(v,1) = THC.NCS(tn(v),1); cs(v,2) = THC.NCS(tn(v),2);
end

% Coordinates of tn:
        
Btn = zeros(2,cudim);
Utn = zeros(2,sudim);
Atn = zeros(2,pudim);
Xtn = zeros(2,xdim);

for v = 1:1:2
    Btn(v,:) = THC.Bv(tn(v),:);
    Utn(v,:) = THC.Uv(tn(v),:);
    Atn(v,:) = THC.Av(tn(v),:);
    Xtn(v,:) = THC.Xv(tn(v),:);     
end
        

if (cs(1,2) > 0 && cs(2,2) > 0) &&  cs(1,2)==cs(2,2)

    mltF = 1 ;
   
    sR = THC.NCS(tn(1),2); % surface ref
    
    % 1.- MISSING LINE BELONGS TO ONE SURFACE OF MLT:
    
    if cs(1,1) > 0 && cs(2,1) > 0

        % 1.1.- Belongs to a contour curve 'cR'

        tnm = min(tn);
        
        cR  = THC.NCS(tnm,1);

        % new node is at mid-location in B-coords
        eBv(1,:) = ( Btn(1,:) + Btn(2,:) ) / 2 ;
        
        % initial points of searching:
        A0(1,:) = ( Atn(1,:) + Atn(2,:) ) / 2 ;
        U0(1,:) = ( Utn(1,:) + Utn(2,:) ) / 2 ;
        
        [eUv, eAv, eXv] = th1327_newCnode (agCurv, cR, rTskin, sR, Spatches, pR, eBv, U0, A0, Xtol, DUtol);

        % Connected T-curve and T-surface:
        eNCS(1,1) = cR ;
        eNCS(1,2) = sR ;
    
    else
        
        % 1.2.- Belongs to a trimming surface 'sR':
        
        % eBv remain = 0
        
     	% new node is at mid-location in U-coords
      	eUv(1,:) = ( Utn(1,:) + Utn(2,:) ) / 2 ;
        
        % initial searching points
        A0(1,:) = ( Atn(1,:) + Atn(2,:) ) / 2 ;
        
        [eBv, eAv, eXv] = th1328_newSnode (rTskin, sR, Spatches, pR, eUv, A0, Xtol, DUtol);
        
        % Connected T-curve and T-surface:
        eNCS(1,1) = 0 ;
        eNCS(1,2) = sR ;

    end
    
else

    mltF = 0 ;
    
    % 2.- MISSING LINE IS INTERNAL:
    
    % eBv and eUv remain as 0
    
	% new node is at mid-location in U-coords
        
	eAv(1,:) = ( Atn(1,:) + Atn(2,:) ) / 2 ;
    
    x = b2601_BNURBS_Xcoord (eAv, pKnotV, pNcp, pDegree, pUsize, pEw, pEP, pudim, xdim);
    
    eXv(1,:) = x(:,1);
    
	% Connected T-curve and T-surface:
	eNCS(1,1) = 0 ;
	eNCS(1,2) = 0 ;
    
    
end



% 3.- ALLOCATES:

eTHC.Bv = eBv ;
eTHC.Uv = eUv ;
eTHC.Av = eAv ;
eTHC.Xv = eXv ;

eTHC.NCS = eNCS ;

eTHC.ANv = eANv ;

end

