function [otpS, B0, U0, At, mNCS, isF] = th1324a_midState (pR, li, THC, tn, ccTp, connT, rTskin)

% RETURNS:
% otpS: object to project selected: "C" / "S" => curve or surface
% B0, U0, At: initial points for pp
% mNCS(1,2): curve and surf ref where the new-mid node is to lie in
% isF: insertion flag = 1 if insertion is still to be done (might not be
% required since detected line may be internal but with both end-nodes at MLT)

% INPUTS:
% tn(2): tet end-nodes of the line to check

agCurv = ccTp(pR).agCurv;

cudim = 1 ;
sudim = 2 ;
pudim = 3 ;
xdim  = 3 ;

nd = 3 ;

Bv = THC.Bv ;
Uv = THC.Uv ;
Av = THC.Av ;
Xv = THC.Xv ;
NCS = THC.NCS ;

pNV = size(NCS,1);

B0 = zeros(1,cudim);
U0 = zeros(1,sudim);
At = zeros(1,pudim);
 
mNCS = zeros(1,2); % T-curve and T-surface where it lies

isF = 0;

% extracts surfaces and curves refs:

cR = [ NCS(tn(1),1); NCS(tn(2),1) ] ;
sR = [ NCS(tn(1),2); NCS(tn(2),2) ] ;

At(1,:) = ( Av(tn(1),:) + Av(tn(2),:) ) / 2 ;


% 1.- ESTABLISHES THE INITIAL POINT for pp AND THE OBJECT TO PROJECT (curve or surface):


otpS = "" ; 


% 1.1.- LINE AT CONTOUR:

if cR(1) > 0 && cR(2) > 0
    
    isF = 1 ;

    % detects which nodes are extremes
    
    enF = zeros(2,1);
    
    for i = 1:1:2 
        if round(Bv(tn(i),1),nd)==0 || round(Bv(tn(i),1),nd)==1 
            enF(i)=1;
        end
    end
    

    if cR(1) == cR(2) 
        
        % both nodes at same curve
        [otpS, B0, mNCS] = th1315_curveA1(agCurv, cR, sR, Bv, tn, enF);
        
        
    elseif cR(1)~=cR(2)
        
        if enF(1)==0 && enF(2)==0
            
            
            % both nodes are internal => searches surface point in the vicinity
            [otpS, U0, mNCS] = th1316_curveB1 (Xv, Uv, tn, NCS, connT);
            
            
        elseif sum(enF)==1 
            
            % only one node at extreme
            [otpS, U0, B0, mNCS] = th1317_curveB2 (li, NCS, connT, agCurv, Xv, Uv, Bv, tn, enF, cR, sR, nd);
            
            
        elseif sum(enF)==2 
            
            
            % both nodes at extreme
            [otpS, B0, mNCS, isF] = th1318_curveB3 (Xv, tn, ccTp, agCurv, pR, sR, nd);
  

        end
        
        
    end
    
end







% 1.2.- LINE INTERNAL TO SURFACE:

if isF == 0
    

    if sR(1) == sR(2) % both nodes at same surface
        
        isF = 1 ;
        
        % checks if it is closed and one nodes lies at the extreme
        
        CS = rTskin(sR(1)).mCS ;

        enF = zeros(2,1);
        cd = 0 ; % closed direction
        
        for di = 1:1:2
            if CS(di)==1
                cd = di ;
            end
        end
        
        if cd > 0
            for k = 1:1:2
                if round(Uv(tn(k),cd),nd)==0 || round(Uv(tn(k),cd),nd)==1
                    enF(k) = 1 ;
                end
            end
        end
        
        if enF(1)==1
            a = 1 ; b = 2 ;
        elseif enF(2)==1
            a = 2 ; b = 1 ;
        end
 
        if sum(CS)>0 && sum(enF>0)
            
            % node a lies at edge (extreme) => needs pp and uses node b (that is IN) for U0
            
            otpS = "S" ;

            U0(1,:) = Uv(tn(b),:);
 
        	mNCS(1,1) = cR(b);
            mNCS(1,2) = sR(b);
            
            
        else
            
            % is either: opens surf, or closed but both nodes IN (not
            % extreme) => direct
            
            otpS = "S" ;

            U0(1,:) = (Uv(tn(1),:) + Uv(tn(2),:)) / 2 ;

            mNCS(1,1) = 0;
            mNCS(1,2) = sR(1);
            
        	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %{
            if (tn(1) == 35 && tn(2) == 15) % || (tn(1) == 15 && tn(2) == 35)


                U0
                Uv(tn(1),:)
                Uv(tn(2),:)

                mNCS
                
            end
            %}
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
            
        end
        
  
    elseif sR(1) ~= sR(2) % nodes have different surfaces refs
        
        % detects node on curve 'a' and node on surface 'b'
        
        a = 0 ; b = 0 ;
        
        if cR(1)>0 
            a = 1; b = 2; 
        elseif cR(2)>0
            a = 2; b = 1; 
        end
        
        % if one node on curve cRa and that curve is common between sR1 and
        % sR1 => uses IN surface point for pp. Otherwise the mid-node lies
        % within the solid and no needs insertion
        
        beF = zeros(2,1); % belong flag
        
        if a>0 && b>0
            
            % checks that cRa belongs to sR1 and sR2

            cRa = cR(a);
            
            for k = 1:1:2
                for lo = 1:1:ccTp(pR).nlo(sR(k))

                    for cu = 1:1:ccTp(pR).ncu(sR(k),lo)

                        if cRa == ccTp(pR).SCcon(sR(k),lo,cu)
                            beF(k,1) = 1 ;
                            break
                        end
                    end

                    if beF(k)==1
                 	break
                    end
                end
            end
            
        end
        
        
        if sum(beF)==2
            
            % uses IN surface point for pp:
            
            isF = 1 ;
            
            otpS = "S" ;
            
            U0(1,:) = Uv(tn(b),:);

        	mNCS(1,1) = cR(b);
            mNCS(1,2) = sR(b);
            
        else
            
            % mid-node lies within the solid, not at surfaces
            isF = 0 ;

        end
 
    end

end




end

