function [connT, THC, NVm, Eiem, Nve] = th1337_upQuad ...
         (connT, NE, THC, Spatches, pR, MLT, rTskin, ccTp, Xtol, DUtol)

% RETURNS: features including mid-vtx where line lies on T-surface

% INPUTS:


% 1.- PRELIMINARIES:

pudim = 3 ;
nd = 3 ;

agCurv = ccTp(pR).agCurv;

Nve = ones(NE,1)*4; % No. vertexes per element

% extended conn. matrix, to account for mid-edge vertexes:

NLI = size(connT.LV,1);

connT.EV = [ connT.EV , zeros(NE,6) ]; 
connT.LV = [ connT.LV , zeros(NLI,1) ];

NVm = size(THC.Av,1);


% local refs conn. matrices:

LN = [ 1 2 3 4 4 4 ; 
       2 3 1 1 2 3 ]'; % lines => nodes 
  
NL = [ 0 1 3 4 ; 
       1 0 2 5 ; 
       3 2 0 6 ; 
       4 5 6 0 ]; % nodes => lines
 

% 2.- INSERTS MID-NODES:
% At lines that are also present in MLT

for li = 1:1:NLI

    tn1 = connT.LV(li,1); % tets nodes refs
    tn2 = connT.LV(li,2);
    
    mn1 = THC.rm(tn1,1); % their corresponding MLT nodes (may be 0)
    mn2 = THC.rm(tn2,1);
    
    
    % 2.1.- Searches line in MLT:
    
    isF = 0 ;
    
    if mn1 > 0 && mn2 > 0
        
        for i = 1:1:size(MLT.tVLcon,2)
            
            mL = MLT.tVLcon(mn1,i); % searches MLT line with the opp node = mn2
            
            if mL > 0
                
                if MLT.tLVcon(mL,1) == mn2 || MLT.tLVcon(mL,2) == mn2
                    isF = 1;
                    break
                end
            end
        end
        
        %{
        for lm = 1:1:MLT.tNL

            m1 = MLT.tLVcon(lm,1);
            m2 = MLT.tLVcon(lm,2);

            if (m1 == mn1 && m2 == mn2) || (m1 == mn2 && m2 == mn1)
                
                isF = 1;
                break
                
            end
        end 
        %}
        
    end
    
    
    % 2.2.- Mid-node state and trial points for pp:
    
    if isF == 1
        
        [otpS, B0, U0, At, mNCS, isF] = th1324a_midState (pR, li, THC, [tn1 tn2], ccTp, connT, rTskin);

    end
    
    
    % 2.3.- Computes mid-node and adds to the tets:

    if isF == 1 
        
        NVm = NVm + 1 ;
        
        % computation:
        
        [mBv, mUv, mAv, mXv] = th1324b_midCoords ...
      	(li, otpS, B0, U0, At, [tn1 tn2] , mNCS, Spatches, pR, rTskin, agCurv, Xtol, DUtol);
            
        
        % updating:

        connT.LV(li,3) = connT.LV(li,2);
        connT.LV(li,2) = NVm ;
  
        THC.Bv = [THC.Bv ; mBv ] ;
        THC.Uv = [THC.Uv ; mUv ] ;
        THC.Av = [THC.Av ; mAv ] ;
        THC.Xv = [THC.Xv ; mXv ] ;
        THC.NCS = [THC.NCS ; mNCS ] ;
        
        nv1 = zeros(1,pudim); % normal is just the average
        nv2 = zeros(1,pudim);
        
        nv1(1,:) = THC.ANv(tn1,:);
        nv2(1,:) = THC.ANv(tn2,:);
        
        nv1 = (nv1 + nv2)/2;
        
        THC.ANv = [THC.ANv ; nv1 ] ;
        
        % ref of MLT corresponding nodes: no MLT nodes associates
        THC.rm = [THC.rm ; zeros(1,1) ] ;

    end

end





% 4.- UPDATES EVcon and Nve:

for e = 1:1:NE
    
    for i = 1:1:6
        
        liR = connT.EL(e,i);
        
        if connT.LV(liR,3)>0
            Nve(e,1) = Nve(e,1) + 1 ;
            connT.EV(e,Nve(e)) = connT.LV(liR,2);
        end
    end
    
end



% 5.- MATRIX Eiem:

Eiem = zeros(NE,4,4);


for e = 1:1:NE

    iF = 0 ;
    iem = zeros(4,4);
	conta = 4 ;
    
    for i = 1:1:6

        lR = connT.EL(e,i);

        if connT.LV(lR,3)>0

            iF = 1 ;
            
            % nodes of the line refs:
            
            tn1 = connT.LV(lR,1);
            n3 = connT.LV(lR,3);

            
            % nodes local refs:
            
            lnr1 = 0 ; lnr3 = 0 ;
            
            for j = 1:1:4
                if connT.EV(e,j) == tn1 
                    lnr1 = j ;
                end
                if connT.EV(e,j) == n3 
                    lnr3 = j ;
                end                
            end
            

            % finds the line position within the tet
            llr = NL(lnr1,lnr3);
            
            conta = conta + 1;
            Eiem(e,LN(llr,1), LN(llr,2)) = conta ;
            Eiem(e,LN(llr,2), LN(llr,1)) = conta ;

            iem(LN(llr,1), LN(llr,2)) = conta ;
            iem(LN(llr,2), LN(llr,1)) = conta ;
            
        end  
    end

end




end