function [z, eR] = tr1348_interpTriLQ (c, Eiem, NE, TVcon, Cv, eCLims, Zv, zdim, nd)

% RETURNS: 
% z(zdim,1): interpolated values for c location
% if c is outside the triangulation => projects to cloasest edge

% eR: element ref. where c lies in


% it a mapping from C(2) to Z(zdim): parameter to physical.

% Rquires to go back to parent space, and then to physical:
% c => rs, and then rs => z

% For linear or quadratic triangles !

% INPUTS:
% Eiem(NE,3,3): stores inserted mid-nodes
% TVcon(NT,3): conn. matrix triangles VS vertexes
% Vc(NV,2): coordiantes of vertexes
% Vz(NV,zdim): image values of vertexes
% c(2): coordinates of point where we want values to be computed
% zdim: No. dims of the image (No. cols of Vz)

Nnc = 3 ; % No. corner nodes
iem = zeros(3,3);


% 1.- TRIANGLE REF. WHERE c BELONGS TO => eR 

Cnod = zeros(3,2);

eR = 0;

% use them in case no elemetn is found (eR = 0)
D = zeros(1,2); % distance to elements, elem. reference and relative ref.
cpr = zeros(NE,2); % coordinate projected on closest element side

contaE = 0 ;

for e = 1:1:NE
    
    % only checks if inside envelope limits:
    
    if (round(eCLims(e,1,1),nd) <= round(c(1),nd) && round(c(1),nd) <= round(eCLims(e,1,2),nd)) && ...
       (round(eCLims(e,2,1),nd) <= round(c(2),nd) && round(c(2),nd) <= round(eCLims(e,2,2),nd)) 
        
        for i = 1:1:3
            Cnod(i,:) = Cv(TVcon(e,i),:); % recall for quadratic triangles: the 3 first nodes are the corners, then the other 3 mid points
        end
   
        [IN,dist,pr] = tr1344_PandTri (c, Cnod, 'closed', nd);
       
        contaE = contaE + 1 ;
        D(contaE,1) = dist;
        D(contaE,2) = e ;
        cpr(e,:)    = pr(:,1) ;
        
        
        if IN==1
           
            iem(:,:) = Eiem(e,:,:);
            Nnt = Nnc + nnz(iem)/2;
            Znod = zeros(Nnt,zdim); % element nodes Zcoord's
            
            for i = 1:1:Nnt
                Znod(i,:) = Zv(TVcon(e,i),:);
            end
            
            eR = e ;
            c2 = c ;
            
            break
           
        end
   
    end 
end




    
% In case even didn't find element enveloping the point =>
% => Searches the closest element.

if contaE == 0
    
    for e = 1:1:NE

        for i = 1:1:3
            Cnod(i,:) = Cv(TVcon(e,i),:); % recall for quadratic triangles: the 3 first nodes are the corners, then the other 3 mid points
        end
        
        [IN, dist, pr] = tr1344_PandTri (c, Cnod, 'closed', nd);

        D(e,1) = dist;
        D(e,2) = e ;
        cpr(e,:) = pr(:,1) ;
    end

end





% in case do not found element => selects closest point to closest element
if eR == 0
    
    D = sortrows(D,1);
    eR = D(1,2);
    
    c2 = zeros(2,1);
    c2(:,1) = cpr(eR,:);
    
    for i = 1:1:3
        Cnod(i,:) = Cv(TVcon(eR,i),:);
    end
    
	iem(:,:) = Eiem(eR,:,:);
	Nnt = Nnc + nnz(iem)/2;
	Znod = zeros(Nnt,zdim); % element nodes Zcoord's
    
    for i = 1:1:Nnt
        Znod(i,:) = Zv(TVcon(eR,i),:);
    end

    
  
end


% 2.- COMPUTATION OF rs COORD'S (PARENT SPACE) (in case found eR);


% c => rs
[rs] = tr1349_ParentCoords (c2, 3, Cnod);



% 3.- COMPUTATION of z :


% rs => z
[z, ~] = mp1352_TRImapp (rs, Znod, 2, zdim, iem);




%{
if round(z(1),3)==3.333 && round(z(2),2)==-16.67 
  Znod
  Nne
  c
  zdim
end
%}

%{
z = zeros(zdim,1);

% extracts triangle nodes x-coor's:
    
xt = zeros(3,2);
for i = 1:1:3
	nR = TVcon(eR,i);
	xt(i,1) = Vc(nR,1);
	xt(i,2) = Vc(nR,2);
end
    
    
% total triangle area:

x = [xt(1,1) ; xt(2,1) ;xt(3,1)];
y = [xt(1,2) ; xt(2,2) ;xt(3,2)];
At = polyarea(x,y);
    
    
% Finds basis functions values using areas
    
ov = [2 3 ; 3 1 ; 1 2]; % opposite vertex local ref.
N = zeros(3,1);
    
for i = 1:1:3
        
    x = [ xt(ov(i,1),1) ; xt(ov(i,2),1) ; c(1) ];
	y = [ xt(ov(i,1),2) ; xt(ov(i,2),2) ; c(2) ];
	Ai = polyarea(x,y);
        
	N(i)= Ai/At;    
end
    
     
% Computes response:
    
for i = 1:1:3
        
    nR = TVcon(eR,i);
    for di = 1:1:zdim
        z(di,1) = z(di,1) + N(i)*Vz(nR,di);
    end
end
%}
  

end