function [a, J2, dJ2] = mp1352_TRImapp (rs, Xnod, Pdim, Xdim, iem)

% RETURNS:
% a(Xdim,1): coord's in mapped space for parent 'r' coord's in triangle
% J2: Jacobian from parent to mapped space

% J2 = | X,r X,s  
%        Y,r Y,s  ;
%        Z,r Z,s | ;

% INPUTS:
% rs(dim,1): coord's in parent to mapp
% Xnod(Nnt,dim): tet nods mapped coordiantes
% Pdim, Xdim: dims of parent and mapped spaces. Pdim = 2
% iem (3,3): 'inserted edge matrix': component indicates inserted node in
% edge ref. and row/cols which corner nodes are involved
%   e.g. iem(2,3) = iem(3,2) = 6 => edge btwn 2 and 3 has insertion, and is the
%   6th node (previously the 5th was inserted somewhre else)
%   eim is symmetric and diagonal is 0


% No. inserted nodes at edges and total number of nodes:

Nne = nnz(iem)/2;
Nnt = 3 + Nne ;

N  = zeros(Nnt,1); % to store basis functions, the first 3 are for the 3 corners
dN = zeros(Nnt,Pdim); % derivatives of basis functions w.r.t. parent directions

da1 = zeros(Xdim,1); % derivatives at location rs
da2 = zeros(Xdim,1);

% coordinates expanded to 3 components:

t = 1-rs(1)-rs(2); % the third dimension
r = [rs(1) ; rs(2) ; t ]; 


% 1.- BASIS FUNCTIONS & DERIVATIVES:

% 1.1.- Edge-nodes:

if Nne > 0

    for i = 1:1:2
        for j = i+1:1:3

            if iem(i,j)~=0

                k = iem(i,j); % reference of edge node
                
                % basis function:
                
                N(k,1) = 4*r(i)*r(j);
                
                
                % derivatives:
                
                [dNk] = mp1355_d4rs (r,i,j);
                
                dN(k,1) = dNk(1) ;
             	dN(k,2) = dNk(2) ;
                    
            end

        end
    end

end


% 1.2.- Corner-nodes:

for i = 1:1:3
    
    % basis function:
    N(i) = r(i);
    
    % derivatives:
    if i==1
        dN(i,1) = 1 ;
        dN(i,2) = 0 ;
    elseif i==2
        dN(i,1) = 0 ;
        dN(i,2) = 1 ;
    elseif i==3
        dN(i,1) = -1 ;
        dN(i,2) = -1 ;
    end
    
 
    for j = 1:1:3

        if iem(i,j)~=0
                    
            k = iem(i,j); % reference of edge node

            % basis function
            N(i,1) = N(i,1) - 0.5*N(k,1);

            % derivatives
            [dNk] = mp1355_d4rs (r,i,j);

            dN(i,1) = dN(i,1) - 0.5*dNk(1) ;
            dN(i,2) = dN(i,2) - 0.5*dNk(2) ;

        end

    end
end



% 2.- COORDINATES & DERIVATIVES CALCULATION:

a = zeros(Xdim,1);

for di = 1:1:Xdim
    
    for n = 1:1:Nnt

        a(di,1) = a(di,1) + N(n)*Xnod(n,di);
        
        da1(di,1) = da1(di,1) + dN(n,1)*Xnod(n,di);
        da2(di,1) = da2(di,1) + dN(n,2)*Xnod(n,di);
        
    end
    
end



% 3.- JACOBIAN:

% 3.1.- Assemblies the jacobian: J2 = zeros(Pdim,Xdim)

%J2 = [da1';da2'];
J2 = [da1,da2];

% 3.2.- Computation:

if Xdim==2
        
    dJ2 = det(J2);
        
elseif Xdim==3
        
	dJ2 = norm(cross(da1,da2));
        
end


end

