function [a, J2, dJ2] = mp1353_TETmapp (rst, Xnod, Pdim, Xdim, iem)

% RETURNS:
% a(dim,1): coord's in mapped space for parent 'r' coord's in tetraedral
% J2: Jacobian from parent to mapped space

% INPUTS:
% rst(dim,1): coord's in parent
% Xnod(Nnt,dim): tet nods mapped coordiantes
% Xdim, Pdim: dims of mapped and parent spaces. Pdim = 3
% iem (4,4): 'inserted edge matrix': component indicates inserted node in
% edge ref. and row/cols which corner nodes are involved
%   e.g. iem(2,3) = iem(3,2) = 6 => edge btwn 2 and has insertion, and is the
%   6th node (previously the 5th was inserted somewhre else)
%   eim is symmetric and diagonal is 0


% No. inserted nodes at edges and total number of nodes:

Nne = nnz(iem)/2;
Nnt = 4 + Nne ;

N = zeros(Nnt,1); % to store basis functions, the first 4 are for the 4 corners
dN = zeros(Nnt,Pdim); % derivatives of basis functions w.r.t. parent directions

da1 = zeros(Xdim,1); % derivatives at location rst
da2 = zeros(Xdim,1);
da3 = zeros(Xdim,1);

% coordinates expanded to 4 components:

u = 1-rst(1)-rst(2)-rst(3); % the fourth dimension
r = [rst(1) ; rst(2) ; rst(3) ; u ]; 


% 1.- BASIS FUNCTIONS & DERIVATIVES:

% 1.1.- Edge-nodes:

if Nne>0

    for i = 1:1:3
        for j = i+1:1:4

            if iem(i,j)~=0

                k = iem(i,j); % reference of edge node
                
                N(k,1) = 4*r(i)*r(j);
                
                
             	% derivatives:
                [dNk] = mp1356_d4rst (r,i,j);
                               
                dN(k,1) = dNk(1) ;
             	dN(k,2) = dNk(2) ;
                dN(k,3) = dNk(3) ;
            end

        end
    end

end



% 1.2.- Corner-nodes:

for i = 1:1:4
    
    % basis function:
    N(i) = r(i);
    
 	% derivatives:
    if i==1
        dN(i,1) = 1 ;
        dN(i,2) = 0 ;
        dN(i,3) = 0 ;
    elseif i==2
        dN(i,1) = 0 ;
        dN(i,2) = 1 ;
        dN(i,3) = 0 ;
    elseif i==3
        dN(i,1) = 0 ;
        dN(i,2) = 0 ;
        dN(i,3) = 1 ;
 	elseif i==4
        dN(i,1) = -1 ;
        dN(i,2) = -1 ;
        dN(i,3) = -1 ;
    end
    
    for j = 1:1:4

        if iem(i,j)~=0
                
            k = iem(i,j); % reference of edge node

            % basis function
            N(i) = N(i) - 0.5*N(k);

           	% derivatives
            [dNk] = mp1356_d4rst (r,i,j);
          
            dN(i,1) = dN(i,1) - 0.5*dNk(1) ;
            dN(i,2) = dN(i,2) - 0.5*dNk(2) ;
            dN(i,3) = dN(i,3) - 0.5*dNk(3) ;
        end

    end
end




% 2.- COORDINATES & DERIVATIVES CALCULATION:
    
a = zeros(Xdim,1);

for di = 1:1:Xdim
    
    for n = 1:1:Nnt
        
        a(di,1) = a(di,1) + N(n)*Xnod(n,di);
        
    	da1(di,1) = da1(di,1) + dN(n,1)*Xnod(n,di);
        da2(di,1) = da2(di,1) + dN(n,2)*Xnod(n,di);
        da3(di,1) = da3(di,1) + dN(n,3)*Xnod(n,di);
        
    end
    
end



% 3.- JACOBIAN:

% 3.1.- Assemblies the jacobian: J2 = zeros(Pdim,Xdim)

J2 = [ da1'; da2'; da3' ];


% 3.2.- Computation:

dJ2 = det(J2);
        

%{

% Jacobian:

J2 = zeros(Pdim,Xdim);

if Nnt == 4 % strcmp(ttp,'linear') == 1 
  
	% Jacobian: each col is the derivative w.r.t. one direction of Parent-space:
    for di = 1:1:Pdim  
        J2(1,di) = Xnod(1,di) - Xnod(4,di);
        J2(2,di) = Xnod(2,di) - Xnod(4,di);
        J2(3,di) = Xnod(3,di) - Xnod(4,di);
    end
    
elseif Nnt == 10  % strcmp(ttp,'quadratic') == 1  % Nnt = 10 
 
else
    
    if Nnt == 5
        
    elseif Nnt == 6
          
    elseif Nnt == 7
         
	elseif Nnt == 8
          
	elseif Nnt == 9
           
    end
end

%}


end

