function [ ] = pt1369_TriMDMesh (stL, stT, Cv, NLI, NE, LVcon, TVcon, Eiem, fr, eF, sF, lrF, vrF, erF, al)

% plots triangular mesh with mixed degrees

% eF, sF: edges and surface flags
% vrF, erF: vtx and elem. refs flags (=1 wirte refs)

if fr > 0
    figure(fr);
end

xdim = 3 ;

% expand conn. matrices in case they are just linear:

if size(LVcon,2)==2
    LVcon = [LVcon zeros(NLI,1)];
end
if size(TVcon,2)==3
    TVcon = [TVcon zeros(NE,3)];
end


% 1.- EDGES:

LinsF = 1 ;

if eF == 1

    for li = 1:1:NLI

        % extracts the line points 
        [Lx] = pt1363_lineCoord (li, stL, Cv, LVcon, xdim, LinsF);
        
        % plots the line
        plot3(Lx(:,1),Lx(:,2),Lx(:,3),'k'); hold on

        if vrF == 1
            for i = 1:1:2
                
                nr = LVcon(li,i);
                
                tx = zeros(3,1);
                for di = 1:1:3
                    tx(di,1)= Cv(nr,di);
                end
             
                text(tx(1), tx(2), tx(3), num2str(nr)); hold on
            end
        end
        
        
    end

end

% elements refs

if erF==1
  
   for e = 1:1:NE
      
       c = zeros(3,1);
       for i = 1:1:3
           
           nr = TVcon(e,i);
           for di = 1:1:3
              c(di) = c(di) + Cv(nr,di)/3; 
           end
       end
       
       text(c(1), c(2), c(3), num2str(e)); hold on
       
   end
    
end


% 2.- SURFACE ITSELF:

if sF == 1

    for e = 1:1:NE

        iem = zeros(3,3);
        iem(:,:) = Eiem(e,:,:);
        mvr = max(max(iem));
        
        % No. nodes per element and step used for r-s:
        if mvr==0
            NnE = 3;
            stTu = 1 ;
        else
            NnE = mvr;
            stTu = stT;
        end
        
        % triangle nodes coordinates:
        Cnod = zeros(NnE,xdim);
        for i = 1:1:NnE
            nR = TVcon(e,i);
            Cnod(i,:) = Cv(nR,:);
        end

 
        % computes triangle points in physical space

        R = zeros(1,2);     % coords in parent space
        P = zeros(1,xdim);  % coords in projected space

        conta = 0 ;

        for r = 0:stTu:1

            s0 = 0 ;
            s1 = 1-r;

            Ns = round((s1-s0)/stTu,0);
            Ds = (s1-s0)/Ns ;

            
            if r == 1
                Ns = 0;
            end

            s = 0 ;
            for i = 1:1:Ns+1

                %C = mp1352_TRImapp (Cnod, NnE, [r;s], xdim);

                [C, ~] = mp1352_TRImapp ([r;s], Cnod, 2, xdim, iem);
                
                conta = conta + 1 ;

                R(conta,1) = r ;
                R(conta,2) = s ;

                for di = 1:1:xdim
                    P(conta,di) = C(di,1);
                end

                s = s + Ds;
            end

        end

        
        
        % triangulates in parent space and transfers triangles coords:

        TRI = delaunay(R); % conn. matrix
        NER = size(TRI,1); % No.elemens within the triangle

        X = zeros(xdim,NER);
        Y = zeros(xdim,NER);
        Z = zeros(xdim,NER);
        G = zeros(xdim,NER);

        for col = 1:1:NER
            for i = 1:1:3

                %nR = conn(col,i);
                nR = TRI(col,i);

                X(i,col) = P(nR,1);
                Y(i,col) = P(nR,2);
                Z(i,col) = P(nR,3);
            end
        end

        % plots:
        ptri = fill3(X,Y,Z,G, 'LineStyle','none'); hold on
        alpha(ptri,al);
    end


    
end



if fr > 0
    axis equal
end




end