function [connS, NSI, Siem] = th1381_connS (NE, connT)

connS = struct( 'SE',[], 'SL',[], 'SV',[], 'VS', [], 'tsF',[] );

% VS(NV,...) only applies to end-nodes, not to mid-nodes
% tsF(NSI,1): side on trimming surface = 1, if no trimm.side = 0

% COMPUTES SEcon and SLcon MATRICES and No. SIDES:

% SEcon:

SEcon = zeros(NE*4,2);

for e = 1:1:NE
    for k = 1:1:4
        
        siR = connT.ES(e,k);
        
        if SEcon(siR,1)==0  
            SEcon(siR,1) = e ;
        else
            SEcon(siR,2) = e ; 
        end
    end
end

for i = 1:1:size(SEcon,1)
    if SEcon(i,1)==0
        break
    end
end

SEcon = SEcon(1:i-1,:);

NSI = size(SEcon,1);


% SLcon:

SLcon = zeros(NSI,3);

for s = 1:1:NSI

    snR = [ connT.SV(s,1) , connT.SV(s,2), connT.SV(s,3) ] ;

    eR = SEcon(s,1); % with first element is sufficient
    
    conta = 0;  
	tmpSL = zeros(1,1);
            
    for l = 1:1:6
                
        lR = connT.EL(eR,l);
                
        lnR1 = connT.LV(lR,1);
        lnR2 = connT.LV(lR,2);
        
        if size(connT.LV,2)==3
            if connT.LV(lR,3)>0
                lnR2 = connT.LV(lR,3);
            end
        end

        
        for i = 1:1:3
            if lnR1 == snR(i)
                for j = 1:1:3
                    if lnR2 == snR(j)
                        conta = conta + 1 ;
                        tmpSL(conta,1) = lR ;
                    end
                end
            end
        end
            
    end
            
	[tmpSL,~] = GEN_16_RemoveRepeatedRows(tmpSL,1,0.1);

    SLcon(s,:) = tmpSL(:,1);      

end


% SVcon: expanded to consider mid-nodes if they exist. 
% The order of nodes is driven by connT.SV


SVcon = [ connT.SV , zeros(NSI,3) ];

Siem = zeros(NSI,3,3);

for s = 1:1:NSI
    
    er = [ 1 2 3 1 ];
    eSV = [ connT.SV(s,1) , connT.SV(s,2) ,connT.SV(s,3) ,connT.SV(s,1) ] ;
    
    conta = 3 ;
    for n = 1:1:3
        
        n1 = eSV(n); n2 = eSV(n+1);
        
        % finds line and if it has mid-node:
        
        for l = 1:1:3
            
            lR = SLcon(s,l);
            
            if size(connT.LV,2)==3
            if connT.LV(lR,3) > 0 % only the mid-noded lines are relevant here
                
                nL1 = connT.LV(lR,1);
                nL2 = connT.LV(lR,3);
                
                if (nL1==n1 && nL2==n2) || (nL1==n2 && nL2==n1) 
                    
                    conta = conta + 1 ;
                    
                    SVcon(s,conta) = connT.LV(lR,2);
                    
                    Siem(s,er(n),er(n+1)) = conta ;
                    Siem(s,er(n+1),er(n)) = conta ;
                    
                    break
                    
                end

            end
            end
            
        end
    end
end


% VScon:

NV = size(connT.VL,1);
VScon = zeros(NV,1);

nsn = zeros(NV,1);  % No. sides per node

for s = 1:1:NSI
    
    for k = 1:1:3
        
        nR = SVcon(s,k);
        
        nsn(nR,1) = nsn(nR,1)+1;
        
        if size(VScon,2)<nsn(nR)
            VScon = [ VScon, zeros(NV,1) ] ;
        end
        
        VScon(nR,nsn(nR)) = s ; 
        
    end
end



% tsF:
% Side is in T-surface if the three end-nodes are on T-surface.
% In case the trhee nodes belong to the same contour curve it is not on T-surface !

tsF = zeros(NSI,1);

% {
for s = 1:1:NSI 
    
    if SVcon(s,6) > 0
        tsF(s,1) = 1 ;
    end
 
end
%}


% allocates:

connS.SE = SEcon ;
connS.SL = SLcon ;
connS.SV = SVcon ;
connS.VS = VScon ;
connS.tsF = tsF ;

end

