function [isi] = ts1601_sidesInv (contaL, ILn, NE, connT, connS, THC, tolL, tolA, nd)

% RETURNS: intersected sides by missing line from MLT

Nsi = 0 ; % No. sides intersected
isi(1,1) = struct('R', 0 , 'Acoor', [], 'tp', 0); % side ref, intersection Acoords, type (1 in, 2 edge)

% INPUTS:
% ILn(2,2): reference of nodes of the missing line in tets (1st col) and in MLT (2nd col)

% 1.- PRELIMINARIES:

Av = THC.Av ;
pudim = 3 ;

% line jointing both nodes in patch parameter space
[rv, rP1, rP2] = ts1602_mLAspace (ILn, Av,0,0,0);



% 2.- SIDES CHECK:
  
NSI = size(connS.SE,1);

for s = 1:1:NSI
    
    % 2.1.- Geometrical features:
    
    % coords of nodes of side:
    
    Anod = zeros(3,pudim);
    for k = 1:1:3 
        nR = connS.SV(s,k);
        Anod(k,:) = Av(nR,:);
    end
    
    
    % plane features:
    
    u = zeros(pudim,1);
    v = zeros(pudim,1);
    P = zeros(pudim,1);
    
    for di = 1:1:pudim
        u(di,1) = Anod(2,di) - Anod(1,di);
        v(di,1) = Anod(3,di) - Anod(1,di);
        P(di,1) = Anod(1,di);
    end
    
    
    % 2.2.- Intersection line-plane:
    
    [IP,~] = go2545_PlaneLineIntersec (u, v, P, rv, rP1);
    
    
    % checks if intersection lies within the triangle:

    IN = go2546_PinTriang (Anod, IP, tolL, tolA, nd);
    ON = go2520_PonSegment (IP, rP1, rP2, pudim, tolL);

    
    % 2.3.- Assesment of intersection:

    if IN > 0 && ON == 1 % detects intersection
        
        CO1 = go2542_coincidentP (IP, rP1, pudim, nd);
        CO2 = go2542_coincidentP (IP, rP2, pudim, nd);
        
        if CO1==0 && CO2==0 % ntersection is not at the line nodes
            
            if IN > 0
                
                % stores ref
                
                Nsi = Nsi + 1 ;
                
                isi(Nsi,1).R = s ;
                isi(Nsi,1).Acoor = IP ;
                isi(Nsi,1).tp = IN ;
 
            end
            
        end
  
    end
    
end


end




