function [mpS, meR] = ts1605_optState (contaL, cnR, meR, THC, pNE, connT, connS, nnR)

% RETURNS:
% State that optimizes the tets after splitting, i.e. measures the
% alternatives with the node at edge and side and selects the option that
% yields the most regular tets.

% INPUTS:
% meR: 'main element ref', or element where the new node lies in, it is 0
% if ne the new node lies outside the tet. mesh
% nnR: new node ref (the last one that was already included from eTHC)


% NOTE: nnR is fixed ! coz is given by MLT

tolL = 0.5 ;
tolA = 0.5 ;
nd = 6 ;

pudim = 3 ;
eA = zeros(pudim,1);
eA(:,1) = THC.Av(nnR,:);

Av = THC.Av;
Anod = zeros(4,pudim);


% 1.- CASE meR = 0 (nnR lies outside the tet. mesh):

if meR == 0
    
    % 1.1.- Selects one line if it's closest than a threshold distance 
    % (which is 1/3 of the edge length). Only can select external lines,
    % i.e. with both nodes at the trimming surface.
    
    % Selects susceptible lines (external and attached to cnR):
    
    Nsl = 0 ;
    slm = zeros(1,2); % stores line ref. and distance.
    
    for k = 1:1:size(connT.VL,2)
        
        lR = connT.VL(cnR,k);
        
        if lR > 0
            
            n1 = connT.LV(lR,1);
            n2 = connT.LV(lR,2);
            
            if THC.NCS(n1,2) > 0 && THC.NCS(n2,2) > 0
                
                a1 = THC.Av(n1,:);
                a2 = THC.Av(n2,:);
                
                L = norm(a2-a1);
                vr = a2-a1;
                
                [dist, ~] = go2527_distPtoRecta (eA, a1, vr, pudim);

                if dist < L/3
                    Nsl = Nsl + 1 ;
                    slm(Nsl,1) = lR;
                    slm(Nsl,2) = dist;
                end
            end
  
        end
    end
    
    
    if Nsl > 0
        
        % assumes it is and 'edge case':
        
        slm = sortrows(slm,2);
        clR = slm(1,1); % closest line ref.
        
        ln1 = connT.LV(clR,1); % refs. of nodes of the line
        ln2 = connT.LV(clR,2);

        pos = 3 ;
        meR = connT.LE(clR,1);
        
        r = [1 2 3 4 4 4;
             2 3 1 1 2 3 ]';

         seR = zeros(2,1); % relative position of the edge
         for k = 1:1:6
             
             n1 = connT.EV(meR,r(k,1));
             n2 = connT.EV(meR,r(k,2));

             if (n1 == ln1 && n2 == ln2) || (n1 == ln2 && n2 == ln1)
                 seR(1) = r(k,1);
                 seR(2) = r(k,2);
                 break
             end
         end
        
        mpS = [ pos meR seR(1) seR(2) ];
        
    end



    % 1.2.- If no edge close, then selects the closest side 
    % (the side must have all nodes at MLT ?? not sure):
    
    if Nsl == 0
        

        Nsf = 0 ;
        sfm = zeros(1,3); % stores element ref, closest facet and distance from eA to such facet
        
        for e = 1:1:size(connT.VE,2)
            
            eR = connT.VE(cnR,e);
            
            if eR > 0
                
                for k = 1:1:4   
                    nr = connT.EV(eR,k);
                    Anod(k,:) = THC.Av(nr,:);
                end
                
                
                [cfR, dmin] = tm1303b_closerFacet (Anod, eA, tolL, tolA, nd);

                if cfR > 0
                    
                    Nsf = Nsf + 1 ;
                    
                    sfm(Nsf,1) = eR ;
                    sfm(Nsf,2) = cfR ;
                    sfm(Nsf,3) = dmin ;
                    
                    %{

                    cfR
                    figure(76)

                    scatter3(eA(1), eA(2), eA(3)); hold on

                    pt1373_pTetConn (eR, connT, Av, 76, 'k', 1, 1, 0, 0)

                    axis equal

                    if gg==0
                    end

                    %}
                end
            end
        end
        
        sfm = sortrows(sfm,3);
        
        
        % assumes it is a 'side case'

        pos = 2 ;
        meR = sfm(1,1) ;
        seR = zeros(2,1); % relative position of the facet
        seR(1) = sfm(1,2) ;
        
        mpS = [ pos meR seR(1) seR(2) ];

    end
   
    return
    
end



% 2.- CASE nnR LIES WITHIN THE TET. MESH:


for k = 1:1:4
    nR = connT.EV(meR,k);
    Anod(k,:) = THC.Av(nR,:);
end


% 2.1.- CHECKS ALL CONFIGURATIONS:
% Measures regularity (R), the higher R the better

% a.- Node IN: 4 tets resultant

R4 = zeros(1,2); % min value (worst case) and average

[NEt, connTt] = mi1671_TetDiv4 (pNE, connT, meR, nnR);

n = 4 ;
er = [meR, pNE+1 , pNE+2 , pNE+3 ]; 

[Rt] = ts1606_Rtets (n, er, connTt, Av);
                 
R4(1,1) = min(Rt);
R4(1,2) = sum(Rt)/4;



% b.- Node on closest side:

R3 = zeros(1,2); % min value (worst case) and average

% finds closest facet (side):

[cfR, dmin] = tm1303b_closerFacet (Anod, eA, tolL, tolA, nd);

%{
r = [2 3 4;
     1 3 4;
     1 2 4;
     1 2 3 ];
 
a = zeros(pudim,1);
b = zeros(pudim,1);
c = zeros(pudim,1);

dmin = 9999999 ;

for k = 1:1:4
    
    a(:,1) = Anod(r(k,1),:);
    b(:,1) = Anod(r(k,2),:);
    c(:,1) = Anod(r(k,3),:);
    
	[A,B,C,D] = go2506_Plane3P (a,b,c);
    [dist,~,~] = go2534_distPtoPlan (eA, A, B, C, D);
    
    if dist < dmin
        
        dmin = dist;
        cfR = k;
    end
end
 %}

% attached tets to such side and the local side ref : atR(Nae,1) & LsRd(Nae,1)

[Nae,atR, LsRd] = ts1654_attESL (cfR, meR, connT, connS);
    
    
% divides at 'sL' side:
    
[NEt, connTt] = ts1653_TetDiv3 (THC, pNE, connT, Nae, atR, nnR, LsRd);
    

% computes the regularity of all the triangles and stores:
    
n = 3*Nae;
er = zeros(n,1);
    
accE = 0 ;
conta = 0 ;
for i = 1:3:n
        
    conta = conta + 1 ;
        
	er(i+0,1) = atR(conta,1);
 	er(i+1,1) = pNE + accE + 1 ;
  	er(i+2,1) = pNE + accE + 2 ;
        
	accE = accE + 2 ;    
end
    
[Rt] = ts1606_Rtets (n, er, connTt, Av);
                  
R3(1,1) = min(Rt);
R3(1,2) = sum(Rt)/n;
    
 

% c.- Node as if it was on closest edge:

R2 = zeros(1,2); % min. value (worst case) and average

% finds closest edge:

r = [1 2 3 4 4 4;
     2 3 1 1 2 3 ]';

dmin = 9999999 ;

Pr = zeros(pudim,1);
vr = zeros(pudim,1);

for k = 1:1:6
    
    Pr(:,1) = Anod(r(k,1),:);
    vr(:,1) = Anod(r(k,1),:) - Anod(r(k,2),:);
    
    [dist,~] = go2527_distPtoRecta (eA, Pr, vr, pudim);
    
    if dist < dmin
        dmin = dist;
        eL = k ;
        ENc = [ r(k,1) r(k,2) ];
    end
end
 
% attached tets to such line and the local edge ref : atR(Nae,1) & LeRd(Nae,2)
[Nae, atR, LeRd] = ei1664_attELL (eL, meR, connT);
    
% divides at 'eL' edge:
[NEt, EVcon] = ei1663_TetDiv2 (THC, pNE, connT.EV, Nae, atR, nnR, LeRd);

connTt.EV = EVcon ;

% computes the regularity of all the triangles and stores:
    
n = 2*Nae;
er = zeros(n,1);
    
accE = 0 ;
conta = 0 ;
for i = 1:2:n
        
	conta = conta + 1 ;
        
	er(i+0,1) = atR(conta,1);
 	er(i+1,1) = pNE + accE + 1 ;
        
 	accE = accE + 1 ;
end
    
[Rt] = ts1606_Rtets (n, er, connTt, Av);

R2(1,1) = min(Rt);
R2(1,2) = sum(Rt)/n;
    
%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%
%{
    
if contaL == 10 
        
        fr = 33 ;
        for k = 1:1:Nae
            pt1373_pTetConn (atR(k), connT, Av, fr, 'k', 0.8, 1.25, 0, 0);
        end
        
        fr = 43 ;
        for k = 1:2:n
            pt1373_pTetConn (er(k), connTt, Av, fr, 'b', 0.8, 1.25, 0, 0);
        end
 
        
        x = zeros(2,pudim);
        
        for j = 1:1:2
            
            n1 = connT.EV(meR,2);
            n2 = connT.EV(meR,3);
            
            x(1,:) = THC.Av(n1,:);
            x(2,:) = THC.Av(n2,:);

        end
        
        plot3( x(:,1),  x(:,2),  x(:,3), 'm'); hold on
        
end
%}
%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%



% 2.2.- SELECTS THE BEST CONFIGURATION:
% Preference of min value of R, arranges first by minimum and then by
% average. Selects the highest one (last in the list)

agg = [ R4 1 0 0 ;
        R3 2 cfR 0 ; 
        R2 3 ENc];
 
agg = sortrows( agg, [1 2] );
  
pos = agg(3,3);
seR(1) = agg(3,4);
seR(2) = agg(3,5);

mpS = [ pos meR seR(1) seR(2) ];





% 3.- Prints (just for checking):

%{

fr = meR ;
pt1373_pTetConn (meR, connT, Av, fr, 'r', 0.1, 3.25, 0); 



if mpS(1)==1
    
    [NEt, connTt] = mi1671_TetDiv4 (pNE, connT, meR, nnR);
    
    n = 4 ;
    er = [ meR, NEt-2, NEt-1, NEt-0 ] ; 
    
elseif mpS(1)==2
    
    [NEt, connTt] = ts1653_TetDiv3 (THC, pNE, connT, 1, meR, nnR, mpS(3));
    
	n = 3 ;
    er = [ meR, NEt-1, NEt-0 ] ; 
    
elseif mpS(1)==3  
    
    [NEt, connTt] = ei1663_TetDiv2 (THC, pNE, connT, 1, meR, nnR, [mpS(3) mpS(4)]);
	
    n = 2 ;
    er = [ meR, NEt-0 ] ; 
end


for k = 1:1:n
    
    pt1373_pTetConn (er(k), connTt, Av, fr, 'k', 0.8, 1.25, 0);

end 

%}


end

