function [mn, B0, U0, A0, csM] = ts1615_info2projN (rTskin, ILn, THC, N)

% RETURNS:
% info to project the point onto the MLT surface
% mn(2,1): nodes ref of MLT aff'd line
% B0, U0, A0: initial points for pp
% csM(2,1): curve and surf refs attached to the node to insert

% INPUTS:
% N(2): proportion of each end node to compute 0 locations

% 1.- PRELIMINARIES:

sudim = 2 ;
pudim = 3 ;

tn = [ ILn(1,1) ; ILn(2,1) ] ; % nodes involved, in tets refs
mn = [ ILn(1,2) ; ILn(2,2) ] ; % nodes involved, in MLT refs
  

% 2.- CURVE AND SURFACE (of the MLT) WHERE THE MID-POINT LIES:
% T-curves and T-surfaces attached to these tet nodes

csM = zeros(2,1); 

csM(1) = 0;
if THC.NCS(tn(1),1)>0 && THC.NCS(tn(2),1)>0
    
    cr1 = THC.NCS(tn(1),1) ;
    cr2 = THC.NCS(tn(2),1) ;
    
    b1 = round(THC.Bv(tn(1),1),6);
    b2 = round(THC.Bv(tn(2),1),6);
    
    if cr1 == cr2
        
        csM(1) = cr1;
        
    else
        
        if b1 == 0 || b1 == 1
            
            csM(1) = cr2;
            
        elseif b2 == 2 || b2 == 1
            
            csM(1) = cr1;
            
        end
        
    end
    
end
    

csM(2) = THC.NCS(tn(1),2);
    
% 3.- MID-POINTS:
%  Mid-point (that is also initial trial points for pp):
   
B0 = 0 ;
U0 = zeros(sudim,1);
A0 = zeros(pudim,1);
   
% 3.1.- Curve:

if csM(1) > 0
    
    B0	= N(1) * THC.Bv(tn(1),1) +  N(2) * THC.Bv(tn(2),1);
    
end


% 3.2.- Surface:

sr = csM(2);

U0(:,1) = N(1) * THC.Uv(tn(1),:) +  N(2) * THC.Uv(tn(2),:);

% detects if:
% - has one closed direction (cd)
% - has one end-coord in cd direction
% - that end-coord is the furthest from the other coord


for di = 1:1:sudim
    
    ad = [2 1];
    
    if rTskin(sr,1).mCS(di)>0
        
        cd = di ;       % closed direction
        op = ad(di);    % open direction
        
        en = 0 ;    % node at limit
        sn = 0 ;    % node inside
        
        ak = [2 1];
        for k = 1:1:2
            
            if round(THC.Uv(tn(k),cd),5)==0 || round(THC.Uv(tn(k),cd),5)==1  
                en = k ;
                sn = ak(k);
                break
            end
        end
        
        if en > 0
        
            if abs(THC.Uv(tn(sn),cd)-THC.Uv(tn(en),cd)) > 0.5

                if round(THC.Uv(tn(en),cd),5)==0
                    ue = 1 ;
                else
                    ue = 0 ;
                end
                U0(cd,1) = N(sn) * THC.Uv(tn(sn),cd) +  N(en) * ue;
            end
            
            break

        end
 
    end
end


% 3.3.- Solid:

A0(:,1) = N(1) * THC.Av(tn(1),:) +  N(2) * THC.Av(tn(2),:);


end

