function [eTHC] = ei1643_eTHCm ...
(contaL, Spatches, pR, rTskin, agCurv, csM, B0, U0, A0, Xtol, DUtol)


% RETURNS:
% eTHC: coordinates of node to insert in the tet and MLT, 
% after projection onto involved entities.

% INPUTS:
% csM(2,1): refs of MLT curve and MLT surface involved 
% A0(pudim,1): initial trial point for pp 


% 1.- PRELIM. OPERATIONS:

cudim = 1 ;
sudim = 2 ;
pudim = 3 ;
xdim  = 3 ;

pDegree = Spatches(pR).INS(1).mDegree;
pNcp    = Spatches(pR).INS(1).mNcp;
pKnotV  = Spatches(pR).INS(1).mKnotV;
pP      = Spatches(pR).INS(1).mP;
pw      = Spatches(pR).INS(1).mw;
pUsize  = Spatches(pR).INS(1).mUsize;
pULims  = [0 1 ; 0 1 ; 0 1 ];
pEP     = Spatches(pR).INS(1).mEP;
pEw     = Spatches(pR).INS(1).mEw;
pCD     = [0 0 0 ];

eTHC = struct...
('Xv',[], 'Av',[], 'Uv',[], 'Bv',[], 'NCS',[], 'ANv',[], 'At', [], 'rm', 0);

% Note: eTHC.At(pudim,1) are the coords of the point before propjection
% ontro MLT. eTHC.At is used to localize the triangle of MLT to update


% 2.- MISSING LINE BELONGS TO ONE SURFACE OF MLT:
    
sR = csM(2);

sDegree = rTskin(sR).mDegree;
sNcp    = rTskin(sR).mNcp;
sKnotV  = rTskin(sR).mKnotV;
sP      = rTskin(sR).mP;
sw      = rTskin(sR).mw;
sUsize  = rTskin(sR).mUsize;
sCD     = rTskin(sR).mCS;
[sEw,sEP] = GEN_12_ExpandBw (sw, sP, xdim, sudim, sNcp);
sULims  = [ 0 1 ; 0 1 ];


if csM(1) > 0 

	% 2.1.- Belongs to a contour curve 'cR'
   
	cR  = csM(1);

	cDegree = agCurv(cR).Degree;
	cNcp    = agCurv(cR).Ncp;
 	cKnotV  = agCurv(cR).KnotV;
  	cP      = agCurv(cR).P;
  	cw      = agCurv(cR).w;
    cUsize  = agCurv(cR).Usize;
   	cCD     = agCurv(cR).CD;
	cULims  = [ 0 1 ];

	Xt = b2601_BNURBS_Xcoord (A0, pKnotV, pNcp, pDegree, pUsize, pEw, pEP, pudim, xdim);
            
	% pp from Xt to curve, using B0 as starting point
	[B,~] = c2815b_onePoint (Xt, B0', cDegree, cNcp, cKnotV, cP ,cw, cULims, cCD, xdim, Xtol, DUtol);

	% physical coordinates:
 	X = b2601_BNURBS_Xcoord (B, cKnotV, cNcp, cDegree, cUsize, cw, cP, cudim, xdim);
            
 	% pp from X to surface:
  	[U, ~] = s2835b_onePoint (X, U0', sDegree, sNcp, sKnotV, sUsize, sP ,sw, sEP ,sEw, sCD, sULims, xdim, Xtol, DUtol);
 	U = U';
            
  	% pp from X to patch:
  	[A, ~] = v2865b_onePoint (X, A0', pDegree, pNcp, pKnotV, pUsize, pP ,pw, pEP ,pEw, pCD, pULims, xdim, Xtol, DUtol);
   	A = A';
            
    eNCS = [csM(1) csM(2)];

elseif csM(2) > 0 
        
	% 2.2.- Belongs to a trimming surface 'sR':

    B = 0 ; % remains = 0
    
	Xt = b2601_BNURBS_Xcoord (A0, pKnotV, pNcp, pDegree, pUsize, pEw, pEP, pudim, xdim);
            
	% pp from X to surface:
  	[U, ~] = s2835b_onePoint (Xt, U0', sDegree, sNcp, sKnotV, sUsize, sP ,sw, sEP ,sEw, sCD, sULims, xdim, Xtol, DUtol);
  	U = U';
               
	% physical coordinates:
 	X = b2601_BNURBS_Xcoord (U, sKnotV, sNcp, sDegree, sUsize, sEw, sEP, sudim, xdim);
            
	% pp from X to patch:
   	[A, ~] = v2865b_onePoint (X, A0', pDegree, pNcp, pKnotV, pUsize, pP ,pw, pEP ,pEw, pCD, pULims, xdim, Xtol, DUtol);
  	A = A';
            
	eNCS = [csM(1) csM(2)];

end
   

% 3.- ALLOCATES:

eTHC.Xv = X ;
eTHC.Av = A ;
eTHC.Uv = U ;
eTHC.Bv = B ;
eTHC.At = A0 ;
eTHC.NCS = eNCS ;


end

