function [f32F] = tf1680_check32 (LR, TR, aeR, connT, MLT, THC, nd)

% RETURNS:
% f32F = 1: flag to indicate that flip 3 to 2 is possible
% fnR(3,1): free nodes refs of Tet (nodes not at the line to remove)
% lnR(2,1): nodes of Tets that lie at ends of line to remove

% INPUTS:
% LR: line ref of the tets that intersects the MLT triangle
% TR: MLT triangle intersected


% 1.- CHECKS IF THERE ARE THREE TETS:

Nae = size(aeR,1);

if Nae ~= 3
    f32F = 0 ;
    return
end

% 2.- CHECKS IF THE 'FREE NODES' OF TETS LIE ON THE MLT TRIANGLES:

% 2.1.- Identifies 'free nodes':

conta = 0 ;
fnR = zeros(1,1);

Ln1 = connT.LV(LR,1);
Ln2 = connT.LV(LR,2);


for e = 1:1:Nae
    
    eR = aeR(e,1);
    
    for i = 1:1:4
        
        nR = connT.EV(eR,i);
        
        if nR ~= Ln1 && nR ~= Ln2
            conta = conta + 1 ;
            fnR(conta,1) = nR;
        end
    end
end


[fnR,~] = GEN_16_RemoveRepeatedRows (fnR, 1, 0.1);
Nfn = size(fnR,1);


if Nfn ~= 3
    f32F = 0 ;
    return
end

% 2.2.- Checks against the MLT triangle nodes:

pudim = 3 ;

Atet = zeros(pudim,1);
Atri = zeros(pudim,1);

cnF = zeros(Nfn,1); % coincident node flag

for i = 1:1:Nfn
    
    nR = fnR(i,1);
    
    Atet(:,1) = THC.Av(nR,:);
    
    for j = 1:1:3
        
        nR = MLT.tTVcon(TR,j);
        
        Atri(:,1) = MLT.tAv(nR,:);
        
        CO = go2542_coincidentP (Atet, Atri, pudim, nd);
        
        if CO == 1
            cnF(i,1) = 1 ;
            break
        end
        
    end
end


if sum(cnF) < 3
    f32F = 0 ;
    return
end

% 3.- ALLOCATES THE FLAG THAT ALLOWS TO 2 => 3 FLIP:

f32F = 1 ;


end

