function [connT, connS, THC, aeR, reR] = ts1611b_tetSLIVER (seR, ilLR, connT, THC, Spatches, pR)

% RETURNS:
% connT, connS: updated after dividing aff'd tets into three sub-tets
% aeR:          refs of affd elements
% reR:          element removed

% INPUTS:
% seR:      ref. of element that is a sliver
% ilLR(2,1): local ref of lines that intersect in the sliver 

pudim = 3 ;
xdim  = 3 ;

pDegree = Spatches(pR).INS(1).mDegree;
pNcp    = Spatches(pR).INS(1).mNcp;
pKnotV  = Spatches(pR).INS(1).mKnotV;
pP      = Spatches(pR).INS(1).mP;
pw      = Spatches(pR).INS(1).mw;
pUsize  = Spatches(pR).INS(1).mUsize;
pULims  = [0 1 ; 0 1 ; 0 1 ];
pEP     = Spatches(pR).INS(1).mEP;
pEw     = Spatches(pR).INS(1).mEw;
pCD     = [0 0 0 ];


% 1.- DETECTS ELEMENTS TO DIVIDE:
% (those ones attached to the intersecting lines)

lR = [ connT.EL(seR,ilLR(1)) ; connT.EL(seR,ilLR(2)) ] ; % refs. of intersecting lines

% ref. of elements to divide into 2:

edR = zeros(2,1);
contaER = zeros(2,1) ; 

for s = 1:1:2
    
    r = lR(s,1);
    
    for j = 1:1:size(connT.LE,2)
        
        er = connT.LE(r,j);
        
        if er > 0 && er~=seR
            contaER(s,1) = contaER(s,1) + 1 ;
            edR(s,contaER(s,1)) = er;
        end 
    end
 
end

aeR = [ edR(1,1:contaER(1))' ; edR(2,1:contaER(2))' ] ;






% 2.- COMPUTES INTERSECTION POINT:

tolP = 0.5 ;

n11 = connT.LV(lR(1),1); n12 = connT.LV(lR(1),2);
n21 = connT.LV(lR(2),1); n22 = connT.LV(lR(2),2);

A1 = zeros(pudim,1);
A2 = zeros(pudim,1);
B1 = zeros(pudim,1);
B2 = zeros(pudim,1);

A1(:,1) = THC.Av(n11,:);
A2(:,1) = THC.Av(n12,:);
B1(:,1) = THC.Av(n21,:);
B2(:,1) = THC.Av(n22,:);

A0 = (A1 + A2) / 2 ;

[IP] = go2549_cPr2r (A0, 0,0, 0,0, A1,A2, B1,B2, tolP);


% 3.- UPDATES THC:

X = b2601_BNURBS_Xcoord (IP , pKnotV, pNcp, pDegree, pUsize, pEw, pEP, pudim, xdim);
    
THC.Xv = [THC.Xv ; X'];
THC.Av = [THC.Av ; IP'];
THC.Uv = [THC.Uv ; zeros(1,2)];
THC.Bv = [THC.Bv ; 0];

THC.NCS = [THC.NCS ; zeros(1,2)];
THC.rm = [THC.rm ; 0];
THC.ANv = [THC.ANv ; zeros(1,3)];



% 4.- UPDATES EVcon:

EVcon = connT.EV ;


% 4.1.- Split the elements:

nnR = size(THC.Av,1);

for s = 1:1:2
    
    % works separately on both lines
    
    Ne = contaER(s,1);
    eR = zeros(Ne,1);
    LeRd = zeros(Ne,2);
    
    for k = 1:1:Ne
    
        eR(k,1) = edR(s,k);
        
        conta = 0 ;
        for j = 1:1:4
            
            nR = connT.EV(edR(s,k),j);
            
            if nR == connT.LV(lR(s),1) || nR == connT.LV(lR(s),2)

                conta = conta+1;
                LeRd(k,conta) = j ;
            end
        end
     
    end
    
    
    if Ne > 0

        pNE = size(EVcon,1);
        aeR = [aeR ; [pNE+1:1:pNE+Ne]'];

        [pNE, EVcon] = ei1663_TetDiv2 (THC, pNE, EVcon, Ne, eR, nnR, LeRd);
 
    end
  
end

        


% 4.2.- just deletes the element
    
[EVcon] = ts1621b_EVdelER (EVcon, seR);
reR = seR ;

for i = 1:1:size(aeR,1)
    if aeR(i,1) > reR
        aeR(i,1) = aeR(i,1) - 1 ;
    end
end




% 5.- UPDATES connT and connS:

NE = size(EVcon,1);
NV = nnR ;

connT = th1333_oConnL (NE, NV, EVcon); 
[connS,~,~] = th1381_connS (NE, connT);



end

