function [Nz, paz, nd] = pr1701_creaPath (exS)

% RETURNS:
% Np: No. paths to obtain data from
% paz(Np,1): paths structure
%           ps:         patch where the path belongs, if 0 no patch is specified
%           n:          No. points of the path
%           C(n,dim):   coordinates of the points (undeformed)
%           x(n,1):     accumulated length of the path
%           y(n,1):     values of parameter to measure
%           yn:         name of the parameter to measure
%                       's' stress, 'u' displacement, 'x' position in deformed shape
%           yc:         component of the parameter to measure (if applies)

% nd: No . digits to round when searching the point within the skins


% exS = 1 ; % selected example %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if exS == 1
    
    nd = 1 ;
    
  	Nz = 2 ;
    paz(Nz,1) = struct('ps',0, 'n', 0, 'C', [], 'x',[], 'y',[], 'yn', 0, 'yc', 0);

        
    % path 1: measures the y-stresses (component of stress = 2)
    
    r = 1 ;
    
    C = [ 
    0 0 650    ; 
    0 500 650  ; 
    0 1000 650 ; 
    0 1500 650 ; 
    0 2000 650 ; 
    0 2500 650 ; 
    0 3000 650 
    ] ;

    n = size(C,1) ;
    [x] = pr1704_accL (C);

    paz(r,1).ps = 0 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 's' ;
    paz(r,1).yc = 2 ;
    
    
	% path 2: measures the z-displacements (component = 3)
    
    r = 2 ;

    paz(r,1).ps = 0 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 'u' ;
    paz(r,1).yc = 3 ;
    
    
elseif exS == 2

    nd = 1 ;
    
  	Nz = 3 ;
    paz(Nz,1) = struct('pR',0, 'n', 0, 'C', [], 'x',[], 'y',[], 'yn', 0, 'yc', 0);

    
	% path 1: measures the z-stresses (component of stress = 3)
    
    r = 1 ;
    
    C = [ 
    32.3272	0	22;
    32.3272	14	22;
    32.3272	28	22;
    32.3272	42	22;
    32.3272	56	22;
    32.3272	70	22;
    32.3272	84	22;
    32.3272	98	22;
    32.3272	112	22;
    32.3272	126	22;
    ] ;

    n = size(C,1) ;
    [x] = pr1704_accL (C);

    paz(r,1).ps = 0 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 's' ;
    paz(r,1).yc = 3 ;
     
        
	% path 2: measures the z-displacement (component = 3)
    
    r = 2 ;

    paz(r,1).ps = 0 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 'u' ;
    paz(r,1).yc = 3 ;
    
    
	% path 3: measures the VM-stresses (component of stress = 7)
    
    r = 3 ;
    
    C = [ 
    32.3272	0	0;
    32.3272	16	0;
    32.3272	32	0;
    32.3272	48	0;
    32.3272	64	0;
    32.3272	80	0;
    32.3272	96	0;
    32.3272	112	0;
    32.3272	128	0;
    32.3272	144	0;
    ] ;

    n = size(C,1) ;
    [x] = pr1704_accL (C);

    paz(r,1).ps = 0 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 's' ;
    paz(r,1).yc = 7 ;
    
elseif exS == 3
    
    nd = 1 ;
    
    Nz = 9 ;
    paz(Nz,1) = struct('pR',0, 'n', 0, 'C', [], 'x',[], 'y',[], 'yn', 0, 'yc', 0);

    
    % path 1: measures the Mon Mises stresses (component of stress = 7)
    
    r = 1 ;
    
    C = [ 
    30.5	37.5	35;
    36.1351	37.5	35;
    36.1351	32.5	35;
    36.1351	27.5	35;
    36.1351	22.5	35;
    36.1351	17.5	35;
    36.1351	12.5	35;
    30.5	12.5	35;
    15.5	12.5	35;
    10.1398	12.5	35;
    10.1398	17.5	35;
    10.1398	22.5	35;
    10.1398	27.5	35;
    10.1398	32.5	35;
    10.1398	37.5	35;
    15.5	37.5	35
        ] ;

    n = size(C,1) ;
    [x] = pr1704_accL (C);

    paz(r,1).ps = 2 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 's' ;
    paz(r,1).yc = 7 ;
    
	
    % path 2: measures the displacement x
    
    r = 2 ;
    
    C = [ 
    37.4141	37.5	15;
    37.4571	37.5	21.9999;
    37.072	37.5	28.9893;
    35.9361	37.5	35.8965;
    33.9629	37.5	42.6126;
    31.8701	37.5	49.2925;
    32.3933	37.5	57.2676;
    41.485	37.5	62.3704;
    48.6703	37.5	69.9248
        ] ;

    n = size(C,1) ;
    [x] = pr1704_accL (C);

    paz(r,1).ps = 0 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 'u' ;
    paz(r,1).yc = 1 ;
    
	
    % path 3: measures the displacement y
    
    r = 3 ;
    
	paz(r,1).ps = 0 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 'u' ;
    paz(r,1).yc = 2 ;


	% path 4: measures the displacement z
    
    r = 4 ;
    
	paz(r,1).ps = 0 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 'u' ;
    paz(r,1).yc = 3 ;
    
    
	% path 5: measures deformed position on contour base of patch 1
    
    r = 5 ;
    
    C = [ 
    50.000	50.000	0.000;
    50.000	37.500	0.000;
    50.000	25.000	0.000;
    42.678	7.322	0.000;
    25.000	0.000	0.000;
    12.500	0.000	0.000;
    0.000	0.000	0.000;
    0.000	12.500	0.000;
    0.000	25.000	0.000;
    7.322	42.678	0.000;
    25.000	50.000	0.000;
    37.500	50.000	0.000
        ] ;

    n = size(C,1) ;
    [x] = pr1704_accL (C);

    paz(r,1).ps = 1 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 'x' ;
    paz(r,1).yc = 0 ;
    
    
	% path 6: measures deformed position on contour of couping surface 2 on patch 1
    
    r = 6 ;
    
    C = [ 
    37.500	37.500	15.000;
    37.500	25.000	15.000;
    37.500	12.500	15.000;
    25.000	12.500	15.000;
    12.500	12.500	15.000;
    12.500	25.000	15.000;
    12.500	37.500	15.000;
    25.000	37.500	15.000
        ] ;

    n = size(C,1) ;
    [x] = pr1704_accL (C);

    paz(r,1).ps = 1 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 'x' ;
    paz(r,1).yc = 0 ;
    
	% path 7: measures deformed position on contour of couping surface 2 on patch 2
    
    r = 7 ;
    
    paz(r,1).ps = 2 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 'x' ;
    paz(r,1).yc = 0 ;
    
    
	% path 8: measures deformed position on contour of couping surface 2 on patch 1
    
    r = 8 ;
    
    C = [ 
    32.393	25.000	57.268;
    32.393	12.500	57.268;
    19.600	12.500	55.002;
    6.862	12.500	57.560;
    6.862	25.000	57.560;
    6.862	37.500	57.560;
    19.600	37.500	55.002;
    32.393	37.500	57.268
        ] ;

    n = size(C,1) ;
    [x] = pr1704_accL (C);

    paz(r,1).ps = 2 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 'x' ;
    paz(r,1).yc = 0 ;
    
	% path 9: measures deformed position on contour of couping surface 2 on patch 2
    
    r = 9 ;
    
    paz(r,1).ps = 3 ;
    paz(r,1).n  = n ;
    paz(r,1).C  = C ;
    paz(r,1).x  = x ;
    paz(r,1).y  = zeros(n,1);
  	paz(r,1).yn = 'x' ;
    paz(r,1).yc = 0 ;
    
    
    
end

end

