function [ ] = e2021_Ex1(h)

% INPTUTS:
% h : step in mm

% 1.- INPUTS: all in N and mm

% 1.0.- Step:

% 1.1.- Geometry and material:

bf = 500 ;
H  = 650 ;
bw = 75 ;
h0 = 0 ;
h1 = 440 ;
tf0 = 550 ;
tf1 = 70 ;
L = 3000 ;

E = 210000; 

% 1.2.- Load: introduce traction at tip (in z axis, downwards)

p = -3.3 ;                       % N/mm2
A1 = bf*tf1 + bw*(H-h1-tf1);    % mm2
F = A1*p ;                      % N


% 1.3.- Intersection curve features:

cNcp = 13;
cDegree = 3;
cUsize = 17;

cP = [0.0375    3.0000    0.5800
    0.0375    2.7498    0.5850
    0.0375    2.4996    0.5899
    0.0375    2.2500    0.5781
    0.0375    1.9990    0.5662
    0.0375    1.7486    0.5374
    0.0375    1.5000    0.4975
    0.0375    1.2473    0.4570
    0.0375    0.9965    0.4051
    0.0375    0.7500    0.3381
    0.0375    0.4970    0.2693
    0.0375    0.2485    0.1847
    0.0375         0    0.1000] *1000 ;

[cP] = GEN_40_invertRowsOrder (cP);

cw = ones(13,1);

cKnotV = [0 0 0 0 0.2573 0.2573 0.2573 0.5081 0.5081 0.5081 0.7547 0.7547 0.7547 1 1 1 1];

% sample points for equivalent polynomial:

sp = zeros(5,3);
conta = 0 ;
for u = 0 : 0.25 : 1 
    conta = conta + 1;
    [X] = b2601_BNURBS_Xcoord (u, cKnotV, cNcp,cDegree,cUsize, cw,cP, 1,3);
    sp(conta,:) = X(:,1);
end



% curve representation: NURBS and polynomial, and error
%{

ns = 21 ; % no. sample points

conta = 0 ;
for u = 0:1/(ns-1):1
    
    conta = conta+1;
    
    [X] = b2601_BNURBS_Xcoord (u, cKnotV, cNcp,cDegree,cUsize, cw,cP, 1,3);
    
    xn(conta,:) = X(:,1);

end

conta = 0 ;
for x = 0:3000/(ns-1):3000
    
    conta = conta + 1 ;
    
    xp(conta,1) = 37.5;
    xp(conta,2) = x;
    xp(conta,3) = -7*10^(-5)*x^2 + 0.37*x + 99.939;

end

% error:

avz = 400;

err = zeros(ns,1);

for i = 1:1:ns
err(i,1) = abs(xp(i,3) - xn(i,3));
end

Re = err/avz *100 ; % relative error at each point

Reav = sum(Re)/ns ;



figure(1)

for i = 1:1:conta-1
    
    plot3(xn(:,1)  , xn(:,2)  , xn(:,3) , 'r'); hold on
    plot3(xp(:,1)  , xp(:,2)  , xp(:,3) , 'b'); hold on
end

axis equal
%}


% 2.- INERTIAS AND MOMENTS AT SAMPLE POINTS:

n = L/h ; % % No. steps

x  = zeros(n+1,1);

y1  = zeros(n+1,1);
y2  = zeros(n+1,1);
y3  = zeros(n+1,1);
yG = zeros(n+1,1);

IG = zeros(n+1,1);
M  = zeros(n+1,1);

xacc = -h ; % acc. position

for i = 1:1:n+1
    
    xacc = xacc+h;
    
    x(i,1) = xacc;
    
    % profile heights:
    
	y1(i,1) = H ;
    y2(i,1) = -7*10^(-5)*xacc^2 + 0.37*xacc + 99.939 ;
    y3(i,1) = (h1-h0)/L * xacc ;
    
    % key dims:
    
    tf = y1(i) - y2(i) ;
    tw = y2(i) - y3(i) ;
    
    Af = bf*tf ;
    Aw = bw*tw ;
    

    % yG:

    yf = y2(i) + tf/2;
    yw = y3(i) + tw/2;
    yG(i,1) = 1/(Af+Aw)*(Af*yf+Aw*yw);
    
    % Inertias:
    
    If = bf*tf^3/12;
    df = abs(yG(i)-yf);
    
    Iw = bw*tw^3/12;
    dw = abs(yG(i)-yw);
    
    IG(i,1) = If + Af*df^2 + Iw + Aw*dw^2;
    
    % moment:
    
    M(i,1) = F*(L-xacc) ;

    
end


%{
figure(1)

plot(x,y1,'k'); hold on
plot(x,y2,'k'); hold on
plot(x,y3,'k'); hold on

plot(x,yG,'r'); hold on

plot(x,y3,'k'); hold on
plot(x,M/1000000,'b'); hold on

axis equal
%}

% 3.- INTEGRALS:

% 3.1.- Slope:

g = zeros(n,1);

for i = 1:1:n
    
    f0 = M(i)/(IG(i)*E);
    f1 = M(i+1)/(IG(i+1)*E);
    
    gi = h*(f0+f1)/2;
    
    if i==1
        g(i,1) = gi ;
    else
        g(i,1) = g(i-1,1) + gi ;
    end

end

% 3.2.- Deflection:

y = zeros(n-1,1);
xr = [0:h:L-2*h]';
for i = 1:1:n-1
    
    g0 = g(i);
    g1 = g(i+1);
    
    yi = h*(g0+g1)/2;
    
    if i==1
        y(i,1) = yi ;
    else
        y(i,1) = y(i-1,1) + yi ;
    end

end


%{
figure(1)

plot(xr,y,'r'); hold on
plot(xr,zeros(n-1,1),'--k'); hold on
ylim([-5 1]);

%}

end

