function [ImE, addE, ecUc, seR] = im2107_subEL ...
         (ImE, addE, ecUc, nEt, Nsue, peR, ULims, NBsk, BskR, gPatch, pa, pKnotV, pNcp, pDegree, pUsize, pEw, pEB, Bskin, bCurv, pudim, pxdim)

% RETURNS:
% ImE: integration mesh elements with division of one element into subelements
% addE: updated No. elements to add to Imesh. Updates due to that partition
% ecUc(nE,Ncor,udim): elements corners Ucoord's, to be used to compute GP's coordinates (out of this routine)
% seR(Nsue,1): new sub-elems refs, note that the first one is the same as the parent element

% INPUTS:
% nEt:  No. elems previous to this partition
% Nsue: No. subelements due to partition
% peR: parent element ref.
% ULims(pudim,2): ULims of parent element 'peR'
% NBsk, BskR: bouned skins related to parent element

% 1.- PRELIMINARIES:

% sub-elems U limits:
[seULims] = im2101_UdivLims (ULims, pudim);
        

% all nodes (resultant from partition) empty matrices. It will fill
% them as needed (if necessary, if not leaves as 0). Saves time as one node
% status is computed only once

allIN   = zeros(27,NBsk);
allTRIM = zeros(27,NBsk);
allSV   = zeros(27,NBsk);
allU    = zeros(27,NBsk, pudim);
allX    = zeros(27,NBsk, pxdim);
allF    = zeros(27,NBsk); % flag indicating the node is computed (1) or not (0)


% conn. matrix of sub-elems and nodes:

SeNo = [ 1 2 4 5 10     11 13 14 ;      2 3 5 6 11   12 14 15 ;
         4 5 7 8 13     14 16 17 ;      5 6 8 9      14 15 17 18 ;
         10 11 13 14    19 20 22 23 ;   11 12 14 15  20 21 23 24 ;   
         13 14 16 17    22 23 25 26 ;   14 15 17 18  23 24 26 27];
    
     
% max. distance from Ctr. and corners  (in param. space) 
% to consider a skin not-trimming 100 % sure

dmax = 0 ;
for di=1:1:pudim
    dmax = dmax + (ULims(di,2) - ULims(di,1))^2 ;
end
CtrUdmax = 1/4 * dmax^0.5 ;
CorUdmax = 1/2 * dmax^0.5 ;


% to store new sub-elements ref's:

seR = zeros(Nsue,1); 




% 2.- LOOP ON SUB-ELEMS:

for sue = 1:1:Nsue
     
    
    % 2.1.- SUB-ELEMENT ULims and ctr. (param. & phys.):

    iULims  = zeros(pudim,2);
    icUcoor = zeros(pudim,1);
            
    for di=1:1:pudim       
        iULims(di,1) = seULims(sue,di,1);
      	iULims(di,2) = seULims(sue,di,2);
     	icUcoor(di) = (iULims(di,1)+iULims(di,2))/2;   
    end

 	icXcoor = b2601_BNURBS_Xcoord (icUcoor, pKnotV, pNcp, pDegree, pUsize, pEw, pEB, pudim, pxdim);
        
    
    % 2.2.- ELEMENS LIST REF'S DUE TO PARTITION:
    
    % sub-elem. in particular 'eR', but stores in seR() matrix the
    % references for he Nsue sub-elems
          	
    if sue==1
        eR = peR ; % substitutes original
    else
    	addE = addE + 1 ; % add element at the rear
      	eR = nEt + addE ; % HERE UPDATES elements to ADD !!
    end
   
	seR(sue) = eR ;
            

    
    % 2.3.- EVALUATES RELATIONSHIP WITH POTENTIAL TRIMMING SKINS
    
    % pre-selects trimming skins from parent element
    
	% stores info in skf(iNBsk,4): BskR, IN, Tr, dCtrS
  	skf = zeros(NBsk,4);
            
    for s = 1:1:NBsk

        skR = BskR(s);
             
        % closest Bskin point of mP collection:
        
        
     	% 2.3.1.- CHECK FROM CTR. POINT OF ELEMENT:

     	[ctIN, ctTRIM, ctSV, iCtrS] = GEN_62_elePinTrim ...
     	(gPatch, pa, skR, Bskin, iULims, icUcoor, icXcoor, CtrUdmax, bCurv);

       	if ctSV == 1
            iIN = ctIN;
        	iTr = ctTRIM;
        end
                
        % 2.3.2.- CHECK CORNERS (in case of not definitive answer from ctr.)

        if ctSV == 0
            [iIN, iTr, allU, allX, allIN, allTRIM, allSV, allF] = GEN_66_cornersINpre ...
          	(gPatch, pa, skR, s, Bskin, CorUdmax, sue, SeNo, iULims, bCurv, pudim, allU, allX, allIN, allTRIM, allSV, allF);
        end
    
      	skf(s,1) = skR;
       	skf(s,2) = iIN;    
       	skf(s,3) = iTr;    
      	skf(s,4) = iCtrS;
    end
            

    % 2.4.- ALLOCATION TO NEW SUB-ELEMENT 'eR':
    
    for di = 1:1:pudim
    	ImE(eR).mPcoLi(di,1) = iULims(di,1);
     	ImE(eR).mPcoLi(di,2) = iULims(di,2);
    end
    
 	ImE(eR).mRicpA = ImE(peR).mRicpA; % inherits icp's from parent element
   	ImE(eR).mNicp  = ImE(peR).mNicp; 
            
	skf = sortrows(skf,4);

  	iIN = 0;
  	iTr = 0 ;
   	iNBsk = 0 ;
   	iBskR = zeros(1,1);
            
   	for s = 1:1:NBsk
   	if skf(s,3)==1
     	iNBsk = iNBsk+1;
     	iBskR(iNBsk,1) = skf(s,1);
      	iTr = 1 ;
       	iIN = 1 ;
    end
    end
            
    if iIN==0 % if no trimming, considers IN state as per closest trimming skin
    	iIN = skf(1,2);
    end
            
  	ImE(eR).mTrim   = iTr;
   	ImE(eR).ON      = iIN;  
            
   	ImE(eR).mNBsk = iNBsk; 
   	ImE(eR).mBskR = iBskR; 

   	ImE(eR).dJ2 = j2313_Jac2_elem (2^pudim, pudim, 0, iULims);
            
    
    
    % 2.5.- UPDATES ELEMENTS CORNERS Ucoords:
    
   	cor = ImE(eR).cornersPcoor(pudim);
    
   	for c=1:1:2^pudim
        ecUc(eR,c,:) = cor(c,:);
    end

    
end
           



end

