function [GPpc, GPw, triCoor] = ip2303_GP_tri (NGP)

% RETURNS: (for triangular 2D element)
% GPpc: GP parent coordinates
% GPw:  GP weights
% triCoor:   triangular coor'ds (acc. to Benito's book)

GPpc = zeros(NGP,2);
GPw = zeros(NGP,1);
triCoor = zeros(NGP,3);

if NGP == 1     % order 1
    
    GPpc(1,1) = 1/3;    GPpc(1,2) = 1/3;
    
    GPw(1) = 0.5 ;

    triCoor(1,1) = 1/3 ; triCoor(1,2) = 1/3 ; triCoor(1,3) = 1/3 ;
    
elseif NGP == 3 % order 2
    
    GPpc(1,1) = 1/2;    GPpc(1,2) = 1/2;
    GPpc(2,1) = 0;      GPpc(2,2) = 1/2;
    GPpc(3,1) = 1/2;    GPpc(3,2) = 0  ;
     
    GPw(1) = 1/6 ;
    GPw(2) = 1/6 ;
    GPw(3) = 1/6 ;

    triCoor(1,1) = 0 ; triCoor(1,2) = 1 ; triCoor(1,3) = 1 ;
    triCoor(2,1) = 1 ; triCoor(2,2) = 0 ; triCoor(2,3) = 1 ;
    triCoor(3,1) = 1 ; triCoor(3,2) = 1 ; triCoor(3,3) = 0 ;
    
elseif NGP == 4 % order 3
    
    GPpc(1,1) = 1/3;    GPpc(1,2) = 1/3;
    GPpc(2,1) = 0.6;    GPpc(2,2) = 0.2;
    GPpc(3,1) = 0.2;    GPpc(3,2) = 0.6;
    GPpc(4,1) = 0.2;    GPpc(4,2) = 0.2;
    
    GPw(1) = -27/96;
    GPw(2) = 25/96;
    GPw(3) = 25/96;
    GPw(4) = 25/96;
    
    triCoor(1,1) = 1 ; triCoor(1,2) = 1 ; triCoor(1,3) = 1 ;
    triCoor(2,1) = 1.8 ; triCoor(2,2) = 0.6 ; triCoor(2,3) = 0.6 ;
    triCoor(3,1) = 0.6 ; triCoor(3,2) = 1.8 ; triCoor(3,3) = 0.6 ;
    triCoor(4,1) = 0.6 ; triCoor(4,2) = 0.6 ; triCoor(4,3) = 1.8 ;
    
elseif NGP == 7 % order 7
    
    GPpc(1,1) = 0.0;    GPpc(1,2) = 0.0;
    GPpc(2,1) = 0.5;    GPpc(2,2) = 0.0;
    GPpc(3,1) = 1.0;    GPpc(3,2) = 0.0;
    GPpc(4,1) = 0.5;    GPpc(4,2) = 0.5;
    GPpc(5,1) = 0.0;    GPpc(5,2) = 1.0;
    GPpc(6,1) = 0.0;    GPpc(6,2) = 0.5;
    GPpc(7,1) = 1/3;    GPpc(7,2) = 1/3;
    
    GPw(1) = 1/40;
    GPw(2) = 1/15;
    GPw(3) = 1/40;
    GPw(4) = 1/15;
    GPw(5) = 1/40;
    GPw(6) = 1/15;
    GPw(7) = 9/40;
    
    triCoor(1,1) = 6 ; triCoor(1,2) = 0 ; triCoor(1,3) = 0 ;
    triCoor(2,1) = 3 ; triCoor(2,2) = 3 ; triCoor(2,3) = 0 ;
    triCoor(3,1) = 0 ; triCoor(3,2) = 6 ; triCoor(3,3) = 0 ;
    triCoor(4,1) = 0 ; triCoor(4,2) = 3 ; triCoor(4,3) = 3 ;
    triCoor(5,1) = 0 ; triCoor(5,2) = 0 ; triCoor(5,3) = 6 ;
    triCoor(6,1) = 3 ; triCoor(6,2) = 0 ; triCoor(6,3) = 3 ;
    triCoor(7,1) = 2 ; triCoor(7,2) = 2 ; triCoor(7,3) = 2 ;
    
end


end

