function [detJ2] = j2313_Jac2_elem (Nne, udim, uN, uL)


% RETURNS:
% dJ2: determinant of Jacobian 2 for elements as detailed below:
% linear line
% linear rectangle
% linear brick
% linear triangle
% linear tetrahedral

% dJ2 is for mapping from parent to parameter space.

% INPUTS:
% Nne:          No. nodes per element
% udim:         No. dims in param. space
% uN(Nne,udim): param. coord's of element nodes (triangles & tetrahedrals)
% uL(udim,1/2): param. space limits of the element (rect's and bricks)

% for uN and uL: 0 if it does not apply.


% 1.- LINES:                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Note !! : is only for J2 in IGA context => assumes from line to line,
% that is, in param. space the line is not distorted

if Nne==2
    
    di = 1 ;
    detJ2 = (uL(di,2) - uL(di,1)) * 1/2 ;
    
    
% 2.- TRIANGLES or TETRAHEDRALS:    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

elseif Nne == (udim + 1)

    %tretra=1
    
    % 2.1.- Plane triangles:
    if Nne == 3

    detJ2 = (uN(1,1)-uN(3,1))*(uN(2,2)-uN(3,2)) - (uN(2,1)-uN(3,1))*(uN(1,2)-uN(3,2)) ;
        

    % 2.2.- Tetrahedrals:
    elseif Nne == 4

        %tet=1
        
        j2 = ones(4,4);

        for fil=2:1:4
        for col=1:1:4
            j2(fil,col) = uN(col,fil-1);
        end
        end

        detJ2 = det(j2);

    end

% 3.- RECTANGLES or BRICKS:         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Note !! : is only for J2 in IGA context => assumes from rect. to rect.,
% that is, in param. space the rectangle is not distorted

elseif Nne == 2^udim
    
    acc = 1 ;
    for di=1:1:udim
    acc = acc * (uL(di,2) - uL(di,1)) * 1/2 ;
    end
    
    detJ2 = acc ;
    
end




end

