function [J1, detJ1] = j2315_J1_Bspline (u, KnotV, Ncp,Degree,Usize, P, w, udim, xdim)

% THIS FUNCTION CALCULATES THE JACOBIAN 1 MATRIX AT ONE LOCATION: for B-splines

%           | X,u   Y,u ...|
%   J1 =    |              |
%           | X,v   Y,v    |
%           | ..


% ensures w is a column matriz for curves

if udim==1 && size(w,2) > 1
    w = w';
end


% 1.- COMPUTES DERIVATIVES:

ord = ones(udim,1);

[dS] = b2701_BNURBS_deriv (Degree, Ncp, KnotV, Usize, P ,w, ord, u, xdim, udim);


% 2.- JACOBIAN:

J1 = zeros(udim, xdim);

for xd = 1:1:xdim
    
    for ud = 1:1:udim
        
        if ud==1
            J1(ud,xd) = dS(2,1,1,xd);
        elseif ud==2
            J1(ud,xd) = dS(1,2,1,xd);
        elseif ud==3
            J1(ud,xd) = dS(1,1,2,xd);
        end
 
    end
end



% 3.- DETERMINANT OF JACOBIAN:

dSu = zeros(xdim,1);
dSv = zeros(xdim,1);
dSw = zeros(xdim,1);

detJ1 = 0 ;

if udim==1
    
    detJ1 = norm(J1);
    
elseif udim==2
    
    if xdim==2
        
        detJ1 = det(J1);
        
    elseif xdim==3
        
        dSu(:,1) = J1(1,:);
        dSv(:,1) = J1(2,:);
        
        detJ1 = (cross(dSu,dSv)) ;
        
        detJ1 = norm(detJ1);
        
    end

elseif udim==3
    
    detJ1 = det(J1);
    
end



end

