function [J1, detJ1] = j2316_J1_BTsp_dRpa (CP, Ricp, Nicp, dRpa, udim, xdim)

% THIS FUNCTION CALCULATES THE JACOBIAN 1 MATRIX AT ONE LOCATION: for B-splines and T-splines

%           | X,u   Y,u ...|
%   J1 =    |              |
%           | X,v   Y,v    |
%           | ..


% STORES THE VALUES IN THE MATRIX J1(udim,xdim)  
% gp location is implicit in dRpa

% INPUTS:
% mA:       anchors object, for physical coord's
% dRi_pa:   derivatives of physical coord's with respect param.

% coord's where Jac is computed are given by dRi_pa implicitly



% 1.- Sizes and initiates relevant matrices:

J1 = zeros(udim, xdim);
x  = zeros(xdim,1);


% 2.- Calculates derivatives and therefore the Jacobian:

for a = 1:1:Nicp
    
    aRef = Ricp(a);
    
    for di=1:1:xdim
        x(di) = CP(aRef).mXcoor(di);
    end
    
    for fil = 1:1:udim
    for col = 1:1:xdim

        J1(fil,col) = J1(fil,col) + dRpa(a,fil) * x(col);
        
    end
    end
     
    
end


% 3.- Determinant:


detJ1 = 0 ;

if udim==1
    
    detJ1 = norm(J1);
    
elseif udim==2
    
    if xdim==2
        
        detJ1 = det(J1);
        
    elseif xdim==3
        
        dSu(:,1) = J1(1,:);
        dSv(:,1) = J1(2,:);
        
        detJ1 = (cross(dSu,dSv)) ;
        
        detJ1 = norm(detJ1);
        
    end

elseif udim==3
    
    detJ1 = det(J1);
    
end



end

