function [u] = m2321_parentMapping (uG, unp, udim, Nne)

% RETURNS: parameter space coordinates from parent coordinate, for 1D, 2D or 3D element

% INPUTS:
% uG:       coordinate in parent space uG(udim,1)
% unp:      coord's in param. space of element nodes unp(Nne,udim)
% udim:     No. dims in parameter space (to match No. dims in parent)
% Nne:      No. nodes per element

% Shape functions (triang, tet, linear, ...) are given indirectly by udim and Nne:

% (note that works for any shape, e.g. rectangles do not have to be orthogonal)

% 0.- Initiates variables:

u = zeros(udim,1);
N = zeros(Nne,1);


% 1.- Shape functions for line, triangle and tetrahedrals:
if (Nne-1) == (udim)
    
    % 1.1.- Lines:
    if udim == 1 
        
        N(1) = (1-uG) * 1/2;
        N(2) = (1+uG) * 1/2;
   
    % 1.2.- 2D Triangles:    
    elseif udim == 2
        
        N(1) = 1-uG(1)-uG(2);
        N(2) = uG(1);
        N(3) = uG(2);
  
    % 1.3.- 3D Tetrahedrals:    
    elseif udim == 3
             
        N(1) = 1-uG(1)-uG(2)-uG(3);
        N(2) = uG(1);
        N(3) = uG(2);
        N(4) = uG(3);
    end
    
    
% 2.- Planar rectangles: bf's arranged by directions jerarqua (1,1) (1,2) (2,1) (2,2) 
elseif Nne == 4 && udim==2
    
    N(1) = 1/4 * (1 - uG(1)) * (1 - uG(2));
    N(2) = 1/4 * (1 - uG(1)) * (1 + uG(2));
    N(3) = 1/4 * (1 + uG(1)) * (1 - uG(2));
    N(4) = 1/4 * (1 + uG(1)) * (1 + uG(2));

% 3.- 3D bricks:
elseif Nne == 8 && udim==3
    
   	N(1) = 1/8 * (1 - uG(1)) * (1 - uG(2))  * (1 - uG(3)) ;
    N(2) = 1/8 * (1 - uG(1)) * (1 - uG(2))  * (1 + uG(3)) ;
    
    N(3) = 1/8 * (1 - uG(1)) * (1 + uG(2))  * (1 - uG(3)) ;
    N(4) = 1/8 * (1 - uG(1)) * (1 + uG(2))  * (1 + uG(3)) ;
    
  	N(5) = 1/8 * (1 + uG(1)) * (1 - uG(2))  * (1 - uG(3)) ;
    N(6) = 1/8 * (1 + uG(1)) * (1 - uG(2))  * (1 + uG(3)) ;
    
    N(7) = 1/8 * (1 + uG(1)) * (1 + uG(2))  * (1 - uG(3)) ;
    N(8) = 1/8 * (1 + uG(1)) * (1 + uG(2))  * (1 + uG(3)) ;
 
end



% 4.- Computes response values:

for di=1:1:udim
for i=1:1:Nne
    u(di) = u(di) + N(i)*unp(i,di);
end
end




end

