function D = e2401_3Dmatrix (yModel, MTL, Vsc, Vve, aH, P, q, Z)

% RETURNS: constitutive matrix for 3 dimensions and elatic or plastic

% INPUTS:
% aH:       adimensionalized hardening modulus
% Vsc,Vve:  params, if Vsc==0 (Dp) , initiates with elastic D matrix


% 0.- Prelim. size, vbles, G and K calculation:

D = zeros(6,6);
Sv = zeros(6,1);

if Vsc(1,1) == 0 
    Dp = 0 ; d = 0 ;
else
    if strcmp(yModel , 'Ho') == true  
        
        Dp = Vsc(1); % increment of plastic strain
        d  = Vsc(3); 
        ty = Vsc(4); % adimensionlized yield limit
        Sv = Vve(2,:);
    end
end

Sv = Sv' ;

% elastic constants:
E1 = MTL.E(1) ;     E2 = MTL.E(2) ; 	E3 = MTL.E(3) ;
G12 = MTL.G(1) ;    G23 = MTL.G(2) ;    G31 = MTL.G(3) ;
nu12 =  MTL.nu(1);  nu23 = MTL.nu(2);   nu31 = MTL.nu(3) ;


% 1.- Elastic regime:

del = ( 1 - nu12^2 - nu23^2  - nu31^2 - 2*nu12*nu23*nu31 ) / (E1*E2*E3) ;

D(1,1) = ( 1 - nu23^2) / (E2*E3*del) ;
D(2,2) = ( 1 - nu31^2) / (E3*E1*del) ;
D(3,3) = ( 1 - nu12^2) / (E1*E2*del) ;

D(1,2) = (nu12 + nu23*nu31) / (E2*E3*del) ;
D(2,1) = D(1,2) ;

D(1,3) = (nu31 + nu12*nu23) / (E2*E3*del) ;
D(3,1) = D(1,3) ;

D(2,3) = (nu23 + nu12*nu31) / (E1*E3*del) ;
D(3,2) = D(2,3) ;

D(4,4) = 1*G12 ;
D(5,5) = 1*G23 ;
D(6,6) = 1*G31 ;
  


% 2.- Plastic regime (modifies D):

if Dp > 0
    
    N = P*Sv + q ;
    ita = (2/3 * N'*Z*N)^0.5;
    D = inv(inv(D) - Dp*P + 1 / (2*ty*aH*ita) * N*N' - 2/3 * Dp/ita^2 * N'*N*Z*P) ;
end


% 3.- Applies damage ( in abeyance ! ! ! ):
D = D*(1-d);



end

