function [Pg] = go2502_RotatePaboutR (P, Pr, vr, tita)

% RETURNS:
% Pg(xdim,1): P rotated tita radians about line Pr,vr

% works in 3D and rotates around horizontal line

% INPUTS:
% P(xdim,1): original point coordinates
% Pr,vr: point and vector por the line
% tita: angle in radians (use RHS rule with vr)


% 0.- PRELIMINARIES:

xdim = 3;

% transforms vr in versor:
vr = vr/norm(vr);

if size(vr,1)~=3
   vr = vr'; 
end

% distance and vector from P to r:
[dist, d] = go2527_distPtoRecta (P, Pr, vr, xdim);


% 1.- POINT 'P' IS ON THE ROTATION AXIS:

if dist < 0.000001
    
    Pg = P; 
    
    
% 2.- REST OF THE CASES: 
 
else
    
    % Lets define u as vector from Q to P and v as rotated vector u (tita angle).
    % Both vectors have the same norm: ||u|| = ||v||.
    % We know u and want to compute v. We use two conditions:
    
    % 1) dot product:   uv = u1 v1 + u2 v2 + u3 v3 = norm(u)^2 cos(tita)
    
    % 2) cross product: u x v = norm(u)^2 cos(tita) 
    % note that u x v = Area of paral. defined by u and v = A
    
    % We assembly the linera system C v = b
    % first condition gives one equation, i.e.: C(1,1)=u1  ...  (C1,3)=u3
    % second condition gives three eqs, we use only two.
    
    % The vector b is { norm(u)^2 cos(tita) , A vr(i) ,  A vr(j) }, with i
    % and j the two directions selected from the cross product
    
    % The solution v is the rotated vector searched.
    
    
    % changes angle sign in case is below line
    if P(3) < Pr(3)
        tita = -tita;
    end
    
    % closest point on vr Q and vector u:
    Q  = P+d;
    u = -d ;
    

    % Area of the trapez. formed by QP and QP rotated => norm of vectorial product:
    A = (norm(u))^2 * sin(tita);

    % preliminary size of the system:
    C = zeros(3,3);
    b = zeros(3,1);
    
    
    % rows from the cross product condition: selects those that are not all zeros
    
	rc = [   0     -u(3)  u(2);
             u(3)   0    -u(1);
            -u(2)   u(1)  0    ] ;

    for di = 1:1:xdim
        
        if u(di)==0
            
            conta = 0 ;
            for i = 1:1:xdim
                
                if i~=di
                
                    conta = conta + 1 ;

                    for j = 1:1:3
                        C(conta,j) = rc(i,j);
                    end

                    b(conta,1)= A*vr(i);
                    
                end
            end
            
            break
        end
        
    end
   
    % add the last row from the dot product condition:
    
    for j = 1:1:3
        C(3,j) = u(j);
    end
    b(3,1) =  (norm(u))^2 * cos(tita) ;
    

    
    % computes rotated vector:
   
  
    
    if abs(det(C))<0.00001 % for 2D problem C is singular
        
        % if it is a 2D problem:
            
        C2 = zeros(2,2);
        b2 = zeros(2,1);
        
        % detects direction fixed 'df' and directions to use 'du'
        
        conta = 0 ;
        du = zeros(2,1);
        
        for di = 1:1:xdim

            if round(vr(di),0) == 1 
                
                df = di ;
                
            else
                
                conta = conta + 1 ;
                du(conta,1) = di ;
                
            end

        end
        
        % selects row to use from rc and components from dot product:
        
        for i = 1:1:3
            
            if round(rc(i,du(1)),6) ~= 0 || round(rc(i,du(2)),6) ~= 0
                
                C2(1,1) = rc(i,du(1));
                C2(1,2) = rc(i,du(2));
                
                b2(1,1) = A*vr(i);
                
                break
            end
            
        end
        
        C2(2,1) = u(du(1));
        C2(2,2) = u(du(2));
        
        b2(2,1) = b(3,1);
        
        % Solves and allocates to v:

        v2 = C2\b2;
        
        v = zeros(xdim,1);
        
        conta = 0;
        for di = 1:1:xdim
            
            if di==df
                
                v(di,1) = u(di,1);
                
            else
                
                conta = conta + 1 ;
                v(di,1) = v2(conta,1);
            end
            
        end


        

    else
        
        % 3D problem:
        
        v = C\b;
        
    end

    
    

    % transfers to Pg:
    Pg = zeros(xdim,1);

    for di=1:1:xdim
        Pg(di) = Q(di) + v(di);
    end
  
end




end

