function [CL, Pr,vr ] = go2508_cloudP_line (np, P)

% RETURNS:
% CL:   0 / 1  if cloud of points P DO NOT form a line or YES, form a line
% Pr:   point of the line 
% vr:   versor (module=1) of the line

% INPUTS:
% np: No. points
% P(np,xdim): cloud of points coordinates


% 1.- PRELIMINARIES:

% size of the problem
xdim = size(P,2);

% pre-sizes answers (considers extended 3 components):
CL = 1 ;
Pr = zeros(3,1);
vr = zeros(3,1);


% 2.- ALLOCATES TO Pr FIRST POINT AND COMPUTES vr WITH SECOND POINT:

for di=1:1:xdim
   Pr(di) = P(1,di);
   vr(di) = P(2,di) - P(1,di);
end

vr = vr / norm(vr);


% 3.- CHECKS IF OTHER POINTS ARE CO-LINEAR:

% checks if trial vector 'tv' from Pr to point is proportional to vr.

tv  = zeros(3,1);
prop = zeros(3,1); % proportion
tolp = 0.000001 ; % tolerance in the proportions

for po = 3:1:np
    
    % computes trial vector and proportion:
    for di = 1:1:xdim
        tv(di) = P(po,di) - Pr(di);
        prop(di) = tv(di) / vr(di);
    end
    
    % check if proportion is the same:
    for di = 2:1:xdim
        err = abs(prop(1) - prop(di));
        if err > tolp
            CL = 0 ;
            break
        end
    end
    
    % breaks in case is not co-linear:
    if CL==0
        break
    end
    
end





end

