function [PV] = go2509_ParallelVectors (u, v, xdim, nd)

% RETURNS:
% PV = 1/0 if vectors are parallel or not

% INPUTS:
% u,v: vectors
% xdim: dims of the problem
% nd: No. decimals to round up


PV = 1 ; % assumes are parallel unless proved otherwise

% 1.- CHECK FOR COMPONENTS WITH ZEROS:

for di = 1:1:xdim
    
    if round(u(di),nd)==0 && round(v(di),nd)~=0
        PV = 0 ;
        break
    end
    
    if round(u(di),nd)~=0 && round(v(di),nd)==0
        PV = 0 ;
        break
    end
    
end


% 2.- IF STILL POTENTIAL PARALLEL:

if PV == 1

    % 2.1.- Proportions in each direction: only considers if no-zeros 
    % stores in cte the first proportion non-zero

    prop = zeros(xdim,1);
    cte = 0 ;
    for di = 1:1:xdim
        if round(u(di),nd)~=0 && round(v(di),nd)~=0

            prop(di) = u(di)/v(di);

            if cte==0
                cte = prop(di);
            end
        end
    end



    % 2.- Checks the proportions (it is assumed PV = 1):

    tolp = 10^(-nd);

    for di = 1:1:xdim

        if prop(di)>0
        
            diff = abs(prop(di)-cte);

            if diff > tolp
                PV = 0;
                break
            end
        end
    end


%{
    % 3.- Checks the zero value components, if still PV = 1:

    if PV==1

        for di = 1:1:xdim

            if round(u(di),nd)==0 && round(v(di),nd)~=0
                PV = 0;
                break
            end

            if round(u(di),nd)~=0 && round(v(di),nd)==0
                PV = 0;
                break
            end
        end

    end
%}
end

end

