function [cp] = go2514_4Pcoplanar (F, ndim, tol)

% RETURNS: if 4 points are coplanar 
% cp = 1 if coplanar, 0 otherwise

% INNPUTS:
% F(4, ndim):   the 4 points coordinates
% ndim:         No. dimensions = 3
% tol:          percentaje of max. length to consider the 4th point is in the plane


% 1.- Computes max. length edge and abs. value of tolerance:

Ned = 3 * (ndim-1); % No. edges
ecM = [1 2 ; 1 3 ; 1 4 ; 2 3 ; 2 4 ; 3 4]; % edges conM
eL = zeros(Ned,1);

for ed = 1:1:Ned
    
    acc = 0 ;
    n1 = ecM(ed,1);  n2 = ecM(ed,2);
    for di = 1:1:ndim
        acc = acc + (F(n2,di) - F(n1,di))^2 ;
    end
    eL(ed) = acc^0.5;
    
end

atol = max(eL) * tol/100 ;


% 2.- Computes plane with two different basis and compare both:

a = zeros(ndim,1);
b = zeros(ndim,1);
c = zeros(ndim,1);
P = zeros(ndim,1);

cp = 0 ; % initially assumes F DO NOT form plane

% 2.1.- First set:

for di=1:1:ndim
    a(di) = F(1,di);
    b(di) = F(2,di);
    c(di) = F(3,di);
    P(di) = F(4,di); 
end

[A,B,C,D] = go2506_Plane3P (a,b,c);



err = A*P(1) + B*P(2) + C*P(3) + D ;

if abs(err) < atol
cp = 1;
end


% 2.2.- Second set:

for di=1:1:ndim
    a(di) = F(2,di);
    b(di) = F(3,di);
    c(di) = F(4,di);
    P(di) = F(1,di); 
end

[A,B,C,D] = go2506_Plane3P (a,b,c);

err = A*P(1) + B*P(2) + C*P(3) + D ;

if abs(err) < atol
cp = 1;
end



end

