function IN = go2515_t_P_in (ct, P, udim, Nne)


% RETURNS:
% if P inside Tr (triangle or tetrahedral), then IN=1. Otherwise IN=0.
% works with 2D triangles or 3D tetrahedrals


% INPUTS:
% ct:    coord's of the Tr, u(Nne,udim)
% P:    point to evaluate, P(udim,1)
% Nne:  No. nodes per element

% if sum of Nne vols. between P and u is equal or less than total vol, P is IN

IN = 0;

% 0.- Computes Jacobian of the Tr and the total vol. (Vt):
% volume = 1/6 of Jacobian for tet.; 1/2 of Jacobian for triang.


denom = (Nne^2 -3*Nne + 2) ; 

J2 = GEN_06_01_Jac2_Element( Nne, udim, ct, 0 );

Vt = J2 / denom;



% 1.- Volumes of each sub-Tr:

V = zeros(Nne,1);
sumV = 0;
ut = zeros(Nne,udim);



for t=1:1:Nne
    

    % passes original nodes
    for it=1:1:Nne
        
        if it ~= t
            
            for di = 1:1:udim
            ut(it,di) = ct(it,di);
            end
            
        elseif it == t % substitutes t node by P
            
            for di=1:1:udim
            ut(it,di) = P(di);    
            end
        end
        
        
    end


    
    % calculates Jacobian and volume:
    
    J2 = GEN_06_01_Jac2_Element ( Nne, udim, ut, 0 );
    V(t) = J2 / denom;
 
    sumV = sumV + abs(V(t));
end




% 2.- Checks:

if sumV <= 1.00000001 * Vt
   IN =1; 
end





end

