function [IP, inter] = go2524_OrtoSegPlaneIntersec (s, p, modo)

% RETUNRS: 
% IP:       intersection point location between a segment and a portion of plane (if they do not intersect)
% inter:    1 if they insertect, 0 otherwise

% for 3D case, assumes:
% - plane defined by 4 points, and rectangular shape
% - segment by 2 points
% - segment and plane are perpendicular

% INPUTS:
% s(2,dim):    points coord's of segment s
% t(4,dim):    points coord's of plane p ('patch')
% modo: 'opened' / 'closed'

dim = 3; % No. dimensions

% 1.- Detects which dimension varies in segment (that is the constant in 'p'):
for di=1:1:dim
    
    if s(1,di) ~= s(2,di)
        cdi = di ; % 'constant' dimension in 'p'
       break 
    end
    
end

% 2.- Defines if intersect or not. Two conditions for intersection:
% a.- segment passes through patch plane (except if is just at the end)
% b.- segment included in patch area (which is a rectangular area)


% 2.a.- Checks if segment passes through patch

Ca = 0; % initially considers contidion not met 

if s(1,cdi)< p(1,cdi) && p(1,cdi)< s(2,cdi)
    Ca = 1 ;
elseif s(2,cdi)< p(1,cdi) && p(1,cdi)< s(1,cdi)
    Ca = 1 ;
end
    

%{
if strcmp(modo,'opened')==1
	if s(1,cdi)==p(1,cdi) || p(1,cdi)==s(2,cdi)
        Ca = 0;
    end
end
%}


% 2.b.- Checks if segment is located inside the area of the patch:

P = zeros(2,1);
domL = zeros(2,2);

conta=0;
for di=1:1:dim
    if di~=cdi
        conta = conta+1;
        P(conta,1) = s(1,di);
        domL(conta,1) = min ([ p(1,di)  p(2,di)  p(3,di)  p(4,di) ]);
        domL(conta,2) = max ([ p(1,di)  p(2,di)  p(3,di)  p(4,di) ]);
    end
end

Cb = GEN_08_Orthodomain_Pin (P, domL, 2, 'closed');



% 2.2.- Defines inter:
inter = 0;
if Ca==1 && Cb==1
    inter = 1 ;
end


% 3.- Computes intersection:

IP = zeros(3,1);

for di=1:1:dim
   
    if di==cdi
        IP(di,1) = p(1,di);
    else
        IP(di,1) = s(1,di);
    end

end




end

