function [dist, d, Q] = go2534b_dP2Plan (P, Tnod, n, a )

% RETURNS:
% dist: distance from P to plane
% d(xdim,1): vector P to plane (Q - P), Q closest plane point to P
% Q(xdim,1): projection onto the plane


% INPUTS:
% P(xdim,1): point coordinates
% Tnod(3,dim): three points of the plane             ( = 0 if not used)
% n(dim,,1), a(dim,1): normal and point of the plane ( = 0 if not used)

dim = 3 ;


% 0.- WORKS OUT Tnod IN CASE THE PLANE IS GIVEN BY n & a:

if Tnod == 0

    % detect any non-zero component of n vector:
    
    for di = 1:1:dim
        
        if round(n(di,1),4)~=0
            nzd = di;
            break
        end
        
    end
    
    odm = [ 2 3 ; 1 3 ; 1 2 ]; % the other directions
    
    od1 = odm(nzd,1);
    od2 = odm(nzd,2);
    

    % two more points:
    
    b = a ; b(od1,1) = b(od1,1) + 1 ; % initial position
    c = a ; c(od2,1) = c(od2,1) + 1 ;
    
    b(nzd,1) = a(nzd,1) - ( (b(od1)-a(od1))*n(od1) + (b(od2)-a(od2))*n(od2) )/n(nzd); % the 'third' coordinate
    c(nzd,1) = a(nzd,1) - ( (c(od1)-a(od1))*n(od1) + (c(od2)-a(od2))*n(od2) )/n(nzd);
    
    
    % assemblies Tnod:
    
    Tnod = [a' ; b' ; c' ];
    
end


% 1.- COEFF of PLANE FORMED BY THE 3 POINTS:

a = zeros(dim,1);
b = zeros(dim,1);
c = zeros(dim,1);

a(:,1) = Tnod(1,:);
b(:,1) = Tnod(2,:);
c(:,1) = Tnod(3,:);


% u,v:  vectors in plane

u = zeros(dim,1);
v = zeros(dim,1);

for di=1:1:dim
    u(di) = b(di)-a(di);
    v(di) = c(di)-a(di);
end

% plane implicit equations coefficients:
A = u(2)*v(3) - u(3)*v(2);
B = u(3)*v(1) - u(1)*v(3);
C = u(1)*v(2) - u(2)*v(1);
D = -A*a(1) - B*a(2) - C*a(3);


% 2.- DISTANCE CALCULATION:

dist = abs( A*P(1) + B*P(2) + C*P(3) + D ) / (A^2 + B^2 + C^2)^0.5 ;


% 3.- PROJECTED POINT FROM P TO PLANE:

Q = zeros(dim,1);

v = [A, B, C]'; % vector of the line P-Q

% 3.1.- Line parameter 't' that cuts the plane

t = ( A*P(1) + B*P(2) + C*P(3) + D ) / (A^2 + B^2 + C^2) ; 
t = round(t,12);

% 3.2.- Computes Q:

for di = 1:1:dim
    Q(di,1) = P(di,1) - t*v(di,1);
end


% 4.- VECTOR d = Q-P:
 
d = zeros(dim,1);

for di = 1:1:dim
    d(di,1) = Q(di,1) - P(di,1);
end


end

