function [L] = go2537_ArcL (Xv, ndim)

% RETURNS:
% L: arc lenght 

% INPUTS:
% Xv(3,ndim): three points of the arc, the second is inside the arc, 1 and 3 are the ends.
% ndim: No. dims of the problem

if ndim==2
   Xv = [ Xv , zeros(3,1) ] ; 
end


% extracts points:

xdim = 3 ;
a = zeros(xdim,1);
b = zeros(xdim,1);
c = zeros(xdim,1);

a(:,1) = Xv(1,:);
b(:,1) = Xv(2,:);
c(:,1) = Xv(3,:);


% checks if they are aligned:

v1 = c-a;
v2 = b-a;
[aT] = go2511_angleUV (v1,v2);

if aT < 0.009 % (half degree)
    
    L = norm(v1);
    return
    
end


% Plane formed by Xv and its normal:

[A,B,C,D] = go2506_Plane3P (a,b,c);

n = [A;B;C];
n = n / norm(n);

% Vectors between points:

v1 = b - a ;
v2 = c - b ;

% Direction of radius:

r1 = cross(v1,n);
r2 = cross(v2,n);


% Mid-points:

m1 = (a + b)/2 ;
m2 = (b + c)/2 ;


% Centre of arc:

[IP, fNI] = go2530_LinInters_nD (m1,r1, m2,r2, 3, 6);


% Angle of the arc:

v1 = a-IP;
v2 = b-IP;
v3 = c-IP;

[aT] = go2511_angleUV (v1, v3); % total angle

[a1] = go2511_angleUV (v1, v2); % from v1 to v2
[a2] = go2511_angleUV (v2, v3); % from v2 to v3

% both angles must be smaller than aT
if aT  <a1 || aT < a2
    aT = 360/57.3 - aT;
end

% Length:

L = aT * norm(v1);


end

