function [IN, sR, eR, nR, pF] = go2539_PinTet (Tnod, P, nd)

% RETURNS:
% IN = 1 if P is inside tet, 2 if is in one side, 3 if at edge, 4 at node. 0 outside
% sR, eR, nR: side, edge, node ref in case it lies at boundary
% pF = 1 if it is a plane tet

% INPUTS:
% Tnod(4,dim): tet nodes
% P(dim,1): point to evaluate

IN = 0 ;
dim = 3 ;

% determinants:

D0 = [Tnod ones(4,1)] ;

D1 = D0;
D2 = D0;
D3 = D0;
D4 = D0;

for j = 1:1:3
    D1(1,j) = P(j) ;
    D2(2,j) = P(j) ;
    D3(3,j) = P(j) ;
    D4(4,j) = P(j) ;
end


dD0 = round(det(D0),nd);

u = zeros(3,1);
v = zeros(3,1);
w = zeros(3,1);

for di = 1:1:3
    
    u(di,1) = Tnod(2,di) - Tnod(1,di);
    v(di,1) = Tnod(3,di) - Tnod(1,di);
    w(di,1) = Tnod(4,di) - Tnod(1,di);
    
end

t = 1/6*abs(dot(cross(u,v),w));

dDi = zeros(4,1);

dDi(1) = round(det(D1),nd);
dDi(2) = round(det(D2),nd);
dDi(3) = round(det(D3),nd);
dDi(4) = round(det(D4),nd);


% evaluates plane tet:

pF = 0 ;
if round(dD0,nd)==0
    pF = 1 ;
end



% evaluates P point:

sR = 0 ;
eR = [0 0];
nR = 0 ;

%{
% if lies at node:

    Pt = zeros(dim,1);
    
    for k = 1:1:4
        
        Pt(:,1) = Tnod(k,:);
        CO = go2542_coincidentP (P, Pt, dim, nd);
        
        if CO == 1
            IN = 3 ;
            return
        end
    end
%} 
    
% stores zeros and non-zeros deteminants
    
    Nz = 0 ;
    zR = zeros(1,1);
    
    Nd = 0 ;
    dR = zeros(1,1);
    
    for i = 1:1:4
        
        if round(dDi(i),nd) == 0
            Nz = Nz + 1 ;
            zR(Nz,1) = i ;
        else
            Nd = Nd + 1 ;
            dR(Nd,1) = i ;
        end

    end


    
if Nz == 0 % lies in or out, but not at boundaries:
        

        IN = 1 ;
        
        for i = 1:1:4
        
            if sign(dD0) ~= sign(dDi(i))
                IN = 0 ;
                break
            end
        end
        
        
elseif Nz == 1 % might lie at one face:
        
      	IN = 2 ;
        sR = zR(1) ; % side ref is the opp. node
        
        for i = 1:1:4
        
            if dDi(i)~=0
                if sign(dD0) ~= sign(dDi(i))
                    IN = 0 ;
                    sR = 0 ;
                    break
                end
            end
        end
        

elseif Nz == 2 % might lie at one edge:
        
        IN = 3 ;
        eR = [zR(1) zR(2)]; % edge ref are its nodes
        
        for i = 1:1:4
        
            if dDi(i)~=0
                if sign(dD0) ~= sign(dDi(i))
                    IN = 0 ;
                    eR = 0 ;
                    break
                end
            end
        end
        
        
elseif Nz == 3 % lies at one node:
        
        IN = 4 ;
        nR = dR(1);
        
end
    

end

