function [IN, eR, nR] = go2548_PinTri2 (Tnod, P, dim, nd)

% RETURNS:
% IN = 1 if P is inside tet, 2 if is at edge, 3 at node. 0 outside
% eR, nR: edge, node ref in case it lies at boundary
% pF = 1 if it is a line triangle

% INPUTS:
% Tnod(3,dim): tri nodes
% P(dim,1): point to evaluate

IN = 0 ;
eR = 0 ;
nR = 0;
rTnod = round(Tnod,nd);


% Vectors from each node to P:

t1 = zeros(dim,1);
t2 = zeros(dim,1);
t3 = zeros(dim,1);

for di = 1:1:dim

    t1(di,1) = P(di,1) - Tnod(1,di);
    t2(di,1) = P(di,1) - Tnod(2,di);
    t3(di,1) = P(di,1) - Tnod(3,di);

end

% Check if it is on node:

if round(norm(t1),nd) == 0
    IN = 3 ;
    nR = 1 ;
    return
elseif round(norm(t2),nd) == 0
	IN = 3 ;
    nR = 2 ;
    return 
elseif round(norm(t3),nd) == 0
	IN = 3 ;
    nR = 3 ;
    return 
end


% Vectors of contour:

u = zeros(dim,1);
v = zeros(dim,1);
w = zeros(dim,1);

u(:,1) = Tnod(2,:) - Tnod(1,:);
v(:,1) = Tnod(3,:) - Tnod(2,:);
w(:,1) = Tnod(1,:) - Tnod(3,:);


% Removes third dim, unless there is other with same coords:

if dim == 3 % (only for 3 dims)

    rd = 3 ;
    for di = 1:1:2
        if rTnod(1,di) == rTnod(2,di) && rTnod(1,di) == rTnod(3,di)
            rd = di ;
        end
    end

    % other dims to keep:

    kd = zeros(2,1);

    conta = 0 ;
    for di = 1:1:dim
        if di ~= rd
            conta = conta + 1 ;
            kd(conta,1) = di;
        end
    end
    
    t1 = [t1(kd(1)) ; t1(kd(2)) ] ;
    t2 = [t2(kd(1)) ; t2(kd(2)) ] ;
    t3 = [t3(kd(1)) ; t3(kd(2)) ] ;
    
    u = [u(kd(1)) ; u(kd(2)) ] ;
    v = [v(kd(1)) ; v(kd(2)) ] ;
    w = [w(kd(1)) ; w(kd(2)) ] ;
    
end


% Checks first angle:

a1 = [u,-w]\t1 ;
a1 = round(a1,nd);

if a1(1)<0 && a1(2)<0
    IN = 0 ;
    return
end
    

% Checks second angle:

a2 = [v,-u]\t2 ;
a2 = round(a2,nd);

if a2(1)<0 && a2(2)<0
    IN = 0 ;
    return
end
    

% If reaches here, P is inside, either within or at the edge:

if (a1(1)>0 && a1(2)>0) && (a2(1)>0 && a2(2)>0)
    
    IN = 1 ;
    
elseif (0 < a1(1) && a1(1) < 1) && a1(2)==0
    
    IN = 2 ;
    eR = 1 ;
    
elseif (0 < a2(1) && a2(1) < 1) && a2(2)==0
    
    IN = 2 ;
    eR = 2 ; 
    
else
    
    a3 = [w,-v]\t3 ;
    a3 = round(a3,nd);

    if (0 < a3(1) && a3(1) < 1) && a3(2)==0
    IN = 2 ;
    eR = 3 ; 
    end
    
end


end

