    function [X] = b2602_BNURBS_Xcoord_gPatch (u, gPatch, pa)

% RETURNS:  X(xdim,1) physical coordinates for NURBS domain for any dimension
% uses gPatch structure to take NURBS data

% INPUTS:
% u:        param. coord's 
% pPatch(pa): patch NURBS data


% 1.- Extracts patch data and initiates response and :

KnotV = gPatch(pa).KnotV;
Ncp = gPatch(pa).Ncp;
Degree = gPatch(pa).Degree;
Usize = gPatch(pa).Usize;
Ew = gPatch(pa).Ew;
EB = gPatch(pa).EP;

udim = gPatch(pa).Udim;
xdim = gPatch(pa).Xdim;

X = zeros(xdim,1);


% 2.- Finds influential cp's in each direction (from span-p to span) at u:

sp = ones(3,1); % considers 3 udims for size, but not all of them might be used

if size(KnotV,2)==1
    KnotV = KnotV';
end

for di=1:1:udim
    U = KnotV(di,1:Usize(di));
    sp(di) = GEN_03_FindSPAN(Ncp(di),Degree(di),u(di), U) ; % span in di direction
end



% 3.- Calculates influential basis functions at u:

N = ones(3,max(Degree)); % considers 3 udims for size, but not all of them might be used

for di=1:1:udim
    
    p =Degree(di);
    i=1;
    
    for bf = sp(di)-p : 1 : sp(di)
        
        U = KnotV(di,1:Usize(di));
        
        [bF,bR] = GEN_01_CalcBF(bf,u(di),p,U,Ncp(di));
        N(di,i) = bR(p+1,1);
        
        i = i + 1;
    end
    
end



% 4.- Calculates W:

W = 0 ;

bf1 = sp(1)-Degree(1);
for i=1:1:Degree(1)+1
    
    
    if udim == 1
        
        W = W + N(1,i) * Ew(bf1);
        
    else
    
        bf2 = sp(2)-Degree(2);
        for j=1:1:Degree(2)+1


            if udim==2

                W = W + N(1,i) * N(2,j) * Ew(bf1,bf2);

            elseif udim == 3

                bf3 = sp(3)-Degree(3);
                for k=1:1:Degree(3)+1 

                    W = W + N(1,i) * N(2,j) * N(3,k) * Ew(bf1,bf2,bf3);

                bf3 = bf3 + 1 ;    
                end

            end


        bf2 = bf2 + 1 ; 
        end   

    end

bf1 = bf1 + 1 ;
end




% 5.- Calculates X:

bf1 = sp(1)-Degree(1);
for i=1:1:Degree(1)+1
    
    
  	if udim == 1
        
        R = N(1,i) * Ew(bf1) / W;
     	for dir=1:1:xdim
            X(dir) = X(dir) + R *  EB(bf1,dir);
      	end

    else
    
        bf2 = sp(2)-Degree(2);
        for j=1:1:Degree(2)+1


            if udim==2

                R = N(1,i) * N(2,j)  * Ew(bf1,bf2) / W;
                for dir=1:1:xdim
                    X(dir) = X(dir) + R *  EB(bf1,bf2,dir);
                end

            elseif udim==3

                bf3 = sp(3)-Degree(3);
                for k=1:1:Degree(3)+1 

                    R = N(1,i) * N(2,j) * N(3,k) * Ew(bf1,bf2,bf3) / W;
                    for dir=1:1:xdim
                        X(dir) = X(dir) + R *  EB(bf1,bf2,bf3,dir);
                    end

                bf3 = bf3 + 1 ;    
                end

            end


        bf2 = bf2 + 1 ; 
        end 

    end

bf1 = bf1 + 1 ;
end







end

