function [X,Ta] = t2601_TNURBS_Xcoord (Relem, RnEL, mA, Degree, u, udim, xdim)

% RETUNS:
% X: coord's in physical space of T-spline domain given param coord's 'u'
% Ta(a): T-NURBS values at each CP at (xi,ita,chi), note that uses local
% refs, that is, a: 1,2,3, ... and not the anchor global reference.

% INPUTS:
% Relem, RnEL:  element objects and No. for rect-mesh
% mA:           anchors object
% u:            param. coord's


% 1.- Rounds params to make sure they are not out of scope:
%  and finds the element where u lies:

u = round(u,6);

eR = GEN_28_FindRelem (Relem, RnEL, u, udim);


% 2.- Determines how many CP's are influential and provide prelim. sizes:

Nicp  = Relem(eR).mNicp;
RicpA = Relem(eR).mRicpA;

X  = zeros(xdim,1);
N  = zeros(Nicp,1);  % T-splines
Ta = zeros(Nicp,1);  % T-NURBS



% 3.- For all the involved CP's, calculates T-splines and total weight :

W = 0 ;
for a = 1:1:Nicp
    
    % anchor reference:
    % aRef = Relem(eR).mRicpA(a);
    aRef = RicpA(a);
    
    % computes basis functions:
    N(a) = 1 ;
    for di=1:1:udim
        
        bR = t2604_Tspline_u (aRef, Degree, mA, u(di), di);
        N(a) = N(a) * bR(Degree(di)+1,1)  ;
        
    end
    
    % Weight:
    W = W + N(a) * mA(aRef).mWeig;  
    
end




% 4.- Calculates T-NURBS and coordinates:

for a=1:1:Nicp
if N(a)~=0
    
    % anchor reference:
    % aRef = Relem(eR).mRicpA(a);
    aRef = RicpA(a);
    
    Ta(a) = N(a) * mA(aRef).mWeig / W ;
    
    for di = 1:1:xdim
        X(di) = X(di) + Ta(a)* mA(aRef).mXcoor(di) ; 
    end
    
end
end




end