function [X, Ta] = t2602_TNURBS_Xcoord_withN (mA, N, Nicp, RicpA, xdim)

% RETUNS:
% X: coord's in physical space of T-spline domain given param coord's 'u'
% Ta(a): T-NURBS values at each CP at (xi,ita,chi), note that uses local
% refs, that is, a: 1,2,3, ... and not the anchor global reference.

% INPUTS:
% mA:           anchors object
% N(icp):       basis functions tensor product values at 'u'

% NOTE: the input N avoids to compute here basis funcions. Location 'u' is
% provided implicitly by N basis functions


% 1.- Prelim. sizes:
X = zeros(xdim,1);
Ta = zeros(Nicp,1);  % T-NURBS


% 2.- calculates total weight :
W = 0 ;
for a=1:1:Nicp
    aRef = RicpA(a); % anchor reference
    W = W + N(a) * mA(aRef).mWeig;  
end


% 3.- Calculates T-NURBS and coordinates:
for a=1:1:Nicp

    
    aRef = RicpA(a); % anchor reference
    
    Ta(a) = N(a) * mA(aRef).mWeig / W ;
    
    for di = 1:1:xdim
        X(di) = X(di) + Ta(a)* mA(aRef).mXcoor(di) ; 
    end
    
  
end


end