function bR = t2604_Tspline_u (aRef, Degree, mA, u, dir)

% calculates basis T-SPLINE functions at u in direction 'dir'
% returns [bR]:  functions results: (p+1,p+1) triangular matrix (symmetric) with function value from p=0 to p=pmax.

% NOTE: uses mA object instead Uan

% aRef: bf or CP or anchor reference
% pqrAn: order of anchor
% Uan: local knot vector
% u: location where to calculates the basis function
% dir: states which direction xi, ita or chi . Then, order and local knot vector changes


% 0.- Rounds to avoid values out of knot vector:
 u = round(u,8);
 
 
% 1.- Pre-sizes and basic parameters:
    
p = Degree(dir); % order
U = zeros(p+2,1); % local knot vector dimension

U(:) = mA(aRef).mLKV(dir,1:1:p+2); % local knot vector

bR = zeros(p+1,p+1) ; % sizes results matrix

    
% 2.- Fills up the bR matrix:
    
for i=1:1:p+1 % from order 0 to p (rows from 1 to p+1)
    for j=1:1:(p+1)-(i-1) % cols. from 1 to vble. (upper triang. symm.)
        
        if i==1 % order zero
            bR(i,j)=BasisN_0(j, u, U, p, 0);
        else
            bR(i,j)=BasisN_h(bR, j,j, u, i-1, U);  
        end

    end
end

    
end