function [Ta] = t2606_TNURBS_at_u (Nicp, RicpA, mA, Degree, u, udim)

% RETUNS:
% Ta(a): T-NURBS values at each 'u' location,for all icp's. Uses relative
% refs, that is, a: 1,2,3, ... and not the anchor global reference.

% INPUTS:
% mA:           anchors object
% u:            param. coord's

% 1.- PRELIMINARIES:

% 1.1.- Rounds params to make sure they are not out of scope:
u = round(u,6);

% 1.2.- prelim. sizes:
N  = zeros(Nicp,1);  % T-splines
Ta = zeros(Nicp,1);  % T-NURBS


% 2.- FOR ALL INVOLVED CP's, calculates T-splines and total weight :

W = 0 ;
for a = 1:1:Nicp
    
    % anchor reference:
    % aRef = Relem(eR).mRicpA(a);
    aRef = RicpA(a);
    
    % computes basis functions:
    N(a) = 1 ;
    for di=1:1:udim
        bR = t2604_Tspline_u (aRef, Degree, mA, u(di), di);
        N(a) = N(a) * bR(Degree(di)+1,1)  ;
    end
    
    % Weight:
    W = W + N(a) * mA(aRef).mWeig;  
end


% 3.- Calculates T-NURBS:

for a=1:1:Nicp
if N(a)~=0
    
    % anchor reference:
    % aRef = Relem(eR).mRicpA(a);
    aRef = RicpA(a);
    Ta(a) = N(a) * mA(aRef).mWeig / W ;
    
end
end




end

