function [dA, dW] = b2702_dA_dW (u, KnotV, Ncp,Degree,Usize, Ew,EP, orP, udim, xdim)

% RETURNS:
% dA: derivative of 'A' about param space
% dW: derivative of weight

% INPUTS:
% u:    location where compute dA
% patch features
% orP:  derivative order plus 1 , useful for matrix indexes



% 1.- ESTABLISHES icp's IN EACH param. DIRECTION AND ITS REF'S:

% consider case of max. dims = 3
Nicp = ones(3,1);
Ricp = ones(3,max(Degree)+1);

for di=1:1:udim
    
    n = Ncp(di);
    p = Degree(di);
    xi = u(di);
    
    U = zeros(Usize(di),1);
    for i = 1:1:Usize(di)
        U(i,1) = KnotV(di,i);
    end
     
    % span where lies in (with no consideration of first and last p-1 spans)
    sp = GEN_03_FindSPAN(n,p,xi,U)-p-1;
    
    % No. icps and ref:
    Nicp(di,1) = p+1;
    for i = 1:1:p+1 
        Ricp(di,i) = sp+i;
    end
end


% 2.- EXTRACTS LKV's IN EACH DIRECTION & STORES in  LKVdi

LKVdi = zeros(udim,max(Nicp),max(Degree)+2); % 3 param dir's, No. LKV in each param dir , LKV components

for di = 1:1:udim
    
	% stores all LKV's in that direction:
    for a = 1:1:Nicp(di)

        aR = Ricp(di,a);

        for i = 1:1:Degree(di)+2 
            LKVdi(di,a,i) = KnotV(di, aR+i-1);
        end
    end  
end



% 3.- COMPUTES DERIVATIVES OF EACH LKV & STORES IN dBF:

dBF = ones(3, max(Nicp), max(orP));

for di = 1:1:udim
   
    p = Degree(di);
    U = zeros(p+2,1);

    for kv = 1:1:Nicp(di) % loop on all LKV's
        
        % extracts LKV:
      	for i=1:1:p+2
      	U(i) = LKVdi(di,kv,i);
        end


        % computes basis functions values ( from degree 0 to p):
      	n = 1 ;
        bf = 1;
        [~,bR] = GEN_01_CalcBF (bf,u(di),p,U,n);
        
        % computes all derivatives orders:
        for k = 1:1:orP(di)
            d = z2701_dN_tpl (k-1, p, bR, U);
            
            
            %{
            if (sum(u)==2 && k==2) && round(U(2),4)==0.7649

                format long
                
                
                KnotV(1,8)
                U(3)
                U(4)
                U(5)
                u(di)
                
                format short
                
                UDI = di

                bR
                kv
                
                d
                
            end
            %}
            
            dBF(di,kv,k) = d ;
        end
        
    end
end





% 4.- COMPUTES DERIVATIVES dA and dW:

dA = zeros(orP(1), orP(2), orP(3), xdim);
dW = zeros(orP(1), orP(2), orP(3)); 

Xp = zeros(xdim,1);

for a=1:1:Nicp(1)
for b=1:1:Nicp(2)
for c=1:1:Nicp(3)
    
    % cp's references:
    aR = Ricp(1,a);
    bR = Ricp(2,b);
    cR = Ricp(3,c);
    
    % coordinates of that cp:
    wp = Ew(aR,bR,cR);
    for di=1:1:xdim
        Xp(di,1) = EP(aR,bR,cR,di);
    end
        
    % computes derivatives contribution of that cp:
    for k = 1:1:orP(1)
    for l = 1:1:orP(2)
    for m = 1:1:orP(3)

        dN = dBF(1,a,k);
        dM = dBF(2,b,l);
        dL = dBF(3,c,m);

        dR = dN*dM*dL;
        
        for di=1:1:xdim
            dA(k,l,m,di) = dA(k,l,m,di) + dR * Xp(di) * wp ;
        end

        %{
        if round(u(1),3)==1 && round(u(2),3)==0.75
        if round(EP(2,3,1,1),4)==17.0431 || round(EP(2,3,1,1),4)==-57.9569
            
            dd=zeros(3,1);
            
            if k==2 && l==1
                dN
                dM
                dR
                dd(:,1) = dA(k,l,m,:);
                dd
            end
            
        end
        end
        %}
        
        dW(k,l,m) = dW(k,l,m) + dR * wp ;
    end
    end
    end
end
end
end






end

