function [dA, dW] = t2702_dA_dW (mA, nAN, Degree, u, Nicp, RicpA, ord, orp, udim, xdim)


% RETURNS:
% dA: derivative of 'A' about param space

% INPUTS:
% u:    location where compute dA
% ord:  orders of derivatives
% orp:  derivative order plus 1 , useful for matrix indexes


% 1.- Extracts LKV's in each direction and stores in  LKVdi

LKVdi = zeros(3,1,max(Degree)+2);
nkv = zeros(3,1); % No. local knot vectors in eahc direction

for di = 1:1:udim
    
    LKVtemp = zeros(Nicp,Degree(di)+2);
    
    % stores all LKV's in that direction, remove repeated and sorts:
    for a = 1:1:Nicp

        aRef = RicpA(a);

        for i = 1:1:Degree(di)+2 
        LKVtemp(a,i) = mA(aRef).mLKV(di,i);
        end
    end

    [LKVtemp,~] = GEN_16_RemoveRepeatedRows (LKVtemp, Degree(di)+2, 0.01);
    LKVtemp = sortrows(LKVtemp,[1:1:Degree(di)+2]);

    nkv(di) = size(LKVtemp,1);
 
    % transfers to LKVdi:
    
    for i = 1:1:nkv(di)
    for j = 1:1:Degree(di)+2
        
        LKVdi(di,i,j) = LKVtemp(i,j);
        
    end
    end
end


% 2.- Computes derivatives of each LKV and stores in dBF:

dBF = ones(3, max(nkv), max(orp));

for di = 1:1:udim
   
    p = Degree(di);
    U = zeros(p+2,1);

    for kv = 1:1:nkv(di) % loop on all LKV's
        
        % extracts LKV:
      	for i=1:1:p+2
      	U(i) = LKVdi(di,kv,i);
        end
        
        % computes basis functions values ( from degree 0 to p):
      	n = 1 ;
        bf = 1;
        [~,bR] = GEN_01_CalcBF (bf,u(di),p,U,n);
        

        % computes derivatives:
        for k = 1:1:orp(di)
            d = z2701_dN_tpl (k-1, p, bR, U);
            dBF(di,kv,k) = d ;
        end
        
    end
end



% 3.- Computes derivatives dA and dW:

dA = zeros(orp(1), orp(2), orp(3), xdim);
dW = zeros(orp(1), orp(2), orp(3));
X  = zeros(xdim,1);


for a = 1:1:Nicp
    
    aRef = RicpA(a);
    kvr = ones(3,1); % correspondant kv ref. (or BF's ref) for aRef in each direction
    
    for di=1:1:xdim
       X(di) = mA(aRef).mXcoor(di); 
    end
    w = mA(aRef).mWeig ;
    
    
    % 3.1.- detects correspondant Bf's derivatives:

    for di = 1:1:udim
        
        p = Degree(di);
        U = zeros(p+2,1);
        
        % extracts knot vector
        for i = 1:1:p+2
      	U(i) = mA(aRef).mLKV(di,i);
        end
        
        % finds LKV ref. in LKVdi and stores in kvr:
        for kv = 1:1:nkv(di)
            
            distin = 0 ;
            for i=1:1:p+2
                if U(i) ~= LKVdi(di,kv,i)
                    distin=1;
                    break
                end
            end
            
            if distin == 0
                kvr(di) = kv ;
               break 
            end
            
        end
    end
    

    
    % 3.2.- Computes product of bf's derivatives and derivatives itself:
    
    dR = zeros(orp(1), orp(2), orp(3)) ;
    
    for k = 1:1:orp(1)
  	for l = 1:1:orp(2)
   	for m = 1:1:orp(3)
   	  
      	dN = dBF(1,kvr(1),k);
        dM = dBF(2,kvr(2),l);
        dL = dBF(3,kvr(3),m);

        dR(k,l,m) = dN*dM*dL;
          
        for di=1:1:xdim
        dA(k,l,m,di) = dA(k,l,m,di) + dR(k,l,m)*X(di)*w ;
        end
        
        dW(k,l,m) = dW(k,l,m) + dR(k,l,m)*w ;
        
    end
    end
    end
    
 
end






end

