function [dS] = t2703_dS (mA, nAN, Degree, dSp, ord, orp, u, dA, dW, udim, xdim)

% RETURNS:
% dS(xdim) derivative of order ord - k,l,m - at u


% INPUTS: 
% dSp(,,):  matrix with derivatives of previous orders
% ord(3):   order of derivative to compute, if 0 no derivative required
% orp(3):   derivative order plus 1 , useful for matrix indexes
% u:        param. coord. where compute the derivative at
% W:        total weight at u


% 1.- First summands:

isum = zeros(3,xdim) ;

for di = 1:1:udim
    
    k = ord(di);
    cw = [1, 1, 1];
    cs = [orp(1), orp(2), orp(3)];

    for i = 1:1:k

        cw(di) = i+1;
        cs(di) = k-i+1;

        for xdi=1:1:xdim
            isum(di,xdi) = isum(di,xdi) + GEN_05_binCo(k,i) * dW(cw(1),cw(2),cw(3))* dSp(cs(1),cs(2),cs(3),xdi);
        end

    end
end


% 2.- Global summand:

% add one order to redundant dimensions in case they are 0
% otherwise the gsum would not execute
for di=1:1:3
if (di > udim) && (ord(di)==0)
    ord(di) = 1 ;
end
end

gsum = zeros(xdim,1) ;

for a =1:1:ord(1)
for b =1:1:ord(2)
for c =1:1:ord(3)
    
    % sub-indexes for dS and dW matrices
    cs = ones(3,1);
    cw = ones(3,1);
    
    cs(1) = orp(1)-a ;
    cw(1) = a+1;
    if udim==2
    cs(2) = orp(2)-b ;
    cw(2) = b+1;
    elseif udim==3
    cs(3) = orp(3)-c ;
    cw(3) = c+1;
    end

    for di=1:1:xdim
        gsum(di) = gsum(di) + GEN_05_binCo(ord(1),a) * GEN_05_binCo(ord(2),b) * GEN_05_binCo(ord(3),c) * ...
                    dW (cw(1), cw(2), cw(3)) * dSp (cs(1), cs(2), cs(3), di) ;
    end
    
end
end
end



% 3.- Computes derivative itself:

dS = zeros(xdim,1);

for di=1:1:xdim
    dS(di) = 1 / dW(1,1,1) * ( dA(orp(1),orp(2),orp(3), di) - isum(1,di) - isum(2,di) - isum(3,di) - gsum(di) ) ;
end









end

