function [dN] = GEN1297_dN_tpl (k, p, bR, U)

% RETURNS:
% dN:   derivative 'k' of one dim. basis function at u

% INPUTS:
% k:    order of derivative (0 is not derivative)
% p:    polynomial degree
% bR:   bf's values at u, from degree 0 to p
% U:    local knot vector


if k > p
    
    dN = 0 ; % special case of derivative order higher than degree
    
else

    % 1.- Initiates and computes matrix of 'a' coeft's:

    a = zeros(k+1,k+1);
    a(1,1) = 1 ;

    for ki = 0:1:k

        fil = ki+1;

        if ki==0
            ini = 2;
        else
            ini = 1;
        end


        for j = ini:1:ki+1

            if j==1
                
                a(fil,j) = a(fil-1,j) / (U(2+p-ki) - U(1)) ;
                
                % case of zero denominator
                if (U(2+p-ki) - U(1))==0 
                    a(fil,j) = 0 ;
                end

                
            elseif j==ki+1

                a(fil,j) = -a(fil-1,j-1) / (U(2+p) - U(1+ki)) ;
                

                % case of zero denominator
                if (U(2+p) - U(1+ki))==0
                    a(fil,j) = 0 ;
                end

            else
                
                a(fil,j) = (a(fil-1,j) - a(fil-1,j-1)) / (U(1+p-ki+j) - U(0+j));

                % case of zero denominator
                if (U(1+p-ki+j) - U(0+j))==0
                    a(fil,j) = 0 ;
                end

            end

        end

    end

  	%if (U(5)==0.5 && U(4)==0) && k == 2
     %a
  	%end

    % 2.- Computes derivative itself:

    dN = 0 ;

    for j = 0:1:k
        su = a(k+1,j+1) * bR(p-k+1,1+j);
        dN = dN + su ;
    end

    dN  = (factorial(p)/factorial(p-k)) * dN ;

end


end

