function [d1Ni, Ni] = z2702_d1NML (u, r, Ncp, Degree, KnotV, udim)

% RETURNS:

% d1N(udim,1): first derivative about parameter of basis functions 
% (referneces in each direction: r(udim,1)) as location u
% Ni(udim,1): basis function at u

d1Ni = zeros(udim,1);
Ni = zeros(udim,1);

for di = 1:1:udim
    
    % Degree, Ncp and Local knot vector in that direction:
    p = Degree(di);
    n = Ncp(di);
    U = zeros(p+n+1,1);
    
    for i = 1:1:(p+n+1)
        U(i,1) = KnotV(di,i);
    end
    

    % basis functions:
    [~,N] = GEN_01_CalcBF (r(di),u(di),p,U,n);
    Ni(di,1) = N(p+1,1);
    
    
    % 1st derivative:
	den1 = U(p+r(di))-U(r(di));
    den2 = U(p+r(di)+1)-U(r(di)+1); 
    
    if den1>0
        d1Ni(di,1) = d1Ni(di,1) + p * N(p,1) / den1 ;
    end
    
    if den2>0
        d1Ni(di,1) = d1Ni(di,1) - p * N(p,2) / den2 ;
    end

end



end