function [dN, dNRHS] = z2799_BsplineDeriv (singu, p, U, Nabs, di, aRef)

% derivative of N function "a" at 'u' with respect direction 'di'.
% bf reference is a.
% dNRHS is at RHS for peaky locations (singu ~= 0)

% singu = -1 , 2 , +1 for peak at ini , mid , end . if 0 => no singular
% in case of singu~=0
% DerivSide: -1/0/+1 : computes LHS / average / RHS

% note: in case of peak, the derivative by default computes at the RHS,
% unless the peak is at the end, in that case computes at the LHS
% That is why here are modifications for singular fucntions to compute LHS & RHS


% 1.- Parameter for computing derivatives, and derivative by default

x_1 = U(1) ;    % knots involved in the equation
x_2 = U(2) ;
x_1p = U(1+p) ;
x_2p = U(2+p) ;

Ni      = Nabs(di,aRef,p,  1); % i   , p-1
Ni1     = Nabs(di,aRef,p,  2); % i+1 , p-1

if x_1p == x_1
    dNdt = 0;
else
	dNdt = p*Ni / (x_1p - x_1);
end

if x_2p == x_2
	dNdt = dNdt - 0;
else
    dNdt = dNdt - p*Ni1 / (x_2p - x_2);
end



% 2.- Case of smooth function: computes derivative normally. Allocates RHS=0

if singu == 0
    
    dN = dNdt;
    dNRHS = 0 ;
    
% 3.- Peaky function: computes LHS and RHS 

else
    % 3.1.- Peak at the beggining:
    if singu==-1 
    
        dN = 0 ;
        dNRHS = dNdt;
        
    % 3.2.- Peak at the middle:
    elseif singu==2
        
        % note that swapping Nip-1 values we obtain  
        % the derivative at the LHS of the peak:
        
        if x_1p == x_1
            dN = 0;
        else
            dN = p*Ni1 / (x_1p - x_1);
        end

        if x_2p == x_2
            dN = dN - 0;
        else
            dN = dN - p*Ni / (x_2p - x_2);
        end

        dNRHS = dNdt;
        
    % 3.3.- Peak at the end:    
    elseif singu==1
        
        dN = dNdt ;
        dNRHS = 0;
    end

end




end

