function [sU] = b2801_PPforSetPoints (nsp, sX, Degree, Ncp, KnotV, Usize, P ,w, CD, xdim, udim, wbON)

% RETURNS:
% sU(nsp,udim): set of patch parameter coordinates correspondant to a set of points (No. points = nsp) defined in physical space

% computation is done by using point projection

% INPUTS:
% nsp: No. points
% sX(nsp,xdim): physical coord's of points
% KnotV, Degree, Usize, nAN, mA: T-apline patch features
% CD (udim): closed domain flag
% wbON: to active wait bar

% 1.- Computes some parameters to use afterwards and first Ucoor for first Xcoor point projection:

firstX = zeros(xdim,1);
for di=1:1:xdim
    firstX(di) = sX(1,di);
end

[ULims, seekst, Ucoor, dmin, tolDist] = ...
    b2841_preparPProj (firstX, Degree, Ncp, KnotV, Usize, P,w, xdim, udim);                          


% 2.- Computes max. separation of Xcoor and pXcoor to re-compute initial Ucoor:
% IT IS SELECTED as 10% of max. distance

Xmin = zeros(xdim,1);
Xmax = zeros(xdim,1);

for di=1:1:xdim
    temp = sortrows(P,di);
    Xmin(di) = temp(1,di);
    Xmax(di) = temp(size(temp,1),di);
end

Dmax = 0.05 * norm(Xmax - Xmin);


if wbON==1
wb = waitbar(0,'computing Ucoor of stress rep. points');
set(wb,'Name','point projection process');
wbPos = [10 250 300 70]; % left bottom wide height
set(wb,'position', wbPos);
end

% 3.- Point projection for each point:
  
sU = zeros(nsp,udim);
Xcoor  = zeros(3,1);


for po = 1:1:nsp
    
    
    % 3.1.- Extracts point to work with:
    
    for di = 1:1:xdim
        Xcoor(di) = sX(po,di);
    end

    
    % 3.2.- Checks distance btwn actual and prev. points. If more than Dmax
    % recompures Ucoor for initial search:
    
    if po>1
        [Ucoor] = b2808_CheckXdist ...
        (po, sX, seekst, ULims, Xcoor, Ucoor, Dmax, dmin, tolDist, Degree, Ncp, KnotV, Usize, P ,w, udim, xdim);
    end

    % Rounds up to achieve same precision as in GEN_03_FindSpan
    Ucoor = round(Ucoor,12); 
    

    % 3.3.- Works out the searching:
    
    [Ucoor] = b2803_PP_with_preSel ...
    (Xcoor, Ucoor, ULims, seekst, dmin, tolDist, Degree, Ncp, KnotV, Usize, P ,w, CD, udim,xdim, po);


    % 3.4.- Transfers U value computed:

    for di = 1:1:udim
        sU(po,di) = Ucoor(di);
    end
    
    if wbON==1
    waitbar(po/nsp,wb, strcat('point= ',num2str(po+1))) ; 
    end
    
end

if wbON==1
delete (wb);
end



end

