function [J, dS1] = b2806_Jpp (Degree, Ncp, KnotV, Usize, P ,w, r, u, udim, xdim)

% RETURNS:
% J(udim,udim): Jacobian at u for N-R
% f(udim,1):    f vector at u for N-R
% dS1(udim,xdim): first derivatives of NURBS about udims, note each derivative have xdim components


% INPUTS:
% P(xdim,1):    target physical point
% S(xdim,1):    actual domain value
% u(udim,1):    current location in param. space
% udim: param. space No. dimensions
% xdim: physical space No. dimensions


% note: derivatives of NURBS 'dS' have xdim components

% 1.- COMPURES DERIVATIVES OF PATCH AT u: 
%   note it needs to compute up to second derivative in each
%   parameter direction dS(3,3,3, xdim), index is derivative 0

if round(r(3),4)==-29.9754
    %u = 0.25;
end


ord = [2,2,2];
[dS] = b2701_BNURBS_deriv (Degree, Ncp, KnotV, Usize, P ,w, ord, u, xdim, udim);

if round(r(3),4)==-29.9754
    
    %format long
    %u
    % format short
    
    %drivave= dS
    
end

% transfers relevant first and second derivatives:

dS1 = zeros(udim,xdim);      % stored as vector of rows, each row: Su, Sv, Sw
dS2 = zeros(udim,udim,xdim); % stored as matrix or rows, each row: Suu, Svv, ... Svw


for udi=1:1:udim
    
    % First derivatives vector:
    
    cs = ones(3,1);
    cs(udi) = 2 ;
    
    for xdi=1:1:xdim
    dS1(udi,xdi) = dS(cs(1),cs(2),cs(3),xdi);
    end
    
    
    % Second derivatives matrix: diagonal:
    
    cd = ones(3,1);
    cd(udi) = 3 ;
    for xdi=1:1:xdim
        dS2(udi,udi,xdi) = dS(cd(1),cd(2),cd(3),xdi);
    end
    
    
    % Second derivatives matrix: triang. components:
    
    if udi <= (udim-1)
    for j =2:1:udim
        
        cs = ones(3,1);
        cs(udi) = 2 ;
        cs(j) = 2 ;
        
        for xdi = 1:1:xdim
            dS2(udi,j,xdi) = dS(cs(1), cs(2), cs(3), xdi) ;
            dS2(j,udi,xdi) = dS(cs(1), cs(2), cs(3), xdi) ;
        end
    end
    end
end




% 2.- ASSEMBLES JACOBIAN AND f VECTOR:

J  = zeros(udim, udim);
f  = zeros(udim,1);

Si = zeros(xdim,1);
Sj = zeros(xdim,1);
Sij= zeros(xdim,1);

for udi = 1:1:udim
    
    for di=1:1:xdim
    Si(di,1) = dS1(udi,di);
    end   
    
    f(udi,1) = dot(r,Si);
    
    for j = 1:1:udim
        
        for di=1:1:xdim
            Sj(di,1)  = dS1(j,di);
            Sij(di,1) = dS2(udi,j,di);
        end
        J(udi,j) = dot(Si,Sj) + dot(r,Sij); 
    end
end




end

