function [iu, NOC] = b2842_intialU ...
    (seekst, ULims, Xcoor, dmin, tolDist, Degree, Ncp, KnotV, Usize, P ,w, udim, xdim)

% RETURNS: 
% iu(udim,1): initial ucoor for searching by point projection Xcoor
% NOC: 'no convergence', becomes 0 (converges) if dmin < tolDist, otherwise is = 1

% INPUTS:
% seekst(3):   step in each direction for searching
% ULims(udim,2): limits of param. space for searching
% Xcoor: point coord's to find closest param's
% tolDist: if dmin < tolDist then that is directly Ucoor

% Notes:
% - just loop on u steps to localize to closest Ucoor to Xcoor
% - assumes all knot vectors from 0 to 1
% - works with extended version of 3D



% 1.- SETS STEPS AND INITIAL/FINAL PARAMS. FOR SEARCHING:

rstp = ones(3,1);
u0   = zeros(3,1); % initial knot value for searching
ue   = ones(3,1);  % end knot value for searching

for di=1:1:udim
    rstp(di) = seekst(di);
    u0(di) = ULims(di,1);
    ue(di) = ULims(di,2);
end



% 2.- LOOPS IN PARAM SPACE (USING STEPS rstp) TO FIND CLOSEST TO Xcoor:

iu = zeros(3,1);

% reduces Xcoor to actual xdim:
rXcoor = zeros(xdim,1);
for di = 1:1:xdim
   rXcoor(di) = Xcoor(di); 
end

% expand (indexes) cp's coordinates:
[Ew,EP] = GEN_12_ExpandBw (w,P,xdim,udim, Ncp);

% checks points looping by step:


if udim==1

    %rstp(1) = 0.05 ;
    
    for u = u0(1):rstp(1):ue(1)

        [X] = b2601_BNURBS_Xcoord (u, KnotV, Ncp,Degree,Usize, Ew,EP, udim,xdim);
        tdis = norm(X-rXcoor);

        if tdis<dmin
            iu = [u;0;0];
            dmin = tdis;
        end
    end
    
else
    for u = u0(1):rstp(1):ue(1)
    for v = u0(2):rstp(2):ue(2)
    for w = u0(3):rstp(3):ue(3)    

        [X] = b2601_BNURBS_Xcoord ([u v w], KnotV, Ncp,Degree,Usize, Ew,EP, udim,xdim);
        tdis = norm(X-rXcoor);
  
        if tdis<dmin
            iu = [u;v;w];
            dmin = tdis;
        end
    end
    end
    end
end

% 3.- CHECKS IF 'iu' IS THE TARGET Ucoord AND CHANGES 'NOC' IF SO:

if dmin < tolDist
    NOC = 0 ;
else
    NOC = 1 ;
end



end

