function [J, dS1] = t2806_Jpp (Degree, nAN, mA, r, u, udim, xdim)

% RETURNS:
% J(udim,udim): Jacobian at u for N-R
% f(udim,1):    f vector at u for N-R
% dS1(udim,xdim): first derivatives of NURTS about udims, note each 
% derivative have xdim components


% INPUTS:
% P(xdim,1):    target physical point
% S(xdim,1):    actual domain value
% u(udim,1):    current location in param. space
% udim: param. space No. dimensions
% xdim: physical space No. dimensions


% note: derivatives of Tnurbs have xdim components


% 1.- Computes derivatives of patch at u: note it needs 
% to compute up to second derivative in each parameter direction
% dS(3,3,3, xdim)

ord = [2,2,2];
[dS] = GEN1299_TNURBS_deriv (Degree, nAN, mA, ord, u, xdim, udim);

Su  = zeros(xdim,1);
Sv  = zeros(xdim,1);
Suu = zeros(xdim,1);
Svv = zeros(xdim,1);
Suv = zeros(xdim,1);

for xdi = 1:1:xdim
    Su(xdi,1)  = dS(2,1,1,xdi);
    Sv(xdi,1)  = dS(1,2,1,xdi);
    Suu(xdi,1) = dS(3,1,1,xdi);
    Svv(xdi,1) = dS(1,3,1,xdi);
    Suv(xdi,1) = dS(2,2,1,xdi);
end

%{
Su
Sv
Suu
Svv
Suv
r
%}

% transfers relevant first and second derivatives:

dS1 = zeros(udim,xdim);      % stored as vector: Su, Sv, Sw
dS2 = zeros(udim,udim,xdim); % stored as matrix: Suu, Svv, ... Svw


for i=1:1:udim
    
    % First derivatives vector:
    
    cs = ones(3,1);
    cs(i) = 2 ;
    
    for xdi=1:1:xdim
    dS1(i,xdi) = dS(cs(1),cs(2),cs(3),xdi);
    end
    
    
    % Second derivatives matrix: diagonal:
    
    cd = ones(3,1);
    cd(i) = 3 ;
    for xdi=1:1:xdim
        dS2(i,i,xdi) = dS(cd(1),cd(2),cd(3),xdi);
    end
    
    
    % Second derivatives matrix: triang. components:
    
    if i <= (udim-1)
    for j =2:1:udim
        
        cs = ones(3,1);
        cs(i) = 2 ;
        cs(j) = 2 ;
        
        for xdi = 1:1:xdim
            dS2(i,j,xdi) = dS(cs(1), cs(2), cs(3), xdi) ;
            dS2(j,i,xdi) = dS(cs(1), cs(2), cs(3), xdi) ;
        end
    end
    end

end




% 2.- Assemblies Jacobian and f vector:

J  = zeros(udim, udim);
f  = zeros(udim,1);

Si = zeros(xdim,1);
Sj = zeros(xdim,1);
Sij= zeros(xdim,1);

for i = 1:1:udim
    
    for di=1:1:xdim
    Si(di,1) = dS1(i,di);
    end   
    
    f(i,1) = dot(r,Si);
    
    for j = 1:1:udim
        
        for di=1:1:xdim
            Sj(di,1)  = dS1(j,di);
            Sij(di,1) = dS2(i,j,di);
        end
        J(i,j) = dot(Si,Sj) + dot(r,Sij); 
    end
end




end

