function [cU] = c2811b_PPcurves (cX, N, p,n,U,P,w, CD, xdim, Xtol, DUtol)

% RETURNS:
% cU(nx): param. coordinates of curve correspondant to cX 
% it's assumed cX is ordered along the curve (either ascendant or descendant)

% INPUTS:
% cX(nx,xdim): points to find parameters
% curve NURBS features

Usize = n+p+1;
cU = zeros(N,1);
ULims = [U(1) U(Usize)];


% 1.- Extracts non-void knot spans and sets trial points to search first u0

[Ntp,tpu] = c2812b_trialU (U, Usize, p, n, P, xdim);



% 2.- Detects interval where it is assumed to be the first point and
% closest trial point that will be used as initial u0:

xt = zeros(xdim,1);
xt(:,1) = cX(1,:);

[u0, Su] = c2813b_FindsUinterval (xt, Ntp,tpu, p,n,U,P,w, CD, xdim);


%{
if round(cX(1,1),4)== 100.0000 && round(cX(1,2),4)==60.0000 && round(cX(1,3),4)==0
%if round(P(1,2),4)==14.4168
    u0
%end
end
%}


% 3.- LOOP ON ALL POINTS: uses the prev. as u0

endF = 1 ; % indicates that estimated ut is at one end, turns to 0 when ut is inside ULims.

for i = 1:1:N
    
    % 3.1.- Target point Xcoords:
    xt(:,1) = cX(i,:);



    % 3.2.- Loops until converges to ut (correspondent to xt):
    
    rsF = 1 ; % re-search flag, = 1 if needs to search u0
    ncF = 1 ; % non-converg. flag = 1 when not converges
    
    while (ncF==1 || rsF==1)
    
        rsF = 0; % assumes initial u0 valid unless finds otherwise
        
        % computes Ucoor:
        [ut, ncF] = c2815b_onePoint (xt, u0, p, n, U, P ,w, ULims, CD, xdim, Xtol, DUtol);


        % 'leaving / arriving' to end points checks:
        if N <= 3 && CD==0
            % here does not do the check
        else
            [cU, ut, u0, endF] = c2816b_ends (cU, ut, u0, endF, ULims, i);
        end
        
        
        %{
        if round(cX(1,1),4)== 100.0000 && round(cX(1,2),4)==60.0000 && round(cX(1,3),4)==0
        if i==3
            ut
            cU
        end
        end
        %}
        

        % control of same sense:
        if i > 2 && N > 3
            
            pDu = (cU(i-1,1) - cU(i-2,1)); % previous increment
            cDu = (ut        - cU(i-1,1)); % current increment

            if pDu~=0 && cDu~=0
            if sign(pDu)~=sign(cDu)
                rsF = 1 ;
            end
            end
        end
        

        % re-searches if no converges or advances in wrong sense:
        if ncF==1 || rsF==1
            [u0, ~] = c2813b_FindsUinterval (xt, Ntp,tpu, p,n,U,P,w, CD, xdim);
        end
    end
    

    % 3.3.- Allocates response and initial u for next point:

    cU(i,1) = ut;
    u0 = ut ;
    
    

    

end

% c2818b_plotDu (Su(1), Su(2), 10, xt,ut, p, n, U, P, w, xdim);

end