function [Ntp,tpu] = c2812b_trialU (U, Usize, p, n, P, xdim)

% RETURNS:
% tpu: initial position of trial points

% span with shortest attached control net segments lenght (cnsL) is divided into p partitions
% the others are divided in a No. partitions proportionally to its cnsL


% 1.- Measures control net (CN) segments length:

CNs = zeros(n-1,1);
x1 = zeros(xdim,1);
x2 = zeros(xdim,1);

for i = 1:1:(n-1)
    
    x1(:,1) = P(i,:);
    x2(:,1) = P(i+1,:);
    CNs(i) = norm(x2-x1);
    
end


% 2.- CN-segments attached to each knot span:
% (by simply division of No. CN-segment by No. knot spans)


% non-void spans:

[nvU, nvUsize] = GEN_18_UnonVoid (U, Usize, p, 1);


% control net (CN) segments attached to each span (nss):

Nsp = nvUsize-1; % No. spans
Nsg = size(P,1)-1; % No. segments of control net
nss = Nsg/Nsp;



% 3.- Control net segments length (cnsL) to each span:

cnsL = zeros(Nsp,1);

conta = 1 ;

anss = 0 ;
for i = 1:1:Nsp
   
    % 3.1.-  CN-segments references for that knot span:
    
    sr1 = floor(conta);     % 1st CNs of that span
    sr2 = floor(conta+nss); % last CNs of that span
    
    % 3.2.- Portion of segments to consider (first and last segments, any
    % intermediate is considered full):
    
    ps1 = (floor(anss)+1) - anss;        % portion to compute of first segment
    ps2 = (anss+nss) - floor(anss+nss) ; % portion to compute of last segment
    
    if i==Nsp
        
       sr2 = Nsg;
       ps2 = 1; 
       
    end
    
    % 3.3.- Computes length:
    
    cnsL(i) = ps1*(CNs(sr1));
    for j = sr1+1:1:sr2

        if j<sr2
            cnsL(i) = cnsL(i) + CNs(j);
        elseif j==sr2
            cnsL(i) = cnsL(i) + ps2*CNs(j);
        end
        
    end
    
    conta = conta + nss ;
    anss = anss + nss ; % accumulated No. segment per span
end


% 4.- Trial points position:
    
% proportion of each cnsL w.r.t. minimum:

pp = zeros(Nsp,1);

for i = 1:1:Nsp
    pp(i) = round(cnsL(i)/min(cnsL),0);
end


% each span is divided intro (p * proportion) sub-segments

tpu = zeros(1,1);

conta = 0 ;

for i = 1:1:Nsp
    
    L = nvU(i+1)-nvU(i);
    
    Nssg = p * pp(i); % No. sub-segments
    
    DL = L/Nssg;
    
    acc = nvU(i);
    for j = 1:1:Nssg
        
        conta = conta+1;
        tpu(conta) = acc;
        acc = acc + DL;
        
    end
end

tpu(conta+1) = nvU(nvUsize);

Ntp = conta+1;


end
