function [ut, ncF] = c2815b_onePoint (xt, u0, p, n, U, P ,w, ULims, CD, xdim, Xtol, DUtol)

% RETURNS:
% ut: param. coordinates for xt
% ncF = 1 if does not converge in Lct No. loops

% INPUTS:
% xt: target point Xcoord's
% u0: initial parameter coordinates
% NURBS features
% Xtol: tolerance in abs value
% DUtol: tolerance in abs value

sF = 0 ; % stop flag:
uu = u0 ;
ct = 0 ;
Lct = 5 ;
ncF = 0 ; % assumes it will converge

while (sF == 0)

    ct = ct+1;
    
    % 1.- Increment of u estimated and updated u:

    Due = c2814b_EstimatesU (uu, xt, p, n, U, P ,w, xdim);
    upr = uu ;
    uu = uu + Due ;

    
    % 2.- Preliminary rounding in case out of limits:
    
    if uu < ULims(1) || ULims(2) < uu
        uu = go2532_RoundToR (uu, DUtol);
    end
    
    
    % 3.- Limits correction:

    if CD == 0
    
        if uu < ULims(1)
            uu = ULims(1);
        elseif ULims(2) < uu
            uu = ULims(2);
        end

    elseif CD == 1
        
        if uu < ULims(1)
            
            uu = ULims(2)-(ULims(1)-uu);
            
        elseif ULims(2) < uu
            
            uu = ULims(1) + (uu-ULims(2));
            
        end
        
    end

    
    % 4.- Computes X-error:

    ux = b2601_BNURBS_Xcoord (uu, U, n, p, n+p+1, w, P, 1,xdim);
    Xerr = norm(ux-xt);
    
    
    % 5.- Stop loop checking:
    
    % 5.1.- Increment of u:
    if abs(upr-uu) < DUtol 
        sF = 1 ;
    end
    
    % 5.2.- Error w.r.t. target:
    if Xerr < Xtol
        sF = 1 ;
    end

    % 6.- No. iteration control:
    if ct == Lct+1
        sF = 1 ;
        ncF = 1 ; % not converges
    end
    
end


% allocates value:
ut = uu;
 
end