function [sU] = s2831b_PPsurf (sX, N, Degree, Ncp, KnotV, Usize, P, w, CD, xdim, Xtol, DUtol)

% RETURNS:
% cU(nx): param. coordinates of curve correspondant to cX 
% it's assumed cX is ordered (as a curve order)

% INPUTS:
% cX(nx,xdim): points to find parameters
% curve NURBS features

udim = 2 ;

[Ew,EP] = GEN_12_ExpandBw (w, P, xdim, udim, Ncp);

sU = zeros(N,udim);
ULims = [...
        KnotV(1,1) KnotV(1,Usize(1));
        KnotV(2,1) KnotV(2,Usize(2))];


% 1.- Extracts non-void knot spans and sets trial points to search first u0

[Nks, Ntp, tpu] = s2832b_trialU (KnotV, Usize, Degree, Ncp, P, xdim);

%{
figure(4)
for k = 1:1:Nks 
    for i = 1:1:Ntp(k)  
        scatter(tpu(k,i,1),tpu(k,i,2),'k'); hold on
    end
end
axis equal
%}


% 2.- Detects interval where it is assumed to be the first point and
% closest trial point that will be used as initial u0:

xt = zeros(xdim,1);
xt(:,1) = sX(1,:);

u0 = z2890b_FindsU (xt, Nks, Ntp, tpu, Degree, Ncp, KnotV, Usize, EP, Ew, CD, udim, xdim);

%{
if round(sX(1,1),4)==30.7180 && round(sX(2,2),4)==42.1163
    u0
end
%}

% 3.- LOOP ON ALL POINTS: uses the prev. as u0

endF = 1 ; % indicates that estimated ut is at one end, turns to 0 when ut is inside ULims.

for i = 1:1:N
 
    % 3.1.- Target point Xcoords:
    xt(:,1) = sX(i,:);

    
    % 3.2.- Initial Xcoor
    x0 = b2601_BNURBS_Xcoord (u0, KnotV, Ncp, Degree, Usize, Ew, EP, udim, xdim);
    
    
    % 3.3.- Checks initial point is already target
    if norm(x0-xt) < Xtol
        ut = u0;
        ncF = 0 ; % already coincident (converged)
        
        % 'leaving / arriving' to end points checks:
        [sU, ut, u0, endF] = s2837b_ends (sU, ut, u0, endF, ULims, i, CD);
        
    else
        ncF = 1 ; % non-converg. flag = 1 when not converges
    end
    

    % 3.4.- Loops until converges to ut (correspondent to xt):
    while (ncF==1)

        % computes Ucoor:
        [ut, ncF] = ...
        s2835b_onePoint (xt, u0, Degree, Ncp, KnotV, Usize, P ,w, EP ,Ew, CD, ULims, xdim, Xtol, DUtol);

        % 'leaving / arriving' to end points checks:
        [sU, ut, u0, endF] = s2837b_ends (sU, ut, u0, endF, ULims, i, CD);

        % re-searches if no converges or advances in wrong sense:
        if ncF==1
            u0 = z2890b_FindsU (xt, Nks, Ntp, tpu, Degree, Ncp, KnotV, Usize, EP, Ew, CD, udim, xdim);
        end

    end

    
    % 3.5.- Allocates response and initial u for next point:
    sU(i,1) = ut(1); sU(i,2) = ut(2);
    u0 = ut ;
 
end


% c2810b_plotDu (Su(1), Su(2), 10, xt,ut, p, n, U, P, w, xdim);


end