function [ut, ncF] = v2865b_onePoint (xt, u0, Degree, Ncp, KnotV, Usize, P ,w, EP ,Ew, CD, ULims, xdim, Xtol, DUtol)

% RETURNS:
% ut: param. coordinates for xt
% ncF = 1 if does not converge in Lct No. loops

% INPUTS:
% xt: target point Xcoord's
% u0: initial parameter coordinates
% NURBS features
% Xtol: tolerance in abs value
% DUtol: tolerance in abs value

Usp = [ ULims(1,2)-ULims(1,1) , ULims(2,2)-ULims(2,1), ULims(3,2)-ULims(3,1)]; % total param. space spans

udim = 3 ;

sF = 0 ; % stop flag:
uu = u0 ;


ct = 0 ; % counter of loops and limit
Lct = 10 ;

ncF = 0 ; % assumes it will converge

while (sF==0)

    ct = ct+1;
    
    % 1.- Increment of u estimated and updated u:

    [Due] = v2864b_EstimatesU (uu, xt, Degree, Ncp, KnotV, Usize, P ,w, EP ,Ew, xdim, Xtol);

    
    upr = uu ;
    uu = uu + Due;

    uu = go2532_RoundToR (uu, DUtol/10);
    
    
    % 2.- Limits correction (here considers closed surface):

    for di = 1:1:udim
        
        if CD(di)==0
            
            if uu(di) < ULims(di,1)
                uu(di) = ULims(di,1);
            elseif ULims(di,2) < uu(di)
                uu(di) = ULims(di,2);
            end 
            
        else
            
            if uu(di) < ULims(di,1)
                
                Du = uu(di) - ULims(di,1);
                uu(di) = mod(Du,1);

            elseif ULims(di,2) < uu(di)
                
                Du = uu(di) - ULims(di,1);
                uu(di) = mod(Du,1);
            end 
        end
    end
    

    % 3.- Computes X-error:
    
    ux = b2601_BNURBS_Xcoord (uu, KnotV, Ncp, Degree, Usize, Ew, EP, udim, xdim);
    Xerr = norm(ux-xt);
    
    
    % 4.- Stop loop checking:
    
    % 4.1.- Increment of u:
    if norm(upr-uu) < DUtol 
        sF = 1 ;
    end
    
    % 4.2.- Error w.r.t. target:
    if Xerr < Xtol
        sF = 1 ;
    end

    % 5.- No. iteration control:
    if ct == Lct+1
        sF = 1 ;
        ncF = 1 ; % not converges
    end
    
end



% allocates value:
ut = uu;

 
    
end

