function [skin] = su2911_Skin (pa, gPatch, sstp, stam, sNmin, ...
                               sNcp, sDegree, sUsize, sKnotV, sEB,sEw, sB,sw, sPL, sXdim, sUdim, ...
                               cstp, ctam, cNmin, contoured, nlo, ncu, Rcu, CS, Curv)

% RETURNS
% skin of surface prepared to be added to the instance

% INPUTS:

% sstp(sUdim):       step for computing preliminary points of surface searching
% stam(udim), sNmin:  desired triangles size and number in surfaces
% sNcp, ..., sUdim: NURBS surface features

% cstp:       step for computing preliminary points of contour curves searching
% ctam, cNmin:  desired triangles size and number in contour curves
% contoured:        1 / 0 for deleting or not point outside contour curves 
% nlo:              No. contour loops, each one with one or more curves
% ncu(nlo), Rcu(nlo): No. curves and references for each loop
% CS(udim): closed surface flag, CS=1 if is closed surface in that parameter direction
% Curv: array with curves structure data, each with: 
%       Curv = struct ('Ncp', 0, 'Degree', 0 , 'Usize' , 0 , 'KnotV', [], 'P',[] , 'w', [])


% 1.- DECLARES SKIN CLASS:

skin = GenSkin;

% U limits of NURBS surface domain
sULims = zeros(sUdim,2); 
for di = 1:1:sUdim
    sULims(di,1) = round(sKnotV(di,1),9);
    sULims(di,2) = round(sKnotV(di,sUsize(di)),9);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% manual modif:

% surface 1
%{
sDegree =   [ 2  3 ];
sNcp =      [ 4  4 ];
sKnotV = [ 0 0 0  0.5  1 1 1 0  ;  0 0 0 0   1 1 1 1  ];
sUsize =    [ 7  8 ];

sB = [  0 -20 -90 ;     0 10 -70 ;    0 30 -70 ;    0 40 -60 ; ...
        10 0 -40 ;    10 10 0 ;   10 30 0 ;   10 40 0 ; ...
        20 0 0 ;    20 10 0 ;   20 30 0 ;   20 40 0 ; ...
        30 0 0 ;    40 10 0 ;   55 30 0 ;   50 40 0 ]; 

sw = [ 0.8 0.5 1 1.2  1 1 1 1   1 1 1 1   1 1 1 1 ];

[sEw,sEB] = GEN_12_ExpandBw (sw, sB,sXdim, sUdim, sNcp);


de = [3 2 3 2];
usz = [8 7 8 7];
kv = [2 1 2 1];

pr = [1 2 3 4 ; 4 8 12 16 ; 16 15 14 13 ; 13 9 5 1];

for cR = 1:1:4

        Curv(cR).Ncp = 4;
        Curv(cR).Degree = de(cR);
        Curv(cR).Usize = usz(cR) ;
        KV = sKnotV(kv(cR),1:usz(cR));
        
        for po =1:1:4
            
            for di = 1:1:3
                Curv(cR).P(po,di) = sB(pr(cR,po),di) ;
            end
            
            Curv(cR).w(po) = sw(pr(cR,po)) ;
        end
        
        
       Curv(cR).conF = 1;
    
       if cR <= 2
           Curv(cR).KnotV = KV;
       else
           KV = GEN_06_ReverseKnotV (KV, 4, de(cR));
           Curv(cR).KnotV = KV;
       end
       
end

Rcu = [1 2 3 4];
ncu = 4 ;
nlo = 1 ;
%}

% surface 2
%{
sDegree =   [ 2  3 ];
sNcp =      [ 5  4 ];
sKnotV = [ 0 0 0  0.333  0.667  1 1 1   ;  0 0 0 0   1 1 1 1  ];
sUsize =    [ 8  8 ];

sB = [  0 0 -10 ;     0 10 -5 ;    0 20 -5 ;    0 30 -10 ; ...
        30 0 -5 ;    26 10 0 ;   23 20 0 ;   20 30 0 ; ...
        60 20 40 ;    50 25 40 ;   40 30 40 ;   20 45 40 ; ...
        60 80 0 ;    50 73 0 ;   40 56 0 ;   20 60 0 ; ... 
        -20 100 -10 ;    -25 80 -5 ;   -30 80 -5 ;   -35 60 -18 ]; 
    
sw = [ 1 1 1 1  1 1 1 1   1 1 1 1   1 1 1 1  1 1 1 1 ];

[sEw,sEB] = GEN_12_ExpandBw (sw, sB,sXdim, sUdim, sNcp);


de = [3 2 3 2];
usz = [8 8 8 8];
kv = [2 1 2 1];

lncp = [4 5 4 5];

pr = [1 2 3 4 0 ; 4 8 12 16 20 ; 20 19 18 17 0 ; 17 13 9 5 1];

for cR = 1:1:4

        Curv(cR).Ncp = lncp(cR);
        Curv(cR).Degree = de(cR);
        Curv(cR).Usize = usz(cR) ;
        KV = sKnotV(kv(cR),1:usz(cR));
        
        for po =1:1:lncp(cR)
            
            for di = 1:1:3
                Curv(cR).P(po,di) = sB(pr(cR,po),di) ;
            end
            
            Curv(cR).w(po) = sw(pr(cR,po)) ;
        end
        
        
       Curv(cR).conF = 1;
    
       if cR <= 2
           Curv(cR).KnotV = KV;
       else
           KV = GEN_06_ReverseKnotV (KV, lncp(cR), de(cR));
           Curv(cR).KnotV = KV;
       end
       
end

Rcu = [1 2 3 4];
ncu = 4 ;
nlo = 1 ;

%}

% surface 3
%{
sDegree =   [ 2  3 ];
sNcp =      [ 5  4 ];
sKnotV = [ 0 0 0  0.333  0.667  1 1 1   ;  0 0 0 0   1 1 1 1  ];
sUsize =    [ 8  8 ];

sB = [  -50 0 0 ;     0 10 0 ;    0 20 0 ;    5 30 0 ; ...
        10 0 0 ;    10 10 0 ;   10 20 0 ;   10 30 0 ; ...
        20 0 0 ;    20 10 0 ;   20 20 0 ;   20 30 0 ; ...
        50 0 0 ;    50 10 0 ;   60 20 0 ;   40 30 0 ; ... 
        190 0 0 ;   100 10 0 ;   100 20 0 ;   100 30 0 ]; 
    
sw = [ 1 1 1 1  1 1 1 1   1 1 1 1   1 1 1 1  1 1 1 1 ];

[sEw,sEB] = GEN_12_ExpandBw (sw, sB,sXdim, sUdim, sNcp);


de = [3 2 3 2];
usz = [8 8 8 8];
kv = [2 1 2 1];

lncp = [4 5 4 5];

pr = [1 2 3 4 0 ; 4 8 12 16 20 ; 20 19 18 17 0 ; 17 13 9 5 1];

for cR = 1:1:4

        Curv(cR).Ncp = lncp(cR);
        Curv(cR).Degree = de(cR);
        Curv(cR).Usize = usz(cR) ;
        KV = sKnotV(kv(cR),1:usz(cR));
        
        for po =1:1:lncp(cR)
            
            for di = 1:1:3
                Curv(cR).P(po,di) = sB(pr(cR,po),di) ;
            end
            
            Curv(cR).w(po) = sw(pr(cR,po)) ;
        end
        
        
       Curv(cR).conF = 1;
    
       if cR <= 2
           Curv(cR).KnotV = KV;
       else
           KV = GEN_06_ReverseKnotV (KV, lncp(cR), de(cR));
           Curv(cR).KnotV = KV;
       end
       
end

Rcu = [1 2 3 4];
ncu = 4 ;
nlo = 1 ;

%}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 2.- COMPUTES POINTS OF CONTOUR LOOPS: 
%  stores in CLx(nlo, np, xdim), arranged as curves sense, that is, to the
%  left is the remaining domain, to the right the trimmed or deactivated.

% {
[CLx, CLxacc, CLu, CLuacc, Nseg1, segL1, Nseg2, segL2] = su2921_contours ...
     (Curv, Rcu, ncu, nlo, ctam, cNmin, sDegree, sNcp, sKnotV, sUsize, sB,sw, sEw,sEB, CS, sXdim,sUdim);
 
figure(88)
plot(CLuacc(:,1), CLuacc(:,2), 'b');hold on
%scatter(CLuacc(:,1), CLuacc(:,2), 'r');
axis equal

figure(89)
plot3(CLxacc(:,1), CLxacc(:,2), CLxacc(:,3), 'b');hold on
%scatter3(CLxacc(:,1), CLxacc(:,2), CLxacc(:,3), 'r');
axis equal
%}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% surface with equal param. represetnation
% {

Du = 0.025 ;
Dv = 0.1 ;

Nsu = 1 / Du+1;
Nsv = 1 / Dv+1;

conta = 0 ;
i = 1 ;
for u = 0:Du:1
j = 1 ;
for v = 0:Dv:1
    
	[X] = b2601_BNURBS_Xcoord ...
   	([u v], sKnotV, sNcp,sDegree,sUsize, sEw,sEB, sUdim,sXdim); 
    
    conta = conta+1;
    for di=1:1:3
        xx(i,j,di) = X(di);
    end

    j = j+1;
end
i=i+1;
end


figure(90)
for i = 1:1:Nsu
    plot3(xx(i,:,1), xx(i,:,2), xx(i,:,3), 'b'); hold on     
end

for j = 1:1:Nsv
    plot3(xx(:,j,1), xx(:,j,2), xx(:,j,3), 'b'); hold on    
end  


axis equal

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 3.- COMPUTES FULL SET OF POINTS INSIDE THE SURFACE:

[sU, sX] = rs3341_Internals ...
    (sDegree,sNcp, sKnotV, sUsize, sB,sw, sEw,sEB, stam, sXdim, sUdim);

figure(89)
%plot3(xr(:,1), xr(:,2), xr(:,3),'b'); axis equal
scatter3(sX(:,1), sX(:,2), sX(:,3),'k'); % hold on
%scatter3(sB(:,1), sB(:,2), sB(:,3),'r'); axis equal

axis equal




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%{
% 3.- COMPUTES ALL POINTS OF THE SURFACE, physical and parameter coord's:

% 3.1.- Computes preliminary collection of points, preP(np,xdim), that
% belong to surface and correspontant to param. space point at
% sstp(udim) step
      
[preP] = GEN_15_PatchP ...
(sstp, sNcp, sDegree, sUsize, sKnotV, sEB, sEw, sXdim, sUdim);
        

% 3.2.- Creates xdim net of rectangles enveloping the surface:
% envR = struct('Lims', [xdim,2] )
        
[envR, neE] = z2941_envR (stam, sNmin, preP, sXdim);



% 3.3.- Stores all rect. corners that have inside any surface point:
        
[cP, ncp] = z2942_closeP (envR, neE, preP, sXdim);




%xx = [3.7766489813959 5.1346019900639 0];
%[uu] = b2801_PPforSetPoints (1, xx, sDegree, sNcp, sKnotV, sUsize, sB ,sw,CS, sXdim, sUdim,0);

% {

% 2.4.- Computes by point projection coordinates on the surface = gross - skin points
% also removes repeated ones:

[sU] = b2801_PPforSetPoints...
(ncp,cP, sDegree, sNcp, sKnotV, sUsize, sB,sw, CS, sXdim, sUdim, 0);

[sU,~] = GEN_16_RemoveRepeatedRows (sU, sUdim, 0.00001);
    


% 2.5.- Computes sX coordinates:

nP = size(sU,1);
sX = zeros(nP,sXdim);
u  = zeros(sUdim,1);

for po = 1:1:nP
    
    u(:,1) = sU(po,:);
    [X] = b2601_BNURBS_Xcoord (u, sKnotV, sNcp, sDegree, sUsize, sEw, sEB, sUdim, sXdim);
    
    sX(po,:) = X(:,1);
end

%}


%{

% 4.- REMOVES FROM sX POINTS COINCIDENT WITH CONTOUR LOOPS:
% then leaves separate contour loops points and internal face loops

% removes internal points coincident with loops, then stores that
% coordiantes in sXr and sUr, which are to be transferred to skin:

ip = zeros(sXdim,1); % internal point
cp = zeros(sXdim,1); % contour  point

nP = size(sU,1);
sXr = zeros(1,sXdim);
sUr = zeros(1,sUdim);

contaP = 0 ;
for ipo = 1:1:nP
    
    ip(:,1) = sX(ipo,:) ;
    
    CO = 0;
    for cpo = 1:1: tnlp
    
        cp(:,1) = CLxacc(cpo,:);
        [CO] = go2542_coincidentP (ip, cp, sXdim, 5);
        if CO==1
            break
        end
    end
    
    if CO==0 
        contaP = contaP + 1 ;
        for di=1:1:sXdim
            sXr(contaP,di) = sX(ipo,di);
        end
        for di=1:1:sUdim
            sUr(contaP,di) = sU(ipo,di);
        end
    end
end

nP = contaP; % updates No. interior points


%}


% 5.- REMOVE POINTS OUTSIDE CONTOUR CURVES (IF ACTIVATED):

b = zeros(sUdim,1);

if contoured == 1
    
    sX = zeros(1,sXdim);
    sU = zeros(1,sUdim);
    
    contaP = 0 ;
    for po = 1:1:nP
        
        for di=1:1:sUdim
            b(di) = round(sUr(po,di),9);
        end

        % check with contour
        [IN] = su2913_INcontours (b, nlo, nPL, CLu, sULims, sUdim);

        % check with segments: only if the point 'b' is aligned with one segment seg1 or seg2:
        if IN ==1
            [IN] = su2916_INsegm (b, Nseg1, segL1, Nseg2, segL2);
        end
        
        
        if IN==1
            
            contaP = contaP + 1 ;
            
            for di = 1:1:sXdim
                sX(contaP,di) = sXr(po,di);
            end
            for di = 1:1:sUdim
                sU(contaP,di) = sUr(po,di);
            end
            
        end
        
    end
    
else
    
    sX = sXr ;
    sU = sUr ; 
    
end




% 6.- ADDITION OF CONTOUR POINTS TO THE SURFACE POINTS:
% note contour points are placed first

sX = [CLxacc; sX];
sU = [CLuacc; sU];

nP = size(sX,1); % updates No. surface points, including the contour also



% 7.- COMPUTES NORMAL TO SURFACE VECTORS (POINTING OUTWARDS THE DOMAIN)

nv  = zeros(nP,sXdim);
ord = ones(sUdim,1);
u   = zeros(sUdim,1);
Su  = zeros(sXdim,1);
Sv  = zeros(sXdim,1);



% computes derivative at first location

for di = 1:1:sUdim
    u(di) = sU(1,di);
end
    
[dS] = b2701_BNURBS_deriv (sDegree, sNcp, sKnotV, sUsize, sB ,sw, ord, u, sXdim, sUdim);
    
for di=1:1:sXdim
    Su(di) = dS(2,1,1,di);
	Sv(di) = dS(1,2,1,di);
end
    
nvi = cross(Su,Sv);
nvi = nvi / norm(nvi);
    
    
% 7.1.- For plane surfaces just copy thefirst value:
if sPL == 1   
    
    for po = 1:1:nP
        for di=1:1:sXdim
            nv(po,di) = nvi(di);
        end
    end 
    
    
% 7.2.- For non-plane computes at all locations:  
else
    for po = 1:1:nP
        
        for di = 1:1:sUdim
            u(di) = sU(po,di);
        end

        [dS] = b2701_BNURBS_deriv (sDegree, sNcp, sKnotV, sUsize, sB ,sw, ord, u, sXdim, sUdim);

        for di=1:1:sXdim
            Su(di) = dS(2,1,1,di);
            Sv(di) = dS(1,2,1,di);
        end

        nvi = cross(Su,Sv);
        nvi = nvi / norm(nvi);
        
        for di=1:1:sXdim
            nv(po,di) = nvi(di);
        end
    end  
    
end





% 8.- CONNECTIVITY MATRIX:
% works in parameter space


figure(88);

scatter(sU(:,1), sU(:,2), 'k'); hold on
scatter(CLu(1,:,1), CLu(1,:,2), 'r');

axis equal;

[conM] = su2912_TriconM (nlo, nPL, CLu, sU, sULims, sUdim);


% 9.- COMPUTES PARAMETR COORD'S OF sX[] IN TERMS OF SOLID PATCH:

% extracts patch NURBS info:

pXdim   = gPatch(pa).Xdim ;
pUdim   = gPatch(pa).Udim ;
pDegree = gPatch(pa).Degree;
pNcp    = gPatch(pa).Ncp;
pKnotV  = gPatch(pa).KnotV;
pUsize  = gPatch(pa).Usize;
pB      = gPatch(pa).P;
pw      = gPatch(pa).w;
pCS     = zeros(pUdim,1);

[sUp] = b2801_PPforSetPoints (nP, sX, pDegree, pNcp, pKnotV, pUsize, pB ,pw, pCS, pXdim, pUdim,0);



% 10.- TRANSFERS INFORMATION TO skin & COMPUTES n AT EACH POINT 
% (n is normal vector pointing outwards)

% 10.1.- dims:

skin.xdim = pXdim ;
skin.udim = pUdim ;


% 10.2.- NURBS features:

skin.mDegree = sDegree;
skin.mKnotV = sKnotV;
skin.mUsize = sUsize;
skin.mNcp   = sNcp;
skin.mB     = sB;
skin.mw     = sw;
skin.sPL    = sPL;
skin.sCD    = CS ;       


% 10.3.- Contour loop curves of the skin:

skin.nlo = nlo ;

skin.ncu = ncu ;

skin.Rcu = Rcu ;


% 10.4.- loops points: 

skin.nPL = nPL ;

mPL(nlo,max(nPL)) = GenPoint;

conta = 0 ;
for lo = 1:1:nlo
for po = 1:1:nPL(lo)
    
    conta = conta + 1 ;
    
    for di=1:1:pXdim
        mPL(lo,po).mXcoor(di,1) = sX(conta,di);
        mPL(lo,po).mXcoorF(di,1) = sX(conta,di);
        mPL(lo,po).mXcoor2(di,1) = sX(conta,di);
        mPL(lo,po).mXdisp(di,1) = 0;
        mPL(lo,po).n = nv(conta,di);  
    end
    
  	for di=1:1:pUdim
        mPL(lo,po).mUcoor(di,1) = sUp(conta,di);
    end 
    
end
end

skin.mPL = mPL;

skin.CLu = CLu; % uCoor of conotur poiints in terms of skin surface param. space



% 10.5.- interior points:

skin.nP = nP ;
mP(nP) = GenPoint;

for po = 1:1:nP
    
    for di=1:1:pXdim
        mP(po).mXcoor(di,1) = sX(po,di);
        mP(po).mXcoorF(di,1) = sX(po,di);
        mP(po).mXcoor2(di,1) = sX(po,di);
        mP(po).mXdisp(di,1) = 0;
        mP(po).n(di,1) = nv(po,di);
    end
    
   	for di=1:1:pUdim
        mP(po).mAcoor(di,1) = sUp(po,di);
    end

    for di=1:1:sUdim
        mP(po).mUcoor(di,1) = sU(po,di);
    end
    
end
skin.mP = mP ;

skin.conM = conM ;





end

