function [CLu] = su2914_CSuLims (CLu, sKnotV, sUsize, nlo, Nlv, CS, sUdim)

% NOTE: CLuacc is removed from inputs and outputs
% [CLu, CLuacc] = su2914_CSuLims (CLu, CLuacc, sKnotV, sUsize, nlo, nPL, CS, sUdim)

% RETURNS:
% CLu: contour points coordinates corrected (if required) if they skipped to the
% other side of a closed surface. It can happen when Ucoor is at the edge
% of parameter direction correspondant to closed surface

% INPUTS:
% CLu: contour points coordinates with coordinates corrected
% sKnotV, sUsize, CS : surface relevant features
% nlo: No. loops of surface
% Nlv(nlo): No. vertexes or point of each loop



% 1.- COMPUTES ENDS OF PARAMETER SPACE IN EACH DIRECTION:

ULims = zeros(sUdim,2);
for di=1:1:sUdim
    ULims(di,1) = round(sKnotV(di,1),6);
    ULims(di,2) = round(sKnotV(di,sUsize(di)),6);
end



% 2.- CHECK DIRECTION WHERE SURFACE IS CLOSED:
% compares points with Ucoor at the ends of parameter space with next
% points, and detect if the latter ones get away from that end or not

for di = 1:1:sUdim
if CS(di)==1 % only if closed surface direction
    
    % in each contour loop checks if there is parameter value at the edge of closed direction
    
    contaT = 0 ;
    
    for lo = 1:1:nlo
    for po = 1:1:Nlv(lo)
            
        contaT = contaT + 1 ;
            
        % actual Ucoor:
    	u = round(CLu(lo,po,di),6) ;
          
        
        % 2.1.- Detects point located at the edge of the parameter space.
        % If that 'u' is in one parameter space end checks it:
        

        if u==ULims(di,1) || u==ULims(di,2)
                 

            % finds points in the vicinity with diffecent u from lims and
            % evaluates where they get away from:
            
            
            % 2.2.- In case those are NEXT points:
            
         	nxuVals = zeros(2,1); % find points
            contaN = 0 ;

            for nxpo = po:1:Nlv(lo)

                nxu = round(CLu(lo,nxpo,di),6) ;
                

                if nxu~=ULims(di,1) && nxu~=ULims(di,2)

                    contaN = contaN+1;
                    nxuVals(contaN) = nxu;

                end

                if contaN==2
                    break
                end
    
            end
            
            if contaN==2 % evaluates
                
                d1 = abs(ULims(di,1) - nxuVals(1));
                d2 = abs(ULims(di,1) - nxuVals(2));

                if d2 > d1 % gets away from Lim1 => u = Lim1

                    cru = ULims(di,1);

                elseif d2 < d1  % gets away from Lim2 => u = Lim2

                    cru = ULims(di,2);

                elseif d2==d1 % here just select which Lim is closer:

                    if ULims(di,2)-nxuVals(1) < nxuVals(1)-ULims(di,1)
                        cru = ULims(di,2);
                    else
                        cru = ULims(di,1);
                    end
                end
 
            end
            
            % 2.3.- In case those are PREV points:
            
            if contaN<2
                
 
                contaN = 0 ;
                for prpo = po:-1:1 % find points

                    pru = round(CLu(lo,prpo,di),6) ;
                    if pru~=ULims(di,1) && pru~=ULims(di,2)

                        contaN = contaN+1;
                        pruVals(contaN) = pru;

                    end

                    if contaN==2
                        break
                    end
                end 
                

                if contaN==2 % evaluates
                
                    d1 = abs(ULims(di,1) - pruVals(1));
                    d2 = abs(ULims(di,1) - pruVals(2));

                    if d2 > d1 % gets away from Lim1 => u = Lim1

                        cru = ULims(di,1);

                    elseif d2 < d1  % gets away from Lim2 => u = Lim2

                        cru = ULims(di,2);

                    elseif d2==d1 % here just select which Lim is closer:

                        if ULims(di,2)-pruVals(1) < pruVals(1)-ULims(di,1)
                            cru = ULims(di,2);
                        else
                            cru = ULims(di,1);
                        end
                    end
 
                end
                
                
            end
            
            
            %{
            % 2.2.- Finds next two points with diffecent u from lims and
            % evaluates where they get away from:
                
            nxuVals = zeros(2,1);
            contaN = 0 ;
            
            for nxpo = po:1:Nlv(lo)

                nxu = round(CLu(lo,nxpo,di),6) ;
                if nxu~=ULims(di,1) && nxu~=ULims(di,2)

                    contaN = contaN+1;
                    nxuVals(contaN) = nxu;

                end

                if contaN==2
                    break
                end
            end
                
            
         	
            
            % in case there are no 2 points with diff u from Ulims
            % starts from the begginig:

            if contaN<2

                contaN = 0 ;
                for nxpo = 1:1:Nlv(lo)

                    nxu = round(CLu(lo,nxpo,di),6) ;
                    if nxu~=ULims(di,1) && nxu~=ULims(di,2)

                        contaN = contaN+1;
                        nxuVals(contaN) = nxu;

                    end

                    if contaN==2
                        break
                    end
                end 
            end
                
            
            % 2.3.- Evaluates which ULims these 2 points get away from.
            % stores corrected 'u' in 'cru'

            d1 = abs(ULims(di,1) - nxuVals(1));
            d2 = abs(ULims(di,1) - nxuVals(2));

            if d2 > d1 % gets away from Lim1 => u = Lim1

                cru = ULims(di,1);

            elseif d2 < d1  % gets away from Lim2 => u = Lim2

                cru = ULims(di,2);

            elseif d2==d1 % here just select which Lim is closer:

                if ULims(di,2)-nxuVals(1) < nxuVals(1)-ULims(di,1)
                    cru = ULims(di,2);
                else
                    cru = ULims(di,1);
                end
            end
            %}
                
            % 2.4.- Allocates 'corrected u' - 'cru' to CLu and CLuacc:

            CLu(lo,po,di) = cru ;  

                
        end  
    end
    end
end   
end


end

