function [Ngp, GP] = su2917_GPskin (skin, BEim, NGPe, Spatches, eudim, xdim, pudim, Xtol, DUtol)

% RETURNS:  GP's info for skin

% INPUTS:
% NGPe: No. integration points per triangle
% BEim: integration method 'NC' / 'GC'


% 1.- EXTRACTS INFO FROM SKIN & PATCH:

eKnotV  = skin.mKnotV ;
eNcp    = skin.mNcp ;
eDegree = skin.mDegree ;
eUsize  = skin.mUsize ;
ew      = skin.mw ;
eP      = skin.mP ;

TVcon   = skin.mTVcon;
Uv      = skin.mUv;
couR    = skin.mCou;



% 2.- PRELIMINARIES:

% 2.1.- Declaration of object:
GP(2,1) = IntgPoint; % 2 will increase up to Ngp


% 2.2.- Parent coord's and weights:  
if strcmp(BEim,'GC')==1    
    [GPpc,GPw,~] = ip2303_GP_tri (NGPe);    
elseif strcmp(BEim,'NC')==1
        
end

Nne = 3 ;

% 2.3.- Patches features:

[pR, pDegree,  pNcp,  pKnotV,  pUsize,  pP,  pw,  pCD , pEw,  pEP, pULims, ...
spR, spDegree, spNcp, spKnotV, spUsize, spP, spw, spCD, spEw, spEP, spULims] = ...
so2938_patchNURBS (skin, Spatches, xdim);

nAN = Spatches(pR).mRefin(1).nAN;
mA = Spatches(pR).mRefin(1).mA;

if couR > 0
    snAN = Spatches(spR).mRefin(1).nAN;
    smA =  Spatches(spR).mRefin(1).mA;
end

% 3.- GROUPS GP'S TO HAVE NO REPETITION:

% Provides elements connection.
% Provides GP's coord's.

[NEg, GEcon, Ngp, Ugp, Xgp, Agp, AgpS, Wgp] = su2924_GPcoor ...
(skin, Spatches, Nne, NGPe, GPpc, GPw, eudim, pudim, xdim, Xtol, DUtol);

%{
figure(1)
scatter(Ugp(:,1), Ugp(:,2),'r','x'); hold on
axis equal

figure(2)
scatter3(Xgp(:,1), Xgp(:,2), Xgp(:,3),'k','x'); hold on
axis equal

figure(3)
scatter3(Agp(:,1), Agp(:,2), Agp(:,3),'b','x'); hold on
axis equal

if GG==0
end
%}


% 4.- COMPUTES OTHER VALUES and ALLOCATES:

Ucoor = zeros(eudim,1);
uNods = zeros(Nne,eudim);

for g = 1:1:Ngp
    
    % 4.1.- Extracts coordinates:
    
    Acoor(:,1)  = Agp(g,:);
    if couR > 0
        AcoorS(:,1) = AgpS(g,:);
    end
    Ucoor(:,1)  = Ugp(g,:);
    Xcoor(:,1)  = Xgp(g,:);
    
    
    % 4.2.- Jacobian_2: computes summation of jacobians_2 of attached elements
    % Weight: sums contributions

    detJ2 = 0; 
    w = 0 ;
    for e = 1:1:NEg(g,1)
        
        eR = GEcon(g,e);
        
        for i = 1:1:eudim
            nR = TVcon(eR,i);
            uNods(i,:) = Uv(nR,:);
        end
        
        tmp = j2313_Jac2_elem (Nne, eudim, uNods, 0);
        detJ2 = detJ2 + tmp; 
        w = w + Wgp(g,e);
    end
    
    GP(g,1).dJ2 = detJ2;
    GP(g,1).mw  = w ; 
    
    
    % 4.3.- Jacobian_1:
        
 	[J1, detJ1] = j2315_J1_Bspline (Ucoor, eKnotV, eNcp, eDegree, eUsize, eP, ew, eudim, xdim);
    
    detJ1 = detJ1 * NEg(g,1);   
    J1    = J1    * NEg(g,1);
        
  	GP(g,1).dJ1 = detJ1 ;
    GP(g,1).J1  = J1 ;
    
    
    % 4.4.- Coordinates:
    
   	GP(g,1).mAcoor = Acoor;
   	if couR > 0
     	GP(g,1).mAcoorS = AcoorS;
    end
  	GP(g,1).mUcoor = Ucoor;
  	GP(g,1).mXcoor = Xcoor;
   	GP(g,1).mXcoorF = Xcoor;
   	GP(g,1).mXdisp = zeros(xdim,1);
 
    
    % 4.5.- Icp's from patch and Bf's values:
    
    [Nicp, Ricp, Nabs, Rrel, dRpa] = su2926_IcpBf (Acoor, pNcp, pKnotV, pUsize, nAN, mA, pDegree, pudim);
    
  	GP(g,1).mNicp  = Nicp;
	GP(g,1).mRicpA = Ricp;
    GP(g,1).Nabs  = Nabs;
    GP(g,1).Rrel  = Rrel;
    GP(g,1).dR_pa = dRpa;
    
    
    if couR > 0

        [Nicp, Ricp, Nabs, Rrel, dRpa] = su2926_IcpBf (Acoor, spNcp, spKnotV, spUsize, snAN, smA, spDegree, pudim);

        GP(g,1).mNicpS  = Nicp;
        GP(g,1).mRicpAS = Ricp;
        GP(g,1).NabsS  = Nabs;
        GP(g,1).RrelS  = Rrel;
        GP(g,1).dR_paS = dRpa;
        
    end
      
end


end