function [CLx, CLxacc, CLus, CLu, CLuacc, CLn, Nseg1, segL1, Nseg2, segL2] = su2921_contours ...
    (Curv, Rcu, ncu, nlo, ctam, cNmin, sDegree, sNcp, sKnotV, sUsize, sB,sw, sEw,sEB, CS, sXdim, sUdim, tolR)

% RETURNS:
% physical coord's of contour curves: CLx(nlo,np,xdim), CLxacc(nptotal,xdim)
% parameter coord's of contour curves: CLu(nlo,np,udim), CLuacc(nptotal,udim)
% CLn: no. points in each contour line (does not count for the final one, that is the starting of the next line !)
% Nsegi, segLi: horiz. and vert. contour curves segments in surface param. space


% INPUTS:

nPL = zeros(nlo,1);

CLus = zeros(nlo,1,2,sUdim); % 1 is No. curves of the loop, and 2 No. point of the curve ('2' does not consider the last as it is equal to the first of the next curve)
CLu = zeros(nlo,2,sUdim);   % 2 is No. points of the loop
CLuacc  = zeros(3,sUdim);   % 3 is total No. points of all loops

CLx = zeros(nlo,2,sXdim); 
CLxacc  = zeros(3,sXdim); 



CLn = zeros(nlo,1);

acc2 = 0 ;


% 1.- COMPUTATION OF COORDINATES:

for lo = 1:1:nlo
    
    % Computation for each curve points attending to ctam & cNmin:
    
    Lnp = 0 ; % loop No. points
    acc1 = 0;
    
    for c = 1:1:ncu(lo)
        
        % 1.1.- Extracts curve NURBS features:
        cR      = Rcu(lo,c) ;
        cNcp    = Curv(cR).Ncp ;
        cDegree = Curv(cR).Degree ;
        cUsize  = Curv(cR).Usize ;
        cKnotV  = Curv(cR).KnotV ;
        cB      = Curv(cR).P ;
        cw      = Curv(cR).w ;
        cXdim   = sXdim;
        cUdim   = 1 ;
        conF    = Curv(cR).conF ; % contour flag = 1 if curve lies in the contour of NURBS surface
        
        % knot vector has to be row vector:
        if size(cKnotV,2)==1
            cKnotV = cKnotV';
        end

        % 1.2.- Physical and param. coord's (in terms of curve param. space):
        
        [cU, cX, ~] = cu2901_Cpartition ...
         (ctam(1), cNmin, cNcp, cDegree, cUsize, cKnotV, cB, cw, cXdim, cUdim, tolR);
           

        np = size(cX,1);
     
      	% 1.3.- Param. coord's (in terms of surface param. space):

        if conF == 1

            [sU] = su2923_UconCurves ...
            (np, cX,cU, sDegree, sNcp, sKnotV, sUsize, sB ,sw, sEw,sEB, CS, sXdim, sUdim);

        elseif conF == 0

            [sU] = b2801_PPforSetPoints ...
            (np, cX, sDegree, sNcp, sKnotV, sUsize, sB ,sw,CS, sXdim, sUdim,0);
        end

        
    	% stores the loop curves points. Note it does not store the last point,
        % because it corresponds to the first point of the next contour curve:
        
        np = size(cX,1)-1;
        cX = cX(1:np,:);
        Lnp  = Lnp + np;
        
        CLn(lo,c) = np ;

        % 1.4.- Allocates to arrays:

        for po = 1:1:np
            
            acc1 = acc1 + 1 ;
            acc2 = acc2 + 1 ;
            
            for di=1:1:sXdim
                CLx(lo,acc1,di) = cX(po,di);
                CLxacc(acc2,di) = cX(po,di);
            end
            for di=1:1:sUdim
                CLus(lo,c,po,di) = sU(po,di);
                CLu(lo,acc1,di) = sU(po,di);
                CLuacc(acc2,di) = sU(po,di);
            end
        end 

    end

           
	% No. points of the loop:
   	nPL(lo,1) = Lnp ;
   
end


   
% 2.- CLOSED SURFACE CHECK:

if sum(CS) > 0
[CLu, CLuacc] = su2914_CSuLims (CLu, CLuacc, sKnotV, sUsize, nlo, nPL, CS, sUdim);
end



% 3.- STORES IN segL() THOSE HOR. OR VERT. CONTOURS IN SKIN PARAM. SPACE:

conta = 0 ; 

Nseg1 = 0 ; % segments paral. to direction 1, dir 2 stats , dir 1 varies
segL1 = zeros(1,sUdim,2);

Nseg2 = 0 ; % segments paral. to direction 2, dir 1 stats , dir 2 varies
segL2 = zeros(1,sUdim,2);

for lo = 1:1:nlo
for po = 1:1:nPL(lo)
    
    conta = conta+1;

    if po > 1
        [Nseg1, segL1] = su2915_segm (Nseg1, segL1, CLu, lo, po, 2, 1);
        [Nseg2, segL2] = su2915_segm (Nseg2, segL2, CLu, lo, po, 1, 2);
    end
        
end
end


end

