function [sU] = su2923_UconCurves ...
         (nv, cX,cU, sDegree, sNcp, sKnotV, sUsize, sB ,sw, sEw,sEB, CS, sXdim, sUdim)

% RETURNS:
% sU(np,sUdim): param. coord's (in terms of surface) of a contour curve
% that lies at the surface edge

% INPUTS:
% cX(np, sXdim): curve physical coord's
% cU(np, sUdim): curve paramet. coord's
% nv:            No. vertexes


sU = zeros(nv, sUdim);

% check if one of the curve end points coincides with one corner
% in that case => copy param. from curve to surface


% 1.- FINDS WHICH CORNERS COINCIDES WITH ANY CURVE END POINTS:

% surface start/end control point references in sEB
cpr  = [ 1 sNcp(1) ; 1 sNcp(2)]; 

curE = zeros(2,sXdim) ; % curve end points

for di=1:1:sXdim
    curE(1,di) = cX(1,di) ;
    curE(2,di) = cX(nv,di);
end


contaC = 0 ;
refC = zeros(1,1);

for po=1:1:2
    
    % curve Xcoor
    
	curX = zeros(sXdim,1);
    for di=1:1:sXdim
        curX(di,1) = curE(po,di);
    end
    
    
    % finds which corner of surface coincides with:
    
    for i = 1:1:2
    for j = 1:1:2

        corX = zeros(sXdim,1);
        for di=1:1:sXdim
            corX(di) = sEB(cpr(1,i),cpr(2,j),di);
        end
        
        [CO] = go2542_coincidentP (curX, corX, sXdim, 6);

        if CO==1
            contaC = contaC + 1 ;
            refC(contaC,1) = i ;
            refC(contaC,2) = j ;
            break
        end

    end
    if CO==1
       break 
    end
    end
 
end

% contaC: No. corner which coincides with curve end points
% refC(contaC,2): corners refs



% 2.- CASE TWO CORNERS ARE NOT COINCIDENT (open surface)

% just copy directly parameters form curve parameters

if (refC(1,1)~=refC(2,1) || refC(1,2)~=refC(2,2))
    
    
    % distinguishes 8 cases and extracts:
    % dv , df : direction that varies annd fixed
    % vdf:      value of direction fixed
    % sdF:      same direction flag = 1 / -1
    
    
    % 2.1.- SAME DIRECTION AS SURFACE:
    
    if (refC(1,1) == 1 && refC(1,2) == 1 ) && (refC(2,1) == 2 && refC(2,2) == 1 ) % same as horiz-btm
        dv = 1 ; % direction that varies
        df = 2 ; % direction fixed
        vdf = sKnotV(df,1); % value of fixed direction
        sdF = 1 ;
    end
    
  	if (refC(1,1) == 1 && refC(1,2) == 2 ) && (refC(2,1) == 2 && refC(2,2) == 2 ) % same as horiz-top
        dv = 1 ; % direction that varies
        df = 2 ; % direction fixed
        vdf = sKnotV(df,sUsize(df)); % value of fixed direction
        sdF = 1 ;
    end
    
  	if (refC(1,1) == 1 && refC(1,2) == 1 ) && (refC(2,1) == 1 && refC(2,2) == 2 ) % same as vert-left
        dv = 2 ; % direction that varies
        df = 1 ; % direction fixed
        vdf = sKnotV(df,1); % value of fixed direction
        sdF = 1 ;
    end
    
  	if (refC(1,1) == 2 && refC(1,2) == 1 ) && (refC(2,1) == 2 && refC(2,2) == 2 ) % same as vert-right
        dv = 2 ; % direction that varies
        df = 1 ; % direction fixed
        vdf = sKnotV(df,sUsize(df)); % value of fixed direction
        sdF = 1 ;
    end
   
    
    
 	% 2.2.- OPPOSITE DIRECTION AS SURFACE:
    
    if (refC(1,1) == 2 && refC(1,2) == 1 ) && (refC(2,1) == 1 && refC(2,2) == 1 ) % opp. as horiz-btm
        dv = 1 ; % direction that varies
        df = 2 ; % direction fixed
        vdf = sKnotV(df,1); % value of fixed direction
        sdF = -1 ;
    end
    
  	if (refC(1,1) == 2 && refC(1,2) == 2 ) && (refC(2,1) == 1 && refC(2,2) == 2 ) % opp. as horiz-top
        dv = 1 ; % direction that varies
        df = 2 ; % direction fixed
        vdf = sKnotV(df,sUsize(df)); % value of fixed direction
        sdF = -1 ;
    end
    
  	if (refC(1,1) == 1 && refC(1,2) == 2 ) && (refC(2,1) == 1 && refC(2,2) == 1 ) % opp. as vert-left
        dv = 2 ; % direction that varies
        df = 1 ; % direction fixed
        vdf = sKnotV(df,1); % value of fixed direction
        sdF = -1 ;
    end
    
  	if (refC(1,1) == 2 && refC(1,2) == 2 ) && (refC(2,1) == 2 && refC(2,2) == 1 ) % opp. as vert-right
        dv = 2 ; % direction that varies
        df = 1 ; % direction fixed
        vdf = sKnotV(df,sUsize(df)); % value of fixed direction
        sdF = -1 ;
    end


    % 2.3.- Copies parameters from curve to surface:
    

    if sdF==-1
        [cU] = GEN_45_reverseDISvector (cU, nv);
         [cU] = GEN_40_invertRowsOrder (cU);
    end
    

    for po=1:1:nv
        sU(po,dv) = cU(po);
      	sU(po,df) = vdf;
    end

end


% 3.- CORNERS ARE SAME (IT IS CLOSED SURFACE)

if (refC(1,1)==refC(2,1) && refC(1,2)==refC(2,2))
    

 	% curve first and second point Xcoor
    
	curX1 = zeros(sXdim,1);
    curX2 = zeros(sXdim,1);
    
    for di=1:1:sXdim
        curX1(di,1) = cX(1,di);
        curX2(di,1) = cX(2,di);
    end
    
    
    % compares to sides 1 and 2:
    
  	corX11 = zeros(sXdim,1);
  	corX11(:,1) = sEB(cpr(1,1),cpr(2,1),:);
    [CO1] = go2542_coincidentP (curX1, corX11, sXdim, 4);
    
	corX22 = zeros(sXdim,1);
  	corX22(:,1) = sEB(cpr(1,2),cpr(2,2),:);
    [CO2] = go2542_coincidentP (curX1, corX22, sXdim, 4);
    
    % selectes directions
    
    if CS(1)==1 
        
        dv = 1 ; % direction that varies
        df = 2 ; % direction fixed
        
        if CO1==1 % ita = 0
            
            vdf = sKnotV(df,1); % value of fixed direction
            
        elseif CO2==1 % ita= 1
            
            vdf = sKnotV(df,sUsize(df)); % value of fixed direction
            
        end
 
    elseif CS(2)==1
        
      	dv = 2 ; % direction that varies
        df = 1 ; % direction fixed
        
        if CO1==1 % xi = 0
            
            vdf = sKnotV(df,1); % value of fixed direction
            
        elseif CO2==1 % xi= 1
            
            vdf = sKnotV(df,sUsize(df)); % value of fixed direction
            
        end
        
    end
    

    % selects sense: computes position of second point in S and compares
    % with second point of curve
    
    Du = cU(nv)-cU(nv-1);

    
    sUcoor1 = zeros(sUdim,1);
    sUcoor2 = zeros(sUdim,1);
    
    sUcoor1(df) = vdf;
    sUcoor1(dv) = sKnotV(dv,1)+Du;
    
	sUcoor2(df) = vdf;
    sUcoor2(dv) = sKnotV(dv,sUsize(dv))-Du;
    
    
    [X1] = b2601_BNURBS_Xcoord (sUcoor1, sKnotV, sNcp,sDegree,sUsize, sEw,sEB, sUdim,sXdim);
    [X2] = b2601_BNURBS_Xcoord (sUcoor2, sKnotV, sNcp,sDegree,sUsize, sEw,sEB, sUdim,sXdim);
    
    
    nrm1 = norm(X1-curX2);
    nrm2 = norm(X2-curX2);
    
    
    
    if nrm1 < nrm2
        
        sdF = 1 ;
        
    else
        
        sdF = -1 ;
        
    end
    
    
	% copies parameters from curve to surface:
    
    if sdF==-1
        [cU] = GEN_45_reverseDISvector (cU, nv);
        [cU] = GEN_40_invertRowsOrder (cU);
    end
    
    for po=1:1:nv
        sU(po,dv) = cU(po);
      	sU(po,df) = vdf;
    end

end
    



end

