function [NEg, GEcon, Ngp, Ugp, Xgp, Agp, AgpS, Wgp] = ...
    su2924_GPcoor (skin, Spatches, Nne, NGPe, GPpc, GPw, eudim, pudim, xdim, Xtol, DUtol)

% RETURNS:
% NEg(Ngp,1), GEcon(Ngp,...):No. elem's per GP and their refs
% Ngp: No. GP's
% Ugp(Ngp,eudim): GP's Ucoord (in terms of surface param. space)
% Wgp(Ngp, NEg)

% note that group those GP's that coincide.
% Only adds if coord's are new w.r.t. already stored.


% skin NURBS features:

eKnotV  = skin.mKnotV ;
eNcp    = skin.mNcp ;
eDegree = skin.mDegree ;
eUsize  = skin.mUsize ;
ew      = skin.mw ;
eP      = skin.mP ;

[eEw, eEP] = GEN_12_ExpandBw (ew, eP,xdim, eudim, eNcp);

TVcon = skin.mTVcon;

Av = skin.mAv;
Uv = skin.mUv;

couR = skin.mCou;
AvS  = skin.mAvS;

NE  = skin.mNE ; % No. triangles

% patches NURBS features:

[~, pDegree,  pNcp,  pKnotV,  pUsize,  pP,  pw,  pCD , pEw,  pEP, pULims, ...
~, spDegree, spNcp, spKnotV, spUsize, spP, spw, spCD, spEw, spEP, spULims] = ...
so2938_patchNURBS (skin, Spatches, xdim);


Ngp = 0 ;
NEg = zeros(2,1); % No. elems to each GP
GEcon = zeros(2,1); % refs of elems attached to each GP

Ugp   = zeros(2,eudim);
Agp   = zeros(2,pudim);
AgpS  = zeros(2,pudim);
Xgp   = zeros(2,xdim);

Wgp   = zeros(2,1);

Uextg = zeros(eudim,1);

uParent = zeros(eudim,1);
uNods = zeros(Nne,eudim); % nodes param. coordinates (in skin param. space)



for e = 1:1:NE
    
    % 1.- Triangle nodes Ucoord's and triangle Amid-point
    
    Amid = zeros(1,pudim);
    AmidS = zeros(1,pudim);
    
    for i = 1:1:Nne
        
        nR = TVcon(e,i); 
      	uNods(i,:) = Uv(nR,:);

        Amid(1,:) = Amid(1,:) + Av(nR,:)/3;
        
     	if couR > 0
            AmidS(1,:) = AmidS(1,:) + AvS(nR,:)/3;
     	end
        
    end
    
        
    % 2.- For first element just adds GPs:
    
    if e==1
    for g = 1:1:NGPe
       
        uParent(:,1) = GPpc(g,:);
        Ucoor = m2321_parentMapping (uParent, uNods, eudim, Nne);
        
        [Xcoor, Acoor, AcoorS] = su2925_XAcoords ...
        (Ucoor, eKnotV, eNcp, eDegree, eUsize, eEw,eEP,...
        Amid,   pDegree,  pNcp,  pKnotV,  pUsize,  pP , pw, pEP , pEw,  pCD,  pULims,...
        AmidS, spDegree, spNcp, spKnotV, spUsize, spP ,spw, spEP ,spEw, spCD, spULims, ...
        couR, eudim, xdim, Xtol, DUtol);
        
        Ngp = Ngp + 1 ;
        
        Ugp(Ngp,:) = Ucoor(:,1);
        Agp(Ngp,:) = Acoor(1,:);
        Xgp(Ngp,:) = Xcoor(:,1); 
        if couR > 0
            AgpS(Ngp,:) = AcoorS(1,:);
        end
        
        NEg(Ngp,1) = 1 ;
        GEcon(Ngp, NEg(Ngp,1)) = e;
        
        Wgp(Ngp, NEg(Ngp,1)) = GPw(g,1);
    end
    end
    
    
    
    
    % 3.- Rest of elements:
    
    nd = 2 ;
    
    if e > 1
    for g = 1:1:NGPe
        
        % candidate coordinates
        uParent(:,1) = GPpc(g,:);
        Ucoor = m2321_parentMapping (uParent, uNods, eudim, Nne);
        
        
        % searches in stored GP's:
        
        CO = 0 ;
        for sg = 1:1:Ngp
            
            Uextg(:,1) = Ugp(sg,:);
            
            CO = go2542_coincidentP (Ucoor, Uextg, eudim, nd);
            if CO == 1
          	break
            end
        end
        

        if CO==0
            
            % if new point => adds to arrays
            
          	[Xcoor, Acoor, AcoorS] = su2925_XAcoords ...
            (Ucoor, eKnotV, eNcp, eDegree, eUsize, eEw,eEP,...
            Amid,  pDegree,  pNcp,  pKnotV,  pUsize,  pP , pw, pEP , pEw,  pCD,  pULims,...
            AmidS,spDegree, spNcp, spKnotV, spUsize, spP ,spw, spEP ,spEw, spCD, spULims, ...
            couR, eudim, xdim, Xtol, DUtol);
            
            Ngp = Ngp + 1 ;
        
           	Ugp(Ngp,:) = Ucoor(:,1);
            Agp(Ngp,:) = Acoor(1,:);
            Xgp(Ngp,:) = Xcoor(:,1); 
            if couR > 0
                AgpS(Ngp,:) = AcoorS(1,:);
            end
            
            NEg(Ngp,1) = 1 ;
            
            GEcon(Ngp,NEg(Ngp,1)) = e;
            Wgp(Ngp,NEg(Ngp,1)) = GPw(g,1);
            
            
        elseif CO==1
            
            % if extg => updates elements relationship
            
            NEg(sg,1) = NEg(sg,1)+1 ;
            
            GEcon(sg,NEg(sg,1)) = e;
            Wgp(sg,NEg(sg,1)) = GPw(g,1);
        end
        
    end
    end
     
end



end

