function [Spatches] = so2931_gSkins (Npa, nGF, gFace, Spatches, tolRc, tolRs, tolD, Nvx, R, ndig, wbF, DerC, DUtol, pRx)

% RETURNS:
% mGskin(nGF): gross skins objects to allocate to Spatches.


% pp tolerance in physical space:

Xtol = pRx * R / 100;


% LOOPS ON PATCHES :


% for pa = 2:1:2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for pa = 1:1:Npa  
    
    % patch features:
    xdim = 3;
    pDegree = Spatches(pa,1).INS(1,1).mDegree;
    pNcp    = Spatches(pa,1).INS(1,1).mNcp;
    pKnotV	= Spatches(pa,1).INS(1,1).mKnotV;
    pUsize	= Spatches(pa,1).INS(1,1).mUsize;
    pP      = Spatches(pa,1).INS(1,1).mP;
    pw      = Spatches(pa,1).INS(1,1).mw;
    pCD     = zeros(xdim,1); % assumes no closed solids, i.e. like a torus


    % 1.- Basic varaibles:
    
    Nsf = nGF(pa);
    skinG(Nsf,1) = cGskin;
    
    
    % 2.- Extracts to 'sf' of each patch using gFace and also allocates
    % some skins features:
    
    sf(Nsf,1) = struct('Ncp',[], 'Degree',[], 'Usize',[], 'KnotV',[], 'EP',[], 'Ew',[], 'P',[],'w',[], 'CS',[0 0]);
    
    for s = 1:1:6
        
        % surface data

        sf(s).Ncp = gFace(pa).mSur(s).Ncp ;
        sf(s).Degree = gFace(pa).mSur(s).Degree ;
      	sf(s).Usize = gFace(pa).mSur(s).Usize ;
        sf(s).KnotV = gFace(pa).mSur(s).KnotV ;  
        
        P = gFace(pa).mSur(s).P;
        w = gFace(pa).mSur(s).w ;

        [Ew,EP] = GEN_12_ExpandBw (w, P,xdim, 2, sf(s).Ncp);
        
        sf(s).P  = P;
        sf(s).w  = w ;
        sf(s).EP = EP ;
        sf(s).Ew = Ew ;
        
        % skins allocation:
        %{
        skinG(s,1).mDegree = sf(s).Degree ;
        skinG(s,1).mKnotV = sf(s).KnotV ;
        skinG(s,1).mUsize = sf(s).Usize ;
        skinG(s,1).mNcp = sf(s).Ncp ;
        skinG(s,1).mP = sf(s).P ;
        skinG(s,1).mw = sf(s).w ;
        skinG(s,1).mCS = gFace(pa).mSur(s).CS ;
        %}
    end

    
    % 3.- Computes the six skins No. elements (triangles), conn. matrix
    % and vertexes coordinates
    
    [QITs] = ts3330_QIT ...
    (tolRc, tolRs, tolD, Nvx, R, Nsf, sf, xdim, ndig, 0, 0, 0, wbF, DerC);
        

    % 4.- Allocates to skins and then those skins to the patch:
    
    for s = 1:1:6
        
     	skinG(s,1).mNE = QITs(s).mNE ;
        skinG(s,1).mNV = size(QITs(s).mXv,1) ;
        skinG(s,1).mTVcon = QITs(s).mTVcon ;
        
        skinG(s,1).mLVcon = QITs(s).mLVcon ;
        
        skinG(s,1).mUv = QITs(s).mUv ;
        skinG(s,1).mXv = QITs(s).mXv ;

        skinG(s,1).mNlo = QITs(s).mNlo ;
        skinG(s,1).mNlv = QITs(s).mNlv ;
        

        % Computes coord's in terms of solid param. space:
        
        sX = skinG(s,1).mXv ;
        NV = skinG(s,1).mNV ;
        
        [sA] = v2861b_PPvol (sX, NV, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, xdim, Xtol, DUtol);
        skinG(s,1).mAv = sA ;
        
        
        % plots skins
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %{
        
        %if pa==1
        
            fr =  22  ;
            Cv = QITs(s).mXv ;

            LVcon = QITs(s).mLVcon ;
            TVcon = QITs(s).mTVcon ;

            NLI = size(LVcon,1);
            NE = QITs(s).mNE ;

             pt1366_pTriLinMesh_nD...
            (Cv, NLI, LVcon, TVcon, xdim, fr, 'b', 1, 0, NV, 0, 0, NE, 0, 0, 0, 0, 0, 0);
    
        if gg==2
        end
        
        %end
    
        %}
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    end

	Spatches(pa,1).mGskin = skinG ;
    Spatches(pa,1).Ngsk = Nsf;
    
    clearvars skinG sf ;
    
end




end

