function [skin] = so2933_AmendNvtx (skin, s1, s2, ncu, SCcon, Ncv, cLXv, pudim, xdim, nd)

% RETURNS:
% skin with amended vertexes normals

% INPUTS:
% skin itself
% s1, s2: surfaces to compare (and search equal vertexes)

% 1.- EXTRACTS DATA:

TVcon1 = skin(s1).mTVcon ;
NV1 = skin(s1).mNV ;
VTcon1 = skin(s1).mVTcon ;

Xv1 = skin(s1).mXv  ;
XNe1 = skin(s1).mXNe ;

Av1 = skin(s1).mAv  ;
ANe1 = skin(s1).mANe ;


TVcon2 = skin(s2).mTVcon ;
NV2 = skin(s2).mNV ;
VTcon2 = skin(s2).mVTcon ;

Xv2 = skin(s2).mXv  ;
XNe2 = skin(s2).mXNe ;

Av2 = skin(s2).mAv  ;
ANe2 = skin(s2).mANe ;
   


% 2.- SEARCHES REPEATED CURVES: recall they can only coincide in the outer loop (loop number 1)
        
cR = zeros(1,1);
Npc = 0; % No. pairs of curves

for c1 = 1:1:ncu(s1,1)
            
	cR1 = SCcon(s1,1,c1);
    
    for c2 = 1:1:ncu(s2,1)
            
        cR2 = SCcon(s2,1,c2);
        
      	if cR1==-cR2
            
            Npc = Npc+1;
            cR(Npc,1) = abs(cR1);
            break
            
        end
        
    end
           
end
      

% 3.- AMENDS VERTEXES OF CURVES:

% There must be 2 vtx with same coordinates as cR vtx, finds them
% and their triangles, then computes weighted average

xc = zeros(xdim,1);

for pr = 1:1:Npc % loop on pairs of curves

    for vc = 1:1:Ncv(cR(pr))
            
        xc(:,1) = cLXv(cR(pr),vc,:);
            
        % 3.1.- Stores vtx refs of the surface with same coord's:
            
        vr = zeros(2,1);
    	xs = zeros(xdim,1);
            
        for vs = 1:1:NV1   
            xs(:,1) = Xv1(vs,:);  
            CO = go2542_coincidentP (xc, xs, xdim, nd);
            if CO == 1
             	vr(1) = vs ;
                break
            end
        end
            
        if s1==s2
            o = vs+1;
        else
            o = 1;
        end
        
        for vs = o:1:NV2    
            xs(:,1) = Xv2(vs,:);  
            CO = go2542_coincidentP (xc, xs, xdim, nd);
            if CO == 1
             	vr(2) = vs ;
                break
            end
        end
        
            
        % 3.2.- Triangles ref's of both vertexes:
            
        Nt1 = 0;
        Nt2 = 0;
        tR1 = zeros(1,1);
        tR2 = zeros(1,1);
            
        for i = 1:1:size(VTcon1,2)
            if VTcon1(vr(1),i)>0
                Nt1 = Nt1+1;
                tR1(Nt1,1) = VTcon1(vr(1),i);
            end
        	if VTcon2(vr(2),i)>0
            	Nt2 = Nt2+1;
            	tR2(Nt2,1) = VTcon2(vr(2),i);
        	end
        end


        
    	% 3.3.- Recomputes normals:
            
    	NA = zeros(pudim,1);
     	NX = zeros(xdim,1);
            
        Aac = 0 ;
      	Axc = 0;
            
        for i = 1:1:Nt1

            [~,~,Aa] = i0103_nTriang (TVcon1, tR1(i), Av1, pudim);
        	[~,~,Ax] = i0103_nTriang (TVcon1, tR1(i), Xv1, xdim);
                
            Aac = Aac + Aa;
        	Axc = Axc + Ax;
                
            for di = 1:1:3
             	NA(di,1) = NA(di,1) + Aa*ANe1(tR1(i),di) ;
              	NX(di,1) = NX(di,1) + Ax*XNe1(tR1(i),di) ;
            end
        end
        
       	for i = 1:1:Nt2

            [~,~,Aa] = i0103_nTriang (TVcon2, tR2(i), Av2, pudim);
        	[~,~,Ax] = i0103_nTriang (TVcon2, tR2(i), Xv2, xdim);
                
            Aac = Aac + Aa;
        	Axc = Axc + Ax;
                
            for di = 1:1:3
             	NA(di,1) = NA(di,1) + Aa*ANe2(tR2(i),di) ;
              	NX(di,1) = NX(di,1) + Ax*XNe2(tR2(i),di) ;
            end
        end
            
        NA = NA/Aac;
    	NX = NX/Axc;

        
     	% 3.4.- Re-allocates:
        
        skin(s1).mANv(vr(1),:) = NA(:,1);
     	skin(s1).mXNv(vr(1),:) = NX(:,1);
            
      	skin(s2).mANv(vr(2),:) = NA(:,1);
       	skin(s2).mXNv(vr(2),:) = NX(:,1);
        
    end
   
end

end

