function [Spatches, ccTp] = so2934_tpSkins...
         (Npa, Spatches, tFace, tCurv, tolRc, tolRs, tolD, Nvx, R, pesw, ndig, wbF, DerC, DUtol, pRx)


% No. dimensions
pudim = 3;
xdim  = 3;


Xtol = pRx * R / 100; % pp tolerance in physical space:

% structure for aggregated contour curves of each patch:
ccTp(Npa,1) = struct ('Nf',0, 'Nc',0, 'nlo',[], 'ncu',[], 'agCurv',[], 'SCcon',[], 'CScon', [], 'NFC',[]);
% NCF: nodes of curves fitted (allocated later on, at fs0700)

% Loop on patches:

for pa = 1:1:Npa 
%for pa = 2:1:2 %%%%%%%%%%%%%%%%%%%%%
    
    % 1.- PRELIMINARIES:
    % Prepares surface and declares skin object:

    Nf = tFace(pa).Nfa;

    sf(Nf,1) = struct('Ncp',[],'Degree',[],'Usize',[],'KnotV',[],...
                        'w',[],'P',[],'Ew',[],'EP',[], 'CS', [], ...
                        'Nol',[],'Noc',[],'Rcu',[], 'Pa', 0);
	Bskin(Nf,1) = cPskin;

    for s = 1:1:Nf
        
        % surface data

        sf(s).Ncp     = tFace(pa).mSur(s).Ncp ;
        sf(s).Degree  = tFace(pa).mSur(s).Degree ;
      	sf(s).Usize   = tFace(pa).mSur(s).Usize ;
        sf(s).KnotV   = tFace(pa).mSur(s).KnotV ;  
        sf(s).CS      = tFace(pa).mSur(s).CS;
        
        P = tFace(pa).mSur(s).P;
        w = tFace(pa).mSur(s).w ;
        
        [Ew,EP] = GEN_12_ExpandBw (w, P,xdim, 2, sf(s).Ncp);
        
        sf(s).P  = P;
        sf(s).w  = w ;
        sf(s).EP = EP ;
        sf(s).Ew = Ew ;
        
        sf(s).Nol = tFace(pa).mSur(s).Nol ;
        sf(s).Noc = tFace(pa).mSur(s).Noc ;
        sf(s).Rcu = tFace(pa).mSur(s).Rcu ;
    end
    


    % 2.- GROUPS ALL CURVES OF THE PATCH:
    
    % (attaches to patch via SCcon) and computes vertexes.
    % Note: computes vtx considering previous curves
    [agCurv, TNcu, SCcon, CScon, ncu, nlo, CPcon] = so2935_AgreCurves (pa, Nf, sf, tCurv, xdim);

    ccTp(pa,1).Nf  = Nf;
    ccTp(pa,1).Nc  = TNcu;
    ccTp(pa,1).nlo = nlo;
    ccTp(pa,1).ncu = ncu;
    ccTp(pa,1).agCurv = agCurv;
    ccTp(pa,1).SCcon  = SCcon;
    ccTp(pa,1).CScon  = CScon;
    
    % Curves vertexes in local curve physical/param's coord's:
    [cLXv, cLUv, Ncv, ~] = ts3312_Cvertexes (agCurv, TNcu, R, Nvx, tolRc, xdim);

    
    % in solid patch param. coordinates:
    [cAv] = so2937_mAcon (TNcu, Ncv, cLXv, CPcon, Spatches , Xtol, DUtol, pudim, xdim);


    
    % 3.- TRIANGULATION:
    
    [QITst] = ts3330_QITst ...
    (tolRc, tolRs, tolD, Nvx, R, pesw, Nf, sf, agCurv, cLXv, cLUv, Ncv, nlo, ncu, SCcon, xdim, ndig, 0,0,0, wbF, DerC);



    % 4.- TRANSFERS TO Bskin AND COMPUTES Acoords :

    for s = 1:1:Nf

        Bskin(s,1).mNE = QITst(s).mNE ;
        
        NV = size(QITst(s).mXv,1) ;
        Bskin(s,1).mNV = NV ;
        
        Bskin(s,1).mTVcon = QITst(s).mTVcon ;
        Bskin(s,1).mVTcon = QITst(s).mVTcon ;
     	Bskin(s,1).mLVcon = QITst(s).mLVcon ;
        Bskin(s,1).mVLcon = QITst(s).mVLcon ;
        Bskin(s,1).mTLcon = QITst(s).mTLcon ;
        
        Bskin(s,1).mNlo = QITst(s).mNlo ;
        Bskin(s,1).mNlv = QITst(s).mNlv ;
        
        
        % Acoords of the vertexes:
        
        Xv = QITst(s).mXv;
        Av = so2936_mAv (NV, Xv, s, nlo, ncu, SCcon, Ncv, cAv, cLXv, Spatches, pa, Xtol, DUtol);
     
            
        % transfers vertexes coordinates:
        
        VP(NV,1) = cVP;
        
        for v = 1:1:NV
            for di = 1:1:xdim
                VP(v,1).mAcoor(di,1) = Av(v,di);
                VP(v,1).mXcoor(di,1) = Xv(v,di); 
            end
        end
        
        Bskin(s,1).mVP = VP ;
        clearvars 'VP'
        
    end

    Spatches(pa,1).INS(1).mNsk = Nf;
	Spatches(pa,1).INS(1).skin = Bskin ;
      
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %{
	s = 1;
      
    figure(33)
    
    Bskin(s,1).mKnotV
    Bskin(s,1).mNcp
    Bskin(s,1).mDegree
    Bskin(s,1).mUsize
    Bskin(s,1).mw
    Bskin(s,1).mP

	NV = Spatches(pa,1).INS(1).skin(s,1).mNV;
     
    for v = 1:1:NV
        Xv = Spatches(pa,1).INS(1).skin(s,1).mVP(v,1).mAcoor;
        scatter3( Xv(1), Xv(2),Xv(3),'b'); hold on
    end

	axis equal   
    %}
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
    
	% plots skins
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%{
    
    for s = 1:1:Nf
	%if pa==2
        
        fr =  22  ;
        Cv = QITst(s).mXv ;

        LVcon = QITst(s).mLVcon ;
        TVcon = QITst(s).mTVcon ;

        NLI = size(LVcon,1);
        NE = QITst(s).mNE ;

        pt1366_pTriLinMesh_nD...
        (Cv, NLI, LVcon, TVcon, xdim, fr, 'b', 1, 0, NV, 0, 0, NE, 0, 0, 0, 0, 0, 0);
    
	%end
    end
	%}
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    

    clearvars Nf sf QITst Bskin ;

end




end

