function [agCurv, TNcu, SCcon, CScon, ncu, Nol, CPcon] = so2935_AgreCurves (pRi, Nf, sf, cCurv, xdim)

% RETURNS:
% curves aggregated (non-repeated) for a set to surface that might share contours

% agCurv: aggregrated contour curves structure 
% TNcu: total number of curves
%   nlo(Nsf,1): No. loops per surface
%   ncu(Nsf,nlo): No. curves for each surface and loop
%   SCcon(Nsf,nlo,max(ncu)): curve references for each surface
% CPcon(TNcu,1): returns patch where the curve lies in




% 1.- INITATES RESPONSE:

TNcu = 0 ;
agCurv(2,1) = struct ('Ncp',0, 'Degree',0, 'Usize',0, 'KnotV',[], 'w',[], 'P',[], 'conF', 0, 'CD', 0);
Nol     = zeros(Nf,1);
ncu     = zeros(Nf,1);
SCcon   = zeros(Nf,1,1);


CPcon = zeros(1,2);


% 2.- FIRST SURFACE: all curves are included unless it is a closed surface, in
% that case it needs to compare against itself

s = 1 ;
Nol(s,1) = sf(s).Nol;



if pRi == 0 % in case of bEnti (and not tFace) chooses the master patch
    pRused = sf(s).Pa(1);
else
    pRused = pRi;
end



for lo = 1:1:Nol(s,1)

    sNoc = sf(s).Noc(lo); % original No. curves in surface - loop 
    ncu(s,lo) = sNoc;
    
    for c = 1:1:sNoc
    
        cR = sf(s).Rcu(lo,c); % curve ref. in bCurv

        
        if c == 1 && lo==1
            
            % 2.1.- First curve of first loop of first surface is always included
            
            TNcu = TNcu + 1 ;
            
            agCurv(TNcu).Ncp    = cCurv(cR).Ncp;
            agCurv(TNcu).Degree = cCurv(cR).Degree;
            agCurv(TNcu).Usize  = cCurv(cR).Usize;
            agCurv(TNcu).KnotV  = cCurv(cR).KnotV;
            agCurv(TNcu).w      = cCurv(cR).w;
            agCurv(TNcu).P      = cCurv(cR).P;
            agCurv(TNcu).conF   = cCurv(cR).conF;
            agCurv(TNcu).CD     = cCurv(cR).CD;

            SCcon(s,lo,c) = TNcu;
            CPcon(TNcu,1) = pRused;

        else
            
            % 2.2.- Rest of the curves are compared with already stored (recall CS)
            
        	Ncp2     = cCurv(cR).Ncp;
            Degree2  = cCurv(cR).Degree;
            Usize2   = cCurv(cR).Usize;
            KnotV2   = cCurv(cR).KnotV;
            w2       = cCurv(cR).w;
            P2       = cCurv(cR).P;
            conF2    = cCurv(cR).conF;
            CD2      = cCurv(cR).CD;
            
            % agregates candidate or just updates SCcon:
            
            [agCurv, TNcu, SCcon, CPcon] = so2932_comp2Extg ...
         	(pRused, s,lo,c, agCurv, TNcu, SCcon, CPcon, Ncp2, Degree2, Usize2, KnotV2, P2, w2, conF2, CD2, xdim);
 
        end

    end

end





% 3.- LOOP ON THE REST OF SURFACES:

for s = 2:1:Nf

    if pRi == 0 % in case of bEnti (and not tFace) chooses the master patch
        pRused = sf(s).Pa(1);
    else
        pRused = pRi;
    end
    
    
    Nol(s,1) = sf(s).Nol;  % No. loops of surface

    for lo = 1:1:Nol(s,1)

        sNoc = sf(s).Noc(lo); % original No. curves in surface - loop 
        ncu(s,lo) = sNoc;
        

        % searches for candidates of curves than are not contained already in cCurv
        
        for c = 1:1:sNoc

            % candidates curve:

            cR = sf(s).Rcu(lo,c); % curve ref. in bCurv
  
            
            Ncp2     = cCurv(cR).Ncp;
            Degree2  = cCurv(cR).Degree;
            Usize2   = cCurv(cR).Usize;
            KnotV2   = cCurv(cR).KnotV;
            w2       = cCurv(cR).w;
            P2       = cCurv(cR).P;
            conF2    = cCurv(cR).conF;
            CD2      = cCurv(cR).CD;
            
            
            % agregates candidate or just updates SCcon:
            
            [agCurv, TNcu, SCcon, CPcon] = so2932_comp2Extg ...
         	(pRused, s,lo,c, agCurv, TNcu, SCcon, CPcon, Ncp2, Degree2, Usize2, KnotV2, P2, w2, conF2, CD2, xdim);

        end
 
    end
 

end




% 4.- CScon:

cNs = zeros(TNcu,1); % No. surfaces per curve
CScon = zeros(TNcu,2);
    
for s = 1:1:Nf

    for lo = 1:1:Nol(s,1)
        
        for cu = 1:1:ncu(s,lo)
            
            cr = abs(SCcon(s,lo,cu));
            
            addF = 1 ;
            for i = 1:1:cNs(cr)
                if CScon(cr,i) == s 
                   addF = 0 ;
                end
            end
            
            if addF == 1
                cNs(cr) = cNs(cr)+1;
                CScon(cr, cNs(cr)) = s;
            end
 
        end
        
    end
      
end




end

