function [] = r3002_draw ...
    (Spatches, Bentities, Nbe, NpaON, prON, Stp, sc, ...
    xsF, usF, gsF, bsF, cponF, cpofF, cprF, cnetF, eonF, erF, nrF, beF, udfF, ksF, gpF, gprF, vmF)

% ACTION:
% Draws selected entities in physical / parameter space

% INPUTS:

% NpaON, prON(NpaON): No. actived patches and their references
% Stp: instance to represent
% sc: scale of deformed shape

% flags: equal to 1 if active, otherwise = 0

% xsF - usF:        physical - parameter space
% gsF, bsF:         gross - bounded skins
% cponF, cpofF, cprF, cnetF: control points ON, OFF, refs, control net
% eonF, erF, nrF:   elements, refs, nodes refs
% beF:              boundary entities
% udfF:             undeformed shape
% ksF, gpF, gprF:   knot spans


xdim = 3;
udim = 3;

for pa = 1:1:NpaON
    
    % 1.- PRELIMINARY BASIC DATA EXTRACTION:
    
    % patch reference
    paR = prON(pa,1); 
    
    
    % patch NURBS features:
    
	KnotV   = Spatches(paR,1).INS(Stp,1).mKnotV;
	Ncp     = Spatches(paR,1).INS(Stp,1).mNcp;
	Degree  = Spatches(paR,1).INS(Stp,1).mDegree;
 	Usize   = Spatches(paR,1).INS(Stp,1).mUsize;
	EP = Spatches(paR,1).INS(Stp,1).mEP;
 	Ew = Spatches(paR,1).INS(Stp,1).mEw;
    trpF    = Spatches(paR,1).trpF ;

    % 2.- PATCH SKINS:

    if gsF == 1
        
        % 2.1.- Gross skin: 
        
        Ns    = Spatches(paR,1).Ngsk;
        Skins = Spatches(paR,1).mGskin;
            
        r3011_gSkins (Skins, Ns, xsF, usF, xdim);
     
	elseif bsF == 1

        % 2.2.- Bounded patches:
        
        r3014_bSkins (Spatches, paR, Stp, xsF, usF, sc, udfF, vmF, xdim);

    end
        

        
    % 3.- CONTROL POINTS:
    
    if (cponF==1 || cnetF==1) || (cpofF==1 || cprF==1)

        r3023_CPs (xsF, usF, cprF, cnetF, Spatches, paR, Stp, sc, xdim);
        
    end
    
    
    % 4.- BOUNDARY ENTITIES: only for 1st instance (insR=0)
    
    if beF == 1 % && Stp == 1
        
        r3012_Bentis (Spatches, paR, Bentities, Stp, xsF, usF, sc, xdim, udim);
        
    end
    

    % 5.- MESHES: at the moment only for instance 0 BUT WILL INCLUDE
    % FURTHER INSTANCES IN THE FUTURE SPECIALLY FOR TIME DISCRETIZATION

    if eonF==1 || (erF==1 || nrF==1)
        
        if trpF == 1
        
            % trimmed patches => tetrahedrons:
            
            stL = 0.25 ;
            
            mCP     = Spatches(paR,1).INS(Stp,1).mCP;
            Imesh  = Spatches(paR,1).INS(Stp,1).mIM;

            r3031_Imesh (stL, Imesh, mCP, KnotV, Ncp, Degree, Usize, Ew, EP, xsF, usF, erF, nrF, xdim, udim);

        elseif trpF == 0
            
            % un-trimmed patches => knot spans
            
            stL = 0.05 ;

            r3032_knotsp (stL, xsF, usF, KnotV, Ncp,Degree,Usize, Ew, EP, xdim, udim) ;
            
        end
        
    end

    
    % 6.- KNOT SPANS:
    
    if ksF == 1

        stL = 0.05 ;
        
        r3032_knotsp (stL, xsF, usF, KnotV, Ncp,Degree,Usize, Ew, EP, xdim, udim) ;

    end
    

    % 7- GPs:

    if gpF == 1
        
        pg3070_patchGP (Spatches, paR, Stp, xsF, usF, gprF);
        
    end
    
end


% 8.- BOUNDARY SURFACES (IN CASE NO PATCHES ARE SELECTED):
% only plots in physical space

if (xsF == 1 && NpaON == 0) && beF == 1 
    
    bt3060_bsTri (Bentities, Nbe, Stp, sc, gpF);
    
end


axis equal


end

