function [bF] = i3101_planeScomp (srf, fNcp, fP, fnv, oNoc, oLCurves, tolp, xdim)

% RETURNS:
% bF: belonging flag = 1 if Xmid belongs to computable face (surface), otherwise = 0

   
% INPUTS:
% sR: surface 'srf' reference
% fR: face reference
% srf: surface
% face NURBS features and curves to compare 'srf'
% tolp: tolerance of the mid-point of 'srf' to be at the plane of the face

bF = 0 ;
udim = 2 ;


% 1.- FIRST CONDITION:
% One cp (the first one) belongs to plane of face:
          
bNcp    = srf.Ncp;
bDegree = srf.Degree;
bUsize  = srf.Usize;
bKnotV  = srf.KnotV;
bw      = srf.w;
bP      = srf.P;
[bEw,bEP] = GEN_12_ExpandBw (bw,bP, xdim, udim, bNcp);


P1 = zeros(xdim,1);
P1(:,1) = bP(1,:);

[~, A,B,C,D] = go2505_cloudP_plane (fNcp(1)*fNcp(2), fP); % face plane coeff.

[dist, ~, ~] = go2534_distPtoPlan (P1, A, B, C, D);


%{
if fnv(2) == 1 && fP(1,2)==50
    dist
    tolp
end
%}

if dist < tolp

    % 2.- SECOND CONDITION:
	% at least one sample point is inside face loop:
         
    % parameter steps for sample points to check if they belong to the contour
    ustp = [0.2 0.2]; 
    
    for u = 0.2:ustp(1):0.8
    for v = 0.2:ustp(2):0.8
    
        % 2.1.- Sample point:
        
        [Xs] = b2601_BNURBS_Xcoord ([u v], bKnotV, bNcp,bDegree,bUsize, bEw,bEP, udim, xdim);

        
        % 2.2.- Finds closest curve and stores in ccR (in uccR the selected parameter):
        
        P1 = zeros(xdim,1);
        
        P1(:,1) = oLCurves(1).P(1,:); % initiates min. distance
        
        [dmin, ~] = go2522_dist_bwtin_2P (Xs, P1, xdim);
     	rmin = zeros(xdim,1) ;
     	ccR  = 1 ;
      	uccR = 0 ;
        xccR = P1 ;
        
        % step in parameter
        nstp = 5 ;
        stp = 1/nstp; 

        for c = 1:1:oNoc

            % curve features
            cKnotV  = oLCurves(c).KnotV;
            cNcp    = oLCurves(c).Ncp;
            cDegree = oLCurves(c).Degree;
            cUsize  = oLCurves(c).Usize;
            cw      = oLCurves(c).w;
            cP      = oLCurves(c).P;

            x = zeros(1,3);
            
            conta = 0 ;
            for uc = 0:stp:1

                xc = b2601_BNURBS_Xcoord (uc, cKnotV, cNcp,cDegree,cUsize, cw,cP, 1,xdim);

                conta = conta+1;
                x(conta,:) = xc(:,1);

                [dXC, r] = go2522_dist_bwtin_2P (Xs, xc, xdim);

                if dXC <= dmin
                    dmin = dXC;
                    rmin = r ;
                    ccR  = c ;
                    uccR = uc ;
                    xccR = xc ;
                end
            end
                
        end
       

        % 2.3.- Check if Xs is at LHS of curve: to do so the versor correspondant to the
        % cross product (r x dC) , 'bnv', must equal the direction of the face 'fnv'
        %       r: vector from Xs to curve at uccR
        %       dC: derivative of ccR curve at uccR

        r = rmin;

        dC = zeros(xdim,1);

        cKnotV  = oLCurves(ccR).KnotV;
        cNcp    = oLCurves(ccR).Ncp;
        cDegree = oLCurves(ccR).Degree;
        cUsize  = oLCurves(ccR).Usize;
        cw      = oLCurves(ccR).w;
        cP      = oLCurves(ccR).P;

        [dS] = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize, cP ,cw, 1, uccR, xdim, 1);

        dC(:,1) = dS(2,1,1,:);


        bnv = cross(r,dC);
        bnv = bnv / norm(bnv);
        CO  = go2542_coincidentP (bnv, fnv, xdim, 5);


        % 2.4.- If same normal versor => that is the correspondant face:
        
        if CO == 1
            bF = 1; 
            break
        end

    end
	if bF == 1
        break
	end
    end
    

    
    
    
end
  


end

