function [aP, aF, SW, np] = i3170_sVSshell (sR, srf, Npa1, Npa2, Shell, udim, xdim)
         
% finds the Shell face where one surface srf(sR) lies on

% RETURNS:
% aP, aF:   patch and patch-face attached to that surface 'srf' (if they coincide)
% SW:       swith directions flag: 
%               1 if 'srf' needs to be switched, 
%               0 otherwise (if 'srf' orientation is pointing outwards the patch)
% np:       No. patches where it lies, normally np=1, if coupling then np=2

% INPUTS:
% sR:       'srf' reference
% srf:      surface structure.
% Npa:      No. patches
% Shell:    structure to compare with

% 0.- PRELIMINARIES:

aP = zeros(2,1);
aF = zeros(2,1);
SW = zeros(2,1);
np = 0;

% surface features:
bNcp    = srf.Ncp;
bDegree = srf.Degree;
bUsize  = srf.Usize;
bKnotV  = srf.KnotV;
bw      = srf.w;
bP      = srf.P;
bPL     = srf.PL;


[bEw,bEP] = GEN_12_ExpandBw (bw,bP,xdim,udim, bNcp);


Umid = ones(udim,1)*0.5; % mid parameter coodinates

Xmid = b2601_BNURBS_Xcoord (Umid, bKnotV, bNcp, bDegree, bUsize, bEw, bEP, udim,xdim);

sXmid = Xmid'; % transforms column Xmid into row, as a list manner

% tolerance for distance to plane to consider is on the plane, computed as
% percentage of first control net edge lenght:

L = 0 ;
for i = 1:1:bNcp(1)-1
    
    d = 0 ;
    for di = 1:1:xdim
        d = d + (bP(i+1,di) - bP(i,di))^2 ;
    end
    d = d^0.5;
    
    L = L + d ;
end
    
tolp = L * 0.05 / 100 ;



for pa = Npa1:1:Npa2 % loops on each shell (one shell per patch)
    
    Nfa = Shell(pa).Nfa;

  	for f = 1:1:Nfa % loop on patch-faces
        
        aPi = 0 ;
        aFi = 0 ;
        SWi = 0 ;
        
        % 1.- EXTRACTION OF PATCH FACES FEATURES:
        % NURBS surface features
        % outer loop curves features from patch face
        
        fNcp    = Shell(pa).face(f).fSurface.Ncp;
        fDegree = Shell(pa).face(f).fSurface.Degree;
        fUsize  = Shell(pa).face(f).fSurface.Usize;
        fKnotV  = Shell(pa).face(f).fSurface.KnotV;
        fw      = Shell(pa).face(f).fSurface.w;
        fP      = Shell(pa).face(f).fSurface.P;
        fnv    = Shell(pa).face(f).fSurface.n; 
        
        fPL    = Shell(pa).face(f).fSurface.PL;
        fCS    = Shell(pa).face(f).fSurface.CS;
        

        [fEw,fEP] = GEN_12_ExpandBw (fw,fP,xdim,udim, fNcp);

        oNoc     = Shell(pa).face(f).fLoop(1).Noc; % outer curves
        oLCurves = Shell(pa).face(f).fLoop(1).LCurves;

        %{
        if sR == 5 && f==4
            
            bPL
            fPL
            
            figure(8)
            scatter3( fP(:,1), fP(:,2), fP(:,3), 'r'); hold on
            scatter3( bP(:,1), bP(:,2), bP(:,3), 'b'); hold on
            axis equal
        end
        %}
        
        % 2.- COMPARISON 'srf' vs FACE:
        
        % only continues to that face if is equally regarding planar state
 
        if bPL == fPL

            if bPL == 1

                %{
                if sR==7 && f==2

                    f
                    srf.P
                    fnv
                   

                end
                %}
                
                % 2.1.- Planar faces:

                [bF] = i3101_planeScomp (srf, fNcp, fP, fnv, oNoc, oLCurves, tolp, xdim);
 
            else

                % 2.2.- Curved faces: 
                
                [bF] = i3102_curvedScomp ...
                (sR, Xmid, fNcp, fDegree, fUsize, fKnotV, fw, fP, fCS, fEw, fEP, xdim);
                
            end
            
            if bF == 1
                
                aPi = pa; % patch and face ref's
                aFi = f ;
             	np = np + 1 ; % No. patches counter
                
            end
            
            
            % 3.- ESTABLISHES IF THE B.E. NEEDS TO BE REVERSED OR NOT 
            % (SW output would change to 1 if needs reversing):
            
            % computes normal versors for patch-face 'fnv' and B.E. 'bnv';
            % Both normal versors are computed by cross product of
            % derivatives about param. directions 1 and 2 in Xmid.
            
            % only computes if Xmid lies in that path-face
            if aPi~=0 && aFi~=0
                
                Su = zeros(xdim,1);
                Sv = zeros(xdim,1);
                
                % 3.1.- patch-face normal versor:

                % param. coord's in patch-face corresopndant to Xmid and dS at that location:
                nsp = 1;
                [sU] = b2801_PPforSetPoints (nsp, sXmid, fDegree, fNcp, fKnotV, fUsize, fP ,fw, fCS, xdim, udim, 0);
                [dS] = b2701_BNURBS_deriv (fDegree, fNcp, fKnotV, fUsize, fP ,fw, [1 1], sU, xdim, udim);
                
                for di=1:1:xdim
                    Su(di) = dS(2,1,di);
                    Sv(di) = dS(1,2,di);
                end
                
                fnv = cross(Su,Sv);
                fnv = round(fnv / norm(fnv),3);
                 

                % 3.2.- boundary normal versor:
                
                [dS] = b2701_BNURBS_deriv (bDegree, bNcp, bKnotV, bUsize, bP ,bw, [1 1], [0.5 0.5], xdim, udim);
                
                for di=1:1:xdim
                    Su(di) = dS(2,1,di);
                    Sv(di) = dS(1,2,di);
                end
                
                bnv = cross(Su,Sv);
                bnv = round(bnv / norm(bnv),3);

                % 3.3.- Checks if they point to the same direction:
                
                SWi = 0 ; % initially assumes it does not have to change
                
                for di = 1:1:xdim
                    
                    % if finds any different sign => needs to be switched
                    if sign(fnv(di))~= sign(bnv(di))
                        SWi = 1;
                        break
                    end
                end

            end
            
        end
            
        
        % transfers info to response arrays:
        
        if aPi~=0 && aFi~=0
            
            aP(np) = aPi;
            aF(np) = aFi;
            SW(np) = SWi;
            
        end
        
        % exit in case finds two patches
        if np==2
      	break
        end
        
    end % loop on Patch-faces
    

    if np==2
  	break
    end
    
end % loop on Patches
    
  


end

