function [cNcp, cDegree, cUsize, cKnotV, cP, cw, cCS] = i3180_copyNURBSsurf ...
            (Ncp, Degree, Usize, KnotV, P, w, CS, xdim, udim, RevD)

% RETURNS:
% NURBS surface features copied from original
% one direction can be reversed, so we attain opposite surface orientation

% INPUTS:
% original NURBS features
% RevD(2): inidcates if direction to be reversed (then value = 1, otherwise = 0)


% 1.- Copy features non-affected by reversion:

cNcp    = Ncp ;
cDegree = Degree ;
cUsize  = Usize ;
cCS     = CS ;

% 2.- Knot Vector:

cKnotV = zeros(udim, max(Usize));
U      = zeros(max(Usize),1);

for di=1:1:udim
    
    for i = 1:1:Usize(di)
        cKnotV(di,i) = KnotV(di,i);
        U(i)         = KnotV(di,i);
    end
    
    if RevD(di) == 1
        U = GEN_06_ReverseKnotV (U, Ncp(di), Degree(di));
      	for i = 1:1:Usize(di)
            cKnotV(di,i) = U(i);
        end
    end
end


% 3.- Control points:

% expand control points in case they are not:

if size(w,2)==1 || size(w,1)==1
    [Ew,EP] = GEN_12_ExpandBw (w, P,xdim, udim, Ncp);
else
    EP = P ;
    Ew = w ;
end

% establishes limits of cp's loop:

i1 = 1;     ist = 1 ;   i2 = Ncp(1);
j1 = 1;     jst = 1 ;   j2 = Ncp(2);

if RevD(1)==1
    i1 = Ncp(1); ist = -1 ; i2 = 1;
elseif RevD(2)==1
    j1 = Ncp(2); jst = -1 ; j2 = 1 ;
end


% loop on points

cEw = zeros(Ncp(1), Ncp(2));
cEP = zeros(Ncp(1), Ncp(2),xdim);

ic = 0 ;
for i = i1:ist:i2
    
    ic = ic+1;
    jc = 0 ;
    
    for j = j1:jst:j2

        jc = jc+1;

        cEw(ic,jc) = Ew(i,j);
        for di = 1:1:xdim
            cEP(ic,jc,di) = EP(i,j,di);
        end

    end
end


% contracts w and P:
[cw,cP] = GEN_12_ContractsBw (cEw, cEP, xdim, udim, Ncp);


end

