function [gShell] = gp3112_gShell (sGMSBO, sGshells, sGfaces, ...
    sGsurfN, sGloops, sGedges, sGcurves, sGverteces, Npa, paoR)

% RETURNS: 
% gShell(Npa):  structure, with all features prepared for filling the patch

ndig = 6 ; % No. digits to compare knots

xdim = 3 ;
udim = 3 ;
budim = udim-1;

% 1.- DECLARES DATA STRUCTURE gShell:

gShell(Npa) = struct ('vR',0, 'vP', [], 'eR',0, 'face',[], 'eCurves',[]); 

% No. faces = No. loops = 6 for each patch
% No. curves = 12


% 2.- COMPUTES afLoopVL (No. all faces , 5): 

% 'afLoopVL' from 'agregated faces loop Vertex List indexes'

%   No. faces = total faces for all patches
%   5 = 1 surface ref. + 4 vertexes per loop (one loop per face)

[afLoopVL] = gp3177_afLoopVP (Npa, sGMSBO, sGshells, sGfaces, sGloops, sGedges);


% LOOP ON PATCHES to GENERATE gShell, each with 'tfaces' and 'teCurves':

for puR = 1:1:Npa
    
    % puR: patch 'user' reference
    % poR: patch 'original' ref. as per IGES files
    poR = paoR(puR); 
    
    % 3.1.- RELATION BETWEEN ORIGINAL AND ARRANGED ORDER OF FACES AND
    % CURVES, AND VERTEXES COORDINATES (already arranged):
    
    [AtoOfaces, AtoOcurves, vP] = gp3183_AtoO ...
 	(poR, afLoopVL, sGMSBO, sGverteces, sGshells, sGfaces, sGloops, sGedges, xdim);


    % 3.2.- COMPUTES teCurves:
    
    [teCurves] = gp3181_teCurves (sGcurves, AtoOcurves, xdim, ndig);

    

    % 3.3.- COMPUTES FACES surfaces NURBS FEATURES AND ALLOCATES TO tfaces:

    [tFaces] = gp3182_tFaces (sGfaces, sGsurfN, teCurves, AtoOfaces, xdim, budim, ndig);

    
    % 3.4.- ALLOCATES TO gShell and clear VBLES.:
    
	shR = sGMSBO(poR).shR;
    faR  = sGshells(shR).faR(1);
    loR  = sGfaces(faR).loR(1); % loop reference
	edR  = sGloops(loR).edR(1) ; % edges ref. (assume one edges ref. per patch), allocates only once
    vR  = sGedges(edR).v1R(1); 

    gShell(puR).vP = vP ;
    gShell(puR).eR = edR ; 
    gShell(puR).vR = vR ;
    
    gShell(puR).face = tFaces;
    gShell(puR).eCurves = teCurves;
    
    clearvars ('tFaces','teCurves');
    
end


end

