function [gPatch] = gp3113_gPatch (Npa, gShell, xdim, udim)

% RETURNS: gPatch will all info except B.E.'s features, that is:
% - dimensions
% - patches NURBS information


% 1.- DECLARES gPatch STRUCTURE:

gPatch(Npa,1) = struct ( ...
'Ncp',[], 'Degree',[], 'Usize',[], 'KnotV',[], 'P',[], 'w',[],'EP',[], 'Ew',[], ...
'nbe', 0, 'Rbe', []);


for pa = 1:1:Npa
    

    % 2.- TRANSFERS ALL FEATURES FROM gShell BUT NOT cp's COORDINATES:
    
    conta = 0 ;
    for e = 1:4:9
        
        conta = conta + 1 ;
        
       	gPatch(pa,1).Ncp(1,conta)    = gShell(pa).eCurves(e).Ncp ;
        gPatch(pa,1).Degree(1,conta) = gShell(pa).eCurves(e).Degree ;
        gPatch(pa,1).Usize(1,conta)  = gShell(pa).eCurves(e).Usize ;

        for k=1:1:gShell(pa).eCurves(e).Usize
            gPatch(pa,1).KnotV(conta,k) = gShell(pa).eCurves(e).KnotV(k);
        end

    end 


    % 3.- Cp's COORDINATES:
    % it has to detect which case the patch belongs to
    
    % No. linear directions and linear direction flag:

    nLd = 0 ;
    fpCur = [ 9 5 1 ]; % faces 1, 2 and 3 perp. eCurve ref.
    
    for di=1:1:udim
        if gPatch(pa,1).Ncp(di)==2
            nLd = nLd+1;
        end
    end
    
    

    % 3.1.- CASE A: uses vertexes directly:
    
    if nLd == 3
    
        gPatch(pa,1).P = gShell(pa).vP;
        gPatch(pa,1).w = [ 1 1 1 1 1 1 1 1 ];
      
        
    % 3.2.- FOR CASES B and C: works out a generation face and interpolates
    % (applies the SANDWICH ALGORTIHM)
    
    elseif nLd == 2 || nLd == 1
        
        [gPatch] = gp3184_soSandwich (pa, gPatch, gShell, nLd, fpCur, xdim);
        

    % 3.3.- FOR CASE D: volume fitting to achieve the solid
    
    elseif nLd == 0 
        
        Ncp = gPatch(pa,1).Ncp;

        [XP] = s3140_soLattFitt (Ncp, gShell, pa);
        gPatch(pa,1).P = XP;
        
        [w] = s3147_ws (Ncp, gShell, pa);
        gPatch(pa,1).w = w;

    end 
    

    % computes expanded cp's coords:
    [Ew,EP] = GEN_12_ExpandBw (gPatch(pa,1).w, gPatch(pa,1).P ,xdim, udim, gPatch(pa,1).Ncp); 
    
    gPatch(pa,1).Ew = Ew ;
    gPatch(pa,1).EP = EP ;
    
    
	% prints patch cp net:
    %{
    figure(13)
    
    scatter3(gPatch(pa).P(:,1),gPatch(pa).P(:,2),gPatch(pa).P(:,3),30,'b','filled');hold on
    
    for i = 3:1:3
    for j = 1:1:Ncp(2)
    for k = 1:1:Ncp(3)
        
        %{
        if k<Ncp(3)
            plot3( ...
                [EP(i,j,k,1) EP(i,j,k+1,1)],...
                [EP(i,j,k,2) EP(i,j,k+1,2)], ...
                [EP(i,j,k,3) EP(i,j,k+1,3)], 'r'); hold on
        end
        
        %}
        
        %{
        if j<Ncp(2)
            plot3( ...
                [EP(i,j,k,1) EP(i,j+1,k,1)],...
                [EP(i,j,k,2) EP(i,j+1,k,2)], ...
                [EP(i,j,k,3) EP(i,j+1,k,3)], 'r'); hold on
        end
        %}
        
        %{
        if i<Ncp(1)
            plot3( ...
                [EP(i,j,k,1) EP(i+1,j,k,1)],...
                [EP(i,j,k,2) EP(i+1,j,k,2)], ...
                [EP(i,j,k,3) EP(i+1,j,k,3)], 'r'); hold on
        end
        %}
        
    end
    end
    end
    
    axis equal
    %}
    
    
    
end % loop on patches







end

