function [gbShell] = gp3114_gbShell (Npa, gShell, xdim)

% RETURNS: gbShell: structure with basic features of gross shells
% it is equivalent to gShell, but re-arranged to be as bShell and be used
% to indentify B.E.

gbShell(Npa) = struct('Nfa',0 , 'face', []) ;
    
% Nfa:  No. faces
% face: boundary curves refs and NURBs surface features


for pa = 1:1:Npa
    
    % 1.- No. FACES OF gbShell:
    
    Nfa = 6;
    gbShell(pa).Nfa = Nfa;

    % loop on faces: stores all curves ref's, surface features and surface orientation:
    
    tface(Nfa) = struct('fO', 0, 'fSurface', 0, 'Nol', 0, 'fLoop', []);
    
    for f = 1:1:Nfa

        %fO = 1;
        Nol = 1;
        
        %tface(f).fO = fO ;
        tface(f).Nol = Nol;
       
        % 2.- LOOPS AND CURVES:
        
        tLoop(Nol) = struct('Noc', 0, 'LCurves', []);
        
        for L = 1:1:Nol
            
            Noc = 4 ;
            
            tCurves(Noc) = struct('Ncp',[], 'Degree', [], 'Usize', [], 'KnotV', [], 'w', [], 'P', []);
            
            % curves order is different acc/to face order
            if mod(f,2)~=0
                
                cR = [  gShell(pa).face(f).eCurveR(2,1) , gShell(pa).face(f).eCurveR(1,2) , ...
                        gShell(pa).face(f).eCurveR(2,2) , gShell(pa).face(f).eCurveR(1,1) ];
                    
            else
                
                cR = [  gShell(pa).face(f).eCurveR(1,1) , gShell(pa).face(f).eCurveR(2,2) , ...
                        gShell(pa).face(f).eCurveR(1,2) , gShell(pa).face(f).eCurveR(2,1) ];
                    
            end
            
            
            for c = 1:1:Noc
                
                % transfers to tCurves information from gShells:
                
                tCurves(c).Ncp    = gShell(pa).eCurves(cR(c)).Ncp;
                tCurves(c).Degree = gShell(pa).eCurves(cR(c)).Degree;
                tCurves(c).Usize  = gShell(pa).eCurves(cR(c)).Usize;
                
                if c <= 2 % note for 3rd and 4th curves inverts
                    
                    sKnotV = gShell(pa).eCurves(cR(c)).KnotV;
                    tCurves(c).KnotV = sKnotV;
                    tCurves(c).w     = gShell(pa).eCurves(cR(c)).w;
                    tCurves(c).P     = gShell(pa).eCurves(cR(c)).P;

                else
                    
                    poc = tCurves(c).Ncp;
                    
                    for po = 1:1:tCurves(c).Ncp 
                        for di = 1:1:xdim
                            tCurves(c).P(po,di) = gShell(pa).eCurves(cR(c)).P(poc,di);
                        end
                        tCurves(c).w(po) = gShell(pa).eCurves(cR(c)).w(poc);
                        poc = poc-1;
                    end
 
                    [rKnotV] = GEN_06_ReverseKnotV (gShell(pa).eCurves(cR(c)).KnotV, gShell(pa).eCurves(cR(c)).Ncp, gShell(pa).eCurves(cR(c)).Degree);
                    tCurves(c).KnotV = rKnotV; 
                end
                
                % makes sure curves KnotV is a row matrix:
                if size(tCurves(c).KnotV,2) == 1
                    tCurves(c).KnotV = tCurves(c).KnotV';
                end
  
            end
            
            tLoop(L).Noc = Noc;
            tLoop(L).LCurves = tCurves;
            
            clearvars tCurves;
        end
        

        
        tface(f).fLoop = tLoop ;
        clearvars tLoop;
        

        % 3.- SURFACE FEATURES :
        
        tSurface = struct('Ncp',[], 'Degree', [], 'Usize', [], 'KnotV', [], 'w', [], 'P', [], 'PL', 0, 'n', [] , 'CS', []);
        
        sNcp   = gShell(pa).face(f).fSurface.Ncp;
        sDegree = gShell(pa).face(f).fSurface.Degree;
        sUsize = gShell(pa).face(f).fSurface.Usize;
        sKnotV = gShell(pa).face(f).fSurface.KnotV;
        sw     = gShell(pa).face(f).fSurface.w;
        sP     = gShell(pa).face(f).fSurface.P;
        sPL    = gShell(pa).face(f).fSurface.PL;
        sCS    = gShell(pa).face(f).fSurface.CS;
        

        
        if mod(f,2)==0
            
            % copies faces  even ref's
            
            tSurface.Ncp   = sNcp;
            tSurface.Degree = sDegree;
            tSurface.Usize = sUsize;
            tSurface.KnotV = sKnotV;
            tSurface.w     = sw;
            tSurface.P     = sP;
            tSurface.PL    = sPL;
            tSurface.CS    = sCS;
        
        
        elseif mod(f,2)~=0
            
            % needs to invert faces of odd ref's

            pdim = [2 1];
            
            tSurface.PL = sPL;
            
 
            for di = 1:1:2
                
                pd = pdim(di);
                
                tSurface.Ncp(di) = sNcp(pd);
                tSurface.Degree(di) = sDegree(pd);
                tSurface.Usize(di) = sUsize(pd);
                tSurface.CS(di)= sCS(pd);
                
                for i = 1:1:sUsize(pd)
                    tSurface.KnotV(di,i) = sKnotV(pd,i);
                end
            end
    
  
            
            [Esw,EsB] = GEN_12_ExpandBw (sw, sP,xdim, 2, sNcp);
            
            for i = 1:1:sNcp(1)
            for j = 1:1:sNcp(2)
                
                Etw(j,i) = Esw(i,j);
                for di=1:1:xdim
                    EtB(j,i,di) =  EsB(i,j,di);
                end
                
            end
            end

            [tw,tB] = GEN_12_ContractsBw (Etw, EtB, xdim, 2, tSurface.Ncp);

            tSurface.w     = tw;
            tSurface.P     = tB;
        end
        

        
        % computes normal to planar surface with outwards shell sense:
        
        n  = zeros(xdim,1);
        
        if tSurface.PL == 1
            
          	Degree  = tSurface.Degree;
            Ncp     = tSurface.Ncp;
            KnotV   = tSurface.KnotV;
            Usize   = tSurface.Usize;
            P       = tSurface.P;
            w       = tSurface.w;

            [n] = i3160_nVersr (Degree, Ncp, KnotV, Usize, P ,w, xdim);
            %[n] = i3160_nVersor (tface, f, xdim);
        end
        
        
        tSurface.n = n;
        
        tface(f).fSurface = tSurface ;
        
        
        %{
        
        % checks gbShell:
        
        FACEEEE = f

 
        figure(f)
        scatter3(tSurface.P(:,1), tSurface.P(:,2), tSurface.P(:,3), 'r'); hold on
        

        for i = 1:1:prod(tSurface.Ncp)
            text(tSurface.P(i,1), tSurface.P(i,2), tSurface.P(i,3), num2str(i)); hold on
        end
        
        axis equal
        
        for c = 1:1:tface(f).fLoop(1).Noc
            
            tface(f).fLoop(1).LCurves(c).P
            
        end
        
        
        %}
        
        
        
        clearvars tSurface;
        
    end
    

    
    % 4.- ALLOCATES to bshell and clearvars:
    
    gbShell(pa).face = tface;
    clearvars tface;
end



end

