function [gFace] = gp3116_gFaces (Npa, gShell)

% RETURNS: 
% gFace(Npa):   faces un-trimmed surfaces features for gross patches, oriented outwards domain
% Ngc:          No. contour curves for gross faces, for all patches
% gCurv(Ngc):   contour curves features, at LHS is the surface

xdim = 3 ;

% 1.- DECLARATION OF STRUCTURES:

gFace(Npa) = struct ...
    ('Nfa', 0 , 'mSur', [] );

contaC = 0 ; % curves counter
 
for puR = 1:1:Npa
    
	% puR: patch 'user' reference

    % 2.- TEMPORARY STRUCTURE:
    
    Nfa = 6 ;
    
 	tSur(Nfa) = struct ...
    ( ...
     'Ncp',[],  'Degree',[], 'Usize',[], 'KnotV',[], 'w',[], 'P',[], 'PL', 0,  ...
     'Nol',0,  'Noc',[], 'Rcu',[], 'CS', []);


    % 3.- READ FROM gShell TO CONSTRUCT TEMP. SURFACES:
    
       
	fxdim = xdim;
	fudim = 2 ;

    for fa = 1:1:Nfa
        
        % 3.1.- Copy NURBS surface info, reverses if required:
        
        gNcp   = gShell(puR).face(fa).fSurface.Ncp;
        Degree = gShell(puR).face(fa).fSurface.Degree;
        Usize  = gShell(puR).face(fa).fSurface.Usize;
        KnotV  = gShell(puR).face(fa).fSurface.KnotV;
        P      = gShell(puR).face(fa).fSurface.P;
        w      = gShell(puR).face(fa).fSurface.w;
        PL     = gShell(puR).face(fa).fSurface.PL;
        CS     = gShell(puR).face(fa).fSurface.CS;
        
        RevD   = [0 0];
        if mod(fa,2)==1
            RevD = [1 0];
        end
        
        [fNcp, fDegree, fUsize, fKnotV, fP, fw, fCS] = i3180_copyNURBSsurf ...
            (gNcp, Degree, Usize, KnotV, P, w, CS, fxdim, fudim, RevD);
        

        % 3.2.- Transfers information to tSur, EXCEPT loops info:
        
        tSur(fa).Ncp    = fNcp;
        tSur(fa).Degree = fDegree;
        tSur(fa).Usize  = fUsize;
        tSur(fa).KnotV  = fKnotV;
        tSur(fa).P      = fP;
        tSur(fa).w      = fw;
        
        tSur(fa).PL      = PL;
        
        tSur(fa).CS     = fCS;


        % 3.3.- LOOPS INFOR FOR tSur:
        
        tSur(fa).Nol = 1 ;
        tSur(fa).Noc(1) = 4 ;
        tSur(fa).Rcu = [ contaC+1 , contaC+2 , contaC+3 , contaC+4 ];
  
    end

            
    contaC = contaC + 4 ;
    
   
    % 4.- ALLOCATES to gFace
 
    gFace(puR).Nfa = Nfa ;
    gFace(puR).mSur = tSur ;
    
	clearvars ('tSur');
    
end



end

