function [bShell] = be3122_bShell (Npa, Sshells, Sfaces, Sloops, Sedges, Scurves, Ssurf, paoR, xdim)

% RETURNS: bshell: structure with basic features of bounded shells


% 1.- DECLARES DATA STRUCTURE:

bShell(Npa) = struct('Nfa',0 , 'face', []) ;
    
% Nfa:  No. faces
% face: boundary curves refs and NURBs surface features


for puR = 1:1:Npa
    
    poR = paoR(puR); % patch ref. to allocate to

    % poR: patch original ref. as per IGES files
    % puR: patch user ref.
    
    % 2.- No. FACES FOR bshell:
    
 
    Nfa = Sshells(poR).Nof;
    bShell(puR).Nfa = Nfa;


    % 3.- DECLARES TEMPORARY STRUTURE tface TO STORE COMPUTED DATA FOR THAT PATCH:

    tface(Nfa) = struct('fSurface', 0, 'Nol', 0, 'fLoop', []);
    
    
    % loop on faces: stores all curves ref's, surface features and surface orientation:
    
    for f = 1:1:Nfa

        % 4.- READ AND TRANSFERS INFORMATION FROM sB array:
        
        faR = Sshells(poR).faR(f);

        Nol = Sfaces(faR).Nol;
        
        tface(f).Nol = Nol;
        
        % 5.- DEFINES TEMPORARY DATA STRUCTURE: CONTOUR LOOP & SURFACE: 
        
        tLoop(Nol) = struct('Noc', 0, 'LCurves', []);
        tSurface = struct('Ncp',[], 'Degree', [], 'Usize', [], ...
            'KnotV', [], 'w', [], 'P', [], 'PL', 0, 'n', [], 'CS',[]);
        
        
        
        % 6.- ALLOCATES TO tLoop No. CURVES AND THEIR NURBS FEATURES:
        
        for L = 1:1:Nol
            
            loR  = Sfaces(faR).loR(L);
            Ned = Sloops(loR).Noe;
            Noc = Ned ;
            
            tCurves(Noc) = struct('Ncp',[], 'Degree', [], 'Usize', [], 'KnotV', [], 'w', [], 'P', []);
            
            for c = 1:1:Noc
                
                % edge ref., list index and orientation:
                eR  = Sloops(loR).edR(c); 
                eLI = Sloops(loR).eLI(c);
                eO  = Sloops(loR).edO(c);
                
                % curve ref.
                cuR = Sedges(eR).cuR(eLI); 
                
                % transfers to tCurves information:
                tCurves(c).Ncp    = Scurves(cuR).Ncp;
                tCurves(c).Degree = Scurves(cuR).Degree;
                tCurves(c).Usize  = Scurves(cuR).Usize;
                
                if eO == 1
                    
                    tCurves(c).KnotV = Scurves(cuR).KnotV;
                    tCurves(c).w     = Scurves(cuR).w;
                    tCurves(c).P     = Scurves(cuR).P;
                    
                else
                    
                    poc = tCurves(c).Ncp;
                    
                    for po = 1:1:tCurves(c).Ncp 
                        for di = 1:1:xdim
                            tCurves(c).P(po,di) = Scurves(cuR).P(poc,di);
                        end
                        tCurves(c).w(po) = Scurves(cuR).w(poc);
                        poc = poc-1;
                    end

                    [rKnotV] = GEN_06_ReverseKnotV (Scurves(cuR).KnotV, Scurves(cuR).Ncp, Scurves(cuR).Degree);
                    tCurves(c).KnotV = rKnotV;

                    %tCurves(c).KnotV
                    
                end
                
            end
            
            tLoop(L).Noc = Noc;
            tLoop(L).LCurves = tCurves;
            
            clearvars tCurves;
        end
        

        tface(f).fLoop = tLoop ;

 
        
        % 7.- ALLOCARES SURFACE NURBS FEATURES:
        
        suR = Sfaces(faR).suR;

        tSurface.Ncp   = Ssurf(suR).Ncp;
        tSurface.Degree = Ssurf(suR).Degree;
        tSurface.Usize = Ssurf(suR).Usize;
        tSurface.KnotV = Ssurf(suR).KnotV;
        tSurface.w     = Ssurf(suR).w;
        tSurface.P     = Ssurf(suR).P;
        tSurface.PL    = Ssurf(suR).PL;
        tSurface.CS    = Ssurf(suR).OC;


        % 8.- COMPUTES NORMAL VECTOR 'n' TO SURFACE (ONLY FOR PLANAR SURFACES),
        % 'n' IS POINTING OUTWARDS THE SHELL
 
        n = zeros(xdim,1);
        

        if tSurface.PL == 1
            
        	Degree  = Ssurf(suR).Degree;
            Ncp     = Ssurf(suR).Ncp;
            KnotV   = Ssurf(suR).KnotV;
            Usize   = Ssurf(suR).Usize;
            P       = Ssurf(suR).P;
            w       = Ssurf(suR).w;
            
            [n] = i3160_nVersr (Degree, Ncp, KnotV, Usize, P ,w, xdim);
        end
        

        tSurface.n = n;
        
        
        % 9.- ALLOCATES TO tface THE TEMPORATY VARIABLES tLoop AND tSurface:
        % also clear variables for next face
        
        tface(f).fSurface = tSurface ;
        
        %{
        
        % checks bShell:
        
        if puR==2

            FACEEEE = f


            figure(f)
            scatter3(tSurface.P(:,1), tSurface.P(:,2), tSurface.P(:,3), 'r'); hold on


            for i = 1:1:prod(tSurface.Ncp)
                text(tSurface.P(i,1), tSurface.P(i,2), tSurface.P(i,3), num2str(i)); hold on
            end

            axis equal

            for c = 1:1:tface(f).fLoop(1).Noc

                tface(f).fLoop(1).LCurves(c).P

            end
        
        end
        
        %}
        
        clearvars tLoop tSurface ;
        
    end

    
    
    
    % 10.- ALLOCATES TO bShell and CLEAR VARIABLES FOR NEXT PATCH:
    
    bShell(puR).face = tface;
    clearvars tface ;
    
    %{
    if puR==1

        for f = 1:1:6
            P = bShell(puR).face(f).fSurface.P;
            Ncp = bShell(puR).face(f).fSurface.Ncp;
            
            figure(f)
            scatter3(P(:,1), P(:,2), P(:,3), 'r'); hold on
            
            for i = 1:1:prod(Ncp)
                text(P(i,1), P(i,2), P(i,3), num2str(i)); hold on
            end
            axis equal
            
            
        end
        
        
    end
    %}
end

%{

for pa = 2:1:2
    
    
    
    for f = 1:1:bShell(pa).Nfa 
        
        figure(f)
        
        P = bShell(pa).face(f).fSurface.P ;
        Ncp = bShell(pa).face(f).fSurface.Ncp ;
        
      	scatter3(P(:,1), P(:,2), P(:,3), 'r'); hold on


            for i = 1:1:prod(Ncp)
                text(P(i,1), P(i,2), P(i,3), num2str(i)); hold on
            end
            
            axis equal
            
        FACEEE=f
        bShell(pa).face(f).fSurface.n
        
    end
    
end

%}

end

