function [pnbe, pRbe, bTr, revs, pCou, bCou, BPcon] = be3123_BEtype ...
    (Npa, bShell, gbShell, Nbe, beoR, Sbsurf, Sbound, Ssurf, Scurves, xdim, udim)


% RETURNS: localizes what patches are attached to each B.E. and the trimming state

% pNbe, pbe: B.E.- patches relationship

% bTr(Nbe,2) :  trimming state: 0 / 0.5 / 1  for non-trimming / trimming no-shaping / trimming + shaping
%               stored for master and slave patches
% revs(Nbe,1): if=1 the NURBS needs to be reversed to point outwards
% bCou(Nbe,1): 0 if they are not coupling B.E's, otherwise stores coupling ref.

% pCou(No. couples, patches ref's)

% Nbe, beoR(): updated No. BE and their original refs. That happens because
% of ducplication on case of coupling

% BPcon(Nbe,1 or 2): patch associated to each B.E. might be two if is coupling

% INPUTS:

% bShell, gbShell: shell with faces information for 'bounded shells' and 'gross shells'
% nop, Nbs: No. patches and B.E.'s
% sBbsurf: boundary entities information (all are NURBS)
% sBsurfN: NURBS surfaces features, they contain all surfaces: B.E's surfaces + others, but only the latter will be used


% 0.- PRELIMINARIES:

pnbe = zeros(Npa,1);
pRbe = zeros(Npa,1); % '1' will increase to pNbe in each patch
bTr = zeros(Nbe,2); 

revs = zeros(Nbe,1);

pCou = zeros(1,2);
Ncou = 0;

bCou = zeros(Nbe,1);  % ref of coupling

BPcon = zeros(Nbe,2); % patch attached, there are 2 if it is coupling surface


% LOOP ON B.E. TO FIND WHAT PATCH / FACE ARE ATTACHED TO:

wb = waitbar(0,'characterization of B.E.');
set(wb,'Name','characterization of B.E.');
wbPos = [10 250 300 70]; % left bottom wide height
set(wb,'position', wbPos);


for buR = 1:1:Nbe 
    
    boR = beoR(buR);
    

    % buR: user reference, to be used when allocating
    % boR: orignal reference, to extract info from Sstructures
   
    
    % 1.- SURFACE REF (in IGES structures):
    
	suR = Sbsurf(boR).suR;
        

    % 2.- COMPARES B.E. WITH BOUNDED (bShell):
    
   	% 2.1.- determines: pNbe(np), pRbe(np,Nbe) and if the boundary needs to be reversed (SW=1)
    % 'np' is no. patches where the B.E. lies on, normally np = 1,  
    % unless is a coupling B.E., in that case np = 2
    
    % bPref: is bounded patch refence, can be 2 refernces if np=2
    [bPref, bFref, SW, np] = i3170_sVSshell (suR, Ssurf(suR), 1, Npa, bShell, udim,xdim);


    pnbe(bPref(1)) = pnbe(bPref(1)) + 1 ;
    pRbe(bPref(1),pnbe(bPref(1))) = buR;
    revs(buR,1) = SW(1);
        
    BPcon(buR,1) = bPref(1);
    
    if np == 2 % for coupling (No. patches = 2) add referencesm
        
        % stores coupling patches:
        Ncou = Ncou + 1 ;
        pCou(Ncou,1) = bPref(1) ;
        pCou(Ncou,2) = bPref(2) ;
        
        % changes bCou flag of involved entity:
        bCou(buR,1) = Ncou;
        
        % add to slave patch the B.E. reference:
        pnbe(bPref(2)) = pnbe(bPref(2)) + 1 ;
        pRbe(bPref(2),pnbe(bPref(2))) = buR;
        
        BPcon(buR,2) = bPref(2);
        
    end
    


    % 2.2.- Shaping feature: 
    
    % extracts contour curves corners from bFref and bEnti. 
    % Compares their max and min limits: if they are the same it is potentially a shaping boundary.
    
    [SHP] = be3127_bTr (bShell, bPref, bFref, np, Sbsurf, boR, Sbound, Scurves);
    

    % 3.- COMPARES B.E. WITH GROSS PATCHES (gbShell):
    
    % determines: bTr(Nbe,2) <= if does not belong to any gross patch is
    % trimming, it needs to determine afterwards if is shaping or not
    % Tr = 1    trimming & shaping
    % Tr = 0.5  on trimming surface, but not trimming
    % Tr = 0    on non-trimming surface
    
    
    % gPref: is gross patch refence, can be 2 references if np = 2
    [gPref, ~, ~, ~] = i3170_sVSshell (suR, Ssurf(suR), 1, Npa, gbShell, udim, xdim);
    
    if gPref(1) ~= bPref(1) 
        bTr(buR,1) = SHP(1) ; 
    end
    
    
    if np == 2 % also checks the second patch in case exists
        if gPref(2) ~= bPref(2)     
            bTr(buR,2) = SHP(2) ;  
        end
    end
    
    waitbar(buR/Nbe,wb, strcat('B.E.= ',num2str(buR+1))) ; 
    
end % loop on B.E.


delete (wb);



end

