function [bEnti, bCurv, Nbc] = be3124_bEnti ...
         (Nbe, beoR, bCou, Sbsurf, Ssurf, Sbound, Scurves, revs, bTr, BPcon, budim, bxdim)

     
% RETURNS:

% - bEnti data structure: contains No. loops bCurv and curves ref's for each
%   loop, already arranged for bEnti correct orientation.
% - bCurv data structure: curves arranged acc/to bEnti orientation


% 1.- DEFINITION OF STRUCTURES:

bEnti(Nbe) =  struct( ...,
'Ncp',[], 'Degree',[], 'Usize',[], 'KnotV',[], 'w',[], 'P',[], ...
'PL', 0,  'CS', [], 'Nol', 0,  'Noc' ,[], 'Rcu' ,[] ,'Tr', 0 , 'Cou', 0,  'Pa', []); 


% counts total No. curves = Nbe * nlo * ncu
tncu = 0 ;
for buR = 1:1:Nbe
    
    boR = beoR(buR);
    nol = Sbsurf(boR).Nob;
    
    for l = 1:1:nol
        
        lR = Sbsurf(boR).boR(l);
        
        noc = Sbound(lR).Noc;
        
        tncu = tncu + noc;
        
    end
end

bCurv(tncu) = struct( ...,
'Ncp',0, 'Degree',0, 'Usize',0, 'KnotV',[], 'w',0, 'P',0, 'PL', 0, 'CD', 0, 'conF', 0);

cxdim = bxdim;
cudim = 1;

Nbc = 0 ;

% loop on boundary entities
for buR = 1:1:Nbe
    
    boR = beoR(buR);

    % boR: boundary 'original' reference for sB structure acc/to IGES files
    % buR: boundary 'user' reference for bEnti structure & other features: revs, bTr
    

    % 2.- STORES B.E. SURFACE NURBS FEATURES:
    
    % 2.1.- Features depending on the orientation:
    
    suR = Sbsurf(boR).suR;

    bEnti(buR).PL    = Ssurf(suR).PL; % planar feature independent of orientation
    
    if revs(buR)==0 % direct reading
        
        bEnti(buR).Ncp   = Ssurf(suR).Ncp;
        bEnti(buR).Degree = Ssurf(suR).Degree;
        bEnti(buR).Usize = Ssurf(suR).Usize;
        bEnti(buR).KnotV = Ssurf(suR).KnotV;
        bEnti(buR).P     = Ssurf(suR).P;
        bEnti(buR).w     = Ssurf(suR).w;
        
        for di=1:1:budim
            bEnti(buR).CS(di,1) = Ssurf(suR).OC(di);
        end
        

        
    elseif revs(buR)==1 % reverses = swaps xi-ita
        
    	w = Ssurf(suR).w;
       	P = Ssurf(suR).P;
      	Ncpo = Ssurf(suR).Ncp;
       	[Ew,EP] = GEN_12_ExpandBw (w, P, bxdim, budim, Ncpo);
        
        mpdi = [2 1];  % perp. udim matrix
        

        % NURBS features except cp's:
        
        for di = 1:1:budim
            
            pdi = mpdi(di); % perp. udim

            bEnti(buR).Ncp(di)  = Ssurf(suR).Ncp(pdi);
            bEnti(buR).Degree(di) = Ssurf(suR).Degree(pdi);
            bEnti(buR).Usize(di) = Ssurf(suR).Usize(pdi);
            
            for i = 1:1:Ssurf(suR).Usize(pdi)
                bEnti(buR).KnotV(di,i) = Ssurf(suR).KnotV(pdi,i);
            end
             
        end
        
        
        % cp's coordinates:

        conta = 0 ;
        
        for i = 1:1:Ncpo(2)
     	for j = 1:1:Ncpo(1)
                
            conta = conta + 1;
            
            bEnti(buR).w(conta) = Ew(j,i);
            
             for di = 1:1:bxdim
                bEnti(buR).P(conta,di) = EP(j,i,di);
             end 
             
     	end
        end
        
        % close loop flag;
        for di=1:1:budim
            
            pdi = mpdi(di); % perp. udim
            bEnti(buR).CS(di,1) = Ssurf(suR).OC(pdi);
        end
  
    end

    
    % 2.2.- Features independent of orientation:
    % trimming flag, attached patches and No. loops
    
    bEnti(buR).Tr = [bTr(buR,1) bTr(buR,2)];
    
    bEnti(buR).Pa = [BPcon(buR,1) BPcon(buR,2)];
    
    
    % No. loopsof that B.E.
    NLo = Sbsurf(boR).Nob; 
    bEnti(buR).Nol = NLo; 

    % edge contours of the surface
    [edCurv] = ts3313_EdgesContours (bEnti(buR).Ncp, bEnti(buR).Degree, ...
    bEnti(buR).Usize, bEnti(buR).P,bEnti(buR).w, bEnti(buR).KnotV);

    
    % 3.- LOOP ON CONTOUR LOOPS to WORK ON CURVES:
    
    % Target: obtaining curves in each loop: No., ref's to bCurv & curves features:
    % Curves as arranged so that the contour points outwards the domain
    
    % at the same time stores curves NURBS features and No. them in bCurv and Ncu
    
    Noc = zeros(NLo,1);
    Rcu = zeros(NLo,1); % '1' might grow
    
    for lo = 1:1:NLo
        
        lR = Sbsurf(boR).boR(lo);
         
        Noc(lo) = Sbound(lR).Noc; % No. curves of the loop
        
        % 3.1.- Loop on loop curves:
        
        % uses this loop for storing curves information in bCurv and Ncu
        % and for storing No. curves and curves ref's for the relevant
        % contour loop
        
        for c = 1:1:Noc(lo)
           
            Nbc = Nbc + 1 ;
            
            Rcu(lo, c) = Nbc;
            
            ocuR = Sbound(lR).cuR(c); % original curve reference
            
            % copies to bCurv data arranged:

            bCurv(Nbc).Ncp    = Scurves(ocuR).Ncp;
            bCurv(Nbc).Degree = Scurves(ocuR).Degree;
            bCurv(Nbc).Usize  = Scurves(ocuR).Usize;

            % direction of curve remains
            if (Sbound(lR).cuO(c)==1 && revs(buR)==1) || (Sbound(lR).cuO(c)~=1 && revs(buR)~=1) 

                KV = round(Scurves(ocuR).KnotV,6) ;
                bCurv(Nbc).KnotV = KV;
                
                for po=1:1:bCurv(Nbc).Ncp 
                    for di=1:1:cxdim
                        bCurv(Nbc).P(po,di) = Scurves(ocuR).P(po,di);
                    end
                    bCurv(Nbc).w(po,1)  = Scurves(ocuR).w(po);
                end
                
                
            % direction of curve must be reversed
            elseif (Sbound(lR).cuO(c)==1 && revs(buR)~=1) || (Sbound(lR).cuO(c)~=1 && revs(buR)==1) 
                
                U = round(Scurves(ocuR).KnotV,6);
                P = Scurves(ocuR).P;
                w = Scurves(ocuR).w;
                Degree = Scurves(ocuR).Degree;
                Ncp = Scurves(ocuR).Ncp;
                
                [rU, rP, rw] = GEN_07_ReverseNURBScurve (U, P, w, Degree, Ncp, cxdim);
                     
                bCurv(Nbc).KnotV = rU;
                
                for po=1:1:bCurv(Nbc).Ncp 
                    for di=1:1:cxdim
                        bCurv(Nbc).P(po,di) = rP(po,di);
                    end
                    bCurv(Nbc).w(po,1)  = rw(po);
                end
            end

            % closed domain curve
            P1 = zeros(bxdim,1);
            P2 = zeros(bxdim,1);
                    
        	P1(:,1) = bCurv(Nbc).P(1,:);
         	P2(:,1) = bCurv(Nbc).P(bCurv(Nbc).Ncp,:);
                    
            [CO] = go2542_coincidentP (P1, P2, bxdim, 3);
                    
            if CO == 1
                bCurv(Nbc).CD = 1 ;
            else
                bCurv(Nbc).CD = 0 ;
            end

            
            % contour flag:
            bCurv(Nbc).conF = 0 ;

        end
        

        % 3.2.- Detects if curves are consecutive, if not, reverse order of
        % curves referencess in Rcu. Only for ncu > 1 and checks curves 1
        % and 2
        
        a = zeros(cxdim,1); % last cp of curve 1
        b = zeros(cxdim,1); % first cp of curve 1
        
        if Noc(lo) > 1
            
            fcr = Nbc - (Noc(lo)-1) ; % first curve ref of that loop
            scr = fcr + 1 ; % second curve ref of that loop
            
            for di=1:1:cxdim
                a(di) = bCurv(fcr).P(bCurv(fcr).Ncp,di);
                b(di) = bCurv(scr).P(1,di);
            end
            [CO] = go2542_coincidentP (a, b, cxdim, 5);
            

            if CO == 0
                
                % reverse curves refs:
                
                cr = Noc(lo);
                temp = zeros(Noc(lo),1);
                
                for c = 1:1:Noc(lo)
                    temp(cr) = Rcu(lo,c);
                    cr = cr-1;
                end
                
                for c = 1:1:Noc(lo)
                    Rcu(lo, c) = temp(c);
                end

            end
            
        end

        

    end
    

    % 4.- Transfers No. curves and bCurv ref's to bEnti:
    
    bEnti(buR).Noc = Noc ;
    bEnti(buR).Rcu = Rcu ;
    
    clearvars 'Noc' 'Rcu' ;
    

    % 5.- coupling reference:
    
    bEnti(buR).Cou = bCou(buR,1) ;


    
end



end

