function [tFace, Ntc, tCurv] = be3125_tFaces (Npa, bShell, gbShell, bxdim, budim)

% RETURNS:
% tFace(Npa): trimmed patch faces

% 1.- DECLARATION OF ARRAY OF STRUCTURES:

tFace(Npa) = struct ('Nfa', 0 , 'mSur', [] );
  
tCurv(1,1) = struct( ...,
'Ncp',0, 'Degree',0, 'Usize',0, 'KnotV',[], 'w',0, 'P',0, 'PL', 0, 'CD', 0, 'conF', 0);

Ntc = 0 ;


for pa = 1:1:Npa
    
    % 2.- GENERATES AUX. STRUCTURE tSur & SETS Nfa TO ZERO: 

    bNfa = bShell(pa).Nfa;
    
    Nfa = 0 ;
    
	tSur(bNfa,1) = struct (...
     'Ncp',[], 'Degree',[], 'Usize',[], 'KnotV',[], 'w',[], 'P',[], 'PL', 0, 'CS',[],   ...
     'Nol',0,  'Noc',[], 'Rcu',[], 'Tr', 0);
 
    % 3.- LOOP ON FACES OF bShell:

    for fa = 1:1: bNfa
        
        Nfa = Nfa + 1 ;

        
        % 3.1.- Detects if is trimming:

    	% Compares bShell.face with gbShell.If there is one
        % coincident gbShell face => it is not trimming:
        
        srf = bShell(pa).face(fa).fSurface;
        
        [aP, ~, ~, ~] = i3170_sVSshell (fa, srf, pa, pa, gbShell, budim, bxdim);

        if aP(1)==pa || aP(2)==pa
            tSur(Nfa).Tr = 0 ;
        else
            tSur(Nfa).Tr = 1 ;
        end

        
        
        % 3.2.- Surface NURBS features:
            
        % in case of needed, changes orientation by reversing one direction:
        RevD = [0 0];

        % NURBS features from bShell:

        fNcp    = bShell(pa).face(fa).fSurface.Ncp;
        fDegree = bShell(pa).face(fa).fSurface.Degree;
        fUsize  = bShell(pa).face(fa).fSurface.Usize;
        fKnotV  = bShell(pa).face(fa).fSurface.KnotV;
        fw      = bShell(pa).face(fa).fSurface.w;
        fP      = bShell(pa).face(fa).fSurface.P;
        fCS     = bShell(pa).face(fa).fSurface.CS;
        bPL     = bShell(pa).face(fa).fSurface.PL;
            
        [cNcp, cDegree, cUsize, cKnotV, cP, cw, cCS] = i3180_copyNURBSsurf ...
        (fNcp, fDegree, fUsize, fKnotV, fP, fw, fCS, bxdim, budim, RevD);

        tSur(Nfa).Ncp  = cNcp;
        tSur(Nfa).Degree = cDegree;
        tSur(Nfa).Usize  = cUsize;
        tSur(Nfa).KnotV  = round(cKnotV,6);
        tSur(Nfa).P     = cP;
            
        tSur(Nfa).PL    = bPL;
            
        tSur(Nfa).w     = cw;
        tSur(Nfa).CS    = cCS;
            
        % 3.3.- Contour loops:
            
        tSur(Nfa).Nol =  bShell(pa).face(fa).Nol;
            
        acc = 0 ;
        for lo = 1:1:tSur(Nfa).Nol
                
            tSur(Nfa).Noc(lo) = bShell(pa).face(fa).fLoop(lo).Noc;
                
            for cur = 1:1:tSur(Nfa).Noc(lo)
                acc = acc + 1 ;
                tSur(Nfa).Rcu(lo,cur) = Ntc + acc ;
            end
        end
            
            
        % 3.4.- Contour curves:
            
        for lo = 1:1:tSur(Nfa).Nol

            for cur = 1:1:tSur(Nfa).Noc(lo)
                    
                Ntc = Ntc + 1 ;

                tCurv(Ntc).Ncp    = bShell(pa).face(fa).fLoop(lo).LCurves(cur).Ncp ;
                tCurv(Ntc).Degree = bShell(pa).face(fa).fLoop(lo).LCurves(cur).Degree ;
                tCurv(Ntc).Usize = bShell(pa).face(fa).fLoop(lo).LCurves(cur).Usize;
                    
                for i = 1:1: tCurv(Ntc).Usize
                    tCurv(Ntc).KnotV(1,i) = round(bShell(pa).face(fa).fLoop(lo).LCurves(cur).KnotV(i),6) ;
                end

                    
                for po = 1:1:tCurv(Ntc).Ncp 
                    tCurv(Ntc).w(po,1) 	= bShell(pa).face(fa).fLoop(lo).LCurves(cur).w(po) ;
                    for di=1:1:bxdim
                        tCurv(Ntc).P(po,di) = bShell(pa).face(fa).fLoop(lo).LCurves(cur).P(po,di) ;
                    end
                        
                end
                    
                P1 = zeros(bxdim,1);
                P2 = zeros(bxdim,1);
                    
                P1(:,1) = tCurv(Ntc).P(1,:);
                P2(:,1) = tCurv(Ntc).P(tCurv(Ntc).Ncp,:);
                    
                [CO] = go2542_coincidentP (P1, P2, bxdim, 3);
                    
                if CO == 1
                    tCurv(Ntc).CD = 1 ;
                else
                    tCurv(Ntc).CD = 0 ;
                end

                tCurv(Ntc).conF = 0 ;
                    
            end
        end
  
    end
    
       
 
    % 4.- TRANSFERS INFO AND DELETE AUX. VBLES:

    tFace(pa).Nfa = Nfa;
    tFace(pa).mSur = tSur;
    
    clearvars('Nfa','tSur');
 
    
end
 




end

