function [XP] = s3140_soLattFitt (Ncp, gShell, pa)

% RETURNS:

% XP(n,m,3): coord's of control points defining a region bounded by 6 NURBS surfaces

% INPUTS:
% XPi(n,m,3): initial control points coord's. Include only perimetral cp's
% coords, the other are '0'.
% n,m: No. cp's in each direction

% E / Ed: young modulus of bars (horizt. and vertical) / diagonals

% creates a rectangular truss (nodes = control points) with diagonals 
% and prescibes displacements at contours to achieve final shape. Internal
% control points will follow the deformation


% Re-orders the system: 'a' unknown, 'b' known

% fa  =  Kaa  Kab  = ua  (uses first matrix row)
% fb     Kba  Kbb    ub

% fa  = Kaa ua + Kab ub => {fa - Kab ub} = Kaa ua

% ua = [Kaa]^-1  {fa - Kab ub}

xdim = 3 ;




% 1.- SHELL ENVELOPE PLANES:
% eXo( Ncp(1), Ncp(2), Ncp(3), xdim), internals are all zero

[eXo] = s3142_PlaneHull (gShell, Ncp, pa, xdim);


% 2.- INTERNALS WITHIN HULL:

[eXo] = s3143_interHull (eXo, Ncp, xdim);


% 3.- EXTRACTS SURFACES-HULL CONTROL POINTS:

[ePh] = s3141_facesCP (Ncp, gShell, pa);


% 3.- COMPUTES No. BARS & CONN. MATRICES:

[NB, ND, TB, TBD, NE, BVcon, RVcon, flagEB] = tr3241_reg3DTrussBars (Ncp);



% 5.- INITIAL POSITIONS VECTOR AND PRESCRIBED DISPLACEMENTS:

[mXo, vXo, ur] = s3144_0posPrDs (Ncp, eXo, ePh);


% [[ur(4) ur(5) ur(6)]' [ur(34) ur(35) ur(36)]' ]



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{

% represents initial lattice:

figure (12)

% bars:

for b = 1:1:TB
    
    r1 = BVcon(b,1);
    r2 = BVcon(b,2);
    plot3( [mXo(r1,1) mXo(r2,1)] , [mXo(r1,2) mXo(r2,2)] , [mXo(r1,3) mXo(r2,3)] , 'b' ); hold on
    
end


% First diag:

d1s = TB + 1 ;
d1e = TB + ND(1)*2 ;

for d = d1s:1:d1e
   	r1 = BVcon(d,1);
    r2 = BVcon(d,2);
    %plot3( [mXo(r1,1) mXo(r2,1)] , [mXo(r1,2) mXo(r2,2)] , [mXo(r1,3) mXo(r2,3)] , 'r' ); hold on
end


% Second diag:

d2s = d1e + 1 ;
d2e = d1e + ND(2)*2 ;

for d = d2s:1:d2e
   	r1 = BVcon(d,1);
    r2 = BVcon(d,2);
    %plot3( [mXo(r1,1) mXo(r2,1)] , [mXo(r1,2) mXo(r2,2)] , [mXo(r1,3) mXo(r2,3)] , 'r' ); hold on
end

% Third diag:

d3s = d2s + 1 ;
d3e = d2e + ND(3)*2 ;

for d = d3s:1:d3e
   	r1 = BVcon(d,1);
    r2 = BVcon(d,2);
    %plot3( [mXo(r1,1) mXo(r2,1)] , [mXo(r1,2) mXo(r2,2)] , [mXo(r1,3) mXo(r2,3)] , 'r' ); hold on
end

scatter3(mXo(:,1), mXo(:,2), mXo(:,3), 22, 'k', 'filled'); hold on

for i = 1:1:nNod
    text(mXo(i,1), mXo(i,2), mXo(i,3), num2str(i)); hold on
end

axis equal

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 6.- COMPUTES STIFFNESS MATRIX (sub-K MATRICES) with E=1:

[K1, K2, K3, Kd] = s3145_subKE1 (Ncp, mXo, NB, TB, TBD, BVcon);


% 7.- DEFINES KNOWN AND UN-KNOWN NODES REFERENCES

[nk, nu, dkr, dur] = tr3242_ShellKnownNodRefs (Ncp, prod(Ncp));


% 8.- WORKS OUT PARAMETRIC STUDY TO OPTIMIZE:

nNod = prod(Ncp);
DoF = nNod * xdim ;
% K = zeros(DoF,DoF);

Ntrials = 10 ;


Er = 100; % reference
Edmax = 200 ; % diag's
Edini = 0 ;
 
DEd = Edmax/Ntrials;

fcMin = 999999 ;

for td = 1:1:Ntrials
 
    Ed = Edini + DEd*td;
    E3 = Er;
	E2 = Er;
 	E1 = Er;
         
    % sums stiffness:
    K = E1*K1 + E2*K2 + E3*K3 + Ed*Kd ;
    
    % computes displacements:
    [u] = ts3231_prescribedUsol (DoF, nNod, K, ur, nk, dkr, nu, dur, xdim);

    
    % works out deformed position:
    vXod = vXo + u ;
    
    
    % Jacobians of elements (measured at center of element):
    [njF, efc, ~] = s3146_elemsJ (RVcon,vXod, NE);
    

    % only considers if non-negative Jacobians are found
    if njF == 0 
        
        efca = sum(efc)/2; % average of worst case
        
        % if worst case is lower than stored, updates
        
        if efca < fcMin
            
            fcMin = efca ;
            Xsd = vXod; % stored positions
            selT = td ;
            
        end

    end

    %%%%%%%%%%%%%%
    %{
    % plots all options
    
    if td < Ntrials+1
    
        figure(42)


        conta = 0 ;
        for i = 1:1:Ncp(1)
        for j = 1:1:Ncp(2)
        for k = 1:1:Ncp(3)

            conta = conta + 1 ;

            for di = 1:1:3

                r = conta*xdim-(xdim-di);
                Xt(conta,di) = vXod(r);
                Xte(i,j,k,di) = vXod(r);
            end

        end
        end
        end



        % one plane

        for i = 3:2:3


            g = zeros(Ncp(2),Ncp(3),xdim);
            g(:,:,:) = Xte(i,1:Ncp(2),1:Ncp(3),:);

            for j = 1:1:Ncp(2)
                plot3( g(j,:,1) , g(j,:,2) , g(j,:,3) , 'r'); hold on
            end
            for k = 1:1:Ncp(3)
                plot3( g(:,k,1) , g(:,k,2) , g(:,k,3) , 'r'); hold on
            end


        end



        % all
        for b = 1:1:TB

            n1 = BVcon(b,1);
            n2 = BVcon(b,2);

            %plot3( [Xt(n1,1) Xt(n2,1)], [Xt(n1,2) Xt(n2,2)],[Xt(n1,3) Xt(n2,3)],'r'); hold on

        end

        axis equal
    
    end
    
    
    %if gg==0
    %end
    
    
    %}
    %%%%%%%%%%%%%%

    %{
    XX(td,1) = Er ;
    XX(td,2) = Ed ;
    XX(td,3) = Er/Ed ;
    XX(td,4) = efca ;
    XX(td,5) = fcAve ;
    %}

end



% 9.- ALLOCATES NEW POSITIONS TO internal CP'S BY ADDING ua to Xo:
% Allocates to XP:

XP = zeros(nNod,xdim);

for i = 1:1:nNod
    
    for di = 1:1:xdim
        
       	r = i*xdim - (xdim) + di;

        XP(i,di) = Xsd(r);
                
    end
    
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{

% represents final lattice:


udim = 3 ;
fr = 3 ;

GEN_19_plotCN (XP, Ncp, xdim, udim, fr);


figure(fr)
scatter3(XP(:,1),XP(:,2),XP(:,3),'b'); hold on

for i = 1:1:nNod
    %text(XP(i,1), XP(i,2), XP(i,3), num2str(i) ); hold on
end

axis equal

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


end

