function [eXo] = s3142_PlaneHull (gShell, Ncp, pa, xdim)

% RETURNS:
% Nodes of envelope hull of lattice in initial position
% Initial position is obtained by planes fitted by LSM to CP's of each surface

% 1.- FITS TO EACH FACE A PLANE BY LSM, THAT PASSES THROUGH ITS AVERAGE POINT:

A = zeros(6,1); % the 6 planes constants
B = zeros(6,1);
C = zeros(6,1);
D = zeros(6,1);

for f = 1:1:6
    
    % sample points
    
    sX = gShell(pa).face(f).fSurface.P; 
    m = size(sX,1);
    
    
    % average of sX
    
    sXa = zeros(xdim,1); 
    for di = 1:1:xdim
        sXa(di) = sum(sX(:,di))/m;
    end
    

    % finds normal to fitted plane (LSM)
    
    [Np] = go2519_planeFit (sXa, sX, m, xdim);
    
    
    % allocates plane coefficients:
    
    A(f) = Np(1);
    B(f) = Np(2);
    C(f) = Np(3);
    
    for di = 1:1:xdim
        D(f) = D(f) - Np(di)*sXa(di);
    end

end


% 2.- COMPUTES THE 8 VERTEXES OF THE CUBOID (IN HIERARCHICAL ORDER):

% vertex VS plane ref:
VPcon = [1 2 3 ; 2 3 4 ; 1 3 5 ; 3 4 5 ; 1 2 6 ; 2 4 6 ; 1 5 6 ; 4 5 6 ];

vX = zeros(8,xdim);

for v = 1:1:8
    
    % matrix of coefficients:
    M = zeros(3,3);
    
    % vector LHS:
    b = zeros(xdim,1);
    
    % planes refs:
    r = zeros(3,1);
    r(1) = VPcon(v,1);
    r(2) = VPcon(v,2);
    r(3) = VPcon(v,3);
    
    % assemblies:
    for fil = 1:1:3
        M(fil,1) = A(r(fil)) ; M(fil,2) = B(r(fil)) ; M(fil,3) = C(r(fil)) ;
        b(fil) = -D(r(fil));
    end
    
    % solves and allocates:
    x = M\b;
    for di=1:1:xdim
        vX(v,di) = x(di);
    end
end


% 3.- DIVIDES EACH EDGE OF THE CUBOID ACC/TO CORRESPONDING SOLID EDGE CP's SPACING:

% edge VS vtx reference:

EVcon = [1 5 ; 2 6 ; 3 7 ; 4 8 ; ...
         1 3 ; 2 4 ; 5 7 ; 6 8 ; ...
         1 2 ; 3 4 ; 5 6 ; 7 8 ] ;

eL = zeros(12,2, xdim); % to store nodes in each segment
  
for e = 1:1:12
    
    % measures solid edge total length (Lc) and accumulated length to each cp (Lca):
    
    cNcp = gShell(pa).eCurves(e).Ncp;
    cP   = gShell(pa).eCurves(e).P ; 
    
    Lc = 0 ;
    Lca = zeros(cNcp,1);
    p1 = zeros(xdim,1);
    p2 = zeros(xdim,1);
    
    for i = 1:1:cNcp-1 
        p1(:,1) = cP(i,:) ;
        p2(:,1) = cP(i+1,:) ;
        Lc = Lc + norm(p2-p1);
        Lca(i,1) = Lc ;
    end
    
    
    % vtx refs of cuboid edge:
    
    r = zeros(2,1); 
    r(1) = EVcon(e,1);
    r(2) = EVcon(e,2);
     
  	% length of segment (by components):
    
    Ls = zeros(xdim,1);
    for di = 1:1:xdim
        Ls(di) = (vX(r(2),di) - vX(r(1),di)) ;
    end

    %{
    n = Ns(e); % No nodes in that line
    
    % lenght of each segment:
    
    L = zeros(xdim,1);
    for di = 1:1:xdim
        L(di) = (vX(r(2),di) - vX(r(1),di)) / (n-1);
    end
    %}
       
 	% computes nodes at edge:
    
    fr = 0;
    for i = 1:1:cNcp
        
        % allocates nodes position
        for di = 1:1:xdim
            eL(e,i,di) = vX(r(1),di) + fr * Ls(di);
        end
        
        % recomputes fraction
        fr = Lca(i)/Lc; % fraction
        
    end
       
    %{
    % computes nodes at each edge:
    
    acc = zeros(xdim,1);
    acc(:,1) = vX(r(1),:);
    
    for i = 1:1:n
        
        for di = 1:1:xdim
            eL(e,i,di) = acc(di);
        end
        acc = acc + L;
    end
    %}
end


% 4.- GRID OF POINTS IN EACH FACE:

% It does it as 2D manner:
% computes lines intersection (initial lattice nodes postitions):

fL = zeros(6,1,1,xdim); % to store nodes in each face

% face VS edge connection
FEcon = [ 1 3 5 7 ; 1 2 9 11 ; 5 6 9 10 ; ...
          2 4 6 8 ; 3 4 10 12 ; 7 8 11 12 ];

a1 = zeros(xdim,1);
a2 = zeros(xdim,1);
b1 = zeros(xdim,1);
b2 = zeros(xdim,1);

for f = 1:1:6
    
    er = [FEcon(f,1) FEcon(f,2) FEcon(f,3) FEcon(f,4) ]; % ref's of edges

    sNcp = gShell(pa).face(f).fSurface.Ncp;
    
    for i = 1:1:sNcp(1)

        a2(:,1) = eL(er(1),i,:);
        b2(:,1) = eL(er(2),i,:);
 
        u2 = b2-a2;
        u2 = u2 / norm(u2);

        for j = 1:1:sNcp(2)

            a1(:,1) = eL(er(3),j,:);
            b1(:,1) = eL(er(4),j,:);

            u1 = b1-a1;
            u1 = u1 / norm(u1);

            [IP, ~] = go2530_LinInters_nD (a1,u1, a2,u2, xdim,4);

            for di = 1:1:xdim
            fL(f,i,j,di) = IP(di);
            end

        end 

    end
    
end

   
% 5.- ALLOCATES TO INITIAL NODES POSITION AT FACES:

eXo = zeros(Ncp(1), Ncp(2), Ncp(3), xdim); % full expanded form

for i = 1:1:Ncp(1)
for j = 1:1:Ncp(2)
    for di = 1:1:xdim
        eXo(i,j,1,di)      = fL(1,i,j,di);
        eXo(i,j,Ncp(3),di) = fL(4,i,j,di);
    end
end
end

for i = 1:1:Ncp(1)
for k = 1:1:Ncp(3)
    for di = 1:1:xdim
        eXo(i,1,k,di)      = fL(2,i,k,di);
        eXo(i,Ncp(2),k,di) = fL(5,i,k,di);
    end
end
end

for j = 1:1:Ncp(2)
for k = 1:1:Ncp(3)
    for di = 1:1:xdim
        eXo(1,j,k,di)      = fL(3,j,k,di);
        eXo(Ncp(1),j,k,di) = fL(6,j,k,di);
    end
end
end




end

