function [aP, Pr, vr, hztF, tita] = sf3154_AbateP (P)

% RETURNS:
% aP(n,m,xdim): collection of co-planar points on rotated to horizontal plane 
% Pr,vr: point and vector of line to rotate about
% hztF = 1 if plane is already horizontal, otherwise = 0
% tita: angle to be rotated



xdim = 3 ;
n = size(P,1);
m = size(P,2);
aP = zeros(n,m,xdim);


% 1.- DETECTS POINT CLOSES TO THE ORIGIN (only computes points at the perimeter):

op = [1,1];
dmin = abs(P(1,1,3));

for i = 1:1:n
    if i==1 || i==n
        st = 1 ;
    else
        st = m-1;
    end
for j=1:st:m
    
    dtrial = abs(P(i,j,3));
    if dtrial < dmin
        
        op(1) = i; op(2) = j;
        dmin = dtrial;
    end
end
end


% 2.- SELECTS 3 POINTS NON-COLINEAR:

a = zeros(xdim,1);
b = zeros(xdim,1);
c = zeros(xdim,1);

for di=1:1:xdim
    a(di) = P(1,1,di);
    b(di) = P(1,2,di);
end

pfound=0;

for j=m:-1:1 % finds c such that is not colinear
    
    for di=1:1:xdim
        c(di) = P(n,j,di);
    end
    
    ON1 = go2520_PonSegment(c, a,b, xdim, 0.05);
    ON2 = go2520_PonSegment(a, c,b, xdim, 0.05);
    ON3 = go2520_PonSegment(b, a,c, xdim, 0.05);

    
    if (ON1==0 && ON2==0) && ON3==0
        pfound=1;
        break
    end
end



if pfound==0
    disp('cloud of points are co-linear');
end


% 3.- COMPUTES PLANE EQUATION COEFFICIENTS:

[A,B,C,D] = go2506_Plane3P (a,b,c);



% 4.- DETECTS WHERTHER IS HORIZONTAL PLANE OR NOT:

Pr = zeros(xdim,1);
vr = zeros(xdim,1);

if (abs(A)+abs(B)) < 0.000001 
    
    % 4.1.- For horizontal plane just copy:
    % note here the z coordinates is copied in all points from the first
    
    hztF = 1 ;
    tita = 0 ;
    
    for i = 1:1:n
   	for j = 1:1:m
        
        for di=1:1:xdim-1
            aP(i,j,di) = P(i,j,di);
        end
        aP(i,j,3) = P(1,1,3);
        
  	end
    end

else
    
    % 4.2.- For non-horizontal plane rotates about horizontal line,
    % that horizontal line is obtained as intersection of actual plane and
    % horizontal plane that passes through point 'op'
    
    hztF = 0 ;
    
    
    % angle of lmp-plane vs horizontal (rads) =  vector VS vertical
    
    if C == 0
        tita = 90*3.14159/180;
    else
        tita = go2501_PlaneHangle (A,B,C,D); 
    end
    
    
    
    % horizontal vector: intersec plane VS horizt.
    
    if A == 0     % special case: vector is along x-axis
        
        vr = [1;0;0];
        
    elseif B == 0 % special case: vector is along y-axis
        
        vr = [0;1;0];
        
    else        % rest of the cases
      
     	x1 = 1 ;
        y1 = -(D+x1*A)/B ;
        x2 = 2 ;
        y2 = -(D+x2*A)/B ;

        vr = [x2-x1;y2-y1;0];
        vr = vr / norm(vr); 
    end

    
    % Origin point: 

    for di=1:1:xdim
        Pr(di,1) = P(op(1),op(2),di);
    end

 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %{
    
    A
    B
    C
    D
    
    vr
    
    sc = 25 ;
    for di = 1:1:3
        x(di,1) = Pr(di);
        x(di,2) = Pr(di)+sc*vr(di);
    end
    
    figure(11)
    plot3( x(1,:), x(2,:),x(3,:),'r'); hold on

    for i = 1:1:3
        scatter3(P(i,:,1),P(i,:,2),P(i,:,3),'r'); hold on
    end
    
    axis equal
    
    %}
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    

    % goes through perimetral points rotating about defined line:
    tP = zeros(xdim,1);

    for i = 1:1:n
        
        if i==1 || i==n
            st = 1 ;
        else
            st = m-1;
        end
        
        for j=1:st:m
 
            tP(:,1) = P(i,j,:);

            
            [Pg] = go2502_RotatePaboutR (tP, Pr, vr, tita);

            aP(i,j,:) = Pg(:,1);
            
        end
        
    end
end

end

