function [XP] = sf3156_plLattFitt (Ncp, XPi, sR)

% RETURNS:

% XP(n,m,2): coord's of control points defining a region bounded by four
% co-planar NURBS curves

% INPUTS:
% XPi(n,m,2): initial control points coord's. Include only perimetral cp's
% coords, the other are '0'.
% n,m: No. cp's in each direction

% E / Ed: young modulus of bars (horizt. and vertical) / diagonals

% creates a rectangular truss (nodes = control points) with diagonals 
% and prescibes displacements at contours to achieve final shape. Internal
% control points will follow the deformation


% Re-orders the system: 'a' unknown, 'b' known

% fa  =  Kaa  Kab  = ua  (uses first matrix row)
% fb     Kba  Kbb    ub

% fa  = Kaa ua + Kab ub => {fa - Kab ub} = Kaa ua

% ua = [Kaa]^-1  {fa - Kab ub}


% 1.- PRELIMINARIES:

% 1.1.- Detects which are the two dimensions
% (those whose all coordinates are not the same):

[d, fd] = at3167_relDirs (XPi, Ncp) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
    figure(1)
    
    for i = 1:1:4
    scatter3(XPi(i,:,1), XPi(i,:,2),XPi(i,:,3),'r'); hold on
    end
    
    axis equal
    
    if gg==0
    end
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1.2.- Sets initial values:

XP = XPi;
xdim = 2 ;


% 2.- BARS AND CONNECTIVITY MATRIX:

[NB, TN, nNod, DoF, NE, BVcon, RVcon, flagEB] = at3161_setBars (Ncp, xdim);



% 3.- ENVELOPE QUADRILATERAL:

[nXo, Xo, ur] = at3162_Xo (d, Ncp, XPi, nNod, DoF, xdim, sR);





% 4.- COMPUTES STIFFNESS MATRIX (sub-K MATRICES):

[K1, K2, Kd] = at3163_K (TN, NB, BVcon, nXo, DoF, flagEB, xdim);





% 5.- DEFINES KNOWN AND UN-KNOWN NODES REFERENCES

[dkr, nk, dur, nu] = at3164_unknows (Ncp, nNod);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
if sR==6
    
    Xo
    if gg==0
    end
end
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 6.- PARAMETRIC STUDY OF RELATIVE STIFFNESS Ediag/Ebars TO OPTIMIZE THE LATTICE:

[Xsd] = at3165_XOptim (Xo, NE, RVcon, K1, K2, Kd, DoF, nNod, ur, nk, dkr, nu, dur, xdim);


 
% 7.- ALLOCATES NEW POSITIONS TO inCP'S BY ADDING u to Xo:
% Allocates to XP:

z = XPi(1,1,fd); % height of the plane: z

conta = 0;
for i = 1:1:Ncp(1)
for j = 1:1:Ncp(2)

    conta = conta + 1 ;

    XP(i,j,d(1)) = Xsd(conta*xdim - 1);
    XP(i,j,d(2)) = Xsd(conta*xdim - 0);
    XP(i,j,fd) = z ;
   
end
end



%{
% 8.- ALLOCATES NEW POSITIONS TO internal CP'S BY ADDING ua to Xo:
% Allocates to XP:

z = XPi(1,1,3); % height of the plane: z

contaU = 0;
for i = 2:1:Ncp(1)-1
for j = 2:1:Ncp(2)-1
  
    contaX = (i-1)*Ncp(2) + j ;
    contaU = contaU + 1 ;
    
    XP(i,j,1) = Xo(contaX*xdim - 1) + ua(contaU*xdim - 1);
    XP(i,j,2) = Xo(contaX*xdim - 0) + ua(contaU*xdim - 0);
    XP(i,j,3) = z ;
end
end
%}

%%%%%%%%%%%%%%%%%% represent
%{
XPi = XP ;

% dots:

n = Ncp(1);
m = Ncp(2);

xc = zeros(n*m,2);

conta = 0 ;
for i=1:1:n
for j=1:1:m
   
    conta = conta + 1 ;
    
    xc(conta,1) = -XPi(i,j,1);
    xc(conta,2) = XPi(i,j,2);
    
end 
end 

fR = abs(round(XPi(1,1,1),0))+1;
figure(fR);

scatter(xc(:,1), xc(:,2), 'k', 'Filled'); hold on

% lines 1:

for i=1:1:n
    
    x =  zeros(m,1);
    y =  zeros(m,1);
    for j=1:1:m
        
        x(j) = -XPi(i,j,1);
        y(j) = XPi(i,j,2);
    end
    figure(fR);
    plot(x,y,'k'); hold on
end



% lines 2:

for j=1:1:m
    
    x =  zeros(n,1);
    y =  zeros(n,1);
    for i=1:1:n
        
        x(i) = -XPi(i,j,1);
        y(i) = XPi(i,j,2);
    end
    figure(fR);
    plot(x,y,'k'); hold on
end


%axis equal
if gg==0
end

%}

end

