function [XP] = sf3158_MembraneFitting (XPi)

% RETURNS:

% XP(n,m,2): coord's of control points defining a region bounded by four
% co-planar NURBS curves

% INPUTS:
% XPi(n,m,2): initial control points coord's. Include only perimetral cp's
% coords, the other are '0'.
% n,m: No. cp's in each direction

% creates a rectangular truss (nodes = control points) with diagonals 
% and prescibes displacements at contours to achieve final shape. Internal
% control points will follow the deformation


% Re-orders the system: 'a' unknown, 'b' known

% fa  =  Kaa  Kab  = ua  (uses first matrix row)
% fb     Kba  Kbb    ub

% fa  = Kaa ua + Kab ub => {fa - Kab ub} = Kaa ua

% ua = [Kaa]^-1  {fa - Kab ub}


% 1.- COMPUTE MAIN QUANTITIES:

XP = XPi;
Ncp(1) = size(XP,1);
Ncp(2) = size(XP,2);

n = Ncp(1);
m = Ncp(2);

ndim = 2 ;

%{
% No. bars:
Nv =   (n)*(m-1);   % verticals
Nh = (n-1)*(m)  ;   % horizontals
Na = (n-1)*(m-1);   % diag. ascending
Nd = (n-1)*(m-1);   % diag. descending

NB = Nv + Nh + Na + Nd ;
%}

% No. nodes and elements:

nNod = n*m;
NE = (n-1)*(m-1);
DoF = nNod*ndim ;


% Known and un-known references:

nk  = 2*n + 2*(m-2); % No. known nodes
dkr = zeros (nk , 1); % known nodes references

contaR = 0;

for j = 1:1:m
    contaR = contaR + 1 ;
    dkr(contaR) = j ;
end

for i = 2:1:n-1
for j = 1:m-1:m
    contaR = contaR + 1 ;
    dkr(contaR) = (i-1)*m + j ;
end
end

for j = 1:1:m
    contaR = contaR + 1 ;
    dkr(contaR) = (n-1)*m + j ;
end


% un-known refs:

nu = nNod - nk ;
dur = zeros(nu , 1); % UN-known nodes references

contaR = 0;
for i = 2:1:n-1
for j = 2:1:m-1
  
    contaR = contaR + 1 ;
    dur(contaR,1) = (i-1)*m + j ;
    
end
end



% flagPN=1 if node has prescribed displacement:

flagPN = zeros(n*m,1);

contaN = 0;
for i = 1:1:n
for j = 1:1:m

    contaN = contaN + 1 ;
    
    if (i==1 || i==n) || (j==1 || j==m)
        flagPN(contaN) = 1 ;
    end

end
end



% flag IN for elements that are Inside (not contours)

conta = 0;
flagIN = zeros(NE,1);
for i = 1:1:Ncp(1)-1
for j = 1:1:Ncp(2)-1
    
    conta = conta + 1 ;
    if 1<i && i<Ncp(1)-1
    if 1<j && j<Ncp(2)-1
        flagIN(conta) = 1 ;
    end
    end
    
end
end


% 2.- CONNECTION MATRIX:

% pre-sizes:
conM = zeros(NE,4);

% fills:
e = 0 ;
for i = 1:1:n-1
for j = 1:1:m-1
    
    e = e+1;
    
    conM(e,1) = (Ncp(2)*(i-1))+j ;
    conM(e,2) = conM(e,1) + 1 ;
    
 	conM(e,3) = (Ncp(2)*i)+j ;
    conM(e,4) = conM(e,3) + 1 ;
    
end
end



% 3.- COMPUTES ENVELOPE OF CP'S COORDINATES (finds max and mins)
% AND COMPTES EQUALLY SPACED INCREMENTS

EX = zeros(ndim,2);

EX(1,1) = XPi(1,1,1); EX(1,2) = XPi(1,1,1);
EX(2,1) = XPi(1,1,2); EX(2,2) = XPi(1,1,2);

for i=1:1:n
for j=1:m-1:m
 
    for di=1:1:ndim
        if XPi(i,j,di) < EX(di,1)
            EX(di,1) = XPi(i,j,di) ;
        end
        if XPi(i,j,di) > EX(di,2)
            EX(di,2) = XPi(i,j,di);
        end
    end
end
end

for i=1:n-1:n
for j=1:1:m
 
    for di=1:1:ndim
        if XPi(i,j,di) < EX(di,1)
            EX(di,1) = XPi(i,j,di) ;
        end
        if XPi(i,j,di) > EX(di,2)
            EX(di,2) = XPi(i,j,di);
        end
    end
end
end


Dx = (EX(1,2) - EX(1,1)) / (n-1) ;
Dy = (EX(2,2) - EX(2,1)) / (m-1) ;


% 4.- ALLOCATES INITIAL POSITIONS TO ALL CP'S & COMPUTE TOTAL PRESCRIBED
% DISPLACEMENTS (only for contour cp's):

contaNod = 0 ;

% vectors:
Xo = zeros(DoF, 1); % initial position of points
ur = zeros(DoF, 1); % increment required for achieving final shape (only applicable to contour points!)

Xacc = zeros(ndim,1); % coordinates accumulated

Xacc(1) = EX(1,1); 

for i=1:1:n
    
    Xacc(2) = EX(2,1);
    
    for j=1:1:m

        contaNod = contaNod+1;
        
        for di=1:1:ndim
            
            r = contaNod*ndim - (ndim-di);
            
            Xo(r,1) = Xacc(di) ;
            ur(r,1) = XPi(i,j,di) - Xo(r,1);
        end
        
        Xacc(2) = Xacc(2) + Dy ;
    end
    
    Xacc(1) = Xacc(1) + Dx ;
end


% Prepares known 'b' vectors of displacements:

ub = a3212_extractU (ur, nk, dkr, ndim);

% info for stiffness matrix:

Anty = 1 ;

N = 4 ;  % No. nodes per element
nr = zeros(N,1); % nodes references

K = zeros(DoF,DoF);

% GP's:

NGP = [2 2] ;

[aGPpc, aGPw, aNGP] = ip2302_GP_nD (NGP, ndim);



% Discretizes in steps

NST = 16 ;
ubST = ub / NST ;

for st = 1:1:NST
    
    ubSTi = ubST * 1 ;
    
   % COMPUTES STIFFNESS MATRIX:

    for e = 1:1:NE

        E = 100;
        enu = 0.3 ;
        if flagIN(e)==1
             E = 20;
             enu = 0.3 ;
        end


        [D] = e2404_2DmatrixIso (E,enu, Anty);


        nr(:,1) = conM(e,:);

        kL = zeros(8,8);

        % coord's of nodes:
        Xnod = zeros(4,ndim);

        Xnod(1,1) = Xo(nr(4)*2-1);
        Xnod(1,2) = Xo(nr(4)*2);
        Xnod(2,1) = Xo(nr(2)*2-1);
        Xnod(2,2) = Xo(nr(2)*2);
        Xnod(3,1) = Xo(nr(1)*2-1);
        Xnod(3,2) = Xo(nr(1)*2);
        Xnod(4,1) = Xo(nr(3)*2-1);
        Xnod(4,2) = Xo(nr(3)*2);

        for g = 1:1:aNGP

            % jacobian at Gp:
            uG = [aGPpc(g,1) aGPpc(g,2)];
            [J, dJ] = j2311_JPlaneRect (uG, Xnod);

            % deformatino matrix:
            [B] = ms3202_2DplaneB (J, uG);

            % add contribution to stiffness:
            kL = kL + B'*D*B*dJ*aGPw(g);

        end

        [K] = a3211_insertinK (K, kL, nr, N, ndim);

    end

    % 6.- COMPUTES Kaa, Kab AND ub:

    mnr = [dur dur] ;
    Kaa = a3213_extractK (K, [nu,nu]', mnr, ndim);

    % nodes ref's matrix
    if nk == nu

        mnr = [dur dkr];

    elseif nk > nu

        mnr = [ [dur; zeros(nk-nu,1)] dkr];

    elseif nk < nu

        mnr = [ dur [dkr; zeros(nu-nk,1)]];

    end

    Kab = a3213_extractK (K, [nu,nk]', mnr, ndim);
 
    
    % SOLVES DISPLACEMENTS ua (for internals):

    fa = -Kab*ubSTi;
    ua = Kaa\fa ;

    % expand to all 

    u = zeros(n*m*ndim,1);

    a = 0 ;
    b = 0 ;
    for i = 1:1:n*m

        if flagPN(i)==1

            b = b + 1 ;

            u(i*ndim-1) = ubSTi(b*ndim-1) ;
            u(i*ndim-0) = ubSTi(b*ndim-0) ;

        else

            a = a + 1 ;

            u(i*ndim-1) = ua(a*ndim-1) ;
            u(i*ndim-0) = ua(a*ndim-0) ;

        end

    end

    
    % Updates positions:
    Xo = Xo + u ;


    
end



% Allocates to XP:

contaU = 0;
for i = 2:1:n-1
for j = 2:1:m-1
  
    contaX = (i-1)*m + j ;
    contaU = contaU + 1 ;
    

    XP(i,j,1) = Xo(contaX*ndim - 1) + ua(contaU*ndim - 1);
    XP(i,j,2) = Xo(contaX*ndim - 0) + ua(contaU*ndim - 0);
end
end

% Allocates response


%%%%%%%%%%%%%%%%%% represent
%{
XPi = XP ;

% dots:

xc = zeros(n*m,2);

conta = 0 ;
for i=1:1:n
for j=1:1:m
   
    conta = conta + 1 ;
    
    xc(conta,1) = XPi(i,j,1);
    xc(conta,2) = XPi(i,j,2);
    
end 
end 

fR = abs(round(XPi(1,1,1),0))+1;
figure(fR);

scatter(xc(:,1), xc(:,2), 'b'); hold on

% lines 1:

for i=1:1:n
    
    x =  zeros(m,1);
    y =  zeros(m,1);
    for j=1:1:m
        
        x(j) = XPi(i,j,1);
        y(j) = XPi(i,j,2);
    end
    figure(fR);
    plot(x,y,'--r'); hold on
end



% lines 2:

for j=1:1:m
    
    x =  zeros(n,1);
    y =  zeros(n,1);
    for i=1:1:n
        
        x(i) = XPi(i,j,1);
        y(i) = XPi(i,j,2);
    end
    figure(fR);
    plot(x,y,'--r'); hold on
end


axis equal

%}

end

