function [nXo, Xo, ur] = at3162_Xo (d, Ncp, XPi, nNod, DoF, xdim, sR)

% RETURNS:
% nXo(nNod, xdim)    % initial lattice nodes positions: node by node matrix form
% Xo(DoF,1);         % vector form
% ur(DoF,1): increment required for achieving final shape (only applicable to contour points!)


% INPUTS:
% d(2): directions that are relevant from the XPi coordinates
% Ncp(2): No. cp's in each param. direction of the surface
% XPi(i,j,3): cp's coordinates


% 1.- FITS TO EACH CONTOUR A LINE BY LSM, THAT PASSES THOUGH ITS AVERAGE POINT:

A = zeros(4,xdim); % line point
D = zeros(4,xdim); % line direction versor

mNcp = [Ncp(1) Ncp(1) Ncp(2) Ncp(2)] ;

sXc = zeros(4,1,xdim); % stores curve cp's coord's
Lc = zeros(4,1); % control polygons lenghts

for c = 1:1:4

    % sample points
    
    m = mNcp(c);
    sX = zeros(m,xdim);
    
    for i = 1:1:m
        
        % stores cp's coordinates:
        for di = 1:1:xdim

            h = d(di);
            
            if c==1

                sX(i,di) = XPi(i,1,h);
                sXc(c,i,di) = XPi(i,1,h);

            elseif c==2

                sX(i,di) = XPi(i,Ncp(2),h);
                sXc(c,i,di) = XPi(i,Ncp(2),h);

            elseif c==3

                sX(i,di) = XPi(1,i,h);
                sXc(c,i,di) = XPi(1,i,h);

            elseif c==4

                sX(i,di) = XPi(Ncp(1),i,h); 
                sXc(c,i,di) = XPi(Ncp(1),i,h);

            end 

        end

        % computes control polygon length
        if i>1
            acc = 0;
            for di = 1:1:xdim
                acc = acc + (sXc(c,i,di)-sXc(c,i-1,di))^2;
            end
            Lc(c,1) = Lc(c,1) + acc^0.5;
        end

    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %{

        
        figure(1)
        
        scatter(sX(:,1), sX(:,2), 'r'); hold on
        
        % axis equal


    %}
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    
    % average of sX
    
    sXa = zeros(xdim,1); 
    for di = 1:1:xdim
        sXa(di,1) = sum(sX(:,di))/m;
    end
    
    
    
    % finds versor of fitted line (LSM)

    [Dp] = go2521_lineFit (sXa, sX, m, xdim);
    
    % allocates line coefficients:
    
    A(c,:) = sXa(:,1);
    D(c,:) = Dp(:,1);

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{



    sc = 500 ;
    figure(sR)
    for c = 1:1:4
        
        scatter( A(c,1), A(c,2) ,'b'); hold on
        plot( ...
            [A(c,1) , A(c,1) + sc*D(c,1)], ...
            [A(c,2) , A(c,2) + sc*D(c,2)],  'b'); hold on
    end
    
    axis equal
    
    if gg==0
    end
    


%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 2.- COMPUTES THE 4 VERTEXES OF THE RECTANGLE (IN HIERARCHICAL ORDER):
% Checks if there is any internal intersection and if any resultant edgle
% leght is too close to 0. If so, amends the corners.

[vX] = at3166_vX (A,D, xdim);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{


    figure(1)
    evX = [vX ; [vX(1,1) vX(1,2)] ];

    r = [1 3 4 2 1 ];

    for i=1:1:4

        plot([evX(r(i),1) evX(r(i+1),1)],[evX(r(i),2) evX(r(i+1),2)],'k'); hold on

    end

    %axis equal

    if gg==0
    end


%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% 3.- DIVIDES EACH OF THE 4 EDGES:

EVcon = [1 3 ; 2 4 ; 1 2 ; 3 4] ; % edge VS vtx reference

eL = zeros(4,max(mNcp),xdim); % to store nodes in lines 1 to 4
  
for e = 1:1:4
    
    r = zeros(2,1); % vtx refs
    r(1) = EVcon(e,1);
    r(2) = EVcon(e,2);

    % length of segment (by components):
    Ls = zeros(xdim,1);
    for di = 1:1:xdim
        Ls(di) = (vX(r(2),di) - vX(r(1),di)) ;
    end


    % computes nodes at edge:
    
    Lca = 0 ; % acc. length of control polygon
    fr = 0;
    for i = 1:1:mNcp(e)-1
        
        % allocates nodes position
        for di = 1:1:xdim
            eL(e,i,di) = vX(r(1),di) + fr * Ls(di);
        end
        
        % recomputes fraction
        s = 0 ;
        for di = 1:1:xdim
            s = s + (sXc(e,i+1,di)-sXc(e,i,di))^2;
        end
        s = s^0.5;
        Lca = Lca + s ;
        
        fr = Lca/Lc(e); % fraction
        
    end
    
    % last vertex is always the corner:
    i = i+1;
    for di = 1:1:xdim
        eL(e,i,di) = vX(r(2),di);
  	end
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555
%{

size(eL)

for e = 1:1:4
    
    figure(88)
    
    scatter(eL(e,:,1), eL(e,:,2), 'r'); hold on
    axis equal
    
end

if gg==0
end

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555


% 4.- COMPUTES LINES INTERSECTIONS:
% Initial lattice nodes positions: Xo and nXo
% Required displacement vector: ur

eXo = zeros(Ncp(1), Ncp(2), xdim); % full expanded form
nXo = zeros(nNod, xdim);    % node by node matrix form
Xo  = zeros(DoF,1);         % vector form

ur = zeros(DoF, 1); % increment required for achieving final shape (only applicable to contour points!)

contaNa = 0 ;
contaNb = 0 ;

% end points of lines in directions 1 and 2:

a1 = zeros(xdim,1);
a2 = zeros(xdim,1);
b1 = zeros(xdim,1);
b2 = zeros(xdim,1);

for i = 1:1:Ncp(1)
    
    a2(:,1) = eL(1,i,:);
    b2(:,1) = eL(2,i,:);
    
    u2 = b2-a2;
    u2 = u2 / norm(u2);
    
    for j = 1:1:Ncp(2)

    	a1(:,1) = eL(3,j,:);
        b1(:,1) = eL(4,j,:);

      	u1 = b1-a1;
        u1 = u1 / norm(u1);
 
        [IP, ~] = go2530_LinInters_nD (a1,u1, a2,u2, xdim,4);

        
        contaNa = contaNa+xdim;
        contaNb = contaNb+1;
        
        for di = 1:1:xdim
            
            h = d(di);
            
          	r = contaNa-xdim+di ;
            
            eXo(i,j,di) = IP(di);
            nXo(contaNb,di) = IP(di);
            
            Xo(r,1) = IP(di);
            
            ur(r,1) = XPi(i,j,h) - Xo(r,1); % note, it will use only the boundary surface
            
        end
        
    end 
    
end




%{

size (eXo)

figure(5)

for i = 1:1:Ncp(1)-1
for j = 1:1:Ncp(2)

    plot( [eXo(i,j,1) eXo(i+1,j,1)], [eXo(i,j,2) eXo(i+1,j,2)], 'k' ); hold on 
    
end
end


for i = 1:1:Ncp(1)
for j = 1:1:Ncp(2)-1

    plot( [eXo(i,j,1) eXo(i,j+1,1)], [eXo(i,j,2) eXo(i,j+1,2)], 'k' ); hold on 
    
end
end

scatter(nXo(:,1),nXo(:,2),30,'filled','k'); hold on
% axis equal

if gg==0
end

%}


end

