function [vX] = at3166_vX (A,D, xdim)

% RETURNS:
% vX(4,xdim): vertex coordinates of the initial quadrilateral
% Does 2 checks:
%   - crossed quadrilateral (see sect. 4)
%   - close to zero side (see sect. 5)

x = zeros(xdim,1); % point (in general)

vr = zeros(xdim,1); % versors
vs = zeros(xdim,1);

br = zeros(xdim,1); % points
bs = zeros(xdim,1);

nd = 4 ;


% 1.- COMPUTES VERTEXES BY INTERSECTION BTW RELEVANT LINES:

VLcon = [1 3 ; 2 3 ; 1 4 ; 2 4 ]; % vertex VS line refs

vX = zeros(4,xdim);

for v = 1:1:4
    
    % lines refs:
    r1 = VLcon(v,1);
    r2 = VLcon(v,2);

	vr(:,1) = D(r1,:);
    vs(:,1) = D(r2,:);

    br(:,1) = A(r1,:);
    bs(:,1) = A(r2,:);
    
    [ix, ~, ~, ~] = go2526c_parLinInts (vr, vs, br, bs, xdim, nd);
    
    for di=1:1:xdim
        vX(v,di) = ix(di);
    end
    
end



% 2.- PARAMETER LIMITS OF THE LINES:
% i.e.: parameter values at computed corners:

tL = zeros(4,2);

LVcon = [1 3 ; 2 4 ; 1 2 ; 3 4 ]; % line VS vertex refs

for c = 1:1:4
    
	vr(:,1) = D(c,:);
    br(:,1) = A(c,:);
    
    for v = 1:1:2
        x(:,1) = vX(LVcon(c,v),:);
        t = go2533_Lpar2x (vr, br, x, xdim, nd);
        tL(c,v) = t ;
    end
    
    % arrange in order:
    if tL(c,2) < tL(c,1)
        tm = tL(c,2);
        tL(c,2) = tL(c,1);
        tL(c,1) = tm ;
    end
end



% 3.- CHECKS IF LINES ARE CROSSED:
% Lines 1 vs 2, and lines 3 vs 4:

clR = 0; % changes to line ref in case the quadrilateral is crossed

for i = 1:2:3
    
    % lines refs:
    r1 = i;
    r2 = i+1;

    % finds intersection and their parameter value 'ti'
    vr(:,1) = D(r1,:);
    vs(:,1) = D(r2,:);

    br(:,1) = A(r1,:);
    bs(:,1) = A(r2,:);
    
    [ix, tir, tis, pF] = go2526c_parLinInts (vr, vs, br, bs, xdim, nd);
    
    % if intersection is within the parameter limits => the quadrialteral is crossed !
    if pF==0
    if tL(r1,1)<tir && tir<tL(r1,2)  
        clR = i ; % crossing lines ref.
        break
    end
    end
end

% lines to check
if clR==1
    cC = [3 4];
elseif clR==3
    cC = [1 2];
end


% 4.- IN CASE OF CROSSED QUADRILATERAL, AMENDS:

if clR > 0
    
    % 4.1.- Rearranges direction to creaate a contour such that the
    % interior lies at LHS of lines sense. Note it keeps the same lines
    % refs, it just rearrange the sense!
    
    cA = zeros(4,xdim);
    cD = zeros(4,xdim);
    
    cLVcon = [1 3 ; 4 2 ; 3 4 ; 2 1 ]; % line VS vertex refs
    
    for c = 1:1:4
        
        v1 = cLVcon(c,1);
        v2 = cLVcon(c,2);
        
        tD = zeros(xdim,1);
        for di = 1:1:xdim
            cA(c,di) = vX(v1,di);
            tD(di,1) = vX(v2,di)-vX(v1,di);
        end
        
        tD = tD/norm(tD);
        cD(c,:) = tD(:,1);
    end
    

    
    % 4.2.- The line that is inverted has the intersection point 'ix' at the
    % RHS of the contour sense:
    
    ir = 0 ; % ref. of inverved line, to amend
    
    for i = 1:1:2
        
        r = cC(i);

        br(:,1) = cA(r,:);
        vr(:,1) = cD(r,:);
        
        [~, d] = go2527_distPtoRecta (ix, br, vr, xdim);
        
        m = [d,vr];
        cp = det(m);
        
        % lies at RHS
        if cp < 0
            ir = r;
            break
        end
    end
    
    
    % 4.3.- Swappes the vertexes position of 'ir' line:
    % (Now uses the original vertex refs)
    
    sR = [LVcon(ir,1) LVcon(ir,2)];
    
    tvX = vX ;
    
    for di = 1:1:di
        tvX(sR(1),di) = vX(sR(2),di);
        tvX(sR(2),di) = vX(sR(1),di);
    end


    % and re-allocates:
    vX = tvX ;
    
end


% 5.- CHECKS IF ONE LINE IS TOO SHORT AND EXTENDS IF RQURIED:
% Any side shortet than 'pt' (percentage threshold) than the average side
% lenght is increase up to 1.2 times that percentage

pt = 5 ;

% 5.1.- Computes lenght average:

Lav = 0 ;

for c = 1:1:4
    
    v1 = LVcon(c,1);
    v2 = LVcon(c,2);
    
    L = 0 ;
    for di = 1:1:xdim
       L = L + (vX(v2,di)-vX(v1,di))^2; 
    end
    
    L = L^0.5;
    Lav = Lav + L/4;
    
end

% 5.2.- Checks if any lenght is shorter and increases if so:
% {
for c = 1:1:4
    
	v1 = LVcon(c,1);
    v2 = LVcon(c,2);

	u = zeros(xdim,1);  % versor from v1 to v2           
    
    for di = 1:1:xdim
       u(di,1) = vX(v2,di)-vX(v1,di);
    end
    
    Lc = norm(u); % length
    u = u / Lc ;

    % if shorter than treshold => extends
    if Lc < pt/100*Lav 
        

        DL = 1.2*pt/100*Lav - Lc ; % DL/2 each end

        % recomputes positions
        
        x1 = zeros(xdim,1);
        x2 = zeros(xdim,1);
        
        
        x1(di,1) = vX(v1,di) - DL/2*u(di);
        
        for di = 1:1:xdim
            x1(di,1) = vX(v1,di) - DL/2*u(di);
            x2(di,1) = vX(v2,di) + DL/2*u(di);
        end
        
        % allocates:
        vX(v1,:) = x1(:,1);
        vX(v2,:) = x2(:,1);

    end
    
end
%}


end

