function [teCurves] = gp3171_curvesHomog (teCurves, rdi, xdim, ndig)

% RETURNS:
% teCurves with same Ncp, Degree and KnotV

% INPUTS:
% teCurves: structure to update
% di: relevant parameter direction


% 1.- READS RELEVANT DATA FROM THE 4 CURVES 
% (THEY CORRESPOND TO PARAMETER DIRECTION 'di')

allNcp =   zeros(1,4);   
allDegree = zeros(1,4);  

for c = 1:1:4
	allNcp(1,c)    = teCurves(c+4*rdi).Ncp;
  	allDegree(1,c) = teCurves(c+4*rdi).Degree;
end


% 1.1.- Target No. cp's, degree and max. knot vector size:

tNcp = max(allNcp);
tDegree = max(allDegree);


% 1.2.- Envelope knot vector 'tuL':
        
tuL=zeros(2,1); % total param. span limits (all curves knots in a single array)
tnu = 0;
        
for c = 1:1:4
            
    cR = c+4*rdi ; % tcurve ref.
            
	[ns, spLims] = GEN_13_NonVoid_spans (teCurves(cR).KnotV,teCurves(cR).Ncp,teCurves(cR).Degree);
            
	% firt curve knot spans
    if c==1
                
        for i = 1:1:ns+1
            tnu = tnu + 1 ;
            tuL(tnu,1) = spLims(i);
        end
            
        % stores next curves if there are not repeated:
    else
                
        for i = 1:1:ns+1
                    
            ucandidate = spLims(i);
            addF = 1 ; % adding to tuL flag
                    
            for j = 1:1:tnu
                if round(ucandidate,ndig) == round(tuL(j,1),ndig)
                    addF = 0;
                    break
                end
            end
                    
            if addF == 1
                tnu = tnu + 1 ;
                tuL(tnu,1) = ucandidate;
            end
        end 
    end
    
end
        
tuL = sortrows(tuL);
tuL = round(tuL,ndig);


% 2.- ACHIEVES CONSISTENCY IN DEGREE and REMOVES ANY REPEATED INTERNAL KNOT:
% (recall that contours must be drawn smoothly)

for c = 1:1:4 
    
    cR = c+4*rdi ; % tcurve ref.
    
    
    % 2.1.- CURVE DATA :
    
    Ncp = teCurves(cR).Ncp;
    KnotV = round(teCurves(cR).KnotV,ndig);
    P = teCurves(cR).P;
    w = teCurves(cR).w;
    Degree = teCurves(cR).Degree ;
    Usize = teCurves(cR).Usize;
    
	if size(KnotV,1)>1
        KnotV = KnotV' ;
	end
    
    % 2.2.- DEGREE:
    
    if Degree < tDegree

        [Ncp, KnotV, P, w, ~, Usize] = GEN_03_02_pRefinement...
      	(Ncp, KnotV, Degree, P, w, tDegree, Ncp, Usize, 1, xdim);
                
     	teCurves(cR).Degree = tDegree;    
    end   
    
    
    % 2.3.- REMOVES REPEATED KNOTS:
    
    NRk = 0; % No. repetated values
    nr = zeros(1,1); % No. of repetitions to remvoe (e.g. if two values then nr = 1)
    ur = zeros(1,1); % knots values
    
    
    [ns, spLims] = GEN_13_NonVoid_spans (KnotV,Ncp,tDegree);
    
    for k = 2:1:ns
        
        utemp = spLims(k);
        ntemp = sum(KnotV(:)==utemp);
        
        if ntemp > 1
            NRk = NRk+1;
            ur(NRk,1) = utemp;
            nr(NRk,1) = ntemp-1;
        end
    end
    
    
    for k = 1:1:NRk % removes them
        
        u = ur(k,1);
        
        for i = 1:1:nr(k) 
            [P, KnotV, w, Ncp, Usize] = ...
            GEN_03_13_1D_kRemoval (Ncp ,tDegree, KnotV, Usize, P, w, u, xdim);
        end
    end
    
    

    % 2.4.- ALLOCATES TO CURVE:
    
    teCurves(cR).Ncp = Ncp ;
    teCurves(cR).KnotV = KnotV;
    teCurves(cR).P = P ;
    teCurves(cR).w = w ;
    teCurves(cR).Usize = Usize;

end




% 3.- ACHIEVES CONSISTENT Ncp:
% if insert knots, they are first taken from locations given by tuL, if not
% inserts new ones as longest mid-span

for c = 1:1:4 % loop on curves
            
    cR = c+4*rdi ; % tcurve ref.
       

    % needs knot insertion
	if teCurves(cR).Ncp < tNcp 
             
        % No. knots to add
        addNcp =  tNcp - teCurves(cR).Ncp ;
               
        for i = 1:1:addNcp 

            % checks if any of the tuL components is not there, if so adds
            for j = 2:1:tnu-1
                
                ucandidate = tuL(j);
                
                [teCurves, addF] = gp3172_tuLins (teCurves,cR, ucandidate, ndig, xdim);

                if addF==1
                    break
                end
            end
            
            % if no tuL knot was inserted, just inserts one at mid-point of
            % the largest knot span
            
            if addF==0
                
                % detects longest span:
                [ns, spLims] = GEN_13_NonVoid_spans (teCurves(cR).KnotV,teCurves(cR).Ncp,teCurves(cR).Degree);
                
                spR = 1 ;
                spL = spLims(2)-spLims(1);
                
                for j = 2:1:ns
                    
                    tspL = spLims(j+1)-spLims(j);
                    if tspL > spL
                        spL = tspL;
                        spR = j;
                    end
                end
                
                % computes its mid-point:
                au = spLims(spR) + 0.5*spL;
                
                % inserts au:
                [teCurves(cR).P, teCurves(cR).KnotV, teCurves(cR).w, teCurves(cR).Ncp, teCurves(cR).Usize] = ...
                GEN_03_10_1D_kSingleExt...
                (teCurves(cR).Ncp, teCurves(cR).Degree, teCurves(cR).KnotV, teCurves(cR).P, teCurves(cR).w, au, 1, xdim, 1);

            end
            
        end
          
    end
    
end



% 4.- ACHIEVES CONSISTENT KNOT VECTOR:

% inserts knots in each curve if required when comparing with tuL:
        
for c = 1:1:4
            
    cR = c+4*rdi ; % tcurve ref.

    for j = 1:1:tnu

        ucandidate = tuL(j);
        [teCurves, ~] = gp3172_tuLins (teCurves,cR, ucandidate, ndig, xdim);

    end   
    
    
end
        




end

