function [tSurface] = gp3175_surfHomog (tSurface, teCurves, ecR, ndig)

% RETURNS;
% tSurface consistent with its edge curves


% 1.- READS RELEVANT DATA FROM THE 4 CURVES 
% (THEY CORRESPOND TO PARAMETER DIRECTION 'di')

udim = 2 ;
xdim = 3 ;

% ref. curves
 
rc1 = ecR(1,1);
rc2 = ecR(2,1);




% target Ncps, degree, Usize and Knot V
 
tNcp = [ teCurves(rc1).Ncp ; teCurves(rc2).Ncp ];
tDegree = [ teCurves(rc1).Degree ; teCurves(rc2).Degree ];
tUsize = [ teCurves(rc1).Usize ; teCurves(rc2).Usize ];

tKnotV = zeros(udim,max(tUsize));

for i = 1:1:tUsize(1)
    tKnotV(1,i) = teCurves(rc1).KnotV(i);
end

for i = 1:1:tUsize(2)
    tKnotV(2,i) = teCurves(rc2).KnotV(i);
end   


% features of surface:

Ncp = tSurface.Ncp;
Degree = tSurface.Degree;
KnotV = round(tSurface.KnotV,ndig);
P = tSurface.P;
w = tSurface.w;
Usize = tSurface.Usize;

noc = Ncp(1)*Ncp(2) ;




% 2.- ACHIEVES CONSISTENCY IN DEGREE and REMOVES ANY REPEATED INTERNAL KNOT:
%    (recall that contours must be drawn smoothly)

if Degree(1)<tDegree(1) || Degree(2)<tDegree(2)

    [Ncp, KnotV, P, w, ~, Usize] = GEN_03_02_pRefinement...
    (Ncp, KnotV, Degree, P, w, tDegree, noc, Usize, udim, xdim);

    Degree = tDegree;    
    
    % pending to remove repeated knots, but I believe this degree issue
    % neves appears since the edge curves degree normally is consistent
    % with these surfaces
    
end


% 3.- ACHIEVES CONSISTENT Ncp:
% if insert knots, they are first taken from locations given by tKnotV, if not
% inserts new ones as longest mid-span

for di = 1:1:udim
    
    if Ncp(di) < tNcp(di)
        
        % No. knots to add
        addNcp =  tNcp(di) - Ncp(di) ;
           
        jstart = Degree(di)+2;
        
        for i = 1:1:addNcp 

            % checks if any of the tKnotV components is not there, if so adds
            for j = jstart : 1 : tUsize(di)-Degree(di)
                
                ucandidate = tKnotV(di,j);
                       
                [P, KnotV, w, Ncp, Usize, addF] = gp3176_tKnotVins ...
                    (P, KnotV, w, Ncp, Degree, Usize, di, ucandidate, ndig, xdim);

                if addF==1
                    break
                end
            end
            

            
            % if no tKnoV knot was inserted, just inserts one at mid-point of
            % the largest knot span
            
            if addF==0
                
                % detects longest span:
 
                U = KnotV(di,1:Usize(di));

                [ns, spLims] = GEN_13_NonVoid_spans (U, Ncp(di), Degree(di));
                
                spR = 1 ;
                spL = spLims(2)-spLims(1);
                
                for j = 2:1:ns
                    
                    tspL = spLims(j+1)-spLims(j);
                    if tspL > spL
                        spL = tspL;
                        spR = j;
                    end
                end
                
                
                % computes its mid-point:
                
                au = spLims(spR) + 0.5*spL;
                
                % inserts au:
                
                U = KnotV(di,1:Usize(di));
                
            	[P, U, w, Ncp, Utam] = GEN_03_10_1D_kSingleExt...
                    (Ncp, Degree(di), U, P, w, au, di, xdim, udim);
                
                for i = 1:1:Utam
                    KnotV(di,i) = U(i);
                end

                Usize(di) = Utam;
      
            end
            
        end
          
    end

end



% 4.- ACHIEVES CONSISTENT KNOT VECTOR:

% inserts knots in each curve if required when comparing with tKnotV:
      
jstart = Degree(di)+2;

for di = 1:1:udim
for j = jstart : 1 : tUsize(di)-Degree(di)
                
    ucandidate = tKnotV(di,j);
                       
	[P, KnotV, w, Ncp, Usize, addF] = gp3176_tKnotVins ...
  	(P, KnotV, w, Ncp, Degree, Usize, di, ucandidate, ndig, xdim);

end
end


    
% 5.- ALLOCATES TO SURFACE:
  
tSurface.Ncp = Ncp;
tSurface.Degree = Degree;
tSurface.KnotV = KnotV;
tSurface.P = P;
tSurface.w = w;
tSurface.Usize = Usize;


end

