function [Nso, Smsbo, Nsh,Sshells, Nfa, Sfaces, Nsn, Ssurf, Nlo, Sloops, Ned, Sedges,...
    Ncu, Scurves, Nve, Svertexes, Nbs, Sbsurf, Nbo, Sbound, Nsr, SsurfR, Nli, Sline] = ...
    i3150_IGEStoMat (Fname, Npa, NL)

% RETURNS: graphic objects transferred to matrices, each type has its own structure

% note that references to dependant objects are already changed to local
% structure order. 
% E.g.: 
% Sshells(s).fR(f) = order in the Sfaces() structure ;


% 1.- TRANSFERS TO NUMERICAL ARRAYS:

[Nso, msbo, Nsh, shells, Nfa, faces, Nsn, surfN, Nlo, loops, Ned, edges, ...
Ncu, curves, Nve, vertexes, Nbs, bsurf, Nbo, bound, Nsr, surfR, Nli, line] = ...
ri3191_toArrays (Npa, NL, Fname);



% 2.- CREATE NUMERICAL STRUCTURES:

% 2.1.- MSBO: shell reference attached to that MSBO

Smsbo(Nso) = struct('shR',0);

for s = 1:1:Nso
    Smsbo(s).shR = msbo(s,2);
end


% 2.2.- Shells: own ref., No. faces, face ref., face orientation

Sshells(Nsh) = struct('R',0,'Nof',0,'fR',[],'fO',[]);


for s = 1:1:Nsh

    Sshells(s).R = shells(s,1);
    Sshells(s).Nof = shells(s,2);
    
    j=1;
    for i=1:1:Sshells(s).Nof
        
        Sshells(s).fR(i)= shells(s,2+j);
        
        % NOTE: orientation is swapped due to IGES data structure of NURBS surfaces
        % that is done at the end of this routine
        
        Sshells(s).fO(i)= shells(s,2+j+1); 
        
        j = j+2;
        
    end
end



% 2.3.- Faces: own ref., surface ref., No. loops, Inner/Outer loop (0/1), Loops ref.

Sfaces(Nfa) = struct('R',0, 'suR',0, 'Nol',0, 'IOL',[],'lR',[]);

for f=1:1:Nfa

    Sfaces(f).R   = faces(f,1);
    Sfaces(f).suR = faces(f,2);
    Sfaces(f).Nol = faces(f,3);
    Sfaces(f).IOL = faces(f,4);
       
    for i=1:1:Sfaces(f).Nol
        Sfaces(f).lR(i)  = faces(f,4+i);
    end
  
end



% 2.4.- Surfaces: own ref., No. control points, Degree, Open/Close (0/1),
% Rational/Polynomial (0/1), non-peridodical/Periodical (0/1),
% Usize, KnotV, cp's w and Xcoors

% NOTE !! in surfaces we swap Ncp, Degree, Usize, and KnotV

udim = 2;
xdim = 3;

Ssurf(Nsn) = struct('R',0,'Ncp',[],'Degree',[], 'OC',[],'RP',0,'PN',[],'Usize',[], 'KnotV',[], 'w',[],'P',[], 'PL', 0);

for s = 1:1:Nsn
    
    Ssurf(s).R = surfN(s,1);
    
    
    % parameters in each direction
    accK = 10;  % position just before Knot vector
    noc = 1 ;   % total No. cp's
    
    sdi = [2 1];
    for di=1:1:udim
        
        pd = sdi(di);
        
        n = surfN(s,1+di)+1;
        p = surfN(s,3+di);
        
        noc = noc * n ;
        
        Ssurf(s).Ncp(pd)    = n;
        Ssurf(s).Degree(pd) = p;
        Ssurf(s).OC(pd)     = surfN(s,5+di);
        Ssurf(s).PN(di)     = surfN(s,8+di);
        
        Ssurf(s).Usize(pd)  = n+p+1;
        
        for i=1:1:n+p+1
            Ssurf(s).KnotV(pd,i) = surfN(s,accK+i);
        end
        
        accK = accK+n+p+1;
    end
    
    
    % normalizes Knot vectors: from 0 to 1
    
    for di=1:1:udim
        
        Us = Ssurf(s).Usize(di); % size of KnotV
        
        % extreme values of knot vector
        k1 = Ssurf(s).KnotV(di,1);
        k2 = Ssurf(s).KnotV(di,Us);
        
        for i=1:1:Us
            Ssurf(s).KnotV(di,i) = (Ssurf(s).KnotV(di,i)-k1) / (k2-k1);
        end 
    end
    
    
    % rational or polynomal:
    Ssurf(s).RP = surfN(s,8);

    
    % cp's weights and Xcoord's:
    
    accP = accK; % position for reading control points
    
    for po = 1:1:noc
        Ssurf(s).w(po) = surfN(s,accP+po);
    end
        
    accP = accP + noc;
    
    j = 0 ;
    for po = 1:1:noc
        
        for di=1:1:xdim
            Ssurf(s).P(po,di) = surfN(s,accP+j+di);
        end
        j = j+3;
    end

    % evaluates if is planar or not:
    [PL, ~,~,~,~] = go2505_cloudP_plane (noc, Ssurf(s).P);
    Ssurf(s).PL = PL ;
   
end



% 2.6.- Loops: own ref., No. edges/vertx (Noe), 
% edge/vertx(0/1),   ed/vetx ref. (evR),   Ed. List index (eLI),   Orientation (eO),   No. underl. param. curve (assumed 0)

Sloops(Nlo) = struct('R',0,'Noe',0,'EV',[], 'evR',[],'eLI',[],'eO',[], 'Npc',[]);

for L=1:1:Nlo
    
    Sloops(L).R   = loops(L,1);
    Sloops(L).Noe = loops(L,2);
    
    j = 0;
    for e = 1:1:Sloops(L).Noe
        
        Sloops(L).EV(e)  = loops(L,3+j);
        Sloops(L).evR(e) = loops(L,4+j);
        Sloops(L).eLI(e) = loops(L,5+j);
        Sloops(L).eO(e)  = loops(L,6+j);
        Sloops(L).Npc(e) = loops(L,7+j);
        
        j = j+5;
    end
end


% 2.7.- Edges: own ref., No. curves, 
% curve ref.(cR),  vertex1 ref.(v1R),   v1 list index (v1LR),    vertex2 ref.,   v2 list index

Sedges(Ned) = struct('R',0,'Noc',0, 'cR',[], 'v1R',[],'v1LR',[],'v2R',[],'v2LR',[]);

for e =1:1:Ned
    
 	Sedges(e).R   = edges(e,1);
    Sedges(e).Noc = edges(e,2);
    
    j = 0;
    for c = 1:1:Sedges(e).Noc
        
     	Sedges(e).cR(c)  = edges(e,3+j);
        Sedges(e).v1R(c) = edges(e,4+j);
        Sedges(e).v1LR(c)= edges(e,5+j);
        Sedges(e).v2R(c) = edges(e,6+j);
        Sedges(e).v2LR(c)= edges(e,7+j);
        
        j = j+5;
    end
end



% 2.8.- Vertex: own ref., No. points, Xcoords

xdim = 3 ;

Svertexes(Nve) = struct('R',0,'Nop',0, 'P',[]);

for e = 1:1:Nve
    
 	Svertexes(e).R   = vertexes(e,1);
    Svertexes(e).Nop = vertexes(e,2);
    
    j = 0;
    for po = 1:1:Svertexes(e).Nop
        
        for di=1:1:xdim
            Svertexes(e).P(po,di)  = vertexes(e,2+j+di);
        end
        j = j+3;
        
    end
end



% 2.9.- Curves: own ref., No. control points, Degree, planar/NONplanar (0/1),
% Open/Close (0/1), Rational/Polynomial (0/1), non-peridodical/Periodical (0/1), 
% Usize, KnotV, cp's w and Xcoors

udim = 1;
xdim = 3;

Scurves(Ncu) = struct('R',0,'Ncp',0,'Degree',0, 'PLN', 0, 'OC',0,'RP',0,'PN',0,'Usize',0, 'KnotV',[], 'w',[],'P',[]);


for s = 1:1:Ncu
    
    Scurves(s).R = curves(s,1);
    
    
    % parameters in each direction
    accK = 7;   % position just before Knot vector
    
    n = curves(s,2)+1;
    p = curves(s,3);
    
    Scurves(s).Ncp    = n;
  	Scurves(s).Degree = p;
    Scurves(s).PLN    = curves(s,4);
  	Scurves(s).OC     = curves(s,5);
    Scurves(s).RP     = curves(s,6);
  	Scurves(s).PN     = curves(s,7);
        
 	Scurves(s).Usize  = n+p+1;
  	for i=1:1:n+p+1
        Scurves(s).KnotV(i) = curves(s,accK+i);
    end
        
  	accK = accK+n+p+1;
 
    
    % normalizes Knot vectors:
    
    Us = Scurves(s).Usize; % size of KnotV
        
    % extreme values of knot vector
   	k1 = Scurves(s).KnotV(1);
 	k2 = Scurves(s).KnotV(Us);
        
    for i=1:1:Us
        Scurves(s).KnotV(i) = (Scurves(s).KnotV(i)-k1) / (k2-k1);
    end 

    
    % cp's weights and Xcoord's:
    accP = accK; % position for reading control points
    
    for po = 1:1:n
        Scurves(s).w(po) = curves(s,accP+po);
    end
        
    accP = accP + n;
    
    j = 0 ;
    for po = 1:1:n
        
        for di=1:1:xdim
            Scurves(s).P(po,di) = curves(s,accP+j+di);
        end
        j = j+3;
    end

end


% 2.10.- Lines:

Sline(1) = struct('R',0,'X1',[],'X2',[]);

if Nli > 0
    
    Sline(Nli) = struct('R',0,'X1',[],'X2',[]);
    for L = 1:1:Nli

    	Sline(L).R = line(L,1);

        for di=1:1:xdim
            Sline(L).X1(di)  = line(L,1+di);
            Sline(L).X2(di)  = line(L,4+di);
        end
    end
end





% 2.11.- Surface revolution:

SsurfR(1) = struct('R',0,'L',0,'C', 0, 'CT', '0',  'A',[]);

if Nsr > 0

    SsurfR(Nsr) = struct('R',0,'L',0,'C', 0, 'CT', '0', 'A',[]);
    for s = 1:1:Nsr

    	SsurfR(s).R = surfR(s,1);
        SsurfR(s).L = surfR(s,2);
        SsurfR(s).C = surfR(s,3);
        SsurfR(s).A(1) = surfR(s,4);
        SsurfR(s).A(2) = surfR(s,5);
        
        
         % detects if is NURBS curve or Line
        
        for c = 1:1:Ncu
            
            cR = Scurves(c).R;
            if cR == SsurfR(s).C 
                SsurfR(s).CT = 'N';
                break
            end
        end
        
        for c = 1:1:Nli
            
            cR = Sline(c).R;
            if cR == SsurfR(s).C 
                SsurfR(s).CT = 'L';
                break
            end
        end
    end

end




% 2.12.- Bounded surfaces and boundaries:

Sbsurf(1) = struct('R',0,'suR',0, 'suT','0',  'Nob',0, 'bR', []);
Sbound(1) = struct('R',0,'Noc',0,'cuR',[], 'cuO', []);

if Nbs > 0

    Sbsurf(Nbs) = struct('R',0,'suR',0, 'suT','0',  'Nob',0, 'bR', []);
    for b = 1:1:Nbs

    	Sbsurf(b).R    = bsurf(b,1);
        Sbsurf(b).suR  = bsurf(b,3);
        Sbsurf(b).Nob  = bsurf(b,4);
    
        for i=1:1:Sbsurf(b).Nob
            Sbsurf(b).bR(i)  = bsurf(b,4+i);
        end
        
        
        % detects if is NURBS or REV surface
        
        for s = 1:1:Nsn
            
            sR = Ssurf(s).R;
            if sR == Sbsurf(b).suR 
                Sbsurf(b).suT = 'N';
                break
            end
        end
        
        for s = 1:1:Nsr
            
            sR = SsurfR(s).R;
            if sR == Sbsurf(b).suR 
                Sbsurf(b).suT = 'R';
                break
            end
        end
        
    end

    Sbound(Nbo) = struct('R',0,'Noc',0,'cuR',[], 'cuO', []);
    for b = 1:1:Nbo
        
    	Sbound(b).R    = bound(b,1);
        Sbound(b).Noc  = bound(b,5);
        
        j = 1 ;
        for i=1:1:Sbound(b).Noc
            Sbound(b).cuR(i)  = bound(b,5+j);
            Sbound(b).cuO(i)  = bound(b,5+j+1);
            j = j+3;
        end
    end

end



% 3.- TRANSFORMS REFERENCES FROM ORIGINAL (IN IGES) TO CORRELATIVE, STARTING BY 1:

% 3.1.- Shells and affected MSBO:

rm = zeros(Nsh,1);
for s =1:1:Nsh
    rm(s) = Sshells(s).R; % shells ref's
end


for p=1:1:Npa % loop on MSBO
    
    shR = Smsbo(p).shR;
    
    for s = 1:1:Nsh
        if shR == rm(s)
            Smsbo(p).shR = s;
            break
        end
    end
end


% 3.2.- Faces and affected shells:

rm = zeros(Nfa,1);
for f =1:1:Nfa
    rm(f) = Sfaces(f).R; % faces refs
end

for s = 1:1:Nsh % loop in shells
    
    for f = 1:1:Sshells(s).Nof
    
        faR = Sshells(s).fR(f);
        
        for gf = 1:1:Nfa
            if faR == rm(gf)
                Sshells(s).fR(f) = gf;
                break
            end
        end
    end  
end



% 3.3.- Surfaces / Loops and affected faces:

rmS = zeros(Nsn,1);

for s = 1:1:Nsn
    rmS(s) = Ssurf(s).R; % surfaces refs
end


for f = 1:1:Nfa % loop on faces
    
    suR = Sfaces(f).suR;
    for s = 1:1:Nsn
        if suR == rmS(s)
            Sfaces(f).suR = s;
            break
        end
    end
    
end


% 3.4.- Loops and affected faces:

rmL = zeros(Nlo,1);
for s = 1:1:Nlo
    rmL(s) = Sloops(s).R; % loops refs
end

for f = 1:1:Nfa % loop on faces
    
    fNol = Sfaces(f).Nol; % No. faces of that loop

    for fl = 1:1:fNol
        
        lR = Sfaces(f).lR(fl);
        
        for gl = 1:1:Nlo
            
            if lR == Sloops(gl).R
                
                Sfaces(f).lR(fl) = gl;
                break
            end
        end
    end
end




% 3.5.- Edges and affected loops:

rm = zeros(Ned,1);
for e=1:1:Ned
   rm(e) = Sedges(e).R ; % edges refs
end


for L = 1:1:Nlo % loop on loops
    
    for e = 1:1:Sloops(L).Noe
        
        eR = Sloops(L).evR(e);
        
        for ge=1:1:Ned
            
            if eR == rm(ge)
                Sloops(L).evR(e) = ge;
                break
            end
        end
    end
end


% 3.6.- Curves / verteces and affected edges:

rm = zeros(Ncu,1);
rmV = zeros(Nve,1);

for c=1:1:Ncu
   rm(c) = Scurves(c).R ; % curves refs
end

for v=1:1:Nve
   rmV(v) = Svertexes(v).R ; % verteces refs
end


for e = 1:1:Ned % loop on edges
    
    for c = 1:1:Sedges(e).Noc
        
        cR  = Sedges(e).cR(c); % curve ref.
        v1R = Sedges(e).v1R(c); %vert.1 ref.
        v2R = Sedges(e).v2R(c); %vert.2 ref.
        
        for gc = 1:1:Ncu
            if cR==rm(gc)
                Sedges(e).cR(c) = gc;
                break
            end
        end
        
        for gv = 1:1:Nve
            if v1R == rmV(gv)
                Sedges(e).v1R(c) = gv;
                break
            end
        end
        
        for gv = 1:1:Nve
            if v2R == rmV(gv)
                Sedges(e).v2R(c) = gv;
                break
            end
        end
        
    end
end


% 3.7.- Boundaries and affected bounded surfaces:

rm = zeros(Nbo,1);

for b =1:1:Nbo
   rm(b) = Sbound(b).R ; % boundaries refs
end


for s = 1:1:Nbs % loop on bounded surfaces
    
    for b = 1:1:Sbsurf(s).Nob
        
        bR = Sbsurf(s).bR(b);
        
        for gb=1:1:Nbo
            
            if bR == rm(gb)
                Sbsurf(s).bR(b) = gb;
                break
            end
        end
    end
end



% 3.8.- Surfaces (NURBS and REV) and affected bounded surfaces:

rmN = zeros(Nsn,1);
rmR = zeros(Nsr,1);

for s = 1:1:Nsn
    rmN(s) = Ssurf(s).R;
end

for s = 1:1:Nsr
    rmR(s) = SsurfR(s).R;
end

for b = 1:1:Nbs % loop on bounded surfaces
    
    sR = Sbsurf(b).suR;
    
    if strcmp(Sbsurf(b).suT,'N')==true
        
       for gs = 1:1:Nsn
           if sR == Ssurf(gs).R
               Sbsurf(b).suR = gs;
               break
           end
       end
       
    elseif strcmp(Sbsurf(b).suT,'R')==true 
       
        for gs = 1:1:Nsr
           if sR == SsurfR(gs).R
               Sbsurf(b).suR = gs;
               break
           end
        end
       
    end
end


% 3.9.- Curves and affected boundaries:

rm = zeros(Ncu,1);

for c =1:1:Ncu
   rm(c) = Scurves(c).R ; % curves refs
end


for s = 1:1:Nbo % loop on boundaries
    
    for c = 1:1:Sbound(s).Noc
        
        cR = Sbound(s).cuR(c);
        
        for gc =1:1:Ncu
            
            if cR == rm(gc)
                Sbound(s).cuR(c) = gc;
                break
            end
        end
    end
end




% 3.10.- Lines/Curves and affected surfaces revolution:

rmN = zeros(Ncu,1); % generatrix Nurbs
rmL = zeros(Nli,1); % generatrix Line

for c = 1:1:Ncu
    rmN(c) = Scurves(c).R;
end

for c = 1:1:Nli
    rmL(c) = Sline(c).R;
end

for s = 1:1:Nsr % loop on surfaces rev.
    
    LR = SsurfR(s).L; % axis ref.
    CR = SsurfR(s).C; % generatrix ref.
    
    
    % axis line:
    
  	for gl = 1:1:Nli
        if LR == Sline(gl).R
            SsurfR(s).L = gl;
            break
        end
    end
    
    
    
    % generatrix:
    
    if strcmp(SsurfR(s).CT,'N')==true
        
       for gl = 1:1:Ncu
           if CR == sGcurve(gl).R
               SsurfR(s).C = gl;
               break
           end
       end
       
    elseif strcmp(SsurfR(s).CT,'L')==true 
       
    	for gl = 1:1:Nli
           if CR == Sline(gl).R
               SsurfR(s).C = gl;
               break
           end
        end
       
    end
end



%{ 
% previous
% 4.- CHANGES ORIENTATION OF FACES THAT HAS ATTACHED A NURBS SURFACE:

for s = 1:1:Nsh
    
    for i=1:1:sGshells(s).Nof
        
         fO = sGshells(s).fO(i);

         if fO == 1 
 
            %sGshells(s).fO(i) = 0 ;
  
         else
             %sGshells(s).fO(i) = 1 ;
         end
  
        %{
         fR = sGshells(s).fR(i);
         
         if fR==1
             
             fR
             sGshells(s).fO(i)

             Ncp = sGsurfN(fR).Ncp;
             P   = sGsurfN(fR).P;
             Degree = sGsurfN(fR).Degree;
             Usize  = sGsurfN(fR).Usize;
             KnotV  = sGsurfN(fR).KnotV;

             Ncp
             Degree
             Usize
             KnotV
             
             figure(1)
             scatter3(P(:,1),P(:,2),P(:,3),'r'); hold on
             for j = 1:1:prod(Ncp)
                 text(P(j,1),P(j,2),P(j,3),num2str(j)); hold on
             end

             axis equal
             
             
             Nol = sGfaces(fR).Nol
             lR = sGfaces(fR).lR
             
             Noe = sGloops(lR).Noe
             eO = sGloops(lR).eO
             
             eLI = sGloops(lR).eLI
             for c = 1:1:4
                 
                 eLI = sGloops(lR).eLI(c) ;
                 sGcurves(eLI).P
                 
             end
             
         end
        %}
         
    end
    
end
%}


% 4.- CHANGES ORIENTATION OF FACES THAT HAS ATTACHED A NURBS SURFACE:

for s = 1:1:Nsh
    
    for i=1:1:Sshells(s).Nof
        
        fO = Sshells(s).fO(i);
        fR = Sshells(s).fR(i);

        if fO == 1 
 
            % Since has been swapped, the NURBS surface needs to be
            % reversed.
            % We do it by reversing direction 1
            
            di = 1 ;
            
            % only affects to KnotV, w and P
             
            KnotV   = Ssurf(fR).KnotV ;
            Ncp     = Ssurf(fR).Ncp ;
            Degree  = Ssurf(fR).Degree ;

            Usize = Ncp(di)+Degree(di)+1;
            U = zeros(Usize,1);
            
            for j = 1:1:Usize
                U(j,1) = KnotV(di,j);
            end

            rU = GEN_06_ReverseKnotV (U, Ncp(di), Degree(di));
            
            for j = 1:1:Usize
                KnotV(di,j) = rU(j,1);
            end
            
            Ssurf(fR).KnotV = KnotV ;
            
            P = Ssurf(fR).P ;
            w = Ssurf(fR).w ;
            
            udim = 2 ;

            [Ew,EP] = GEN_12_ExpandBw (w, P,xdim, udim, Ncp);
            
            rEw = zeros(Ncp(1),Ncp(2));
            rEP = zeros(Ncp(1),Ncp(2));
            
            for b = 1:1:Ncp(2)
                
                ar = Ncp(di)+1;
                for a = 1:1:Ncp(di)
                    
                    ar = ar - 1;

                    rEw(ar,b) = Ew(a,b);
                    
                    for xd = 1:1:xdim
                        rEP(ar,b,xd) = EP(a,b,xd);
                    end
                    
                end
 
            end
            
            [rw,rP] = GEN_12_ContractsBw (rEw, rEP, xdim, udim, Ncp);
            
            Ssurf(fR).P = rP ;
            Ssurf(fR).w = rw ;

            %{
            if fR==1
             
             fR
             sGshells(s).fO(i)

             Ncp = sGsurfN(fR).Ncp;
             P   = sGsurfN(fR).P;
             Degree = sGsurfN(fR).Degree;
             Usize  = sGsurfN(fR).Usize;
             KnotV  = sGsurfN(fR).KnotV;

             Ncp
             Degree
             Usize
             KnotV
             
             figure(fR+1)
             scatter3(P(:,1),P(:,2),P(:,3),'r'); hold on
             for j = 1:1:prod(Ncp)
                 text(P(j,1),P(j,2),P(j,3),num2str(j)); hold on
             end

             axis equal
             
             
             Nol = sGfaces(fR).Nol
             lR = sGfaces(fR).lR
             
             Noe = sGloops(lR).Noe
             eO = sGloops(lR).eO
             
             eLI = sGloops(lR).eLI
             for c = 1:1:4
                 
                 eLI = sGloops(lR).eLI(c) ;
                 sGcurves(eLI).P
                 
             end
            
 
         end
            %}
            
 
        end

        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %{  
        if s == 2
            
                figure(1)
                %figure(fR)
                
                scatter3(rP(:,1), rP(:,2), rP(:,3), 'r'); hold on
                
                acc = zeros(3,1) ;
                for j = 1:1:prod(Ncp)
                    
                    for di = 1:1:3
                        acc(di,1) = acc(di,1) + rP(j,di)/prod(Ncp);
                    end
                    
                    
                    %text(rP(j,1), rP(j,2), rP(j,3), num2str(j)); hold on
                end
                
                text(acc(1,1), acc(2,1), acc(3,1), num2str(fR)); hold on
                
                
                
                axis equal
        end      
        %}
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
        
    end
    
  
end

    if gg==0
    end

end

