function [Nso, MSBO, Nsh, shells, Nfa, faces, Nsn, surfN, Nlo, loops, Ned, edges, ...
     	 Ncu, curves, Nve, vertexes, Nbs, bsurf, Nbo, bound, Nsr, surfR, Nli, line] = ...
         ri3191_toArrays (Npa, NL, Fname)

% RETURNS:
% numerical arrays with info directly from IGES file

% 1.- DETECTS SHELLS, FACES, .. and all entities that are stored in their respective string matrices:

% Note that first col. of each matrix is always the reference

MSBO	= zeros(Npa,1);
shells	= zeros(Npa,12); % for each patch: face1, orint1, .... face6, orient6.
faces	= zeros(1,1);
surfN	= zeros(1,1);
loops	= zeros(1,1);
edges	= zeros(1,1);
curves	= zeros(1,1);
vertexes = zeros(1,1);
bsurf	= zeros(1,1);
bound	= zeros(1,1);
surfR 	= zeros(1,1);
line  	= zeros(1,1);

Nso = 0;
Nsh = 0;
Nfa = 0;
Nsn = 0;
Nlo = 0;
Ned = 0;
Ncu = 0;
Nve = 0;
Nbs = 0;
Nbo = 0;
Nsr = 0;
Nli = 0;


fid = fopen(Fname);

ON = 0;
startLine = 1 ; % 1 indicates prev. line was end of entity and then the current line is starting

for L = 1:1:NL
    
    wline = fgetl(fid); % read whole line
    pFL = wline(1:3); % prev. entity code
    
    if L > 8 && strcmp(pFL,'   ')== false
        ON = 1 ; % activates the retrieving data when finds entities at the first 3 characters
    end
    
    if ON==1
    
        notFindEnd = strfind(wline,';'); % end of entity definition

        
        % 1.1.- First line to read (can be unique):

        if (isempty(notFindEnd)==false && startLine==1) || (isempty(notFindEnd)==true && startLine==1)

            FL = wline(1:3); % entity code

            NparP = length(strfind(wline,',')); % No. params to read
            if isempty(notFindEnd)==false 
                NparP = NparP+1;
            end

            % reads and allocates first line (can be unique)
            [Nso,MSBO,Nsh,shells,Nfa,faces,Nsn,surfN,Nlo,loops,Ned,edges,Ncu,curves,Nve,vertexes,...
          	Nbs,bsurf, Nbo,bound, Nsr, surfR, Nli, line] = ...
         	i3151_MaddFline ...
            (Nso,MSBO,Nsh,shells,Nfa,faces,Nsn,surfN,Nlo,loops,Ned,edges,Ncu,curves,Nve,vertexes, ...
            Nbs,bsurf, Nbo,bound, Nsr, surfR, Nli, line, FL, wline, NparP);

            % switches startLine in case ther eis not end ';':
            if isempty(notFindEnd)==true
                startLine = 0 ;
            end
        
        
        
        % 1.2.- Aggregate lines below:    

        else

            Npar = length(strfind(wline,',')) ;  % No. params to read
            if isempty(notFindEnd)==false 
                Npar = Npar+1;
            end

            % reads and allocates line
            [Nso,MSBO,Nsh,shells,Nfa,faces,Nsn,surfN,Nlo,loops,Ned,edges,Ncu,curves,Nve,vertexes, ...
          	Nbs,bsurf, Nbo,bound, Nsr, surfR, Nli, line] = ...
            i3152_MaddSline ...
            (Nso,MSBO,Nsh,shells,Nfa,faces,Nsn,surfN,Nlo,loops,Ned,edges,Ncu,curves,Nve,vertexes, ...
            Nbs,bsurf, Nbo,bound, Nsr, surfR, Nli, line, FL, wline, Npar, NparP);


            % accumulates previous No. parameters:
            NparP = NparP + Npar;


            % in case finds end ';' switches:
            if isempty(notFindEnd)==false
                startLine = 1 ;
            end

        end
    end
end

fclose(fid);



end

