function [Smsbo, Sshells, Sfaces, Ssurf, Sloops, Sedges, Scurves, Svrtx] = ...
    ri3192_Solids (msbo, shells, faces, surf, loops, edges, curves, vertexes, ...
    Nso, Nsh, Nfa, Nsu, Nlo, Ned, Ncu, Nve)


% RETURNS:
% structures to define the CAD solids


% 1.- MSBO: shell reference attached to that MSBO

Smsbo(Nso) = struct('shR',0);

for s = 1:1:Nso
    Smsbo(s).shR = msbo(s,2);
end


% 2.- Shells: own ref., No. faces, face ref., face orientation

Sshells(Nsh) = struct('R',0,'Nof',0,'faR',[],'faO',[]);


for s = 1:1:Nsh

    Sshells(s).R = shells(s,1);
    Sshells(s).Nof = shells(s,2);
    
    j=1;
    for i=1:1:Sshells(s).Nof
        
        Sshells(s).faR(i)= shells(s,2+j);
        
        % NOTE: orientation is swapped due to IGES data structure of NURBS surfaces
        % that is done at the end of this routine
        
        Sshells(s).faO(i)= shells(s,2+j+1); 
        
        j = j+2;
        
    end
     
end



% 3.- Faces: own ref., surface ref., No. loops, Inner/Outer loop (0/1), Loops ref.

Sfaces(Nfa) = struct('R',0, 'suR',0, 'Nol',0, 'IOL',[],'loR',[]);

for f=1:1:Nfa

    Sfaces(f).R   = faces(f,1);
    Sfaces(f).suR = faces(f,2);
    Sfaces(f).Nol = faces(f,3);
    Sfaces(f).IOL = faces(f,4);
       
    for i=1:1:Sfaces(f).Nol
        Sfaces(f).loR(i)  = faces(f,4+i);
    end
  
end



% 4.- Surfaces: own ref., No. control points, Degree, Open/Close (0/1),
% Usize, KnotV, cp's w and Xcoors

% NOTE !! in surfaces we swap Ncp, Degree, Usize, and KnotV

udim = 2;
xdim = 3;

Ssurf(Nsu) = struct('R',0,'Ncp',[],'Degree',[], 'OC',[],'Usize',[], 'KnotV',[], 'w',[],'P',[], 'PL', 0);

for s = 1:1:Nsu
    
    Ssurf(s).R = surf(s,1);
    
    
    % parameters in each direction
    accK = 10;  % position just before Knot vector
    noc = 1 ;   % total No. cp's
    
    pdim = [2 1];
    for di=1:1:udim
        
        pd = pdim(di);
        
        n = surf(s,1+di)+1;
        p = surf(s,3+di);
        
        noc = noc * n ;
        
        Ssurf(s).Ncp(pd)    = n;
        Ssurf(s).Degree(pd) = p;
        Ssurf(s).OC(pd)     = surf(s,5+di);
        
        Ssurf(s).Usize(pd)  = n+p+1;
        
        for i=1:1:n+p+1
            Ssurf(s).KnotV(pd,i) = surf(s,accK+i);
        end
        
        accK = accK+n+p+1;
    end
    
    
    % normalizes Knot vectors: from 0 to 1
    
    for di=1:1:udim
        
        Us = Ssurf(s).Usize(di); % size of KnotV
        
        % extreme values of knot vector
        k1 = Ssurf(s).KnotV(di,1);
        k2 = Ssurf(s).KnotV(di,Us);
        
        for i=1:1:Us
            Ssurf(s).KnotV(di,i) = (Ssurf(s).KnotV(di,i)-k1) / (k2-k1);
        end 
    end
    

    % cp's weights and Xcoord's:
    
    accP = accK; % position for reading control points
    
    for po = 1:1:noc
        Ssurf(s).w(po) = surf(s,accP+po);
    end
        
    accP = accP + noc;
    
    j = 0 ;
    for po = 1:1:noc
        
        for di=1:1:xdim
            Ssurf(s).P(po,di) = surf(s,accP+j+di);
        end
        j = j+3;
    end

    % evaluates if is planar or not:
    [PL, ~,~,~,~] = go2505_cloudP_plane (noc, Ssurf(s).P);
    Ssurf(s).PL = PL ;
    
end



% 5.- Loops: own ref., No. edges/vertx (Noe), 
% ed/vetx ref. (evR),   Ed. List index (eLI),   Orientation (eO),   No. underl. param. curve (assumed 0)

Sloops(Nlo) = struct('R',0,'Noe',0,'edR',[],'eLI',[],'edO',[]);

for L=1:1:Nlo
    
    Sloops(L).R   = loops(L,1);
    Sloops(L).Noe = loops(L,2);
    
    j = 0;
    for e = 1:1:Sloops(L).Noe
        
        Sloops(L).edR(e) = loops(L,4+j);
        Sloops(L).eLI(e) = loops(L,5+j);
        Sloops(L).edO(e)  = loops(L,6+j);
        
        j = j+5;
    end
end


% 6.- Edges: own ref., No. curves, 
% curve ref.(cR),  vertex1 ref.(v1R),   v1 list index (v1LR),    vertex2 ref.,   v2 list index

Sedges(Ned) = struct('R',0,'Noc',0, 'cuR',[], 'v1R',[],'v1LR',[],'v2R',[],'v2LR',[]);

for e =1:1:Ned
    
 	Sedges(e).R   = edges(e,1);
    Sedges(e).Noc = edges(e,2);
    
    j = 0;
    for c = 1:1:Sedges(e).Noc
        
     	Sedges(e).cuR(c) = edges(e,3+j);
        Sedges(e).v1R(c) = edges(e,4+j);
        Sedges(e).v1LR(c)= edges(e,5+j);
        Sedges(e).v2R(c) = edges(e,6+j);
        Sedges(e).v2LR(c)= edges(e,7+j);
        
        j = j+5;
    end
end



% 7.- Vertex: own ref., No. points, Xcoords

xdim = 3 ;

Svrtx(Nve) = struct('R',0,'Nop',0, 'P',[]);

for e = 1:1:Nve
    
 	Svrtx(e).R   = vertexes(e,1);
    Svrtx(e).Nop = vertexes(e,2);
    
    j = 0;
    for po = 1:1:Svrtx(e).Nop
        
        for di=1:1:xdim
            Svrtx(e).P(po,di)  = vertexes(e,2+j+di);
        end
        j = j+3;
        
    end
end



% 8.- Curves: own ref., No. control points, Degree, planar/NONplanar (0/1),
% Open/Close (0/1), Usize, KnotV, cp's w and Xcoors

xdim = 3;

Scurves(Ncu) = struct('R',0,'Ncp',0,'Degree',0, 'PL', 0, 'OC',0,'Usize',0, 'KnotV',[], 'w',[],'P',[]);


for s = 1:1:Ncu
    
    Scurves(s).R = curves(s,1);
    
    
    % parameters in each direction
    accK = 7;   % position just before Knot vector
    
    n = curves(s,2)+1;
    p = curves(s,3);
    
    Scurves(s).Ncp    = n;
  	Scurves(s).Degree = p;
    Scurves(s).PL    = curves(s,4);
  	Scurves(s).OC    = curves(s,5);
        
 	Scurves(s).Usize  = n+p+1;
  	for i=1:1:n+p+1
        Scurves(s).KnotV(i) = curves(s,accK+i);
    end
        
  	accK = accK+n+p+1;
 
    
    % normalizes Knot vectors:
    
    Us = Scurves(s).Usize; % size of KnotV
        
    % extreme values of knot vector
   	k1 = Scurves(s).KnotV(1);
 	k2 = Scurves(s).KnotV(Us);
        
    for i=1:1:Us
        Scurves(s).KnotV(i) = (Scurves(s).KnotV(i)-k1) / (k2-k1);
    end 

    
    % cp's weights and Xcoord's:
    accP = accK; % position for reading control points
    
    for po = 1:1:n
        Scurves(s).w(po) = curves(s,accP+po);
    end
        
    accP = accP + n;
    
    j = 0 ;
    for po = 1:1:n
        
        for di=1:1:xdim
            Scurves(s).P(po,di) = curves(s,accP+j+di);
        end
        j = j+3;
    end

end




end

