function [KG] = ms3201_KGrod3D (E,A,L, u)

% RETURNS:
% K: matriz of one rod (truss element, only tension-compression) in global coordinates

% works in 2D

% E,A,L: bar features
% u: rod versor direction, from node 1 to 2 , note that ||u|| = 1

% local matrix:

k = E*A/L ;

KL = [  k  0  0 -k  0  0 ;
        0  0  0  0  0  0 ;
        0  0  0  0  0  0 ;
       -k  0  0  k  0  0 ;
        0  0  0  0  0  0 ;
        0  0  0  0  0  0 ];
 
    
% local coordinates system: u, v, w (centred on 0)

% plane perp. to u: Ax + By + Cz

A = u(1);
B = u(2);
C = u(3);

if round(B,2)==0 && round(C,2)==0
    
    % u is (1,0,0)
    
    v = [0 1 0]';
    w = [0 0 1]';
    
elseif round(A,2)==0 && round(C,2)==0

    % u is (0,1,0)
    
 	v = [0 0 1]';
    w = [1 0 0]';
    
elseif round(A,2)==0 && round(B,2)==0
    
    % u is (0,0,1)
    
	v = [1 0 0]';
    w = [0 1 0]';
    
elseif round(C,2)==0 
    
    % u is (a,b,0) = plane perp. to z

    w = [1 0 0]';
    
    v = cross(w,u);
    v = v/norm(v);
    
elseif round(B,2)==0 
    
    % u is (a,0,c) = plane perp. to y

 	w = [0 1 0]';
    
    v = cross(w,u);
    v = v/norm(v);
    
elseif round(A,2)==0 
    
    % u is (0,b,c) = plane perp. to x

	w = [1 0 0]';
    
    v = cross(w,u);
    v = v/norm(v);
    
else
    
    % other orientations:
    
    % point to define v:
    x = 1 ;
    y = 10 ;
    z = -(A*x + B*y)/C;
    
    v = [x y z]';
    v = v / norm(v);
    
    
    % w as u x v:
    
    w = cross(u,v);
    w = w/norm(w);
    
end


% rotation matrix:
   
R = [u,v,w];
R = [R zeros(3) ; zeros(3) R ];


% global matrix:

KG = R*KL*R' ;



end

