function [NB, ND, TB, TBD, NE, BVcon, RVcon, flagEB] = tr3241_reg3DTrussBars (Ncp)

% RETURNS: features for regular trusses in 3D
% NB(3): No bars
% ND(3): No. diagonals
% TB:  total No. bars
% TBD:   total No. bars + diags
% BVcon(TBD,2): bar - vtx conn. matrix
% RVcon(NE,8):  element - vtx conn. matrix

% 1.- NUMBER OF BARS:


NB = zeros(3,1); % No. bars in each param. direction

NB(1) = (Ncp(1)-1)*(Ncp(2)*Ncp(3));
NB(2) = (Ncp(2)-1)*(Ncp(3)*Ncp(1));
NB(3) = (Ncp(3)-1)*(Ncp(1)*Ncp(2));

ND = zeros(3,1); % No. of pairs of diagonals (in plane perp. to each direction)

ND(1) = Ncp(1)*(Ncp(2)-1)*(Ncp(3)-1);
ND(2) = Ncp(2)*(Ncp(3)-1)*(Ncp(1)-1);
ND(3) = Ncp(3)*(Ncp(1)-1)*(Ncp(2)-1);

TB = sum(NB); % total No. bars
TBD = sum(NB) + 2*sum(ND); % total No. bars + diags.


% No. elems :
NE = (Ncp(1)-1)*(Ncp(2)-1)*(Ncp(3)-1);



% 2.- CONNECTIVITY MATRICES:


BVcon = zeros(TBD,2);

RVcon = zeros(NE,8);

flagEB = zeros(TBD,1); % turns to 1 if is external bar, i.e. at least one node is permitral, except diagonals


% 3.1.- Bars & diags:

contaN = 0 ;

b1 = 0; % bars counter
b2 = 0;
b3 = 0;

d1 = 0 ; % diagonals counter
d2 = 0 ;
d3 = 0 ;



for i = 1:1:Ncp(1)
for j = 1:1:Ncp(2)
for k = 1:1:Ncp(3)
    
    contaN = contaN + 1;
    
    % bars:
    
    if i < Ncp(1)
        
        b1 = b1 + 1;
        
        r = b1 ;
        
        BVcon(r,1) = contaN;
        BVcon(r,2) = contaN + Ncp(2)*Ncp(3);
        
        if i==1 || i==Ncp(1)-1
            flagEB(r) = 1 ;
        elseif j==1 || j==Ncp(2)
            flagEB(r) = 1 ;
        elseif k==1 || k==Ncp(3)
            flagEB(r) = 1 ;
        end

    end
    
    if j < Ncp(2)
        
        b2 = b2 + 1 ;
        
        r = NB(1) + b2 ;
        
        BVcon(r,1) = contaN ;
        BVcon(r,2) = contaN + Ncp(3);
        
       	if i==1 || i==Ncp(1)
            flagEB(r) = 1 ;
        elseif j==1 || j==Ncp(2)-1
            flagEB(r) = 1 ;
        elseif k==1 || k==Ncp(3)
            flagEB(r) = 1 ;
        end
        
    end
    
    if k < Ncp(3)
        
        b3 = b3 + 1 ;
        
        r = NB(1)+ NB(2) + b3 ;
        
        BVcon(r,1) = contaN ;
        BVcon(r,2) = contaN + 1;
        
        if i==1 || i==Ncp(1)
            flagEB(r) = 1 ;
        elseif j==1 || j==Ncp(2)
            flagEB(r) = 1 ;
        elseif k==1 || k==Ncp(3)-1
            flagEB(r) = 1 ;
        end
        
    end
    
    
    % diagonals:
    
    if j < Ncp(2) && k < Ncp(3)
        
        d1 = d1 + 1 ;
        BVcon(TB + d1,1) = contaN ;
        BVcon(TB + d1,2) = contaN + Ncp(3)+1 ;
        
      	BVcon(TB + ND(1) + d1,1) = contaN+1 ;
        BVcon(TB + ND(1) + d1,2) = contaN + Ncp(3) ;
    end
    
    if i < Ncp(1) && k < Ncp(3)
        
        d2 = d2 + 1 ;
        BVcon(TB + 2*ND(1) + d2,1) = contaN ;
        BVcon(TB + 2*ND(1) + d2,2) = contaN + Ncp(3)*Ncp(2) + 1 ;
        
      	BVcon(TB + 2*ND(1) + ND(2) + d2,1) = contaN+1 ;
        BVcon(TB + 2*ND(1) + ND(2) + d2,2) = contaN + Ncp(3)*Ncp(2) ;
    end
    
    if i < Ncp(1) && j < Ncp(2)
        
        d3 = d3 + 1 ;
        BVcon(TB + 2*(ND(1)+ND(2)) + d3,1) = contaN ;
        BVcon(TB + 2*(ND(1)+ND(2)) + d3,2) = contaN + Ncp(2)*Ncp(3) + Ncp(3) ;
        
      	BVcon(TB + 2*(ND(1)+ND(2)) + ND(3) + d3,2) = contaN + Ncp(3) ;
        BVcon(TB + 2*(ND(1)+ND(2)) + ND(3) + d3,1) = contaN + Ncp(2)*Ncp(3) ;
    end
end
end
end



% 3.2.- Elements: ordered hierarchically

contaE = 0;
for i = 1:1:Ncp(1)-1
for j = 1:1:Ncp(2)-1
for k = 1:1:Ncp(3)-1
    
    contaE = contaE+1;

    RVcon(contaE,1) = Ncp(2)*Ncp(3)*(i-1) + Ncp(3)*(j-1) + k  ;
    RVcon(contaE,2) = RVcon(contaE,1)  + 1 ;
    
    RVcon(contaE,3) = RVcon(contaE,1) + Ncp(3) ;
    RVcon(contaE,4) = RVcon(contaE,3) + 1 ;
    
   	RVcon(contaE,5) = RVcon(contaE,1) + Ncp(2)*Ncp(3) ;
    RVcon(contaE,6) = RVcon(contaE,5) + 1 ;
    
    RVcon(contaE,7) = RVcon(contaE,5) + Ncp(3) ;
    RVcon(contaE,8) = RVcon(contaE,7) + 1 ;
    
end
end
end




end

