function [B2, Q, dS1, dS2, nv, vep] = ar3304_Xplanes ...
    (Cx, Cu, oCx, ULims, NE, NEt, grEp, ULimEr, ULimEpr, Dspline, sDegree, sNcp, sKnotV, sUsize, sEP,sEw, ...
    fpeR, fceR, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, xdim, udim, DerC)

% RETURNS: Characterization of tangent plane at location Cx (phys.) and Cu (param.)
% alfa, beta: angles of nv w.r.t. z axis
% dS1, dS2: main deriv. at disc centre
% nv: Normal vector to tangent plane
% vep: vector of edge projected on tangent plane

% INPUTS:
% Cu: disc center param. coordinates


% main derivatives and normal 'nv' to tangent plane (pi-plane):

[NeR, eR] = dp3355_FindRele (ULims, NE, NEt, grEp, ULimEr, ULimEpr, fpeR, Cu);

[dS1,dS2] = ds3324_maindSu (Cu, Dspline, sDegree, sNcp, sKnotV, sUsize, sEP,sEw, ...
    NeR, eR, fpeR, fceR, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, xdim, udim, DerC);

nv = cross(dS1,dS2);
nv = nv / norm(nv);


% alfa and beta angles( nv w.r. to z axis):

alfa = atan2(-nv(2),nv(3)) ;
beta = atan2( nv(1),nv(3)) ;


% projection of t-edge onto pi-plane: 'vep'

[Pp] = go2518_POntoP(oCx, nv, Cx); % end point projection


vep = zeros(xdim,1);
for di = 1:1:xdim
    vep(di,1) = Pp(di)-Cx(di);
end

vep = vep/norm(vep);

% LOCAL BASE OF TAN PLANE:
% x2 = vox (origin axis) : parallel to global X axis and in tan-plane
% y2 = z2 x x2
% z2 = nv

% finds 'origin' vector (parallel to x axis, that passes through C and is contained in pi-plane): 'vox'

Po = Cx; % end of origin radius
Po(1) = Po(1) + 1*cos(beta);
Po(3) = Po(3) - 1*sin(beta);

vox = Po-Cx;
vox = vox/norm(vox);

x2 = vox ;
y2 = cross(nv,x2);
z2 = nv ;

B2 = [x2 , y2 , z2 ]; % each column is a vector
Bc = eye(3); % canonic base

% measures angles from new base to cartesian base and computes R and Q = inv(R)

V2 = zeros(xdim,1);
Vc = zeros(xdim,1);
R = zeros(xdim, xdim);

for i = 1:1:3
    
    % new base vector:
    V2(:,1) = B2(:,i);
    
    for j = 1:1:3

        % canonic base:
        Vc(:,1) = Bc(:,j);

        R(i,j) = cos(go2511_angleUV (V2, Vc));

    end
end


% inverts R:

Q = R^(-1);


end

