function [tX, tU] = ar3305_Uangles ...
    (Cx, Cu, oCx, B2, Q, dS1, dS2, nv, vep, tita, tamp, NSg, xdim, ra, rb, NL)

% RETURNS:
% tU: set of angles in phys. / param. space correspondant to arc

% INPUTS:
% Cx: disc center phys. coordinates
% oCx: oposite disc centrer phys. coordinates (to define t-edge vector)
% tita: angle required in phys. space measured from t-edge
% NSg: No. segment of the arcs
% tamp: angle amplitude of arc in phys. space
% vep: projection of t-edge onto pi-plane: 'vep'

%{
% 1.- COMPUTES NEW BASE & TRANSFORMATION MATRIX 'R':

% x2 = vox (origin axis)
% y2 = z2 x x2
% z2 = nv

% finds 'origin' vector (parallel to x axis, that passes through C and is contained in pi-plane): 'vox'

Po = Cx; % end of origin radius
Po(1) = Po(1) + 1*cos(beta);
Po(3) = Po(3) - 1*sin(beta);

vox = Po-Cx;
vox = vox/norm(vox);

x2 = vox ;
y2 = cross(nv,x2);
z2 = nv ;

B2 = [x2 , y2 , z2 ]; % each column is a vector
Bc = eye(3); % canonic base

% measures angles from new base to cartesian base and computes R:

V2 = zeros(xdim,1);
Vc = zeros(xdim,1);
R = zeros(xdim, xdim);

for i = 1:1:3
    
    % new base vector:
    V2(:,1) = B2(:,i);
    
    for j = 1:1:3

        % canonic base:
        Vc(:,1) = Bc(:,j);

        R(i,j) = cos(go2511_angleUV (V2, Vc));

    end
end

% inverts R:

Q = R^(-1);

%}


% 1.- COMPUTES rX: mid-angle referred to 'vox' axis measure in plane:

vox = zeros(xdim,1);
vox(:,1) = B2(:,1);

% angle from vox to vep = angle of vep w.r.t. origin: tref

[tref] = ar3306_angUtoVplane (vox, vep, nv, xdim);


% initial mid-angle referred to vox: negative if clock-wise (looking from side where nv points)

rX = tref + tita ;


% 2.- SET OF ANGLES IN PHYS. SPACE:

Dt = 2*tamp / NSg ;

tX = [rX-tamp : Dt : rX+tamp];




% 3.- CORRESPONDANT ANGLES IN PARAMETER SPACE:

% Computes vectors from C with each of those phys. angles 'tX' and 
% then computes  set of angles in parameter space:

Nvtr = size(tX,2);

tU = zeros(1,Nvtr);

vX = zeros(xdim, Nvtr);

for vtr = 1:1:Nvtr
    
    r = 1 ;
    
    aX = tX(vtr);
    
    % vector
    
    DL = [r*cos(aX);r*sin(aX);0];  % increment in plane local coordinates
    
    DG = Q*DL; % transform increment to global
    
    DG = DG/norm(DG);
    
    vX(:,vtr) = DG(:,1);
    

    % correspondent angle in param. space
    aU = ar3307_AngleU (DG, dS1, dS2, xdim); 
    
    tU(vtr) = aU;
    
end



% 121 122
%{
if (ra == 1 && rb == 122 ) && NL ==0
    
    DENTRO_____________ = 1
    
    vox
    nv
    
    vep
    
    B2
    
    tref*57.3
    rX*57.3
    
    Dt*57.3
    tamp*57.3
    
    tX'*57.3
    vX
    tU'*57.3
    
    % plot base, t-edge and abanico:
    
    figure(1)
    plot3([0 vep(1)],[0 vep(2)],[0 vep(3)],'k'); hold on
    
    for i = 1:1:3
        plot3([0 B2(1,i)],[0 B2(2,i)],[0 B2(3,i)],'b'); hold on
    end
    

    
    for i = 1:1:Nvtr
    
        plot3([0 vX(1,i)],[0 vX(2,i)],[0 vX(3,i)],'g'); hold on
    
    end
    
    axis equal
    
end

%}

end

