function [conF, eR] = ts3310_conF05_ (edCurv, cX, xdim, Xtol)

% RETURNS:
% conF: updated to 0.5 in case curve lies on edge
% eR: edge ref. where the curve lies on
% sg: +/-1 if goes according to edge direction or opposite
% (recallthe the 4 edges start at (0,0) and move counter-clock-wise)

% INPUTS:
% cX: row of points to check
% CS(2): closed surface flag

% NOTE: in close directions, the curve cX must have same sense as edge

nd = abs(round(log10(Xtol)));



conF = 0 ;
eR = 0 ;
sg = 0 ;

ecp = zeros(xdim,1); % edge corner point
fcp = zeros(xdim,1); % first curve point
scp = zeros(xdim,1); % second curve point

%CSe = [CS(1) CS(2) CS(1) CS(2)]; % closed durface flag for each contour curve

% 1.- CORNER CHECK     

% checks first edge corners: first curve 2 point VS edge corners,
% number of corner is first point of correspondent edge curve
 
for e = 1:1:4
                
    % edCurv(4) = struct ('Ncp',0,  'Degree',0, 'Usize',0, 'KnotV',[], 'w',[], 'P',[], 'conF', 0, 'CD', 0);

    % edge features
    eNcp    = edCurv(e).Ncp ;
    eDegree = edCurv(e).Degree ;
    eUsize  = edCurv(e).Usize ;
    eKnotV  = edCurv(e).KnotV ;
    ew      = edCurv(e).w ;
    eP      = edCurv(e).P ;
    eCD     = edCurv(e).CD ;

    if size(eKnotV,1)>1
        eKnotV = eKnotV';
    end
    
    ecp(:,1) = eP(1,:);
    fcp(:,1) = cX(1,:);
    fcpr(:,1) = cX(2,:);
    
    CO1  = go2542_coincidentP (ecp, fcp, xdim, nd);
    CO1r = go2542_coincidentP (ecp, fcpr, xdim, nd);
    
    % find fcp coincident with curve corner: now checks second
    % point of curve
    if CO1 == 1

        scp(:,1) = cX(2,:);
        u = b2811_PPonePoint (scp, eDegree, eNcp, eKnotV, eUsize, eP ,ew, eCD, xdim, 1);
        x = b2601_BNURBS_Xcoord (u, eKnotV, eNcp,eDegree,eUsize, ew,eP, 1,xdim);

        CO2 = go2542_coincidentP (x, scp, xdim, nd);

        % if second point also coincides then it is on the edge
        if CO2==1
            conF = 0.5 ;
            eR = e ;
            sg = 1 ;
            break
        end
    end
          
    %{
    % same but reversed case:
    if CO1r == 1 && CSe(e)==0 % valid for not-closed directions

        scpr(:,1) = cX(1,:);
        u = b2811_PPonePoint (scpr, eDegree, eNcp, eKnotV, eUsize, eP ,ew, eCD, xdim, 1);
        x = b2601_BNURBS_Xcoord (u, eKnotV, eNcp,eDegree,eUsize, ew,eP, 1,xdim);

        CO2r = go2542_coincidentP (x, scp, xdim, 3);

        % if second point also coincides then it is on the edge
        if CO2r==1
            conF = 0.5 ;
            eR = e ;
            sg = -1 ;
            break
        end
    end
    %}
end
      



            
% 2.- FURTHER CHECKS: in case no-corner coincides

if conF==0
for e = 1:1:4
                
    % edCurv(4) = struct ('Ncp',0,  'Degree',0, 'Usize',0, 'KnotV',[], 'w',[], 'P',[], 'conF', 0, 'CD', 0);

    % edge features
    eNcp    = edCurv(e).Ncp ;
    eDegree = edCurv(e).Degree ;
    eUsize  = edCurv(e).Usize ;
    eKnotV  = edCurv(e).KnotV ;
    ew      = edCurv(e).w ;
    eP      = edCurv(e).P ;
    eCD     = edCurv(e).CD ;

    if size(eKnotV,1)>1
        eKnotV = eKnotV';
    end
    
    % compares to edge:
    
    
    % first vtx of curve:
    
    fcp(:,1) = cX(1,:);
    fu = b2811_PPonePoint (fcp, eDegree, eNcp, eKnotV, eUsize, eP ,ew, eCD, xdim, 1);
 	fx = b2601_BNURBS_Xcoord (fu, eKnotV, eNcp,eDegree,eUsize, ew,eP, 1,xdim);
    CO1 = go2542_coincidentP (fx, fcp, xdim, nd);
    
    
 	% second vtx of curve:
    
    scp(:,1) = cX(2,:);
    su = b2811_PPonePoint (scp, eDegree, eNcp, eKnotV, eUsize, eP ,ew, eCD, xdim, 1);
 	sx = b2601_BNURBS_Xcoord (su, eKnotV, eNcp,eDegree,eUsize, ew,eP, 1,xdim);
    CO2 = go2542_coincidentP (sx, scp, xdim, nd);
    


    % checks coincidence & sense:
    
    if CO1==1 && CO2==1
    if fu < su
        
        conF = 0.5 ;
        eR = e ;
        break
        
    end
    end
    
end 
end
        

end

