function [cCurv, TNcu, SCcon, ncu, nlo, CScon, nsf] = ts3311_ECcurves (Nsf, sf, xdim)

% RETURNS:
% cCurv: edge contour curves structure 
% TNcu: total number of curves
% sf updated with curves references information:
%   nlo(Nsf,1): No. loops per surface
%   ncu(Nsf,nlo): No. curves for each surface and loop
%   SCcon(Nsf,nlo,max(ncu)): curve references for each surface
%   CScon(TNcu,nsf): surfaces attached to each curve
%   nsf(TNcu): No. surfaces attached to each curve

%   in SCcon, if sign is negative the curve in sf is in opposite sense

%   LOOPS MUST LEAVE COMPUTABLE DOMAIN AT LHS


% Ncp, Degree, Usize, KnotV, B,w

cCurv(4) = struct ('Ncp',0,  'Degree',0, 'Usize',0, 'KnotV',[], 'w',[], 'P',[], 'conF', 0);

% 1.- COUNTS No. TOTAL CURVES: 

TNcu = 0;

SCcon = zeros(1,1,4);
ncu = ones(Nsf,1)*4;
nlo = ones(Nsf,1); % one loop per surface

CScon = zeros(4,1);
nsf = zeros(4,1);

for s = 1:1:Nsf
    
    
    % 1.1.- Extracts surface features:
    
    necu = 4 ; % here assumes 4 edges per sf, no internal contour loops
    sNcp =      sf(s).Ncp;
    sDegree =  sf(s).Degree;
    sUsize =    sf(s).Usize;
    sP =        sf(s).P;
    sw =        sf(s).w;
    sKnotV =    sf(s).KnotV;
    
    

    % 1.2.- Extracts edges curves features (start at 0,0 and go ANTI-clock-wise:
    
    [edCurv] = ts3313_EdgesContours (sNcp, sDegree, sUsize, sP,sw, sKnotV);
    

    
    % 1.3- Allocates:
    
    if s==1 
        
    % - for 1st surface takes the 4 edges directly:
    
        for c = 1:1:necu
        
            TNcu = TNcu + 1 ;
            
            cCurv(TNcu).Ncp     = edCurv(c).Ncp;
            cCurv(TNcu).Degree  = edCurv(c).Degree;
            cCurv(TNcu).Usize   = edCurv(c).Usize;
            cCurv(TNcu).KnotV   = round(edCurv(c).KnotV,6);
            cCurv(TNcu).w       = edCurv(c).w;
            cCurv(TNcu).P       = edCurv(c).P;
            cCurv(TNcu).conF    = 1 ;
            
            SCcon(s,1,c) = c;
            
            nsf(TNcu,1) = 1;
            CScon(TNcu,nsf(TNcu)) = s ;
            
        end
        
        

    else
        
    % - for rest of surfaces checks if one of the existing curves coincides
    %   with the new proposed:
        
        for c = 1:1:necu
            
            Ncp2     = edCurv(c).Ncp;
            Degree2  = edCurv(c).Degree;
            Usize2   = edCurv(c).Usize;
            KnotV2   = edCurv(c).KnotV;
            w2       = edCurv(c).w;
            P2       = edCurv(c).P;

            for exc = 1:1:TNcu
                
                Ncp1     = cCurv(exc).Ncp;
                Degree1  = cCurv(exc).Degree;
                Usize1   = cCurv(exc).Usize;
                P1       = cCurv(exc).P;
 
                CMP = ts3314_Compares2curves (Ncp1,Degree1, Usize1,P1, Ncp2,Degree2, Usize2,P2, xdim);

                if CMP~=0
                    
                % - finds coincidence: just add ref. to surface
                
                    SCcon(s,1,c)=CMP*exc;
                    nsf(exc,1) = nsf(exc,1)+1;
                    CScon(exc,nsf(exc)) = s ;
                    break
                    
                end
                
            end
            
            % if no coincidence was found => new curve:

            if CMP==0
                    
                % - new curve:
                    
                TNcu = TNcu + 1;
                    
                cCurv(TNcu).Ncp     = Ncp2;
             	cCurv(TNcu).Degree  = Degree2;
              	cCurv(TNcu).Usize   = Usize2;
             	cCurv(TNcu).KnotV   = round(KnotV2,6);
              	cCurv(TNcu).w       = w2;
              	cCurv(TNcu).P       = P2;
               	cCurv(TNcu).conF     = 1 ;
                    
            	SCcon(s,1,c)=TNcu;
             	nsf(TNcu,1) = 1;
                CScon(TNcu,nsf(TNcu)) = s ;
            end
        end
    end
    
    
end






end

