function [edCurv] = ts3313_EdgesContours (sNcp, sDegree, sUsize, sP,sw, sKnotV)

% INPUTS:
% surface NURBS features

% RETURNS:
% The 4 edge curves features:
% curves start at 0,0 and move ANTI-clock-wise

xdim = 3 ;

% 1.- DECLARES STRUCTURE:

edCurv(4) = struct ('Ncp',0,  'Degree',0, 'Usize',0, 'KnotV',[], 'w',[], 'P',[], 'conF', 0, 'CD',0);
% Notes:
% conF is contour flag, which = 1 if curve fully coincides with contour of surface limit
% CD: closed domain flag, which =1 if curve is a closed loop


% 2.- PREPARES DATA FROM SURFACE: assuming to work from (0,0) and moving couter-clock-wise:

de =  [sDegree(1) sDegree(2) sDegree(1) sDegree(2)];    % Degree
usz = [sUsize(1) sUsize(2) sUsize(1) sUsize(2)];        % Kv size
kv = [1 2 1 2];                                         % Kv direction
lncp = [sNcp(1) sNcp(2) sNcp(1) sNcp(2)];               % No. cp's

pr = zeros(4,max(sNcp));                                % cp's and weights refs to use to pick from sP

% curves 1 and 3:
for j = 1:1:sNcp(1)
    
    pr(1,j) = 1 + sNcp(2)*(j-1) ;
    pr(3,j) = sNcp(1)*sNcp(2) - sNcp(2)*(j-1) ;
end

% curves 2 and 4:
for j = 1:1:sNcp(2)
    
    pr(2,j) = sNcp(1)*sNcp(2) - sNcp(2) + j ;
    pr(4,j) = sNcp(2) + 1 - j ;
end



% 3.- LOOPS ON CURVES ATTACHING CORRESPONDANT VALUES:

for cR = 1:1:4

    % 3.1.- degree, Kv size & No. cp's:
    
	edCurv(cR).Degree = de(cR);
	edCurv(cR).Usize = usz(cR) ;
	edCurv(cR).Ncp = lncp(cR);
      
    
	% 3.2.- Knot vector:
    
    KV = sKnotV(kv(cR),1:usz(cR));
    
    if cR <= 2
        edCurv(cR).KnotV = KV;
    else
        KV = GEN_06_ReverseKnotV (KV, lncp(cR), de(cR));
        edCurv(cR).KnotV = KV;
    end
    

    % 3.3.- Control points:
    
    for po = 1:1:lncp(cR)
            
        for di = 1:1:3
            edCurv(cR).P(po,di) = sP(pr(cR,po),di) ;
        end
            
        edCurv(cR).w(po) = sw(pr(cR,po)) ;
    end
        
        

    % 3.4.- Contour flag always 1:
    
    edCurv(cR).conF = 1;
    
    
    % 3.5.- Closed curve: if first and last points coincide
    
    n = edCurv(cR).Ncp ;
    
    fcp = zeros(xdim,1); % first and last curve points
    lcp = zeros(xdim,1);
    
    fcp(:,1) = edCurv(cR).P(1,:);
    lcp(:,1) = edCurv(cR).P(n,:);
    
    [CO] = go2542_coincidentP (fcp, lcp, xdim, 4);
    
    edCurv(cR).CD = CO;
    
end


end

