function [cU, cX, Rc] = ts3315b_oneCvtxs_Cur (R, cNmin, cNcp, cDegree, cUsize, cKnotV, cB, cw, cXdim, cUdim, tolR, tolC)

% same as oneCvtxs but refining where cruvature produces error greater than tolC

% RETURNS:
% lU(nlp,udim), lX(nlp,xdim): coordinate of points of curve attending to
% max. no points and tam() criteria and min. No. segments cNmin
% note it arranges points to achieve ascending parameter order on them


% INPUTS:
% tolC: is tolerance in % w.r.t. segment length 'Rc' to insert new vertex


% 1.- DIVIDES INTO dC-mesh:

[Un, ndC, NE] = ts3316_dCmesh (cNcp, cDegree, cUsize, cKnotV, cB, cw, cXdim, cUdim, tolR);




% 2.- ESTIMATES Current SEGMENT REFERENCE LENGTH (Rc) & No. SEGMENTS (NS):

[Ltotal,Rc,NS] = ts3317_FixSize ...
(cKnotV, cNcp,cDegree,cUsize, cB, cw, cUdim, cXdim, cNmin,R);



% 3.- COMPUTES ACCUMULATED R up to each segment:

Lacc = zeros(NE+1,1);

for e=1:1:NE 
    Du = Un(e+1)-Un(e);
	Lacc(e+1,1) = Lacc(e,1) + 0.5*Du*(ndC(e)+ndC(e+1)); 
end
Lacc(NE+1,1) = Ltotal;


% 4.- ESTIMATES u VALUES FOR ACHIEVING SEGMENTS OF LRx:

cU  = zeros(1,cUdim); % they ll increase to fit No. sub-segments
cX  = zeros(1,cXdim);

uLim = cKnotV(cUsize);
Dumax = [0 uLim-cKnotV(1)];

% 4.1.- First vertex:

conta= 1;
cU(conta,1) = cKnotV(1);

[X] = b2601_BNURBS_Xcoord (cU(conta), cKnotV, cNcp,cDegree,cUsize, cw,cB, cUdim, cXdim);

for di=1:1:cXdim
    cX(conta,di) = X(di);
end 


% 4.2.- Intermediate vertexes:

dC1 = zeros(cXdim,1);
dCt = zeros(cXdim,1);

Ra = 0;
Radd = Rc ;
ouF = 0 ;

while (ouF == 0)

    % temporary target phys. length
    Rat = Ra + Radd;
   
    if Ltotal < Rat % amends to avoid overflow

        Rat = Ltotal;
        ut = uLim ;
        xt = b2601_BNURBS_Xcoord (ut, cKnotV, cNcp, cDegree, cUsize, cw, cB, cUdim, cXdim);

        conta = conta + 1 ;
      	cU(conta,1) = ut;
        cX(conta,:) = xt(:,1);

        ouF = 1;

    else

        % estimates Ucoord and Xcoord to achieve Rat length:
        [ut, xt, ouF] = ts3315c_estimUX (NE, Lacc, Rat, Un, ndC, Dumax, cKnotV, cNcp, cDegree, cUsize, cw, cB, cUdim, cXdim, uLim);

        % checks curvature (circumf-arc analogy):

        % derivative a u1 and ut:

        u1 = cU(conta);
        dC1m = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize, cB ,cw, 1, u1, cXdim,cUdim);
        dC1(:,1) = dC1m(2,1,1,:);

        dCtm = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize, cB ,cw, 1, ut, cXdim,cUdim);
        dCt(:,1) = dCtm(2,1,1,:);


        % angle, chord radius and difference (b): 

        tita  = go2511_angleUV (dC1, dCt) ;

        Vc = zeros(cXdim,1);
        for di = 1:1:cXdim
            Vc(di,1) = cX(conta,di) - xt(di,1) ;
        end

        c = norm(Vc);
        r = c / (2*sin(tita/2)) ;
        a = (r^2-(c/2)^2)^0.5;
        b = r-a;
        

        % computes error and evaluates:

        err = 100*b / Rc;
        if abs(tita) < 0.001
            err = 0 ;
        end

        if err < tolC

            Radd = Rc ;

            Ra = Ra + Rc ;

            conta = conta + 1 ;
            cU(conta,1) = ut;
            cX(conta,:) = xt(:,1);

        else

            % in case too much b => estimates new position using circum-arc assumption:

            badm = tolC/100*Rc ; % admisible error
            cadm = 2*(2*r*badm-badm^2)^0.5 ; % admissible chord

            Radd = cadm;

            Ra = Ra + cadm;

            % estimates Ucoord and Xcoord to achieve Rat length:
            [ut, xt, ouF] = ts3315c_estimUX (NE, Lacc, Ra, Un, ndC, Dumax, cKnotV, cNcp, cDegree, cUsize, cw, cB, cUdim, cXdim, uLim);

            conta = conta + 1 ;
            cU(conta,1) = ut ;
            cX(conta,:) = xt(:,1);

        end

    end

end




%{
% 4.3.- Last vertex:

conta = conta + 1 ;

x = b2601_BNURBS_Xcoord (uLim, cKnotV, cNcp, cDegree, cUsize, cw, cB, cUdim, cXdim);

cU(conta) = uLim;
cX(conta,:) = x(:,1);
%}

%{
% estimates last segment phys. length: if too close, move last point to
% end, if too far, just adds a new last point

dCu = zeros(cXdim,1);

u1 = cU(conta);
dC = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize, cB ,cw, 1, u1, cXdim,cUdim);
dCu(:) = dC(2,1,1,:);
ndC1 = norm(dCu); 

u2=uLim;
ndC2 = ndC(NE+1);

Rlast = 0.5*(u2-u1)*(ndC1+ndC2);

if Rlast < 0.5*Rc % move to end
    
    cU(conta) = uLim;
    
else % create a new one at the end
    
    conta = conta+1;
    cU(conta) = uLim;
end
%}


end
