function [Un, ndC1, NE] = ts3316_dCmesh (cNcp, cDegree, cUsize, cKnotV, cB, cw, cXdim, cUdim, tolR)

% RETURNS:
% Un(NE+1):     curve param. coord's of dCmesh nodes
% ndC(NE+1):    norm of 1st derivative at each node
% NE:           No. elements of the mesh


% 1.- INITIAL ARRANGEMENT AS KNOT SPANS:

% nip = 2 ; % No. internal points for sampling, used in first iteration only

[Un, NN] = GEN_18_UnonVoid (cKnotV, cUsize, cDegree, cUdim);

NE = NN-1 ; % No. elements
eULim = zeros(NE,2);

for e = 1:1:NE
    eULim(e,1) = Un(e);
    eULim(e,2) = Un(e+1);
end


dC1 = zeros(NN, cXdim); 
dC2 = zeros(NN, cXdim); 
dC3 = zeros(NN, cXdim); 

ndC1 = zeros(NN, 1); 


for i=1:1:NN
    
    % derivatives at each node:
    
    u = Un(i);
    dC = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize, cB ,cw, 3, u, cXdim,cUdim);

    dC1(i,:) = dC(2,1,1,:);
    dC2(i,:) = dC(3,1,1,:);
    dC3(i,:) = dC(4,1,1,:);

    ndC1(i,1) = norm(dC1(i,:));
    
end


% 2.- REFINES TO REDUCE ERROR BELOW tolR:

dg1 = zeros(cXdim,1);
dg2 = zeros(cXdim,1);
dg3 = zeros(cXdim,1);

errEm=ones(NE,1);
outF = 0 ;

contaL = 0 ;

while(outF==0)
    
    contaL = contaL+1;
    
    % 2.1.- Computes errors and stores elms to divide:
    
    NEer=0;
    Eer = zeros(1,1);
    
    for e = 1:1:NE
         
        Du = Un(e+1) - Un(e);
        
      	% Estimates length:
        
    	L = 0.5*Du*(ndC1(e+1)+ndC1(e));
        
        
        % Computes f'' = g''' in both locations and selects the greatest f'': 
        
        df2t = zeros(2,1);
        
        for i = 1:1:2
            
            nR = e+i-1;

            f = ndC1(nR);
            
            dg1(:,1) = dC1(nR,:);
            dg2(:,1) = dC2(nR,:);
            dg3(:,1) = dC3(nR,:);
            
            [df12] = ts3321_df12 (f, dg1, dg2, dg3, cXdim);
            df2t(i) = df12(2);
        end
        
        df2 = max(df2t);
        

        % Absolute and relative error:
        
        aE = -1/12 * df2 *Du^3;
        errEm(e) = 100*abs(aE/L);
        
        if errEm(e)>tolR
            NEer = NEer+1;
            Eer(NEer,1) = e ;
        end
        
    end
    
    
    % 2.2.- Divides elements if error > tolR:


    outF = 1;
    if NEer > 0
        
        outF = 0;
        kn = zeros(NEer,1);
        
        % computes mid-span value:
        
        for er = 1:1:NEer
            
            e = Eer(er);
            kn(er,1) = (Un(e)+Un(e+1))/2;
            
        end
        
        
        % inserts and computes derivatives:
        
        for i=1:1:NEer
        for j=1:1:NE+1
            
            if Un(j)<kn(i) && kn(i)<Un(j+1)

                Un = [Un(1:1:j), kn(i), Un(j+1:1:NE+1)];
                
                dC = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize, cB ,cw, 3, kn(i), cXdim, cUdim);
                
                dg1(:) = dC(2,1,1,:);
                dg2(:) = dC(3,1,1,:);
                dg3(:) = dC(4,1,1,:);
                
                dC1 = [ dC1(1:j,:) ; dg1' ; dC1(j+1:NE+1,:) ] ;
                dC2 = [ dC2(1:j,:) ; dg2' ; dC2(j+1:NE+1,:) ] ;
                dC3 = [ dC3(1:j,:) ; dg3' ; dC3(j+1:NE+1,:) ] ;
                
                ndC1 = [ndC1(1:1:j); norm(dg1); ndC1(j+1:1:NE+1)];

                
                break
            end  
            
        end
        NE = NE+1;
        end

    end

end


end

