function [cLXvP, cLUvP, NcvP, qcfP] = ts3322_CvtxsPrev (TNcuP, agCurvP, TNcuE, agCurvE, cLUvE, cLXvE, NcvE, qcfE, R, Nvx, tolRc, xdim, tolC, curveError)

% RETURNS:

% cLXvP, cLUvP, NcvP: vertexes fo curves 
% qcfP(TNcuP,1): quadratic curve flag =1 if curve is quadratic
% uses 'normal procedure' : ts3315_oneCvertexes
% OR
% transfers from already computed curves if coincides: agCurvE, TNcuE, cLUvE, cLXvE

% INPUTS:
% tolC: admisible error in % w.r.t. Rc of actual curve and segment (assumes cicumf. arc)
% curveError = 'yes' if considers tolC

    
% 1.- Preliminar sizes of vertexes arrays:
    
cUdim = 1 ;

cLXvP = zeros(TNcuP,1,xdim);
cLUvP = zeros(TNcuP,1,cUdim);
NcvP  = zeros(TNcuP,1);
qcfP  = zeros(TNcuP,1);

pccF = zeros(TNcuP,1); % patch curves computed Flag = 1 if curve has been computed in B.E. curves
    
    
% 2.- Detects if any coincides with B.E. aggregated curves:
    
for cP = 1:1:TNcuP
        
    % curve to check:  
	NcpP     = agCurvP(cP).Ncp;
	DegreeP  = agCurvP(cP).Degree;
	UsizeP   = agCurvP(cP).Usize;
	PP       = agCurvP(cP).P;

    for cE = 1:1:TNcuE

        % curve to compare with: 
        NcpE     = agCurvE(cE).Ncp;
        DegreeE  = agCurvE(cE).Degree;
    	UsizeE   = agCurvE(cE).Usize;
    	PE       = agCurvE(cE).P;
            
        CMP = ts3314_Compares2curves (NcpP,DegreeP,UsizeP,PP, NcpE,DegreeE,UsizeE,PE, xdim);
            
    	% finds coincident curve (either same or opp. sense):
            
        if CMP ~= 0
            
            pccF(cP) = 1; % switches flag to computed
                
            % fills vertexes arrays
                
            NcvP(cP,1) = NcvE(cE,1);
            qcfP(cP,1) = qcfE(cE,1); % quadratic curve flag    
            
            % first the corner-vtx
            
            N = NcvE(cE,1);
            
            ut = zeros(N,1);
            xt = zeros(N,xdim);
            
            for i = 1:1:N         
                ut(i,1) = cLUvE(cE,i,1); 
                xt(i,:) = cLXvE(cE,i,:) ;
            end
            
            if CMP == -1 % reverses if necessary:
                ut = GEN_40_invertRowsOrder(ut);
                xt = GEN_40_invertRowsOrder(xt);
            end
            
            U = ut;
            X = xt ;
            
            % if quadratic => add mid-vtx
            
            if qcfP(cP,1)==1
                
                umt = zeros(N-1,1);
                xmt = zeros(N-1,xdim);
                
                for mvP = 1:1:NcvE(cE,1)-1
                
                    j = N ;
                    for i = 1:1:N-1
                        
                        j = j+1;
                        umt(i,1) = cLUvE(cE,j,1); 
                        xmt(i,:) = cLXvE(cE,j,:) ;
                    end
                end
                
                if CMP == -1 % reverses if necessary:
                    umt = GEN_40_invertRowsOrder(umt);
                    xmt = GEN_40_invertRowsOrder(xmt);
                end
                
                U = [ut;umt];
                X = [xt;xmt];
                N = N + N - 1;
                
            end
            
            
            % allocates:
            for i = 1:1:N
                cLUvP(cP,i,1) = U(i,1) ; 
                cLXvP(cP,i,:) = X(i,:) ; 
            end
            
            %{
            vE = 1;
            if CMP==-1
                vE = NcvE(cE,1);
            end
                
            for vP = 1:1:NcvE(cE,1)
                        
                cLUvP(cP,vP,1) = cLUvE(cE,vE,1) ; 
                
                for di = 1:1:xdim
                    cLXvP(cP,vP,di) = cLXvE(cE,vE,di) ;
                end
                
                vE = vE + CMP;
            end
            %}
            
            break
            
        end
            
    end  
end
   
    
    
% 3.- Computes vertexes of curves not computed previously:
    
for cP = 1:1:TNcuP
    if pccF(cP)==0
            
        % extracts that curve features:

        NcpP     = agCurvP(cP).Ncp;
        DegreeP  = agCurvP(cP).Degree;
        UsizeP   = agCurvP(cP).Usize;
        KnotVP   = agCurvP(cP).KnotV;
    	wP       = agCurvP(cP).w;
    	PP       = agCurvP(cP).P;
        

        % computes its vtxs:
        
        [cU, cX,~,~] = ts3315_oneCvtxs (R, Nvx, NcpP, DegreeP, UsizeP, KnotVP, PP, wP, xdim, 1, tolRc);


        % allocates vertexes:
        
        NcvP(cP,1) = size(cX,1);
        
        for i = 1:1:NcvP(cP,1)
                        
            cLUvP(cP,i,1) = cU(i) ;
       
            for di = 1:1:xdim
            cLXvP(cP,i,di) = cX(i,di) ;
            end     
        end
            
    end
end
    


end

