function [ndS, dSu, dSv] = ds3328_dSspline (u, tita, Dspline, udim, xdim)

% RETURNS:
% ndS: norom of tita-directional derivative
% dSu: main deriv. w.r.t. u
% dSv: main deriv. w.r.t. v

ELim = Dspline.ELim;
NE = Dspline.NE;


% 1.- FINDS ELEMENT(S) WHERE u LIES IN:

terU = zeros(1,1); % temporary elems reference in U direction
nU = 0 ;

for e = 1:1:NE
    if ELim(e,1) <= u(1) && u(1) <= ELim(e,2)
        nU = nU+1;
        terU(nU,1) = e ;
    end
end

terV = zeros(1,1); % temporaty elems refernce in V direction
nV = 0 ;

for e = terU(1):1:terU(nU)
    if ELim(e,3) <= u(2) && u(2) <= ELim(e,4)
        nV = nV+1;
        terV(nV,1) = e ;
    end
end


% 2.- SELECTS RELEVANT ELEMENT:  

% 2.1.- Case of 1 element:

if nV == 1
    eR = terV(1);
end


% 2.2.- Case of 4 elements => corner => selects the simplest element

if nV == 4

    for e = 1:1:nV
        
        ter = terV(e);
        
        if e == 1
            
            n1 = Dspline.fs(ter).dNcp(1) * Dspline.fs(ter).dNcp(2);
            eR = ter;

        else
            
            ntem = Dspline.fs(ter).dNcp(1) * Dspline.fs(ter).dNcp(2);
            if ntem < n1
                n1 = ntem;
                eR = ter;
            end 

        end
    end
end


% 2.3.- Case of 2 elements => edge => selectst the finest one

if nV == 2

    for e = 1:1:nV
        
        ter = terV(e);
        
        if e == 1
            
            n1 = Dspline.fs(ter).dNcp(1) * Dspline.fs(ter).dNcp(2);
            eR = ter;

        else
            
            ntem = Dspline.fs(ter).dNcp(1) * Dspline.fs(ter).dNcp(2);
            if ntem > n1
                n1 = ntem;
                eR = ter;
            end  

        end
    end
end



% 3.- EXTRACTS SPLINE FEATURES & COMPUTES BASIS FUNCTIONS & MAIN DERVATIVES

dKnotV  = Dspline.fs(eR).dKnotV;
dNcp    = Dspline.fs(eR).dNcp;
dDegree = Dspline.fs(eR).dDegree;
dUsize  = Dspline.fs(eR).dUsize;

Nicp = [dDegree(1)+1 ; dDegree(2)+1];

% span where u lies:
spR = zeros(udim,1);

for di = 1:1:udim
    
    if u(di)==dKnotV(di,1)
        
        spR(di) = 1;
        
    elseif u(di) == dKnotV(di,dUsize(di))
        
        spR(di) = dNcp(di)-dDegree(di);
        
    else
        
        for i = dDegree(di)+1:1:dUsize(di)

            if dKnotV(di,i)<= u(di) < dKnotV(di,i+1)
                spR(di) = i-dDegree(di)-1;
                break
            end
            
        end
        
    end
end



dSu = zeros(xdim,1);
dSv = zeros(xdim,1);

EP = zeros(6,dNcp(1), dNcp(2));

for f = 1:1:6
    EP(f,:,:) = Dspline.fs(eR).cp(f).dEP(:,:,3);
end


for i = spR(1):1:spR(1)+Nicp(1)-1
for j = spR(2):1:spR(2)+Nicp(2)-1
    
    B = b2604_MNL (u, [i,j], dKnotV, dNcp, dDegree, dUsize, udim);
    
    for xd = 1:1:xdim 
        
        dSu(xd,1) = dSu(xd,1) + B * EP(xd,i,j);
        dSv(xd,1) = dSv(xd,1) + B * EP(xd+xdim,i,j);

    end
    
end
end



% 5.- DIRECTIONAL DERIVATIVE:

dS = cos(tita)*dSu + sin(tita)*dSv;

ndS = norm(dS);



end

