function [NE, TVcon, VTcon, LVcon, VLcon, TLcon, Rv, Uv, Xv, XNe, XNv, Ae] = rs3334_Rtriang ...
                (cRv,sRv, cUv,sUv, cXv,sXv, Nv, NL, Nlv, Nlcv, ncu, SCcon, sR, xdim)

% RETURNS
% NE: No. elements of tri-mesh
% TVcon(NE,3): connectivy matrix
% LVcon(NE,2): connectivy matrix
% All In-vtx coordinates
% XNe, XNv: normals to triangles and vertexes


% Note: the routine meshes in pattern space

%{
if sR==3
   figure(sR)
    scatter( sRv(:,1), sRv(:,2), 'r'); hold on
    axis equal
end
%}

% 1.- ASSEMBLIES ALL VERTEXES COORDINATES: first are contours edges vertexes

if Nv > 0

    % 1.1.- Case there are SI-vtx:
    
    Rv = sRv;
    Uv = sUv;
    Xv = sXv;

    % transfers contour vtx to column arrays to attach to total vtx:
    % note keep the order, i.e. first contour will apear first in Rv/Uv/Xv:

    for lo = NL:-1:1

        r = zeros(Nlv(lo),2);
        u = zeros(Nlv(lo),2);
        x = zeros(Nlv(lo),xdim);

        for v = 1:1:Nlv(lo)
            r(v,:) = cRv(lo,v,:);
            u(v,:) = cUv(lo,v,:);
            x(v,:) = cXv(lo,v,:);
        end

        Rv = [r ; Rv];
        Uv = [u ; Uv];
        Xv = [x ; Xv];

    end

    %{
    if sR==3
       figure(sR)
        scatter( cRv(1,:,1), cRv(1,:,2), 'k'); hold on
        axis equal
    end
    %}
else
    
    % 1.2.- Case there are not SI-vtx:
    
    lo = 1 ;
	Rv = zeros(Nlv(lo),2);
	Uv = zeros(Nlv(lo),2);
 	Xv = zeros(Nlv(lo),xdim);
    
    for v = 1:1:Nlv(lo)
        Rv(v,:) = cRv(lo,v,:);
       	Uv(v,:) = cUv(lo,v,:);
     	Xv(v,:) = cXv(lo,v,:);
    end
    
    for lo = NL-1:-1:1

        r = zeros(Nlv(lo),2);
        u = zeros(Nlv(lo),2);
        x = zeros(Nlv(lo),xdim);

        for v = 1:1:Nlv(lo)
            r(v,:) = cRv(lo,v,:);
            u(v,:) = cUv(lo,v,:);
            x(v,:) = cXv(lo,v,:);
        end

        Rv = [r ; Rv];
        Uv = [u ; Uv];
        Xv = [x ; Xv];

    end
    
end


%{

figure(23)
scatter(Rv(:,1),Rv(:,2),'r'); hold on
for i=1:1:size(Rv,1)
text(Rv(i,1),Rv(i,2),num2str(i)); hold on
end
axis equal

figure(24)
scatter3(Xv(:,1),Xv(:,2),Xv(:,3),'r'); hold on
for i=1:1:size(Rv,1)
text(Xv(i,1),Xv(i,2),Xv(i,3),num2str(i)); hold on
end
axis equal

%}

% 2.- DEFINES CONSTRAINTS:

[Cnst] = tr1345_CnstTri (NL, Nlv);



% 3.- GENERATES THE 'GROSS' MESH with constraints & REMOVES OUTSIDE TRIANGLES:

[TVcon,~, NE, NV] = tr1346_triangCnst (Rv, Cnst, sR);

%{
%if sR==3

    xnod = zeros(3,xdim);
    for t = 1:1:NE

        Xav = zeros(xdim,1);
        for i = 1:1:3
           xnod(i,:) = Xv(TVcon(t,i),:);
           
           for di = 1:1:xdim
               Xav(di,1) = Xav(di,1) + Xv(TVcon(t,i),di)/3;
           end
        end

        pt1361_plotTri_nD (xnod, xdim, sR, 'b')

        axis equal
        
        %figure(sR)
        %text(Xav(1), Xav(2), Xav(3),num2str(t)); hold on
        %axis equal
    end

%end
%}




% 4.- IMPROVES THE MESH BY AMEDING THE EDGE STRIP:

[TVcon] = rs3335_ESimprov(sR, NL, ncu, NE, TVcon, Nlcv, Xv, xdim);

%{
if sR==3

    xnod = zeros(3,xdim);
    for t = 1:1:NE

        Xav = zeros(xdim,1);
        for i = 1:1:3
           xnod(i,:) = Xv(TVcon(t,i),:);
           
           for di = 1:1:xdim
               Xav(di,1) = Xav(di,1) + Xv(TVcon(t,i),di)/3;
           end
        end

        pt1361_plotTri_nD (xnod, xdim, sR, 'r')

        %figure(sR)
        %text(Xav(1), Xav(2), Xav(3),num2str(t)); hold on
        %axis equal
    end

end
%}



% 5.- OTHER CONN. MATRICES:

[LVcon, LTcon, VLcon, TLcon, VTcon] = tr1347_otherConn (NL, Nlv, NE, NV, TVcon, Uv);

%{
NLI = size(LVcon,1) ;
Eiem = zeros(NE,3,3);
pt1369_TriMDMesh (0.5, 0.5, Xv, NLI, NE, LVcon, TVcon, Eiem, 1, 1, 0, 0, 1, 1);
%}

% 6.- NORMAL VECTORS & TRIANGLES AREA IN PHYS SPACE:
spF = 0 ;
[Ae, XNe, XNv] = tm1308_TmeshNA (spF, NE, NV, TVcon, VTcon, Xv, xdim);

%{
Ae  = zeros(1,xdim);
XNe = zeros(1,xdim);
XNv = zeros(NV,xdim);

for e = 1:1:NE 
    
	% normals to triangles (norm=1) and area:
        
	[nv, ~, At] = i0103_nTriang (TVcon, e, Xv, xdim);
        
 	Ae(e,1) = At;
    
    for di = 1:1:xdim
        XNe(e,di) = nv(di);
    end
end


% Normals to vertexes (averages attached triangles weighting the area):

for v = 1:1:NV
    
    [nvv] = i0106_nVtx (v, VTcon, Ae, XNe, xdim);
    
    for di = 1:1:xdim
        XNv(v,di) = nvv(di); 
    end
    
end


%}


end

