function [TVcon] = rs3335_ESimprov(sR, NL, ncu, NE, TVcon, Nlcv, Xv, xdim)

% RETURNS:
% TVcon matrix with edges improved

% INPUTS:
% NL: No. loops
% Nc: No curves of that surface
% Nlcv(NL,cu); No. vtx per curve of each loop



for lo = 1:1:NL
    
    % 1.- PRESELECTS ALL TRIANGLES THAT CONTAIN ONE EDGE VERTEX 

    % stores them in etr(Nc,NTe) (separated by curves)

    Nc = ncu(sR,lo);
    
    NTe = zeros(Nc,1);  % No. triangles at each edge curve
    etR = zeros(Nc,1);  % edge triangles references, second 1 will increase

    for t = 1:1:NE

        for v = 1:1:3

            accV = 0 ;
            for c = 1:1:Nc

                fvR = accV + 1;         % first vtx ref of that curve 'c'
                evR = Nlcv(lo,c)+accV;  % end   vtx ref of that curve 'c'

                if c < Nc
                   evR = evR + 1 ; 
                end

                if fvR <= TVcon(t,v) && TVcon(t,v)<= evR % belongs to that edge

                    % only adds if is 1st triangle OR is not repeated

                    if NTe(c,1)==0

                        NTe(c,1) = NTe(c,1)+1;
                        etR(c,NTe(c,1)) = t ;

                    else

                        if etR(c,NTe(c,1))~= t 
                            NTe(c,1) = NTe(c,1)+1;
                            etR(c,NTe(c,1)) = t ;
                        end

                    end
                end

                accV = accV + Nlcv(lo,c) ;
            end

        end

    end

   

    

    % 2.- ARRANGES TRIANGLES TO FOLLOW THE CONTOUR CURVES FOR 1ST CURVE:

    ucF = 1 ; % unique curve flag = 1 if only one curve in the contour
    if Nc==1
        ucF = 0;
    end
    
    accV = 0 ;
    
    pcr = 0 ; % prev. curve reference
	ncr = 1; % curve local reference

	Ntrp = 0 ;

	[atRc, NT, accV] = rs3338a_concTr (sR, Nlcv, lo, pcr, ncr, NTe, etR, accV, Nc, Ntrp, TVcon, ucF, Xv);

    NTe(1,1) = NT ;
    etR(1,1:NT) = atRc(1:NT,1);



    
    % 3.- COMPARES TRIANGLES BY PAIRS, FORMING A QUADRILATERAL AND SELECTING THE SHORTEST DIAGONAL:

    ncRm = [2:1:Nc]; ncRm(Nc) = 1 ; % next curve reference matrix

    for c = 1:1:Nc

        
        for t = 1:1:NT-1

            % 3.1.- Triang. ref's:

            tr1 = atRc(t);
            tr2 = atRc(t+1);


            % 3.2.- Constructs the 4 edges of the quadrilateral:

            % detects party edge vtxs (they're the two shared by the triangles):
            conta = 0 ;

            for i= 1:1:3

                v1 = TVcon(tr1,i);
                for j = 1:1:3

                    v2 = TVcon(tr2,j);
                    if v2 == v1
                        conta = conta+1;
                        pev(conta) = v1;
                        break
                    end
                end
            end

            

            % detects corner vtx's (the two opposites ones):
            cov = zeros(2,1);

            for i= 1:1:3
            if TVcon(tr1,i)~=pev(1) && TVcon(tr1,i)~=pev(2)
                cov(1) = TVcon(tr1,i);
            end
            end

            for i= 1:1:3
            if TVcon(tr2,i)~=pev(1) && TVcon(tr2,i)~=pev(2)
                cov(2) = TVcon(tr2,i);
            end
            end


            % constructs contour (either clock or counter-clock wise)
            % vertexes 2 and 4 are the current diagonal
            vR = [cov(1) pev(1) cov(2) pev(2)]';

            

            % 3.4.- Measures both diagonals:

         	d1 = norm(Xv(vR(1),:) - Xv(vR(3),:));
          	d2 = norm(Xv(vR(2),:) - Xv(vR(4),:)); % diag. 2 is as it comes

            
            % might swap the diagonal: updates TVcon is the change is allowed (see inside the function)

            if d1 < d2 
                [TVcon] = rs3335b_swapDiag (t, tr1, tr2, TVcon, Xv, vR, NT, atRc, xdim);
            end
            
        end


        % 3.4.- Arranges next curve triangles:

        if c < Nc

            ncr = ncRm(c); % next curve reference

            Ntrp = 6 ;
            
            if NTe(c,1)<6
                Ntrp = round(NTe(c,1)*0.3,0) ;
            end
            
            
            [atRc, NT, accV] = rs3338a_concTr (sR, Nlcv, lo, c, ncr, NTe, etR, accV, Nc, Ntrp, TVcon, ucF, Xv);
            
            % updates etR:
         	NTe(ncr,1) = NT ;
            etR(ncr,1:NT) = atRc(1:NT,1);
            
        end
    


    end

end % loop on contour loops

end

