function [cRv] = rs3337_coRcoor (sRvt, sUvt, coF, cUv, NL, Nlv, udim, sR)

% RETURNS:
% cRv(nlo,Npo, 2): pattern coords of contour loops points

% INPUTS:
% sRvt(n,2), sUvt(n,2): pattern and param. coord's of all surface vexts
% coF(n): computed flag of all surface vtxs, if 1 they are computed
% cUv(NL,Nlv(i),2): param coords (in terms of surface param. space) of contour loops of surface
% NL: No. loops of surface
% Nlv(NL,1): No. vtxs of each contuour loop
% udim 
% sR: surface reference, used only to print plots (see end of the routine)

  
% 1.- PICKS UP ALL COMPUTED VERTEXES FROM SURFACE
% pattern and param. coordinates
    
NVt = 0 ; % No. total computed vertexes
    
sRvc = zeros(1,udim);
sUvc = zeros(1,udim);
    
for po = 1:1:size(sUvt,1)
        
    if coF(po)==1

        NVt = NVt+1;

        for di=1:1:udim
        sRvc(NVt,di) = sRvt(po,di);
        sUvc(NVt,di) = sUvt(po,di);
        end
    end
end
    

% 2.- TRIANGULATES THOSE POINTS IN PARAMETER SPACE:

[DT, NE, TVcon] = tm1301_genTri (sUvc, 0);
    

% 3.- RUNS ON ALL CONTOUR POINTS TO FIND THEIR PATTERN COORD'S USING THE
% TRIANGULATION COMPUTED IN PREV. STEP:
    
cRv = zeros(NL,max(Nlv),udim) ;
    
Vc = sUvc;
Vz = sRvc;
    
zdim = 2 ;
    
    
for lo = 1:1:NL
for v = 1:1:Nlv(lo)
        
    
    % 3.1.- Coordinates to find in param. space trianguation:
    
   	c = [ cUv(lo,v,1) , cUv(lo,v,2)];
    
    
    % 3.2.- Carries out triangular linear interpolation:
    
	[z, outF] = tm1302_interpTri (DT, TVcon, Vc, Vz, c, zdim);
        
    
    
	% 3.3.- If out of domain searches for closer triangle side, 
  	% projects on it and interlopates:
    
    if outF==1
                
        % cevR(2): closest edge-vertex referecnes to c
                
        [cevR] = tm1303_closerEdge (Vc, c);
                
                
        % coordinates of closest boundary vtx:
                
        ce1 = [Vc(cevR(1),1) Vc(cevR(1),2)];
        ce2 = [Vc(cevR(2),1) Vc(cevR(2),2)];
                
                         
        % finds projection from c onto ce1-ce2: cpj
                
      	[~, d] = go2527_distPtoRecta (c, ce1, [ce1-ce2], udim);
    	cpj = c+d';
         
                
    	% interplates between ce1 and ce2:
                
      	N = zeros(2,1);
                
      	L = norm(ce1-ce2);
                
      	N(1) = norm(cpj-ce2)/L;
    	N(2) = norm(cpj-ce1)/L;
                

        for di = 1:1:zdim
                    
            ze1 = Vz(cevR(1),di);
            ze2 = Vz(cevR(2),di);
                    
            z(di) = N(1)*ze1 + N(2)*ze2;
        end
    end
         
    
    % 3.4.- Allocates values:
    
	cRv(lo,v,1) = z(1);
 	cRv(lo,v,2) = z(2);
    
end
end
    
    
%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sR==3
    
    figure(sR)
    scatter(sUvc(:,1),sUvc(:,2),'k'); hold on
    
    figure(10*sR)
    scatter(sRvc(:,1),sRvc(:,2),'k'); hold on
    
    for lo = 1:1:NL
        
        figure(sR)
        scatter(cUv(lo,:,1),cUv(lo,:,2),'r'); hold on
     	axis equal
            
        figure(10*sR)
    	scatter(cRv(lo,:,1),cRv(lo,:,2),'r'); hold on
    	axis equal
        

        
    end
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
%}
    
    
end

