function [sRvi, sUvi, sXvi, Nvi] = sv3344_SvforT (sR, eswP, sRv, sUv, sXv, Nv, cUv, cXv, NL, Nlv, xdim, udim)

% RETURNS: coord's of surface vtx valid for triangulation (at LHS of
% contours) and separated from them a distance equal or greater than ews

% INPUTS:
% sR: surface ref, for debugging purpose

% vtxs to check if are in or out:
    
Nvi = Nv ;
sRvi = zeros(1,udim);
sUvi = zeros(1,udim);
sXvi = zeros(1,xdim);


if Nvi > 0 % carries on if there are internal vtx to check

    vt1 = zeros(Nv,1);
    vt2 = zeros(Nv,1);

    vt1(:,1) = sUv(:,1);
    vt2(:,1) = sUv(:,2);
    

    % piece-wise contours: Note they leave at LHS computable domain

    ct1 = zeros(sum(Nlv)+NL-1,1); % plan U-coord's of contours
    ct2 = zeros(sum(Nlv)+NL-1,1);

    acc = 0;
    for lo = 1:1:NL

        j = 0;
        for i = acc+1 : 1 : acc+Nlv(lo)
            j = j + 1 ;
            ct1(i) = cUv(lo,j,1);
            ct2(i) = cUv(lo,j,2);
        end

        if NL > 1
            ct1(i+1) = NaN;
            ct2(i+1) = NaN;
        end

        %{
        
        if sR==8
            figure(1)
            plot(cUv(lo,:,1),cUv(lo,:,2),'b'); hold on
            axis equal
        end
        %}

        acc = acc+Nlv(lo)+1;
    end


    % works out the check:

    vtinF = inpolygon(vt1,vt2,ct1,ct2); % flag = 1 if in


    % allocates to response:

    Nvt = sum(vtinF); % No. vtx inside

    sRvt = zeros(Nvt,udim);
    sUvt = zeros(Nvt,udim);
    sXvt = zeros(Nvt,xdim);

    i = 0;
    for j = 1:1:Nv
        if vtinF(j)==1

            i = i+1;
            sRvt(i,:) = sRv(j,:);
            sUvt(i,:) = sUv(j,:);
            sXvt(i,:) = sXv(j,:);
        end
    end
 
    
    % Checks distances: delete in case is closer than ews in physical space:
    
    Nvi = 0 ;
 	sRvi = zeros(1,udim);
    sUvi = zeros(1,udim);
    sXvi = zeros(1,xdim);
    
    x = zeros(xdim,1);
    x1 = zeros(xdim,1);
    xm = zeros(xdim,1);
    x2 = zeros(xdim,1);
    
    for j = 1:1:Nvt

        x(:,1) = sXvt(j,:);
        
        
        % searches closest loop and point:
        
        cLP = zeros(NL,3); % loop ref, point ref, dist
        
        for lo = 1:1:NL
            
            dmin = 9999999999;
            for po = 1:1:Nlv(lo)
               
                xm(:,1) = cXv(lo,po,:);
                
                if norm(xm-x) < dmin
                    
                    dmin = norm(xm-x);
                    cLP(lo,1) = lo;
                    cLP(lo,2) = po;
                    cLP(lo,3) = dmin;
                end
            end
        end
        
        cLP = sortrows(cLP,3);
        
        lr = cLP(1,1);
        pr = cLP(1,2);
        distP = cLP(1,3); % distance to point
        
        pr1 = pr - 1;
        pr2 = pr + 1;
        
        if pr == 1
            pr1 = Nlv(lr);
        elseif pr == Nlv(lr)
            pr2 = 1;
        end
        
        % find distance to both segments:
        
        x1(:,1) = cXv(lr,pr1,:);
        x2(:,1) = cXv(lr,pr ,:);
        vr = x2-x1;
        s1 = norm(vr);
        
        [dist1, d] = go2527_distPtoRecta (x, x1, vr, xdim);
        
        Q = x+d;
        ON = go2520_PonSegment (Q, x1,x2, xdim, 0.5);
        
        if ON==0
            dist1 = 10*distP;
        end
        
        
      	x1(:,1) = cXv(lr,pr ,:);
        x2(:,1) = cXv(lr,pr2 ,:);
        vr = x2-x1;
        s2 = norm(vr);
        
        [dist2, d] = go2527_distPtoRecta (x, x1, vr, xdim);
        
        Q = x+d;
        ON = go2520_PonSegment (Q, x1,x2, xdim, 0.5);
        
        if ON==0
            dist2 = 10*distP;
        end
        
        % Computes min. distance esw and compares:
        
        inF = 0;
        
        if dist1 < distP 
            esw = s1*eswP/100;
            if esw < dist1
                inF = 1 ;
            end
        elseif dist2 < distP
            esw = s2*eswP/100;
            if esw < dist2
                inF = 1 ;
            end
        else
            esw = (s1+s2)/2*eswP/100;
            if esw < distP
                inF = 1 ;
            end
        end
        
        % includes if is further than esw:
        if inF==1
            
            Nvi = Nvi+1 ;
            
        	sRvi(Nvi,:) = sRvt(j,:);
            sUvi(Nvi,:) = sUvt(j,:);
            sXvi(Nvi,:) = sXvt(j,:);

        end
        
        
    end
    
    
    
end



end

