function [strF, Ld] = sv3346_Strip (Ncp, EP, R, pt, xdim, udim)

% RETURNS:
% strF = 1 if the surface is a strip
% Ld: longitudional direction of the strip

strF = 0 ;
Ld = 0 ;

a = zeros(xdim,1);
b = zeros(xdim,1);

sL = zeros(udim, max(Ncp));

% 1.- MEASURES CONTROL NET SEGMENT PARALLEL TO:

% to direction 2

di = 2 ;

for i = 1:1:Ncp(1)
    for j = 1:1:Ncp(2)-1

        a(:,1) = EP(i,j,:);
        b(:,1) = EP(i,j+1,:);
        d = norm(a-b);
        
        sL(di,i) = sL(di,i) + d;
    end
end


% to direction 1

di = 1 ;

for j = 1:1:Ncp(2)
    for i = 1:1:Ncp(1)-1

        a(:,1) = EP(i,j,:);
        b(:,1) = EP(i+1,j,:);
        d = norm(a-b);
        
        sL(di,j) = sL(di,j) + d;
    end
end


% maximuns in each direction:
wmax = zeros(udim,1);

wmax(1) = max(sL(1,:));
wmax(2) = max(sL(2,:));



% 2.- EVALUATES AVERAGE of width AND VARIATION COEFFICIENT:

wav = zeros(udim,1);
Sw  = zeros(udim,1); % variance in percentage

pdm = [2 1];

for di = 1:1:udim
    
    pd = pdm(di);
    
    % average:
    wav(di,1) = 0 ;
    for i = 1:1:Ncp(pd)
        wav(di,1) = wav(di,1) + sL(di,i)/Ncp(pd) ;
    end
    
    % Variance:
  	for i = 1:1:Ncp(pd)
        Sw(di,1) = Sw(di,1) + (sL(di,i)-wav(di,1))^2 ;
    end
    
    Sw(di,1) = 100/wav(di) * (Sw(di,1)/Ncp(pd))^0.5 ;
    
end


% 3.- CHECKS STRIP CONDITIONS:

% - conditions 1 and 2 must meet simultaneoulsly
% - condition 3 is sufficient by itself

% 3.1.- Condition 1:
% Average width is less than 2R 

c1F = zeros(udim,1) ;

for di = 1:1:udim
    
    if wav(di)<2*R
        c1F(di)=1 ;
    end
end


% 3.2.- Condition 2: var. coeff is less that threshold 'pt'

c2F = zeros(udim,1) ;

for di = 1:1:udim
    
    if Sw(di) < pt
        c2F(di)=1 ;
    end
end


% 3.3.. Condition 3: maximum widht is less than 2R

c3F = zeros(udim,1) ;

for di = 1:1:udim
    
    if wmax(di)<2*R
        c3F(di)=1 ;
    end
end



% 4.- DETERMINES IF IS STRIP AND LONGITUDINAL DIRECTION:

% 4.1.- Direction 1:

di = 1 ;

if c1F(di) == 1 && c2F(di) == 1   
    strF = 1 ;
	Ld = 2 ;
end
    

% 4.2.- Direction 2:

di = 2 ;

if c1F(di)==1 && c2F(di) == 1   
    
    
    if wav(2)<wav(1)
    
        strF = 1 ;
        Ld = 1 ;
    
    end
end

% 4.3.- Checks condition 3:
% only happens if strF = 0 and one of the widths is 5 times larger than the
% other

if strF==0
    
    if c3F(1)==1 
        if wav(1)*5 < wav(2)
          	strF = 1;
            Ld = 2;
        end
    end
    
    
	if c3F(2)==1 
        if wav(2)*5 < wav(1)
          	strF = 1;
            Ld = 1;
        end
    end
    
end



end

