function [u] = dp3350_DPincrement ...
        (R,tita, ua, dSmesh, Dspline, Degree, Ncp, KnotV, Usize, EP,Ew, Dumax, ULims, xdim, nd, DerC)
  
% RETURNS:
% u(2): Estimated Ucoor for achieving distance R in physical space at tita angle in parameter space
% rF: rejected flag: 0 / 1  for  non-rejected / rejected, at the moment always is non-refected (rF = 0)
% tF: track back flag = 1 if the estimation lied outsude limits and has been pulled back (otherwise = 0)

% refines mesh if error is larger than tolerance

% INPUTS:
% R: physical space distance to achieve from position in ua
% tita: angle (rads) w.r.t. horizontal in parameter space used to measure R
% ua(2): param. space starting position 
% Dumax: maximum increment of u (assumed half patch diagonal)
% nd: No. digits for rounding
% DerC: derivative computation mode 'ex' exact - 'ip' interpolation


udim = 2 ;

% for tracking buggs:
pua = [ 0.9497 ; 0.9641 ];
ptita = 1.5536 ;


% 1.- PRELIMINARY OPERATIONS:

u1 = round(ua,nd);

% limits tita to be between 0 - 360 (but in radians)

if tita > 0

    Nvueltas = floor(tita / 6.283185);
    tita = tita - Nvueltas* 6.283185 ;

elseif tita < 0
    
	Nvueltas = floor(tita / 6.283185) + 1;
    tita = tita - Nvueltas* 6.283185 ;
end

if tita > 6.283185
    tita = tita - 6.283185;
end
if tita < 0
    tita = 6.283185 + tita;
end



nDumax = norm(Dumax);

% extracts from dSmesh structure:

dSu = dSmesh.mdSu ;

dSv = dSmesh.mdSv ;

NE = dSmesh.mNE ;

NEt = dSmesh.mNEt ;

fpeR = dSmesh.mfpeR ;

fceR = dSmesh.mfceR ;

conM = dSmesh.mconM ;

conMp = dSmesh.mconMp ;

conMc = dSmesh.mconMc ;

grEp = dSmesh.mgrEp ;

vEgr = dSmesh.mvEgr ;

vDirp = dSmesh.mvDirp ;

ULimEr = dSmesh.mULimEr ;
ULimEpr = dSmesh.mULimEpr ;
nUcoorr = dSmesh.mnUcoorr ;


% 2.- FINDS FIRST ELEMENTS WHERE ua LIES IN (can be multiple):
% set them as 'current elements', & computes mdC at uar

[NceR, ceR] = dp3351_findFirstE ...
(u1,tita, ULims, NE, NEt, ULimEr, ULimEpr, fpeR, fceR, conM, conMp, conMc, grEp, vEgr, vDirp, nUcoorr, nd);

ndS1 = ds3323_ndSutita (u1, tita, Dspline, NceR, ceR, fpeR, fceR, Degree, Ncp, KnotV, Usize, EP,Ew, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, udim, xdim, DerC);

%{
if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) &&  (round(tita,4)==ptita)
    
    pua
    tita
    NceR
    ceR
    ndS1
    % ULimEr
    
end
%}





% 3.- FINDS NEXT INTERSECTION POINT WITHIN feR:

% 3.1.- Checks if extension of segment goes to infinite direction:

Ttol = 2 / 57.296 ; % tolerance for perim. & corner elements to detemine Rmax=infinite direction   

% iiF: 'infinite intersection flag'
iiF = dp3353_RmaxInf (u1, tita, ULims, nd, Ttol); 



% 3.2.- Sets u2 in case goes to infinite:

if iiF == 1
    
    % sets u2 and its deriv. module at Dmax distance from u1:
    u2 = [ u1(1) + nDumax*cos(tita) ; u1(2) + nDumax*sin(tita) ];
    
    ndS2 = ndS1 ; % assumes same derivative as intersection with ULims
     
    Lacc = 0;
    
end


% 3.3.- If is not infinite => 
% => computes u2: second intersection point in ceR & measures Rmax:

oeF = 0;

if iiF==0

    
    % finds intersection:
    
    u2 = dp3354_rayEleInts (NceR, ceR, u1, tita, conM,conMp,conMc, nUcoorr, fpeR,fceR, grEp, vDirp, nd, Ttol);
    
	% finds next elements:
	[NneR, neR] = dp3355_FindRele (ULims, NE, NEt, grEp, ULimEr, ULimEpr, fpeR, u2);
    
    % computes ndS2:
	ndS2 = ds3323_ndSutita (u2, tita, Dspline, NneR, neR, fpeR, fceR, Degree, Ncp, KnotV, Usize, EP,Ew, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, udim, xdim, DerC);

    % maximum physical distance achievable:
    Lacc = dp3358_OEPlength (u1, u2, ndS1, ndS2);
  
    % If one element is sufficient activaes one element flag:
    if R <= Lacc
        oeF = 1 ;
    end
    
end


%{
if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) &&  (round(tita,4)==ptita)
    
    u2
    NneR
    neR
    ndS2
    Lacc
    
end
%}

% 4.- LOOPS UNTIL Lacc OVERCOMES R:

Repath = 0 ;

while (Lacc < R && iiF==0)

    % updates initial points and elements:
    u1 = u2 ;
    ndS1 = ndS2;
    NceR = NneR;
    ceR  = neR;
    
    % checks infinite direction:
    iiF = dp3353_RmaxInf (u1, tita, ULims, nd, Ttol);
    
    %{
    if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) &&  (round(tita,4)==ptita)

        u1
        ndS1
        NceR
        ceR
        iiF

    end
    %}
  
    
    if iiF==0
    
    % - case finite segment
        
        % finds intersection:
        u2 = dp3354_rayEleInts (NceR, ceR, u1, tita, conM,conMp,conMc, nUcoorr, fpeR,fceR, grEp, vDirp, nd, Ttol);

        % finds next elements:
        [NneR, neR] = dp3355_FindRele (ULims, NE, NEt, grEp, ULimEr, ULimEpr, fpeR, u2);

        % computes ndS2:
      	ndS2 = ds3323_ndSutita (u2, tita, Dspline, NneR, neR, fpeR, fceR, Degree, Ncp, KnotV, Usize, EP,Ew, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, udim, xdim, DerC);

        % maximum physical distance achievable for that element:
        Repath = dp3358_OEPlength (u1, u2, ndS1, ndS2);

        % accumulated maximum physical length:
        Lacc = Lacc + Repath;


    elseif iiF==1
        
  	% - case infinite segment
        
        % sets u2 and its deriv. module at Dmax distance from u1:
        u2 = [ u1(1)+nDumax*cos(tita) ; u1(2)+nDumax*sin(tita) ];
        
        ndS2 = ndS1 ; % assumes same derivative as intersection with ULims
        
    end
   
end



% 5.- WORKS OUT Du TO ACHIEVE R WITHIN THE LAST SEGMENT:

if oeF == 1
    Lacc_prev = 0;
elseif iiF==0
    Lacc_prev = Lacc - Repath; 
elseif iiF==1
	Lacc_prev = Lacc;
end

Reff = R - Lacc_prev ;



[u] = dp3359_LDPincrement (u1, u2, tita, ndS1, ndS2, Reff, Dumax);
u = round(u,nd);



end

