function [NiE, iE] = dp3352_upEdgInts (NiE, iE, vdi, u1,u2, nua, nub, nd)

% RETURNS:
% NiE(iE): updated intersections with element edges, comsidering one side

% vdi: direction that varies in element side
% u1,u2: segment end points coordinates
% nua,nub: element edge end nodes coordinates

% assumed:
% - all coordinates rounded to nd:
% - nua is prev. to nub

pdirm = [2 1];
pdi = pdirm(vdi); % perp. direction to vdi

% 1.- CHECK PARALELISM:

% if paralell adds those nodes of element inside the segment u1-u2

if (u1(pdi) == u2(pdi))

    if (u1(pdi)==nua(pdi)) % in addition is in the same line
    
      	uSmin = min(u1(vdi),u2(vdi))-10^(-nd);
      	uSmax = max(u1(vdi),u2(vdi))+10^(-nd);  
        
        if uSmin <= nua(vdi) && nua(vdi) <= uSmax

            NiE = NiE + 1 ;
            iE(NiE,1) = nua(1); iE(NiE,2) = nua(2);

        end

        if uSmin <= nub(vdi) && nub(vdi) <= uSmax

            NiE = NiE + 1 ;
            iE(NiE,1) = nub(1); iE(NiE,2) = nub(2);

        end
    
    end
    
    return
end


% 2.- INTERSECTION WITH VERTICAL EDGE:

if vdi==2
    
    uSmin = min(u1(1),u2(1))-10^(-nd);
	uSmax = max(u1(1),u2(1))+10^(-nd);
    
    if u1(2) == u2(2)
    % 2.1.- Case segment is horizontal:
    
        if  (nua(2) <= u1(2) && u1(2) <= nub(2)) && ...
            (uSmin <= nua(1) && nua(1) <= uSmax)  

            NiE = NiE + 1 ;
            iE(NiE,1) = nua(1); iE(NiE,2) = u1(2);

        end
    
    else
    % 2.2.- Other cases:
    
        % horizontal edge coordinate:
        x = nua(pdi);

        % finds y for that x in the segment line u1-u2:
        m = (u2(2)-u1(2)) / (u2(1)-u1(1));
        n = u1(2) - m * u1(1);
        y = round ( m*x + n , nd) ;
        
        % if y belongs to nua-nub  & x belongs to u1-u2 => is intersection point
        if (nua(vdi) <= y && y <= nub(vdi)) ...
        && (uSmin <= x && x <= uSmax)
            NiE = NiE + 1 ;
            iE(NiE,1) = x; iE(NiE,2) = y;
        end
    
    end

    
    return
end

% 3.- INTERSECTION WITH HORIZONTAL EDGE:

if vdi==1
    
	uSmin = min(u1(2),u2(2))-10^(-nd);
	uSmax = max(u1(2),u2(2))+10^(-nd); 
    
    if u1(1) == u2(1)
    % 3.1.- Case segment is vertical:     
        
        if  (uSmin <= nua(2) && nua(2)<= uSmax) && ...
            (nua(1) <= u1(1) && u1(1) <= nub(1))  

            NiE = NiE + 1 ;
            iE(NiE,1) = u1(1); iE(NiE,2) = nua(2);
        end
        
    else
 	% 3.2.- Other cases:  

        % vert. edge coordinate:
        y = nua(pdi);

        % finds x for that y in the segment line u1-u2:
        m = (u2(2)-u1(2)) / (u2(1)-u1(1));
        n = u1(2) - m * u1(1);
        x = round ( (y-n)/m , nd) ;
        
        % if x belongs to nua-nub & y belongs to u1-u2 =>  is intersection point
        if (nua(vdi) <= x && x <= nub(vdi)) ...
        && (uSmin <= y && y <= uSmax)
            NiE = NiE + 1 ;
            iE(NiE,1) = x; iE(NiE,2) = y;
        end
        
    end
    

    return 
end



end

