function [riF] = dp3353_RmaxInf (u, tita, ULims, nd, Ttol)

% RETURNS:
% riF: 'Rmax is infinite' flag, if so turns to 1

% INPUTS:
% semi-infinite segment given by u and tita
% ULims: patch param. space limits
% Ttol: tita tolerance, to be considered both sides, w.r.t. 'infinite Rmax' angle (in radians)

% special case 1: u lies at corner elements and angle within infinite range direction
   
sc = 1/57.29578;

riF = 0 ; % 'Rmax is infinite' flag
    
ur = round(u,nd);
    
if ur(1)<=ULims(1,1) && ur(2)<=ULims(2,1) % corner 1

    if 180*sc - Ttol <= tita && tita <= 270*sc + Ttol
        riF = 1 ;
    end

elseif ur(1)<=ULims(1,1) && ULims(2,2)<=ur(2) % corner 2 

    if 90*sc - Ttol <= tita && tita <= 180*sc + Ttol
        riF = 1 ;
    end

elseif ULims(1,2)<=ur(1) && ur(2)<=ULims(2,1) % corner 3

    if (270*sc - Ttol <= tita && tita <= 360*sc + Ttol) || ...
       (  0*sc - Ttol <= tita && tita <=   0*sc + Ttol)   
        riF = 1 ;
    end

elseif ULims(1,2)<=ur(1) && ULims(2,2)<=ur(2) % corner 4

    if (  0*sc - Ttol <= tita && tita <=  90*sc + Ttol) || ...
       (360*sc - Ttol <= tita && tita <= 360*sc + Ttol)   
        riF = 1 ;
    end
end
    

    
% special case 2: u lies at perimeter elements and tita to infinite direction
    
if ur(1)<=ULims(1,1) && (ULims(2,1)<=ur(2) && ur(2)<=ULims(2,2)) % Left

    if 180*sc - Ttol <= tita && tita <= 180*sc + Ttol
        riF = 1 ;
    end
        
elseif ULims(1,2)<=ur(1) && (ULims(2,1)<=ur(2) && ur(2)<=ULims(2,2))  % Right

    if (360*sc - Ttol <= tita  &&  tita <= 360*sc + Ttol) || ...
       (  0*sc - Ttol <= tita  &&  tita <=   0*sc + Ttol)  
        riF = 1 ;
    end

elseif ur(2)<=ULims(2,1) && (ULims(1,1)<=ur(1) && ur(1)<=ULims(1,2))  % Bottom
        
    if 270*sc - Ttol <= tita && tita <= 270*sc + Ttol
        riF = 1 ;
    end
   
elseif ULims(2,2)<=ur(2) && (ULims(1,1)<=ur(1) && ur(1)<=ULims(1,2)) % Top

    if 90*sc - Ttol <= tita && tita <= 90*sc + Ttol
        riF = 1 ;
    end
end

end

