function [iE, NiE] = dp3360_intersecLadosElem (u1, u2, ueR, conM,conMp,conMc, nUcoorr, fpeR, fceR, grEp, vDirp, nd)

% RETURNS:
% iE(NiE,2): intersection points of segment u1-u2 with edges of element
% note that NiE can vary from 1 to 2 
% repeated intersection points (corners) are removed

% INPUTS:
% u1-u2: end points of the intersection
% ueR: element reference whose edges are to be compared against the segment

% Note: ueR is single element (not an array of elements)

udim = 2 ;
iE  = zeros(1,2);
NiE = 0;

u1 = round(u1,nd);
u2 = round(u2,nd);

% 1.- ueR IS INTERNAL ELEMENT:

if ueR < fpeR

    lNod = [ 1 2 ; 3 4 ; 1 3 ; 2 4 ];
    lVdi = [ 2 2 1 1];
    
    for lado = 1:1:4

        r1 = lNod(lado,1); % order of corners
        r2 = lNod(lado,2);
        na = conM(ueR,r1); % nodes references
        nb = conM(ueR,r2);
        nua = [nUcoorr(na,1) nUcoorr(na,2)]; % nodes coordinates
        nub = [nUcoorr(nb,1) nUcoorr(nb,2)];

        [NiE, iE] = dp3352_upEdgInts (NiE, iE, lVdi(lado), u1,u2, nua, nub, nd);
        
    end

    
    % Removes repeated intersections:
    [iE,~] = GEN_16_RemoveRepeatedRows (iE, udim, 10^(-nd));
    NiE = size(iE,1);
    return
  
end


% 2.- ueR IS PERIMETER ELEMENT:

if fpeR <= ueR && ueR < fceR
    
    ueReff = ueR - (fpeR-1);
    
    % detects which group ueR belongs to: stores in 'gr':
    
    for g = 1:1:4
        if grEp(g,1)<=ueReff  && ueReff<=grEp(g,2)
            gr = g;
            break
        end
    end
    
    
    % variable and semi-infinite edge directions:
    
    pdir = [2 1];
    
    vdi = vDirp(ueReff,1);
    sied = pdir(vdi);
    
    
    % Builds up 3 edges:
 
    na = conMp(ueReff,1);
    nb = conMp(ueReff,2);
    nua = [nUcoorr(na,1) nUcoorr(na,2)]; % nodes coordinates
 	nub = [nUcoorr(nb,1) nUcoorr(nb,2)];

    
	% - edge touching param. space limits
    
    [NiE, iE] = dp3352_upEdgInts (NiE, iE, vdi, u1,u2, nua, nub, nd);
    
    
    
    % - semi-infinite edges:
    
  	umin = 10*[min(u1(1),u2(1))   min(u1(2),u2(2))];
    uMax = 10*[max(u1(1),u2(1))   max(u1(2),u2(2))];
    
    nua2 = zeros(1,2);
    nub2 = zeros(1,2);
    
    nua2(vdi) = nua(vdi);
    nub2(vdi) = nub(vdi);
    
    % furthest edge coordinate 'extreme coordinate':
    
    if gr==1
        
        nua1 = [umin(1) nua(2)];
        nua2 = nua ;
     	
        nub1 = [umin(1) nub(2)];
        nub2 = nub ;
        
    elseif gr == 2
        
        nua1 = nua ;
      	nua2 = [uMax(1) nua(2)];
        
     	nub1 = nub ;
        nub2 = [uMax(1) nub(2)];
        
    elseif gr ==3
        
        nua1 = [nua(1) umin(2)];
        nua2 = nua ;
        
     	nub1 = [nub(1) umin(2)];
        nub2 = nub ;
        
    elseif gr==4
        
        nua1 = nua ;
    	nua2 = [nua(1) uMax(2)];
        
        nub1 = nub ;
     	nub2 = [nub(1) uMax(2)];

    end
    

    [NiE, iE] = dp3352_upEdgInts (NiE, iE, sied, u1,u2, nua1, nua2, nd);
    [NiE, iE] = dp3352_upEdgInts (NiE, iE, sied, u1,u2, nub1, nub2, nd);
    

    return
    
end


% 3.- ueR IS CORNER ELEMENT:

if fceR <= ueR 
    
    ueReff = ueR - (fceR-1);
    
    % builds up 2 semi-infinite edges:
    
    na  = conMc(ueReff,1);
    ncor = [nUcoorr(na,1) nUcoorr(na,2)]; % nodes coordinates
    
    umin = 100*[min(u1(1),u2(1))   min(u1(2),u2(2))];
    uMax = 100*[max(u1(1),u2(1))   max(u1(2),u2(2))];
    
    if ueReff == 1
        
        uah = [umin(1) , ncor(2)] ;
        ubh = ncor ;
        
        uav = [ncor(1), umin(2)] ;
        ubv = ncor ;

    elseif ueReff == 2
        
        uah = [umin(1) , ncor(2)] ;
        ubh = ncor ;
        
        uav = ncor ;
        ubv = [ncor(1), uMax(2)] ;
        
  	elseif ueReff == 3
        
        uah = ncor ;
        ubh = [uMax(1) , ncor(2)] ;
        
        uav = [ncor(1), umin(2)] ;
        ubv = ncor ;
        
 	elseif ueReff == 4
        
        uah = ncor ;
        ubh = [uMax(1) , ncor(2)] ;
        
        uav = ncor ;
        ubv = [ncor(1), uMax(2)] ;
        
    end
    
 	[NiE, iE] = dp3352_upEdgInts (NiE, iE,1, u1,u2, uah, ubh, nd);
    [NiE, iE] = dp3352_upEdgInts (NiE, iE,2, u1,u2, uav, ubv, nd);
    
end




end

