function [Ierr] = ds3380_Ierr (eUlim, eDU, xs, ...
                sDegree, sNcp, sKnotV, sUsize, sP ,sw, ah, xdim, udim, nd, np, dro)

% RETURNS:
% Ierr(6): relative error of element (selects maximum) for each function
% f11, f12, f13, f21, f22, f23,  being f11=g,u_x, f12=g,u_y , ... f23=g,v_z

% INPUTS:
% n: No points to measure derivative per side

Ierr = zeros(udim,xdim);

% 1.- Maximum abs. value of products of Dx:
            
Dxs = zeros(udim,1) ;

for di = 1:1:udim
    step = xs(di,2) - xs(di,1);
    Dxs(di) = 0.72 * step;
end

    
% 2.- Computes errors using 3rd derivatives of f at nxn points of
% element; selects the max. value of error:

ust = eDU/(np-1);

u = eUlim(1,1);
for i = 1:1:np
    
    v = eUlim(2,1);
    for j = 1:1:np

        % param. coord's
        % ensures uc inside the element in case uc is at element limits:

        uc = [u v]; 

        ftr = 0.001 ; % factor to multiply to elem span to add/deduct to uc 

        for di = 1:1:udim

            if round(uc(di),nd) == round(eUlim(di,1),nd)
                uc(di) = uc(di) + ftr*eDU(di);
            elseif round(uc(di),nd) == round(eUlim(di,2),nd)
                uc(di) = uc(di) - ftr*eDU(di);
            end

        end

        % derivatives of f for each param, direction and physical component:
        [dfo] = ds3377_df_dro (uc, dro, sDegree, sNcp, sKnotV, sUsize, sP ,sw, xdim, udim);
  

        % temporal error calculation and compare to max. stored (for both all components)
        n = dro;
        for a = 1:1:udim
            for b = 1:1:xdim
                
                % computes error at uc coordinate
                acc = 0 ;
                for m = 0:1:n 
                    bn = factorial(n) / (factorial(m) - factorial(n-m)) ;
                    df = dfo(a,b,m+1,n-m+1);
                    acc = acc + bn * df * Dxs(1)^m * Dxs(2)^(n-m);
                end
                
                % relative error:
                if ah(a,b)<0.01
                    tIerr = 0;
                else
                    tIerr = 100*acc/ah(a,b);
                end
                
                
                
                % stores error if overcomes maximum stored so far:
                if i==1 && j==1
                    
                    % first corner
                    Ierr(a,b) = tIerr;
                else
                    
                    % other corners: stores if overcomes maximum stored so far
                    if Ierr(a,b) < tIerr
                        Ierr(a,b) = tIerr;
                    end
                end
                
            end
        end
        

        
        v = v + ust(2);
        if j == np-1
            v = eUlim(2,2);
        end
        
    end
    
  	u = u + ust(1);
    if i == np-1
    	u = eUlim(1,2);
    end
    
end

end

