function [df0,df1,df2,df3] = ds3377_df3 (dSa, xdim)

% RETURNS: for f=g,u and f=g,v

% df0(2,1):  g,u  , g,v

% in each row (for each function):
% df1(2,2):  fu, fv
% df2(2,4):  fuu, fvv, fuv, fvu
% df3(2,4):  fuuu, fvvv, fuuv, fvvu



% INPUTS:
% dSa: surface main derivatives up to 4 for this node

df0 = zeros(2,1);
df1 = zeros(2,2);
df2 = zeros(2,4);
df3 = zeros(2,4);

% 1.- EXTRACTS COMPONENTS OF DERIVATIVES OF g:

g = zeros(xdim,1);

gu = zeros(xdim,1);
guu = zeros(xdim,1);
guuu = zeros(xdim,1);
guuuu = zeros(xdim,1);

gv = zeros(xdim,1);
gvv = zeros(xdim,1);
gvvv = zeros(xdim,1);
gvvvv = zeros(xdim,1);

guv = zeros(xdim,1);
guuv = zeros(xdim,1);
guuuv = zeros(xdim,1);

guvv = zeros(xdim,1);
guuvv = zeros(xdim,1);

guvvv = zeros(xdim,1);


for di = 1:1:xdim
    
	g(di,1) = dSa(1,1,1,di);
    
    gu(di,1) = dSa(2,1,1,di);
    guu(di,1) = dSa(3,1,1,di);
    guuu(di,1) = dSa(4,1,1,di);
    guuuu(di,1) = dSa(5,1,1,di);
    
  	gv(di,1) = dSa(1,2,1,di);
    gvv(di,1) = dSa(1,3,1,di);
    gvvv(di,1) = dSa(1,4,1,di);
    gvvvv(di,1) = dSa(1,5,1,di);
    
 	guv(di,1) = dSa(2,2,1,di);
    guuv(di,1) = dSa(3,2,1,di);
    guuuv(di,1) = dSa(4,2,1,di);
    
	guvv(di,1) = dSa(2,3,1,di);
    guuvv(di,1) = dSa(3,3,1,di);
    
    guvvv(di,1) = dSa(2,4,1,di);
end


gvu = guv;
gvuu = guuv;
gvuuu = guuuv;

gvvu = guvv;
gvvuu = guuvv;

gvvvu = guvvv;


% 2.- OPERATES ON FUNCTION f = g,u:

% 2.1.- Coefficients A: 

Au = 0 ;
Auu = 0 ;
Auuu = 0 ;

Av = 0 ;
Avv = 0 ;
Avvv = 0 ;

Auv = 0 ;
Auuv = 0 ;
Avu = 0 ;
Avvu = 0 ;


for di = 1:1:xdim
    
    Au   = Au   + gu(di)*guu(di);
    Auu  = Auu  + guu(di)*guu(di) + gu(di)*guuu(di);
    Auuu = Auuu + 3*guu(di)*guuu(di) + gu(di)*guuuu(di);
    
  	Av   = Av   + gu(di)*guv(di);
    Avv  = Avv  + guv(di)*guv(di) + gu(di)*guvv(di);
    Avvv = Avvv + 3*guv(di)*guvv(di) + gu(di)*guvvv(di);
    
    Auv  = Auv  + guv(di)*guu(di) + gu(di)*guuv(di);
    Auuv = Auuv + 2*guu(di)*guuv(di) + guv(di)*guuu(di) + gu(di)*guuuv(di);
    
    Avu  = Avu  + guu(di)*guv(di) + gu(di)*guuv(di);
    Avvu = Avvu + 2*guv(di)*guuv(di) + guu(di)*guvv(di) + gu(di)*guuvv(di);
    
end

% 2.2.- Derivatives:

f = norm(gu);

fu = Au/f;

fuu = (Auu*f - Au*fu)/f^2;

fuuu = (Auuu*f^2 - Au*fuu*f - 2*Auu*fu*f + 2*Au*fu*fu) / f^3;

fv = Av/f;

fvv = (Avv*f - Av*fv)/f^2 ;

fvvv = (Avvv*f^2 - Av*fvv*f - 2*Avv*fv*f + 2*Av*fv*fv) / f^3;


fuv = (Auv*f - Au*fv) / f^2 ;
fuuv = (Auuv*f^2 - Auu*fv*f - Auv*fu*f - Au*fuv*f + 2*Au*fu*fv) / f^3;

fvu = (Avu*f - Av*fu) / f^2 ;
fvvu = (Avvu*f^2 - Avv*fu*f - Avu*fv*f - Av*fvu*f + 2*Av*fu*fv) / f^3;



% 2.3.- Allocates to response:

df0(1,1) = f;

df1(1,1) = fu;
df1(1,2) = fv;

df2(1,1) = fuu;
df2(1,2) = fvv;
df2(1,3) = fuv;
df2(1,4) = fvu;

df3(1,1) = fuuu;
df3(1,2) = fvvv;
df3(1,3) = fuuv;
df3(1,4) = fvvu;


% 3.- OPERATES ON FUNCTION f = g,v:


% 3.1.- Coefficients B: 

Bu = 0 ;
Buu = 0 ;
Buuu = 0 ;

Bv = 0 ;
Bvv = 0 ;
Bvvv = 0 ;

Buv = 0 ;
Buuv = 0 ;
Bvu = 0 ;
Bvvu = 0 ;


for di = 1:1:xdim
    
    Bu   = Bu   + gv(di)*gvu(di);
    Buu  = Buu  + gvu(di)*gvu(di) + gv(di)*gvuu(di);
    Buuu = Buuu + 3*gvu(di)*gvuu(di) + gv(di)*gvuuu(di);
    
  	Bv   = Bv   + gv(di)*gvv(di);
    Bvv  = Bvv  + gvv(di)*gvv(di) + gv(di)*gvvv(di);
    Bvvv = Bvvv + 3*gvv(di)*gvvv(di) + gv(di)*gvvvv(di);
    
    Buv  = Buv  + gvu(di)*gvv(di) + gv(di)*gvvu(di);
    Buuv = Buuv + 2*gvu(di)*gvvu(di) + gvv(di)*gvuu(di) + gv(di)*gvvuu(di);
    
    Bvu  = Bvu  + gvv(di)*gvu(di) + gv(di)*gvvu(di);
    Bvvu = Bvvu + 2*gvv(di)*gvvu(di) + gvu(di)*gvvv(di) + gv(di)*gvvvu(di);
    
end


% 3.2.- Derivatives:

f = norm(gv);

fu = Bu/f;

fuu = (Buu*f - Bu*fu)/f^2;

fuuu = (Buuu*f^2 - Bu*fuu*f - 2*Buu*fu*f + 2*Bu*fu*fu) / f^3;

fv = Bv/f;

fvv = (Bvv*f - Bv*fv)/f^2 ;

fvvv = (Bvvv*f^2 - Bv*fvv*f - 2*Bvv*fv*f + 2*Bv*fv*fv) / f^3;


fuv = (Buv*f - Bu*fv) / f^2 ;
fuuv = (Buuv*f^2 - Buu*fv*f - Buv*fu*f - Bu*fuv*f + 2*Bu*fu*fv) / f^3;

fvu = (Bvu*f - Bv*fu) / f^2 ;
fvvu = (Bvvu*f^2 - Bvv*fu*f - Bvu*fv*f - Bv*fvu*f + 2*Bv*fu*fv) / f^3;


% 3.3.- Allocates to response:

df0(2,1) = f;

df1(2,1) = fu;
df1(2,2) = fv;

df2(2,1) = fuu;
df2(2,2) = fvv;
df2(2,3) = fuv;
df2(2,4) = fvu;

df3(2,1) = fuuu;
df3(2,2) = fvvv;
df3(2,3) = fuuv;
df3(2,4) = fvvu;



end

