function [dSmesh] = rs3381_dSmesh ...
        (sDegree,sNcp, sKnotV, sUsize, sP,sw, Dspline, sXdim, sUdim, ULims, Dumax, tolRs, nd, esw, cUv, Nlv, Nlcv, DerC, trimmed)

% RETURNS:
% mesh structure to discretize dS (to compute lengths in physical space)
% NE:  No. elements (excluding perimeter / corner elems)
% NEt: total No. elements (including p/c elements)
% fpeR, fceR: first perimeter / corner elements ref. Perim and corner elems are placed after internals
% NN:  No. nodes
% NNt: No. nodes in total
% eULims(2,2): extended ULims, considering perimeter elements

% INPUTS:
% surface NURBS features
% tolRs:contur strip width in physical space

% cUv,Nlv, Nlcv: loops param. coordinates
% note that uses only the 1st contur loop with is the external one

udim = 2 ;
xdim = 3 ;

[sEw,sEP] = GEN_12_ExpandBw (sw, sP, sXdim, sUdim, sNcp);


% 1.- SETS FIRST UNIFORM MESH:

% 1.1.- initial trial param. space steps:

[nvU, nvUsize] = GEN_18_UnonVoid (sKnotV, sUsize, sDegree, udim);

nvU = round(nvU,nd);



% 1.2.- Generates first trial regular mesh acc/to ncp in each direction:

dSmesh = struct ...
('mdSu',[], 'mdSv',[], 'mNE',0, 'mNEt',0, 'mfpeR',0,'mfceR',0, 'mNN',0, ...
'mconM',[],'mconMp',[], 'mconMc',[], 'mnUcoorr',[], 'mULimEr',[],'mULimEpr',[],...
'mnUcoor',[], 'mULimE',[],'mULimEp',[], 'mgrEp',[], 'mvEgr',[], 'mvDirp',[] , ...
'mNesc',[], 'mUesc',[], 'mUescM',[], 'mXesc',[]);

[dSu, dSv, dSn, NE, NN, conM, nUcoor, ULimE, edeR, NedeR, ETcon] = rs3382_FirstMesh ...
    (sDegree, sNcp, sKnotV, sUsize, sP,sw, nvU, nvUsize, sXdim, sUdim);


% 2.- EVALUATES ERROR IN SIDES & DIAGONALS INITIALLY:

% max. of the errors (percentage) for:
errSD = zeros(NE,1); % 4 sides and 2 diag's per elem (4 sides: 2 vert. + 2 horzt.)

for e = 1:1:NE
    [errEsd] = rs3383_ErrorEdsDiag (sDegree, sNcp, sKnotV, sUsize, sP,sw, dSn, conM, nUcoor, e);
    errSD(e) = max(errEsd);
end



% 3.- LOOPS UNTIL ERRORS ARE ACCEPTABLE < tolR:

% tic %%%%%%%

NL = 0 ;
NLim = 7 ;
outF = 0 ;

while (NL < NLim  && outF == 0)
   
    
    % 3.1.- references of elements to divide and outFlag:
    
    outF = 1 ;
    NEer = 0 ;
    
    for e = 1:1:NE
        
        if (errSD(e)> tolRs)
            NEer = NEer + 1 ;
        	eD(NEer) = e ; 
            outF = 0;
        end 
        
    end
    
    
    
    % 3.2.- split elements and recomputes error:
    
    if outF == 0
    for e = 1:1:NEer
            
        eR = eD(e);

        [dSu,dSv, dSn, NE,NN, conM, nUcoor, ULimE, edeR, NedeR, ETcon] = rs3384_DivideRect...
            (sDegree, sNcp, sKnotV, sUsize, sP,sw, eR, dSu,dSv, dSn, NE, NN, conM, nUcoor, ULimE, edeR, NedeR, ETcon, sXdim, sUdim);
           
        
        
        % error of element eR:
     	[errEsd] = rs3383_ErrorEdsDiag (sDegree, sNcp, sKnotV, sUsize, sP,sw,dSn, conM, nUcoor, eR);
        errSD(eR) = max(errEsd);

        
        % error of new 3 elemetns:
        for i = 1:1:3
            
            eRi = NE-3+i;
            [errEsd] = rs3383_ErrorEdsDiag (sDegree, sNcp, sKnotV, sUsize, sP,sw,dSn,conM, nUcoor, eRi);
            errSD(eRi) = max(errEsd);
  
        end
    end
    end

    NL = NL+1;
     
end

% toc %%%%%%%%%

% 4.- ADD ELEMENTS AT THE PERIMETER: uses edeR and NedeR matrices:

NedeRtotal = sum(NedeR);

% 4.1.- Grouped elements: L, R, B, T

grEp = [ ...
        1                               NedeR(1) ;
        NedeR(1)+1                      NedeR(1)+NedeR(2);  
        NedeR(1)+NedeR(2)+1             NedeR(1)+NedeR(2)+NedeR(3);
        NedeR(1)+NedeR(2)+NedeR(3)+1    NedeRtotal ];

    
vEgr = [
        1*ones(grEp(1,2)-grEp(1,1)+1,1) ;
        2*ones(grEp(2,2)-grEp(2,1)+1,1) ;
        3*ones(grEp(3,2)-grEp(3,1)+1,1) ;
        4*ones(grEp(4,2)-grEp(4,1)+1,1) ];
    

% 4.2.- Sets fpeR, vDirp and prepares vlbles for the loop:

fpeR = NE + 1 ;

vDirp = zeros(NedeRtotal,1);
vDirp(1 : NedeR(1)+NedeR(2)) = 2 ;
vDirp(NedeR(1)+NedeR(2)+1 : NedeRtotal) = 1 ;

conMp = zeros(NedeRtotal,2);
ULimEp = zeros(NedeRtotal,2);

ieRnode = [1 2 ; 3 4 ; 1 3 ; 2 4 ]; % 1st & 2nd nodes ref's used from adjacent elems

contaEp = 0 ;

for ed = 1:1:4 % edges 1,2,3,4 correspond to L, R, B, T strips

    for e = 1:1:NedeR(ed)
        
     	contaEp = contaEp + 1 ;
        
        ieR = edeR(ed,e); % internal element reference

        n1 = conM(ieR,ieRnode(ed,1)) ; % nodes references for new perim. element
        n2 = conM(ieR,ieRnode(ed,2)) ;
        
        % connectivity matrix:
        conMp(contaEp,1) = n1 ;
     	conMp(contaEp,2) = n2 ;
  
        % ULimEp: perim. elem's limits (no special order, just piled as it goes)
        vd = vDirp(contaEp);
        ULimEp(contaEp,1) = nUcoor (n1, vd);  
        ULimEp(contaEp,2) = nUcoor (n2, vd); 
        
    end
end


% 5.- ADD ELEMENTS AT CORNERS:

% connection matrix:


conMc = [ 1 ;...
        nvUsize(2); ...
        nvUsize(1)*nvUsize(2)-nvUsize(2)+1; ...
        nvUsize(1)*nvUsize(2)]; 

% First corner element ref:
fceR = NE+contaEp + 1;
    
% Updates No. elems:
NEt = NE+contaEp + 4;


% 6.- ALLOCATES TO ANSWER structure EXCEPT CONTOURS INFO:

dSmesh.mdSu = dSu ;

dSmesh.mdSv = dSv ;

dSmesh.mNE = NE ;

dSmesh.mNEt = NEt ;

dSmesh.mfpeR = fpeR ;

dSmesh.mfceR = fceR ;

dSmesh.mNN = NN ;

dSmesh.mconM = conM ;

dSmesh.mconMp = conMp ;

dSmesh.mconMc = conMc ;

dSmesh.mnUcoor = nUcoor ;

dSmesh.mULimE = ULimE ;

dSmesh.mULimEp = ULimEp ;

dSmesh.mgrEp = grEp ;

dSmesh.mvEgr = vEgr ;

dSmesh.mvDirp = vDirp ;

dSmesh.mULimEr  = round(ULimE,  nd);
dSmesh.mULimEpr = round(ULimEp, nd);
dSmesh.mnUcoorr = round(nUcoor, nd);



% 7.- EDGE STRIPS: REMOVED

%{
if strcmp(trimmed,'no')==1

% 7.1.- Preliminaries:

    Nesc = zeros(4,1);
    Uesc = zeros(4,1,udim);
    UescM = zeros(4,1);
    Xesc= zeros(4,1,xdim);

    ua = zeros(udim,1);
    edSu = zeros(xdim,1);
    edSv = zeros(xdim,1);

    % accumulated edge nodes at ends of each curve
    accEN = zeros(5,1); % first value is 0

    for ed = 1:1:4
        accEN(ed+1) = accEN(ed)+Nlcv(1,ed);
    end

    Utemp = zeros(1,udim);


    % 7.2.- Loop on edge-contour vertexes to compute their correspondant 
    % parameter position at esw offset from perimeter in physical:

    % remember: curves start at (0,0) and moves counter-clock wise

    for po = 1:1:Nlv(1)

        ua(:,1) = cUv(1,po,:);


        % a.- Identifies edge : edr = 'edge referece'

        if po <= accEN(2)
            edr = 1 ;
        elseif accEN(2)<po && po<=accEN(3)
            edr = 2 ;
        elseif accEN(3)<po && po<=accEN(4)
            edr = 3 ;
        elseif accEN(4)<po && po<=accEN(5)
            edr = 4 ;
        end


        % b.- derivatives at edge:

        [dS] = b2701_BNURBS_deriv (sDegree, sNcp, sKnotV, sUsize, sEP ,sEw, [1 1], ua, xdim, udim);
        edSu(:,1) = dS(2,1,1,:);
        edSv(:,1) = dS(1,2,1,:);


        % c.- normal to surface (ns) and to edge curve (nc):

        ns = cross(edSu,edSv);

        if edr == 1
            nc = cross(-edSu,ns);
        elseif edr == 2
            nc = cross(-edSv,ns);
        elseif edr == 3
            nc = cross( edSu,ns);   
        elseif edr == 4
            nc = cross( edSv,ns);    
        end

        % d.- Computes angle in parameter space correspondant to normal vector nc:

        [alfa] = ar3307_AngleU (nc, edSu, edSv, xdim);


        % f.- Estimates position at ews from edge and stores in Utemp:

        [ub] = dp3350_DPincrement ...
        (esw, alfa, ua, dSmesh,Dspline, sDegree, sNcp, sKnotV, sUsize, sEP,sEw, Dumax, ULims, xdim, nd, DerC);
        Utemp(po,:) = ub(:);

    end


    % 7.3.- Amends ends:

    %   computes intersection and moves to initial of each edge

    ua = zeros(udim,1);
    ub = zeros(udim,1);
    uc = zeros(udim,1);
    ud = zeros(udim,1);

    effEN = zeros(4,2); % start/end edge nodes references

    for ed = 1:1:4

        % refernces of initial and final vtx of that curve
        ra = accEN(ed)+1;
        rd = accEN(ed+1);

        % a.- First segment:

        % relevant surface limits:
        % defines the limit perp. to the edge-curve

        if ed == 1
            ea = [ULims(1,1) ULims(2,1)];
            eb = [ULims(1,1) ULims(2,2)];
        elseif ed==2
            ea = [ULims(1,1) ULims(2,1)];
            eb = [ULims(1,2) ULims(2,1)];
        elseif ed==3
            ea = [ULims(1,2) ULims(2,1)];
            eb = [ULims(1,2) ULims(2,2)];
        elseif ed==4
            ea = [ULims(1,1) ULims(2,2)];
            eb = [ULims(1,2) ULims(2,2)];     
        end

        % finds intersecton btwn ea-eb and initial segments of temporary ews (Utemp):
        ouF = 0 ;
        while(ouF == 0)

            rb = ra+1;
            
            ua(:) = Utemp(ra,:) ;
            ub(:) = Utemp(rb,:) ;

            if ULims(1,1)<=ub(1) && ub(1)<=ULims(1,2)
            if ULims(2,1)<=ub(2) && ub(2)<=ULims(2,2)

                [IP, ~] = go2524_TwoSegmIntersec (ua,ub, ea,eb);

                effEN(ed,1) = ra;
                Utemp(ra,1) = IP(1);
                Utemp(ra,2) = IP(2);
                ouF = 1;

            end
            end
            ra = rb ;
        end


        % b.- Last segment:

        % relevant surface limits:
        % defines the limit perp. to the edge-curve

        if ed==1
            ea = [ULims(1,2) ULims(2,1)];
            eb = [ULims(1,2) ULims(2,2)];
        elseif ed==2
            ea = [ULims(1,1) ULims(2,2)];
            eb = [ULims(1,2) ULims(2,2)];
        elseif ed==3
            ea = [ULims(1,1) ULims(2,1)];
            eb = [ULims(1,1) ULims(2,2)];
        elseif ed==4
            ea = [ULims(1,1) ULims(2,1)];
            eb = [ULims(1,2) ULims(2,1)];     
        end


        % finds intersecton btwn ea-eb and final segments of temporary ews (Utemp):

        ouF = 0 ;
        while(ouF==0)

            rc = rd-1;
            uc(:) = Utemp(rc,:) ;
            ud(:) = Utemp(rd,:) ;
            if ULims(1,1)<=ud(1) && ud(1)<=ULims(1,2)
            if ULims(2,1)<=ud(2) && ud(2)<=ULims(2,2)

                [IP, ~] = go2524_TwoSegmIntersec (uc,ud, ea,eb);

                effEN(ed,2) = rd;
                Utemp(rd,1) = IP(1);
                Utemp(rd,2) = IP(2);
                ouF = 1;

            end
            end
            rd = rc ;
        end
    end


    % 7.4.- Allocates to edges variables:

    for ed = 1:1:4

        Nesc(ed) = effEN(ed,2)-effEN(ed,1)+1;

        conta = 0 ;
        edgUtemp = zeros(1,udim);

        for i = effEN(ed,1):1:effEN(ed,2)

            conta = conta+1;

            Uesc(ed,conta,1) = Utemp(i,1) ;
            Uesc(ed,conta,2) = Utemp(i,2) ;

            edgUtemp(conta,1) = Utemp(i,1);
            edgUtemp(conta,2) = Utemp(i,2);
        end

        if ed==1
            UescM(ed) = max(edgUtemp(:,2));
        elseif ed==2
            UescM(ed) = min(edgUtemp(:,1));
        elseif ed==3
            UescM(ed) = min(edgUtemp(:,2));
        elseif ed==4
            UescM(ed) = max(edgUtemp(:,1));    
        end


        % X coord's:

        u = zeros(udim,1);
        for po = 1:1:Nesc(ed)

            u(:,1) = Uesc(ed,po,:) ;

            [X] = b2601_BNURBS_Xcoord ...
            (u, sKnotV, sNcp,sDegree,sUsize, sEw,sEP, sUdim,sXdim);

            Xesc(ed,po,1)=X(1);
            Xesc(ed,po,2)=X(2);
            Xesc(ed,po,3)=X(3);
        end

    end

    
    % 7.6.- Transfers to dSmesh structure:

    dSmesh.mNesc=	Nesc ;
    dSmesh.mUesc =  Uesc ;
    dSmesh.mUescM =	UescM ;
    dSmesh.mXesc =  Xesc ;

end

%}

end

