function [QITs] = ts3330_QIT ...
    (tolRc, tolRs, tolD, Nvx, R, Nsf, sf, xdim, ndig, plotF, UisoF,BIT, wbF, DerC)

% RETURNS:

% structure QITs with triangulation features

% INPUTS:
% Nvx: minimum No. vertexex per edge
% R: distance between vertexes
% esw: distance offset from edge to delete vertexes (edge strip width)
% tolRc, tolRs: percentages for tolerance (curves and surfaces)
% tolD. tolerance for calculation for surfaces derivatives
% Nsf: No. surfaces of the patch
% sf(Nsf): structure with all surfaces features as below:
% struct('Ncp',[], 'Degree',[], 'Usize',[], 'KnotV',[], 'EP',[], 'Ew',[], 'P',[],'w',[], 'CS',[0 0]);
% 
% xdim: No. physical space dimensions
% ndig: No. digits when rounding up
% plotF: if = 1 patch surfaces efatures are to be plotted in this routine
% (useful for analysing the triangulation itself)
% UisoF: if = 1 computes U-isotropic mesh and compares with QIT mesh
% BIT: if = 1 computes BIT mesh and its features
% DerC =  'ex' for exact derivative calculation, 'ip' for interpolated, 'sp' for spline surface



QITs(Nsf,1) = struct ('mNE',0, 'mTVcon',[],'mLVcon',[],'iem',[], 'mRv',[], 'mUv',[], 'mXv',[], 'mNlo', 0 , 'mNlv',[]);
udim = 2 ;

% 1.- REPRESENTATION PARAMETERS: 

% *** inidicates to be introduced by the user. ONLY necessary if is to plot

if plotF == 1

    Da = 10 /57.296 ;   % *** angle interval
    a1 = 15 /57.296 ;   % *** minimum angle interval centre
    a2 = 105 /57.296 ;  % *** maximum angle interval centre

    Nmv = 8 ;           % *** No. max. of valences

    % area of equilateral triangle of side R: A = R^2 / 4 * tan(60) = 0.433* R^2
    
    Atx = 0.433*R^2 ;
    
    At = round(Atx/2,0)*2;      % *** central area, rounded up to 2
   
    nia = 4 ;                   % *** No. intervals per side
    Ds =  5 ;                   % *** area interval
    s1 = 35 ; %At - nia*Ds ;
    s2 = At + nia*Ds ;

    % preliminary sizes:

    % mesh generated by QIT
    angl = zeros(1,1);
    vale = zeros(1,1);
    area = zeros(1,1);

    % mesh to compare, generated as isotropic in parameter space
    Uangl = zeros(1,1);
    Uvale = zeros(1,1);
    Uarea = zeros(1,1);
    
    % BIT mesh to compare with:
 	Bangl = zeros(1,1);
    Bvale = zeros(1,1);
    Barea = zeros(1,1);
    
    
    stKS = 0.025 ;      % *** step for knot spans physical space representation
    Ust = 0.1 ;         % *** step in U-iso mesh
    
    
    % figures references
    
    frcXv = 0;         % *** contour vertexes
    frKS  = 61;         % *** knot spans and contours in phys. space
    frUim = 22;         % *** U isomesh in param. space
    frXim = 23;         % *** U isomesh in physical space
    
    frXvOLD  = 0;       % ***
    frdSm   = 0; %12;       % ***
    fr6     = 0; %6;        % ***
    frCT    = 0; %7;        % ***
    frUv    = 0; %8;        % ***
    frXv    = 9;        % ***
    frdSm3D = 0; %10;       % ***
    
    ewsON = 1 ;         % *** plots edge width strip
    
 	fAng = 51 ;         % *** frquency of angles
    fVal = 52 ;         % *** frquency of valences
    fAre = 53 ;         % *** frquency of areas
    
end



% 2.- CONTOUR CURVES EXTRACTION:
% one structure with all non-repeated curves for all surfaces

% 2.1.- Extracts curves structure and relation with surfaces:
[cCurv, TNcu, SCcon, ncu, nlo, CScon, nsf] = ts3311_ECcurves(Nsf, sf, xdim);


% 2.2.- Curves vertexes (local physical/param's coord's):
[cLXv, cLUv, Ncv, cRc] = ts3312_Cvertexes (cCurv, TNcu, R, Nvx, tolRc, xdim);

% plots contour curves
%{


figure(99)

%scatter3(cCurv(9).P(:,1), cCurv(9).P(:,2),cCurv(9).P(:,3),'r'); hold on

for i = 1:1:cCurv(9).Ncp
    
    %text(cCurv(9).P(i,1), cCurv(9).P(i,2),cCurv(9).P(i,3),num2str(i)); hold on
    
end


for i = 1:1:TNcu
    
    x = zeros(max(Ncv),3);
    x(:,:) = cLXv(i,:,:);
    
    x = x(1:Ncv(i),:);
    
    plot3(x(:,1),x(:,2),x(:,3),'k'); hold on
    
    text(x(3,1),x(3,2),x(3,3),num2str(i)); hold on
    
    for j = 1:1:Ncv(i)
        %text(x(j,1),x(j,2),x(j,3),num2str(j)); hold on
    end
    
    
    
end

axis equal

if gg==0
end

%}


% 3.- COMPUTES SURFACE ISOTROPIC MESH FOR EACH SURFACE: 

if wbF == 1
    wb = waitbar(0,'surface 1');
    wbPos = [10 250 300 70]; % left bottom wide height
    set(wb,'position', wbPos);
end


% for sR = 5:1:5 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for sR = 1:1:Nsf 


    % 3.1.- Surface features:
    
    Ncp     = sf(sR).Ncp;
    Degree  = sf(sR).Degree;
    KnotV   = sf(sR).KnotV;
    Usize   = sf(sR).Usize;
    P       = sf(sR).P;
    w       = sf(sR).w;

    [Ew,EP] = GEN_12_ExpandBw (w, P, xdim, udim, Ncp);
    

    ULims = [ KnotV(1,1) KnotV(1,Usize(1))  ;  KnotV(2,1) KnotV(2,Usize(2))];
    
    NL = nlo(sR); % No. loops
    
	eswP = 30 ;
    esw = eswP/100 * R ;
    
    % 3.2.- Detects if it is strip or not:
    
    pt = 15 ; % percentage of deviation of var. coeff. to consider strip
    [strF, Ld] = sv3346_Strip (Ncp, EP, R, pt, xdim, udim);
    

    
    % 3.3.- Contours coord's in terms of surface:
    
    [cXv, cUv, Nlv, Nlcv] = rs3336_cUvSurface (cLXv, cLUv, Ncv, SCcon, ncu, nlo, cCurv, sR, sf, xdim);
                   

    
    %{
    if sR==5
        
        figure(sR)
        scatter3(cXv(1,:,1),  cXv(1,:,2),  cXv(1,:,3),  'r'); hold on
        axis equal
        
        figure(sR*10)
        scatter(cUv(1,:,1),  cUv(1,:,2),  'r'); hold on
        axis equal
        
        figure(sR*20)
        scatter3(P(:,1),  P(:,2),  P(:,3),  'b'); hold on
        
        for i = 1:1:prod(Ncp)
            text(P(i,1),  P(i,2),  P(i,3),  num2str(i)); hold on
        end
        
        axis equal
        
        if kk==0
        end
        
    end
    %}
    

    % 3.4.- Surface vertexes:
    
    [NCT, cnR, sRvt, sUvt, sXvt, inF, coF, dSmesh, sRv, sUv, sXv, Nv] = sv3340_Svertexes ...
    (sR, strF, Ld, pt, Degree,Ncp, KnotV, Usize, P,w, Ew,EP, ULims,R, esw, xdim, udim, ndig, tolRs,tolD, cUv, Nlv, Nlcv, DerC);
    


    % 3.5.- Estimates cRv of boundary curves in pattern space:
    
    [cRv] = rs3337_coRcoor (sRvt, sUvt, coF, cUv, NL, Nlv, udim, sR);

    %{
    if sR==3

        figure(sR)
        scatter(cRv(1,:,1),cRv(1,:,2),'.','k'); hold on
        
        scatter(sRvt(:,1),sRvt(:,2),'.','r');
        
        axis equal
        
       
        
    end
    %}
    
 	% 3.6.- Selects only surface coordinates that are within the loops
    % from sUv. Selects them and stores in sUvi

    [sRvi, sUvi, sXvi, Nvi] = sv3344_SvforT (sR, eswP, sRv, sUv, sXv, Nv, cUv, cXv, NL, Nlv, xdim, udim);
    
    
    % 3.7.- Merge all coordinates, triangulates and amends edge strip:
    
    [NE, TVcon, VTcon, LVcon, VLcon, TLcon, Rv, Uv, Xv,~,~,~] = rs3334_Rtriang ...
    (cRv,sRvi, cUv,sUvi, cXv, sXvi, Nvi, NL, Nlv, Nlcv, ncu, SCcon, sR, xdim);


    % 3.8.- Allocates to response:
    
    QITs(sR).mNE = NE ;
    QITs(sR).mTVcon = TVcon ;
    QITs(sR).mLVcon = LVcon ;
    QITs(sR).mRv = Rv ;
    QITs(sR).mUv = Uv ;
    QITs(sR).mXv = Xv ;
    
    QITs(sR).iem = zeros(NE,3,3);
    
    QITs(sR).mNlo = 1 ;
    QITs(sR).mNlv = Nlv ;
    
    
 	% 3.9.- operations if plotF is activated:
    
    if plotF == 1
        
        % plots contours and knot spans in physical space:
        ts3394_plotKnSp (frKS, stKS, KnotV, Ncp,Degree,Usize, Ew,EP,xdim); 
    
        if UisoF == 1

            % U-isotropic mesh generation:
            [uTVcon, uUv, uXv, uNE, uNV] = ts3392_UisoMesh (KnotV, Ncp,Degree,Usize, Ew,EP, Ust, xdim); 

            % plots U-isoMesh:
            ts3393_plotUisomesh (uNE, uUv, uXv, uTVcon, frUim, frXim)

            % matrices to store angles, valences and areas for UisoMesh and BIT mesh:
            [Uangl, Uvale, Uarea] = ts3395_AVA (Uangl, Uvale, Uarea, uNE, uXv, uTVcon, uNV);
        end
        
        
        % BIT mesh:
        if BIT == 1
            NVc = size(cXv,2);
            [Bangl, Barea, Bvale] = ts3398_BITmesh (Bangl, Barea, Bvale, At, NE, NVc);
        end
        
        
        NV = size(Xv,1);
        [angl, vale, area] = ts3395_AVA (angl, vale, area, NE, Xv, TVcon, NV);
        
        % plots QIT-mesh:
        ts3396_plotQITmesh (frXvOLD, frdSm, fr6, frCT, frUv, frXv, frdSm3D, ...
                cXv, sXv, dSmesh, sUvt, coF, cnR, NCT, ewsON, Xv, Uv, TVcon, NE);
    end

    if wbF == 1
    waitbar(sR/Nsf,wb, strcat('surface : ',num2str(sR+1))) ; 
    end
    
end

if wbF == 1
delete (wb);
end




%{
figure(frcXv)
scatter3(sXvt(:,1),sXvt(:,2),sXvt(:,3)); hold on

figure(2)
scatter3(sXvt(:,1),sXvt(:,2),sXvt(:,3)); hold on
%}

% 5.- PLOTS:

if plotF == 1
    
    % 5.1.- Curves vertxes plotting:
    
    NC = size(Ncv,1);
    [Lavc] =  ts3391_plotcXv (NC, Ncv, cLXv, frcXv);
    
    
    % 5.2.- Characterization plots:

    clor = 'r';
    xoff = 0 ;
    [Fangl, Farea, Fvale] = ts3397_plotAVA ...
    (angl, area, vale, Da, a1, a2, Ds, s1, s2, Nmv, fAng,fVal,fAre, clor, xoff);

    if UisoF==1
        
        clor = 'b';
        xoff = 22 ;
        [Fangl, Farea, Fvale] = ts3397_plotAVA ...
        (Uangl, Uarea, Uvale, Da, a1, a2, Ds, s1, s2, Nmv, fAng,fVal,fAre, clor, xoff);
    
    end 
    
    
    if BIT==1

    	clor = 'b';
        xoff = 22 ;
        [Fangl, Farea, Fvale] = ts3397_plotAVA ...
        (Bangl, Barea, Bvale, Da, a1, a2, Ds, s1, s2, Nmv, fAng,fVal,fAre, clor, xoff);
        
    end
    
    
    
    % sets limits of axis and legends:
    
    figure(fAng)
    ylim([0 100])
    xlabel(strcat('ANGLES intervals=',  num2str(round(57.298*Da,0)), ' - first interval centre=',num2str(round(57.298*a1,0))))
    ylabel('frequency (%)')
    
    figure(fVal)
    xlim([1 Nmv])
    ylim([0 100])
        xlabel(strcat('VALENCES from 1 to  ', num2str(Nmv)))
    ylabel('frequency (%)')
    
    figure(fAre)
    ylim([0 100])
    xlabel(strcat('SIZES intervals=', num2str(Ds), ' - first interval centre=',num2str(s1)))
    ylabel('frequency (%)')
end




end

