function [QITst] = ts3330_QITst ...
    (tolRc, tolRs, tolD, Nvx, R, pesw, Nsf, sf, agCurv, cLXv, cLUv, Ncv, nlo, ncu, SCcon, xdim, ndig, plotF, UisoF,BIT, wbF, DerC)

% structure QITs with triangulation features, but for trimmed surface

% INPUTS:
% Nvx: minimum No. vertexex per edge
% R: distance between vertexes
% esw: distance offset from edge to delete vertexes (edge strip width)
% eswP: distance offset from edge to delete vertexes (edge strip width) in percentaje of the segment length
% tolRc, tolRs: percentages for tolerance (curves and surfaces)
% tolD. tolerance for calculation for surfaces derivatives
% Nsf: No. surfaces of the patch
% sf(Nsf): structure with all surfaces features as below:
% struct('Ncp',[], 'Degree',[], 'Usize',[], 'KnotV',[], 'EP',[], 'Ew',[], 'P',[],'w',[], 'CS',[0 0]);
% 
% Ncr: No. curves non-repeated
% cr(Ncr): curves structure

% xdim: No. physical space dimensions
% ndig: No. digits when rounding up
% plotF: if = 1 patch surfaces efatures are to be plotted in this routine
% (useful for analysing the triangulation itself)
% UisoF: if = 1 computes U-isotropic mesh and compares with QIT mesh
% BIT: if = 1 computes BIT mesh and its features
% DerC =  'ex' for exact derivative calculation, 'ip' for interpolated, 'sp' for spline surface

% NON-REPEATED CURVES !!
% one structure with all non-repeated curves for all surfaces
% cCurv: edge contour curves structure  
%       struct ('Ncp',0,  'Degree',0, 'Usize',0, 'KnotV',[], 'w',[],
%       'P',[], 'conF', 0); conF is contour flag = 1 if curve coincides
%       with at edge contours


%   nlo(Nsf,1): No. loops per surface
%   ncu(Nsf,nlo): No. curves for each surface and loop
%   SCcon(Nsf,nlo,max(ncu)): curve references for each surface

% cLXv, cLUv, Ncv: cCurvs vertexes


esw = pesw/100 * R ; % 'edge strip width'

% 1.- DECLARES STRUCTURE:

QITst(Nsf,1) = struct ('mNE',0, 'mTVcon',[], 'mVTcon',[], 'mLVcon',[], 'mVLcon',[], 'mTLcon',[], 'mRv',[], 'mUv',[], 'mXv',[],'mAe',[], 'mXNv',[], 'mXNe',[], 'mNlo', 0 , 'mNlv',[], 'mNlcv',[]);
udim = 2 ;


% 2.- CONTOUR CURVES COMPUTATION:

% now is an input, as it is done previously to this function:
% cLXv, cLUv, Ncv



% 3.- COMPUTES SURFACE ISOTROPIC MESH FOR EACH SURFACE: 

if wbF == 1
    wb = waitbar(0,'surface 1');
    % set(wb,'Name','surface 1');
    wbPos = [10 250 300 70]; % left bottom wide height
    set(wb,'position', wbPos);
end

for sR = 1:1:Nsf
%for sR = 2:1:2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    

    % 3.1.- Surface features:
    
    Ncp     = sf(sR).Ncp;
    Degree  = sf(sR).Degree;
    KnotV   = sf(sR).KnotV;
    Usize   = sf(sR).Usize;
    P       = sf(sR).P;
    w       = sf(sR).w;
    CS      = sf(sR).CS;

    [Ew,EP] = GEN_12_ExpandBw (w, P, xdim, udim, Ncp);
    
    
    ULims = [ KnotV(1,1) KnotV(1,Usize(1))  ;  KnotV(2,1) KnotV(2,Usize(2))];
    
    NL = nlo(sR); % No. loops
    
    
    % 3.2.- Contours coord's in terms of surface: Note only selects curves
    % relevant to that surface
    
    [cXv, cUv, Nlv, Nlcv] = rs3336_cUvSurface (cLXv, cLUv, Ncv, SCcon, ncu, nlo, agCurv, sR, sf, xdim);
       


    
    
    % 3.3.- Edge contour loop for that surface alone: 'unique surface' = usf
    
    usf = struct('Ncp',[], 'Degree',[], 'Usize',[], 'KnotV',[], 'EP',[], 'Ew',[], 'P',[],'w',[], 'CS',[0 0]); % un-trimmed surface feat.
    
	usf.Ncp     = sf(sR).Ncp;
    usf.Degree  = sf(sR).Degree;
    usf.KnotV   = sf(sR).KnotV;
    usf.Usize   = sf(sR).Usize;
    usf.P       = sf(sR).P;
    usf.w       = sf(sR).w;
	usf.EP      = sf(sR).EP;
    usf.Ew      = sf(sR).Ew;
    usf.CS      = sf(sR).CS;

    [ecCurv, eTNcu, eSCcon, encu, enlo, ~, ~] = ts3311_ECcurves (1, usf, xdim);
    
    [ecLXv, ecLUv, eNcv, ecRc] = ts3312_Cvertexes (ecCurv, eTNcu, R, Nvx, tolRc, xdim);

    [ecXv, ecUv, eNlv, eNlcv] = rs3336_cUvSurface (ecLXv, ecLUv, eNcv, eSCcon, encu, enlo, ecCurv, 1, usf, xdim);
    

	% 3.x.- Detects if it's strip or not:
    
    pt = 15 ; % percentage of deviation of var. coeff. to consider strip
    [strF, Ld] = sv3346_Strip (Ncp, EP, R, pt, xdim, udim);
    


    % 3.4.- Surface vertexes for un-trimmed surface:
    % separates vertexes inside surf. param. limits (sUv) from a bigger group of all of them (sUvt):
    
    [NCT, cnR, sRvt, sUvt, sXvt, inF, coF, dSmesh, sRv, sUv, sXv, Nv] = sv3340_Svertexes ...
    (sR, strF, Ld, pt, Degree,Ncp, KnotV, Usize, P,w, Ew,EP, ULims,R, esw, xdim, udim, ndig, tolRs,tolD, ecUv, eNlv, eNlcv, DerC);

   
    % 3.5.- Estimates pattern coordinates of contours:
    % cRv = zeros(NL,max(Nlv),udim) ;

    [cRv] = rs3337_coRcoor (sRvt, sUvt, coF, cUv, NL, Nlv, udim, sR);
    

    
    % 3.6.- Selects only surface coordinates that are within the loops
    % from sUv selectes them and stores in sUvi

    [sRvi, sUvi, sXvi, Nvi] = sv3344_SvforT (sR, pesw, sRv, sUv, sXv, Nv, cUv, cXv, NL, Nlv, xdim, udim);
    

    
    % 3.7.- Merge all coordinates and triangulates:
    
    % section 3 of this routine (rs3334) needs updated when having internal contour loops

    [NE, TVcon, VTcon, LVcon, VLcon, TLcon, Rv, Uv, Xv, XNe, XNv, Ae] = rs3334_Rtriang (cRv, sRvi, cUv, sUvi, cXv, sXvi, Nvi, NL, Nlv, Nlcv, ncu, SCcon, sR, xdim);

    

    
    
    % 3.8.- Allocates to response:
    
    QITst(sR).mNE = NE ;
    
    QITst(sR).mTVcon = TVcon ;
    QITst(sR).mVTcon = VTcon ;
    
    QITst(sR).mLVcon = LVcon ;
    QITst(sR).mVLcon = VLcon ;
    
    QITst(sR).mTLcon = TLcon ;
    
    QITst(sR).mRv = Rv ;
    QITst(sR).mUv = Uv ;
    QITst(sR).mXv = Xv ;
    
    QITst(sR).mXNe = XNe ;
    QITst(sR).mXNv = XNv ;
    
    QITst(sR).mAe = Ae ;
    
    QITst(sR).mNlo = NL ;
    QITst(sR).mNlv = Nlv ;
    
    QITst(sR).mNlcv = Nlcv ;
    
    if wbF == 1
    waitbar(sR/Nsf,wb, strcat('surface : ',num2str(sR+1))) ; 
    end
    
end


if wbF == 1
delete (wb);
end



end

