function [fs] = bf3403_fitSp2D_ (xf, Np, Degree, eUlim, xdim, paramPos, ktype, homCoord)

% RETURNS:
% fs: fitted spline Surface to a set of points

% INPUTS:
% xf(Npd1,Npd2,xdim): coordinates of points to fit
% Np(udim): No. points in each direction
% Degree(udim): desired degree


% 1.- DEFINES fs STRUCTURE: is a Bspline

udim = 2 ;
Npt = Np(1)*Np(2);

fs = struct('dNcp',[], 'dDegree',[], 'dUsize',[], 'dKnotV',[], 'dEP',[], 'ULim',[] );


% 2.- BASIC FEATURES:

dNcp = zeros(udim,1);
dDegree = zeros(udim,1);
dUsize = zeros(udim,1);

for di = 1:1:udim
    dNcp(di) = Np(di);
    dDegree(di) = Degree(di);
    dUsize(di) = dNcp(di) + dDegree(di) + 1;
end



% 3.- COMPUTES PARAMETER POSITIONS FOR EACH xf POINT:
%     uses selected method

uxf = zeros(Np(1), Np(2), udim);

if strcmp(paramPos,'as-points')==1

    for i = 1:1:Np(1)
    for j = 1:1:Np(2)

        for di = 1:1:udim
            uxf(i,j,di) = xf(i,j,di);
        end
        
    end
    end

else  
    %{
    % PENDING ! 
    
	xhom = xf ; 
	if homCoord == 1
        
        xs = zeros(Npt,xdim); % coord's in a single row
        
        conta = 0 ;
        for i = 1:1:Np(1)
       	for j = 1:1:Np(2)
            
            conta = conta+1;
            for di = 1:1:xdim
                xs(conta,di) = xf(i,j,di);
            end
        end
        end
        
        % works out averags in each direcion:
        
        for di = 1:1:udim
            
        end
        
        % homogeniezed coordinates in each direction
        xhoms = bf3404_homCoor (xs, Npt, xdim);
        
    end
        
  	% parameters coorespondant to each point:
   	u0 = xf(1,1,di);
 	u1 = xf(Np(1),Np(2),di);
   	[uxf] = bf3407_paramPos (Np, xhom, u0, u1, paramPos);
        
 	%}
end




% 4.- ALLOCATES KNOT VECTOR:

dKnotV = zeros(udim, max(dUsize));
pdm = [2 1];

for di = 1:1:udim
    
    pd = pdm(di); % perp. direction
    
    % knot vector limits:
  	u0 = xf(1,1,di);
 	u1 = xf(Np(1),Np(2),di);  
    
    % averages uxf for that direction:
    uxd = zeros(Np(di),1);
    
    for i = 1:1:Np(di)
        acc = 0 ;
        for j = 1:1:Np(pd) 
            if di==1
                acc = acc + uxf(i,j,di);
            else
                acc = acc + uxf(j,i,di);
            end
        end
        uxd(i,1) = acc / Np(pd);
    end
    
    
    % computes knot vector:
    U = bf3408_knotV (Np(di), Degree(di), u0, u1, ktype, uxd);  
    
    % allocates to response:
    for i = 1:1:dUsize(di)
        dKnotV(di,i) = U(i);
    end
end



% 5.- CONTROL POINTS COORDINATES:

% 5.1.- Coefficients matrix: A c = xf

A = zeros(Npt,Npt);

fil = 0 ;
for ir = 1:1:Np(1)
for jr = 1:1:Np(2)
    
    fil = fil + 1 ;
    u = [uxf(ir,jr,1) uxf(ir,jr,2)];
    
    col = 0 ;
    for ic = 1:1:Np(1)
    for jc = 1:1:Np(2)
        
        col = col + 1 ;
        br = [ic jc];
        B = b2604_MNL (u, br, dKnotV, dNcp, dDegree, dUsize, udim); 

        A(fil,col) = B ;
        
    end
    end
end
end


% 5.2.- Computes coordinates of cp's:

dEP = zeros(Np(1), Np(2), xdim);

for di = 1:1:xdim

    b = zeros(Npt,1);
    
    conta = 0 ;
    for i = 1:1:Np(1)
    for j = 1:1:Np(2)

        conta = conta + 1 ;
        b(conta,1) = xf(i,j,di);
    end
    end
    
 	c = A\b;

    conta = 0;
    for i = 1:1:Np(1)
    for j = 1:1:Np(2)
        conta = conta + 1;
        dEP(i,j,di) = c(conta,1);
    end
    end

end

% 6.- ALLOCATES:

fs.dNcp = dNcp;
fs.dDegree = dDegree;
fs.dUsize = dUsize;
fs.dKnotV = dKnotV;
fs.dEP = dEP;
fs.ULim = eUlim;

end

