function [fc] = bf3406_fitSp1D (xf, Np, Degree, xdim, paramPos, ktype, homCoord, pupp)

% RETURNS:
% fc: fitted spline Curve to a set of points

% INPUTS:
% xf(Np,xdim): coordinates of points to fit
% Np: No. points to fit
% Degree: desired degree
% xdim: physical dimensions
% paramPos: 'equally_spaced', 'chord_length' or 'centripetal' or
% 'pre-established' (in the last case uses pupp)
% homCoord: if 1 homogenezes coordinates for computing parametrization

udim = 1 ;

% 1.- DEFINES fc STRUCTURE: is a Bspline

fc = struct('dNcp',[], 'dDegree',[], 'dUsize',[], 'dKnotV',[], 'dP',[] );


% 2.- BASIC FEATURES:

dNcp = Np;
dDegree = Degree;
dUsize = dNcp + dDegree + 1;


% 3.- COMPUTES PARAMETER POSITIONS FOR EACH xf POINT:
%     uses selected method




    u0 = 0 ;
    u1 = 1 ;

    if strcmp(paramPos,'pre-established')==1

        upp = pupp;

    else
        xfh = xf ;
        if homCoord == 1
            xfh = bf3404_homCoor (xf, Np, xdim);
        end
        [upp] = bf3407_paramPos (Np, xfh, u0, u1, paramPos);
    end


% 4.- ALLOCATES KNOT VECTOR:

[dKnotV] = bf3408_knotV (Np, Degree, u0, u1, ktype, upp);



% 5.- CONTROL POINTS COORDINATES:

% 5.1.- Coefficients matrix: A c = y

A = zeros(Np,Np);
for fil = 1:1:Np

    col = 0 ;
    for i = 1:1:Np

        col = col + 1 ;
        br = i;

        B = b2604_MNL (upp(fil), br, dKnotV, dNcp, dDegree, dUsize, udim);  
        A(fil,col) = B ;
    end
end


% 5.2.- Computes coordinates of cp's:

dP = zeros(Np,xdim);



for di=1:1:xdim
    
    b = zeros(Np,1);
    b(:,1) = xf(:,di);
    
    c = A\b;
    
    for i = 1:1:dNcp
        dP(i,di) = c(i);
    end
end


% 6.- ALLOCATES:

fc.dNcp = dNcp;
fc.dDegree = dDegree;
fc.dUsize = dUsize;
fc.dKnotV = dKnotV;
fc.dP = dP;


end

