function [lm] = tn3607_SiLvsMLR (connS, connT, THC, sr, A1, u)

% RETURNS:
% lm(3,3): info of the 3 lines of the side to a missing line: 
%           tet line ref, distance to mLR and tet line length
% lm is returned already arranged from closest to furthest distance

% INPUTS:
% connS, connT: tet. conn. matrices.
% THC: nodes of tet.
% sr: side ref of the tets.
% A1,u: point and vector of the missing line.

lm = zeros(3,3); % lines refs, distance to mLR and length of the side line

for k = 1:1:3
    
    lR = connS.SL(sr,k);

    B = THC.Av(connT.LV(lR,1),:)';
    v = THC.Av(connT.LV(lR,2),:)' - B;

    h = go2547_dBtwn2Lines (A1,u, B,v);
    
    lm(k,1) = lR ;
    lm(k,2) = h ;
    lm(k,3) = norm(v) ;
    
end

lm = sortrows(lm,2);

end

