function [dtT, dtF] = dt3701_typeDistrn (Anod, Aav, bave)

% RETURNS:
% dtT: distorted tetrahedron type: "CAP", "SLIVER", "WEDGE"
% dtF: distorted tet. features (depends on the type)

% works in the 'projected' plane

nd      = 6 ;
pudim   = 3 ;
tolL    = 0.1 ; % tol. to compute if it is within segment (in % segment)
tolD    = 5.0 ; % tol. to assume two nodes are too close (in % of bave)

Tnod = zeros(3,pudim);

LNcon = [ 1 2 ; 2 3 ; 3 1 ; 4 1 ; 4 2 ; 4 3 ]; % line VS local-node connectivity.

dtT = "0" ;
dtF = 0 ;

% thresholds:
ppd_th = 2 ;
DL_th = 10 ;


% 1.- CHARACTERISTICS OF THE DISTORTED TET:


% 1.1.- Fits plane and computes ppd:

[pAnod, ppd] = dt3702_ppd (Anod, Aav, bave);



% 1.2.- Checks if there is intersection between projected lines:

[ipF, ilLR] = dt3703_intersLines (pAnod, tolL, nd);


% 1.3.- Lines lengths and increments, and max. increment and position in scale:

[mL, DL] = dt3704_mL (LNcon, Anod);


% 1.4.- Local ref of centre node (in case exist) and pcd:




mL
ppd
ipF
DL

if ppd <= ppd_th  % plane
    
    if DL(1) > DL_th && DL(2)==1
        
        dtT = "WEDGE" ;
        
    else
        
        if ipF==0
            
            dtT = "CAP" ;
            
        else
            
            dtT = "SLIVER" ;
            
        end
        
    end
    
    
else        % not plane
    
    
    if DL(1)>DL_th && DL(2)==1
        
        dtT = "WEDGE" ;
        
    else
        
        if DL(1)>DL_th && DL(2)==3
            
            dtT = "NEEDLE" ;
            
        else
            
            dtT = "SPINDLE" ;
            
        end
        
        
    end
    
    
end

dtT


% 2.- CHECKS IF IT IS A "CAP" CASE:
% conditions:
% - no intersection and points separated more than tolD % of bave
% - internal points separted from contour lines more than tolD from % bave

if ipF == 0 && mL(1,2) > tolD * bave /100
    
    % detects internal point:
    
    ipR = 0 ;
    mnr = [ 2 3 4 ; 1 3 4 ; 1 2 4 ; 1 2 3 ] ;
    
    for i = 1:1:4
        
        a1(:,1) = pAnod(i,:);
        
        for j = 1:1:3
            nr = mnr(i,j);
            Tnod(j,:) = pAnod(nr,:);
        end
        
        [IN,~,~] = go2548_PinTri2 (Tnod, a1, pudim, nd);
        
        if IN > 0
            ipR = i ;
            break
        end
        
    end
    

    % measures distance from internal point to contour lines:
    
    r = [ mnr(ipR,1) mnr(ipR,2) mnr(ipR,3) mnr(ipR,1) ];
    
    a1(:,1) = pAnod(ipR,:);
    
    mDc = zeros(3,2); % contour ref and distance
    
    for j = 1:1:3
        
        r1 = r(j);
        r2 = r(j+1);
        
        a2(:,1) = pAnod(r1,:);
        b2(:,1) = pAnod(r2,:);
        v2 = b2-a2 ;
        
        [dist, d] = go2527_distPtoRecta (a1, a2, v2, pudim);
        
        mDc(j,1) = j ;
        mDc(j,2) = dist ;
        
    end
    
    mDc = sortrows(mDc,2);

    if mDc(1,2) > tolD * bave /100
        dtT = "CAP" ;
        dtF = ipR; % internal node local ref.
        return
    end
     
end




% 3.- CHECKS IF IT IS A "SLIVER" CASE:
% conditions:
% - intersection of two lines and points separated more than tolD % of bave

if ipF == 1 && mL(1,2) > tolD * bave /100
    
	dtT = "SLIVER" ;
    
    dtF = ilLR; % local refs of lines that intersect

	return
 
end






end

