function [Xnr, Unr, cvF, Niter] = ap1A_9711_NRiter (Nmax, u0, xt, KnotV, Ncp, Degree, Usize, Ew, EP, w, P, CD, udim, xdim, Xtol, DUtol)

% RETURNS:

% positions computed for N-R iteration for N increments
% Xnr(N,xdim): physical space
% Unr(N,udim): param. space

% INPUTS:
% Nmax: maximum No. steps to do
% u0: initial position for searching
% xt: target physical space position

cvF = 0 ;

Xnr = zeros(1,xdim);
Unr = zeros(1,udim);

Niter = 0 ;

ru1 = zeros(udim,1);
ru2 = zeros(udim,1);
for di = 1:1:udim
    ru1(di) = u0(di);
    ru2(di) = u0(di);
end


% first coordinate and checks if it coincides already:

X = b2601_BNURBS_Xcoord (u0, KnotV, Ncp, Degree, Usize, Ew, EP, udim, xdim);
Xnr(1,:) = X(:,1);
Unr(1,:) = ru1(1,:);

DX = zeros(xdim,1);

for di=1:1:xdim
    DX(di,1) = xt(di)-X(di);
end
nDX = norm(DX);

if nDX <= Xtol
    cvF = 1 ;
end




% LOOP ON N stepsm only if is not already convergent

if cvF==0
for k = 1:1:Nmax

    Niter = Niter+1;
    
    % 1.- Rounds up to achieve same precision as in GEN_03_FindSpan:
    
    ru1 = round(ru1,12); 
    ru2 = round(ru2,12); 

    
    % 2.- Computes physical position 'S' correspondant to u:
    
    Unr(k,:) = ru1(:,1);
    [X] = b2601_BNURBS_Xcoord (ru2, KnotV, Ncp,Degree,Usize, Ew,EP, udim,xdim);
    Xnr(k,:) = X(:,1);
    
    
    % 3.- Computes increment of u, (needs Jacobian and 'force' vector 
    %     calculated from first derivatives of S about parameter space)
    
    % previous iteration 'ru' value
    pru1 = ru1 ; 
    pru2 = ru2 ; 
    
    r = (X-xt);

    [Ju, dS1] = b2806_Jpp (Degree, Ncp, KnotV, Usize, P ,w, r, ru2, udim, xdim);
    
    fu = zeros(udim,1);
    for di = 1:1:udim
        
        Si = zeros(xdim,1);
        
        for xdi = 1:1:xdim
      	Si(xdi) = dS1(di,xdi);
        end

        fu(di) = dot (r, Si);
    end

    du = -inv(Ju)*fu;
    ru2 = ru2 + du ;
    

 	% 4.- Makes sure is ru goes back to domain in case is out:
  	% fBackD(udim)=1 if was out of domain and goes back
	% fLoopD(udim)=1 if passes the limit in the loop direction
    
    ru1 = ru2 ;
    [ru2, fBackD, fLoopD] = z2803_Ulimits(KnotV, Usize, ru1, udim, CD);
    
    
    % updates X:
    
    X = b2601_BNURBS_Xcoord (ru2, KnotV, Ncp, Degree, Usize, Ew, EP, udim, xdim);
    
	% Computes X-error:
    Xerr = norm(X-xt);


    % 5.- Stop loop checking:

    % 5.1.- Increment of u:
    if norm(pru2-ru2) < DUtol 
        %cvF = 1 ;
    end
    
    % 5.2.- Error w.r.t. target:
    if Xerr < Xtol
        cvF = 1 ;
    end


    Xnr(k+1,:) = X(:,1);
    for di = 1:1:udim
        Unr(k+1,di) = ru2(di);
    end
    
    if cvF==1
        break
    end
    
    % 5.- Computes errors:
    %{
    [err1, err2, E2] = z2802_errors(Xcoor, S, dS1, xdim, udim, Lerr2);
    [err1o] = z2801_errorO (pru1, ru1, udim);
    
    if err1o < Lerr1o
       cerr1o = cerr1o + 1 ; 
    end
    

    % 2.5.- No. loops and stores Ucoord's
    
    cL = cL + 1 ;
    
    for di = 1:1:udim
        strdU(cL,di) = ru2(di);
    end


    %}

end
end




end

