function [Xtc, Utc, cvF, Niter] = ap1A_9712_TCiter (Nmax, u0, xt, KnotV, Ncp, Degree, Usize, Ew, EP, w, P, CD, udim, xdim, Xtol, DUtol)

% RETURNS:

% positions computed for tracing iteration for N increments
% Xtc(N,xdim): physical space
% Utc(N,udim): param. space
% cvF = 1 if converges before Nmax iterations
% Niter: No. used iterations

% INPUTS:
% N: max. No. steps to do
% u0: initial position for searching
% xt: target physical space position

cvF = 0 ;
Xtc = zeros(1,xdim);
Utc = zeros(1,udim);

Niter = 0 ;

ru1 = u0;


% first coordinate and checks if it coincides already:

X = b2601_BNURBS_Xcoord (u0, KnotV, Ncp, Degree, Usize, Ew, EP, udim, xdim);
Xtc(1,:) = X(:,1);
Utc(1,:) = ru1(1,:);

DX = zeros(xdim,1);

for di=1:1:xdim
    DX(di,1) = xt(di)-X(di);
end
nDX = norm(DX);

if nDX <= Xtol
    cvF = 1 ;
end


% LOOP ON N steps only if is not already convergent

if cvF==0
for k = 1:1:Nmax

    
    Niter = Niter+1;
    
    % 1.- Increment of u estimated and updated u and x:

    if udim==1
        [Due] = c2814b_EstimatesU (ru1, xt, Degree, Ncp, KnotV, P ,w, xdim);
    elseif udim==2
        [Due,~] = s2834b_EstimatesU (ru1, xt, Degree, Ncp, KnotV, Usize, P ,w, EP ,Ew, xdim, Xtol);
    elseif udim==3
        [Due] = v2864b_EstimatesU (ru1, xt, Degree, Ncp, KnotV, Usize, P ,w, EP ,Ew, xdim);
    end
    

    upr = ru1 ;
    ru1 = ru1 + Due ;
    
    [ru1, fBackD, fLoopD] = z2803_Ulimits(KnotV, Usize, ru1, udim, CD);
    

    X = b2601_BNURBS_Xcoord (ru1, KnotV, Ncp, Degree, Usize, Ew, EP, udim, xdim);

    
    % 2.- Limits correction (here considers closed surface):
    %{
    for di = 1:1:udim
        
        if CD(di)==0
            
            if uu(di) < ULims(di,1)
                uu(di) = ULims(di,1);
            elseif ULims(di,2) < uu(di)
                uu(di) = ULims(di,2);
            end 
            
        else
            
            if uu(di) < ULims(di,1)
                
                Du = uu(di) - ULims(di,1);
                uu(di) = mod(Du,1);

            elseif ULims(di,2) < uu(di)
                
                Du = uu(di) - ULims(di,1);
                uu(di) = mod(Du,1);
            end 
        end
    end
    %}

    % 3.- Computes X-error:
    Xerr = norm(X-xt);
    
    
    % 4.- Stop loop checking:

    % 4.1.- Increment of u:
    if norm(upr-ru1) < DUtol 
        %cvF = 1 ;
    end
    
    % 4.2.- Error w.r.t. target:
    if Xerr < Xtol
        cvF = 1 ;
    end


    Xtc(k+1,:) = X(:,1);
    for di = 1:1:udim
        Utc(k+1,di) = ru1(di);
    end
    
    if cvF==1
        break
    end
    
end
end



end

