function [ss, su, Np, EVcon, NE] = ts99_02_3_Tsamp (n, Unod)


% RETURNS: sample points coordinates in parent and parameter space
% ss(Nv,2): 
% su(Nv,2):
% Nv: No. vertexes or sample points

% INPUTS:
% n: No point in one side of the triangle (not the hypotenusa)
% B(2,udim): base line ends coordinates
% v(udim,1): opposite vertex coordinates
% Nr: No. rows = No. point on B
% np: No. points per row

udim = 2 ;

iem = [ 0 4 6 ; 4 0 5 ; 6 5 0 ];

st = 1/(n-1); % step (same in r and s directions)
Np = 0 ;

ss = zeros(1,udim);
su = zeros(1,udim);

% loop on vertical lines at parent space:

r = 0 ;
for i = 1:1:n-1
    
    s = 0 ;
    for j = 1:1:n-(i-1)
        
        Np = Np + 1;
        
        ss(Np,1) = r ;
        ss(Np,2) = s ;
        
        [u, ~, ~] = mp1352_TRImapp ([r s]', Unod, udim, udim, iem);
        
        su(Np,1) = u(1);
        su(Np,2) = u(2);
        
        s = s + st ;
    end
    
    r = r + st;
end


% last point:

Np = Np + 1 ;

ss(Np,1) = 1 ;
ss(Np,2) = 0 ;

[u, ~, ~] = mp1352_TRImapp ([1 0]', Unod, udim, udim, iem);
        
su(Np,1) = u(1);
su(Np,2) = u(2);


% triangulates:

DT = delaunayTriangulation(ss);

% temporary triangles, remove linear triangles:

tEVcon = DT.ConnectivityList;
tNE = size(tEVcon,1);

Ar = (1/(n-1))^2 / 2 ; % reference triangle area

NE = 0 ;
EVcon = zeros(1,3);

for e = 1:1:tNE
    
    Snod = zeros(3,udim);
    for i = 1:1:3
        Snod(i,1) = ss(tEVcon(e,i),1);
        Snod(i,2) = ss(tEVcon(e,i),2);
    end
    
    [At,~] = tm1306_tArea_n (tEVcon, e, ss, Snod, udim);
    
    if At > Ar/10
        
        NE = NE + 1 ;
        EVcon(NE,:) = tEVcon(e,:);

    else

        
    end
    
end

 

%{
figure(88)
scatter(ss(:,1), ss(:,2), 'r');
axis equal

figure(86)
scatter(su(:,1), su(:,2), 'r');
axis equal
%}


end

