function [Ncp,Degree,KnotV,B,w, noc,Usize] = GEN_0300_NURBSrefnt...
         (Ncp,Degree,KnotV,B,w, noc,Usize, padd, nX, nIn, udim, xdim, pa)

% CARRIES OUT NURBS "TRADITIONAL" REFINEMENT => RETURNS refined patch features

% INPUTS:
% Ncp,Degree,KnotV, Usize:  patch features
% B,w:	Cp's X coor's and weights (collapsed form)
% pa:   patch reference (cor nX and nIn matrices)  




% 1.- MESH REFINEMENT: degree elevation %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1.- Elevated degrees:
% pe denotes desired degrees for each udim direction
% if pe=p no refinement is required

pe = zeros(udim,1);
p_ref = 0 ;


for d = 1:1:udim

    pe(d) = Degree(d) + padd(pa,d+1);   
    
    if padd(pa,d+1) > 0
    p_ref = 1 ;
    end
    
end


if p_ref == 1 % only carries on if there is refinement

    % calculates elevated parameters
    [Ncpe,KnotVe, Be, we, noce, Usizee] = GEN_03_02_pRefinement...
        (Ncp, KnotV, Degree, B,w, pe, noc, Usize, udim, xdim  );

    % transfers values
    KnotV = zeros(udim,max(Usizee));

    noc = noce ;
    B = Be;
    w = we;

    for d = 1:1:udim
        Ncp(d)    = Ncpe(d);
        Degree(d) = pe(d) ;
        Usize(d)  = Usizee(d);
        for i=1:1:Usizee(d)
           KnotV(d,i) = KnotVe(d,i); 
        end
    end
    
end
    


% 2.- MESH REFINEMENT: knot insertion general refinement %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % nXpa(udim) indicates how many times intermediate knots in each direction
    % are inserted; if = 0 then there is no refinement
    
    nXpa = zeros(udim,1); % insertion multipliers for that patch
    for di=1:1:udim
        nXpa(di,1) = nX(pa,di+1);
    end

    if sum(nXpa)>0

       [Ncpet,KnotVet,Bet,wet,Usizeet] = GEN_03_08_kRefinement...
           (Ncp, Degree, KnotV, B, w, nXpa, Usize, udim, xdim);

        Ncp = Ncpet;
        KnotV = KnotVet; 
        B = Bet;
        w = wet;
        Usize = Usizeet;
        noc = 1 ;
        for di=1:1:udim
           noc = noc * Ncp(di); 
        end
        
    end


% 3.- MESH REFINEMENT: knot insertion single knot insertion %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % nIn(4,4) indicates which patch, and point param. coord's to be
    % inserted
    % are inserted; if = 0 then there is no refinement
    
    % xie => % new knot coordinates, changes for each direction
    
    
    for i=1:1:size(nIn,1)
    if nIn(i,1) == pa
            
        % 3.1.- Insertion in each direction:
        
        for d=1:1:udim
            
            sal = 0 ; % changes to 1 if new knot already exists
            xie = nIn(i,d+1); % new knot position to insert

            % A.- copy knot vector in that direction, and detects if xie
            % exists already:
            
            Uet = zeros(Usize(d),1);
            for j=1:1:Usize(d)
               Uet(j) = KnotV(d,j); 
               if round(xie,5) == round(Uet(j),5)
                   sal = 1 ;
               end
            end
    
            
            % B.- Only continues if new knot position is non-existing:
            
            if sal ==0
               
                [Be, Ue, we, Ncpe, Utame] = GEN_03_10_1D_kSingleExt...
                                (Ncp, Degree(d), Uet, B, w, xie, d, xdim, udim);
                
                % updates geometrical parameters to refined mesh:
                Ncp = Ncpe ;
                B = Be;
                Usize(d) = Utame;
                for j=1:1:Utame
                   KnotV(d,j) = Ue(j); 
                end
                w = we;
            end

        end
      
    end
    end
    
    
    % 3.2.- Updates total No. control points:
    
    noc = 1 ;
    for di=1:1:udim
        noc = noc * Ncp(di); 
    end
  
    

end