function [EBwz,Noz, cK, iK] = GEN_03_03_1D_DivideBezier(p, Ncpe, U, EBw,d, xdim, udim) 

% RETURNS, for one parameter direction:
% Noz:              No. Bezier segments, say "z" segments
% EBwz(Noz,p+1,4):  CPs for each segment
% cK(Noz-1):        continuity at each knot
% ik(Noz-1):        internal knots

% INPUTS:   
% p:    degree in that direction
% Ncpe: No. cps in each direction
% U:    knot vector in that direction
% Bw:   projected cp's coordinates
% d:    direction 
% noce: No. cp's
% xdim: No. dims in physical space
% udim: No. dims in parameter space


% 1.- No. dims in each param. direction:

nL = ones(3,1);
for di=1:1:udim
   nL(di) = Ncpe(di); 
end



% 2.- Identifies how many curves or Bezier segments are in:
% count how many non-void knot spans exits

Noz=0;
for i=p+1:1:Ncpe(d)+1
if U(i)~=U(i+1)
        Noz=Noz+1;     
end
end



% 3.- Computes multiplicity and continuity on each internal knot:
% mK(Noz-1):    multiplicity of each internal knot 
% cK(Noz-1):    continuity of each internal knot


% counts No. unrepeated internal knots:
conta=1;
for i=p+2:1:Ncpe(d)
    if U(i)~=U(i+1)
        conta=conta+1;
    end
end

% sizes:
mK = zeros(conta-1,1);
cK = zeros(conta-1,1);
mKt= zeros(conta-1,1);
iK = zeros(conta-1,1);
mKt(1) = 1 ;
iK(1) = U(p+2) ;

% computation:
conta=1;
for i=p+2:1:Ncpe(d)
    if U(i)==U(i+1)
        mKt(conta) = mKt(conta) + 1;
    else
        iK(conta) = U(i) ;
        conta=conta+1;
        mKt(conta) = 1;
    end
end

% transfering values:
for i=1:1:conta-1
    mK(i) = mKt(i);
    cK(i) = p - mK(i);
end



% 4.- Runs through each knot and insert until the multiplicity achieved is
%       equal to p in order to split the curve. Considers that insert the 
%       knot at the end of each knot group:

posi = p+1;     % knot position inside the knot vector
Ut1  = U ;      % inital value for temporal knot vector
EBwt1 = EBw ;   % inital value for temporal CPs
nL(d) = Ncpe(d);% inital value for No CPs in relevant direction

for kn=1:1:Noz-1 % loops on knots groups
    
    posi = posi + mK(kn); % at the end of that group
    
    alfa = zeros(nL(d)+1); % size of alfas array
    
    for mul = mK(kn):1:p-1 % loops on each knot group to achieve p multiplicity

        uN = Ut1(posi);
        
        % updates No CPs
        nL(d)=nL(d)+1; 
        
        
        for a=1:1:nL(d) % alfa factors in relevant direction
            if a<=(posi-p)
                alfa(a)=1;
            elseif a>=(posi+1)
                alfa(a)=0;
            else
                alfa(a)= (uN - Ut1(a))/(Ut1(a+p) - Ut1(a)) ;
            end 
        end
        
        % size of storage:
        EBwt2=zeros(nL(1),nL(2),nL(3),xdim+1);

        for i=1:1:nL(1)
        for j=1:1:nL(2)  
        for k=1:1:nL(3) 
            
            for dir=1:1:(xdim+1) 
                
                if d==1
                    
                    if i==1
                        EBwt2(i,j,k,dir) = EBwt1(i,j,k,dir);
                    elseif i==nL(d)
                        EBwt2(i,j,k,dir) = EBwt1(i-1,j,k,dir);
                    else
                        EBwt2(i,j,k,dir) = EBwt1(i,j,k,dir)  *    alfa(i) + ...
                                           EBwt1(i-1,j,k,dir)* (1-alfa(i));
                    end
                    
                elseif d==2
                    
                    if j==1
                        EBwt2(i,j,k,dir) = EBwt1(i,j,k,dir);
                    elseif j==nL(d)
                        EBwt2(i,j,k,dir) = EBwt1(i,j-1,k,dir);
                    else
                        EBwt2(i,j,k,dir) = EBwt1(i,j,k,dir)  *    alfa(j) + ...
                                           EBwt1(i,j-1,k,dir)* (1-alfa(j));
                    end
                    
                elseif d==3
                       
                    if k==1
                        EBwt2(i,j,k,dir) = EBwt1(i,j,k,dir);
                    elseif k==nL(d)
                        EBwt2(i,j,k,dir) = EBwt1(i,j,k-1,dir);
                    else
                        EBwt2(i,j,k,dir) = EBwt1(i,j,k,dir)  *    alfa(k) + ...
                                           EBwt1(i,j,k-1,dir)* (1-alfa(k));
                    end
                end
                
            end
            
        end
        end
        end
        
        
       
        % updates knot vector
        
        Ut2 = zeros(nL(d)+p+1,1);
        
        for i= 1 : 1 : posi 
            Ut2(i)= Ut1(i);
        end
            Ut2(posi+1)= Ut1(posi);
        for i = posi+1 : 1 : nL(d)+p
            Ut2(i+1)= Ut1(i);
        end
        
        
        % moves to new knot position
        posi = posi+1; 
        


        % transfers values for new loop
        EBwt1 = EBwt2 ;
        Ut1  = Ut2 ;
     
    end % loops to achieve p multiplicity
  
    
end % loops on knots groups


% 5.- Tranfers values to EBwz:

if d==1
    
    EBwz = zeros(Noz,p+1,nL(2),nL(3),xdim); % extended indexes cp's matrix
    conta=1;
    i2=1;

    for z=1:1:Noz

        i1 = i2;
        i2 = (p+1) + p*(z-1);

        i0=1;
        for i=i1:1:i2
        for dir=1:1:(xdim+1) 
            
            EBwz(conta,i0,:,:,dir) = EBwt1(i,:,:,dir);
            
        end
        i0=i0+1;
        end
        conta=conta+1;
    end
    
elseif d==2
    
    EBwz = zeros(Noz,nL(1),p+1,nL(3),xdim); % extended indexes cp's matrix
    conta=1;
    j2=1;

    for z=1:1:Noz

        j1 = j2;
        j2 = (p+1) + p*(z-1);

        j0=1;
        for j=j1:1:j2
        for dir=1:1:(xdim+1) 
            
            EBwz(conta,:,j0,:,dir) = EBwt1(:,j,:,dir);
            
        end
        j0=j0+1;
        end
        conta=conta+1;
    end  
    

elseif d==3
            
    EBwz = zeros(Noz,nL(1),nL(2),p+1,xdim); % extended indexes cp's matrix
    conta=1;
    k2=1;

    for z=1:1:Noz

        k1 = k2;
        k2 = (p+1) + p*(z-1);

        k0=1;
        for k=k1:1:k2
        for dir=1:1:(xdim+1) 
            
            EBwz(conta,:,:,k0,dir) = EBwt1(:,:,k,dir);
            
        end
        k0=k0+1;
        end
        conta=conta+1;
    end  
    
end





end