function [EQwz] = GEN_03_04_1D_BezierPmax(EBwz,p,tinc,Noz,Ncpe,d,xdim,udim)

% RETURNS: EQwz(Noz,i,j,k,xdim+1): each Bezier curve coord's

% For each Bezier segment calculates new CP's to comply with 
% new degree(pe). They are stored in EQwz

% INPUTS:
% EBwz:     extended cp's projected physical coord's with Bezier extraction
% p:        degree in that direction
% tinc:     No. necessary increments in degree in that direction
% Noz:      No. Bezier segments
% Ncpe:     No. cp's in param. directions
% d:        param. direction to consider
% xdim:     No. dims in physical space


% 1.- Prepares response dimensions:

nze = p+tinc+1 ;
nL = ones(3,1);
for di=1:1:udim
    if di==d
        nL(di) = nze ;
    else
        nL(di) = Ncpe(di); 
    end
end
EQwz(Noz,nL(1), nL(2), nL(3), xdim+1)=0;
    


% 2.- Loop on Bezier segments and cp's

for z=1:1:Noz % Loop on Bezier segments
    
for i=1:1:nL(1) % Bezier CP's loop
for j=1:1:nL(2) 
for k=1:1:nL(3)  
    
    for dir=1:1:xdim+1 % loop on directions

        % selects relevant index
        if d==1
            re=i;
        elseif d==2
            re=j; 
        elseif d==3
            re=k;
        end
        
        c1 = max(1  ,  re-tinc);
        c2 = min(p+1,  re     );

        for c=c1:1:c2 % sumatorio

            if d==1
                re1=c; re2=j; re3=k;
            elseif d==2
                re1=i; re2=c; re3=k;
            elseif d==3
                re1=i; re2=j; re3=c;
            end

            EQwz(z,i,j,k,dir) = EQwz(z,i,j,k,dir) + EBwz(z,re1,re2,re3,dir)* ...
            factorial(p)/(factorial(c-1)*factorial(p-c+1)) * ...
            factorial(tinc)/(factorial(re-c)*factorial(tinc-re+c)) / ...
            (factorial(p+tinc) /(factorial(re-1)*factorial(p+tinc-re+1)));  

        end % sumatorio



    end % loop on directions

    
end   
end 
end % Bezier CP's loop

end % Bezier segments









end