function [Be, Ue, we, Ncpe, Utame] = GEN_03_09_1D_kextended(Ncp, p, U, B,w, NoRefinements,d, xdim,udim)

% THIS FUNCTION CALCULATES THE MATRICES FOR REFINEMENT 
% BY KNOT INSERTION REGARDING ALONG Xi DIRECTION

% p:	 order of xi NURBS
% n:	 No control points initial for Xi direction
% m:	 No control points initial for Ita direction
% Uxi:	 knot vector initial
% B:    initial CP's tensor
% w:    weights
% NoRefinements:   No loops to apply the refinement

% to consider coordinates and weights, the coordinates   
% need to be transformed from projection to real coordinates




% 1.- Starts CP's tensor, expand to indexes and transform to "real" coordinates:
%     Considers also weight matrix

nL = ones(3,1);
Ncpe=ones(udim,1);
for di=1:1:udim
    nL(di) = Ncp(di);
    Ncpe(di) = Ncp(di); % initializes also Ncpe
end

Be1 = zeros(nL(1),nL(2),nL(3),xdim);
we1 = zeros(nL(1),nL(2),nL(3));

conta=1;
for i=1:1:nL(1)
for j=1:1:nL(2)
for k=1:1:nL(3)
    
    for dir=1:1:xdim
        Be1(i,j,k,dir) = B(conta,dir) * w(conta);
    end
    we1(i,j,k) = w(conta);
    conta = conta+1;
end
end
end



% 2.- nT: No CP's "Temporal" at each loop:

nT = Ncp(d) ;



% 3.-Does the refinement as many times as required, which is = NoRefinements:

for refinement =1:1:NoRefinements

    % 3.1.- Counts gaps or elements to duplicate, ignore null elements
    %       AND defines new xi values "xie"
    NoG = 0;
    for i=p+1:1:nT
        if U(i)~=U(i+1)
            NoG = NoG + 1;
            xie(NoG) = (U(i) + U(i+1)) / 2;
        end
    end

    
    % 3.2.- Calculates extended knot vector:
    
    % initiates
    Ue(nT+p+1+NoG)=0;
    
    % calculates Uxie: INITIAL knots
    for i = 1 : 1 : p+1
        Ue(i) = U(i);
    end

    % calculates Ue: INTERMEDIATE knots
    % also, stores in posi() adress of new knots in Uxie()
    j = p+2;
    k = 1;
    conta=1;
    
    for i= p+2 : 1 : nT+NoG
        if U(j)<xie(k)
            Ue(i)=U(j);
            j=j+1;
        else
            Ue(i)=xie(k);
            posi(conta)=i;
            
            conta=conta+1;
            k=k+1;
        end
        if j>size(Ue,2) % avoids overflow
            j=size(Ue,1);
        end
        if k>size(xie,2)
            k=size(xie,1);
        end
    end
    
    % calculates Uxie: FINAL knots
    j=nT+1;
    for i= nT+NoG+1 : 1 : nT+p+1+NoG
        Ue(i) = U(j);
        j=j+1;
    end
    
    
    % 3.3.- Loop increasing the Control points tensor one by one:
    posiIndex=1;
    for ig=1:1:NoG

        % temporary knot vectors, UNextended and extended for that loop
        UxieT1(nT+p+ig  ) = 0 ;
        UxieT2(nT+p+ig+1) = 0 ;

        for j1=1:1:posi(posiIndex)-1
            UxieT1(j1) = Ue(j1);
        end
        for j2=1:1:posi(posiIndex)
            UxieT2(j2) = Ue(j2);
        end

        j1 = posi(posiIndex)  ;
        j2 = posi(posiIndex)+1;
        
        for k=posi(posiIndex)-posiIndex+1 : 1 : nT+p+1
            UxieT1(j1) = U(k);
            UxieT2(j2) = U(k);
            j1=j1+1;
            j2=j2+1;
        end

        NewXi =  xie(ig); % new knot xi value introduced

        % temporary CP's tensor and alfa vector
        Bei(nT+ig, Ncp(d) ,2)=0 ;
        alfa(nT+ig)=0;
        
        % position of older knot vector UxieT1 where insert new knot after
        k = posi(posiIndex)-1 ;

        % calculates alfa vector
        for j=1:1:nT+ig 
            if j<=(k-p)
                alfa(j) = 1;
            elseif (k-p+1)<=j && j<=k
                alfa(j) =  (NewXi-UxieT1(j)) / (UxieT1(j+p)-UxieT1(j));
            elseif (k+1)<=j
                alfa(j) = 0;
            end  
        end

        % calculates CP's tensor "Be2" using prior CP's tensor "Be1"
        % and the same with weight tensor:

        if d==1
        
            for j =1:1:nL(2)
            for k =1:1:nL(3)

                for dir=1:1:xdim

                Be2(1,j,k,dir) =  Be1(1,j,k,dir);
                we2(1,j,k)     =  we1(1,j,k);

                for i=2:1:nT+ig-1  
                    Be2(i,j,k,dir) = alfa(i)*Be1(i,j,k,dir) + (1-alfa(i)) * Be1(i-1,j,k,dir);
                    we2(i,j,k)     = alfa(i)*we1(i,j,k)     + (1-alfa(i)) * we1(i-1,j,k);
                end

                Be2(nT+ig,j,k,dir) =  Be1(nT+ig-1,j,k,dir);
                we2(nT+ig,j,k)     =  we1(nT+ig-1,j,k);

                end
            end
            end

        elseif d==2
            
            for i =1:1:nL(1)
            for k =1:1:nL(3)

                for dir=1:1:xdim

                Be2(i,1,k,dir) =  Be1(i,1,k,dir);
                we2(i,1,k)     =  we1(i,1,k);

                for j=2:1:nT+ig-1  
                    Be2(i,j,k,dir) = alfa(j)*Be1(i,j,k,dir) + (1-alfa(j)) * Be1(i,j-1,k,dir);
                    we2(i,j,k)     = alfa(j)*we1(i,j,k)     + (1-alfa(j)) * we1(i,j-1,k);
                end

                Be2(i,nT+ig,k,dir) =  Be1(i,nT+ig-1,k,dir);
                we2(i,nT+ig,k)     =  we1(i,nT+ig-1,k);

                end
            end
            end
            
        elseif d==3
                        
            for i =1:1:nL(1)
            for j =1:1:nL(2)

                for dir=1:1:xdim

                Be2(i,j,1,dir) =  Be1(i,j,1,dir);
                we2(i,j,1)     =  we1(i,j,1);

                for k=2:1:nT+ig-1  
                    Be2(i,j,k,dir) = alfa(k)*Be1(i,j,k,dir) + (1-alfa(k)) * Be1(i,j,k-1,dir);
                    we2(i,j,k)     = alfa(k)*we1(i,j,k)     + (1-alfa(k)) * we1(i,j,k-1);
                end

                Be2(i,j,nT+ig,dir)  =  Be1(i,j,nT+ig-1,dir);
                we2(i,j,nT+ig)     =  we1(i,j,nT+ig-1);

                end
            end
            end
        
        end
        
        
        % transfers the value
        Be1=Be2;
        we1=we2;
        
        posiIndex = posiIndex + 1 ;
    end

    % updates No CP's
    nT = nT + NoG ;
    
    % updates Uxi
    U = Ue;
    
    % transfers values
    Bei=Be2;
    wei=we2;

end % loop on NoRefinements



% 4.- Amends No CP's:

Ncpe(d) = nT;
nL = ones(3,1);
for di=1:1:udim
    nL(di) = Ncpe(di);
end
Utame = size(Ue,2);


% 5.- Does the projection (back to NURBS) and UN-extend (removes i,j,k indexes):

Be = zeros(nL(1)*nL(2)*nL(3),xdim);
we = zeros(nL(1)*nL(2)*nL(3),1);

conta=1;
for i=1:1:nL(1) 
for j=1:1:nL(2)
for k=1:1:nL(3)
    
    for dir=1:1:xdim
         Be(conta,dir) = Bei(i,j,k,dir)/wei(i,j,k);
    end
    we(conta) = wei(i,j,k);
    conta=conta+1;
end
end
end




end
