function [Be, Ue, we, Ncpe, Utame] = GEN_03_10_1D_kSingleExt...
    (Ncp, p, U, B, w, xie, d, xdim, udim)

% THIS FUNCTION CALCULATES THE MATRICES FOR REFINEMENT 
% BY KNOT INSERTION REGARDING ALONG Xi DIRECTION
% considers only one insertion at xie

% p:	 order of xi NURBS
% n:	 No. control points initial for Xi direction
% m:	 No. control points initial for Ita direction
% Uxi:	 knot vector initial
% B:    initial CP's tensor
% w:    weights
% xie:  new desired knot
% d:    direction considered

% to consider coordinates and weights, the coordinates   
% need to be transformed from projection to real coordinates


% 1.- Starts CP's tensor, expand to indexes and transform to "real" coordinates:
%     Considers also weight matrix

nL = ones(3,1);
Ncpe=ones(udim,1);
for di=1:1:udim
    nL(di) = Ncp(di);
    Ncpe(di) = Ncp(di); % initializes also Ncpe
end

Be1 = zeros(nL(1),nL(2),nL(3),xdim);
we1 = zeros(nL(1),nL(2),nL(3));

conta=1;
for i=1:1:nL(1)
for j=1:1:nL(2)
for k=1:1:nL(3)
    
    for dir=1:1:xdim
        Be1(i,j,k,dir) = B(conta,dir) * w(conta);
    end
    we1(i,j,k) = w(conta);
    conta = conta+1;
end
end
end



% position just before new insertion: posi, in the original knot vector U:

n = Ncp(d);
for i=1:1:n+p+1
if U(i) <= xie && xie < U(i+1)
    posi = i ;
end
end



% 3.1.- Calculates extended knot vector Ue, inserting xie:
    
Ue = zeros(1,n+p+2) ;
    
for i=1:1:posi
    Ue(i) = U(i);
end

Ue(posi+1) = xie ;

j=i+1;
for i= (posi+2):1:(n+p+2)
    Ue(i) = U(j);
    j=j+1;
end
    



% 3.3.- Inserts new knot:

% temporary CP's tensor and alfa vector
alfa = zeros(n+1);
        


% calculates alfa vector , here Ue or U ???
k = posi ;
for j=1:1:n+1
    
	if j<=(k-p)
        alfa(j) = 1;
 	elseif (k-p+1)<=j && j<=k
        alfa(j) =  (xie-U(j)) / (U(j+p)-U(j));
  	elseif (k+1)<=j
        alfa(j) = 0;
    end  
    
end

% calculates CP's tensor "Be2" using prior CP's tensor "Be1"
% and the same with weight tensor:
        

if d==1
        
            for j =1:1:nL(2)
            for k =1:1:nL(3)

                for dir=1:1:xdim

                Be2(1,j,k,dir) =  Be1(1,j,k,dir);
                we2(1,j,k)     =  we1(1,j,k);

                for i=2:1:n 
                    Be2(i,j,k,dir) = alfa(i)*Be1(i,j,k,dir) + (1-alfa(i)) * Be1(i-1,j,k,dir);
                    we2(i,j,k)     = alfa(i)*we1(i,j,k)     + (1-alfa(i)) * we1(i-1,j,k);
                end

                Be2(n+1,j,k,dir) =  Be1(n,j,k,dir);
                we2(n+1,j,k)     =  we1(n,j,k);

                end
            end
            end

elseif d==2
            
            for i =1:1:nL(1)
            for k =1:1:nL(3)

                for dir=1:1:xdim

                Be2(i,1,k,dir) =  Be1(i,1,k,dir);
                we2(i,1,k)     =  we1(i,1,k);

                for j=2:1:n
                    Be2(i,j,k,dir) = alfa(j)*Be1(i,j,k,dir) + (1-alfa(j)) * Be1(i,j-1,k,dir);
                    we2(i,j,k)     = alfa(j)*we1(i,j,k)     + (1-alfa(j)) * we1(i,j-1,k);
                end

                Be2(i,n+1,k,dir) =  Be1(i,n,k,dir);
                we2(i,n+1,k)     =  we1(i,n,k);

                end
            end
            end
            
elseif d==3
                        
            for i =1:1:nL(1)
            for j =1:1:nL(2)

                for dir=1:1:xdim

                Be2(i,j,1,dir) =  Be1(i,j,1,dir);
                we2(i,j,1)     =  we1(i,j,1);

                for k=2:1:n  
                    Be2(i,j,k,dir) = alfa(k)*Be1(i,j,k,dir) + (1-alfa(k)) * Be1(i,j,k-1,dir);
                    we2(i,j,k)     = alfa(k)*we1(i,j,k)     + (1-alfa(k)) * we1(i,j,k-1);
                end

                Be2(i,j,n+1,dir)  =  Be1(i,j,n,dir);
                we2(i,j,n+1)     =  we1(i,j,n);

                end
            end
            end
        
end
        

% 4.- Amends No CP's:

Ncpe(d) = n+1;
nL = ones(3,1);
for di=1:1:udim
    nL(di) = Ncpe(di);
end
Utame = size(Ue,2);


% 5.- Does the projection (back to NURBS) and UN-extend (removes i,j,k indexes):

Be = zeros(nL(1)*nL(2)*nL(3),xdim);
we = zeros(nL(1)*nL(2)*nL(3),1);

conta=1;
for i=1:1:nL(1) 
for j=1:1:nL(2)
for k=1:1:nL(3)
    
    for dir=1:1:xdim
         Be(conta,dir) = Be2(i,j,k,dir)/we2(i,j,k);
    end
    we(conta) = we2(i,j,k);
    conta=conta+1;
end
end
end




end
