function [Pr,Ur,wr,Ncpr, Utamr] = GEN_03_13_1D_kRemoval (Ncp, p, U, Utam, P,w,ur,xdim)


% RETURNS:
% 1D NURBS features after removing knot ur
% Note:     ur must be a repeated knot in U
%           it is convenient to round first knots and ur with certain No.digits


% 1.- Preliminary sizes:

Ncpr = Ncp-1;
Utamr = Utam-1;
Ur = zeros(Ncpr+p+1,1);

wr = zeros(Ncpr,1);
Pr = zeros(Ncpr,xdim);


% 2.- Expand coordinates to work with B-splines:

ePr = zeros(Ncpr,xdim+1);
eP = zeros(Ncp,xdim+1);

for po = 1:1:Ncp
    
    for di = 1:1:xdim
        eP(po,di) = P(po,di)*w(po);
    end
    eP(po,xdim+1) = w(po);
    
end



% 3.- Multiplicity 's':

s = 0;
for po = 1:1:Utam
    
    if U(po)==ur
        s = s + 1;
    end
end


% 4.- Position 'r' to remove (the last of the repated ur),
%     and reduced knot vector:

for po = Utam:-1:1
    
    if U(po)==ur
        r = po;
        break
    end
end

pj = 0 ;
for po = 1:1:Utam
    if po~=r
        pj = pj+1;
        Ur(pj) = U(po);
    end
end


% 5.- Control points affected:

i = r-p;
j = r-s;

ijgap = j-i; % gap = No. cp's affected or moved


% 6.- Copy points not affected:

for a = 1:1:i-1
    
    for di = 1:1:xdim+1
        ePr(a,di) = eP(a,di);
    end
end

a = a + ijgap;

for b = j+1:1:Ncp

    a = a+1;
    for di = 1:1:xdim+1
        ePr(a,di) = eP(b,di);
    end
end




% 7.- SINGLE KNOT TO REMOVE - Computes affected points: THIS IS IN ABEYANCE
% AS NEEDS TO USE CURVE FITTING !!! SINCE GENERALLY IS NOT POSSIBLE !!

if s==1
    
        if mod(p,2)==0 % even degree

            Ni = ijgap/2 + 0.5;
            Nj = ijgap/2 - 0.5;

        else    % odd degree

            Ni = ijgap/2 ;
            Nj = ijgap/2 ;

        end

        % allocates is:

        for k = i : 1 : i+Ni-1

            ak = (ur-U(k)) / (U(k+p+1)-U(k)) ;

            for di = 1:1:xdim+1
            ePr(k,di) = (eP(k,di)-(1-ak)*ePr(k-1,di))/ak;
            end 

        end

        % allocates js

        for k = j : -1 : j-Nj+1


            ak = (ur-U(k)) / (U(k+p+1)-U(k)) ;

            for di = 1:1:xdim+1
            ePr(k-1,di) = (eP(k,di)-ak*ePr(k,di))/(1-ak);
            end

        end


    % check is a single knot can be removed:
    
    k = i ;
    ak = (ur-U(k)) / (U(k+p+1)-U(k)) ;

    for di = 1:1:xdim+1
        
        ePtrial = ak*ePr(k,di) + (1-ak)*ePr(k-1,di);
        
        if round(ePtrial,5)~= eP(k,di)
           
            disp('knot cannot be removed');
            break
        end
    end
        
        
end


% 8.- KNOT TO REMOVE IS MULTIPLE - Computes affected points:

if s > 1
    
    conta = 0;
    while (j-i > 0)

        ai = (ur - U(i)) / (U(i+p+1) - U(i));
        aj = (ur - U(j)) / (U(j+p+1) - U(j));

        conta = conta+1;
        for di = 1:1:xdim+1
            ePr(i,di) = (eP(i,di) - (1-ai)*ePr(i-1,di))/ai;
        end



        if conta < ijgap % only adds if have not reached the maximum  (ijgap)

            conta = conta+1;

            for di = 1:1:xdim+1
                ePr(j-1,di) = (eP(j,di) - (1-aj)*ePr(j,di))/aj; % WATCH THIS !! NOT SURE !!
            end

        end

        i = i+1;
        j = j-1;

    end
    
end



% 9.- WEIGHTES THE COORDINATES BACK:

for po = 1:1:Ncpr
    
    wr(po) = ePr(po,xdim+1);
    
    for di = 1:1:xdim
        Pr(po,di) = ePr(po,di)/wr(po);
    end

end



end

