function [x, nr] = GEN_11_cubicPolyRoot (a, b, c, d)

% a x^3 + b x^2 + c x + d = 0 

% RETURNS: uses Cardano algorithm
% x(nr,1): real roots of polynomial, from lowest to highest value
% nr: No. real roots


% special case 1:

if a == 0 && (b ~= 0 && c ~= 0)
    
    a = b ;
    b = c ;
    c = d ;

    R = b^2 - 4*a*c;
    
	if R > 0
        
        nr = 2 ;
        x = zeros(nr,1);
        
        x(1) = (-b + R^0.5)/(2*a);
        x(2) = (-b - R^0.5)/(2*a);
        
        x = sortrows(x);
        
    elseif R == 0
        
      	nr = 2 ;
        x = zeros(nr,1);
        
        x(1) = (-b )/(2*a);
        x(2) = (-b )/(2*a);
        
    elseif R < 0
        
      	nr = 0 ;
        x = zeros(1,1);
    end
    
    return
    
    
end


% special case 2:

if a == 0 && (b == 0 && c ~= 0)
    
    nr = 1;
    x = zeros(nr,1);
    x(1) = -d/c;
    
    return
    
end

% special case 3:

if a == 0 && (b == 0 && c == 0)
    
    nr = 0;
    x = zeros(1,1);
    
    return
    
end

% special case 4

if d==0
    
    R = b^2 - 4*a*c;
    
    if R > 0
        
        nr = 3 ;
        x = zeros(nr,1);
        
        x(1) = 0;
        x(2) = (-b + R^0.5)/(2*a);
        x(3) = (-b - R^0.5)/(2*a);
        
        x = sortrows(x);
        
    elseif R == 0
        
      	nr = 3 ;
        x = zeros(nr,1);
        
        x(1) = 0;
        x(2) = (-b )/(2*a);
        x(3) = (-b )/(2*a);
        
    elseif R < 0
        
      	nr = 1 ;
        x = zeros(nr,1);
        x(1) = 0;
        
    end
    
    return
    
end








% starndard case:


a1 = b/a ;
a2 = c/a ;
a3 = d/a ;


Q = (3*a2-a1^2)/9;

R = (9*a1*a2 - 27*a3 - 2*a1^3)/54;

D = Q^3 + R^2 ; % discriminant




if D > 0
    
    % one real root
    
    nr = 1 ;
    
    Ds = D^0.5;
    
    S1 = sign(R+Ds)*abs(R+Ds)^(1/3);
    S2 = sign(R-Ds)*abs(R-Ds)^(1/3);
    
    
    x = zeros(1,1);
    x(1) = S1 + S2 - a1/3;
    
    
    
elseif D == 0
    
    % three real roots, with at least one with m = 2
    
    Ds = D^0.5;
    
    S1 = sign(R+Ds)*abs(R+Ds)^(1/3);
    S2 = sign(R-Ds)*abs(R-Ds)^(1/3);
    
    x(1) = S1 + S2 - a1/3 ;
    
    x(2) = -(S1+S2)/2 - a1/3 ;
    
    x(3) = -(S1+S2)/2 - a1/3 ;
    
    x = sortrows(x);
    
elseif D < 0
    
    % three real roots:
    
    p = (3*a*c - b^2) / (3*a^2);
    q = (2*b^3 - 9*a*b*c + 27*a^2*d)/(27*a^3);
    
  	nr = 3 ;
    
    z = zeros(3,1);
    x = zeros(3,1);
    
    fi = acos( ((q^2/4)/(-p^3/27))^0.5 );
    
    for k =1:1:3
        z(k) = +2*(-p/3)^0.5 * cos(fi/3 + 2.0944*(k-1)) ;
        x(k) = z(k) - b/(3*a);  
    end
    
    x = sortrows(x);
    
end

%{

% obtain reduced from (with no quadratic term)

% z^3 + p z + q = 0 

p = (3*a*c - b^2) / (3*a^2);
q = (2*b^3 - 9*a*b*c + 27*a^2*d)/(27*a^3);


% discriminant of reduced from:

D = (-4*p^3 - 27*q^2);




if D > 0 && p < 0

    % 3 different real roots:
    
    nr = 3 ;
    
    z = zeros(3,1);
    x = zeros(3,1);
    
    fi = acos( ((q^2/4)/(-p^3/27))^0.5 );
    
    for k =1:1:3
        z(k) = 2*(-p/3)^0.5 * cos(fi/3 + 2.0944*(k-1)) ;
        x(k) = z(k) - b/(3*a);  
    end
 
elseif D == 0 && (p~=0 && q~=0)
    
    nr = 3 ;
    
    % 2 real roots: one repeated + one unique
    
    
    
    
elseif D == 0 && (p==0 && q==0)
    
    nr = 3 ;
    
    % 1 real root: multiplicity = 3
    
    x = ones(3,1) * (-b/3*a) ;

end

%}



end

