function [r,d,Pside] = GEN_14_distPc(P, curve, xdim, evalCurP)

% RETURNS: distance from a point P to a NURBS curve, in particular
% r:        vector from point to curve
% d:        distance (module of r)
% Pside:    'left': on the LHS of the curve, 'right': on the RHS
%           evaluates sign(r x du): postivive => 'left' ; negative => 'right'
%           only applicable for 2D planar curves. For 3D curves is not relevant


% INPUTS:
% P:        point coord's P(xdim), is a fila vector
% curve:    curve object
% xdim:     No. dims in physical space
% evalCurP: evaluation point of the curve (in curve physical space)



% Note: now is very basic, needs IMPROVEMENT


% 1.- Extract data from curve and initiates:
n = curve.mNcp ;
p = curve.mDegree ;
U = curve.mKnotV ;
B = curve.mB ;
w = curve.mw ;

r = zeros(xdim,1) ;
d = 0 ;
Pside = 'rite';



% 2.- Evaluates distance at each evaluation point and selects the one closer to P:

Nnp = size(evalCurP,1);

pc(:,1) = evalCurP(1,:); % 1 st evaluation point
dmin = norm(P'-pc);
curP = 1; % evaluation point reference


for np = 1:1:Nnp
    
    pc(:) = evalCurP(np,:); % eval. point of curve in physical space
    dtem = norm(P'-pc);  % distance evaluation
    
    if dtem<dmin
       dmin = dtem;
       curP = np;
    end
end



% 3.- Establishes vector distance results:

r = evalCurP(curP,:)-P ; % vector from P to curve
d = dmin ;  % distance



% 4.- Computes if is left of right (only relevant for 2D planar curves):

u    = (curP-1) * (U(n+p+1)-U (1)) / (Nnp-1) ; % location where to calculate derivative
Su   = GEN_03_FindSPAN(n,p,u,U);
derC = GEN_02_01_derRcurve(Su,u,p,U,n,B,w,1,xdim) ; % derivatives of curve about parameter u

for di=1:1:xdim
    dC(di) = derC(2,di); % 1st derivative
end


rxc = cross([r 0], [dC 0]); % cross product

if rxc(3) > 0
    Pside = 'left' ;
else
    Pside = 'rite';
end




end