function [M_un , RetatinedDirs] = GEN_16_RemoveRepeatedRows (M_o, Ncols, tol)


% RETURNS:
% M_un: same matrix as M_o but with no repeated rows
% directions:   matrix with all retained rows directions from M_o

% INPUTS:
% M_o:  matrix to eliminate repeated rows from
% Ncol: No. cols of M_o matrix
% tol:  tolerance to remove a row : count No. decimals of tol, eg: tol=0.01 => round(num,2)

Nd = round(log10(1/tol)); % No. decimals in the tolerance

% 1.- Preliminary sizes:

rCan = zeros(Ncols,1);    % candidate row to add from M_o to M_un
rAin = zeros(Ncols,1);     % row already in M_un 

Nfil = size(M_o,1); % No. rows of original matrix

M_un = zeros(1,Ncols); % pre-sizes response
RetatinedDirs = zeros(2,1);


% 2.- Allocates first row to response:

Nfilu = 1;
for c=1:1:Ncols
   M_un(1,c) = M_o(1,c); 
end
RetatinedDirs(1) = 1 ;

% 3.- Loop on original matrix rows:

for i=1:1:Nfil
    
    for di=1:1:Ncols % candiate row
    rCan(di) = M_o(i,di);
    end
    
    
    % 3.1.- Compares M_o row (rCan) with already stored in M_un (rAin):
    
    for j = 1:1:Nfilu
        
        for di=1:1:Ncols
        rAin(di) = M_un(j,di);
        end

        alreadyIN = 0; % rCan initially considered Not IN unless find equal row in M_un
        
        % round components
        rCan = round(rCan,Nd);
        rAin = round(rAin,Nd);
        
        
        % compares each component:
        dif = zeros(Ncols,1);
        for di=1:1:Ncols
            dif(di) = abs(rCan(di)- rAin(di));
        end
        if sum(dif)==0
        	alreadyIN=1;
            break
        end
         
    end
    
    
    % 3.2.- In not already IN includes in the responses:
    
    if alreadyIN == 0
        
        Nfilu = Nfilu + 1;
        for c=1:1:Ncols
           M_un(Nfilu,c) = M_o(i,c); 
        end
        RetatinedDirs(Nfilu) = i;
    end
    
    
end



end

