function [ix] = GEN_25_LdivBias (x1, y1, x2, y2, n, k45)

% RETURNS:
% ix(n,1): nodes of partition of line using linear bias of function y =
% f(x), with two end values given x1,y1 and x2,y2

% INPUTS:
% n: No. nodes, i.e. No divisions = n-1
% k45: proportion factor for angle y2-y1/x2-x1 = 45 degs (this must be btw 0 and 1, a good value is 0.6667)

% contratints: for a=0 k=1,  for a=90 k=0, for a=-90 k=2
% the funciton a=>k is defined in each half domain using:
% k = 2-k45 for negative semi-domain
% k = k45   for positive semi-domain


ix = zeros(n,1);

% angle:
a = round(atan((y2-y1)/(x2-x1))*180/3.141569,0);

% interpolates k factor:
if a<=0
    
    L0 = (a+45)*a /  4050;
    L1 = (a+90)*a / -2025;
    L2 = (a+45)*(a+90) / 4050;
    
    k = L0*2 + L1*(2-k45) + L2*1;
    
else
    
	L0 = (a-45)*(a-90)/  4050;
    L1 = (a-90)*a / -2025;
    L2 = (a-45)*a /  4050;
    
    k = L0*1 + L1*(k45) + L2*0;
    
end



% total length:
L = x2-x1 ;

% No. segments:
s = n-1;

% divisor:
d = 0 ;
for i = 1:1:n-1
    d = d + k^(s-i);
end

% final lenght:
lf = L/d;


% computes nodes positions:

ix(1) = x1 ;
for i = 2:1:s
    
    li = k^(s-(i-1)) * lf ;
    ix(i) = ix(i-1) + li ;
    
end

% last node:
ix(i+1) = x2 ;


%{

figure(1)
plot([x1 x2 x2 x1 x1], [0 0 y2 y1 0], 'k'); hold on
scatter(ix, zeros(n,1),'b'); hold on

%}

end

