function [ eNpo, eCpo, edCpo ] = GEN_26_DividesLine( Npo, Cpo, mDpr, dim)


% RETURNS: line divided according to min. distance precribed (mDpr) 
% eNpo:     No. point of the extended array
% eCpo:     Coordinates of the extended array (eNpo,dim)
% edCpo:    derivat. of the coord's of ext. array, indicates direction


% makes sure each array distance is equal or less than mDpr


% INPUTS:
% Npo:      No. points of actual array
% Cpo:      Coord's of actual array (Npo, dim)
% mDpr:     min. prescribed distance
% dim:      dimensions of the geometry


% 0.- Initiates responses and some variables:

eNpo  = 0 ;
eCpo  = zeros(1,1); % extended crack line points, considering triDpd
edCpo = zeros(1,1);

a = zeros(dim,1);
b = zeros(dim,1);
acc = zeros(dim,1);
deriv = zeros(dim,1);
dCpo = zeros(Npo,dim);


% 1.- Computes derivatives of original array:

for i = 1:1:Npo-1

    for di=1:1:dim
    deriv(di) = Cpo(i+1,di) - Cpo(i,di);
    end
    
    for di=1:1:dim
    dCpo(i,di) = deriv(di) / norm(deriv);    
    end 
end

for di=1:1:dim
dCpo(Npo,di) = dCpo(Npo-1,di);
end



% 2.- Loop on array segments:

for i = 1:1:Npo-1
    
    % 2.1.- Segment limits:
    for di=1:1:dim
        a(di) = Cpo(i  ,di);
        b(di) = Cpo(i+1,di);  
    end
    
    
    % 2.2.- Evaluates distance of segment a-b:
    [dis, r ] = GEN_22_dist_bwtin_2P( a, b, dim );

    
    % 2.3.- Computes extact value of step (equal or less than mDpr):
    
    Nsp = 1; % initially step is as segment, unless needs division
    step = r;
    
    if dis > mDpr
        
        Nsp = floor(dis/mDpr + 1); % No. spans to divide segment by
        
        for di=1:1:dim
        step(di) = r(di) / Nsp ;   
        end
    end
    
    
    % 2.4.- Set the last point, only last segment is considered until end:
    if i < Npo-1
        NspEnd = Nsp ;
    else
        NspEnd = Nsp+1;
    end
    
    
    % 2.5.- Computes segment points and deriv. of the extended array:
    
    acc = a;
    for j = 1:1:NspEnd
               
        eNpo = eNpo + 1 ; % updates No. points

        for di=1:1:dim

            eCpo(eNpo,di) = acc(di); % updates array coord's
            
            edCpo(eNpo,di) = dCpo(i,di); % transfers derivative
            
            % updates position
            acc(di) = acc(di) + step(di);
            if j==Nsp
            acc(di) = b(di) ; 
            end

        end
    end
    

    
    
    
end
    









end

