function [x] = GEN_31_inElemInterpol (TE, pU, eUnod, eX, udim, Nne, Nfil, Ncol)

% RETURNS:
% x:    value of variable at P location inside element interpolated from nodes - x(fil,col)

% notes: 
% works in parameter space, assumes linear elements !
% for rect-elems nodes are arragned hierarchically, ex. 2D : (1,1) (1,2) (2,1) (2,2)


% INPUTS:
% TE:               type of element 'T' or 'E'
% pU:               location (Ucoord's) where compute values
% eUnod(Nne, udim): location of element nodes 
% eX(Nne,ndi):      values at element nodes 
% Nfil,Ncol:        dimensions of vble x - eleX(Nne,fil,col)


x = zeros(Nfil,Ncol);


% A.- TRIANGULAR ELEMENTS:

if strcmp(TE,'T')==1
    

    % A.0.- Computes Jacobian of the Tr and the total vol. (Vt):
    % volume = 1/6 of Jacobian for tet.; 1/2 of Jacobian for triang.

    denom = (Nne^2 -3*Nne + 2) ; 

    J2 = GEN_06_01_Jac2_Element (Nne, udim, eUnod, 0);

    Vt = J2 / denom;


    % A.1.- Volumes of each sub-Tr:

    V = zeros(Nne,1);
    unod = zeros(Nne,udim);

    for nod=1:1:Nne

        % passes original nodes
        for it=1:1:Nne

            if it ~= nod

                for di = 1:1:udim
                unod(it,di) = eUnod(it,di);
                end

            elseif it == nod % substitutes t node by P

                for di=1:1:udim
                unod(it,di) = pU(di);    
                end
            end
        end

        % calculates Jacobian and volume:
        J2 = GEN_06_01_Jac2_Element (Nne, udim, unod, 0);
        V(nod) = J2 / denom;
    end



    % A.2.- Computes x:

    for nod=1:1:Nne

        for i=1:1:Nfil
        for j=1:1:Ncol
            x(i,j) = x(i,j) + eX(nod,i,j)*V(nod);
        end
        end
    end

    x = x / Vt ;

end


% B.- RECT. ELEMENTS:

if strcmp(TE,'R')==1

    denom = (2^udim);
    
    % extracts element domain limits:
    
    EdomL = zeros(udim,2);
    
    temp = sortrows(eUnod,[1:1:udim]);  
    for di = 1:1:udim
        EdomL(di,1) = temp(1,di);
        EdomL(di,2) = temp(end,di);
    end

    
    % transform pU Ucoord's in parent coord's & sets limits of i,j,k
    
    pP  = zeros(3,1); % general case of 3D
    Lim = ones(3,1);
    
    for di = 1:1:udim
        pP(di,1) = ( pU(di) - ((EdomL(di,2)+EdomL(di,1))*0.5) ) / ((EdomL(di,2)-EdomL(di,1))*0.5);
        Lim(di,1) = 2 ;
    end
    

    
    % shape functions value:
    
    N = zeros(Nne,1);
    
    conta = 0;
    for i=1:1:Lim(1)
  	for j=1:1:Lim(2)
   	for k=1:1:Lim(3)
        
        conta = conta + 1;
        N(conta,1) = (1+(-1)^i*pP(1)) * (1+(-1)^j*pP(2)) * (1+(-1)^k*pP(3)) ;
  	end
    end
    end
    
    N = N / denom ;

    % computes value:
    
    for nod = 1:1:Nne
        
        for i=1:1:Nfil
        for j=1:1:Ncol
            x(i,j) = x(i,j) + eX(nod,i,j)*N(nod,1);
        end
        end 
    end

    
end

end

