function [INc, Cxc, Cus] = GEN_53_xINcontour ...
         (u, x, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sCD, nlo, nPL, CLu, tncu, tRcu, bCurv, sudim, xdim)

% RETURNS:
% INc = 1 if the point with coordinates u & x (param. / physical) is inside contour loop of the skin
% Cxc: closest curve closest point to x


% 1.- CHECK IF PARAMETER COORDINATES 'u' ARE INSIDE THE CONTOUR POLYGONS

% works in skin parameter space:

sULims = zeros(sudim,2);

for di =1:1:sudim
    sULims(di,1) = sKnotV(di,1);
    sULims(di,2) = sKnotV(di,sUsize(di));
end

[INc, corte] = su2913_INcontours (u, nlo, nPL, CLu, sULims, sudim);

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (round(x(1),4)==24.0283 && round(x(2),4)==87.9911) && round(x(3),4)==0

    INc
    corte

    figure (222);
    plot(CLu(1,:,1) , CLu(1,:,2) );
    axis equal;
    
end
%}


% 2.- FINER CHECK IN PHYSICAL SPACE:

% closest point coord's of closest curve to 'x':
Cxc = zeros(xdim,1);
Cuc = 0;
Cus = zeros(sudim,1);

% if INc is still 0, does a finer check in physical space:
% this is in case x is very close to the contour, inside the
% contour curves, but outside the skin point polygon
        
if INc==0 && ((corte(1)==0 || corte(2)==0) || (corte(1)==0 && corte(2)==0) )
       
    % detects closest contour curve and computes closest point of that closest curve: ccx
    
    % sets distance:
  	dminC = 999999999999;
       
    
    % 2.1.- Loop on contour curves: finds closest curve to 'x', stores:
    % ccR: closest curve ref.
    % Cxc, Cuc: closest point of ccR to 'x' xcoord's & ucoord's
    
    for cu = 1:1:tncu
             
        % Extracts bCurve properties:
        
        cR = tRcu(cu);
        [cDegree, cNcp, cKnotV, cUsize, cP ,cw,cEw,cEB, cCD, cxdim, cudim] = GEN_54_curveProp (bCurv, cR);  

             
        % Finds closest curve point to 'x', stores temporarily in 'uc' and 'xc' :
        [uc] = b2811_PPonePoint (x, cDegree, cNcp, cKnotV, cUsize, cP ,cw, cCD, cxdim, cudim);

      	xc = b2601_BNURBS_Xcoord (uc, cKnotV, cNcp,cDegree,cUsize, cEw,cEB, cudim, cxdim); 
            
        
        % If distance from that curve closest point to 'x' is less than
        % current minimum distance => stores that curve and point as closest curve:
        
     	dist = norm(xc-x);
                
        if dist<dminC
        	ccR = cR; % curve ref.
            Cxc = xc; % x pos of curve
        	Cuc = uc; % u pos of curve
            dminC = dist;
        end
                
    end
        

    % 2.2.- Compares vectors: 'nv': normal to skin at Cxc; and 'vc' = v x dC:
   	% if 'nv' and 'vc' are in the same side => the point is IN the skin contour
    
  	% 2.2.1.- Computes normal of surface at that point 'Cxc' and stores in 'nv'  
            
  
    % param. coordinates of Cxc in terms of skin
    [Cus] = b2811_PPonePoint (Cxc, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sCD, xdim, sudim);
          
  	[nv] = go2551_nToBsurf (Cus, sDegree, sNcp, sKnotV, sUsize, sP ,sw, xdim, sudim);
     
    % 2.2.2.- Computes cross product vc = v x dC
    % v is vector from x to ccx , and dC derivative of curve at ccx
    
   	v = Cxc - x ;
          
    [cDegree, cNcp, cKnotV, cUsize, cP ,cw, ~, ~, ~, cxdim, cudim] = GEN_54_curveProp (bCurv, ccR);
         
  	[dCtemp] = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize,cP ,cw, [1], Cuc, cxdim, cudim);
   	dC = zeros(cxdim,1);
   	for di=1:1:cxdim
    	dC(di) = dCtemp(2,di);
    end
            
  	vc = cross(v,dC);
            
      
    % 2.2.3.- Checks if they are in the same side:
            
  	[ES] = go2510_vectorEQside (nv, vc);
            
   	if ES==1
        INc = 1 ;
    end
         
    
end





end

