function [IN, Tr, NBsk, BskR] = GEN_61_FcubeIN ...
         (gPatch, pa, ULims, cUcoor, cXcoor, Bskin, nBS, bCurv)

% RETURNS: for FIRST cubes
% IN = 1 if a cube is inside the domain
% Tr = 1 if cube trimmed by skin, 0 otherwise
% NBsk: No. trimming boun. skins
% BskR(NBsk,1): Bskins references

% INPUTS:
% ULims(udim, 2): limits of cube domain in parameter space
% Bskin() objects to compare with
% cUcoor, cXcoor(Nnod, dim): ctr. elem coor's, in terms of patch spaces

% 0.- INITIATES RESPONSES TO 0:

IN = 0 ;
Tr = 0 ;
NBsk = 0 ;
BskR = zeros(1,1);

% patch dims:
pudim = gPatch(pa).Udim ;
pxdim = gPatch(pa).Xdim ;
    

% 1.- FIRST INFORMATION: stores preliminary information using skin points

% stores for each skin: 
% (1) skin ref., (2) closest point ref, (3) min. param. space dist., (4) if trims the element
MdistS1 = zeros(nBS,4);

% works in parameter space

for s = 1:1:nBS
    
    nP = Bskin(s).nP;
    
    dminS = 999999999999999;
    sTr = 0 ;

    for po = 1:1:nP
        
        u = Bskin(s).mP(po).mAcoor;

        if Bskin(s).mTr > 0
            
            sIN = GEN_08_Orthodomain_Pin (u, ULims, pudim, 'opened', 0.000001);
            if sIN==1
            sTr = 1;
            end
        end
        
        % stores closes point of the skin to the center of anchor
        dist = norm(u-cUcoor);
        if dist < dminS
            dminS = dist;
            cprS = po;
        end
    end
    
    % stores in distS the minimum distance to the skin:
    MdistS1(s,1) = s ;
    MdistS1(s,2) = cprS ;
    MdistS1(s,3) = dminS ;
    MdistS1(s,4) = sTr;
end


% 2.- DETECTS IF IS TRIMMED: 
% in case of trimmed elem (Tr=1) it is also IN=1. Also computes BskR(NBsk)

% 2.1.- Loop on skins, but only compares to trimmed skins:

for s = 1:1:nBS
if Bskin(s).mTr > 0

    % uses MdistS1 first (it is cheaper!) :
    if MdistS1(s,4)==1
    
        TRIM = 1 ;
        
    % if it is not trimmed in the prelim. matrix, needs further check
    else
        
        % one skin check, we are only interested in if trims or not:
        [~,TRIM,~] = GEN_65_checkOneSk ...
        (gPatch, pa, s, Bskin, ULims, cUcoor, cXcoor, bCurv, pudim);
    end
    

    % transfers skin ref. it trims the elemnt:
   	if TRIM==1 
      	NBsk = NBsk+1;
      	BskR(NBsk,1) = s ;
    end
    
end
end

% 2.2.- If there are trimming skins => is a trimmed cube:

if NBsk > 0
    Tr = 1 ;
end


% 3.- DETECTS IF IS IN: uses closest NS skins and TRIM info

MdistS2 = sortrows(MdistS1,3);
NS = 2 ;

% 3.1.- Preliminary check 1: 
% if is TRIM then it is IN

if Tr==1
   IN = 1 ; 
end

   
% 3.2.- Preliminary check 2: 
% if closest NS skins (using MdistS matrix) are no-trimming => is IN
 
if IN==0
    
    acc = 0 ;
    for sk = 1:1:NS
        skR = MdistS2(sk,1); % skin ref.
        acc = acc + Bskin(skR).mTr ;
    end

    if acc == 0
        IN = 1;
    end
end


% 3.3.- Goes to further check if is not still IN:

% compares vectors directions: 
% 'nv' normal to skin at xClo
% 'v' fro cXcoor to xClo

% Compares vectors: 'nv': normal to skin at xClo; and 'v': vector from cube centre 'cXcoor' to 'xClo'  
% if 'nv' and 'v' are in the same side => the point is IN the domain

if IN ==0

    % 3.3.1.- First check: just take prelim. points
    
    skRclo = MdistS2(1,1);
    poRclo = MdistS2(1,2);
    
    xClo = Bskin(skRclo).mP(poRclo).mXcoor;
 	nv   = Bskin(skRclo).mP(poRclo).n;
    
  	% Computes vector from cube centre 'cXcoor' to closest point of skin 'xClo':
   	v = zeros(pxdim,1);
  	for di=1:1:pxdim
        v(di) = xClo(di) - cXcoor(di);
    end
    
    % computes angle between from nv to v (rads)
    [angu] = go2511_angleUV (nv, v);
    
    % angu less than 30 degrees => is IN
    if angu < 3.14159/6
        
        IN = 1 ;

    % angu greater than 150 degrees => is OUT
    elseif angu > 3.14159*5/6
        
        IN = 0 ;
        
    % 3.3.2.- Otherwise needs further check: 
    else
        
        % recomputes closest skin / point:
        [~, ~, ~, xClo, nv] = GEN_63_cloNS (pa, gPatch, cXcoor, NS, MdistS2, Bskin, bCurv);
        
        % Computes vector from cube centre 'cXcoor' to closest point of skin 'xClo':
        v = zeros(pxdim,1);
        for di=1:1:pxdim
            v(di) = xClo(di) - cXcoor(di);
        end

        % Checks if is in the same direction as normal vector of skin at xClo:
        [ES] = go2510_vectorEQside (nv, v);
        if ES==1
            IN = 1 ;
        end
        
    end
    
end



end

