function [IN, TRIM, SV, dbpS] = GEN_62_elePinTrim ...
         (gPatch, pa, skR, Bskin, ULims, bpU, bpX, udmax, bCurv)

% RETURNS:
% IN: if element is IN
% TRIM. if skin skR trims the elementn
% SV: if it finds definetly te solution of not (solved / not solved = 1/0)

% skin skR is assumed to be trimming skin of the patch
% computes point P: closer point of Bskin(skR) to bpU

% INPUTS:
% Bskin(skR): skin object to compare the element with
% ULims: element limits in param. space
% bpU: base point in patch param. space
% bpX: base point in physical space
% udmax: distance in param. space that if the distance from bpU to P is
% higher , then P is definetly out 


% patch features:
pNcp     = gPatch(pa).Ncp ;
pDegree  = gPatch(pa).Degree ;
pUsize   = gPatch(pa).Usize ;
pKnotV   = gPatch(pa).KnotV ;
pP       = gPatch(pa).P ;
pw       = gPatch(pa).w ;

pudim = gPatch(pa).Udim ;
pxdim = gPatch(pa).Xdim ;
    
pCD = zeros(pudim,1) ;

IN = 0 ;
TRIM = 0 ;
SV = 0;


% 1.- COMPUTES Ucoor (in terms of patch para. space) OF P POINT: PUp

% 1.1.- Extracts skin properties:

[sDegree, sNcp, sKnotV, sUsize, sP ,sw, sEw,sEB, sCD, nlo, nPL, CLu, tncu, tRcu, sxdim,sudim] ...
= GEN_52_skinProp (Bskin, skR);

% 1.2.- Initial computation of skin closest point:

%[Uclo] = im2199_skinCmP (Bskin(skR), bpX);
%PUs = b2812_PPonePo_pre(bpX, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sCD, sxdim, sudim, Uclo);

PUs = b2811_PPonePoint (bpX, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sCD, sxdim, sudim);

PX = b2601_BNURBS_Xcoord (PUs, sKnotV, sNcp,sDegree,sUsize, sEw,sEB, sudim, sxdim);

% 1.3.- Corrects position if is out of the contour of the skin:

[INc, Cxc, Cus] = GEN_53_xINcontour ...
(PUs, PX, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sCD, nlo, nPL, CLu, tncu, tRcu, bCurv, sudim, sxdim);

if INc == 0
    PX = Cxc ;
	PUs = Cus ;
end

% 1.4.- Computes param. coord's (in terms of patch) of skin closest point 'PX': PUp

%PUp = b2811_PPonePoint  (PX, pDegree, pNcp, pKnotV, pUsize, pP ,pw, pCD, pxdim, pudim);
PUp = b2812_PPonePo_pre (PX, pDegree, pNcp, pKnotV, pUsize, pP ,pw, pCD, pxdim, pudim, bpU);

% 2.- COMPUTES DISTANCES VECTORS FROM BASE POINT TO SKIN CLOSEST POINT:
% (in patch param. space & physical space):

d  = PUp - bpU;
dx = PX - bpX;
dbpS = norm(dx); % this is one of the answers


% 3.- FIRST CHECK: d > dmax => 
% P is out of the element limits, just determines if is IN
% location IS fully solved (SV = 1);

if norm(d) > udmax
    
    TRIM = 0 ;
  	SV = 1 ;
    
    % check directions of 'dx' and normal to skin 'nv':

  	[nv] = go2551_nToBsurf (PUs, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sxdim, sudim);

   	[ES] = go2510_vectorEQside (nv, dx);
    
  	if ES==1
        IN = 1 ;
    else
        IN = 0 ;
  	end

end

% 4.- FURTHER CHECK: in case we can not ensure P is outside the domain:

if norm(d) <= udmax
    
    % Checks if PUp is inside the domain of the parameter limits:
    
    INd = GEN_08_Orthodomain_Pin (PUp, ULims, pudim, 'opened', 0.000001);
  

    % 4.1.- P belongs to elements limits:
    % location IS fully solved (SV = 1);
    
    if INd ==1
        
        IN = 1 ;
        TRIM = 1 ;
        SV = 1 ;
        
        
    % 4.2.- P does not belongs to elements limits:   
    
    elseif INd==0
        
        % just check direction of normal to skin to know if is in or not
        % location is not fully solved (SV = 0);
        
        SV = 0 ;
        
      	[nv] = go2551_nToBsurf (PUs, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sxdim, sudim);

      	[ES] = go2510_vectorEQside (nv, dx);

        if ES==1
            IN = 1 ;
        else
            IN = 0 ;
        end
          
  
    end
    
end



%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% internal check
if (round(bpX(1),4)==87.5 && round(bpX(2),4)==37.5) && round(bpX(3),4)==10

    PX
    PUs
    PUp

    
end
%}




end

