function [skRclo, suClo, puClo, xClo, nv] = GEN_63_cloNS (pa, gPatch, cXcoor, NS, MdistS2, Bskin, bCurv)

% RETURNS: obtains closest point of closest skin to a point in physical space cXcoor
% skRclo: closest skin ref. to cXcoor
% suClo: closest skin point Ucoor in terms of skin param. space
% puClo: closest skin point Ucoor in terms of patch param. space
% xClo: closest skin point Xcoor
% nv: normal at skin puClo point

% INPUTS:
% cXcoor: base point to compare the NS skins to
% NS: No. skins to check
% MdistS2: skins ararnged by nearing to cXcoor, with 4 cols: 
% sk ref, sk closest prelim. point ref, distance from cXcoor, element trimmed by that skin flag

% patch features:
pNcp     = gPatch(pa).Ncp ;
pDegree  = gPatch(pa).Degree ;
pUsize   = gPatch(pa).Usize ;
pKnotV   = gPatch(pa).KnotV ;
pP       = gPatch(pa).P ;
pw       = gPatch(pa).w ;

pudim = gPatch(pa).Udim ;
pxdim = gPatch(pa).Xdim ;
    
pCD = zeros(pudim,1) ;


Xdistmin = 99999999999999999 ;
   
% 1.- LOOPS ON 'NS' closest skins:
% target: find closest point of closest skin

for sk = 1:1:NS

	skR = MdistS2(sk,1); % skin ref.

 	% 1.1.- Extracts skin features:
    
  	[sDegree, sNcp, sKnotV, sUsize, sP ,sw, sEw,sEB, sCD, nlo, nPL, CLu, tncu, tRcu, sxdim,sudim] ...
   	= GEN_52_skinProp (Bskin, skR);


   	% 1.2.- Finds closest point of skin 'skR' NURBS surface: 
    % computes U and X coord's of the closest point, u & x (u in terms of skin param. space):
    
   	u = b2811_PPonePoint (cXcoor, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sCD, sxdim, sudim);
   	x = b2601_BNURBS_Xcoord (u, sKnotV, sNcp,sDegree,sUsize, sEw,sEB, sudim, sxdim);

    % 1.3.- In case u (or x) is out of the contour loop, re-computes it:
 	% finds the closest point of closest contour curve to cXcoor, that will be the updated u (or x).

   	% INc= 1 if x inside the contour
   	% Cxc: Xcoor'd of closest contour point to x

  	[INc, Cxc, Cus] = GEN_53_xINcontour (u, x, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sCD, nlo, nPL, CLu, tncu, tRcu, bCurv, sudim, pxdim);

   	% re-allocates x and u to move back to point to be at the
   	% contour in case is out of the contour:
   	if INc == 0
        x = Cxc ;
      	u = Cus ;
    end


  	% 1.4.- Checks distance and stores if is less than Xdistmin:

  	dist = norm(x - cXcoor);
        
    if dist < Xdistmin  

        Xdistmin = dist; % distance
      	skRclo   = skR ; % skin ref.

     	% coordinates of closest point of closest skin:
      	xClo = x ;
      	suClo = u ;
      	puClo = b2811_PPonePoint (xClo, pDegree, pNcp, pKnotV, pUsize, pP ,pw, pCD, pxdim, pudim);
    end 
end


% 2.- COMPUTES NORMAL TO CLOSEST SKIN AT CLOSEST POINT puCLo:

[sDegree, sNcp, sKnotV, sUsize, sP, sw,~,~,~,~,~,~,~,~, sxdim, sudim] ...
      	= GEN_52_skinProp (Bskin, skRclo);

[nv] = go2551_nToBsurf (puClo, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sxdim, sudim);
    

end

