function [ NRefin ] = c1402_CopytoNextRefin (UsInte, ORefin, Du, NTs, udim, xdim)

% RETURNS:
% NRefin:   prepares NRefin for the next refinement (if exists) and next step. 
%           NRefin inially is a copy of Refin

% INPUTS:
% ORefin:   original instance to be copied
% Du:       anchors displacements to be added to ORefin previously to copy
%           Du is extracted from u total , specifically for this patch  
% NTs:       time step of the new instance
% SndO:     if =1  considers 2nd order effect, that is, copied parameters
%           are updated alongside anchors displacements

% 0.-Declares new instance:

NRefin = GenInstance;


% 1.- Direct parameters:

NRefin.mTstep   = NTs ;  
NRefin.mDegree  = ORefin.mDegree;
NRefin.mUsize   = ORefin.mUsize;
NRefin.mKnotV   = ORefin.mKnotV;
NRefin.mKnotVI  = ORefin.mKnotVI;
NRefin.mNcpOn   = ORefin.nAN;
NRefin.mcpOn    = ORefin.mcpOn;
NRefin.mcpRef   = ORefin.mcpRef;
NRefin.Anet     = ORefin.Anet;
NRefin.nAN      = ORefin.nAN;
NRefin.nSP      = ORefin.nSP;
NRefin.sfT      = ORefin.sfT;
%NRefin.mAc      = ORefin.mAc;
%NRefin.mBsp     = ORefin.mBsp;

% 2.- Anchors:

OmA = ORefin.mA ;
nAN = ORefin.nAN ;
CmA = c1404_CopyAnchors ( OmA, nAN , Du ,udim, xdim ) ;
NRefin.mA = CmA ;



% 3.- R-Mesh:

NDegree = NRefin.mDegree;

ORmesh = ORefin.mRM;

CRmesh = c1405_CopyRmesh (ORmesh, CmA, NDegree, xdim, udim);
NRefin.mRM = CRmesh ;


% 4.- Stress field points:

Relem   = NRefin.mRM.mE ;
RnE     = NRefin.mRM.nEL ;
NDegree = NRefin.mDegree;
nSP     = NRefin.nSP;
OmSf   = ORefin.mSf;

mSf(nSP) = GenPoint ;

for sp = 1:1:nSP

    mSf(sp) = p1203_CopyPointInfo...
   (UsInte, OmSf(sp), Relem, RnE, CmA, NDegree, udim, xdim);
end
NRefin.mSf = mSf ;



% 4.- Greville points:
%{
Relem   = NRefin.mRM.mE ;
RnE     = NRefin.mRM.nEL ;
NDegree = NRefin.mDegree;
nGv     = NRefin.nGv;
OmGre   = ORefin.mGre;

mGre(nGv) = GenPoint ;

for gv = 1:1:nGv

    mGre(gv) = p1203_CopyPointInfo...
   (UsInte, OmGre(gv), Relem, RnE, CmA, NDegree, udim, xdim);
end
NRefin.mGre = mGre ;
%}



% 5.- T-Mesh:

Relem = NRefin.mRM.mE ;

RnE = NRefin.mRM.nEL ;
Degree = ORefin.mDegree;

OTmesh = ORefin.mTM;

CTmesh = c1407_CopyTmesh (UsInte, OTmesh, Relem, RnE, CmA, Degree, xdim, udim);
NRefin.mTM = CTmesh ;


end

