function [sU] = b2802_3DPPforSetPoints_V02 ...
    (nsp, sX, Degree, Ncp, KnotV, Usize, P ,w, CD, tolX, dmin, xdim, udim, wbON)

% RETURNS:
% sU(nsp,udim): set of patch parameter coordinates correspondant to a set of points (No. points = nsp) defined in physical space

% computation is done by using point projection

% INPUTS:
% nsp: No. points
% sX(nsp,xdim): physical coord's of points
% KnotV, Degree, Usize, nAN, mA: T-apline patch features
% CD (udim): closed domain flag
% tolX: abs value of tolerance (error = norm(X2-X1)
% wbON: to active wait bar

% 1.- Computes first Ucoor for first Xcoor point projection:

firstX = zeros(xdim,1);
firstX(:) = sX(1,:);

% 2.- U LIMITS AND STEP FOR SEEKING:

ULims = zeros(xdim,2);
seekst = zeros(xdim,1);
    
for di = 1:1:udim
        
    ULims(di,1) = KnotV(di,1);
    ULims(di,2) = KnotV(di,Usize(di));

    seekst(di) = (ULims(di,2) - ULims(di,1) )*0.25;
end

% finds closest Ucoor with seekst:
[Ucoor,~] = b2842_intialU (seekst, ULims, firstX, dmin, tolX, Degree, Ncp, KnotV, Usize, P ,w, udim, xdim);
    
[Ew,EP] = GEN_12_ExpandBw (w, P, xdim, udim, Ncp);




if wbON==1
wb = waitbar(0,'computing Ucoor of stress rep. points');
set(wb,'Name','point projection process');
wbPos = [10 250 300 70]; % left bottom wide height
set(wb,'position', wbPos);
end

% 2.- LOOP ON POINTS TO SEARCH Ucoor's:
  
X1  = zeros(3,1);
X2  = zeros(3,1);
sU = zeros(nsp,udim);
ord = [1 1 1];

for po = 1:1:nsp
    
    % 2.1.- Sets initial phs. coord's (X1) and target (X2):
    
    if po==1
        [X1] = b2601_BNURBS_Xcoord (Ucoor, KnotV, Ncp,Degree,Usize, Ew,EP, udim,xdim);
    else
        X1(:,1) = sX(po-1,:);
    end

    X2(:,1) = sX(po ,:);


    
    % 2.2.- Recomputes X1 in case is too far from X2:
    
    seekst2 = seekst;
    while dmin < norm(X1-X2) 
        
        [Ucoor,~] = b2842_intialU ...
        (seekst2, ULims, X2, dmin, tolX, Degree, Ncp, KnotV, Usize, P ,w, udim, xdim);
    
        [X1] = b2601_BNURBS_Xcoord (Ucoor, KnotV, Ncp,Degree,Usize, Ew,EP, udim,xdim);
        
        seekst2=seekst2/2; 
        
    end
    
    
    seekst2 = seekst;
    endF = 0;
    contaL = 0 ;
       
    contaLim = 10 ;
    
    errors = zeros(contaLim,1);
    flagOut = zeros(contaLim,1); % = 1 for iteration where tries to go out of the domain with perp. trajectory
    
    if tolX < norm(X1-X2) 
        
        while endF == 0

            contaL = contaL + 1 ;

            % 2.2.- Increment in phys. space and vector x12:

            dX = norm(X1-X2);
            x12 = (X2-X1)/dX;

            % 2.3.- Finds correspondent vector to x12 in param. space u12:

            % deriv. at u1
            [dS] = b2701_BNURBS_deriv (Degree, Ncp, KnotV, Usize, P ,w, ord, Ucoor, xdim, udim);

            A = zeros(3,3);

            for fil = 1:1:3
                A(fil,1) = dS(2,1,1,fil);
                A(fil,2) = dS(1,2,1,fil);
                A(fil,3) = dS(1,1,2,fil);
            end

            u12 = A\x12;    
            u12 = u12/norm(u12);


            % 2.4.- Direc-derivative norm at U1:

            dSl = zeros(xdim,1);

            for xdi = 1:1:xdim
                dSl(xdi) = u12(1)*dS(2,1,1,xdi) + u12(2)*dS(1,2,1,xdi) + u12(3)*dS(1,1,2,xdi);
            end

            ndSl = norm(dSl);


            % 2.5.- Estimates step in param. and updates:

            Dl = dX/ndSl;

            for di = 1:1:udim

                Ucoor(di) = round(Ucoor(di)+u12(di)*Dl,9);

                if Ucoor(di) < ULims(di,1)
                    
                    Ucoor(di) = ULims(di,1);
                    
                    for dif = 1:1:xdim
                        if abs(round(u12(dif),4))==1
                            flagOut(contaL) = 1 ;
                        end
                    end
                    
                elseif ULims(di,2) < Ucoor(di)
                    
                    Ucoor(di) = ULims(di,2);
                    
                    for dif = 1:1:xdim
                        if abs(round(u12(dif),4))==1
                            flagOut(contaL) = 1 ;
                        end
                    end
                    
                end
            end

            % 2.6.- Recomputes X1:

            [X1] = b2601_BNURBS_Xcoord (Ucoor, KnotV, Ncp,Degree,Usize, Ew,EP, udim,xdim);


            % difference:
            err = norm(X2-X1);

            errors(contaL) = err;
            
            if err < tolX
                endF = 1 ;
            end

            if contaL == contaLim
                  
                if (round(errors(contaL),6) == round(errors(contaL-1),6) && ...
                   round(errors(contaL),6) == round(errors(contaL-2),6)) && ... 
                   ((flagOut(contaL)==1 && flagOut(contaL)==1) && flagOut(contaL)==1) 
                    
                    endF = 1 ;
                else
                    
                    contaL = 0;

                    [Ucoor,~] = b2842_intialU ...
                    (seekst2, ULims, X2, dmin, tolX, Degree, Ncp, KnotV, Usize, P ,w, udim, xdim);

                    [X1] = b2601_BNURBS_Xcoord (Ucoor, KnotV, Ncp,Degree,Usize, Ew,EP, udim,xdim);

                    seekst2 = seekst2/2; 
                end

            end   
        end

    end

    % 2.7.- Transfers U value computed:
    
    for di = 1:1:udim
        sU(po,di) = Ucoor(di);
    end
    
    if wbON==1
    waitbar(po/nsp,wb, strcat('point= ',num2str(po+1))) ; 
    end
    
end

if wbON==1
delete (wb);
end

end

