function [u, rF] = pc2851_PPcurves (x, u0, Ncp, Degree, KnotV, Usize, P, w, CD, xdim, nd, xerrTol)

% RETURNS:
% u: curve param. coordinate correspondant to x
% rF=1 if the point is not found, i.e. does not belong to the curve

% INPUTS:
% x(xdim,1): point to evaluate
% u0: initial curve param. coordinate for searching

% curve Ulims
ULims = [round(KnotV(1,1),nd) round(KnotV(1,Usize(1)),nd)];
dC = zeros(xdim,1);
spF = 0 ; % stop flag

rF = 0;

contaLMax = 12;
contaL = 0 ;
upr = zeros(2,1);

while (spF==0)

    contaL = contaL  + 1;
    
    % 1.- COMPUTES X & dC AT u0:

    x0 = b2601_BNURBS_Xcoord (u0, KnotV, Ncp,Degree,Usize, w,P, 1,xdim);
    dCm = b2701_BNURBS_deriv (Degree, Ncp, KnotV, Usize, P ,w, [1], u0, xdim, 1);

    for di = 1:1:xdim
        dC(di) = dCm(2,1,1,di);
    end
    Dx = norm(x0-x);
 
    % stores u prev. beforec chages it: 
    upr(contaL) = round(u0,nd) ;
    

    % 2.- ESTIMATES NEW u POSITION:

    Du = Dx/norm(dC);

    uP = round(u0+Du,nd); % forward 

    if uP > ULims(2)
        uP = ULims(2);
    end
    xP = b2601_BNURBS_Xcoord (uP, KnotV, Ncp,Degree,Usize, w,P, 1,xdim);
    dP = norm(xP-x);

    uN = round(u0-Du,nd); % backwards
    
    if uN < ULims(1)
        uN = ULims(1);
    end
    xN = b2601_BNURBS_Xcoord (uN, KnotV, Ncp,Degree,Usize, w,P, 1,xdim);
    dN = norm(xN-x);

    % selectes the one closer to x:
    
    if dP<dN
        u0 = uP; 
        x0 = xP;
    else
        u0 = uN; 
        x0 = xN;
    end


    % 3.- EVALUATES ERROR:

    % 3.1.- Coincidence of locations:

    xerr = norm(x0-x);

    if xerr < xerrTol
        spF = 1 ;
    end


    % 3.2.- Normal location:

    dCm = b2701_BNURBS_deriv (Degree, Ncp, KnotV, Usize, P ,w, [1], u0, xdim, 1);
    for di = 1:1:xdim
        dC(di) = dCm(2,1,1,di);
    end
    v = x - x0;

    if abs(dot(v,dC)) < 0.001
        spF = 1 ;
    end

    % 3.3.- Extreme:
    
    if (u0 == ULims(1) && upr(contaL) == ULims(1)) || (u0 == ULims(2) && upr(contaL) == ULims(2))
        
        if CD==0 % for open curves juest stop
            
            spF = 1 ;
            
        elseif CD==1 % for close curves passes to the other side
            
            if u0 == ULims(1)
                
                u0 = ULims(2);
                
            elseif u0 == ULims(2)
                
                u0 = ULims(1);
                
            end

        end
        
    end
    

    
    
    % 3.4.- If bounces => rejects
    
    if contaL>5
        
        if upr(contaL)==upr(contaL-2) && upr(contaL-1)==upr(contaL-3)
            
            rF = 1 ;
            spF = 1 ;
            u0 = (upr(contaL) + upr(contaL-1))/2; % estimates the average

        end
        
    end
 
    % 3.5.- Max No. loops:
    if contaL == contaLMax
            rF = 1 ;
            spF = 1 ;
            u0 = (upr(contaL) + upr(contaL-1))/2; % estimates the average
    end
    
end


% allocates:
u = u0 ;


end

