function [iu, NOC] = t2804_intialU (seekst, ULims, Xcoor, dmin, tolDist, Degree, nAN, mA, udim, xdim)

% RETURNS: 
% iu(udim,1): initial ucoor for searching by point projection Xcoor
% NOC: 'no convergence', becomes 0 (converges) if dmin < tolDist, otherwise is = 1

% INPUTS:
% seekst(3):   step in each direction for searching
% ULims(udim,2): limits of param. space for searching
% tolDist: if dmin < tolDist then that is directly Ucoor

% Notes:
% - just loop on u steps to localize to closest Ucoor to Xcoor
% - assumes all knot vectors from 0 to 1
% - works with extended version of 3D


% 1.- Sets relevant steps dimensions and sets initial / final u values

rstp = ones(3,1);
u0   = zeros(3,1); % initial knot value for searching
ue   = ones(3,1);  % end knot value for searching

for di=1:1:udim
    rstp(di) = seekst(di);
    u0(di) = ULims(di,1);
    ue(di) = ULims(di,2);
end


% 2.- Loops in param. space and find Ucoor closest to Xcoor using step rstp:

iu = zeros(3,1);

% reduces Xcoor to actual xdim:
rXcoor = zeros(xdim,1);
for di = 1:1:xdim
   rXcoor(di) = Xcoor(di); 
end

for u = u0(1):rstp(1):ue(1)
for v = u0(2):rstp(2):ue(2)
for w = u0(3):rstp(3):ue(3)    
    
  	[pNicp, pRicpA] = p1204_icpsPo (nAN, mA, [u v w], udim, 'closed');
    [N, Nicp, RicpA]= GEN_0908_N_upIPCs (pNicp, pRicpA, Degree, mA, [u v w], udim);
    [X, ~]          = GEN_0803_Xcoor_Tsplin_withN (mA, N, Nicp, RicpA, xdim);
    
    tdis = norm(X-rXcoor);

    if tdis<dmin
        iu = [u;v;w];
        dmin = tdis;
    end
    
end
end
end


% 3.- Determines if 'iu' is already the target Ucoord 
% and allocates value to NOC consequently:

if dmin < tolDist
    NOC = 0 ;
else
    NOC = 1 ;
end




end

