function [u, NOC] = t2805_UfromX (KnotV, Usize, Degree, nAN, mA, P, u0, Lerr1, Lerr2, udim, xdim)

% RETURNS:
% u: param. space coord's correspondant to a physical point P
% NOC: 0 - 1 if converges or not in less than cLLIM loops

% INPUTS:
% P:    physical coord's where find u from
% u0:   initial parameter for searching
% Lerr1, Lerr2:    errors limits for point coincidence and zero cosine, respectively, when convergence check, (abs. value, not percentaje)
% udim: dimension of parameter space


% 0.- Initiates errors and 'u' value:

err1 = Lerr1+1;

err2 = zeros(udim,1);
for di=1:1:udim
   err2(di) = Lerr2+1; 
end
E2 = 1; % turns to 0 when all err2 are less than Lerr2


% 2.- Loops until converges:

NOC = 0 ;
cL = 0 ;
cLLIM = 12 ;

% reduces P and u0 to actual xdim and udim
rP = zeros(xdim,1);
for di = 1:1:xdim
    rP(di) = P(di);
end

ru = zeros(udim,1);
for di = 1:1:udim
    ru(di) = u0(di);
end



while (Lerr1 < err1 && E2==1)


    % 2.1.- Computes physical position 'S' correspondant to u:
    
    [pNicp, pRicpA] = p1204_icpsPo (nAN, mA, ru, udim, 'closed');
    [N, Nicp, RicpA]= GEN_0908_N_upIPCs (pNicp, pRicpA, Degree, mA, ru, udim);
    [S, ~]          = GEN_0803_Xcoor_Tsplin_withN (mA, N, Nicp, RicpA, xdim);
    
    
    % 2.1.- Computes increment of u, by computing Jacobian.
    % In addition returns first derivatives of S about param. space
    
    rupr = ru ; % previous 'u' value
    
    r = (S-rP);
    
    [Ju, dS1] = t2806_Jpp (Degree, nAN, mA, r, ru, udim, xdim);
    

    fu = zeros(udim,1);
    for di = 1:1:udim
        
        Si = zeros(xdim,1);
        
        for xdi = 1:1:xdim
      	Si(xdi) = dS1(di,xdi);
        end

        fu(di) = dot( r , Si );
    end

    du = -inv(Ju)*fu;

    ru = ru + du ;
    

    
    % 2.3.- Recomputes errors:

    [err1, err2] = z2802_errors(P, S, dS1, xdim, udim);

    E2=0;
    for di=1:1:udim
        
        if err2(di)>Lerr2
        E2=1;
        break
        end
    end
    

    
    
    % 1.4.- In case still not converging, checks out of domain and not variation:
    
    if (Lerr1 < err1 || E2==1)
        
        % makes sure is ru goes back to domain in case is out:
        [ru] = x2803_Ulimits(KnotV, Usize, ru, udim);
        
     	% if there is no variation in 'ru' also converges!:
        nd = 12;
        [err1, E2] = z2804_checkSameU (err1, E2, ru, rupr, nd, udim);
        
    end
    

    
    % Control of No. loops:
    cL = cL + 1 ;
    if cL == cLLIM
        NOC = 1 ;
        break
    end
    
end



% 3.- Transfers to 'u' values from 'ru':

u = zeros(3,1);
for di=1:1:udim
   u(di) = ru(di); 
end



end

