function [ru, BO] = z2805_bouncingU (strdU, Lerrb, udim)

% RETURNS:
% BO: founcing flag, 0 is is not bouncing, 1 if so
% ru(udim): estimated u value if is a bouncing iteration (BO=1)

% INPUTS:
% strdU(np,udim): all Ucoor computed
% Lerrb: tolerance to consider bouncing (applied ot each other points)



% we arrange point in to arrays from: odd an even points oU, eU
% we assume bouncing happens in one parameter direction only
% Conditions for bouncing interation (in ieach array oU and eU):

% a.- tendency of error with respect the previous is decreasing
% b.- error is lower than certain tolerance, say Lerrb = 100*Lerr1
% c.- error is lower than error between arrays, say 2 to 10 times



np = size(strdU,1); % No. points

% 1.- CONSTRUCTS ARRAYS ODD AND EVEN:

oU = zeros(1,udim);
eU = zeros(1,udim);

conta = 0 ;
for po = 1:2:np
    
    if po<np
    
        conta = conta+1;
        for di = 1:1:udim
            oU(conta,di) = strdU(po,di);
            eU(conta,di) = strdU(po+1,di);
        end
    
    end
end

np = size(oU,1);

% 2.- DETECTS WHICH IS THE BOUNCING DIRECTION: 'bdir'
% the one with highes gap between oU and eU
% uses third point to decide

gap = zeros(udim,1);

for di=1:1:udim
    gap(di) = abs(oU(3,di) - eU(3,di));
end

Mgap = max(gap);

for di=1:1:udim
    if gap(di)==Mgap
        bdir = di;
        break
    end
end



% 3.- COMPUTES ERROR IN EACH ROW WITH RESPECT PREV. VALUE:
% also computes error bewtween arrays (or their difference)

errO = zeros(np-1,1);
errE = zeros(np-1,1);
diffOE = zeros(np-1,1);


for po = 1:1:np-1
    
    errO(po) = abs(oU(po+1,bdir) - oU(po,bdir)) / oU(po,bdir) ;
    errE(po) = abs(eU(po+1,bdir) - eU(po,bdir)) / eU(po,bdir) ;
    
    diffOE(po) = abs(oU(po,bdir) - eU(po,bdir));
    
end


% 4.- CHECK CONDITION 'a':
% a.- tendency of error with respect the previous is decreasing
% at least 75% of the poins need to comply this in each array

condA = 0 ; % condition a flag

contaO = 0; contaE = 0;

for po = 2:1:np-1
    
    if errO(po)<errO(po-1)
        contaO = contaO+1;
    end
   if errE(po)<errE(po-1)
        contaE = contaE+1;
   end
end


if contaO > 0.75*(np-1) && contaE > 0.75*(np-1)
    condA = 1;
end


% 5.- CHECK CONDITION 'b':
% % b.- error is lower than certain tolerance, say Lerrb = 100*Lerr1
% checks at the end locations:

condB = 0 ;

if errO(np-1) < Lerrb && errE(np-1) < Lerrb
    condB = 1 ;
end


% 6.- CHECK CONDITION 'c':
% c.- error is lower than error between arrays, say 2 times
% at least 75% of the poins need to comply this in each array

condC = 0 ;
Dfactor = 2 ;
contaD = 0 ;

for po = 1:1:np-1
    if diffOE(po)/errO(po) > Dfactor && diffOE(po)/errE(po) > Dfactor
        contaD = contaD+1;
    end
end

if contaD > 0.75*(np-1)
    condC = 1;
end


condC = 1 ; % condition C is cancelled so far




% 7.- IF IT COMPLIES ALL CONDITIONS => ESTIMATES ru AS INTERSECTION OF 2 LINES:
% uses last two points of each array for lines

BO = 0;

% preliminary ru values:
ru = zeros(udim,1);

for di = 1:1:udim
    
    ru(di) = (oU(np-1,di) + eU(np-1,di))/2;
    
end


if (condA==1 && condB==1) && condC==1
    
    BO = 1 ; % indicates bouncing iteration happened
    
    ar = 1*ones(2,1); br = 3*ones(2,1);
    at = 2*ones(2,1); bt = 4*ones(2,1);
    
    ar(2) = oU(np-2,bdir) ;
    br(2) = oU(np-1,bdir) ;
    
    at(2) = eU(np-2,bdir) ;
    bt(2) = eU(np-1,bdir) ;
    

    [IP, ~] = go2524_TwoSegmIntersec (ar,br, at,bt);
    
    ru(bdir) = IP(2); % updates releavant ucoordinate

end










end

