function [uresp, rF, lF] = ar3301_Ainters ...
    (ua,ub, ta,tb, ra, rb, rc, tamp, NSg, Rstd, Ra, Rb, dSmesh,Dspline, ...
    sDegree, sNcp, sKnotV, sUsize, sEP,sEw, Dumax, ULims, xdim, nd, DerC)

% RETURNS:
% uresp(2): intersection points between 2 circunf. in parameter space, 
% given their in radio defined in physical space

% rF: reject flag
% lF: at the limit flag = 1 if the computed ui is rigth on one of the
%     limits values ULims

% INPUTS:
% ua, ub: param. coordinates of origin of both circumferences
% ta, tb: ref. angle in parameter space for the searching arc (angle of arc bi-sectriz)
% tamp: semi-amplitude of search arc angle
% Ra, Rb: circ. radius in physical space from ra and rb 
% rc: reference of target point


% 1.- PRELIMINARIES TO LOOP:

% 1.1.- Transforms ta and tb accounting for current position of ra and rb 
% w.r.t. their pattern position:

[uaPatt] = ar3303_Rposi (ra, Rstd);
[ubPatt] = ar3303_Rposi (rb, Rstd);
[ucPatt] = ar3303_Rposi (rc, Rstd);


% tracking points: for debugging
pua = [ 0.9897 ; 0.1461];
pub = [ 0.9926 ; 0.2343];


anPatt = atan2((ubPatt(2)-uaPatt(2)),(ubPatt(1)-uaPatt(1))); % angle from a to b in pattern
anPara = atan2((ub(2)-ua(2)),(ub(1)-ua(1))); % curernt angle from a to b

Dangle = anPara - anPatt ; % angle diff. from parameter (current) to pattern (ideal)

ta = ta + Dangle ; % re-orientates
tb = tb + Dangle ;


% 1.2.- Selects which side of the vector ua-ub the point uc should be located:

% uses standard space:

vabStd = [(ubPatt - uaPatt) 0 ]; % vector a-b
vacStd = [(ucPatt - uaPatt) 0 ]; % vector a-c

% cross product vab x vac and 3rd component:
v = cross(vabStd, vacStd);
v3rd = v(3);


% 1.3.- Others:

udim = 2 ;

ui = zeros(1,udim);
uresp = zeros(1,udim);

tol = 0.5 ; % percentage of variation with respect latest value
err = tol + 1 ;

NLim = 12;
NL = 0 ;

uprev = ua;


% 2.- LOOPS UNTIL CONVERGE OR REACHES NLim No. LOOPS:

prNits = 0 ;



while err > tol && NL < NLim

    % 2.1.- Split into NS segments: computes NS+1 points of each cirfumference

    Dt = 2*tamp / NSg ;
  	tap = [ta-tamp : Dt : ta+tamp];
    tbp = [tb-tamp : Dt : tb+tamp];
    
    uaCf = zeros(NSg+1,udim); % coordinates of cirf. points
    ubCf = zeros(NSg+1,udim);

    for po = 1:1:NSg+1 
        [uaCf(po,:)] = dp3350_DPincrement (Ra,tap(po), ua, dSmesh,Dspline, sDegree, sNcp, sKnotV, sUsize, sEP,sEw, Dumax, ULims, xdim, nd, DerC);  
        [ubCf(po,:)] = dp3350_DPincrement (Rb,tbp(po), ub, dSmesh,Dspline, sDegree, sNcp, sKnotV, sUsize, sEP,sEw, Dumax, ULims, xdim, nd, DerC);
    end

    %{
	if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) && (round(ub(1),4)==pub(1) && round(ub(2),4)==pub(2))
        NL
        uaCf
        ubCf
        %ub
        %[ubCf , tbp']
    end
    %}
    
    % 2.2.- Computes intersection:

    Nits = 0 ;
    its = zeros(1,2);
    
    for sa = 1:1:NSg

        ua1 = [uaCf(sa,1)    uaCf(sa,2)];
        ua2 = [uaCf(sa+1,1)  uaCf(sa+1,2)];

        for sb = 1:1:NSg

            ub1 = [ubCf(sb,1)    ubCf(sb,2)];
            ub2 = [ubCf(sb+1,1)  ubCf(sb+1,2)];

            [ui, iF, pF] = go2526_TwoLinIntersec (ua1,ua2, ub1,ub2, 'closed',nd);

            % for parallel segments checks if one of the points of ub1 is between ua1
            if pF == 1
                [ui, iF1] = go2517_ParalelSegmInters2D (ua1, ua2, ub1, ub2, nd);
                iF = iF1 * ones(udim,1) ;
            end
            
            if sum(iF)==2
                
                Nits = Nits + 1 ;
                its(Nits,1) = ui(1); its(Nits,2) = ui(2);
            end
        end

    end

    its = round(its,nd);

    % 2.3.- Checks if found intersection in the previous loop but not in the current loop:
    % that happens coz the angle amplitude 'tamp' is drastically reduced
    % and does not find a trim
    
    prNSg = NSg;
    
	%{
	if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) && (round(ub(1),4)==pub(1) && round(ub(2),4)==pub(2))
        F_LOOOP = NL
    end
	%}
    
    if prNits > 0 && Nits == 0      % CASE A : lost the intersection that it had calculated previously

     	%{
        if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) && (round(ub(1),4)==pub(1) && round(ub(2),4)==pub(2))
            CASE_A = 1
        end
        %}
        
        tamp = tamp *2 ;

    else
        
    % 2.4.- In other 'normal' cases: updates the searching angle:

        if Nits > 0 % in case found intersection within both lines 
        
        %{
        if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) && (round(ub(1),4)==pub(1) && round(ub(2),4)==pub(2))
            Nits
            v3rd
            CASE_BorC = 1
        end
        %}
            
        % - if intersection is on the correct side narrows the angle & updates nominal angles
        % compares with the standard space side

            selIts = 0 ;

            v3s = zeros(Nits,1); % stores all 3rd components of
            for i = 1:1:Nits

                uits = [its(i,1); its(i,2)];

                % vector a-b
                vab = [(ub   - ua); 0 ]; % vector a-b
                vac = [(uits - ua); 0 ]; % vector a-c

            	% cross product vab x vac and 3rd component:
             	vits = cross(vab, vac);
             	v3s(i) = vits(3);
                    
            	% select the point in the correct side
                if sign(v3s(i)) ==  sign(v3rd) 
                    selIts = i ;
                end
                
            	%{
                if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) && (round(ub(1),4)==pub(1) && round(ub(2),4)==pub(2))
                    i
                    uits
                    v3s
                    selIts
                end
                %}
                                
            end

            
            % checks in case did not find correct positioned location:
            
            if selIts == 0

                
            	%{
                if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) && (round(ub(1),4)==pub(1) && round(ub(2),4)==pub(2))
                    non_corr_pos = 1
                end
                %}
                
                if Nits == 1

                % - single location: checks if 
                %   wrong side but very close => considers the point        C A N C E L L E D   !!

                    uits = [its(1,1) its(1,2)];
                    [dist,~] = go2527_distPtoRecta (uits, ua, [ub-ua], udim);

                    toldist = norm(ua-ub)/1000;
                    
                    if dist < toldist 
                        % selIts = 1 ;
                        
                        %{
                        if (round(ua(1),4)==0.9877 && round(ua(2),4)==0.0135) && (round(ub(1),4)==0.9928 && round(ub(2),4)==0.0893)
                            
                            toldist 
                            dist
                            
                            non_corr_pos_SEL1 = 1
                        end
                        %}
                        
                    end

                elseif Nits == 2

                    % double intersection point
                    % selects the one closest to the line ua-ub

                    dist = zeros(2,1);
                    dmin = 999999999;
                    selIts = 1 ;
                    for i = 1:1:2
                        
                        uits = [its(i,1) its(i,2)];
                        [dist(i),~] = go2527_distPtoRecta (uits, ua, [ub-ua], udim);
                        
                        if dist(i)<dmin
                            dmin = dist(i);
                            selIts = i ;
                        end
                    end
                    
                    %{
                	if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) && (round(ub(1),4)==pub(1) && round(ub(2),4)==pub(2))
                        non_corr_pos_SEL__2 = 1
                        uits
                        dist
                    end
                  	%}
                    
                end
            end
            
            
            if selIts>0
            
                % CASE B
                
                
                
                ui = zeros(udim,1);
                
                ui(1,1) = its(selIts,1) ;  ui(2,1) = its(selIts,2) ;

                tamp = tamp / (NSg-1) ;
                ta = atan2((ui(2)-ua(2)),(ui(1)-ua(1)));
                tb = atan2((ui(2)-ub(2)),(ui(1)-ub(1)));
 
                err = 100 * norm(ui - uprev) / norm(uprev);
                uprev = ui ;
                uresp = ui ;

                
                %{
            	if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) && (round(ub(1),4)==pub(1) && round(ub(2),4)==pub(2))
                    CASE_B = 1
                end
                %}
                

                % only updates iFpr is reaches intersection
                prNits = Nits ;
                
            else
                
                % CASE C:
                
                % if the encountered intersection is n the wrong side
                % threats as no intersection found
                
             	%{
                if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) && (round(ub(1),4)==pub(1) && round(ub(2),4)==pub(2))
                    CASE_C = 1
                end
                %}
                
                tamp = tamp * 2 ;
     
            end
            
        elseif Nits == 0 % in case intersection is not in both => increase the angle and NSg
        
        % - CASE D:

            %{
            if (round(ua(1),4)==pua(1) && round(ua(2),4)==pua(2)) && (round(ub(1),4)==pub(1) && round(ub(2),4)==pub(2))
                CASE_D = 1
            end
            %}
        
            tamp = tamp * 2 ;
            NSg = NSg*2 ;
            err = tol + 1;

        end
    end
    
  
    % 2.6.- In case completes a circumferences:
    % estimates a point between ua and ub
    
    if tamp > 6.3

        % intersection of ua and ub with their circumf.:
        
        Na = 0;
        Nb = 0 ;
        for s = 1:1:prNSg
        
            ua1 = [uaCf(s,1)  ;  uaCf(s,2)];
            ua2 = [uaCf(s+1,1) ; uaCf(s+1,2)];
            
            [ui, iF, ~] = go2526_TwoLinIntersec (ua1,ua2, ua,ub,'closed',nd);
            if sum(iF)>0
                Na = Na + 1 ;
                Aits(Na,1) = ui(1); Aits(Na,2) = ui(2);
            end
            
            ub1 = [ubCf(s,1) ;   ubCf(s,2)];
            ub2 = [ubCf(s+1,1);  ubCf(s+1,2)];
            
            [ui, iF, ~] = go2526_TwoLinIntersec (ub1,ub2, ua,ub,'closed',nd);
            if sum(iF)>0
                Nb = Nb + 1 ;
                Bits(Nb,1) = ui(1); Bits(Nb,2) = ui(2);
            end
        end
        
        % selects the intersection closest to the opposite point
        
        if Na==1
            uia = Aits;
        else
            
            dmin = norm(ub-[Aits(1,1);Aits(1,2)]);
            uia = [Aits(1,1); Aits(1,2)];
            
            for i = 1:1:Na
            
                dist = norm(ub-[Aits(i,1);Aits(i,2)]);
                if dist < dmin
                    dmin = dist;
                    uia = [Aits(i,1);Aits(i,2)];
                end
            end
        end
        
        
        if Nb==1
            uib = Bits;
        else
            
            dmin = norm(ua-[Bits(1,1); Bits(1,2)]);
            uib = [Bits(1,1);Bits(1,2)];
            
            for i = 1:1:Nb
            
                dist = norm(ua-[Bits(i,1); Bits(i,2)]);
                if dist < dmin
                    dmin = dist;
                    uib = [Bits(i,1); Bits(i,2)];
                end
            end
        end
        
        % takes the average of uia-uib:
        
        uresp = (uia + uib)/2;
        uresp = round(uresp,nd) ;

        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    	%{
        ua
        ub
        uaCf
        ubCf
        uresp
        %}
        
        err = 0 ;
        
    end
    

    % 2.5.- Add NL's:

    NL = NL + 1 ;
    
    
end





% 3.- FLAGS:

% 3.1.- Rejects if does not find intersection:

if sum(iF)<2
    rF = 1 ;
else
    rF = 0 ;
end

% 3.2.- Limit flag:

lF = 0 ;

for di = 1:1:udim
    if round(ui(di),nd) == round(ULims(di,1),nd) ...
  	|| round(ui(di),nd) == round(ULims(di,2),nd)
        lF = 1 ;
    end
end



end

