function [cE2, cDu, cN3d,crF] = fs0721_ErrL (Spatches, Ntc, tcR, agCurv, Ns, Cs, tolA, Xtol, DUtol, h)

% RETURNS: for each triangle of each Tskin:
% cE2(Ntc,1): relative error to reduce below tolerance
% cDu(Ntc,1): required distance in curve param space to reduce below tolerance
% crF(Ntc,1): curve elemetn refinement flag, turns to 1 if needs refinement

% INPUTS:

cudim = 1 ;
pudim = 3 ;
xdim = 3 ;

cE2 = zeros(Ntc,1);
cDu = zeros(Ntc,1);
cN3d = zeros(Ntc,1);
crF = zeros(Ntc,1);

for j = 1:1:Ntc

    cR = tcR(j,1);
    
    % 1.- EXTRACTS DATA FROM RELEVANT PATCH:
    
    pR = tcR(j,2);

    pDegree = Spatches(pR,1).INS(1).mDegree ;
    pNcp    = Spatches(pR,1).INS(1).mNcp ;
    pKnotV  = Spatches(pR,1).INS(1).mKnotV ;
    pUsize  = Spatches(pR,1).INS(1).mUsize;
    pP      = Spatches(pR,1).INS(1).mP ;
    pw      = Spatches(pR,1).INS(1).mw ;
    pCD     = zeros(pudim,1);
    
    
    % param. limits and admisible error in absolute value
    
    pULims = zeros(xdim,2);
    Udiag = 0 ;
    
    for di = 1:1:pudim
        
        pULims(di,1) = pKnotV(di,1);
        pULims(di,2) = pKnotV(di,pUsize(di));
        
        Udiag = Udiag + (pULims(di,2) - pULims(di,1))^2 ;
        
    end
    Udiag = Udiag^0.5;
    
    Eadm = Udiag*tolA / 100;
    
    
    % 2.- EXTRACTS DATA FROM CURVE:
 
    NV = Ns;

    Uv = zeros(NV,cudim);  
    Uv(:,1) = Cs(cR,1:NV);

    cDegree = agCurv(cR,1).Degree ;
    cNcp    = agCurv(cR,1).Ncp ;
    cKnotV  = agCurv(cR,1).KnotV ;
    cUsize  = agCurv(cR,1).Usize ;
    cP      = agCurv(cR,1).P ;
    cw      = agCurv(cR,1).w ;
    cULims = [ cKnotV(1,1)  cKnotV(1,cUsize)]; 

    % 3.- COMPUTES ERROR AND DISTANCE to 
	% reduce error < tolS in patch param. space.

    for e = 1:1:NV

        
        %s = (Uv(e+1,1) - Uv(e,1))/2; % step between nodes 
        
        % 3.1.- Centre point:

        Umid = Uv(e,1) ; % + Uv(e+1,1))/2;


        % 3.2.- Computes points for derivative:

        [Ai] = fs0746_Cdi ...
        (pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, cKnotV, cNcp, cDegree, cw, cP, cULims, Umid, h, xdim, Xtol, DUtol);



        % 3.3.- Relative error and req'd distance bt'n nodes to reduce it to tolS:

        s = 1 ; %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        [E2c, smax, nC3d] = fs0745_Err2C (s, Ai, h, Eadm, xdim, cR);

        %{
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        if cR==6 
            
            figure(88)
            scatter3(Ai(:,1), Ai(:,2), Ai(:,3), 'b'); hold on

            figure(91)
            plot([0 1], [0 0], 'k'); hold on
            scatter(Umid,nC3d,'r'); hold on
            
            
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %}
        
        if smax > 1
           smax = 1 ; 
        end
        
        cE2(j,e) = E2c ;
        cDu(j,e) = smax ;
        cN3d(j,e) = nC3d ;
        
        crF(j,e) = 0 ;
        if Eadm < E2c
            crF(j,e) = 1 ;
        end

    end

end




end

