function [Ncv, cUv, cXv, cAv] = fs_0760_fit2conto (Spatches, TNcu, agCurv, Ntc, tcR, Ns, tolA, Xtol, DUtol)

% RETURNS:
% Nodes of contour curves of trimming surface. with this nodes the
% quadratic interpolation has an error less than the tolerance

% Ncv(TNcu,1): No. nodes per curve, include initial and final, and
% mid-nodes for each curve !
% cUv, cXv, cAv: coordiantes in curve param., physical and patch param.

% Note: those curves that are not trimming are just filled with zeros

% INPUTS:
% TNcu: total No.curves
% Ns: No. sample points


pudim = 3 ;
xdim = 3 ;


% 1.- SAMPLE POINTS:

Cs = zeros(TNcu,Ns); % coordinates
h = 1 /(Ns-1); % uses same step for sampling and for derivative estimation

for j = 1:1:Ntc
    
    cR = tcR(j,1);
    
    cKnotV  = agCurv(cR,1).KnotV ;
    cUsize  = agCurv(cR,1).Usize ;

    cULims = [ cKnotV(1,1)  cKnotV(1,cUsize)]; 
    uacc = cULims(1);
    
    for v = 1:1:Ns-1
        Cs(cR,v) = uacc ;
        uacc = uacc + h ;
    end
    
    Cs(cR,v+1) = cULims(2) ;
    
end

% 2.- ERRORS AND MAX. ALLOWABLE NODAL STEP IN EACH CURVE:

[E2c, cDu, cN3d, ~] = fs0721_ErrL (Spatches, Ntc, tcR, agCurv, Ns, Cs, tolA, Xtol, DUtol, h);


% 3.- NODES INSERTION: inserts all nodes, inclueing mid-nodes of the quadratic interpolation

[Ncv, cUv] = fs0748_iNodes (TNcu, Ntc, tcR, Cs, Ns, cDu);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
cR = 6 ;

figure(11)
plot(cUv(cR,:),cN3d(cR,:),'r'); hold on

figure(12)
plot(cUv(cR,:),cDu(cR,:),'b'); hold on

scatter(cUv(cR,1:Ncv(cR,1)), zeros(Ncv(cR,1),1), 'b');

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 4.- COMPUTES X AND A COORDS:

cXv = zeros(TNcu,1, xdim);
cAv = zeros(TNcu,1, pudim);

for j = 1:1:Ntc
    
    cR = tcR(j,1);
    pR = tcR(j,2);
    
	cDegree = agCurv(cR,1).Degree ;
    cNcp    = agCurv(cR,1).Ncp ;
    cKnotV  = agCurv(cR,1).KnotV ;
    cP      = agCurv(cR,1).P ;
    cw      = agCurv(cR,1).w ;
    
	pDegree = Spatches(pR,1).INS(1).mDegree ;
    pNcp    = Spatches(pR,1).INS(1).mNcp ;
    pKnotV  = Spatches(pR,1).INS(1).mKnotV ;
    pUsize  = Spatches(pR,1).INS(1).mUsize;
    pP      = Spatches(pR,1).INS(1).mP ;
    pw      = Spatches(pR,1).INS(1).mw ;
    pCD     = zeros(pudim,1);
    
    cX = zeros(Ncv(cR,1),xdim);


    
    % physical coordinates:
    
    for v = 1:1:Ncv(cR,1)
        
        [X] = b2601_BNURBS_Xcoord (cUv(cR,v), cKnotV, cNcp,cDegree, cNcp+cDegree+1 , cw,cP, 1, xdim);
        
        cX(v,:) = X(:,1);
        cXv(cR,v,:) = X(:,1);
    end
    
    
    % patch param. coordinates:
    
    [cA] = v2861b_PPvol (cX, Ncv(cR,1), pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, xdim, Xtol, DUtol);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %{
    figure(88)
    plot3(cA(:,1), cA(:,2),cA(:,3),'k'); hold on
    scatter3(cA(:,1), cA(:,2),cA(:,3),'r'); hold on
    
    text( cA(2,1), cA(2,2), cA(2,3), num2str(cR)); hold on
    
    axis equal
    %}
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    for v = 1:1:Ncv(cR,1)
        cAv(cR,v,:) = cA(v,:);
    end
    
    
end

end

