function [aP] = go2503_AbateHpoints (n, P)

% RETURNS:
% aP(n,xdim): cloud of points abatidos over horizontal plane
% the horizontal plane passes through the point closest to the plane zero

% note: if plane is already horizontal, detects it and just copy to aP.

% INPUTS:
% n: No. points
% P(n,xdim): points coordinates

xdim = 3 ;
aP = zeros(n,xdim);

% 1.- Detects point closest to the origin:

op = 1;
dmin = abs(P(1,3));

for i = 2:1:n
    
    dtrial = abs(P(i,3));
    
    if dtrial < dmin
        op = i;
        dmin = dtrial;
    end
end


% 2.- Selects three points non-colinear:

a = zeros(xdim,1);
b = zeros(xdim,1);
c = zeros(xdim,1);

for di=1:1:xdim
    a(di) = P(1,di);
    b(di) = P(2,di);
end

pfound=0;

for i=n:-1:3 % finds c such that is not colinear
    
    for di=1:1:xdim
        c(di) = P(i,di);
    end
    
    ON1 = go2520_PonSegment(c, a,b, xdim, 0.05);
    ON2 = go2520_PonSegment(a, c,b, xdim, 0.05);
    ON3 = go2520_PonSegment(b, a,c, xdim, 0.05);

    
    if (ON1==0 && ON2==0) && ON3==0
        pfound=1;
        break
    end
end

if pfound==0
    disp('cloud of points are co-linear');
end


% 3.- Plane equation:

[A,B,C,D] = go2506_Plane3P (a,b,c);


% 4.- Detects if is horizontal plane or not:

if (abs(A)+abs(B)) < 0.000001 
    
    % 4.1.- For horizontal plane just copy:
    
    for i = 1:1:n
    for di=1:1:xdim
        aP(i,di) = P(i,di);
    end
    end
    
else
    
    % 4.2.- For non-horizontal plane rotates about horizontal line,
    % that horizontal line is obtained as intersection of actual plane and
    % horizontal plane that passes through point 'op'
    
    % finds horizontal line: y = ax + b, and its Pr & vr
    
    zo = P(op,3); % level of the horizontal plane
    
    a = -A/B;
    b = -(C*zo+D)/B;
    
   	Pr = zeros(xdim,1); 
    vr = zeros(xdim,1); 
    
    for di = 1:1:xdim
     	Pr(di) = P(op,di);
    end
    
    vr(1) = Pr(1)+10;
    vr(2) = a*vr(1) + b;
    vr(3) = 0;
    
    Pi = zeros(xdim,1); % initial point 'i' coordinates
    Qi = zeros(xdim,1); % proyection of 'i' on the line
    Ai = zeros(xdim,1); % aux. point
    Pip= zeros(xdim,1); % point 'i' abatido
    
   	x = zeros(2,1);
   	xu= zeros(2,1);
   	u = zeros(2,1);
    
    for i = 1:1:n
    if i~=op
        
        for di = 1:1:xdim
            Pi(di) = P(i,di);
            Ai(di) = P(i,di);
        end
        Ai(3) = Ai(3)*2;
        
        % distance to line
        [dist, d] = go2527_distPtoRecta (Pi, Pr, vr, xdim);
        
        % point on line Qi:
        for di = 1:1:xdim
            Qi(di) = Pi(di) + d(di);
        end
            
        % vertical plane passing by: Pi, Qi
        % and intersection 'v' line with horizontal plane: y = av x + bv
        [Av,Bv,Cv,Dv] = go2506_Plane3P (Ai,Pi,Qi);
        
        
        
        av = -Av/Bv;
        bv = -(Cv*zo+Dv)/Bv;
    

      
        
        % intersection btw'n both lines and line 'v' versor (u):
  
        x(1) = (bv-b) / (a-av);
        x(2) = a*x(1) + b ;
        
       
        xu(1) = x(1)+10;
        xu(2) = av*xu(1) + bv;
        
       
        
        u(1) = xu(1) - x(1);
        u(2) = xu(2) - x(2);
        
        u = u/norm(u);
        
        % add distance in line 'v' direction
        
        s = sign(Pi(3)-zo);
   
        
        Pip(1) = Qi(1) + s*u(1)*dist;
        Pip(2) = Qi(2) + s*u(2)*dist;
        Pip(3) = Qi(3);
        
        
        
  	end
    end
    
    
end

    
end






