function [pNn, pXv, pAv] = im0145c_PreNodes (KnotV, Ncp, Degree,Usize, Ew,EP, xdim, udim)

% RETURNS: 
% preliminary location of tet's nodes without considering trimming surfaces

% the constraint is: 2 G = C
% where:
% G: No. integration points
% C: No. control points 

% 0.- PRELIMINARIES:

t = 6 ; % No. tet's per cell
tol = 0.05 ; % if NC is 1+d times greater than 2 NG => increases No. cells (g)

[V, Vsize] = GEN_18_UnonVoid (KnotV, Usize, Degree, udim); % knots of non-void spans


% 1.- COMPUTES No. CELLS IN EACH DIRECTION: 

g = zeros(xdim,1);
ge = zeros(xdim,1); % exact values

gmin = 2 ; % minimum No. cells in each direction

% 1.1.- Initial estimation:

for di = 1:1:xdim
    
    ge(di,1) = Ncp(di)/(2*t)^(1/3);
    g(di,1)  = round(ge(di,1),0);
    
    if g(di,1) < gmin
        ge(di,1) = gmin;
        g(di,1)  = gmin;  
    end
    
end


% 1.2.- Corrects if min. No. estimated tets is not achieved:
% Increases to next integer, the closest one.

NG = t*g(1)*g(2)*g(3);      % estimates No. tets
NC = Ncp(1)*Ncp(2)*Ncp(3);  % No. cp's
    
r = [ 2 3 ; 3 1 ; 1 2 ];

d = (NC-2*NG)/NC ;

while (d > tol)
    
    % increments to next integer:
    
    Dg = zeros(udim,2); % distance and dir ref.
    
    for di = 1:1:udim
        Dg(di,1) = GEN_41_roof(ge(di)) - ge(di) ;
        Dg(di,2) = di ;
    end
    
    % finds closer to next integer:
    
    Dg = sortrows(Dg,1);
    
    ds = 0 ;
    for di = 1:1:udim
        if 0.5 < Dg(di,1) && Dg(di,1) > 0 
            ds = Dg(di,2); % selected direction
            break
        end
    end
    

    % increases tat direction:
    
    g(ds,1)  = GEN_41_roof(ge(ds));
    ge(ds,1) = GEN_41_roof(ge(ds));
    
    NG = t*g(1)*g(2)*g(3); 
    d = (NC-2*NG)/NC ;
end

%{
if 2*NG < NC % NT does not reach min => adds the one where requeied No. is closer to integer
        
	dti = zeros(udim,3); % dist. to next integer, xroof, and direction ref
        
    for di = 1:1:udim
            
        dv = 2*t;
        for di2 = 1:1:2
            dv = dv * s(r(di,di2));
        end
        
        x = NC/dv;
        xroof = GEN_41_roof (x);
           
        dti(di,1) = xroof - x ;
        dti(di,2) = xroof ;
        dti(di,3) = di ;
    end
     
    dti = sortrows(dti,1);

    sd = dti(1,3); % selected direction
    
    s(sd,1) = dti(1,2);

end
%}
    
    
% 2.- WORKS IN EACH DIRECTION: 
% Obtains xi-coord's required in each direction : u

alf = zeros(xdim,1); 
Nalf = zeros(xdim,1); 

AVt = zeros(xdim,1,2); % xi => alfa  transformation

u = zeros(xdim,1);
    
for di = 1:1:xdim
    
    % 2.2.- Alfa-coordinates required (equally spaced): alf
    
  	Nalf(di,1) = g(di,1) + 1 ;
    
    stp = 1/g(di,1); % step in param. space
    
    acc = 0 ;
    for i = 1:1:Nalf(di,1)
        
        if i==Nalf(di,1)
           acc = 1 ; 
        end
        alf(di,i)=acc;
        
        acc = acc + stp;
    end 
    
    
    % 2.3.- Matrix with alfa => xi transformation: AVt:
    
	stp = 1 / (Vsize(di)-1);
    
    acc1 = 0 ;
    acc2 = stp ;
    for i = 1:1:Vsize(di)-1
        
        if i == Vsize(di)-1
            acc1 = 1-stp;
            acc2 = 1 ;
        end
        
        AVt(di,i,1) = acc1 ;
        AVt(di,i,2) = acc2 ;
        
        acc1 = acc1 + stp;
        acc2 = acc2 + stp ;
    end

    
    % 2.4.- Computes xi-coord's required: u
    
    for i = 1:1:Nalf(di)
        
        a = alf(di,i); % alfa coord

        % finds segment of AVt where a lies:
        
        for j = 1:1:Vsize(di)-1 
            
            if AVt(di,j,1)<= a && a <= AVt(di,j,2)
                sg = j;
                break
            end
        end
        
        % interpolares within sg:
        A = AVt(di,sg,2) - AVt(di,sg,1); % alfa-segment lenght
        Da = a - AVt(di,sg,1); % alfa-increment
        X = V(di,sg+1) - V(di,sg) ; % xi-segment increment
        
        Dx = Da/A * X ;
        
        % allocates:
        u(di,i) = V(di,sg) + Dx ;
 
    end
end

nu = Nalf;


% 3.- TRANSFERS TO THE NET OF NODES:

pNn = 1;
for di = 1:1:xdim
    pNn = pNn * nu(di);
end

pXv = zeros(pNn, xdim);
pAv = zeros(pNn, udim);

a = zeros(udim,1);

conta = 0 ;
for i = 1:1:nu(1)
for j = 1:1:nu(2)
for k = 1:1:nu(3)
    
    conta = conta + 1 ;
    
    a(1,1) = u(1,i);
    a(2,1) = u(2,j);
    a(3,1) = u(3,k);
    
    pAv(conta,:) = a(:,1);
    
    x = b2601_BNURBS_Xcoord (a, KnotV, Ncp, Degree,Usize, Ew,EP, udim, xdim);
    pXv(conta,:) = x(:,1);
    
end 
end
end



%{ 
% plots

for di = 1:1:xdim
    
    pt = zeros(Vsize(di),2);
    
    for i = 1:1:Vsize(di)-1
        
        pt(i,1) = V(di,i);
        pt(i,2) = AVt(di,i,1);
        
    end
    
    pt(i+1,1) = V(di,i+1);
    pt(i+1,2) = AVt(di,i,2);
    
    figure(di)
    plot(pt(:,1), pt(:,2),'b'); hold on
    axis equal
    
end

%}





end

