function [TTR] = im0149_TTR (pR, pnbe, pRbe, Nbe, rTskin, Nts, tsR, TNcuE)


% RETURNS:
% TTR: trimming-triangulation features

% TTR contains triangles of all trimming Eskin in a single structure.
% ASSUMES TRIANGLES ARE LINEAR, i.e. DOES NOT CONSIDER MID-VTX

xdim = 3 ;
udim = 3 ;

TTR = struct('tNE',0, 'tNV',0, 'tNL', 0 , ...
            'tTVcon', [], 'tLVcon',[], 'tVLcon',[], 'tTLcon',[], 'tVTcon',[], ...
            'tAv',[], 'tXv',[], 'tANv',[], 'tANe',[], 'tXNv',[], 'tXNe',[], ...
            'tSR',[], 'tCR',[] , 'tUs',[], 'tUc', [] , 'tNS',[], 'tNC', []  );

% tSR is the skin reference where the vertex belongs to
% tER is the edge curve reference where the vertex belongs to

Acn = zeros(xdim,1); % candidate
Aex = zeros(xdim,1); % existing

tAv = zeros(1,udim);
tXv = zeros(1,xdim);

tANv = zeros(1,udim);
tANe = zeros(1,xdim);

tXNv = zeros(1,udim);
tXNe = zeros(1,xdim);

tNE = 0 ;
tNV = 0 ;

tTVcon = zeros(1,3);

tNS = zeros(1,1); % No. surfaces where the node lies in
tNC = zeros(1,1); % No. curves where the node lies in

tSR = zeros(1,1); % surfaces ref's where the node lies in
tCR = zeros(1,1); % curves ref's where the node lies in

tUs = zeros(1,Nbe, 2); % point, surface, coord's
tUc = zeros(1,TNcuE,1); % point, curve, coord's


% conn. matrix: local node (sR,nR)=> TTR node ref (for repeated nodes and
% their relation with TTR elements).
% 'Local to global' node connectivity

LGcon = zeros(pnbe,1);

for s = 1:1:pnbe  
    
    % surface ref. and patches:
    
    sR = pRbe(s);
    Pa = rTskin(sR).mPa ;


	% Slave patch? slpF = 1 if yes
        
	slpF = 0 ;
	if Pa(2)==pR
        slpF = 1 ;
    end
        
    TVcon = rTskin(sR,1).mTVcon;

    % 1.- NODES:

        % 1.1.- First node:

        if tNV == 0 % (no elements added coz it is the first shaping surface)

            tNV = tNV + 1 ;

            [tAv, tXv, tUs, tUc, tANv, tXNv, tNS, tSR, tNC, tCR] = im0148_NNttr ...
            (sR, tNV, 1, tAv, tXv, tUs, tUc, tANv, tXNv, tNS, tSR, tNC, tCR, rTskin(sR), slpF);

            LGcon(sR,1) = tNV;
        end



        % 1.2.- Checks the rest of the nodes (if not checked previusly) repeated:

        for i = 1:1:rTskin(sR).mNV

            % candidate:

            if slpF == 0
                Acn(:,1) = rTskin(sR).mAv(i,:);
            elseif slpF == 1
                Acn(:,1) = rTskin(sR).mAvS(i,:);
            end
    
            flagRN = 0; % 'repeated node' flag


            % compares with already stored:

            for j = 1:1:tNV

                Aex(:,1) = tAv(j,:);
                CO = go2542_coincidentP (Acn, Aex, xdim, 3);

                if CO==1
                    flagRN = 1;
                    break
                end
            end


            % case it is new node:

            if flagRN == 0

                tNV = tNV + 1 ;

                [tAv, tXv, tUs, tUc, tANv, tXNv, tNS, tSR, tNC, tCR] = im0148_NNttr ...
                (sR, tNV, i, tAv, tXv, tUs, tUc, tANv, tXNv, tNS, tSR, tNC, tCR, rTskin(sR), slpF);

                LGcon(sR,i) = tNV;


            % case it is repeated node:  ith node of rEskin coincides with jth stored node:

            elseif flagRN == 1

                [tANv, tXNv, tNS, tSR, tUs, tNC, tCR, tUc] = im0153_ENttr ...
                (rTskin(sR), tANv, tXNv, tNS, tSR, tUs, tNC, tCR, tUc, i,j, sR, slpF);
                
                % node local to global conn. matrix:
                LGcon(sR,i) = j;

            end

        end


        % 2.- ELEMENTS: they are directly passed

        for e = 1:1:rTskin(sR).mNE

            eR = tNE + e;

            if slpF==0
                
                tANe(eR,:) = rTskin(sR).mANe(e,:);
                tXNe(eR,:) = rTskin(sR).mXNe(e,:);
                
            elseif slpF==1
                
                tANe(eR,:) = rTskin(sR).mANeS(e,:);
                tXNe(eR,:) = - rTskin(sR).mXNe(e,:);
                
            end
            


            for i = 1:1:3
                nlr = TVcon(e,i);
                ngr = LGcon(sR,nlr);
                tTVcon(eR,i) = ngr ;
            end

        end

        tNE = tNE + rTskin(sR).mNE;
      
     
end


%{
% add rows to curves matrices to match tNV:
nr = size(tNC,1);

if nr < tNV
    
    nc = size(tNC,2);
    tNC = [tNC ; zeros(tNV-nr,nc)]; 
    
	nc = size(tCR,2);
    tCR = [tCR; zeros(tNV-nr,nc)]; 
    
end
%}


% 3.- OTHER CONN. MATRICES:

[tLVcon, tVLcon, tTLcon, tVTcon] = tr1347_otherConn (0, 0, tNE, tNV, tTVcon, tAv);

tNL = size(tLVcon,1);


% 4.- ALLOCATES:

TTR.tNE = tNE;
TTR.tNV = tNV;
TTR.tNL = tNL;

TTR.tTVcon = tTVcon;
TTR.tLVcon = tLVcon;
TTR.tVLcon = tVLcon;
TTR.tTLcon = tTLcon;
TTR.tVTcon = tVTcon;

TTR.tAv = tAv;
TTR.tXv = tXv;

TTR.tANv = tANv;
TTR.tANe = tANe;
TTR.tXNv = tXNv;
TTR.tXNe = tXNe;

TTR.tSR = tSR;
TTR.tCR = tCR;

TTR.tNS = tNS;
TTR.tNC = tNC;

TTR.tUs = tUs;
TTR.tUc = tUc;


%[[1:tNV]' tNS tSR]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{

NLI = size(tLVcon,1);
dim = 3 ;
Mcolr = 'k';
Ncolr = 'r';

erF = 1 ; % elems. ref. flag
vrF = 1 ; % vertex ref. flag

nvF = 1 ; % normal to vtx flag
neF = 1 ; % normal to elem flag

fr = 1 ; %%%%%%%%%%%%%%%%%%%%%%%%%

sc = 100 ;

Nvx = tXNv ;
Nel = tXNe ;

pt1366_pTriLinMesh_nD ...
(tXv, NLI, tLVcon, tTVcon, dim, fr, Mcolr, nvF, tNV, Nvx, neF, tNE, Nel, sc, Ncolr, 0, vrF, 0);


scatter3(tXv(:,1), tXv(:,2), tXv(:,3), 'r'); hold on


fr = 2 ; %%%%%%%%%%%%%%%%%%%%%%%%%

sc = 0.1 ; 

Nvx = tANv ;
Nel = tANe ;

pt1366_pTriLinMesh_nD ...
(tAv, NLI, tLVcon, tTVcon, dim, fr, Mcolr, nvF, tNV, Nvx, neF, tNE, Nel, sc, Ncolr, 0, vrF, erF);

scatter3(tAv(:,1), tAv(:,2), tAv(:,3), 'r'); hold on

if gg==0
end

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


end

