function [] = r3002_draw ...
    (Spatches, Eskin, NpaON, prON, Stp, sc, ...
    xsF, usF, gsF, bsF, cponF, cpofF, cprF, cnetF, eonF, erF, nrF, beF, udfF, ksF, gpF, gprF, vmF)

% ACTION:
% Draws selected entities in physical / parameter space

% INPUTS:

% NpaON, prON(NpaON): No. actived patches and their references
% Stp: instance to represent
% sc: scale of deformed shape

% flags: equal to 1 if active, otherwise = 0

% xsF - usF:        physical - parameter space
% gsF, bsF:         gross - bounded skins
% cponF, cpofF, cprF, cnetF: control points ON, OFF, refs, control net
% eonF, erF, nrF:   elements, refs, nodes refs
% beF:              boundary entities
% udfF:             undeformed shape
% ksF, gpF, gprF:   knot spans





xdim = 3;
udim = 3;

for pa = 1:1:NpaON
    
    % 1.- PRELIMINARY BASIC DATA EXTRACTION:
    
    % patch reference
    paR = prON(pa,1); 
    

    % patch NURBS features:
    
	KnotV   = Spatches(paR,1).INS(Stp,1).mKnotV;
	Ncp     = Spatches(paR,1).INS(Stp,1).mNcp;
	Degree  = Spatches(paR,1).INS(Stp,1).mDegree;
 	Usize   = Spatches(paR,1).INS(Stp,1).mUsize;
	EP = Spatches(paR,1).INS(Stp,1).mEP;
 	Ew = Spatches(paR,1).INS(Stp,1).mEw;
    

    % 2.- PATCH SKINS:

	if gsF == 1
        
        % 2.1.- Gross skin: 
        
        Ns    = Spatches(paR,1).Ngsk;
        Skins = Spatches(paR,1).mGskin;
            
        r3011_gSkins (Skins, Ns, xsF, usF, xdim);
     
	elseif bsF == 1
          
        % 2.2.- Bounded patches:
        
        r3014_bSkins (Spatches, paR, Stp, xsF, usF, sc, udfF, vmF, xdim);
        
	end
        

    
    
    % 3.- BOUNDARY ENTITIES: only for 1st instance (insR=0)
    
    if beF == 1 && Stp == 1
        
        r3012_Eskins (Spatches, paR, Eskin, xsF, usF, Stp, sc, udfF, xdim, udim);
        
    end
    
    
    
    % 4.- CONTROL POINTS:
    
    if (cponF==1 || cnetF==1) || (cpofF==1 || cprF==1)

        r3023_CPs (xsF, usF, cprF, cnetF, Spatches, paR, Stp, sc, xdim);
        
    end
    
    
    % 5.- MESH: at the moment only for instance 0 BUT WILL INCLUDE
    % FURTHER INSTANCES IN THE FUTURE SPECIALLY FOR TIME DISCRETIZATION
 
    if eonF==1  || (erF==1 || nrF==1)
        
        stL = 0.25 ;

      	mCP     = Spatches(paR,1).INS(Stp,1).mCP;
       	Imesh  = Spatches(paR,1).INS(Stp,1).mIM;
        
        NE = Imesh.mNE ;
          
        r3031_Imesh (stL, Imesh, mCP, KnotV, Ncp, Degree, Usize, Ew, EP, xsF, usF, eonF, erF, nrF, xdim, udim);
                                        
    end

    
    
    % 6.- KNOT SPANS:
    
    if ksF == 1

        r = [ 2 3 ; 3 1 ; 1 2 ]; 

        st = 0.1 ;

        [nvU, nvUsize] = GEN_18_UnonVoid (KnotV, Usize, Degree, udim);

        u = zeros(udim,1);

        for di = 1:1:udim

            i2 = nvUsize(r(di,1));
            j2 = nvUsize(r(di,2));

            for i = 1:1:i2
            for j = 1:1:j2

                Lx = zeros(1,xdim);
                conta = 0 ;

                for a = 0:st:1

                    u(r(di,1)) = nvU(r(di,1),i);
                    u(r(di,2)) = nvU(r(di,2),j);
                    u(di) = a ;

                    conta = conta + 1 ;

                    if xsF == 1
                        [X] = b2601_BNURBS_Xcoord (u, KnotV, Ncp,Degree,Usize, Ew,EP, udim,xdim);
                    elseif usF == 1
                        X = u ;
                    end

                    Lx(conta,:) = X(:,1);
                end


                if ((i==1 && j==1) ||(i==1 && j==j2)) || ((i==i2 && j==1) ||(i==i2 && j==j2)) 
                    LW = 3 ;
                else
                    LW = 1 ;
                end

                plot3(Lx(:,1), Lx(:,2), Lx(:,3), 'k', 'LineWidth', LW); hold on

            end
            end
    
        end
    end
    
    % 7- GPs:
    
    
    if gpF == 1
        
        Imesh  = Spatches(paR,1).INS(Stp,1).mIM;
        
        Ngp = Imesh.mNgp;
        GP  = Imesh.mGP;
        

        Gcoor = zeros(Ngp,3);
        
        for g = 1:1:Ngp
            
            if xsF == 1  
                Gcoor(g,:) = GP(g).mXcoor(:);
            elseif usF == 1
                Gcoor(g,:) = GP(g).mAcoor(:);
            end
            
            if gprF == 1
                text(Gcoor(g,1),Gcoor(g,2),Gcoor(g,3), num2str(g)); hold on
            end
            
        end
        
        scatter3(Gcoor(:,1), Gcoor(:,2), Gcoor(:,3), 'x', 'r'); hold on
        
        
    end
    
    
    
end


axis equal


end

