function [NE, EVcon] = im0147_TetMesh (iXnod, iAnod, pnbe, pRbe, Eskin, Bskin, pxdim, pudim)


% RETURNS:
% NE: No. elements
% EVcon: conn. matrix element => vertex

%{
% segments partitions: sg(u,i,j,n,N, pxdim)
Nmin = 2 ;
[sg, Nps] = im0142_Segments (pNcp, pEP, Nmin, Rt, pxdim, pudim);
%}

% 1.- PRELIMINARY TRIANGULATION:

DT = delaunayTriangulation(iXnod);

pEVcon = DT.ConnectivityList; % conn. matrix element => vertex

pNE = size(pEVcon,1); % No. elements


% 2.- DELETE TRETRAHEDRALS THAT ARE QUASI-PLANE (in both spaces):

NE = 0;
EVcon = zeros(1,4);
xCt = zeros(1,pxdim); % stores tet's centres
aCt = zeros(1,pudim);
Xvol = zeros(pNE,1); % stores volumes
Avol = zeros(pNE,1);

% 2.1.- Measures all vols:

for e = 1:1:pNE

    x = zeros(4,pxdim);
    a = zeros(4,pxdim);
    for v = 1:1:4
        for di = 1:1:pxdim
            x(v,di) = iXnod(pEVcon(e,v),di);
            a(v,di) = iAnod(pEVcon(e,v),di);
        end
    end
    
    Mx = zeros(pxdim,3);
    Ma = zeros(pxdim,3);
    for di = 1:1:pxdim
        Mx(di,1) = x(2,di)-x(1,di);
        Mx(di,2) = x(3,di)-x(1,di);
        Mx(di,3) = x(4,di)-x(1,di);
        
        Ma(di,1) = a(2,di)-a(1,di);
        Ma(di,2) = a(3,di)-a(1,di);
        Ma(di,3) = a(4,di)-a(1,di);
    end

    Xvol(e,1) = det(Mx)/6;
    Avol(e,1) = det(Ma)/6;
    
end


% 2.2.- Delete all elements whose volume is less that a portion of average:

tolVx = (sum(Xvol)/pNE)/20;
tolVa = (sum(Avol)/pNE)/20;

for e = 1:1:pNE
    
    if Xvol(e)>tolVx && Avol(e)>tolVa
        
        NE = NE + 1 ;
        
        for i=1:1:4
        EVcon(NE,i) = pEVcon(e,i);
        end
        
        Xc = zeros(pxdim,1); 
        Ac = zeros(pudim,1);
        for i = 1:1:4
            
            for di=1:1:pxdim
            Xc(di,1) = Xc(di,1) + iXnod(EVcon(NE,i),di)/4;
            Ac(di,1) = Ac(di,1) + iAnod(EVcon(NE,i),di)/4;
            end
        end
        
        xCt(NE,:) = Xc(:,1);
        aCt(NE,:) = Ac(:,1);
    end
end


% 3.- DELETE TRIANGLES WHOSE CENTER LIES OUTSIDE THE BOUNDARY (in physical space):

pNE = NE ;
pEVcon = EVcon;
pxCt = xCt ;
paCt = aCt ;

NE = 0;
EVcon = zeros(1,4);
xCt = zeros(1,pxdim);
aCt = zeros(1,pxdim);
ac = zeros(pxdim,1);
xc = zeros(pxdim,1);

for e = 1:1:pNE
    
    xc(:,1) = pxCt(e,:);
    IN = i0102_pIN_2 (xc, pnbe, pRbe, Eskin, pxdim,'open', 2);
    
    if IN == 1
        
        NE = NE + 1 ;
        
        for i=1:1:4
        EVcon(NE,i) = pEVcon(e,i);
        end
        
       	xc(:,1) = pxCt(e,:);
        ac(:,1) = paCt(e,:);
        for di=1:1:pxdim
            xCt(NE,di) = xc(di,1);
            aCt(NE,di) = ac(di,1);
        end
        
    end
end



% 4.- Detects elements whose one side is outside Eskin (param. space) PENDING !!! :

%{

r = [1 2 3 ; 2 3 4 ; 3 4 1 ; 4 1 2 ];
Nefo = 0 ;
efo = zeros(1,1);

for e = 1:1:NE
    
    for f = 1:1:4
        
        xc = zeros(pxdim,1);
        
        for i = 1:1:3
            
            nlr = r(f,i);
            nr = EVcon(e,nlr);
            
            for di = 1:1:pudim
                xc(di,1) = xc(di,1) + iXnod(nr,di)/3;
            end
        end
        
        IN = i0102_pIN_2 (xc, pnbe, pRbe, Eskin, pudim, 'closed', 2);
        
        %{
        if e == 504
            
            ac
            IN
            
        end
        %}
        
        if IN==0
            Nefo = Nefo + 1 ;
            efo(Nefo,1) = e ;
            break
        end
    end

end

%}


end

