function [rskin, NcvE, cLUvE, cLXvE] = re0161_refSkin (bR, Spatches, skin, tolS, Xtol, DUtol, NcvE, cLUvE, cLXvE, ncuE, nloE, SCconE)

% RETURNS:

% rskin: skin refined to have error in physical space < tolS. Approximate by quadratric triangles.

% NcvE(TNcu), cLUvE(TNcu,NcvE,1), cLXvE(TNcu,NcvE,3): updated curves vertexes.

rskin = GenSkinBE;
xdim = 3 ;
pudim = 3 ;
eudim = 2 ;

% 1.- EXTRACTS DATA FROM SKIN:

NV = skin.mNV ;
NE = skin.mNE ;
    
TVcon = skin.mTVcon ;
VTcon = skin.mVTcon ;
    
LVcon = skin.mLVcon ;
VLcon = skin.mVLcon ;
    
TLcon = skin.mTLcon ;
    
NLI = size(LVcon,1);
NSL = ones(NLI,1); % No. segments per edge line due to refinement (1 means no division)
    
Xv = skin.mXv ;
Uv = skin.mUv ;
Rv = skin.mRv ;
  
Nlv = skin.mNlv ;
Nlo = skin.mNlo ;
   
Ncv = sum(Nlv);
    
eDegree = skin.mDegree ;
eNcp = skin.mNcp ;
eKnotV = skin.mKnotV ;
eUsize = skin.mUsize ;
eP = skin.mP ;
ew = skin.mw ;
    
[Eew,EeP] = GEN_12_ExpandBw (ew, eP,xdim, eudim, eNcp);

ePN = skin.mPN ;
eCS = skin.mCS;

Tr = skin.mTr ;
Pa  = skin.mPa;
Cou = skin.mCou ;

rNV = NV;
rNlv = Nlv;
rXv = Xv;
rUv = Uv;
rRv = Rv;


% 2.- PRELIMINARY REFINEMENT: computes error and nodal distance to reduce to tolS

% measures error and includes necessary internal vtx to the triangles
% edge triangles required vtx are stored in NSL array.
  
rULims = zeros(NE,eudim,2);
rRLims = zeros(NE,eudim,2);

for e = 1:1:NE
        
    % 2.1.- Data from triangle:
    
	At = skin.mArea(e) ;

	umid = zeros(eudim,1);
	u = zeros(eudim,1);
 	unod = zeros(3,eudim);
    rnod = zeros(3,eudim);
    
    for i = 1:1:3
        
        nR = TVcon(e,i);
        unod(i,:) = Uv(nR,:);
        rnod(i,:) = Rv(nR,:);
        
      	u(:,1) = Uv(nR,:);
      	umid = umid + u/3;
        
    end

	[D12,~, eLims] = tr1304_maxD1D2 (unod);
    rULims(e,:,:) = eLims(:,:);
    
    [~, ~, rLims] = tr1304_maxD1D2 (rnod);
    rRLims(e,:,:) = rLims(:,:);
        
    
	% 2.2.- Relative error and distance bt'n nodes to reduce it to tolS:
 
  	[rEr, newD] = tr1374_Err_Dreqd (eDegree, eNcp, eKnotV, eUsize, eP ,ew,  umid, D12, At, tolS, eudim, xdim);
        
        
  	% 2.3.- Add internal nodes and updates partition to edges:

  	if rEr > tolS           
     	pL = 50 ; % limit in percentage to not consider internal vtx    
     	[NSL, Uv, NV] = tr1343_addNodes (e, pL, newD, unod, eLims, NSL, Uv, NV, LVcon, TLcon);
    end
      
    m(e,1) = rEr;
    
end
   

%{
if bR == 2
    
	fr = bR ;
    %tm1305_plotMesh_nD (Uv, NLI, LVcon, eudim, fr, 'k') ;
    figure(fr)
    scatter(Uv(:,1),Uv(:,2),'b')
    
    NV
end
%}
  

% 3.- ADD LINES NODES: subdivide triangles lines acc to NSL:
    
u1 = zeros(2,1);
u2 = zeros(2,1);
    
% 3.1.- Add internal lines => just adds vtrxs
    
for li = sum(Nlv)+1:1:NLI
if NSL(li)>1
            
	n1 = LVcon(li,1);
  	n2 = LVcon(li,2);
            
   	u1(:,1) = Uv(n1,:);
   	u2(:,1) = Uv(n2,:);
            
  	D = norm(u2-u1)/NSL(li);
            
   	[N, sd] = go2531_DivSegment (u1, u2, 2, D, 2, 1);
  
    if N > 2
  
    	nUv = zeros(N-2,2);  
        for i = 1:1:N-2
            nUv(i,:)=sd(i+1,:);
        end
                    
    	Uv = [Uv;nUv];
        NV = NV + N-2;
    end
end     
end
    

% 3.2.- Then the contour lines:
% create temporary arrays that later substitute originals
    
tUv = zeros(1,2);
tNV = 0;
tNlv = Nlv;
    
li = 0 ;

for lr = 1:1:Nlo
for j = 1:1:Nlv(lr)
            
	li = li + 1;
            
  	% adds first vertex:
            
  	tNV = tNV+1;
   	tUv(tNV,:) = Uv(li,:);

   	% Potential partition:
            
   	if NSL(li) > 1
                
        n1 = LVcon(li,1);
      	n2 = LVcon(li,2);

      	u1(:,1) = Uv(n1,:);
      	u2(:,1) = Uv(n2,:);

      	D = norm(u2-u1)/NSL(li);

       	[N, sd] = go2531_DivSegment (u1, u2, 2, D, 2, 1);
                
        % Partition of the side => add internal partition of the side    
        if N > 2
            for i = 1:1:N-2 
                        
                tNlv(lr) = tNlv(lr)+1;
            	tNV = tNV+1;
              	tUv(tNV,:) = sd(i+1,:);
            end
        end         
    end
end
end
    

% updates original:

tUv2 = Uv(sum(Nlv)+1:NV,:); 
Uv = [tUv ; tUv2] ; 
NV = size(Uv,1);
Nlv = tNlv;
    

%{
if bR==2
    
    Nlv
    
    figure(bR)
    
    for i = 1:1:Nlv
        scatter(Uv(i,1),Uv(i,2),'r'); hold on
    end
    
    for i = Nlv+1:1:NV
        scatter(Uv(i,1),Uv(i,2),'b'); hold on
    end
   
    axis equal
    
end
%}



% 4.- COORDINATES IN PATTERN SPACE:

Rv = zeros(NV,2);

% 4.1.- Triangulation in pattern space:

[rNE, rNV, rNLI, rTVcon, rVTcon, rLVcon, rVLcon, rTLcon] = tr1340_globalTriang (Nlo, rNlv, rRv);


% 4.2.- Coputes Rv of updated arrangements:

nd = 4 ;
u = zeros(eudim,1);

for i = 1:1:NV
    
    u(:,1) = Uv(i,:);
    
    r = tr1348_interpTri (u, rNE, rTVcon, rUv, rULims, rRv, eudim, nd); % pending !
                            
    Rv(i,:) = r(:,1);
    
end


%{
if bR==2
    

    fr = bR;
    tm1305_plotMesh_nD (rRv, rNLI, rLVcon, 2, fr, 'r') ;
    %scatter(Rv(:,1), Rv(:,2),'k'); hold on
    
  	fr = bR+1;
    tm1305_plotMesh_nD (rUv, rNLI, rLVcon, 2, fr, 'r') ;
    %scatter(Uv(:,1), Uv(:,2),'k'); hold on
    
    fr = 1 ;
    tm1305_plotMesh_nD (rXv, rNLI, rLVcon, 3, fr, 'b') ;
end
%}

% 4.- TRIANGULATION:

% 4.1.- Triangulation of new vrtx (considers contour with new points):
    
[NE, NV, NLI, TVcon, VTcon, LVcon, VLcon, TLcon] = tr1340_globalTriang (Nlo, Nlv, Rv);
    



% 4.2.- Physical coordinates:
    
Xv = zeros(NV,xdim);

for i = 1:1:NV
	u(:,1) = Uv(i,:);
	[X] = b2601_BNURBS_Xcoord (u, eKnotV, eNcp,eDegree,eUsize, Eew, EeP, eudim,xdim);
	Xv(i,:) = X(:,1);
end

   
%{
if bR == 2
    
 	fr = 10*bR ;
    tm1305_plotMesh_nD (Rv, NLI, LVcon, eudim, fr, 'b') ;
    %scatter(Uv(:,1),Uv(:,2),'r')
    
	fr = 10*bR+1 ;
    tm1305_plotMesh_nD (Uv, NLI, LVcon, eudim, fr, 'b') ;
    %scatter(Uv(:,1),Uv(:,2),'r')
    
   	fr = 13 ;
    tm1305_plotMesh_nD (Xv, NLI, LVcon, 3, fr, 'b') ;

    if gg==2
    end
 
end
%} 
    
% 5.- LOCAL REFINEMENT WHERE REQUIRED in the new triangulation:
    
for e = 1:1:NE
        
 	umid = zeros(eudim,1);
 	u = zeros(eudim,1);
  	unod = zeros(3,eudim);

  	for i = 1:1:3
        nR = TVcon(e,i);
     	unod(i,:) = Uv(nR,:);
        u(:,1) = Uv(nR,:);
       	umid = umid + u/3;
    end

	[D12, eD12, eLims] = tr1304_maxD1D2 (unod);
        
        
 	% relative error and add nodes in case greater than tolS:
 
   	[rEr, newD] = tr1374_Err_Dreqd (eDegree, eNcp, eKnotV, eUsize, eP ,ew,  umid, D12, At, tolS, eudim, xdim);
        
        
 	% local refinement: PENDING !    
    if rEr > tolS
 	% [TVcon, VTcon, Xv, Uv, NE, NV, Ncv] = tr1371_RemeshTri (tR, TVcon, VTcon, Xv, Uv, NE, NV, Ncv); 
    end  
    
end
   

% 6.- COMPUTES REST OF THE SKIN FEATURES AND ALLOCATES:

% 6.1.- Patch(es) features:

[pR, pDegree,  pNcp,  pKnotV,  pUsize,  pP,  pw,  pCD , pEw,  pEP,  pULims, ...
spR, spDegree, spNcp, spKnotV, spUsize, spP, spw, spCD, spEw, spEP, spULims] = ...
so2938_patchNURBS (skin, Spatches, xdim);

% 6.2.- Patch param. coord's and normals and physical area:

Av = v2861b_PPvol (Xv, NV, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, xdim, Xtol, DUtol);

[~,  ANe, ANv] = tm1308_TmeshNA (NE, NV, TVcon, VTcon, Av, pudim);
[Ae, XNe, XNv] = tm1308_TmeshNA (NE, NV, TVcon, VTcon, Xv, pudim);

AvS = 0 ;
ANeS = 0 ;
ANvS = 0 ;

if spR > 0
    AvS = v2861b_PPvol (Xv, NV, spDegree, spNcp, spKnotV, spUsize, spP, spw, spCD, xdim, Xtol, DUtol);
    [~,  ANeS, ANvS] = tm1308_TmeshNA (NE, NV, TVcon, VTcon, AvS, pudim);
end



% 7.- ALLOCATES:

rskin.mDegree = eDegree ;
rskin.mKnotV = eKnotV ;
rskin.mUsize = eUsize ;
rskin.mNcp = eNcp ;
rskin.mP = eP ;
rskin.mw = ew ;

rskin.mPN = ePN ;
rskin.mCS = eCS ;

rskin.mNlo = Nlo ;
rskin.mNlv = Nlv ;

rskin.mNE = NE ;
rskin.mNV = NV ;

rskin.mRv = Rv ;
rskin.mUv = Uv ;
rskin.mAv  = Av ;
rskin.mAvS = AvS ;

rskin.mXv = Xv ;
rskin.mXv0 = Xv ;

rskin.mTVcon = TVcon ;
rskin.mVTcon = VTcon ;
rskin.mLVcon = LVcon ;
rskin.mVLcon = VLcon ;
rskin.mTLcon = TLcon ;

rskin.mArea = Ae ;

rskin.mANv = ANv ;
rskin.mANe = ANe ;

rskin.mANvS = ANvS ;
rskin.mANeS = ANeS ;

rskin.mXNv = XNv ;
rskin.mXNe = XNe ;

rskin.mTr = Tr ;
rskin.mPa = Pa ;
rskin.mCou = Cou ;





% represetnation to check:
%{

if bR==2
    
    fr = 51 ;
    
    
    %{
    for e = 1:1:NE
        
        unod = zeros(3,2);
        um = zeros(2,1);
        
        for i = 1:1:3 
            nR = TVcon(e,i);
            unod(i,:) = Uv(nR,:);
            
            for di = 1:1:2
                um(di,1) = um(di,1) + unod(i,di);
            end
 
        end
        
        um = um/3;
        
        % text(um(1), um(2), num2str(e)); hold on
        
        tm1304_plotTri_nD (unod, 2, fr, 'k')
        
    end
    %}

    tm1305_plotMesh_nD (Av, NLI, LVcon, xdim, fr, 'k');
    tm1305_plotMesh_nD (Uv, NLI, LVcon, eudim, fr+1, 'k');
    tm1305_plotMesh_nD (Xv, NLI, LVcon, xdim,  fr+2, 'k');
    
    
    
    gAv = zeros(Ngp,3);
    gUv = zeros(Ngp,2);
    gXv = zeros(Ngp,3);
    
    for g = 1:1:Ngp
        gAv(g,:) = GP(g).mAcoor(:);
        gUv(g,:) = GP(g).mUcoor(:);
        gXv(g,:) = GP(g).mXcoor(:);
    end
   
    sc = 1 ;
    
    figure(fr)
    %scatter3(gAv(:,1),gAv(:,2),gAv(:,3),'r','x'); hold on

    figure(fr+1)
    %scatter(gUv(:,1),gUv(:,2),'r','x'); hold on
    
    figure(fr+2)
    for i = 1:1:NV
        
        nv = zeros(2,xdim);
        for di = 1:1:xdim
            nv(1,di) = Xv(i,di);
            nv(2,di) = Xv(i,di) + sc * XNv(i,di);
        end
        
        plot3( nv(:,1),  nv(:,2),  nv(:,3), 'r'); hold on
        axis equal
        
    end
    %scatter3(gXv(:,1),gXv(:,2),gXv(:,3),'r','x'); hold on
    
    %scatter(Uv(:,1),Uv(:,2),'r'); hold on
    %scatter(tUv(:,1),tUv(:,2),'b'); hold on
    
    axis equal


end

%}
    
 

end

