function [nWIp, conEWp, conWEp, conWNp, pDir] = w0152_connWires (nELp, conENp, udim)

% routine devised for first instance allocation, where there are not T-junctions

% RETURNS: connectivity matrices between elements and wires 'EW' and
% reversed 'WE' for parameter and index meshes. Also conn. wires - nodes

% conEWp (nELp, No. wires per element)
% conWEp (nWp,  No. elements for that wire)
% conWNp (nWp,  2)
% pDir   (nWp,1): parameter direcion of the cable


% INPUTS:

% nELp: No. Imesh elems


% 1.- STORES PAIRS OF NODES CONNECTED BY WIRES:

contaW = 0 ;
wnod = zeros(1,1);

% temporary matrices:

% tconEWp = zeros(nELp,1);
% tconWEp = zeros(1,1);

tpDir = zeros(1,1);


% 1.1.- Set local conn. matrix: wire-nodes

if udim == 2
    Nw = 4 ;
    cn = [ 1 3 ; 2 4 ; 1 2 ; 3 4 ];
elseif udim == 3
    Nw = 12 ;
    cn = [ 1 5 ; 2 6 ; 3 7 ; 4 8 ;   1 3 ; 2 4 ; 5 7 ; 6 8  ;   1 2 ; 3 4 ; 5 6 ; 7 8 ];
end

% 1.2.- Sets connected nodes, stored in wnod:

for e = 1:1:nELp
    
    % corners nodes refs:
    
    cNR = zeros(2^udim,1); 
    for n = 1:1:2^udim
        cNR(n,1) = conENp(e,n);
    end
    
    
    % wires ends:
    
    for i=1:1:Nw
        
        contaW = contaW + 1 ;
        
        wnod(contaW,1) = cNR(cn(i,1));
        wnod(contaW,2) = cNR(cn(i,2));

        tpDir(contaW,1) = GEN_41_roof( i / ( 2^(udim-1) )); 
    end
     
end


% 1.3.- Removes repeated => transfers data to conWNp, nWIp and pDir:

[conWNp,retRows] = GEN_16_RemoveRepeatedRows (wnod, 2, 0.01);

nWIp = size(conWNp,1);
pDir = zeros(nWIp,1);

for wi = 1:1:nWIp
    nr = retRows(wi);
    pDir(wi,1) = tpDir(nr);
end



% 2.- THE OTHER CONN. MATRICES:

conEWp = zeros(nELp,Nw);
conWEp = zeros(nWIp,1);

for e = 1:1:nELp
    
    
   	% corners nodes refs:
    
    cNR = zeros(2^udim,1); 
    for n = 1:1:2^udim
        cNR(n,1) = conENp(e,n);
    end
    
    
    % works with wires ends:
    
    for i=1:1:Nw
        
        contaW = contaW + 1 ;
        
        nod1 = cNR(cn(i,1));
        nod2 = cNR(cn(i,2));

        % finds those ref's in conWNp
        for wi = 1:1:nWIp
            if conWNp(wi,1) == nod1 && conWNp(wi,2) == nod2 
                conEWp(e,i) = wi;
                break
            end
        end
        
        
        for j = 1:1:size(conWEp,2)
            if conWEp(wi,j) == 0
                conWEp(wi,j) = e ;
                conWEp(wi,j+1) = 0 ;
                break
            end
        end
   
    end
 
end





end

