function [pWires, nWIp, iWires, nWIi, conEWi, conWEi, conWNi, conEWp, conWEp, conWNp] = w0131_firstRwires ...
    (elem, FtoIe, mInodi, mUnod, nodNicp, nodRicpA, nELp, nELi, gPatch, pa, pDegree, pKnotV, pKnotVI, pUsize, repI, pudim, pxdim)

% RETURNS:
% Wires:	object, including physical space coord's for representation
% nWI:   	No. wires

% for rect-mesh

% INPUTS:
% FRefin:   first instace, necessary for mXcoorF calculation
% Elem:     element objects
% mA:       anchors object
% mu:       displacements vector (un-scaled)
% nELp:     No. filled elements of the rect-mesh, physical or param. space
% nELi:     total No. elem's, index space
% KnotV:    global knot vector
% repI:     representation increment

% Strategy: 


% 1.- RELATIONS BETWEEN WIRES, ELEMENTS AND NODES:
%   uses connectivity matrices

[conEWi, conWEi, conWNi, iDir, conEWp, conWEp, conWNp, pDir] = ...
w0132_connWires (nELi, nELp, pKnotV, pKnotVI, pUsize, pDegree, pudim);

        
% 2.- DECLARES WIRES OBJECTS:

% parameter wires
nWIp = size(conWNp,1);
pWires(nWIp) = GenWire();

% index space wires:
nWIi = size(conWNi,1);
iWires(nWIi) = GenWire();


% 3.- FILLS INDEX WIRES FEATURES:

for w = 1:1:nWIi
    
    % 3.1.- Directions:
    
    iWires(w).mDir = iDir(w);
    
    
    % 3.2.- No. point of the wire, 2 for this case. Declares points object:
    
    iWires(w).nPO = 2;
    mPi(2) = GenGeomPoint;
    
    
    % 3.3.- Indexes coord's of wire end points:
    
    n1 = conWNi(w,1);
    n2 = conWNi(w,2);
    
    for di=1:1:pudim
        mPi(1).mIcoor(di) = mInodi(n1,di);
        mPi(2).mIcoor(di) = mInodi(n2,di);
    end
    
    iWires(w).mP = mPi;
    clearvars mPi;
    
end




% 4.- FILLS AND COMPUTES PARAMTER/PHYSICAL WIRES FEATURES:

% aux. variables:
ap   = zeros(pudim,1);
bp   = zeros(pudim,1);

for w = 1:1:nWIp
    
	% 4.1.- Directions:
    
    pWires(w).mDir = pDir(w);
    
    
    % 4.2.- Points of wire, icp's, number and mP object:
    
  	n1 = conWNp(w,1);
    n2 = conWNp(w,2);
    
    for di=1:1:pudim
        ap(di) = mUnod(n1,di);
        bp(di) = mUnod(n2,di);
    end
    
    % extracts and merges icps from both nodes:
    
    v1 = zeros(nodNicp(n1),1);
    v2 = zeros(nodNicp(n2),1);
    
    for a = 1:1:nodNicp(n1)
        v1(a) = nodRicpA(n1,a);
    end
    for a = 1:1:nodNicp(n2)
        v2(a) = nodRicpA(n2,a);
    end
    tRicpA = GEN_22_merge2vec (v1,v2);
    tNicp = size(tRicpA,1);



    reDir = pDir(w);
        
	[nPO, mPp] = i0199_First_nPOmP (ap,bp, repI, reDir, tNicp, tRicpA,  gPatch, pa, pudim, pxdim);

  	pWires(w).nPO = nPO;
 	pWires(w).mP  = mPp;
    
    
     % 4.3.- Stores if is perimeter wire: internal / surface / edge
    
    pWires(w).mPeri = w0502_Perimeter (ap, bp, pKnotV, pUsize, reDir, pudim);

    
    % 4.4.- Wire ON flag:
    
    pWires(w).ON = 1 ;
    
    acc = 0 ;
    
    for i = 1:1:size(conWEp,2)

        eR = conWEp(w,i);
        if eR==0
            break
        else
            eR = FtoIe(eR);
            acc = acc + elem(eR).ON;
        end
    end
    
    if acc==0 
        pWires(w).ON = 0 ;
    end

    
end



end

