function [conEWi, conWEi, conWNi, iDir, conEWp, conWEp, conWNp, pDir] = ...
            w0132_connWires ...
            (nELi, nELp, KnotV, KnotVI, Usize, Degree, udim)

% routine devised for first instance allocation, where there are not T-junctions

% RETURNS: connectivity matrices between elements and wires 'EW' and
% reversed 'WE' for parameter and index meshes. Also conn. wires - nodes

% conEWi (nELi, No. wires per element)
% conWEi (nWi,  No. elements for that wire)
% conWNi (nWi,  2)
% iDir   (nWi,1): parameter direcion of the cable

% and similar for parameter space

% INPUTS:

% nELi: No. all elements, or elements in index space
% nELp: No. non-void - or 'filled' - elements - or elements in param/physical space


% 1.- COUNTS NO. NON-VOID SPANS IN KnotV and STORES THEIR No. KNOTS:

% U for index space:
iUsize = Usize ;

% U for parameter space:
[~, pUsize] = GEN_18_UnonVoid (KnotV, Usize, Degree, udim);


% 2.- ESTABLISHES TOTAL No. WIRES:

if udim==2
    nWi = (iUsize(1)-1) * iUsize(2) + (iUsize(2)-1) * iUsize(1) ;
    nWp = (pUsize(1)-1) * pUsize(2)	+ (pUsize(2)-1) * pUsize(1) ;
    
elseif udim==3
    
    nWi = (iUsize(1)-1) *iUsize(2)*iUsize(3)  + (iUsize(2)-1) *iUsize(1)*iUsize(3)  + (iUsize(3)-1) *iUsize(1)*iUsize(2) ;
    nWp = (pUsize(1)-1) *pUsize(2)*pUsize(3)  + (pUsize(2)-1) *pUsize(1)*pUsize(3)  + (pUsize(3)-1) *pUsize(1)*pUsize(2) ;
end


% 3.- WORKS OUT CONN. MATRICES:

% 3.1.- index cables:
[conWNi, iDir]   = w0133_WnodCon (nWi, iUsize, udim);
[conEWi, conWEi] = w0134_WeleCon (nELi, nWi, iUsize, udim);

% 3.2.- parameter cables:
[conWNp, pDir] = w0133_WnodCon (nWp, pUsize, udim);
[conEWp, conWEp] = w0134_WeleCon (nELp, nWp, pUsize, udim);


end

