function [conWN, Dir] = w0133_WnodCon (nW, Usize, udim)

% RETUNRS:
% conWN[nW,2]: connectivoty matrix wires-nodes refs
% Dir(nW,1): parameter direction of the wire


% INPUTS:
% nW: No. wires
% Usize: knot vectors sizes
% udim: No. parameter dimensions

% 1.- INITIATION OF RESPONSE:

conWN = zeros(nW,2);
Dir = zeros(nW,1);
contaW = 0 ; % wire counter = 0



% 2.- TWO-DIMENSIONAL:

if udim==2

    % 2.1.- direction 1:

    Xacc = 0 ;
    for i = 1:1:Usize(1)-1

        for j = 1:1:Usize(2)

            contaW = contaW+1;
            n1 = Xacc + j ;
            n2 = Xacc + j + Usize(2);

            conWN(contaW,1) = n1 ;
            conWN(contaW,2) = n2 ;
            Dir(contaW,1) = 1 ;
        end

        Xacc = Xacc + (Usize(2));
    end


    % 2.2.- direction 2:

    Yacc = 0 ;
    for i = 1:1:Usize(1)

        for j = 1:1:Usize(2)-1

            contaW = contaW+1;
            n1 = Yacc + j ;
            n2 = Yacc + j + 1;

            conWN(contaW,1) = n1 ;
            conWN(contaW,2) = n2 ;
            Dir(contaW,1) = 2 ;
        end

        Yacc = Yacc + (Usize(2));
    end

end



% 3.- THREE DIMENSIONAL:

if udim==3
    
    % 3.1.- direction 1:

    spanW = Usize(2) * Usize(3);
    
    Xacc1 = 0 ;
    for i = 1:1:Usize(1)-1

        Xacc2 = 0 ;
        for j = 1:1:Usize(2)

            for k = 1:1:Usize(3)
            
                contaW = contaW+1;
                
                n1 = Xacc1 + Xacc2 + k ;
                n2 = Xacc1 + Xacc2 + k + spanW;

                conWN(contaW,1) = n1 ;
                conWN(contaW,2) = n2 ;
                Dir(contaW,1) = 1 ;
            
            end
            
            Xacc2 = Xacc2 + Usize(3);
        end

        
        Xacc1 = Xacc1 + Usize(2) * Usize(3);
    end
    
    
    
    % 3.2.- direction 2:
    
    spanW = Usize(3);
    
    Xacc1 = 0 ;
    for i = 1:1:Usize(1)

        Xacc3 = 0 ;
        for j = 1:1:Usize(2)-1
            
            for k = 1:1:Usize(3)
                
                contaW = contaW+1;
                
                n1 = Xacc1 + Xacc3 + k ;
                n2 = Xacc1 + Xacc3 + k + spanW;

                conWN(contaW,1) = n1 ;
                conWN(contaW,2) = n2 ;
                Dir(contaW,1) = 2 ;
            
            end
            
            Xacc3 = Xacc3 + Usize(3);
        end

        Xacc1 = Xacc1 + Usize(2) * Usize(3);
    end
    
    
 	% 3.2.- direction 3:
    
    spanW = 1;
    
   	Xacc1 = 0 ;
    for i = 1:1:Usize(1)

        Xacc3 = 0 ;
        for j = 1:1:Usize(2)
            
            for k = 1:1:Usize(3)-1
                
                contaW = contaW+1;
                
                n1 = Xacc1 + Xacc3 + k ;
                n2 = Xacc1 + Xacc3 + k + spanW;

                conWN(contaW,1) = n1 ;
                conWN(contaW,2) = n2 ;
                Dir(contaW,1) = 3 ;
            
            end
            
            Xacc3 = Xacc3 + Usize(3);
        end

        
        Xacc1 = Xacc1 + Usize(2) * Usize(3);
    end
    
end


end

