function [conEW, conWE] = w0134_WeleCon (nE, nW, Usize, udim)

% RETURNS:
% conEW, conWE: connection matrices relating wires with elements

% INPUTS:
% nE: No. elements
% nW: No. wires
% Usize: knot vectors sizes
% udim: No. parameter dimensions

% 1.- INITIATION OF RESPONSE:


conEW   = zeros(nE,2^udim);
conWE   = zeros(nW,2^(udim-1));

contaE  = 0 ; % elem counter = 0 
contaW  = 0 ; % wire counter = 0



% 2.- TWO-DIMENSIONAL:

if udim==2
    
    % 2.1.- Elem-Wire connection:
    
    step1 = ((Usize(1)-1) * Usize(2)) ;
    
    acc1 = 0 ;
    acc2 = 0 ;
    for i =1:1:Usize(1)-1

        for j =1:1:Usize(2)-1    

            contaE = contaE + 1 ;
            
            conEW(contaE,1) = acc1  + j ;
            conEW(contaE,2) = acc1  + j + 1 ;
            
            conEW(contaE,3) = step1 + acc2 + j  ;
            conEW(contaE,4) = step1 + acc2 + Usize(2)-1 + j ;
        end 
        
        acc1 = acc1 + Usize(2) ;
        acc2 = acc2 + Usize(2)-1 ;
    end
    
    
    % 2.2.- Wire-Elem connection:
    
    % wires in dir 1:
    
    acc = 0 ;
    for i = 1:1:Usize(2)
        for j = 1:1:Usize(1)-1
            
            contaW = contaW + 1 ;
            
            eR1 = acc + j - 1 ;
            eR2 = acc + j ;
            
            if j==1 
                conWE(contaW,1) = eR2 ;
                
            elseif j>1 && j<Usize(2)
                
                conWE(contaW,1) = eR1 ;
                conWE(contaW,2) = eR2 ;
                
            elseif j == Usize(2)
                
                conWE(contaW,1) = eR1 ;
            end
            
        end
        acc = acc + Usize(2)-1;
    end
    
    
    % wires in dir 2:
    
    acc = 0 ;
    for i = 1:1:Usize(1)
        for j = 1:1:Usize(2)-1
        
            contaW = contaW + 1 ;
            
            eR1 = acc + j - (Usize(2)-1) ;
            eR2 = acc + j ;
            
            if i==1 
                conWE(contaW,1) = eR2 ;
                
            elseif i>1 && i<Usize(1)
                
                conWE(contaW,1) = eR1 ;
                conWE(contaW,2) = eR2 ;
                
            elseif i == Usize(1)
                
                conWE(contaW,1) = eR1 ;
            end
        end
        
        acc = acc + (Usize(2)-1);
        
    end
    
    
    
    
end


% 3.- THREE DIMENSIONAL:

if udim==3
    
    % 3.1.- Elem-Wire connection:
    
    ini1 = (Usize(1)-1) * Usize(2) * Usize(3); 
    ini2 = ini1 + (Usize(2)-1) * Usize(1) * Usize(3) ; 
    step2 = (Usize(2)-1) * Usize(3);            

    acc1 = 0 ;
    acc2 = 0 ;
    acc3 = 0 ;
    
    for i = 1:1:Usize(1)-1
        
        for j = 1:1:Usize(2)-1  
            
            for k = 1:1:Usize(3)-1
                
                contaE = contaE + 1 ;
                
                conEW(contaE,1) = acc1 + k ;
                conEW(contaE,2) = acc1 + k + 1 ;
                conEW(contaE,3) = acc1 + Usize(3) + k ;
                conEW(contaE,4) = acc1 + Usize(3) + k + 1;
                
                conEW(contaE,5) = ini1 + acc2 + k ;
                conEW(contaE,6) = ini1 + acc2 + k + 1 ;
                conEW(contaE,7) = ini1 + acc2 + step2 + k  ;
                conEW(contaE,8) = ini1 + acc2 + step2 + k + 1 ;
                
                conEW(contaE,9)  = ini2 + acc3 + k ;
                conEW(contaE,10) = ini2 + acc3 + k + 2 ;
                conEW(contaE,11) = ini2 + acc3 + step2 -1 + k  ;
                conEW(contaE,12) = ini2 + acc3 + step2 -1 + k + 2 ;
                
            end
            
            acc1 = acc1 + Usize(3) ;
            acc2 = acc2 + Usize(3) ;
            acc3 = acc3 + Usize(3) - 1 ; 
        end
        
        acc1 = acc1 + Usize(3);
        acc3 = acc3 + Usize(3)-1;
    end
    
    
  	% 3.2.- Wire-Elem connection:
    
    % wires in dir 1:
    
    acc2 = 0 ;
    for i = 1:1:Usize(1)-1
        
        acc1 = 0 ;
        
   	for j = 1:1:Usize(2)
        
        for k = 1:1:Usize(3)
            
            contaW = contaW + 1 ;
            
            eR1 = acc2 + acc1 + k - (Usize(3)-1) - 1 ;
            eR2 = acc2 + acc1 + k - (Usize(3)-1) ;
            eR3 = acc2 + acc1 + k - 1 ;
            eR4 = acc2 + acc1 + k ;
            
            if j==1 
                
                if k == 1
                    conWE(contaW,1) = eR4 ;
                elseif k>1 && k<Usize(3)
                    
                    conWE(contaW,1) = eR3 ;
                    conWE(contaW,2) = eR4 ;
                    
                elseif k == Usize(3)
                    conWE(contaW,1) = eR3 ;
                end
                
            elseif j>1 && j<Usize(2)
                
                if k == 1
                    conWE(contaW,1) = eR2 ;
                    conWE(contaW,2) = eR4 ;
                elseif k>1 && k<Usize(3)
                    
                    conWE(contaW,1) = eR1 ;
                    conWE(contaW,2) = eR2 ;
                    conWE(contaW,3) = eR3 ;
                    conWE(contaW,4) = eR4 ;
                    
                elseif k == Usize(3)
                    conWE(contaW,1) = eR1 ;
                    conWE(contaW,2) = eR3 ;
                end
                
            elseif j == Usize(2)
                
                if k == 1
                    conWE(contaW,1) = eR2 ;
                elseif k>1 && k<Usize(3)
                    
                    conWE(contaW,1) = eR1 ;
                    conWE(contaW,2) = eR2 ;
                    
                elseif k == Usize(3)
                    conWE(contaW,1) = eR1 ;
                end
            end
            
        end
        acc1 = acc1 + Usize(3)-1;
    end
    
    acc2 = acc2 + (Usize(3)-1) * (Usize(2)-1);
    end
    
    
 	% wires in dir 2:
    
    acc2 = 0 ;
    for i = 1:1:Usize(1)
        
        acc1 = 0 ;
        
   	for j = 1:1:Usize(2)-1
        
        for k = 1:1:Usize(3)
            
            contaW = contaW + 1 ;
            
            eR1 = acc2 + acc1 + k - (Usize(3)-1)*(Usize(2)-1) - 1 ;
            eR2 = acc2 + acc1 + k - (Usize(3)-1)*(Usize(2)-1) ;
            eR3 = acc2 + acc1 + k - 1 ;
            eR4 = acc2 + acc1 + k ;
            
            if i==1 
                
                if k == 1
                    conWE(contaW,1) = eR4 ;
                elseif k>1 && k<Usize(3)
                    
                    conWE(contaW,1) = eR3 ;
                    conWE(contaW,2) = eR4 ;
                    
                elseif k == Usize(3)
                    conWE(contaW,1) = eR3 ;
                end
                
            elseif i>1 && i<Usize(1)
                
                if k == 1
                    conWE(contaW,1) = eR2 ;
                    conWE(contaW,2) = eR4 ;
                elseif k>1 && k<Usize(3)
                    
                    conWE(contaW,1) = eR1 ;
                    conWE(contaW,2) = eR2 ;
                    conWE(contaW,3) = eR3 ;
                    conWE(contaW,4) = eR4 ;
                    
                elseif k == Usize(3)
                    conWE(contaW,1) = eR1 ;
                    conWE(contaW,2) = eR3 ;
                end
                
            elseif i == Usize(1)
                
                if k == 1
                    conWE(contaW,1) = eR2 ;
                elseif k>1 && k<Usize(3)
                    
                    conWE(contaW,1) = eR1 ;
                    conWE(contaW,2) = eR2 ;
                    
                elseif k == Usize(3)
                    conWE(contaW,1) = eR1 ;
                end
            end
            
        end
        acc1 = acc1 + Usize(3)-1;
    end
    
    acc2 = acc2 + (Usize(3)-1) * (Usize(2)-1);
    end
    
        
 	% wires in dir 3:
    
    acc2 = 0 ;
    for i = 1:1:Usize(1)
        
        acc1 = 0 ;
        
   	for j = 1:1:Usize(2)
        
        for k = 1:1:Usize(3)-1
            
            contaW = contaW + 1 ;
            
            eR1 = acc2 + acc1 + k - ((Usize(3)-1) * (Usize(2)-1)) - (Usize(3)-1) ;
            eR2 = acc2 + acc1 + k - ((Usize(3)-1) * (Usize(2)-1));
            eR3 = acc2 + acc1 + k -  (Usize(3)-1) ;
            eR4 = acc2 + acc1 + k ;
            
            if i==1 
                
                if j == 1
                    conWE(contaW,1) = eR4 ;
                elseif j>1 && j<Usize(2)
                    
                    conWE(contaW,1) = eR3 ;
                    conWE(contaW,2) = eR4 ;
                    
                elseif j == Usize(2)
                    conWE(contaW,1) = eR3 ;
                end
                
            elseif i>1 && i<Usize(1)
                
                if j == 1
                    conWE(contaW,1) = eR2 ;
                    conWE(contaW,2) = eR4 ;
                elseif j>1 && j<Usize(2)
                    
                    conWE(contaW,1) = eR1 ;
                    conWE(contaW,2) = eR2 ;
                    conWE(contaW,3) = eR3 ;
                    conWE(contaW,4) = eR4 ;
                    
                elseif j == Usize(2)
                    conWE(contaW,1) = eR1 ;
                    conWE(contaW,2) = eR3 ;
                end
                
            elseif i == Usize(1)
                
                if j == 1
                    conWE(contaW,1) = eR2 ;
                elseif j>1 && j<Usize(2)
                    
                    conWE(contaW,1) = eR1 ;
                    conWE(contaW,2) = eR2 ;
                    
                elseif j == Usize(2)
                    conWE(contaW,1) = eR1 ;
                end
            end
            
        end
        acc1 = acc1 + Usize(3)-1;
    end
    
    acc2 = acc2 + (Usize(3)-1) * (Usize(2)-1);
    end
    
    
    
    
    
end





end

