function [Nacp, CP, Non, CPref, GP] = a0111_CPs (Spatches, pR, Ngp, GP)

% RETURNS:
% control points related variables, including updated GPs of Tet-mesh with icps information

% 1.- PRELIMINARIES:

pudim = 3 ;
xdim = 3 ;

pNcp     = Spatches(pR,1).INS(1).mNcp ;
pDegree  = Spatches(pR,1).INS(1).mDegree ;
pUsize   = Spatches(pR,1).INS(1).mUsize ;
pKnotV   = Spatches(pR,1).INS(1).mKnotV ;
pP       = Spatches(pR,1).INS(1).mP ;
pw       = Spatches(pR,1).INS(1).mw ;

Nacp = 1 ; % No. anchors is just the No. control points

for di = 1:1:pudim
    Nacp = Nacp * pNcp(di);
end

CP(Nacp,1) = cCpoint ;


% 2.- PARAM. COORD'S. and INDEXES:

U = zeros(1,max(pUsize)); % global knot vector in one direction

a=0 ;
for i=1:1:pNcp(1)
    
    for j=1:1:pNcp(2)
    for k=1:1:pNcp(3)  
        
        a = a+1;
        IX = [i,j,k];
        
        for di=1:1:pudim
            
            if di==1
                x = i ;
            elseif di==2
                x = j ;
            elseif di==3
                x = k ;
            end
            
            U(:) = pKnotV(di,:);
            index = (pDegree(di)+3)/2 + x - 1 ;
            
            u = GEN_34_ParamFromIndex (index, U);
            
            CP(a,1).mAcoor(di) = round (u,5) ;
        end

        CP(a,1).mIX = IX;
        
    end    
    end
end



% 3.- PHYSICAL COORD'S:

for a = 1:1:Nacp
 
    CP(a,1).mw = pw(a);

    for di=1:1:xdim
        CP(a,1).mXcoor(di)  = pP(a,di);
    end

end



% 4.- PARAM. LIMITS:

a = 0 ;
U = pKnotV(1,1:pUsize(1));
V = pKnotV(2,1:pUsize(2));
W = pKnotV(3,1:pUsize(3));

for i = 1:1:pNcp(1) 
    
    [~,ULims,~] = a0115_aLKV (pDegree(1), U, i);
    
    for j = 1:1:pNcp(2)
        
        [~,VLims,~] = a0115_aLKV (pDegree(2), V, j);
        
        for k = 1:1:pNcp(3)
            
            [~,WLims,~] = a0115_aLKV (pDegree(3), W, k);

            a = a + 1;

         	CP(a,1).mAcoLi(1,1)= ULims(1);
            CP(a,1).mAcoLi(1,2)= ULims(2);
         	CP(a,1).mAcoLi(2,1)= VLims(1);
            CP(a,1).mAcoLi(2,2)= VLims(2);
          	CP(a,1).mAcoLi(3,1)= WLims(1);
            CP(a,1).mAcoLi(3,2)= WLims(2);
            
        end 
    end
end



% 5.- CPs STATE, GPs THEY HAVE INFLUENCE ON, DERIVATIVES wrt PARAM.:

tol = 0.0001 ; % tolerance to neglect one cp influence

gNicp = zeros(Ngp,1);
gRicp = zeros(Ngp,1); % "1" will grow

gBF   = zeros(Ngp,1); % influential B-spline basis functions value at each GP

gBFdir  = zeros(Ngp,1,pudim); % influential B-splines basis function values at each GP, but separated by pram. direction
gdBFdir = zeros(Ngp,1,pudim); % and 1st derivatives in each direction

for a = 1:1:Nacp
    
    aULims = zeros(pudim,2);
    aULims(:,:) = CP(a,1).mAcoLi(:,:);
    
    CP(a,1).ON = 0 ;
    
    % conditions to be active: GP lies inside aULims and its [b.funct OR any derivative] are not 0 

    for g = 1:1:Ngp
        
        gAcoor = GP(g,1).mAcoor;

        IN = GEN_08_Orthodom_Pin_ndec (gAcoor, aULims, pudim, 'closed', 6);
        
        % lies inside:
        if IN == 1
            
            br = CP(a,1).mIX;

            % evaluates b. function and its derivatives:
            [d1Ni, Ni] = z2702_d1NML (gAcoor, br, pNcp, pDegree, pKnotV, pudim);
            
            % accounts only if any derivative Or fucntion > 0
            if sum(abs(d1Ni)) > tol || abs(prod(Ni)) > tol
                
                CP(a,1).ON = 1 ;
                
                gNicp(g,1) = gNicp(g,1) + 1 ;
                gRicp(g,gNicp(g,1)) = a ;
                
                gBF(g,gNicp(g,1)) = prod(Ni) ;
                
                for di = 1:1:pudim
                    gdBFdir(g,gNicp(g,1),di) = d1Ni(di);
                    gBFdir (g,gNicp(g,1),di) = Ni(di);
                end
                
            end
            
          	%[Nabs,~,Rrel,Usingu] = p1209_basisFuncts ...
            %(pKnotV, pUsize, nAN, mA, pDegree, gNicp, gRicp, gAcoor, pudim);
            %[dRpa, dRpaRHS] = p1210_mdTpa (u, Usingu, eNicp, eRicpA, mA, pDegree, Nabs, pudim);

        end
         
    end
    
end



% NURBS basis functions and their derivatives w.r.t. parameters at Gauss
% points. Also allocates values to GP:

for g = 1:1:Ngp
    
    % Extracts GP info:
    
    Nicp = gNicp(g,1);
    Ricp = gRicp(g,1:gNicp(g));
    BF   = gBF(g,1:gNicp(g));
    
    dBFdir = zeros(gNicp(g),pudim);
    dBFdir(:,:)  = gdBFdir(g,:,:);
    
    BFdir = zeros(gNicp(g),pudim);
    BFdir(:,:)  = gBFdir(g,:,:);
    
    
    % Computes Bf's and drivatives:
    
    [R, dRpa] = a0112_RdRgp (Nicp, Ricp, BF, BFdir, dBFdir, pw, pudim);

        
    % Allocates:
        
    GP(g,1).mNicp = gNicp(g,1);
    
    for i = 1:1:gNicp(g,1)
        
        GP(g,1).mRicp(i,1) = gRicp(g,i);
        GP(g,1).Rrel(i,1) = R(i);
        
        for di = 1:1:pudim
            GP(g,1).dRpa(i,di) = dRpa(i,di);
        end
    end

    
end




% 6.- NO. ACTIVE ANCHORS & POSITION FOR STIFFNESS MATRIX:

Non = 0 ;
CPref = zeros(Nacp,1);

for a = 1:1:Nacp
if CP(a).ON == 1
    
    Non = Non + 1 ;
    CPref(a,1) = Non ;
    
end
end




end