function [MLT] = um0621_upMLT_DEL (MLT, ILn, eTHC)

% RETURNS: MLT with added end-node
 

% INPUTS:
% MLT:              initial
% Atn(2,pudim):     coordinates of nodes of missing line (the are existent)
% eXv(1,xdim), eAv(1,pudim): coordinates of new node to insert

% 1.- PRELIMINARIES:

pudim = 3 ;
xdim  = 3 ;

% extracts arrays to facilitate coding:

tNE = MLT.tNE ;
tNV = MLT.tNV ;
tNL = MLT.tNL ;
tTVcon = MLT.tTVcon ;
tLVcon = MLT.tLVcon ;
tVLcon = MLT.tVLcon ;
tTLcon = MLT.tTLcon ;
tVTcon = MLT.tVTcon ;

tBv  = MLT.tBv ;
tUv  = MLT.tUv ;
tAv  = MLT.tAv ;
tXv  = MLT.tXv ;

tANv = MLT.tANv ;
tXNv = MLT.tXNv ;
tANe = MLT.tANe ;
tXNe = MLT.tXNe ;
tNCS = MLT.tNCS ;

mn = [ ILn(1,2) ; ILn(2,2) ] ; % nodes refs of missing line in terms of MLT


% 2.- DETECTS LINE AND TRIANGLE OF MLT THAT CONTAINS mn POINTS:
   
% Line:

for i = 1:1:size(tVLcon,2) 
    
    LR = tVLcon(mn(1),i);
    
    if LR > 0
        if tLVcon(LR,1)==mn(2) || tLVcon(LR,2)==mn(2)
            break
        end
    end  
end
   


% Triangle

etR = 0 ;
for i = 1:1:size(tVTcon,2)
        
	tR = tVTcon(mn(1),i);
        
    if tR>0
        for j = 1:1:3

            nR = tTVcon(tR,j);

            if nR == mn(2)
                etR = tR;
                break
            end
        end
    end
        
	if etR>0
      	break
	end
end
 

% Stores refs of nodes, triangles and lines, the new ones at the rear:

inr = [ tTVcon(etR,1), tTVcon(etR,2), tTVcon(etR,3), tNV+1 ] ; 
itr = [ etR , tNE+1 ] ;        
ilr = [ LR, tNL+1 , tNL+2 ] ;

Nti = 2; % No. involved triangles (it is prepares only for 2 triangles, 
         % i.e. the new node lies on a contour curve)

Nni = 4 ; % No. involved nodes, considering the new inserted

Nli = 3 ; % No. lines involved

Nnli = 2 ; % No. new lines involved

% 3.- UPDATES COORDINATES AND NUMBERS:
    
nr = tNV + 1; % node to insert
 
tNE = tNE + 1 ;
tNV = tNV + 1 ;
tNL = tNL + 2 ;
    
% node is added at the end

tBv = [ tBv ; eTHC.Bv ] ;
tUv = [ tUv ; eTHC.Uv ] ;
tAv = [ tAv ; eTHC.Av ] ;
tXv = [ tXv ; eTHC.Xv ] ;

tNCS = [ tNCS ; zeros(1,2) ] ;

tNCS(tNV,1) = eTHC.NCS(1); 
tNCS(tNV,2) = eTHC.NCS(2);


% 4.- CONNECTIVITY MATRICES:

% 4.1.- TVcon :

TVe = [tTVcon(etR,1) , tTVcon(etR,2) , tTVcon(etR,3) , tTVcon(etR,1) ];

% selects index of nodes of etR just before the insertion

for i = 1:1:4
    
    if (TVe(i) == mn(1) && TVe(i+1) == mn(2)) ...
    || (TVe(i) == mn(2) && TVe(i+1) == mn(1))

        isd = i ; % index selected
        break
    end
    
end

if isd == 1
    a = [3 1];
    b = [2 3];
elseif isd == 2
    a = [1 2];
    b = [3 1];
elseif isd == 3
    a = [2 3];
    b = [1 2];
end

% temporal nodes order for elements

tmp1 = [ tTVcon(etR,a(1)) , tTVcon(etR,a(2)) , nr ] ;
tmp2 = [ nr ,  tTVcon(etR,b(1)) , tTVcon(etR,b(2)) ] ;

tTVcon = [tTVcon ; zeros(1,3) ];

for i = 1:1:3
	tTVcon(etR,i) = tmp1(i);
    tTVcon(tNE,i) = tmp2(i);
end


% 4.2.- LVcon:

tLVcon = [ tLVcon ; zeros(1,2) ] ;

tLVcon(ilr(1),1) = tmp1(2);
tLVcon(ilr(1),2) = nr;

tLVcon(tNL-1,1) = nr;
tLVcon(tNL-1,2) = tmp2(2);

tLVcon(tNL,1) = nr;
tLVcon(tNL,2) = tmp1(1);


% 4.3.- VLcon:
[tVLcon] = um0634_upVLcon (tVLcon, tLVcon, Nni, inr, Nli, ilr);


% 4.4.- TLcon :
[tTLcon] = um0633_upTLcon (tTLcon, tTVcon, tLVcon, etR, tNL, Nnli, Nti, itr);


% 4.5.- VTcon:
[tVTcon] = um0632_upTVcon (tVTcon, tTVcon, etR, Nti, itr, Nni, inr);


% 5.- VECTORS:
[tANv, tANe, tXNv, tXNe] = um0631_upNvects (tANv, tANe, tXNv, tXNe, tTVcon, tAv, tXv, Nti, itr);

 

% 6.- RE-ALLOCATES:

MLT.tNE = tNE;
MLT.tNV = tNV;
MLT.tNL = tNL;
MLT.tTVcon = tTVcon;
MLT.tLVcon = tLVcon;
MLT.tVLcon = tVLcon;
MLT.tTLcon = tTLcon;
MLT.tVTcon = tVTcon;

MLT.tBv = tBv;
MLT.tUv = tUv;
MLT.tAv = tAv;
MLT.tXv = tXv;

MLT.tANv = tANv;
MLT.tXNv = tXNv;
MLT.tANe = tANe;
MLT.tXNe = tXNe;
MLT.tNCS = tNCS ;



end

