function [MLT] = um0660_inNodMLT (MLT, eTHC, eAt, tolL, tolA, nd)

% RETURNS: MLT with added end-node
 
% INPUTS:
% MLT: initial
% eTHC: coordinates of new node to insert already pp to MLT surface
% eAt(pudim,1): coordinates of new insertion previous to pp. This is the
%               one used to find out in which triangle / edge the new point lies


% 1.- PRELIMINARIES:

pudim = 3 ;
xdim  = 3 ;

% extracts arrays to facilitate coding:

tNE = MLT.tNE ;
tNV = MLT.tNV ;
tNL = MLT.tNL ;
tTVcon = MLT.tTVcon ;
tLVcon = MLT.tLVcon ;
tVLcon = MLT.tVLcon ;
tTLcon = MLT.tTLcon ;
tVTcon = MLT.tVTcon ;

tBv  = MLT.tBv ;
tUv  = MLT.tUv ;
tAv  = MLT.tAv ;
tXv  = MLT.tXv ;

tANv = MLT.tANv ;
tXNv = MLT.tXNv ;
tANe = MLT.tANe ;
tXNe = MLT.tXNe ;
tNCS = MLT.tNCS ;


% 2.- DETECTS TRIANGLES and EDGES WHERE THE NEW NODE LIES IN:

% - If lies at edge, there are two triangles or one.
% - If lies within one triangle and no edge ref is stored, it is internal.

[Na, ate] = um0661_affTL (eAt, tNE, tTVcon,tTLcon, tLVcon, tAv, tolL, tolA, nd);


% 3.- CHARACTERIZATION OF INSERTION:

[inr, itr, ilr, Nti, Nni, Nli, Nnli, DNE, DNV, DNL] = um0662_ChrtIns (Na, ate, tNE, tNV, tNL, tTVcon);




% 4.- UPDATES COORDINATES AND NUMBERS:
    
nr = tNV + 1; % node to insert
 
tNE = tNE + DNE ;
tNV = tNV + DNV ;
tNL = tNL + DNL ;
    
% node is added at the end

tBv = [ tBv ; eTHC.Bv' ] ;
tUv = [ tUv ; eTHC.Uv' ] ;
tAv = [ tAv ; eTHC.Av' ] ;
tXv = [ tXv ; eTHC.Xv' ] ;

tNCS = [ tNCS ; zeros(1,2) ] ;

tNCS(tNV,1) = eTHC.NCS(1); 
tNCS(tNV,2) = eTHC.NCS(2);


% 5.- CONNECTIVITY MATRICES:


% 5.1.- TV and LVcon:
[tTVcon, tLVcon] = um0663_TVLVcon (tTVcon, tLVcon, tNE, tNL, Nti, itr, Nnli, ilr, Na, ate, nr);
                                   

% 5.2.- VLcon:
[tVLcon] = um0634_upVLcon (tVLcon, tLVcon, Nni, inr, ilr);


% 5.3.- TLcon :
[tTLcon] = um0633_upTLcon (tTLcon, tTVcon, tLVcon, Na, ate, ilr, Nti, itr, DNE);


% 5.4.- VTcon:
[tVTcon] = um0632_upTVcon (tVTcon, tTVcon, Na, ate, Nti, itr, Nni, inr);


% 6.- VECTORS:
[tANv, tANe, tXNv, tXNe] = um0631_upNvects (tANv, tANe, tXNv, tXNe, tTVcon, tAv, tXv, Nti, itr);

 

% 7.- RE-ALLOCATES:

MLT.tNE = tNE;
MLT.tNV = tNV;
MLT.tNL = tNL;
MLT.tTVcon = tTVcon;
MLT.tLVcon = tLVcon;
MLT.tVLcon = tVLcon;
MLT.tTLcon = tTLcon;
MLT.tVTcon = tVTcon;

MLT.tBv = tBv;
MLT.tUv = tUv;
MLT.tAv = tAv;
MLT.tXv = tXv;

MLT.tANv = tANv;
MLT.tXNv = tXNv;
MLT.tANe = tANe;
MLT.tXNe = tXNe;
MLT.tNCS = tNCS ;



end

