function [ESV] = fs0731_insertVtx (Spatches, Eskin, Nbe, tolS, DUtol, Xtol, ncuE, nloE, NcvE, SCconE)

% RETURNS:
% ESV: with tri-mesh nodes to achieve error < tolS (in patch param space).
% The approximation is by quadratic triangles.

% Stores vertexes of each skin in structure 'Entity Skin Vertexes'

ESV(Nbe,1) = struct('NVc', [] ,'NVi',0,  'Xvc', [], 'Xvi', [], 'Uvc', [] , 'Uvi', [] ); 


% NVc, NVi: No. vtx for contours (by curve) and internals
% ESV(b).Xvi(NVi,3)
% ESV(b).Xvc(nlo, ncu, NVc(nlo,ncu), 3)

% UVi is filled here, UVc later onse contour vertexes are selected, in
% another function

% NOTE !! NVc includes for each curve initial and final vertex !

xdim  = 3 ;
eudim = 2 ;
pudim = 3 ;

for b = 1:1:Nbe

    skin = Eskin(b,1);
    
    % 1.- EXTRACTS DATA FROM RELEVANT PATCH:
    
    pR = skin.mPa;
    
    pDegree = Spatches(pR,1).mRefin(1).mDegree ;
    pNcp    = Spatches(pR,1).mRefin(1).mNcp ;
    pKnotV  = Spatches(pR,1).mRefin(1).mKnotV ;
    pUsize  = Spatches(pR,1).mRefin(1).mUsize;
    pP      = Spatches(pR,1).mRefin(1).mP ;
    pw      = Spatches(pR,1).mRefin(1).mw ;
    pCD     = zeros(pudim,1);
    
    
    % 2.- EXTRACTS DATA FROM SKIN:

    NV = skin.mNV ;
    NE = skin.mNE ;

    TVcon = skin.mTVcon ; 
    LVcon = skin.mLVcon ; 
    TLcon = skin.mTLcon ;

    NLI = size(LVcon,1);
    NSL = ones(NLI,1); % No. segments per edge line due to refinement (1 means no division)

    Uv = skin.mUv ;
    Av = skin.mAv ;
    Xv = skin.mXv ;
    
    Nlv = skin.mNlv ;

    NVc = sum(Nlv); % these will increase
    NVi = NV-NVc;
    
    eDegree = skin.mDegree ;
    eNcp = skin.mNcp ;
    eKnotV = skin.mKnotV ;
    eUsize = skin.mUsize ;
    eP = skin.mP ;
    ew = skin.mw ;

    [Eew,EeP] = GEN_12_ExpandBw (ew, eP,xdim, eudim, eNcp);

    Uvi = Uv(NVc+1:NV, :);


    % 3.- REFINEMENT: computes error and triangle nodal distances to 
    % reduce error < tolS in patch param. space.

    % Measures error and includes necessary INTERNAL vtx to the triangles.
    % Required vtxs for EDGES triangles are stored in NSL array, later on
    % they are added considering the most populated option.

    for e = 1:1:NE

        % 3.1.- Data from triangle:

        [At, ~] = tm1306_tArea_n (TVcon, e, Av, 0, pudim); % area in patch param. space 
        
        Umid = zeros(eudim,1);
        Amid = zeros(pudim,1);

        u = zeros(eudim,1);
        a = zeros(pudim,1);

        Unod = zeros(3,eudim);
        Anod = zeros(3,pudim);

        for i = 1:1:3

            nR = TVcon(e,i);
            Unod(i,:) = Uv(nR,:);
            Anod(i,:) = Av(nR,:);

            u(:,1) = Uv(nR,:);
            Umid = Umid + u/3;

            a(:,1) = Av(nR,:);
            Amid = Amid + a/3;
        end


        % 3.2.- Computes Eadm and estimates error E2:
        
        L = (At/0.433)^0.5;      
        Eadm = L*tolS/100; % abs. error tolerance
        
        [dQ] = fs0716_estim3der ...
        (eKnotV, eNcp,eDegree,eUsize, Eew,EeP, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, ...
        Umid, Xtol, DUtol, pudim, eudim, xdim); % derivatives at triangle center
    
        [aEr] = fs0713_ErrorE2_simple (Du(1), Du(2), dQ, xdim); % error computation
    
        
        
        % 3.2.- Relative error and req'd distance bt'n nodes to reduce it to tolS:

        [rEr, Du0] = fs0712_Err_Dreqd ...
        (eDegree, eNcp, eKnotV, eUsize, EeP ,Eew, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, ...
        Umid, Amid, Unod, At, tolS, DUtol, Xtol);


    
        % 3.3.- Add internal nodes and updates partition to edges:

        if rEr > tolS   

            pL = 50 ; % limit in percentage to not consider internal vtx    
            [NSL, Uvi, NVi] = fs0718_upCorVtx1 (e, pL, Du0, Unod, NSL, Uvi, NVi, Uv, LVcon, TLcon);
        
        end
   
        m(e,1) = rEr;

    end


    
    % 4.- ADD INTERNAL LINES NODES: subdivide triangles lines according to NSL:
    
    [Uvi, NVi] = fs0715_upCorVtx2 (Uvi, NVi, Uv, LVcon, Nlv, NLI, NSL);
    
    
    
    % 5.- PHYSICAL COORD'S OF INTERNAL VXTS:

    Xvi = zeros(NVi,xdim);
    
    for i = 1:1:NVi
        
        u(:,1) = Uvi(i,:);
        [X] = b2601_BNURBS_Xcoord (u, eKnotV, eNcp,eDegree,eUsize, Eew, EeP, eudim,xdim);
        Xvi(i,:) = X(:,1);

    end
    
  
    
    % 6.- CONTOUR VTX.: physical coord's only
    
    [NVc, Xvc] = fs0719_addContLines (b, Uv, LVcon, NSL, nloE, ncuE, SCconE, NcvE, eKnotV, eNcp, eDegree, eUsize, Eew, EeP, eudim,xdim);
 
    
    
    % 7.- ALLOCATES TO STRUCTURE:
    
    ESV(b,1).NVc = NVc ;
    ESV(b,1).NVi = NVi ;
    
    ESV(b,1).Xvc = Xvc ;
    ESV(b,1).Xvi = Xvi ;
    ESV(b,1).Uvi = Uvi ;
    
    clearvars('NVc','NVi', 'Xvc', 'Xvi', 'Uvi');
    

end
  

end