function [ui, ust] = fs0703_sampleP3dcc (um, st, nd, cU, Nc)

% RETURNS:
% ui(2,4,sudim): two rows of 4 points to compute the 3rd derivative,
%                considering the contour of the surface
% ust(sudim,1): updated step used

% moves the sample points to compute the 3rd derivative in case they lie
% outside the contour:


% INPUTS:
% sd: selected direction
% pd: perpendicular direction

sudim = 2 ;

ui = zeros(sudim, 4, sudim);

pdm = [2 1];

for sd = 1:1:sudim % selected direction

    pd = pdm(sd); % perperndicular direction

    h = st(sd);
    
    % 1.- Finds closest distances to the contour in 'sd' direction:

    % 1.1.- Measures distances:
    
    srL = zeros(1,2); % selected refs and distace, at L and RHS
    srR = zeros(1,2);

    NL = 0 ;
    NR = 0;

    a1 = um ;
    u1 = [0;0]; u1(sd,1) = 1 ;

    a2 = zeros(sudim,1);
    u2 = zeros(sudim,1);


    for i = 1:1:Nc-1

        if cU(i,pd)<= um(pd) && um(pd)<=cU(i+1,pd)

            % intersection between lines:

            a2(:,1) = cU(i,:);
            for di = 1:1:sudim
                u2(di,1) = cU(i+1,di) - cU(i,di);
            end

            [IP,~] = go2530_LinInters_nD (a1,u1, a2,u2, sudim, nd);


            % stores:

            if IP(sd) < um(sd)

                NL = NL + 1 ;
                srL(NL,1) = i ;
                srL(NL,2) = norm(IP-um) ;

            else

                NR = NR + 1 ;
                srR(NR,1) = i ;
                srR(NR,2) = norm(IP-um) ;

            end
        end

    end



    % 1.2.- Selects closest distances:

    srL = sortrows(srL,2);
    srR = sortrows(srR,2);

    dL = srL(1,2);
    dR = srR(1,2);


    % 2.- Accomodates step length to intersections with contour:
    % Checks if closest distance in 'sd' direction is more than 1.5*h


    if  1.5*h < dL && 1.5*h < dR

        % distance can accomodate the steps

    elseif dL < 1.5*h || dR < 1.5*h

        % distance is shorter => redims h:

        if dL < 1.5*h

           ht = dL + 1.5*h ;

        elseif  dR < 1.5*h

            ht = 1.5*h + dR ;

        end

        h = ht/3;

    end

    % 3.- Computes points:

    ust(sd,1) = h ;
    
    ui(sd,1,sd) = round(um(sd) - 1.5*h, nd);
    ui(sd,2,sd) = round(um(sd) - 0.5*h, nd);
    ui(sd,3,sd) = round(um(sd) + 0.5*h, nd);
    ui(sd,4,sd) = round(um(sd) + 1.5*h, nd);

 	ui(sd,1,pd) = round(um(pd) , nd);
    ui(sd,2,pd) = round(um(pd) , nd);
    ui(sd,3,pd) = round(um(pd) , nd);
    ui(sd,4,pd) = round(um(pd) , nd);
    
end
    
end

