function [ESV] = fs0711_insertVtx ...
    (Spatches, Eskin, Nbe, Nts, tsR, tolS, DUtol, Xtol, ncuE, nloE, NcvE, SCconE, agCurvE)

% RETURNS:
% ESV: with tri-mesh nodes to achieve error < tolS (in patch param space).
% The approximation is by quadratic triangles.

% Stores vertexes of each skin in structure 'Entity Skin Vertexes'

ESV(Nbe,1) = struct('NVc', [] ,'NVi',0,  'Xvc', [], 'Xvi', [], 'Uvc', [] , 'Uvi', [] ); 

% NVc, NVi: No. vtx for contours (by curve) and internals
% ESV(b).Xvi(NVi,3)
% ESV(b).Xvc(nlo, ncu, NVc(nlo,ncu), 3)

% UVi is filled here, UVc later onse contour vertexes are selected, in
% another function

% NOTE !! NVc includes for each curve initial and final vertex !

xdim  = 3 ;
eudim = 2 ;
pudim = 3 ;

for j = 1:1:Nts

    sR = tsR(j,1);
    
    skin = Eskin(sR,1);
    
    % 1.- EXTRACTS DATA FROM RELEVANT PATCH:
    
    pR = tsR(j,2);

    pDegree = Spatches(pR,1).INS(1).mDegree ;
    pNcp    = Spatches(pR,1).INS(1).mNcp ;
    pKnotV  = Spatches(pR,1).INS(1).mKnotV ;
    pUsize  = Spatches(pR,1).INS(1).mUsize;
    pP      = Spatches(pR,1).INS(1).mP ;
    pw      = Spatches(pR,1).INS(1).mw ;
    pCD     = zeros(pudim,1);
    
    
    % param. limits and admisible error in absolute value
    
    pULims = zeros(xdim,2);
    Udiag = 0 ;
    
    for di = 1:1:pudim
        
        pULims(di,1) = pKnotV(di,1);
        pULims(di,2) = pKnotV(di,pUsize(di));
        
        Udiag = Udiag + (pULims(di,2) - pULims(di,1))^2 ;
        
    end
    Udiag = Udiag^0.5;
    
    Eadm = Udiag*tolS/100; 
    
    
    % 2.- EXTRACTS DATA FROM SKIN:

    NV = skin.mNV ;
    NE = skin.mNE ;

    TVcon = skin.mTVcon ; 
    LVcon = skin.mLVcon ; 
    TLcon = skin.mTLcon ;
    
    NLI = size(LVcon,1);
    NSL = ones(NLI,1); % No. segments per edge line due to refinement (1 means no division)

    Uv = skin.mUv ;
    Av = skin.mAv ;
    Xv = skin.mXv ;
    
    Nlv = skin.mNlv ;

    NVc = sum(Nlv); % these will increase
    NVi = NV-NVc;
    
    eDegree = skin.mDegree ;
    eNcp    = skin.mNcp ;
    eKnotV  = skin.mKnotV ;
    eUsize  = skin.mUsize ;
    eP      = skin.mP ;
    ew      = skin.mw ;

    [Eew,EeP] = GEN_12_ExpandBw (ew, eP,xdim, eudim, eNcp);

    eULims = [ eKnotV(1,1)  eKnotV(1,eUsize(1)) ; eKnotV(2,1)  eKnotV(2,eUsize(2)) ]; 
    

    Uvi = Uv(NVc+1:NV, :);

    % 3.- REFINEMENT: computes error and nodal distances to 
	% reduce error < tolS in patch param. space.

	% Measures error and includes necessary INTERNAL vtx to the triangles.
	% Required vtxs for EDGES of each triangle are stored in NSL array,
	% later on, in this same procedure they are included considering the 
	% most populated option.

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    fr = 1 ;
    pt1366_pTriLinMesh_nD ...
    (Xv, NLI, LVcon, TVcon, xdim, fr, 'k', 0, NV, 0, 0, 0, 0, 0, 0, 0, 0, 0);

    fr = 2 ;
    pt1366_pTriLinMesh_nD ...
    (Uv, NLI, LVcon, TVcon, eudim, fr, 'k', 0, NV, 0, 0, 0, 0, 0, 0, 0, 0, 0);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    

    for e = 1:1:NE


        % 3.1.- Data from triangle:

        Umid = zeros(eudim,1);
        Amid = zeros(pudim,1);

        u = zeros(eudim,1);
        a = zeros(pudim,1);

        Unod = zeros(3,eudim);
        Anod = zeros(3,pudim);

        for i = 1:1:3

                nR = TVcon(e,i);
                Unod(i,:) = Uv(nR,:);
                Anod(i,:) = Av(nR,:);

                u(:,1) = Uv(nR,:);
                Umid = Umid + u/3;

                a(:,1) = Av(nR,:);
                Amid = Amid + a/3;
        end


        % 3.2.- Relative error and req'd distance bt'n nodes to reduce it to tolS:

        [E2, Du0] = fs0712_Err_Du0 ...
        (eULims, eDegree, eNcp, eKnotV, eUsize, EeP ,Eew, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, ...
        Umid, Unod, Eadm, DUtol, Xtol);

        
        % 3.3.- Add internal nodes and updates partition to edges:

        if E2 > Eadm   
                
            pL = 30 ; % limit in percentage to not consider internal vtx    
            [NSL, Uvi, NVi] = fs0718_upCorVtx1 (e, pL, Du0, Unod, NSL, Uvi, NVi, Uv, LVcon, TLcon);

        end

        m(e,1) = E2;
        m(e,2) = e;
    end


	m = sortrows(m,1);
       


	% 4.- ADD INTERNAL LINES NODES: subdivide triangles lines according to NSL:

	[Uvi, NVi] = fs0715_upCorVtx2 (Uvi, NVi, Uv, LVcon, Nlv, NLI, NSL);


    
	% 5.- PHYSICAL COORD'S OF INTERNAL VXTS:

	Xvi = zeros(NVi,xdim);

    for i = 1:1:NVi

        u(:,1) = Uvi(i,:);
        [X] = b2601_BNURBS_Xcoord (u, eKnotV, eNcp, eDegree, eUsize, Eew, EeP, eudim, xdim);
        Xvi(i,:) = X(:,1);

    end

  

	% 6.- CONTOUR VTX.: physical coord's only

	[NVc, Xvc] = fs0719_addContLines (sR, Xv, LVcon, NSL, nloE, ncuE, SCconE, NcvE, agCurvE, Xtol, DUtol);
  
        

    
    %{
    if sum(Tr) == 0
        
        % UNTRIMMED SURFACES => just copy
        
        Uvi = Uv(NVc+1:NV, :);
        Xvi = Xv(NVc+1:NV, :);
        
        NVc = zeros(nloE(sR,1),1);
        Xvc = zeros(nloE(sR,1),1,1,xdim);
        
        for lo = 1:1:nloE(sR,1)
            
            for cu = 1:1:ncuE(sR,lo)
            
                cR = abs(SCconE(sR,lo,cu));
                
                NVc(lo,cu) = NcvE(cR,1);
                
                for i = 1:1:NcvE(cR)
                
                    for di = 1:1:xdim
                        Xvc(lo,cu,i,di) = cLXvE(cR,i,di);
                    end
                end
            end
        end
        
        
    end
    %}
    
    % 7.- ALLOCATES TO STRUCTURE:
    
    ESV(sR,1).NVc = NVc ;
    ESV(sR,1).NVi = NVi ;
    
    ESV(sR,1).Xvc = Xvc ;
    ESV(sR,1).Xvi = Xvi ;
    ESV(sR,1).Uvi = Uvi ;
    
    clearvars('NVc', 'NVi', 'Xvc', 'Xvi', 'Uvi');
    

end
  







end