function [aEr, A,B,C,D ] = fs0713b_ErrorE2 (D1, D2, du, xdim, aEtol)

% RETURNS:
% nEr: abs. error btw estimated and actual surface coordinates for order 2 (norm value)
% A,B,C,D: factors of cubic equation to estimate the longest new increment


% Increments products:

Dxxx = 3/64*D1^3 ;
Diii = 3/64*D2^3 ;
Dxxi = 1/64*D1^2*D2 ;
Diix = 1/64*D2^2*D1 ;


% Errors for the 4 combinations:

Er = zeros(xdim,4); 
    
sB = [ 1  1 -1 -1 ]; % combinations signs for factors
sC = [ 1 -1  1 -1 ];

for di = 1:1:xdim
            
    fxxx = du(4,1,di) ;
    fiii = du(1,4,di) ;
    fxxi = du(3,2,di) ;
    fiix = du(2,3,di) ;

    for c = 1:1:4
        Er(di,c) = (Dxxx*fxxx + sB(c)*3 * Dxxi*fxxi + sC(c)*3 * Diix*fiix + Diii*fiii)/6;
    end
end
        

% Largest error norm of the 4 combinations and A,B,C,D to estimate D1 or D2 (the longest one):
        
aEr = 0 ;
   
for c = 1:1:4
            
    Erc = [Er(1,c); Er(2,c); Er(3,c)];

    if norm(Erc) >= aEr
        
        aEr = norm(Erc);
        
        if D1 > D2 % x is D1
            
            A = fxxx;
            B = sB(c)*fxxi*D2;
            C = sC(c)*fiix*D2^2/4 ;
            D = fiii*D2^3/8-128*aEtol;
            
            if round(A,6)==0 && round(B,6)==0 && round(C,6)==0
                
                A = fiii;
                D = -128*aEtol;
                
            end
            
            
        else  % x is D2
            
          	A = fiii;
            B = sC(c)*fiix*D1;
            C = sB(c)*fxxi*D1^2/4 ;
            D = fxxx*D1^3/8-128*aEtol;
            
         	if round(A,6)==0 && round(B,6)==0 && round(C,6)==0
                
                A = fxxx;
                D = -128*aEtol;
                
            end
            
        end
        
    end

end

end

