function [dQ] = fs0716_estim3der ...
    (eULims, eKnotV, eNcp,eDegree,eUsize, Eew,EeP, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, ...
    Umid, Xtol, DUtol, pudim, eudim, xdim)

% RETURNS:
% dQ(4,pudim): 3rd derivatives of surface Q (in patch physical space)
% Qrrr, Qrrs, Qssr, Qsss

% the surface on patch param. space is called 'Q'
dQ = zeros(4,3);  % stores derivatives in array: cols =3 : No. dims in patch param. space

% sample points location in surface param. space

Nsp = 16 ;
Usp = zeros(Nsp,eudim); 

f = [-1.5 -1 -0.5 0   0.5 0 -0.5 -1   -0.5 0 0.5 1   1.5 1 0.5 0 ;
        0 0.5 1 1.5   1 0.5 0 -0.5    -1 -0.5 0 0.5  0 -0.5 -1 -1.5 ]';
    
endF = 0;
du = DUtol*20; % step to estimate derivatives

while endF == 0
    
    endF = 1;
    
    for i = 1:1:Nsp
        
        for di = 1:1:eudim
            Usp(i,di) = Umid(di) + f(i,di)*du;
            
            if Usp(i,di) < eULims(di,1) ||eULims(di,2) < Usp(i,di)
                endF=0;
                break
            end
        end
        
        if endF==0
            break
        end
        
    end

    if endF==0
        du = du/2;
    end
    
    
end



% phys. space
Xsp = zeros(Nsp,xdim); 

u = zeros(eudim,1);

for i = 1:1:Nsp
    
    u(:,1) = Usp(i,:);
    x = b2601_BNURBS_Xcoord (u, eKnotV, eNcp,eDegree,eUsize, Eew,EeP, eudim, xdim);
    Xsp(i,:) = x(:,1);
    
end


% patch param. space

Asp = v2861b_PPvol (Xsp, Nsp, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, xdim, Xtol, DUtol);

for di = 1:1:pudim
    dQ(1,di) = 1/du^3 * ( -Asp(1,di)  + 3*Asp(7,di)   - 3* Asp(11,di)  + Asp(13,di));
    dQ(2,di) = 1/du^3 * ( (Asp(2,di)  -   Asp(8,di))  - 2*(Asp(6,di)  - Asp(10,di)) + (Asp(12,di) - Asp(14,di)));
    dQ(3,di) = 1/du^3 * ( (Asp(15,di) -   Asp(9,di))  - 2*(Asp(11,di) - Asp(7,di))  + (Asp(5,di)  - Asp(3,di)));
    dQ(4,di) = 1/du^3 * (  -Asp(16,di) + 3*Asp(10,di) - 3* Asp(6,di)   + Asp(4,di));
end




%{
%%%%%%%%%%%%%%%%%%%%%%%%%%

ord = [3 3];

[ew,eP] = GEN_12_ContractsBw (Eew, EeP, xdim, eudim, eNcp);

[dS] = b2701_BNURBS_deriv (eDegree, eNcp, eKnotV, eUsize, eP ,ew, ord, Umid, xdim, eudim);

S = zeros(3,1);

Sx = zeros(3,1);
Si = zeros(3,1);

dS1 = zeros(3,1);
dS2 = zeros(3,1);
dS3 = zeros(3,1);
dS4 = zeros(3,1);

S(:,1) = dS(1,1,:);

Sx(:,1) = dS(2,1,:);
Si(:,1) = dS(1,2,:);

dS1(:,1) = dS(4,1,:);
dS2(:,1) = dS(3,2,:);
dS3(:,1) = dS(2,3,:);
dS4(:,1) = dS(1,4,:);



for di = 1:1:3
    dX(1,di) = 1/Dr^3 * ( -Xsp(1,di)  + 3*Xsp(7,di)   - 3* Xsp(11,di)  + Xsp(13,di));
    dX(2,di) = 1/Dr^3 * ( (Xsp(2,di)  -   Xsp(8,di))  - 2*(Xsp(6,di)  - Xsp(10,di)) + (Xsp(12,di) - Xsp(14,di)));
    dX(3,di) = 1/Dr^3 * ( (Xsp(15,di) -   Xsp(9,di))  - 2*(Xsp(11,di) - Xsp(7,di))  + (Xsp(5,di)  - Xsp(3,di)));
    dX(4,di) = 1/Dr^3 * (  -Xsp(16,di) + 3*Xsp(10,di) - 3* Xsp(6,di)   + Xsp(4,di));
end

format long

[dS1' ; dS2'; dS3'; dS4']

dX
dQ

format short


figure(10)
scatter3( Xsp(:,1),  Xsp(:,2),  Xsp(:,3), 'x'); hold on
axis equal

if ggg==0
end

%%%%%%%%%%%%%%%%%%%%%%%%%%

%}


end

