function [NSL, Uvi, NVi] = fs0718_upCorVtx1 (eR, pL, Dur, Unod, NSL, Uvi, NVi, Uv, LVcon, TLcon)

% RETURNS:
% NSL(NLI,1): updated No. segment per line, if =1 means no division
% Uvi(NVi,2): updated vtx param. coordinates, adding at the rear new vtx.
%           internal to the triangle (vtx at triangle edge are still not added , just
%           stored in NSL).
% NVi: updated No. vtx, considering only new internal vtx. to the triangle
% 


Dur = 2*Dur; % doubles nodal spacing (coz these are quadratic triangles)
dD = norm(Dur); % diagonal


% envelope rectangular grid:

eLims = [min(Unod(:,1)) max(Unod(:,1)) ; min(Unod(:,2)) max(Unod(:,2)) ] ;

[P] = GEN_20_RectGrid_2D (eLims, Dur, 2);



% add nodes internals and separated more than dD*pL/100 from triang. edge

u = zeros(2,1);
nUv = zeros(1,2); % new U coord's

conta = 0 ;

for i = 1:1:size(P,1)
for j = 1:1:size(P,2)
    
    u(:,1) = P(i,j,:);
    
    [IN, dist, upr] = tr1344_PandTri (u, Unod, 'open', 3);

    %IN = 0 ; %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if IN==1 && dist > dD*pL/100
        
        conta = conta + 1;
        nUv(conta,:) = u(:,1);
        
        %{
        % for paper
        if eR < 3
            %{
            fr2 = 15 ;
            figure(fr2)
            scatter(u(1), u(2), 'r'); hold on
            %}
        end
        %}
        
    end
end
end


% for paper: prints 2 elems toguether with valid nodes
if eR < 3
    %{
    figure(fr2)
    
    pt1361_plotTri_nD (Unod, 2, fr2, 'k');
    
    axis equal
    %}
end


% adds new vertexes to current (adds at the end of Uv):

if conta > 0 
    Uvi = [Uvi; nUv];
    NVi = NVi + conta ;
end


% for paper: prints candiadates nodes for triangles
if eR < 3
    %{
    fr = eR + 10 ;
    figure(fr)

    for i = 1:1:size(P,1)
    for j = 1:1:size(P,2)

        scatter(P(i,j,1), P(i,j,2), 'r'); hold on

    end
    end
    
    pt1361_plotTri_nD (Unod, 2, fr, 'k');
    
    axis equal
    %}
end

% Computes No. segments in correspondent lines and stores if is greater than current number:


for l = 1:1:3
    
    lR = TLcon(eR,l);
    
    u1 = zeros(2,1);
    u2 = zeros(2,1);
    
    vR = LVcon(lR,1);
  	u1(:,1) = Uv(vR,:);
        
  	vR = LVcon(lR,2);
  	u2(:,1) = Uv(vR,:);

    V = u2-u1; % orientation versor
    V = round(V/norm(V),3);
    

    
    % computes lambda vector:
    % intersection with vert at Du(1) and horiz at Du(2), and chooses the shortest 
    
    if abs(V(1))==1
        
        Lv = Dur(1);
        
    elseif abs(V(2))==1
        
        Lv = Dur(2);
        
    else
        
        t1 = Dur(1)/V(1) ;
        t2 = Dur(2)/V(2) ; 
        
        x1 = [ V(1) * t1 ; V(2) * t1 ];
        x2 = [ V(1) * t2 ; V(2) * t2 ];

        Lv = min(norm(x1), norm(x2));

    end
    
    
    [N, sd] = go2531_DivSegment (u1, u2, 2, Lv, 2, 1);

    if N-1 > NSL(lR,1)
        NSL(lR,1) = N-1;
    end
    
    %NSL(lR,1) = 1 ;  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % for paper: prints edge nodes
    if eR < 3
        %{
        fr = eR + 10 ;
        figure(fr)

        scatter(sd(:,1), sd(:,2), 'b'); hold on

        axis equal
        %}
    end
    
    if eR < 3
        %{
        
        figure(fr2)
        
        if lR~=9

            scatter(sd(:,1), sd(:,2), 'b'); hold on
            
        elseif lR==9 && eR==2

            scatter(sd(:,1), sd(:,2), 'b'); hold on
        end
        
        %}
    end
    
    
end



end