function [tE2, tDu] = fs0720_ErrT (Spatches, Nts, tsR, Tskin, tolA, Xtol, DUtol)

% RETURNS: for each triangle of each Tskin:
% tE2(Nts,eudim): relative error to reduce below tolerance
% tDu(Nts,eudim,2):  required distance in surface param space to reduce below tolerance

% INPUTS:

eudim = 2 ;
pudim = 3 ;
xdim = 3 ;

tE2 = zeros(Nts,1);
tDu = zeros(Nts,1,2);

for j = 1:1:Nts

    sR = tsR(j,1);
    
    skin = Tskin(sR,1);
    
    % 1.- EXTRACTS DATA FROM RELEVANT PATCH:
    
    pR = tsR(j,2);

    pDegree = Spatches(pR,1).INS(1).mDegree ;
    pNcp    = Spatches(pR,1).INS(1).mNcp ;
    pKnotV  = Spatches(pR,1).INS(1).mKnotV ;
    pUsize  = Spatches(pR,1).INS(1).mUsize;
    pP      = Spatches(pR,1).INS(1).mP ;
    pw      = Spatches(pR,1).INS(1).mw ;
    pCD     = zeros(pudim,1);
    
    
    % param. limits and admisible error in absolute value
    
    pULims = zeros(xdim,2);
    Udiag = 0 ;
    
    for di = 1:1:pudim
        
        pULims(di,1) = pKnotV(di,1);
        pULims(di,2) = pKnotV(di,pUsize(di));
        
        Udiag = Udiag + (pULims(di,2) - pULims(di,1))^2 ;
        
    end
    Udiag = Udiag^0.5;
    
    Eadm = Udiag*tolA/100; 
    
    
    % 2.- EXTRACTS DATA FROM SKIN:


    NE = skin.mNE ;

    TVcon = skin.mTVcon ; 

    Uv = skin.mUv ;
    Av = skin.mAv ;

    eDegree = skin.mDegree ;
    eNcp    = skin.mNcp ;
    eKnotV  = skin.mKnotV ;
    eUsize  = skin.mUsize ;
    eP      = skin.mP ;
    ew      = skin.mw ;

    [eEw,eEP] = GEN_12_ExpandBw (ew, eP,xdim, eudim, eNcp);

    eULims = [ eKnotV(1,1)  eKnotV(1,eUsize(1)) ; eKnotV(2,1)  eKnotV(2,eUsize(2)) ]; 
    

    % 3.- COMPUTES ERROR AND DISTANCE to 
	% reduce error < tolS in patch param. space.

    for e = 1:1:NE

        % 3.1.- Data from triangle:

        Umid = zeros(eudim,1);
        Amid = zeros(pudim,1);

        u = zeros(eudim,1);
        a = zeros(pudim,1);

        Unod = zeros(3,eudim);
        Anod = zeros(3,pudim);

        for i = 1:1:3

            nR = TVcon(e,i);
            Unod(i,:) = Uv(nR,:);
            Anod(i,:) = Av(nR,:);

            u(:,1) = Uv(nR,:);
        	Umid = Umid + u/3;

         	a(:,1) = Av(nR,:);
        	Amid = Amid + a/3;
        end


        % 3.2.- Relative error and req'd distance bt'n nodes to reduce it to tolS:

        [E2, Du0] = fs0712_Err_Du0 ...
        (eULims, eDegree, eNcp, eKnotV, eUsize, eEP ,eEw, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, ...
        Umid, Unod, Eadm, DUtol, Xtol);


        tE2(j,e) = E2 ;
        tDu(j,e,1) = Du0(1) ;
        tDu(j,e,2) = Du0(2) ;
        
    end


end


end

