function [Nn, Uin] = fs0744_nodesIN_DEL (tNs, tUs, Nc, cU, NL, pc)

% RETURNS:
% Nn: No. nodes within the contour cU and with longer distance to contour than md
% Uin(Nn,sudim): coordinates of those nodes

% INPUTS:
% tNs:              No. nodes to check
% tUs(tNs,sudim):   their coordinates
% Nc:               No. nodes of the contour, the first and last node coincides
% cU(NL,Nc, sudim):    contour nodes coordinates
% NL: No. loops
% pc: percentaje of envelope distance to compute md

sudim = 2 ;

Nn = 0 ;
Uin = zeros(1,sudim);

cLim = [min(cU(1,:,1)) max(cU(1,:,1)) ; min(cU(1,:,2)) max(cU(1,:,2)) ];


% 1.- Constraint polygons (as many as NL):

xv = cU(1,1:Nc(1),1);
yv = cU(1,1:Nc(1),2);

if NL > 1
    for i = 2:1:NL
        
        xt =  cU(i,1:Nc(i),1);
        yt =  cU(i,1:Nc(i),2);
        
        xv = [xv NaN xt ] ;
        yv = [yv NaN yt ] ;
        
    end
end



% 2.- Determines which are IN:

xq = tUs(1:tNs,1);
yq = tUs(1:tNs,2);

[in] = inpolygon(xq,yq,xv,yv);



% 3.- Checks if distance is more than md on those that are IN:

md = zeros(2,1);

md(1) = pc/100 * (cLim(1,2) - cLim(1,1));
md(2) = pc/100 * (cLim(2,2) - cLim(2,1));

uc = zeros(sudim,1);
ax = zeros(sudim,1); % aux. vectors
Pr = zeros(sudim,1); 

for i = 1:1:tNs
    
    if in(i)==1
        
        uc(:,1) = tUs(i,:); % candidate

        % closest distances to the contour: 
        
        dmin  = 9999999;
        XYmin = ones(sudim,1)*99999;    % vertically and horizontally
        vmin  = zeros(sudim,1);         % to closest point
        
        for L = 1:1:NL
        
            rn = 0 ;
            
            for j = 1:1:Nc(L)-1

                % 3.1.- Closest location:
                
                Pr(:,1) = cU(L,j,:);
                ax(:,1) = cU(L,j+1,:);
                vr = ax-Pr;

                [dL, v] = go2527_distPtoRecta (uc, Pr, vr, sudim);
                d1 = norm(ax-uc);
                d2 = norm(Pr-uc);
                
                % checks if the closest point of the line is within the segment:
                
                Q = uc+v;
                
                [ON] = go2520_PonSegment (Q, Pr,ax, sudim, 5);
                
                if ON==1
                    dist = dL ;
                else
                    dist = min(d1, d2);
                end
                
                % allocates in case dist is less:
                
                if dist < dmin 
                    rn = j ;
                    dmin = dist ;
                    vmin = abs(v) ;
                end

            end

        end
        
  
        % 3.2.- Checks distances and allocates if required:
        
        if dmin > min(md)
            Nn = Nn + 1 ;
            Uin(Nn,:) = uc(:,1);
        end
 
    end
    
end



end

