function [NE, NV, NVm, Uvm, Avm, Xvm, TVcon, VTcon, LVcon, LTcon, VLcon, TLcon, Nc] = fs0751_trianCnst...
            (fNcn, fcXn, fcAn, eNcn, ecXn, ecAn, fNsn, fsUn, fsXn, fsAn, tFace, sR, Spatches, pR, ccTp, Xtol, DUtol)

ncuT    = ccTp(pR,1).ncu;
NL      = ccTp(pR,1).nlo(sR,1);
SCconT  = ccTp(pR,1).SCcon;

        
% 1.- CONTOURS WITH ALL NODES AND WITH ONLY END-NODES:  

% cU(NL, Nc, sudim) are the coords in surface parameter space
% Nc(NL,1): No. conotur points per loop, remember 1st and last are repeated

[Nc, cX, cA, cU]     = fs0792_Sconto (pR, sR, tFace, fNcn, fcXn, fcAn, NL, ncuT, SCconT, Xtol, DUtol);
[eNc, ecX, ecA, ecU] = fs0792_Sconto (pR, sR, tFace, eNcn, ecXn, ecAn, NL, ncuT, SCconT, Xtol, DUtol);
    
     
% 1.- GENERATES MERGED ARRAY OF NODES AND CONSTRAINTS FOR TRIANGULATION:
% Note the coordiantes do not include at the moment the mid-nodes

[Uvm, Avm, Xvm, Cnst] = fs0752_merX_Cnst (ecU, ecA, ecX, eNc, NL, fNsn, fsUn, fsXn, fsAn, sR);



% 2.- PRELIMINARY TRIANGULATION (ALL TRIANGLES):

NV = size(Uvm,1);
DT = delaunayTriangulation(Uvm,Cnst);
tTVcon = DT.ConnectivityList;
tNE = size(tTVcon,1);

% plots prelim. triangulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{

if sR==4
    
    fr = 9 ;
    
    figure(fr)

    Cv = Avm ;
    dim = size(Cv,2);
    
    if dim==2
        scatter( Cv(:,1),  Cv(:,2), 'r'); hold on
    elseif dim==3
        scatter3( Cv(:,1),  Cv(:,2),  Cv(:,3), 'r'); hold on
    end
    
    

    axis equal

    for t = 1:1:tNE
        
        Cnod = zeros(3,dim);
        
        for i = 1:1:3
            Cnod(i,:) = Cv(tTVcon(t,i),:);
        end
        
        pt1361_plotTri_nD (Cnod, dim, fr, 'k') ;
    end
    
    
    if gg==0
    end

end


%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% 3.- REMOVES TRIANGLES THAT ARE QUASI-PLANE OR OUTSIDE Cnst:

[NE, TVcon, ~] = fs0753_INtri (DT, tNE, NV, NL, Nc, cU, Uvm, tTVcon);

% plots after removal triangulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{

if sR==4
    
    fr = 11 ;
    
    figure(fr)

    Cv = Uvm ;
    dim = size(Cv,2);
    
    if dim==2
        scatter( Cv(:,1),  Cv(:,2), 'r'); hold on
    elseif dim==3
        scatter3( Cv(:,1),  Cv(:,2),  Cv(:,3), 'r'); hold on
    end
    
    

    axis equal

    for t = 1:1:NE
        
        Cnod = zeros(3,dim);
        
        for i = 1:1:3
            Cnod(i,:) = Cv(TVcon(t,i),:);
        end
        
        pt1361_plotTri_nD (Cnod, dim, fr, 'k') ;
    end
    
    
    if gg==0
    end

end


%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% 4.- OTHER CONN. MATRICES:

Nlv = zeros(NL,1);
for L = 1:1:NL
    Nlv(L,1)= eNc(L,1)-1;
end

[LVcon, LTcon, VLcon, TLcon, VTcon] = tr1347_otherConn (NL, Nlv, NE, NV, TVcon, Uvm);

LVcon1 = LVcon ;
TVcon1 = TVcon ;
NE1 = NE ;

conn = struct ('TV',[], 'LV',[], 'LT',[], 'VL',[], 'TL',[],'VT',[]);

conn.TV = TVcon;
conn.LV = LVcon;
conn.LT = LTcon;
conn.VL = VLcon;
conn.TL = TLcon;
conn.VT = VTcon;



% 5.- IMPROVEMENT OF TRIANGLES IN PATCH PARAMETER SPACE:
% here triangles are still linear:

%{

[TVcon, VTcon, LVcon, LTcon, VLcon, TLcon] = fs0706_IMPRVtri ...
    (sR, pR, tFace, Spatches, TVcon, VTcon, LVcon, LTcon, VLcon, TLcon, Avm, Uvm, NL, eNc);

LVcon2 = LVcon ;
TVcon2 = TVcon ;
NE2 = NE ;
%}

% plots after improvement
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{

if sR==4
    
    fr = 13 ;
    
    figure(fr)

    Cv = Avm ;
    dim = size(Cv,2);
    
    if dim==2
        scatter( Cv(:,1),  Cv(:,2), 'r'); hold on
    elseif dim==3
        scatter3( Cv(:,1),  Cv(:,2),  Cv(:,3), 'r'); hold on
    end
    
    

    axis equal

    for t = 1:1:NE
        
        Cnod = zeros(3,dim);
        
        for i = 1:1:3
            Cnod(i,:) = Cv(TVcon(t,i),:);
        end
        
        pt1361_plotTri_nD (Cnod, dim, fr, 'k') ;
    end
    
    
    if gg==0
    end

end


%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 6.- INSERTS MID-NODES & IMPROVES:
% At contours it retrieves from contour curves, at surface adds just at the U-middle

[NVm, Uvm, Avm, Xvm, econn] = fs0761_finalizeT ...
(NV, Uvm, Avm, Xvm, conn, NE, NL, eNc, cU, cX, cA, Nc, Nlv, tFace, sR, Spatches, pR, Xtol, DUtol);

TVcon = econn.TV ;
VTcon = econn.VT ;
LVcon = econn.LV ;
LTcon = econn.LT ;
VLcon = econn.VL ;
TLcon = econn.TL ;

% 7.- PLOTTING (only for explanations):

%{

ssR = 4 ; % selected surface reference

Cvm = Avm ;

Cv = Cvm(1:NV,:) ;

fr = [ 9 10 20 30 ] ;

dim = size(Cv,2);
NLI = size(LVcon,1);




if sR == ssR
    
    
    % 7.1.- End-nodes:
    
    for i = 1:1:size(fr,2)
    
        Rv = Cv;
        if i==size(fr,2)
            Rv = Cvm;
        end
        
        figure(fr(i))

        if dim==2
            scatter( Rv(:,1),  Rv(:,2),  'b'); hold on 
        elseif dim==3
            scatter3( Rv(:,1),  Rv(:,2), Rv(:,3), 'b'); hold on 
        end

        axis equal
    
    end
    

    % 7.2.- Triangulation 0 : with all triangles, including the outsiders

    for e = 1:1:tNE

        unod = zeros(3,dim);

        for i=1:1:3
            unod(i,:) = Cv(tTVcon(e,i),:);
        end

        pt1361_plotTri_nD (unod, dim, fr(1), 'k');

    end

    % contour:
    vacc = 0 ;
    for L = 1:1:NL
        
        x = zeros(eNc(L)-1,dim);
        
        for v = 1:1:eNc(L) -1
 
            vacc = vacc + 1 ;
            x (v,:) = Cv(vacc,:);
        end
        
        if dim==2
            plot( x(:,1), x(:,2),'r', 'LineWidth',2);
            plot( [x(end,1) x(1,1)], [x(end,2) x(1,2)],'r', 'LineWidth', 3); 
        elseif dim==3
            plot3( x(:,1), x(:,2), x(:,3),'r', 'LineWidth',2);
            plot3( [x(end,1) x(1,1)], [x(end,2) x(1,2)], [x(end,3) x(1,3)],'r', 'LineWidth', 3); 
        end
    end
    
    
    % 7.3.- Triangulation 1: with no outside triangles:

    Mcolr = 'k'; 
    Mthk = 0.5 ;
    
    nvF = 0;
    NV = 0;
    Nvx = 0;
    neF = 0;
    Nel = 0;
    sc = 0;
    Ncolr = 0;
    lrF = 0;
    vrF = 0;
    erF = 0;
    
     pt1366_pTriLinMesh_nD ...
    (Cv, NLI, LVcon1, TVcon1, dim, fr(2), Mcolr, Mthk, nvF, NV, Nvx, neF, NE1, Nel, sc, Ncolr, lrF, vrF, erF)


    % 7.4.- Triangulation 2: improved in patch parameter space

    %{
    Mcolr = 'k';
    Mthk = 0.5 ;
    
    nvF = 0;
    NV = 0;
    Nvx = 0;
    neF = 0;
    Nel = 0;
    sc = 0;
    Ncolr = 0;
    lrF = 0;
    vrF = 0;
    erF = 0;
    
     pt1366_pTriLinMesh_nD ...
    (Cv, NLI, econn.LV, econn.TV, dim, fr(3), Mcolr, Mthk, nvF, NV, Nvx, neF, NE2, Nel, sc, Ncolr, lrF, vrF, erF)
    %}

    % 7.5.- Triangulation 3: mid-nodes inserted
    
    stL = 0.1 ;
    stT = 0.1 ;
    
	eCol = 'k';

    eF = 1;
    sF = 0;
    nF = 0;

    lrF = 1;
    vrF = 0;
    erF = 0;
    
    
    pt1367_pTriQuaMesh ...
    (stL, stT, Cvm, NLI, econn.LV, econn.TV, fr(4), NE, eF, sF, nF, lrF, vrF, erF, dim, eCol);


end


%}


end