function [NV, NE, Bvm, Uvm, Avm, Xvm, conn, NL, Nlv, mNC] = fs0754_IMPRVtri1 ...
         (NV, NE, Bvm, Uvm, Avm, Xvm, conn, NL, Nlv, mNC, ccTp, pR, contaL)

% RETURNS: improved triangulation in patch parameter space

% Swap edge-line between 2 triangles if:
% - the alternative line is shorter
% and
% - the resultant triangles improve the regularity (condition number)

pudim = 3 ;
sudim = 2 ;

% 1.- PRELIMINARIES:

endF = 0 ;  % ends flag turns to 0 if changes happen

% No. lines and verteces:
NLI = size(conn.LT,1);


% 2.- LOOP ON LINES TO ANALYZE PAIRS OF TRIANGLES:

while endF == 0

    endF = 1 ;
    
    LRc = 0 ; % stores temporarily the lastest changed line ref.

    for LR = 1:1:NLI

        % triangles refs and nodes involved:
        [tR, nR] = fs0755_NTrefs (conn, LR);

        if tR(2) > 0 && LR~=LRc % It does not consider: edge-lines and LR = previously changed LR

            
            % 2.1.- Diagonals lengths:

            Lc = 0 ;
            La = 0 ;
            
            for di = 1:1:pudim
                Lc = Lc + (Avm(nR(1),di) - Avm(nR(3),di))^2 ;
                La = La + (Avm(nR(2),di) - Avm(nR(4),di))^2 ;
            end
            
            Lc = Lc^0.5 ;
            La = La^0.5 ;

            
            % 2.2.- Regularity of the triangles:
            
            [roc, roa] = fs0756_regTri (tR, nR, Avm);
            
 
            % 2.3.- All alternative nodes not at contour curves:

            coF = fs0757_alterContr(nR, mNC, Bvm, Xvm, ccTp, pR) ; % contour flag

            
            % 2.4.- Carries on to second condition if :
            % - Lc/La ratio is 1.00 or larger  AND
            % - alternative mid-point is not at contour (coF = 1)

            if (La/Lc < 1 && coF==0) && (sum(roc)/sum(roa)<1 && min(roc)/min(roa)<1)

                % Updates conn. matrices:
                [conn] = fs0758_upconnLin (NV, conn, tR, nR, LR);

                endF = 0 ; % ending flag turns to 0 
                LRc = LR; % line ref. changed
                break

            end
            
            
        end
    end

    
end


end

