function [NVm, Bvm, Uvm, Avm, Xvm, econn, mNC] = fs0762_iMnodes ...
    (NLI, NVm, Bvm, Uvm, Avm, Xvm, conn, mNC, NE, NL, CNT, tFace, sR, Spatches, pR, Xtol, DUtol)

% RETURNS: updated triangulation with mid-nodes inserted


% 1.- PRELIMINARIES:

sudim = 2 ;
pudim = 3 ;
xdim  = 3 ;
     
sDegree = tFace(pR).mSur(sR).Degree ;
sNcp    = tFace(pR).mSur(sR).Ncp ;
sKnotV  = tFace(pR).mSur(sR).KnotV ;
sUsize  = tFace(pR).mSur(sR).Usize ;
sP      = tFace(pR).mSur(sR).P ;
sw      = tFace(pR).mSur(sR).w ;
sCD     = tFace(pR).mSur(sR).CS ;
[sEw,sEP] = GEN_12_ExpandBw (sw, sP, xdim, sudim, sNcp);  

pDegree = Spatches(pR,1).INS(1).mDegree ;
pNcp    = Spatches(pR,1).INS(1).mNcp ;
pKnotV  = Spatches(pR,1).INS(1).mKnotV ;
pUsize  = Spatches(pR,1).INS(1).mUsize;
pP      = Spatches(pR,1).INS(1).mP ;
pw      = Spatches(pR,1).INS(1).mw ;
pCD     = zeros(pudim,1);
[pEw,pEP] = GEN_12_ExpandBw (pw, pP, xdim, pudim, pNcp);  
pULims = [ 0 1 ; 0 1 ; 0 1 ];

econn = struct ('TV',[], 'LV',[], 'LT',[], 'VL',[], 'TL',[],'VT',[]);

eTV = [ conn.TV  zeros(NE,3) ];
eLV = [ conn.LV(1:NLI,1) zeros(NLI,1) conn.LV(1:NLI,2)]; % sets for mid-node insertion


% 2.- eLV MATRIX:

% 2.1.- Contour curves:

accL = 0 ; % accumulated line ref.

for L = 1:1:NL

    Ncmid = (CNT.Nc(L,1)-1)/2 ;
    
    j = 0 ;
    for i = 1:1:Ncmid
        
        accL = accL + 1 ;
        
        j = j+2;
        
        NVm = NVm+1;
        
        Bvm(NVm,1) = CNT.cB(L,j);
        Uvm(NVm,:) = CNT.cU(L,j,:);
        Xvm(NVm,:) = CNT.cX(L,j,:);
        Avm(NVm,:) = CNT.cA(L,j,:);

        mNC(NVm,1) = CNT.cRef(L,j);
        
        eLV(accL,2) = NVm ;
 
    end
end


% 2.2.- Internal lines: just computes the mid-point
% HERE ONLY COMPUTES Ucoords

u1 = zeros(sudim,1);
u2 = zeros(sudim,1);

Ntem = NVm ;

for r = accL+1:1:NLI
    
    n1 = eLV(r,1);
    n2 = eLV(r,3);
    
    u1(:,1) = Uvm(n1,:);
	u2(:,1) = Uvm(n2,:);

    um = (u1+u2)/2;

    Ntem = Ntem + 1 ;
    
    eLV(r,2) = Ntem;
	Uvm(Ntem,:) = um(:,1);
    
end



% 3.- eTV MATRIX:

nd = 3 ;

r = [ 1 2 ; 2 3 ; 3 1 ];

u1 = zeros(sudim,1);
u2 = zeros(sudim,1);

lu1 = zeros(sudim,1);
lu2 = zeros(sudim,1);

for e = 1:1:NE
    
    % loop on edges:
    
    for i = 1:1:3
    
        n1 = eTV(e,r(i,1));
        n2 = eTV(e,r(i,2));
        
        u1(:,1) = Uvm(n1,:);
        u2(:,1) = Uvm(n2,:);
        
        % searches line with those end nodes:
        
        for j = 1:1:3
            
            lr = conn.TL(e,j);
            
            lu1(:,1) = Uvm(eLV(lr,1),:);
            lu2(:,1) = Uvm(eLV(lr,3),:);
            
            CO11 = go2542_coincidentP (lu1, u1, sudim, nd);
            CO22 = go2542_coincidentP (lu2, u2, sudim, nd);
            
        	CO12 = go2542_coincidentP (lu1, u2, sudim, nd);
            CO21 = go2542_coincidentP (lu2, u1, sudim, nd);
            
            % finds the line:
            
            if (CO11==1 && CO22==1) || (CO12==1 && CO21==1)
                eTV(e,3+i) = eLV(lr,2);
                break
            end
            
            
        end
        
    end
    
end


% 4.- COMPUTES COORDINATES IN OTHER SPACES:

um = zeros(sudim,1);
a0 = zeros(pudim,1);

for r = accL+1:1:NLI
    
    n1 = eLV(r,1);
    n2 = eLV(r,2);
    n3 = eLV(r,3);

    um(:,1) = Uvm(n2,:);

    a0(:,1) = (Avm(n1,:) + Avm(n3,:))/2;
    
    [X] = b2601_BNURBS_Xcoord (um, sKnotV, sNcp, sDegree, sUsize, sEw, sEP, sudim, xdim);
    
    [A,~] = v2865b_onePoint (X, a0', pDegree, pNcp, pKnotV, pUsize, pP ,pw, pEP ,pEw, pCD, pULims, xdim, Xtol, DUtol);
    
    % [Uf, Af, Xf] = fs0779_innerMn (n1, n2, tFace, sR, Spatches, pR, Uvm, Avm, Xtol, DUtol)
    
    NVm = NVm + 1 ;
    
    eLV(r,2) = NVm;
	Uvm(NVm,:) = um(:,1);
	Xvm(NVm,:) = X(:,1);
 	Avm(NVm,:) = A(1,:);
    
end


% 5.- UPDATES CONN MATRIX:

econn.TV = eTV;
econn.LV = eLV;
econn.LT = conn.LT;
econn.VL = conn.VL;
econn.TL = conn.TL;
econn.VT = conn.VT;


end

