function [metS, otpS, Xt, B0, U0, A0, mBv, mUv, mNCS, isF] = th1324a_midState (pR, li, THC, n, ccTp, connT, rTskin)

% RETURNS:
% mid-node coordinates
% flag to indicate if the mid node was inserted

% INPUTS:
% THC: tet nodes coords and NCS
% n(2): end-nodes of the line to check

agCurv = ccTp(pR).agCurv;

cudim = 1 ;
sudim = 2 ;
pudim = 3 ;
xdim  = 3 ;

nd = 3 ;

Bv = THC.Bv ;
Uv = THC.Uv ;
Av = THC.Av ;
Xv = THC.Xv ;
NCS = THC.NCS ;

pNV = size(NCS,1);

Xt = zeros(1,xdim);
B0 = zeros(1,cudim);
U0 = zeros(1,sudim);
A0 = zeros(1,pudim);

mBv = zeros(1,cudim); % new end-node
mUv = zeros(1,sudim); 
mNCS = zeros(1,2); % T-curve and T-surface where it lies

isF = 0;

% extracts surfaces and curves refs:

cR = [ NCS(n(1),1); NCS(n(2),1) ] ;
sR = [ NCS(n(1),2); NCS(n(2),2) ] ;

A0(1,:) = ( Av(n(1),:) + Av(n(2),:) ) / 2 ;



% 1.- COMPUTES THE INITIAL POINT AND METHOD TO COMPUTE:

metS = "" ; % method selected: "d" / "p" => direct or point projection
otpS = "" ; % object to project selected: "C" / "S" => curve or surface


% 1.1.- LINE AT CONTOUR:

if cR(1) > 0 && cR(2) > 0
    
    isF = 1 ;

    % detects which nodes are extremes
    
    enF = zeros(2,1);
    
    for i = 1:1:2 
        if round(Bv(n(i),1),nd)==0 || round(Bv(n(i),1),nd)==1 
            enF(i)=1;
        end
    end
    
    if cR(1) == cR(2) % both nodes at same curve
        
        if agCurv(cR(1)).CD == 0 % open curve => direct
            
            metS = "d" ;
            otpS = "C" ;
            
            mBv(1,:) = ( Bv(n(1),:) + Bv(n(2),:) ) / 2 ;
            
            mNCS(1,1) = cR(1);
            mNCS(1,2) = sR(1);
  
        elseif agCurv(cR(1)).CD == 1 % closed curve
            
            if enF(1)==0 && enF(2)==0
                
                % no extremes nodes
                
                metS = "d" ;
                otpS = "C" ;
                
                mBv(1,:) = ( Bv(n(1),:) + Bv(n(2),:) ) / 2 ;

                mNCS(1,1) = cR(1);
                mNCS(1,2) = sR(1);
                
            else
                
                % node a is extreme and b is IN
                
                if enF(1)==1
                    a = 1; b = 2 ;
                elseif enF(2)==1
                    a = 2; b = 1 ;
                end
     
                metS = "p" ;
                otpS = "C" ;
                
                B0(1,1) = Bv(n(b),1);
                
                Xt(1,:) = ( Xv(n(a),:) + Xv(n(b),:) ) / 2 ;
                
                mNCS(1,1) = cR(1);
                mNCS(1,2) = sR(1);
            end

        end
        
        
    elseif cR(1)~=cR(2)
        
        if enF(1)==0 && enF(2)==0
            
            % both nodes are internal => searches surface point in the vicinity
            
            % stores all opposite nodes of lines of both nodes:
            
            onR = zeros(2,1);
            Non = zeros(2,1);
            
            for k = 1:1:2  
                
                conta = 0 ;
                for i = 1:1:size(connT.VL,2)

                    lR = connT.VL(n(k),i);
                    if lR > 0
                        for j = 1:1:size(connT.LV,2)

                            nR = connT.LV(lR,j);
                            
                            if (nR > 0 && NCS(nR,2)>0) && (nR~=n(1) && nR~=n(2))

                                conta = conta + 1 ;
                                onR(k,conta) = nR ; 

                            end

                        end
                    end
                end
                Non(k) = conta ;
                
            end
            
            
            % detects common surface:
            
            sRs = 0 ;
            
            for i = 1:1:Non(1)
                
                or = onR(1,i);
                s1 = NCS(or,2);
                
                for j = 1:1:Non(2)
                    
                    or = onR(2,j);
                    s2 = NCS(or,2);
                    
                    if s1==s2
                        sRs = s1 ;
                        break
                    end
                end
                
                if sRs>0
                    break
                end
            end
            
            
            
            % selects those nodes that have shared surface between n1 and
            % n2, chooses the one closer to both nodes:
            
            Xa = zeros(1,xdim);
            Xb = zeros(1,xdim);
            Xo = zeros(1,xdim);
            
            Xa(1,:) = Xv(n(1),:);
            Xb(1,:) = Xv(n(2),:);
            
            conta = 0 ;
            
            don = zeros(1,2); % opp. node ref and distance to opp nodes
            
            for k = 1:1:2
                for i = 1:1:Non(k)
                    
                    or = onR(k,i);
                    
                    if NCS(or,2)==sRs
                        
                        conta = conta + 1 ;
                        
                        don(conta,1) = or;
                        
                        Xo(1,:) = Xv(or,:);
                        
                        d1 = norm(Xo'-Xa');
                        d2 = norm(Xo'-Xb');
                        
                        don(conta,2) = (d1 + d2) / 2 ;
                        
                    end
                    
                end
            end
            
            don = sortrows(don,2);
            

            metS = "p" ;
            otpS = "S" ;
            
            U0(1,:) = Uv(don(1),:);
            
            Xt(1,:) = ( Xv(n(1),:) + Xv(n(2),:) ) / 2 ;
                
        	mNCS(1,1) = 0;
        	mNCS(1,2) = sRs;
            
        elseif sum(enF)==1 % only one node at extreme
            
            % node a is extreme and b is internal=> uses node b
            
            if enF(1)==1
                a = 1; b = 2 ;
            elseif enF(2)==1
                a = 2; b = 1 ;
          	end
            
          	metS = "p" ;
            otpS = "C" ;
            
         	B0(1,1) = Bv(n(b),1);
                
            Xt(1,:) = ( Xv(n(a),:) + Xv(n(b),:) ) / 2 ;
                
        	mNCS(1,1) = cR(b);
        	mNCS(1,2) = sR(b);
            
        elseif sum(enF)==2 % both nodes at extreme
            
            % searches if any contour curve has the same end cps. If not, 
            % the potential mid-node is an internal point and no need to be inserted
            
            icR = 0;
            
            Xv1 = zeros(1,xdim);
            Xv2 = zeros(1,xdim);
            
            Xc1 = zeros(1,xdim);
            Xc2 = zeros(1,xdim);
            
            Xv1(1,:) = Xv(n(1),:);
            Xv2(1,:) = Xv(n(2),:);
            
            for k = 1:1:2
                
                for L = 1:1:ccTp(pR).nlo(sR(k),1)
                
                    for c = 1:1:ccTp(pR).ncu(sR(k),L)
                        
                        tcR = abs(ccTp(pR).SCcon(sR(k),L,c));
                        
                        tcNcp = agCurv(tcR).Ncp ;
                        tcP   = agCurv(tcR).P ;
                        
                        Xc1(1,:) = tcP(1,:);
                        Xc2(1,:) = tcP(tcNcp,:);
                        
                        CO11 = go2542_coincidentP (Xv1, Xc1, xdim, nd);
                        CO22 = go2542_coincidentP (Xv2, Xc2, xdim, nd);
                        
                        CO12 = go2542_coincidentP (Xv1, Xc2, xdim, nd);
                        CO21 = go2542_coincidentP (Xv2, Xc1, xdim, nd);
                        
                        if (CO11==1 && CO22==1) || (CO12==1 && CO21==1)
                            icR = tcR;
                            break
                        end
                    end
                
                    if icR>0
                    break
                    end
                end
                
                if icR>0
               	break
                end
            end
                

            if icR > 0 
                
                % node to insert lies at mid-location of the curve
                metS = "d" ;
                otpS = "C" ;

                mBv(1,1) = 0.5;

                mNCS(1,1) = icR;
                mNCS(1,2) = sR(1);

            elseif icR==0
                
                % no control curve connects n1 to n2 => not mid-node
                isF = 0 ;
            end
  
        end
        
        
    end
    
end



% 1.2.- LINE INTERNAL TO SURFACE:

if isF == 0
    
    if sR(1) == sR(2) % both nodes at same surface
        
        isF = 1 ;
        
        % checks if it is closed and one nodes lies at the extreme
        
        CS = rTskin(sR(1)).mCS ;

        enF = zeros(2,1);
        cd = 0 ; % closed direction
        
        for di = 1:1:2
            if CS(di)==1
                cd = di ;
            end
        end
        
        if cd > 0
            for k = 1:1:2
                if round(Uv(n(k),cd),nd)==0 || round(Uv(n(k),cd),nd)==1
                    enF(k) = 1 ;
                end
            end
        end
        
        if enF(1)==1
            a = 1 ; b = 2 ;
        elseif enF(2)==1
            a = 2 ; b = 1 ;
        end
 
        if sum(CS)>0 && sum(enF>0)
            
            % node a lies at edge (extreme) => needs pp and uses node b (that is IN) for U0
            
            metS = "p" ;
            otpS = "S" ;

            U0(1,:) = Uv(n(b),:);
            
            Xt(1,:) = ( Xv(n(1),:) + Xv(n(2),:) ) / 2 ;
            
        	mNCS(1,1) = cR(b);
            mNCS(1,2) = sR(b);
            
            
        else
            
            % is either: opens surf, or closed but both nodes IN (not
            % extreme) => direct
            
            metS = "d" ;
            otpS = "S" ;

            mUv(1,:) = (Uv(n(1),:) + Uv(n(2),:)) / 2 ;

            mNCS(1,1) = cR(1);
            mNCS(1,2) = sR(1);
        end
        
  
    elseif sR(1) ~= sR(2) % nodes have different surfaces refs
        
        % detects node on curve 'a' and node on surface 'b'
        
        a = 0 ; b = 0 ;
        
        if cR(1)>0 
            a = 1; b = 2; 
        elseif cR(2)>0
            a = 2; b = 1; 
        end
        
        % if one node on curve cRa and that curve is common between sR1 and
        % sR1 => uses IN surface point for pp. Otherwise the mid-node lies
        % within the solid and no needs insertion
        
        beF = zeros(2,1); % belong flag
        
        if a>0 && b>0
            
            % checks that cRa belongs to sR1 and sR2

            cRa = cR(a);
            
            for k = 1:1:2
                for lo = 1:1:ccTp(pR).nlo(sR(k))

                    for cu = 1:1:ccTp(pR).ncu(sR(k),lo)

                        if cRa == ccTp(pR).SCcon(sR(k),lo,cu)
                            beF(k,1) = 1 ;
                            break
                        end
                    end

                    if beF(k)==1
                 	break
                    end
                end
            end
            
        end
        
        
        if sum(beF)==2
            
            % uses IN surface point for pp:
            
            isF = 1 ;
            
          	metS = "p" ;
            otpS = "S" ;
            
            U0(1,:) = Uv(n(b),:);
            
            Xt(1,:) = ( Xv(n(1),:) + Xv(n(2),:) ) / 2 ;
            
        	mNCS(1,1) = cR(b);
            mNCS(1,2) = sR(b);
            
        else
            
            % mid-node lies within the solid, not at surfaces
            isF = 0 ;

        end
 
    end

end




end

