function [metS, otpS, Xt, B0, U0, A0, mBv, mUv, mNCS, isF] = th1324c_curveState ...
    (pR, cR, sR, n, Bv, Uv, Av, Xv, NCS, ccTp, VLcon)

% RETURNS:
% metS: method to find the mid-point: "d" direct, "p" point projection
% otpS: object to project to: "C" curve, "S" surface

% Xt(1,xdim): trial point for pp
% B0(1,1): initial point for pp on curves
% mBv(1,1): mid-point coords in curve param. space

% mNCS(1,2): curve and surface ref for the mid-node

% INPUTS:
% n(2):     end-nodes refs
% cR,sR(2): curves and surfs refs where both end-nodes belong
% Bv, Uv, Av, Xx, NCS(NV,dim): coords of the nodes


agCurv = ccTp(pR).agCurv;

isF = 1 ; % 'inserted flag', turns to 0 if no node is inserted using the contour curves

cudim = 1 ;
sudim = 2 ;
pudim = 3 ;
xdim  = 3 ;

nd = 3 ;

Xt = zeros(1,xdim);
B0 = zeros(1,cudim);
U0 = zeros(1,sudim);
A0 = zeros(1,pudim);

mBv = zeros(1,cudim); % new end-node
mUv = zeros(1,sudim); 
mNCS = zeros(1,2); % T-curve and T-surface where it lies

% 1.- DETECTS WHICH NODES ARE AT EXTREMES OF CURVE (IF THEY EXIST):

enF = zeros(2,1);
    
for i = 1:1:2 
    if round(Bv(n(i),1),nd)==0 || round(Bv(n(i),1),nd)==1 
        enF(i)=1;
    end
end

% 2.- BOTH CURVES REFS ARE THE SAME => both nodes at same curve for sure

if cR(1) == cR(2) 
        
	if agCurv(cR(1)).CD == 0 % open curve => direct
            
        metS = "d" ;
        otpS = "C" ;
            
        mBv(1,:) = ( Bv(n(1),:) + Bv(n(2),:) ) / 2 ;
            
    	mNCS(1,1) = cR(1);
        mNCS(1,2) = sR(1);
  
	elseif agCurv(cR(1)).CD == 1 % closed curve
            
        if enF(1)==0 && enF(2)==0
                
            % no extremes nodes
                
            metS = "d" ;
         	otpS = "C" ;
                
        	mBv(1,:) = ( Bv(n(1),:) + Bv(n(2),:) ) / 2 ;

        	mNCS(1,1) = cR(1);
          	mNCS(1,2) = sR(1);
                
        else
                
            % node a is extreme and b is IN
                
        	if enF(1)==1
                a = 1; b = 2 ;
         	elseif enF(2)==1
            	a = 2; b = 1 ;
            end
     
         	metS = "p" ;
          	otpS = "C" ;
                
         	B0(1,1) = Bv(n(b),1);
                
        	Xt(1,:) = ( Xv(n(a),:) + Xv(n(b),:) ) / 2 ;
                
          	mNCS(1,1) = cR(1);
          	mNCS(1,2) = sR(1);
        end

	end
        
        
% 3.- CURVES REFS ARE DIFFERENT =>  needs to check if are the same or not
% If not, the isF = 0, i.e. the node is not at mid-point of the curve

elseif cR(1)~=cR(2)
        
    if enF(1)==0 && enF(2)==0
            
        % 3.1.- Both nodes are internal => searches surface point in the vicinity
            
        % stores all opposite nodes of lines of both nodes:
            
        onR = zeros(2,1);
        Non = zeros(2,1);
            
        for k = 1:1:2  
                
            conta = 0 ;
            for i = 1:1:size(VLcon,2)

                lR = VLcon(n(k),i);
                if lR > 0
                    for j = 1:1:size(VLcon,2)

                        nR = VLcon(lR,j);
                            
                        if (nR > 0 && NCS(nR,2)>0) && (nR~=n(1) && nR~=n(2))

                            conta = conta + 1 ;
                            onR(k,conta) = nR ; 

                        end

                    end
                end
            end
            Non(k) = conta ;
                
        end
            
            
        % detects common surface:
            
        sRs = 0 ;
            
        for i = 1:1:Non(1)
                
            or = onR(1,i);
            s1 = NCS(or,2);
                
            for j = 1:1:Non(2)
                    
                or = onR(2,j);
                s2 = NCS(or,2);
                    
                if s1==s2
                    sRs = s1 ;
                    break
                end
            end
                
            if sRs>0
                break
            end
        end
            
            
            
        % selects those nodes that have shared surface between n1 and
        % n2, chooses the one closer to both nodes:
            
        Xa = zeros(1,xdim);
        Xb = zeros(1,xdim);
        Xo = zeros(1,xdim);
            
        Xa(1,:) = Xv(n(1),:);
        Xb(1,:) = Xv(n(2),:);
            
        conta = 0 ;
            
        don = zeros(1,2); % opp. node ref and distance to opp nodes
            
        for k = 1:1:2
        for i = 1:1:Non(k)
                    
            or = onR(k,i);
                    
            if NCS(or,2)==sRs
                        
                conta = conta + 1 ;
                        
                don(conta,1) = or;
                        
                Xo(1,:) = Xv(or,:);
                        
                d1 = norm(Xo'-Xa');
                d2 = norm(Xo'-Xb');
                        
                don(conta,2) = (d1 + d2) / 2 ;
                        
            end       
        end
        end
            
        don = sortrows(don,2);
            
        metS = "p" ;
        otpS = "S" ;
            
        U0(1,:) = Uv(don(1),:);
            
        Xt(1,:) = ( Xv(n(1),:) + Xv(n(2),:) ) / 2 ;
                
        mNCS(1,1) = 0;
     	mNCS(1,2) = sRs;
           
        
	% 3.2.- Only one node at extreme:
    % node a is extreme and b is internal=> uses node b   
	elseif sum(enF)==1 
            
        if enF(1)==1
            a = 1; b = 2 ;
    	elseif enF(2)==1
            a = 2; b = 1 ;
        end
            
     	metS = "p" ;
     	otpS = "C" ;
            
        B0(1,1) = Bv(n(b),1);
                
      	Xt(1,:) = ( Xv(n(a),:) + Xv(n(b),:) ) / 2 ;
                
     	mNCS(1,1) = cR(b);
     	mNCS(1,2) = sR(b);

        
 	% 3.3.- Both nodes at extreme
    % searches if any contour curve has the same end cps. If not, 
  	% the potential mid-node is an internal point and no need to be inserted
	elseif sum(enF)==2 
            
            icR = 0;
            
            Xv1 = zeros(1,xdim);
            Xv2 = zeros(1,xdim);
            
            Xc1 = zeros(1,xdim);
            Xc2 = zeros(1,xdim);
            
            Xv1(1,:) = Xv(n(1),:);
            Xv2(1,:) = Xv(n(2),:);
            
            for k = 1:1:2
                
                for L = 1:1:ccTp(pR).nlo(sR(k),1)
                
                    for c = 1:1:ccTp(pR).ncu(sR(k),L)
                        
                        tcR = abs(ccTp(pR).SCcon(sR(k),L,c));
                        
                        tcNcp = agCurv(tcR).Ncp ;
                        tcP   = agCurv(tcR).P ;
                        
                        Xc1(1,:) = tcP(1,:);
                        Xc2(1,:) = tcP(tcNcp,:);
                        
                        CO11 = go2542_coincidentP (Xv1, Xc1, xdim, nd);
                        CO22 = go2542_coincidentP (Xv2, Xc2, xdim, nd);
                        
                        CO12 = go2542_coincidentP (Xv1, Xc2, xdim, nd);
                        CO21 = go2542_coincidentP (Xv2, Xc1, xdim, nd);
                        
                        if (CO11==1 && CO22==1) || (CO12==1 && CO21==1)
                            icR = tcR;
                            break
                        end
                    end
                
                    if icR>0
                    break
                    end
                end
                
                if icR>0
               	break
                end
            end
                

            if icR > 0 
                
                % node to insert lies at mid-location of the curve
                metS = "d" ;
                otpS = "C" ;

                mBv(1,1) = 0.5;

                mNCS(1,1) = icR;
                mNCS(1,2) = sR(1);

            elseif icR==0
                
                % no control curve connects n1 to n2 => not mid-node
                isF = 0 ;
            end
  
    end
        
        
end



end

