function [connT, Avm, Xvm, NVm, Eiem, Nve] = th1337_upQuad ...
         (connT, Av, NE, Nts, tsR, rTskin, agCurv)

% RETURNS: features including mid-vtx where line lies on T-surface

% INPUTS:


pudim = 3 ;
nd    = 3 ;


% 1.- PRELIMINARIES:

Nve = ones(NE,1)*4; % No. vertexes per element

% extended conn. matrix, to account for mid-edge vertexes:

NLI = size(connT.LV,1);

connT.EV = [ connT.EV , zeros(NE,6) ]; 
connT.LV = [ connT.LV , zeros(NLI,1) ];

NVm = size(THC.Av,1);
Avm = THC.Av ;
Xvm = THC.Xv ;

% local refs conn. matrices:

LN = [ 1 2 3 4 4 4 ; 
       2 3 1 1 2 3 ]'; % lines => nodes 
  
NL = [ 0 1 3 4 ; 
       1 0 2 5 ; 
       3 2 0 6 ; 
       4 5 6 0 ]; % nodes => lines
 

% 2.- IDENTIFICATION: tet lines - rTskin and its line

C32 = zeros(NLI,2); % connect tet-line to (rTskin and its line ref.)

for li = 1:1:NLI
    
    % coordinates of end-nodes of the line:
    
    n1 = connT.LV(li,1);
	n2 = connT.LV(li,2);
         
    C = zeros(2,pudim);
            
	C(1,:) = Av(n1,:);
	C(2,:) = Av(n2,:);
    
    
    % Searches if there is one rTskin line that coincides with this tet-line:
    
    for s = 1:1:Nts
        
        sR = tsR(s);

        sLVcon = rTskin(sR,1).mLVcon ;
            
        sAv = rTskin(sR,1).mAv ;    
        tNLI = size(sLVcon,1);
            
        adF = 0 ; % adding flag
            
        for tli = 1:1:tNLI % searches line within this surface
                
                sn1 = sLVcon(tli,1);
                sn3 = sLVcon(tli,3);
                
             	sCd = zeros(2,pudim);
                sCd(1,:) = sAv(sn1,:);
                sCd(2,:) = sAv(sn3,:);
                
                sCr = zeros(2,pudim);
                sCr(2,:) = sAv(sn1,:);
                sCr(1,:) = sAv(sn3,:);
                
                COd = go2541_coincidentM (C, sCd, 2,pudim, nd);
                COr = go2541_coincidentM (C, sCr, 2,pudim, nd);
                
                if COd==1 || COr==1
                    adF = 1 ;
                    break
                end
        end
            
        % adds ref. in case finds it
        if adF == 1
            C32(li,1) = sR ;
            C32(li,2) = tli ;
        end

        if C32(li,1)>0
      	break 
        end
    end
    
end



% 3.- INSERTS MID-NODES AT LINES AND COORDINATES ARRAYS:
% Exports direclty from rTskin

for li = 1:1:NLI
if C32(li,1)>0
    
    % surface information:
    
    sR   = C32(li,1);
    tliR = C32(li,2);
    
    sLVcon = rTskin(sR,1).mLVcon ;
    
    tn = sLVcon(tliR,2);
    
	sAv = rTskin(sR,1).mAv ;
    sXv = rTskin(sR,1).mXv ;
        
    % updates tetrahedrons:
    
    NVm = NVm + 1 ;
    
    connT.LV(li,3) = connT.LV(li,2);
    connT.LV(li,2) = NVm ;
    
    for di = 1:1:pudim
        Avm(NVm,di) = sAv(tn,di);
        Xvm(NVm,di) = sXv(tn,di);
    end
end
end


% 4.- UPDATES EVcon and Nve:

for e = 1:1:NE
    
    for i = 1:1:6
        
        liR = connT.EL(e,i);
        
        if connT.LV(liR,3)>0
            Nve(e,1) = Nve(e,1) + 1 ;
            connT.EV(e,Nve(e)) = connT.LV(liR,2);
        end
    end
    
end


% 5.- MATRIX Eiem:

Eiem = zeros(NE,4,4);

for e = 1:1:NE

	conta = 4 ;
	for i = 1:1:6

        lR = connT.EL(e,i);

        if connT.LV(lR,3)>0

            
            % nodes of the line refs:
            
            n1 = connT.LV(lR,1);
            n3 = connT.LV(lR,3);

            
            % nodes local refs:
            
            lnr1 = 0 ; lnr3 = 0 ;
            
            for j = 1:1:4
                if connT.EV(e,j) == n1 
                    lnr1 = j ;
                end
                if connT.EV(e,j) == n3 
                    lnr3 = j ;
                end                
            end
            

            % finds the line position within the tet
            llr = NL(lnr1,lnr3);
            
            conta = conta + 1;
            Eiem(e,LN(llr,1), LN(llr,2)) = conta ;
            Eiem(e,LN(llr,2), LN(llr,1)) = conta ;
            
        end
	end
end


end