function [a,J2] = tm1351_TETmapping2 (rst, Xnod, Nnt, Pdim, Adim, ttp)

% RETURNS:
% a(dim,1): coord's in mapped space for parent 'r' coord's in tetraedral
% J2: Jacobian from parent to mapped space

% INPUTS:
% rst(dim,1): coord's in parent
% Xnod(Nnt,dim): tet nods mapped coordiantes
% ttp: tetraedral type, eg. 'linear' (see below the cases)

% Adim, Pdim: dims of mapped and parent spaces

r = rst(1);
s = rst(2);
t = rst(3);
u = 1-rst(1)-rst(2)-rst(3); % the fourth dimension

N = zeros(Nnt,1);

a = zeros(Pdim,1);
J2 = zeros(Pdim,Adim);

if Nnt == 4 % strcmp(ttp,'linear') == 1 
  
    % Basis functions values at r:
    
    N(1) = r ;
    N(2) = s ;
    N(3) = t ;
    N(4) = u ;
    
	% Jacobian: each col is the derivative w.r.t. one direction of Parent-space:

    for di = 1:1:Pdim  
        J2(1,di) = Xnod(1,di) - Xnod(4,di);
        J2(2,di) = Xnod(2,di) - Xnod(4,di);
        J2(3,di) = Xnod(3,di) - Xnod(4,di);
    end
    

elseif Nnt == 10  % strcmp(ttp,'quadratic') == 1  % Nnt = 10 

    N(1) = r*(-1+2*r);
    N(2) = s*(-1+2*s);
    N(3) = t*(-1+2*t);
    N(4) = u*(-1+2*u);
    
    N(5) = 4*r*s ;
    N(6) = 4*s*t ;
    N(7) = 4*t*u ;
    N(8) = 4*r*u ;
    N(9) = 4*r*t ;
    N(10)= 4*s*u ;
    
else
    
    if Nnt == 5
        
        N(1) = r*(1-2*s);
        N(2) = s*(1-2*r);
        N(3) = t ;
        N(4) = u ;

        N(5) = 4*r*s ;
        
    elseif Nnt == 6
        
        N(1) = r*(1-2*s);
        N(2) = s*(1-2*r-2*t);
        N(3) = t*(1-2*s) ;
        N(4) = u ;

        N(5) = 4*r*s ;
        N(6) = 4*s*t ;
        
    elseif Nnt == 7
        
        N(1) = r*(1-2*s);
        N(2) = s*(1-2*r-2*t);
        N(3) = t*(-1-2*r+2*t) ;
        N(4) = u*(1-2*t) ;

        N(5) = 4*r*s ;
        N(6) = 4*s*t ; 
        N(7) = 4*t*u ;
        
	elseif Nnt == 8
        
        N(1) = r*(-1+2*r+2*t);
        N(2) = s*(1-2*r-2*t);
        N(3) = t*(-1-2*r+2*t) ;
        N(4) = u*(1-2*t-2*r) ;

        N(5) = 4*r*s ;
        N(6) = 4*s*t ; 
        N(7) = 4*t*u ;   
        N(8) = 4*r*u ;
        
	elseif Nnt == 9
        
        N(1) = r*(-1+2*r);
        N(2) = s*(1-2*r-2*t);
        N(3) = t*(-1+2*t);
        N(4) = u*(1-2*t-2*r);

        N(5) = 4*r*s ;
        N(6) = 4*s*t ;
        N(7) = 4*t*u ;
        N(8) = 4*r*u ;
        N(9) = 4*r*t ;
        
    end
    
    
end


% Mapping of coordinates:
    
for di = 1:1:Adim
    
    for n = 1:1:Nnt
        
        a(di,1) = a(di,1) + N(n)*Xnod(n,di);
        
    end
    
end


end

