function [isi] = ts1601_sidesInv (contaL, ILn, NE, connT, connS, THC, tolL, tolA, nd)

% RETURNS: intersected sides by missing line from MLT

Nsi = 0 ; % No. sides intersected
isi(1,1) = struct('R', 0 , 'Acoor', [], 'tp', 0); % side ref, intersection Acoords, type (1 in, 2 edge)

% INPUTS:
% ILn(2,2): reference of nodes of the missing line in tets (1st col) and in MLT (2nd col)

% 1.- PRELIMINARIES:

Av = THC.Av ;
pudim = 3 ;

% line jointing both nodes in patch parameter space
[rv, rP1, rP2] = ts1602_mLAspace (ILn, Av,0,0,0);


% 2.- SIDES CHECK:
  
NSI = size(connS.SE,1);

for s = 1:1:NSI
    
    % 2.1.- Geometrical features:
    
    % coords of nodes of side:
    
    Anod = zeros(3,pudim);
    for k = 1:1:3 
        nR = connS.SV(s,k);
        Anod(k,:) = Av(nR,:);
    end
    
    
    % plane features:
    
    u = zeros(pudim,1);
    v = zeros(pudim,1);
    P = zeros(pudim,1);
    
    for di = 1:1:pudim
        u(di,1) = Anod(2,di) - Anod(1,di);
        v(di,1) = Anod(3,di) - Anod(1,di);
        P(di,1) = Anod(1,di);
    end
    
    
    % 2.2.- Intersection line-plane:
    
    [IP,~] = go2545_PlaneLineIntersec (u, v, P, rv, rP1);
    
    
    % checks if intersection lies within the triangle:

    IN = go2546_PinTriang (Anod, IP, tolL, tolA, nd);
    ON = go2520_PonSegment (IP, rP1, rP2, pudim, tolL);
    
    
    %{
    if contaL==11 && s==3031
        
        figure(76)
        
        scatter3(Anod(:,1), Anod(:,2), Anod(:,3), 'k', 'filled'); hold on
        scatter3(IP(1), IP(2), IP(3), 'r'); hold on
        
        axis equal
        
        if gg==0
        end
        
    end
    %}
    
    
    % 2.3.- Assesment of intersection:

    if IN > 0 && ON == 1 % detects intersection
        
        CO1 = go2542_coincidentP (IP, rP1, pudim, nd);
        CO2 = go2542_coincidentP (IP, rP2, pudim, nd);
        
        if CO1==0 && CO2==0 % ntersection is not at the line nodes
            
            % if IN == 1
            if IN > 0
                
                % stores ref
                
                Nsi = Nsi + 1 ;
                
                isi(Nsi,1).R = s ;
                isi(Nsi,1).Acoor = IP ;
                isi(Nsi,1).tp = IN ;
  
            end
            
        end
  
    end
    
end


%{
if contaL==11

    for i = 1:1:Nsi

        fr = 100;
        figure(fr)
        pt1374_pSidConn (isi(i).R, connS, Av, fr, 'b', 0.8, 1.2, 0);
        
        x = [ rP1' ; rP2' ] ;
        
        plot3(x(:,1), x(:,2), x(:,3),'r'); hold on
        
    end

    
    if jj==4
    end
end
%}

% plots intersected sides:
%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%

if Nsi > 0
   
    fr = contaL ;

    figure(fr)

    for s = 1:1:Nsi

        sR = siR(s);

        Anod = zeros(3,pudim);
        for k = 1:1:3 
            nR = connS.SV(sR,k);
            Anod(k,:) = Av(nR,:);
        end

        pt1361_plotTri_nD (Anod, pudim, fr, 'k');

        scatter3( Aip(s,1),  Aip(s,2), Aip(s,3),'r'); hold on

    end

    plot3( [rP1(1) rP2(1)] , [rP1(2) rP2(2)] , [rP1(3) rP2(3)] ,'r'); hold on
    
    axis equal

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%

%}

% plots intersected lines:
%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%

if ieR > 0
   
    fr = contaL ;

    figure(fr)

    Anod = zeros(2,pudim);
    
    for k = 1:1:2 
        nR = connT.LV(ieR,k);
     	Anod(k,:) = Av(nR,:);
    end

	pt1361_plotTri_nD (Anod, pudim, fr, 'k');

    scatter3( Aip(1,1),  Aip(1,2), Aip(1,3),'r'); hold on
    
	plot3( [rP1(1) rP2(1)] , [rP1(2) rP2(2)] , [rP1(3) rP2(3)] ,'r'); hold on 

    % tets:
    for e = 1:1:Nte
        
        eR = teR(e,1);
        
        Anod = zeros(4,pudim);
        
        for k = 1:1:4
            nR = connT.EV(eR,k);
            Anod(k,:) = Av(nR,:);
        end
        
        pt1362_pTet (Anod, pudim, fr, 0.5, 0.1); 
        
    end
    
    
    axis equal

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%

%}

end




