function [mpS] = ts1605_optState (contaL, meR, THC, pNE, connT, connS, nnR)

% RETURNS:
% State that optimizes the tets after splitting, i.e. measures the
% alternatives with the node at edge and side and selects the option that
% yields the most regular tets.

% INPUTS:
% meR: 'main element ref', or element where the new node lies in
% nnR: new node ref (the last one that was already included from eTHC)

% NOTE: nnR is fixed ! coz is given by MLT

Av = THC.Av;
pudim = 3 ;



% 1.- CHECKS ALL CONFIGURATIONS:
% Measures regularity (R), the higher R the better

% 1.1.- Node IN: 4 tets resultant

R4 = zeros(1,2); % min value (worst case) and average

[NEt, connTt] = mi1671_TetDiv4 (pNE, connT, meR, nnR);

n = 4 ;
er = [meR, pNE+1 , pNE+2 , pNE+3 ]; 

[Rt] = ts1606_Rtets (n, er, connTt, Av);
                 
R4(1,1) = min(Rt);
R4(1,2) = sum(Rt)/4;



% 1.2.- Node as if it was at each side:

R3 = zeros(1,2); % min value (worst case) and average

for sL = 1:1:4
    
    % attached tets to such side and the local side ref : atR(Nae,1) & LsRd(Nae,1)

    [Nae,atR, LsRd] = ts1654_attESL (sL, meR, connT, connS);
    
    
    % divides at 'sL' side:
    
    [NEt, connTt] = ts1653_TetDiv3 (THC, pNE, connT, Nae, atR, nnR, LsRd);
    

    % computes the regularity of all the triangles and stores:
    
    n = 3*Nae;
    er = zeros(n,1);
    
    accE = 0 ;
    conta = 0 ;
    for i = 1:3:n
        
        conta = conta + 1 ;
        
        er(i+0,1) = atR(conta,1);
        er(i+1,1) = pNE + accE + 1 ;
        er(i+2,1) = pNE + accE + 2 ;
        
        accE = accE + 2 ;
        
    end
    

    [Rt] = ts1606_Rtets (n, er, connTt, Av);
                  
    R3(sL,1) = min(Rt);
    R3(sL,2) = sum(Rt)/n;
    
 
end




% 1.3.- Node as if it was at each edge:

R2 = zeros(6,2); % min. value (worst case) and average


for eL = 1:1:6
    
    
    % attached tets to such line and the local edge ref : atR(Nae,1) & LeRd(Nae,2)

    [Nae, atR, LeRd] = ei1664_attELL (eL, meR, connT);
    

    % divides at 'eL' edge:
    
    [NEt, connTt] = ei1663_TetDiv2 (THC, pNE, connT, Nae, atR, nnR, LeRd);
    

	% computes the regularity of all the triangles and stores:
    
    n = 2*Nae;
    er = zeros(n,1);
    
    accE = 0 ;
    conta = 0 ;
    for i = 1:2:n
        
        conta = conta + 1 ;
        
        er(i+0,1) = atR(conta,1);
        er(i+1,1) = pNE + accE + 1 ;
        
        accE = accE + 1 ;
    end
    
    [Rt] = ts1606_Rtets (n, er, connTt, Av);
                  
    R2(eL,1) = min(Rt);
    R2(eL,2) = sum(Rt)/n;
    
    %%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%
    %{
    
    if contaL == 11 && eL == 2
        
        fr = 10 + eL ;
        for k = 1:1:Nae
            pt1373_pTetConn (atR(k), connT, Av, fr, 'k', 0.8, 1.25, 0, 0);
        end
        
        fr = 20 + eL ;
        for k = 1:2:n
            pt1373_pTetConn (er(k), connTt, Av, fr, 'b', 0.8, 1.25, 0, 0);
        end
 
        
        x = zeros(2,pudim);
        
        for j = 1:1:2
            
            n1 = connT.EV(meR,2);
            n2 = connT.EV(meR,3);
            
            x(1,:) = THC.Av(n1,:);
            x(2,:) = THC.Av(n2,:);

        end
        
        plot3( x(:,1),  x(:,2),  x(:,3), 'm'); hold on
        
    end
    %}
    %%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%%%%%%
    
end


% 2.- SELECTS THE BEST CONFIGURATION:
% Preference of min value of R, arranges first by minimum and then by
% average. Selects the highest one (last in the list)

ENc = [1 2 3 4 4 4 ; 
       2 3 1 1 2 3 ]' ;

agg = [ R4 1 0 0 ;
        R3 [2 1 0 ; 
            2 2 0 ;
            2 3 0 ;
            2 4 0 ];
        R2 [3;3;3;3;3;3] ENc];
 
agg = sortrows( agg, [1 2] );
  
pos = agg(11,3);
seR(1) = agg(11,4);
seR(2) = agg(11,5);

mpS = [ pos meR seR(1) seR(2) ];



% 3.- Prints (just for checking):


%{

fr = meR ;
pt1373_pTetConn (meR, connT, Av, fr, 'r', 0.1, 3.25, 0); 



if mpS(1)==1
    
    [NEt, connTt] = mi1671_TetDiv4 (pNE, connT, meR, nnR);
    
    n = 4 ;
    er = [ meR, NEt-2, NEt-1, NEt-0 ] ; 
    
elseif mpS(1)==2
    
    [NEt, connTt] = ts1653_TetDiv3 (THC, pNE, connT, 1, meR, nnR, mpS(3));
    
	n = 3 ;
    er = [ meR, NEt-1, NEt-0 ] ; 
    
elseif mpS(1)==3  
    
    [NEt, connTt] = ei1663_TetDiv2 (THC, pNE, connT, 1, meR, nnR, [mpS(3) mpS(4)]);
	
    n = 2 ;
    er = [ meR, NEt-0 ] ; 
end


for k = 1:1:n
    
    pt1373_pTetConn (er(k), connTt, Av, fr, 'k', 0.8, 1.25, 0);

end 

%}


end

