function [oisi, Nte, teR] = ts1611_OrderedTets (ILn, Av, isi, connS, connT)

% RETURNS:
% oisi(Nsi,1):      ascendently ordered isi structure
% Nte:              No. tets intersected by the missing line
% teR(Nte,1):       ascendently ordered intersected tets


pudim = 3 ;
Nsi = size(isi,1);

oisi(Nsi,1) = struct('R', 0 , 'Acoor', [], 'tp', 0); % side ref, intersection Acoords, type (1 in, 2 edge)

% 1.- ORDERS SIDES:

% 1.1.- Line jointing both nodes in patch parameter space:

[rv, rP1, rP2] = ts1602_mLAspace (ILn, Av);


% 1.2.- 'Aggregated sides' last col. is for distance to rP1:

agS = zeros(Nsi,6) ; % ref, Acoord, type, distance to rP1

a = zeros(pudim,1);

for s = 1:1:Nsi
    
    agS(s,1) = isi(s,1).R ;
    
    for di = 1:1:pudim
        agS(s,di+1) = isi(s,1).Acoor(di,1);    
    end
    
    agS(s,5) = isi(s,1).tp ;
    
    a(:,5) = isi(s,1).Acoor(:,1);
    
    agS(s,6) = norm(a-rP1);
    
end


% 1.3.- Arranges and re-allocates:

agS = sortrows(agS,6);

for s = 1:1:Nsi
    
    oisi(s,1).R = agS(s,1);
    
    for di = 1:1:pudim
        oisi(s,1).Acoor(di,1) = agS(s,di+1);
    end
    
    oisi(s,1).tp = agS(s,5);
 
end

% 2.- IDENTIFIES TETS AND PUT THEM IN ASCENDING ORDER:

Nte = Nsi+1;
teR = zeros(Nte,1);

% 2.1.- First couple of tets:

sR = oisi(1,1).R; % first side

teR(1,1) = connS.SE(sR,1);
teR(2,1) = connS.SE(sR,2);

nR1 = ILn(1,1);
swF = 1 ; % swap flag

% first tet must have node 1 of the missing line
for k = 1:1:4 
    
    nR = connT.EV(teR(1),k);
    if nR == nR1
        swF = 0 ;
        break
    end
    
end

if swF == 1
    teR(1,1) = connS.SE(sR,2);
    teR(2,1) = connS.SE(sR,1);
end


% 2.2.- Rest of the tets (if applicable):

conta = 2 ;

for s = 2:1:Nsi % rest of the sides
    
    conta = conta + 1 ;
    
    sR = oisi(s,1).R;
    
    for e = 1:1:2
        
        eR = connS.SE(sR,e);
        if eR ~= teR(conta-1)
            
            teR(conta,1) = eR;
            break
            
        end
    end
    
end





end

