function [Nip, ip] = ts1614_iAip ...
                (Spatches, pR, rTskin, agCurv, ILn, oisi, THC, Xtol, DUtol)

cudim = 1 ;
sudim = 2 ;
pudim = 3 ;
xdim  = 3 ;

pDegree = Spatches(pR).INS(1).mDegree;
pNcp    = Spatches(pR).INS(1).mNcp;
pKnotV  = Spatches(pR).INS(1).mKnotV;
pP      = Spatches(pR).INS(1).mP;
pw      = Spatches(pR).INS(1).mw;
pUsize  = Spatches(pR).INS(1).mUsize;
pULims  = [0 1 ; 0 1 ; 0 1 ];
pEP     = Spatches(pR).INS(1).mEP;
pEw     = Spatches(pR).INS(1).mEw;
pCD     = [0 0 0 ];


% 1.- INTERMEDIATE POINTS INITIAL LOCATION:

Nsi = size(oisi,1);

Nip = Nsi - 1 ;

if Nip == 0
    
    ip = 0 ;
    return
    
else
    
    ip(Nip,1) = struct('Bi',[], 'Ui',[], 'Ai',[], 'Xi',[], 'NCSi',[], 'nAi',[]);
    % nAi is the non-projected coordinates, that lie on MLT. It is necessary for updating MLT
    
    AiL = zeros(Nip,pudim);
    
    for i = 1:1:Nip
        AiL(i,:) = (oisi(i,1).Acoor(:,1) + oisi(i+1,1).Acoor(:,1))/2;
    end
    
end



% 2.- CURVES AND SURFACES ATTACHED: 

tn = [ ILn(1,1) ; ILn(2,1) ] ; % nodes involved, in tets refs

cs = zeros(2,2); % T-curves and T-surfaces attached to these tet nodes

for v = 1:1:2
    cs(v,1) = THC.NCS(tn(v),1); cs(v,2) = THC.NCS(tn(v),2);
end



% 3.- PROJECTS BACK TO SURFACE/CURVE WHERE IT BELONGS:

sR = THC.NCS(tn(1),2); % surface ref
    
sDegree = rTskin(sR).mDegree;
sNcp    = rTskin(sR).mNcp;
sKnotV  = rTskin(sR).mKnotV;
sP      = rTskin(sR).mP;
sw      = rTskin(sR).mw;
sUsize  = rTskin(sR).mUsize;
sCD     = rTskin(sR).mCS;
[sEw,sEP] = GEN_12_ExpandBw (sw, sP, xdim, sudim, sNcp);
sULims  = [ 0 1 ; 0 1 ];
    
    
% 3.1.- MISSING LINE BELONGS TO ONE SURFACE OF MLT:
    
if cs(1,1) > 0 && cs(2,1) > 0

        % -.- Belongs to a contour curve 'cR'

        tnm = min(tn);
        
        cR  = THC.NCS(tnm,1);

        cDegree = agCurv(cR).Degree;
        cNcp    = agCurv(cR).Ncp;
        cKnotV  = agCurv(cR).KnotV;
        cP      = agCurv(cR).P;
        cw      = agCurv(cR).w;
        cUsize  = agCurv(cR).Usize;
        cCD     = agCurv(cR).CD;
        cULims  = [ 0 1 ];

        At = zeros(pudim,1);
        
        for i = 1:1:Nip

            At(:,1) = AiL(i,:);
            Xt = b2601_BNURBS_Xcoord (At, pKnotV, pNcp, pDegree, pUsize, pEw, pEP, pudim, xdim);
            
            % pp from Xt to curve, using B0 as starting point
            B0 = (THC.Bv(tn(1),1) + THC.Bv(tn(2),1)) / 2 ;
            [B,~] = c2815b_onePoint (Xt, B0, cDegree, cNcp, cKnotV, cP ,cw, cULims, cCD, xdim, Xtol, DUtol);

            % physical coordinates:
            X = b2601_BNURBS_Xcoord (B, cKnotV, cNcp, cDegree, cUsize, cw, cP, cudim, xdim);
            
            % pp from X to surface:
            U0 = (THC.Uv(tn(1),:) + THC.Uv(tn(2),:)) / 2 ;
            [U, ~] = s2835b_onePoint (X, U0, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sEP ,sEw, sCD, sULims, xdim, Xtol, DUtol);
            U = U';
            
         	% pp from X to patch:
            A0 = At' ;
            [A, ~] = v2865b_onePoint (X, A0, pDegree, pNcp, pKnotV, pUsize, pP ,pw, pEP ,pEw, pCD, pULims, xdim, Xtol, DUtol);
            A = A';
            
            
            % allocates:
            
            ip(i,1).Bi = B ;
            ip(i,1).Ui = U ;
            ip(i,1).Ai = A ;
            ip(i,1).Xi = X ;
            ip(i,1).NCSi = [cR sR];
            
            ip(i,1).nAi = At ;
            
        end
else
        
        % -.- Belongs to a trimming surface 'sR':

        At = zeros(pudim,1);
        
        for i = 1:1:Nip
            
            At(:,1) = AiL(i,:);
            Xt = b2601_BNURBS_Xcoord (At, pKnotV, pNcp, pDegree, pUsize, pEw, pEP, pudim, xdim);
            
            % pp from X to surface:
            U0 = (THC.Uv(tn(1),:) + THC.Uv(tn(2),:)) / 2 ;
            [U, ~] = s2835b_onePoint (Xt, U0, sDegree, sNcp, sKnotV, sUsize, sP ,sw, sEP ,sEw, sCD, sULims, xdim, Xtol, DUtol);
            U = U';
            
            
          	% physical coordinates:
            X = b2601_BNURBS_Xcoord (U, sKnotV, sNcp, sDegree, sUsize, sEw, sEP, sudim, xdim);
            
        	% pp from X to patch:
            A0 = At' ;
            [A, ~] = v2865b_onePoint (X, A0, pDegree, pNcp, pKnotV, pUsize, pP ,pw, pEP ,pEw, pCD, pULims, xdim, Xtol, DUtol);
            A = A';
            
            % allocates:
            
            ip(i,1).Bi = 0 ; % remains = 0
            ip(i,1).Ui = U ;
            ip(i,1).Ai = A ;
            ip(i,1).Xi = X ;
            ip(i,1).NCSi = [0 sR];
            
            ip(i,1).nAi = At ;
            
        end

end
   




end

