function [ipS, uNip, uip] = ts1615_ipState (ip, Nte, teR, Av, connT, nd)

% RETURNS:
% ipS: [ 1:     intermediate points state, 
%        2:     sub-state, in case it lies in another element (is the state in that outside element)
%        3,4:   local ref of side or edge affected if applies
%        5:     relevant element/side global ref ]
% Updated intermediate points, that form an 'inside row'

% In case of one point at boundary or other tet, stores the involved point
% in the first position of uip, and the rest must be ignored.

pudim = 3 ;

Anod = zeros(4,pudim);
P = zeros(pudim,1);

ipS = [ 1 0 0 0 0] ;
uNip = 0 ;
uip(1,1) = struct( 'Bi',[], 'Ui',[], 'Ai',[], 'Xi',[], 'NCSi',[], 'nAi',[] );


for e = 2:1:Nte-1
    
    % intermediate point A coords:
    
    P(:,1) = ip(e-1).Ai(:,1);
    
    
    % tet where it should lie in:
    
    eR = teR(e);

    for k = 1:1:4
        nR = connT.EV(eR,k);
        Anod(k,:) = Av(nR,:);
    end

    
    % evaluates:
    % sR and lR are local refs of side and line, in case P lies on them
    
    [IN, sR, lR, ~, ~] = go2539_PinTet (Anod, P, 2*nd);
    
    
    if IN == 1
       
        % inside:
        
        uNip = uNip + 1 ;
        
        uip(uNip,1) = ip(e-1,1);
        
        
    elseif IN == 2
        
        % side of current tet:
        
        uip(1,1) = ip(e-1,1);
        ipS = [ IN 0 sR 0 eR ] ;
        break
        
    elseif IN == 3
        
        % edge of current tet:
        
        uip(1,1) = ip(e-1,1);
        ipS = [ IN 0 lR(1) lR(2) eR];
        break
        
    elseif IN == 4
        
    	if NOTCHECKED == 0
        end
        
    elseif IN == 0
       
        if NOTCHECKED == 0
        end
        
        % outside current tet:
        
        uip(1,1) = ip(e-1,1);
        
        % finds element:
        % outS: same as ipS without first member and for the outside element
        [outS] = ts1619_outElem (IP, connT, Av, nd);
        
        ipS = [ IN outS ] ;
        break
        
    end


end



end

