function [pNE, connT, THC] = ts1695_CorrC ...
         (pNE, connT, THC, deR, Spatches, pR, rTskin, agCurv, MLT,  Xtol, DUtol)

% RETURNS:
% connT and pNE updated with correction of CAP or SLIVER distorted element

% INPUTS:
% pNE, connT: current tets
% deR: distorted element ref.
% RP: ref to print, just for checking purpose

% Removes one edge and splits all the elemnts attached to the removed edge.

pudim = 3 ;
xdim = 3 ;

pDegree = Spatches(pR).INS(1).mDegree;
pNcp    = Spatches(pR).INS(1).mNcp;
pKnotV  = Spatches(pR).INS(1).mKnotV;
pP      = Spatches(pR).INS(1).mP;
pw      = Spatches(pR).INS(1).mw;
pUsize  = Spatches(pR).INS(1).mUsize;
pULims  = [0 1 ; 0 1 ; 0 1 ];
pEP     = Spatches(pR).INS(1).mEP;
pEw     = Spatches(pR).INS(1).mEw;
pCD     = [0 0 0 ];


% 1.- DETECTION OF LINES CANDIDATE TO REMOVE:
% These are a couple lines that crosses each other in a point, and such 
% distance is the lesser

olr = [ 1 2 3 4 ; 2 3 1 4 ; 1 3 2 4]; % opposite lines refs

A = zeros(pudim,1);
B = zeros(pudim,1);
x = zeros(pudim,1);

dmin = 9999;
ps = 0 ;

for p = 1:1:3
    
    % nodes refs:
    
    a1 = connT.EV(deR, olr(p,1));
    a2 = connT.EV(deR, olr(p,2));
    
    b1 = connT.EV(deR, olr(p,3));
    b2 = connT.EV(deR, olr(p,4));
    
    
    % lines points/vects:
    
    A(:,1) = THC.Av(a1,:);
    x(:,1) = THC.Av(a2,:);
    u = x-A;
    
 	B(:,1) = THC.Av(b1,:);
    x(:,1) = THC.Av(b2,:);
    v = x-B;
    
    h = go2547_dBtwn2Lines (A,u, B,v);
    
    if h < dmin
        dmin = h ;
        ps = p;
    end
    
end


% local and global refs of selected nodes and lines:

r = olr(ps,:); 

gN = [ connT.EV(deR,r(1)) , connT.EV(deR,r(2)) , connT.EV(deR,r(3)) , connT.EV(deR,r(4)) ] ;


gL = zeros(2,1);

for l = 1:1:6
    
    r = connT.EL(deR,l);
    
    n1 = connT.LV(r,1);
    n2 = connT.LV(r,2);
    
    if (n1 == gN(1) && n2 == gN(2)) || (n2 == gN(1) && n1 == gN(2))
        gL(1) = r ;
    elseif (n1 == gN(3) && n2 == gN(4)) || (n2 == gN(3) && n1 == gN(4))
        gL(2) = r ;
    end
end




% 2.- CLOSEST POINT TO BOTH LINES:

N = 2 ;

a = zeros(N,pudim); % points of lines
v = zeros(N,pudim); % vectors of lines

a(1,:) = THC.Av(gN(1),:);
a(2,:) = THC.Av(gN(3),:);

v(1,:) = THC.Av(gN(2),:) - THC.Av(gN(1),:);
v(2,:) = THC.Av(gN(4),:) - THC.Av(gN(3),:);

[CP] = go2523_P_to_nLines (a, v, N, pudim);


% projection of CP on both lines and its position w.r.t. ends of segment:

CPp = zeros(2,pudim);
ON  = zeros(2,1); % projection ON segment or not
pD = zeros(2,1); % percejage of distance to end of segment, w.r.t. segment length

tolL = 0.5 ;

for r = 1:1:2
    
    A(:,1) = THC.Av(gN(2*r-1 + 0),:);
    B(:,1) = THC.Av(gN(2*r-1 + 1),:);
    x(:,1) = v(r,:);
    
    [~, d] = go2527_distPtoRecta (CP, A, x, pudim);
    
    tmp = CP + d ;
    
    CPp(r,:) = tmp(:,1);
    
    ON(r,1) = go2520_PonSegment (tmp, A, B, pudim, tolL);
    
    dmin = min(norm(tmp-A), norm(tmp-B));
    
    pD(r,1) = 100*dmin / norm(B-A);
    
end



% 3.- DETERMINES TYPE: CAP or SLIVER:

% If both projections CPp are within the segment and not more than 20% far
% away from ends, then it is is SLIVER, otherwise it is CAP.

pDLim = 20 ;

if sum(ON)==2 && (pD(1) > pDLim && pD(2) > pDLim)
    typ = "S";
else
    typ = "C";
end


% 4.- SLIVER tets:

if typ == "S"
    
    [eTHC, Ne, eR, neR] = ts1696_CorrCS ...
    (deR, THC, MLT, connT, gN, gL, CPp, Spatches, pR, rTskin, agCurv, Xtol, DUtol);

    pNV = size(THC.Av,1) + 1; 
    nnR = pNV ;
    
    % updates TETS vtx coords:
        
  	THC.Bv = [ THC.Bv ; eTHC.Bv ];
  	THC.Uv = [ THC.Uv ; eTHC.Uv' ];
   	THC.Av = [ THC.Av ; eTHC.Av' ];
   	THC.Xv = [ THC.Xv ; eTHC.Xv' ];
        
   	THC.NCS = [ THC.NCS ; eTHC.NCS ];
   	THC.rm  = [ THC.rm ; eTHC.rm ];
       
    
    oconnT = connT ;
    
  	% updates conn and NE:
        
    for k = 1:1:2
    
        nR = [ neR(k,1) , neR(k,2) ] ;
        
        LeRd = zeros(Ne(k),2);

        for i = 1:1:Ne(k)

            leR = ts1620_LedgeR (oconnT, nR, eR(k,i));
            LeRd(i,1) = leR(1);
            LeRd(i,2) = leR(2);
            
        end
        
        ieR = [eR(k,1:Ne(k))]';
        
        [pNE, connT] = ei1663_TetDiv2 (THC, pNE, connT, Ne(k), ieR, nnR, LeRd);
        
    end
    
    [pNE, connT.EV] = th1322_remTetNS (pNE, connT.EV, deR); % deletes element deR
    
    [connT] = th1333_OtherConnLin (pNE, pNV, connT.EV);

    
	% Updates MLT:
        
        if jj==0
        end
        
        
	% Prints for checking:
 	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 	% {
        
 	fr = deR ;
   	figure(fr)
        
  	pt1373_pTetConn (deR, oconnT, THC.Av, fr, 'r', 0.33, 3.2, 0, 0);
        
  	for k = 1:1:2
        for i = 1:1:Ne(k)
            
            pt1373_pTetConn (eR(k,i), oconnT, THC.Av, fr, 'k', 1, 0.55, 0, 0);
            pt1373_pTetConn (eR(k,i),  connT, THC.Av, 100+fr, 'k', 1, 0.55, 0, 0);
            
        end
    end
        
 	eR = [pNE-sum(Ne)+1:1:pNE]';
   	for i = 1:1:sum(Ne)
        pt1373_pTetConn (eR(i), connT, THC.Av, 100+fr, 'k', 1, 0.55, 0, 0);
    end
        
  	axis equal
        
  	%}
 	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        

end


% 5.- CAP tets:

if typ == "C"
    
    
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plots for checking
%{

if typ=="S"

fr = deR ;
figure(deR)

L = zeros(2,pudim);

for p = 1:1:3
    
    cr = 'k';
    if p == ps
       cr = 'r'; 
    end

    a1 = connT.EV(deR, olr(p,1));
    a2 = connT.EV(deR, olr(p,2));

    L(1,:) = THC.Av(a1,:);
    L(2,:) = THC.Av(a2,:);
    
    plot3(L(:,1), L(:,2), L(:,3), cr); hold on
    
    
    b1 = connT.EV(deR, olr(p,3));
    b2 = connT.EV(deR, olr(p,4));
    
 	L(1,:) = THC.Av(b1,:);
    L(2,:) = THC.Av(b2,:);

    plot3(L(:,1), L(:,2), L(:,3), cr); hold on
 
    
    % nodes to remove
    for v = 1:1:2 
        scatter3 ( THC.Av(nGR(v),1), THC.Av(nGR(v),2),THC.Av(nGR(v),3),'m'); hold on
    end
    
    % new node:
    scatter3 ( eTHC.Av(1), eTHC.Av(2),eTHC.Av(3),'r', 'filled'); hold on
    
 	% closest point:
    scatter3 ( CP(1), CP(2),CP(3),'b', 'filled'); hold on
    
    
end

end






%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if deR > 300
    
    if gg==0
    end
    
end


end

