function [lUc, lXc, Lseg] = L2901_Lpart ...
    (ctam, cNmin, cNcp, cDegree, cUsize, cKnotV, cB, cw, cCD, cXdim, cUdim)

% RETURNS:
% lU(nlp,udim), lX(nlp,xdim): coordinate of points of curve attending to
% max. no points and tam() criteria and min. No. segments cNmin
% note it arranges points to achieve ascending parameter order on them

% Lseg(NS,1): length of each segment

Ltol = 0.51 ; % error in % of LR of curve to accept point location


% 1.- ESTIMATES SEGMENT REFERENCE LENGTH (LR) & No. SEGMENTS (NS):

[Ltotal, LR, NS] = Li2909_FixSize ...
(cKnotV, cNcp,cDegree,cUsize, cB, cw, cUdim, cXdim, cNmin, ctam);

tol = LR*Ltol/100; % abs. value of tolerance


% 2.-  PRE-SIZES ANSWERS AND DEFINES PRELIMINARY u VALUES:

lUc = zeros(NS+1,1);
lXc = zeros(NS+1,cXdim);

Lseg = zeros(NS,1); 

uF = cKnotV(1) ; % First u
uL = cKnotV(cUsize) ; % Last u


% 3.- COMPUTES POINTS COORDINATES:

% residual to evaluate is: r(u) = LR - l
% where l is computed 

% loop on No. segments (computes second point of the segment)
% note first and last points coincide with extreme points

NGPs = 4 ;

lUc(1,1) = uF;
lUc(NS+1,1) = uL;

u1 = uF;

uSpan = uL - uF ;
Du = uSpan / NS ;
pDu = 2 * Du * Ltol/100; % for potential small increments
%u2 = u1+Du;

upr = [uF:Du:uL];



for s = 2:1:NS
    
    
    % 3.1.- Computes actual length and residual:
    
    u2 = upr(s); % estimation of second point
    
    L = L2902_Lsegm (u1,u2, NGPs, cKnotV, cNcp,cDegree,cUsize, cB, cw, cUdim, cXdim);
    r = LR - L ;

    if abs(r)<tol
        
        % 3.1.1.- In case of residual accepable just stores de value.
        
        lUc(s,1) = u2;
        Lseg(s-1,1) = L ;
        ufinal = u2 ;
        
    else
             
        % 3.1.2.- Otherwise recomputes until gets into tolerance
        
        sg = sign(r);
        
        % 3.3.- L is less than required:
        
        % search 2nd point where r changes sign
        
        u3 = u2 ;
        u4 = u2 ;
        while (sg*r > 0 && abs(r) > tol)
            
            u3 = u4 ;
            u4 = u4 + sg*pDu ;
            
            L = L2902_Lsegm (u1,u4, NGPs, cKnotV, cNcp,cDegree,cUsize, cB, cw, cUdim, cXdim);
            r = LR - L ;
        end
        

        % if r is acceptable just allocates
        if abs(r)<tol

            lUc(s,1) = u4 ;
            Lseg(s-1,1) = L ;
            ufinal = u4 ;
            
        % if not, applies mid-point (between u3 & u4) to achieve abs(r1)<tol    
        else
            
            um = (u3 + u4)/2 ;
            while (abs(r) > tol)
                
                L = L2902_Lsegm (u1,um, NGPs, cKnotV, cNcp,cDegree,cUsize, cB, cw, cUdim, cXdim);
                r = LR - L ;
                
                % updates um:
                if sg*r > 0
                    % u3 and um same side => updates u3 as um
                    u3 = um ;
                elseif sg*r < 0
                    % u4 and um same side => updates u4 as um
                    u4 = um ; 
                end
                um = (u3 + u4)/2 ;
                
            end
            
            lUc(s,1) = um;
            Lseg(s-1,1) = L ;
            ufinal = um ;
            
        end
    end

    
    % 3.2.- sets initial u and estimates final u of this increment:
    
    u1 = ufinal ; % starting point is the latest computed
    %uSpan = uL - ufinal ;
    %Du = uSpan / (NS - (s-1)) ;
    %pDu = 2 * Du * Ltol/100; % for potential small increments
    %u2 = u1 + Du ;
    
end

% latest segment
Lseg(NS,1) = L2902_Lsegm (lUc(NS,1),lUc(NS+1,1), NGPs, cKnotV, cNcp,cDegree,cUsize, cB, cw, cUdim, cXdim);


% 4.- SPREAD ERROR IN THE LATEST SEGMENT BETWEEN THE PREVIOUS:

% do it proportionally

errL = Lseg(NS,1) - LR ;





% 4.- COMPUTES PHYSICAL COORDINATES OF SELECTED POINTS:

for i = 1:1:NS+1
    
    [X] = b2601_BNURBS_Xcoord (lUc(i), cKnotV, cNcp, cDegree, cUsize, cw,cB, cUdim, cXdim);

    for di=1:1:cXdim
    lXc(i,di) = X(di);
    end
end


%[LR-tol LR+tol]
%Lseg

%lUc
%lXc

end

