function [lUc, lXc, Lseg] = Li2905_Lpart ...
         (ctam, cNmin, cNcp, cDegree, cUsize, cKnotV, cB, cw, cXdim, cUdim)

% RETURNS:
% lU(nlp,udim), lX(nlp,xdim): coordinate of points of curve attending to
% max. no points and tam() criteria and min. No. segments cNmin
% note it arranges points to achieve ascending parameter order on them

% Lseg(NS,1): length of each segment


% works from parameters space, estimating with derivatives



% 1.- ESTIMATES SEGMENT REFERENCE LENGTH (LR) & No. SEGMENTS (NS):

[Ltotal,LRx,NS] = Li2909_FixSize ...
(cKnotV, cNcp,cDegree,cUsize, cB, cw, cUdim, cXdim, cNmin,ctam);


lUc  = zeros(NS+1,cUdim); % they might increase to fit No. sub-segments
lXc  = zeros(NS+1,cXdim);
Lseg = zeros(NS,1);

% 2.- FIXES Dutrial & 
% - STORES TRIAL COORDINATES                       => uo(NS+1), xo(NS+1,cXdim)
% - MEASURES EACH SEGMENT LENGTH IN PHYSICAL SPACE => Ltx(NS)
% - STORES curve DERIVATIVE w.r.t. u AT EACH uo    => dCu(NS+1), mCu(module)

NGPs = cDegree + 1 ;

uF = cKnotV(1) ; % First u
uL = cKnotV(cUsize) ; % Last u

Dutrial = (uL - uF) / NS ;

uo   = [uF:Dutrial:uL];   % initial u value of each segment (also stored the last one)
xo   = zeros(NS+1,cXdim); % correspondant positions in physical space

Ltx  = zeros(NS,1);
dCu = zeros(NS+1,cXdim);
mCu = zeros(NS+1,1);


for s = 1:1:NS
    
    u1 = uo(s);
    u2 = uo(s+1);
    
    % correspondant X coordinates:
    [X] = b2601_BNURBS_Xcoord (u1, cKnotV, cNcp,cDegree,cUsize, cw,cB, cUdim,cXdim);
    for di=1:1:cXdim
        xo(s,di) = X(di);
    end
    
    % trial segment length:
    Lsegt = Li2902_Lsegm (u1,u2, NGPs, cKnotV, cNcp,cDegree,cUsize, cB, cw, cUdim, cXdim);
    Ltx(s,1) = Lsegt ;
    
    % dC,u at u1:
    dC = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize, cB ,cw, 1, u1, cXdim,cUdim);
    acc = 0 ;
    for j = 1:1:3
        dCu(s,j) = dC(2,j);
        acc = acc + dC(2,j)^2 ;
    end
    mCu(s,1) = acc^0.5 ;
end



% derivative and Xcoor of lastest point:

dC = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize, cB ,cw, 1, u2, cXdim,cUdim);

acc = 0 ;
for j = 1:1:3
    dCu(s+1,j) = dC(2,j);
    acc = acc + dC(2,j)^2 ;
end
mCu(s+1,1) = acc^0.5 ;

[X] = b2601_BNURBS_Xcoord (u2, cKnotV, cNcp,cDegree,cUsize, cw,cB, cUdim,cXdim);
 for di=1:1:cXdim
    xo(s+1,di) = X(di);
 end

figure(1);
plot(xo(:,1), xo(:,2)); hold on
scatter(xo(:,1), xo(:,2),'r');
axis equal
 
 



% 3.- COMPUTES No. SUB-SEGMENTs IN EACH SEGMENT: => nss(NS,1)

nss = zeros(NS,1);

for s = 1:1:NS
    nss(s,1) = round (Ltx(s)/LRx + 0.4 , 0);
end


% 4.- DIVIDE EACH SEGMENT CONSIDERING dCu VALUES:

lUc(1) = uF ;
for di=1:1:cXdim
	lXc(1,di) = cB(1,di); 
end

contaU = 1;

for s = 1:1:NS
    
    % target sub-segments length
    Ssx = Ltx(s) / nss(s); 
    
    
    % interpolation line parameters:
    m = (mCu(s+1,1) - mCu(s,1)) / (uo(s+1) - uo(s)) ;
    n = mCu(s) - m*uo(s);

    
    % loop on sub-segments:
    
    accSsx = Ssx ;
    
    for ss = 1:1:nss(s)
        
        % computes u2 for achieving desired increment in curve:
        
        if ss<nss(s)
            
            a = m/2 ;
            b = n ;
            c = -accSsx - m/2*uo(s)^2 - n*uo(s);
            ur = GEN_24_rootQuadPol (a,b,c, [uo(s) uo(s+1)]);
            
        elseif ss == nss(s)
            
            ur = uo(s+1);
            
        end
        

        % allocates values:
        
        contaU = contaU + 1 ;
        
        lUc(contaU) = ur(1) ;
        
        
        [X] = b2601_BNURBS_Xcoord (ur(1), cKnotV, cNcp,cDegree,cUsize, cw,cB, cUdim,cXdim);
      	for di=1:1:cXdim
            lXc(contaU,di) = X(di);
        end
        
        Lsegt = Li2902_Lsegm ...
       	(lUc(contaU-1),lUc(contaU), NGPs, cKnotV, cNcp,cDegree,cUsize, cB,cw, cUdim,cXdim);
        Lseg(contaU) = Lsegt;
   
        
        % updates accumulated target length:
        
        accSsx = accSsx + Ssx ;
        
    end
    
end




end

