function [cU, cX, Lseg] = cu2901_Cpartition ...
         (ctam, cNmin, cNcp, cDegree, cUsize, cKnotV, cB, cw, cXdim, cUdim)

% RETURNS:
% lU(nlp,udim), lX(nlp,xdim): coordinate of points of curve attending to
% max. no points and tam() criteria and min. No. segments cNmin
% note it arranges points to achieve ascending parameter order on them

% Lseg(NS,1): length of each segment


% works from parameters space, estimating with derivatives




% 1.- ESTIMATES SEGMENT REFERENCE LENGTH (LR) & No. SEGMENTS (NS):

[~,LRx,NS] = cu2909_FixSize ...
(cKnotV, cNcp,cDegree,cUsize, cB, cw, cUdim, cXdim, cNmin,ctam);

cU  = zeros(NS+1,cUdim); % they might increase to fit No. sub-segments
cX  = zeros(NS+1,cXdim);
Lseg = zeros(NS,1);

NGPs = cDegree + 1 ;


% 2.- TRIAL POINTS ALLOCATION:
% distribute evenly in parameter space NS+1 points and computes
% parameter coordinates and their segments lengths

uF = cKnotV(1) ; % First u
uL = cKnotV(cUsize) ; % Last u

Dutrial = (uL - uF) / NS ;

uo   = [uF:Dutrial:uL];   % initial u value of each segment (also stored the last one)
Ltx = zeros(NS,1);
accLtx = zeros(NS,1);

for s = 1:1:NS

    % trial segment length:
	Ltx(s,1) = cu2902_Lsegm ...
    (uo(s),uo(s+1), NGPs, cKnotV, cNcp,cDegree,cUsize, cB,cw, cUdim,cXdim);

    if s == 1
        accLtx(s,1) = Ltx(s,1);  
    else
        accLtx(s,1) = accLtx(s-1) + Ltx(s,1);    
    end
end


% 3.- CORRECTS POSITION OF TRIAL POINTS
%   the corrections points towards more even distribution in physical space,
%   then it is proportional to the trial segments length in physical space

accLRx = 0 ;

uc = zeros(NS+1,1);

uc(1) = uF;
contaU = 1 ;
for s = 1:1:NS-1
    
    accLRx = accLRx + LRx;
    
    f = accLRx/accLtx(s); % proportional factor

    utemp = uo(s+1) * f ;
    
    if utemp <= uL
       contaU = contaU+1;
       uc(contaU) = utemp;
    end
end

if uc(contaU)~= uL
    contaU = contaU+1;
    uc(contaU) = uL;
end

uc = uc(1:1:contaU);


% 4.-   COMPUTES DERIVATIVES AT EACH uc POSITION & 
%       m,n INTERPOLATION LINES COEFFICIENTS:

dCu = zeros(contaU,cXdim);
mCu = zeros(contaU,1);

sm = zeros(contaU-1,1);
sn = zeros(contaU-1,1);

for i = 1:1:contaU
    
	% dC,u at uc(i):
    dC = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize, cB ,cw, 1, uc(i), cXdim,cUdim);
    acc = 0 ;
    for j = 1:1:cXdim
        dCu(i,j) = dC(2,j);
        acc = acc + dC(2,j)^2 ;
    end
    mCu(i,1) = acc^0.5 ;
    
    % interpolation coeff's:
    if i>1
        
        [sm(i-1),sn(i-1)] = go2516_mnCoeff ( [uc(i-1) uc(i)] , [mCu(i-1) mCu(i)]);
        
        %sm(i-1) = (mCu(i) - mCu(i-1)) / (uc(i) - uc(i-1)) ;
        %sn(i-1) = mCu(i) - sm(i-1)*uc(i);
    end
end




% 5.-   INSERTS INTERMEDIATE POINTS PROPORTIONAL TO 
%       DERIVATIVE INCREMENT IN EACH SEGMENT:
%       also computes slope coeff's.

Imin = min(abs(sm)); % max. and min. slopes of dC/du
Imax = max(abs(sm));

% inserts intermediate where slopes are in the pg % higher slopes:
pg = 0.40;
Sthr = Imin + (1-pg)*(Imax-Imin);

dCu2 = zeros(contaU,cXdim);
mCu2 = zeros(contaU,1);

for di=1:1:cXdim
dCu2(1,di) = dCu(1,di);
end
mCu2(1) = mCu(1);

sm2 = zeros(contaU-1,1);
sn2 = zeros(contaU-1,1);

uc2 = zeros(contaU,1);
uc2(1) = uc(1);
contaU2 = 1;

for s = 1:1:contaU-1
    
    if abs(sm(s)) < Sthr
        
        % copy
        contaU2 = contaU2 + 1 ;
        uc2(contaU2) = uc(s+1);
        
        for di=1:1:cXdim
        dCu2(contaU2,di) = dCu(s+1,di);
        end
        mCu2(contaU2) = mCu(s+1);
        
        sm2(contaU2 - 1) = sm(s);
        sn2(contaU2 - 1) = sn(s);
        
    else
        
        % inserts
        contaU2 = contaU2 + 1 ;
        uc2(contaU2) = (uc2(contaU2-1) + uc(s+1)) / 2 ;
        
        dC = b2701_BNURBS_deriv (cDegree, cNcp, cKnotV, cUsize, cB ,cw, 1, uc2(contaU2), cXdim,cUdim);
        acc = 0 ;
        for j = 1:1:cXdim
            dCu2(contaU2,j) = dC(2,j);
            acc = acc + dC(2,j)^2 ;
        end
        mCu2(contaU2,1) = acc^0.5 ;
        
        [sm2(contaU2-1),sn2(contaU2-1)] = go2516_mnCoeff ...
       	([uc2(contaU2-1) uc2(contaU2)] , [mCu2(contaU2-1) mCu2(contaU2)]);
        
        %sm2(contaU2 - 1) = (mCu2(contaU2) - mCu2(contaU2-1)) / (uc2(contaU2) - uc2(contaU2-1)) ;
        %sn2(contaU2 - 1) = mCu2(contaU2) - sm2(contaU2-1)*uc2(contaU2);
        
        
        % copy
        contaU2 = contaU2 + 1 ;
        uc2(contaU2) = uc(s+1);
        
      	for di=1:1:cXdim
        dCu2(contaU2,di) = dCu(s+1,di);
        end
        mCu2(contaU2) = mCu(s+1);
        
        sm2(contaU2 - 1) = sm(s);
        sn2(contaU2 - 1) = sn(s);
        
    end
end



% Redefines vbles:

Nts = contaU2-1; % No. trial segments
uc  = uc2;
%dCu = dCu2;
mCu = mCu2 ;
%sm  = sm2 ;
%sn  = sn2 ;


% Computes X-length of each trial segment:

Ltx = zeros(Nts,1);
accLtx = zeros(Nts,1);

for s = 1:1:Nts
    
    Ltx(s,1) = cu2902_Lsegm ...
    (uc(s),uc(s+1), NGPs, cKnotV, cNcp,cDegree,cUsize, cB,cw, cUdim,cXdim);
    
    if s==1
        accLtx(1) = Ltx(1);
    else
        accLtx(s) = accLtx(s-1) + Ltx(s);
    end
end



% 6.- APPLIES INTEGRATION OF u TO ESTIMATE POSITION OF EQUIDIST POINTS:

% First point is always for u=uF;

cU(1,1) = uF;

[X] = b2601_BNURBS_Xcoord (cU(1), cKnotV, cNcp,cDegree,cUsize, cw,cB, cUdim,cXdim);
for di=1:1:cXdim
    cX(1,di) = X(di);
end


% computes 'end' of each segment

contaS = 0 ;
nss = 3 ;

for sr = 1:1:NS
    
    rjF = 0 ;
    
    % 6.1.- Target length:
    
    if sr == 1
        Lacc = LRx ;
    else
        Lacc =  Lacc + LRx ;
    end
    
    
    % Finds measured segment where it belongs and computes u ( = ur(1)) :

    % 6.2.- Case searched length is within Uspan:
    if sr < NS

        % 6.2.1.- Finds first span that contains our target length Lacc
        for s = 1:1:Nts
      	if Lacc <= accLtx(s) 
            
            s1 = s ;
            
            % 6.2.2.- Computes increment between start point 
            % of trial segment and target length:
            
            if s1 == 1
                DL = Lacc;
            else
                DL = Lacc - accLtx(s1-1);
            end
            
            segL = Ltx(s1);
            u1 = uc(s1);
            u2 = uc(s1+1);
            mCu1 = mCu(s1);
            mCu2 = mCu(s1+1);
            upr = cU(sr);
            
            % 6.2.3.- Estimates u to achieve the target length (uses linear
            % piecewise approximation, dividing the trial segment into nss
            % No. sub-segments)
            
            [ur, rjF] = cu2908_urToL ...
                (segL, u1, u2, mCu1, mCu2, upr, uL, DL, nss, ...
                 cDegree, cNcp, cKnotV, cUsize, cB,cw, cXdim,cUdim);
            
            break
        end
        end
      
        ucomp = ur;
        

        
    % 6.3.- Case last point:
    
    elseif sr==NS
        ucomp = uL;
    end
    
    
    % 6.4.- In case the computation is not rejected, allocates to responses arrays:
    
    if rjF==0
        
        contaS = contaS + 1 ;
        
        % computes X position and segment length:
        [X] = b2601_BNURBS_Xcoord (ucomp, cKnotV, cNcp,cDegree,cUsize, cw,cB, cUdim,cXdim);
        Ls = cu2902_Lsegm (cU(sr,1),ucomp, NGPs, cKnotV, cNcp,cDegree,cUsize, cB,cw, cUdim,cXdim);

        % allocates:
        cU(contaS+1,1) = ucomp ;
        for di=1:1:cXdim
            cX(contaS+1,di) = X(di);
        end
        Lseg(contaS,1) = Ls ; 
    end
end


% re-sizes responses (in case reaches end earlier than expected latest
% segments are zero):
if contaS < NS
    cU = cU(1:1:contaS+1,:);
    cX = cX(1:1:contaS+1,:);
    Lseg = Lseg(1:1:contaS);
end




end

