function [ur, rjF] = cu2908_urToL ...
                     (segL, u1, u2, mCu1, mCu2, upr,uL, DL, nss, ...
                      cDegree, cNcp, cKnotV, cUsize, cB,cw, cXdim,cUdim)

% RETURNS:
% ur: value of u that corresponds to Lacc
% rjF: reject flag, rjF = 1 if the calcuation is not to be considered

% INPUTS:
% segL: real x-length segment where search in
% u1,u2: u-lims of segement 
% mCu1, mCu2: dC/du at u1 and u2
% upr: position of previous point. Starting point of new length.
%       upr belongs to u1-u2
% uL: last value of knot V
% DL: target increment of length
% nss: No. subsegments to seach.

rjF = 0;

% 1.- If DL is very close to segL => solution end of the segment

Lerr1 = 100*abs(segL-DL) / segL ;
Lerr2 = 100*DL / segL ;


if Lerr1 < 1.5
    ur = u2 ;
    return
end

if Lerr2 < 1.5
    ur = u1 ;
    return
end

% 2.- Detects if ur is out of expected limits:

m = (mCu2 - mCu1) / (u2 - u1) ;
n = mCu2 - m*u2;

a = m/2 ;
b = n ;
c = -DL - m/2*u1^2 - n*u1;
[utemp, niF] = GEN_24_rootQuadPolm (a,b,c, [upr u2], 0.00001);

if niF == 1 % not in the limits flag actived
    
    % 2.1.- if utemp less than upr => rejects 
    if u1<=utemp(1) && utemp(1)<=u2
        ur = 0;
        rjF = 1 ;
        return
        
    % 2.2.- if utemp more than lims, but inside KnotV assumes is the answer     
    elseif u2<utemp(1) && utemp(1)<uL
        ur = utemp(1);
        return
        
    % 2.3.- if utemp is greater than knot span => rejects    
    elseif uL <= utemp(1)
        ur = 0;
        rjF = 1 ;
        return
    end
    
    
    
end


% In normal cases (not the previous) carry on:

% 3.- Computes derivatives at each sub-segment:

Du = (u2-u1)/nss ;

uss = [u1:Du:u2];
mCu = zeros(nss+1,1);

mCu(1) = mCu1;
mCu(nss+1) = mCu2;

for s = 2:1:nss
   	dC = b2701_BNURBS_deriv ...
    (cDegree, cNcp, cKnotV, cUsize, cB,cw, 1, uss(s), cXdim,cUdim);
   	acc = 0 ;
   	for j = 1:1:cXdim
        acc = acc + dC(2,j)^2 ;
    end
	mCu(s) = acc^0.5 ;
end


% 4.- Computes line coeff's and estimated length in each:

sm = zeros(nss,1);
sn = zeros(nss,1);

Leacc = zeros(nss,1);

for s=1:1:nss
    
    sm(s) = (mCu(s+1) - mCu(s)) / (uss(s+1) - uss(s)) ;
    sn(s) = mCu(s+1) - sm(s)*uss(s+1);
    
    if s==1
        Leacc(s) = (uss(s+1) - uss(s)) * 0.5*(mCu(s+1) + mCu(s));
    else
        Leacc(s) = Leacc(s-1) + (uss(s+1) - uss(s)) * 0.5*(mCu(s+1) + mCu(s));
    end

end


% 5.- Finds sub-segment:

for s = 1:1:nss

    if DL<=Leacc(s)

        m = sm(s);
        n = sn(s);
        if s==1
            sDL = DL ;
        else
            sDL = DL-Leacc(s-1);
        end
        
        su1 = uss(s);
        su2 = uss(s+1);
        supr = su1;
        
        a = m/2 ;
        b = n ;
        c = -sDL - m/2*su1^2 - n*su1;
        [utemp, niF] = GEN_24_rootQuadPolm (a,b,c, [supr su2], 0.00001);
        
        ur = utemp(1);
        
        break
    end

end




end

