function [lUc, lXc] = li2901_Lskin (cstp, ctam, cNmin,...
                                  cNcp, cDegree, cUsize, cKnotV, cB, cw,cCD, cXdim, cUdim)

% RETURNS:
% lU(nlp,udim), lX(nlp,xdim): coordinate of points of curve attending to
% max. no points and tam() criteria
% note it arranges points to achieve ascending parameter order on them

% 1.- PRELIMINARY SIZES & AUX. OPERATIONS:

%lUc = zeros(1,cUdim);
%lXc = zeros(1,cXdim);

% knot vector has to be row vector:
if size(cKnotV,2)==1
    cKnotV = cKnotV';
end


% 2.- COMPUTES U-COORDINATES:

% 2.1.- Computes preliminary collection of points, preP(np,xdim), that
% belong to surface and correspontant to param. space point at stp(udim) step   

[preP] = GEN_15_PatchP ...
(cstp, cNcp, cDegree, cUsize, cKnotV, cB, cw, cXdim, cUdim);
        
   
% 2.2.- Creates xdim net of rectangles enveloping the surface:
% envR = struct('Lims', [xdim,2] )    

[envR, neE] = z2941_envR (ctam, cNmin, preP, cXdim);


% 2.3.- Stores all rect. corners that have inside any surface point:

[cP, ncp] = z2942_closeP (envR, neE, preP, cXdim);
       

% 2.4.- Computes by point projection coordinates on the curve,

[lUc] = b2801_PPforSetPoints...
(ncp,cP, cDegree, cNcp, cKnotV, cUsize, cB,cw,cCD, cXdim,cUdim,0);
  

% 2.5.- Rounds up to 4 decimal knot span and sorts ascending:

lUc = round(lUc,4);
lUc = sortrows(lUc,1);


% 2.6.- Removes repeated:

temp = zeros(1,1);
conta = 0 ;

for i = 1:1:size(lUc,1)-1
if lUc(i,1) ~= lUc(i+1,1)
    conta = conta + 1 ;
    temp(conta,1) = lUc(i,1);
end
end

if lUc(size(lUc,1),1) ~= temp(conta,1)
    conta = conta + 1 ;
    temp(conta,1) = lUc(size(lUc,1),1) ;
end
lUc = temp ;

% 2.7.- Add initial and final parameter if they does not exist:

if lUc(1,1)~= cKnotV(1)
    lUc = [ cKnotV(1) ; lUc ];
end

if lUc(size(lUc,1),1)~= cKnotV(cUsize)
    lUc = [ lUc ; cKnotV(cUsize) ];
end



% 3.- COMPUTES X-COORD'S:

np = size(lUc,1);

lXc = zeros(np,cXdim);

for po = 1:1:np
    
    u = lUc(po,1);
    
    [X] = b2601_BNURBS_Xcoord (u, cKnotV, cNcp,cDegree,cUsize, cw,cB, cUdim,cXdim);
    
    for di=1:1:cXdim
        lXc(po,di) = X(di);
    end

end



% 4.- ARRANGES COORDINATES ACCORDING TO ASCENDING Ucoor:

temp = [lUc, lXc];
temp = sortrows(temp,1);
        
for po=1:1:np
    
    for di=1:1:cUdim
        lUc(po,di) = temp(po, 1);
    end
    
    for di=1:1:cXdim
        lXc(po,di) = temp(po, di+1);
    end
end


end

