function [ui, rF] = s2923_Xcircf ...
    (ua,ub, ta,tb, tamp, R,Du, NE,ULimE,conM, dSu,dSv, Dumax, ULims, xdim)

% RETURNS:
% ui(2): intersection points between 2 circunf. in parameter space, 
% given their in radio defined in physical space

% INPUTS:
% ua, ub: param. coordinates of origin of both circumferences
% ta, tb: ref. angle in parameter space for the searching arc (angle of arc bi-sectriz)
% tamp: semi-amplitude of search arc angle
% R: circ. radius in physical space

rF = 0;
rFp = zeros(4,1);
ui = zeros(1,2);


% 1.- First couple of segments:

[ua1,rFp(1)] = s2924_DuForDx (R,ta-tamp, ua,Du, NE,ULimE,conM, dSu,dSv, Dumax, ULims, xdim);
[ua2,rFp(2)] = s2924_DuForDx (R,ta+tamp, ua,Du, NE,ULimE,conM, dSu,dSv, Dumax, ULims, xdim);

[ub1,rFp(3)] = s2924_DuForDx (R,tb-tamp, ub,Du, NE,ULimE,conM, dSu,dSv, Dumax, ULims, xdim);
[ub2,rFp(4)] = s2924_DuForDx (R,tb+tamp, ub,Du, NE,ULimE,conM, dSu,dSv, Dumax, ULims, xdim);


% 2.- Rejects:


if sum(rFp)>1
    
    rF = 1 ;

    
% 3.- Computes intersection:

elseif sum(rFp)==0
    
    [ui] = go2526_TwoLinIntersec (ua1,ua2, ub1,ub2);

    % 3.1.- If intersection lies outside the domain => rejects
    if (ui(1) < ULims(1,1) || ULims(1,2) < ui(1)) ...
    || (ui(2) < ULims(2,1) || ULims(2,2) < ui(2))


        rF = 1;
    
        
    % 3.2.- If inside finds solution:    
    else

        NLim = 12 ;
        NL = 0;
        tol = 0.5 ; % percentage of variation with respect previous estimation
        err = tol + 1 ;

        while (err > tol && NL < NLim) && rF==0

            uprev = ui ;

            %tamp = tamp/1.5;

            ta = atan2((ui(2)-ua(2)),(ui(1)-ua(1)));
            tb = atan2((ui(2)-ub(2)),(ui(1)-ub(1)));

            [ua1,rFp(1)] = s2924_DuForDx (R,ta-tamp, ua,Du, NE,ULimE,conM, dSu,dSv, Dumax, ULims, xdim);
            [ua2,rFp(2)] = s2924_DuForDx (R,ta+tamp, ua,Du, NE,ULimE,conM, dSu,dSv, Dumax, ULims, xdim);

            [ub1,rFp(3)] = s2924_DuForDx (R,tb-tamp, ub,Du, NE,ULimE,conM, dSu,dSv, Dumax, ULims, xdim);
            [ub2,rFp(4)] = s2924_DuForDx (R,tb+tamp, ub,Du, NE,ULimE,conM, dSu,dSv, Dumax, ULims, xdim);

            %{
        	if round(ua(1),4) == 0.4022 && round(ua(2),4) == 0.4970
                if round(ub(1),4) == 0.4537 && round(ub(2),4) == 0.4136
                NL
                err
                ui
                rF
                end
            end
            %}
            
            if sum(rFp)>1
                rF = 1;
            end
            
            [ui, iF] = go2526_TwoLinIntersec (ua1,ua2, ub1,ub2);


            
            %{
        	if round(ua(1),4) == 0.4022 && round(ua(2),4) == 0.4970
                if round(ub(1),4) == 0.4537 && round(ub(2),4) == 0.4136
                ui
                end
            end
            %}
            

            if (ui(1) < ULims(1,1) || ULims(1,2) < ui(1)) ...
            || (ui(2) < ULims(2,1) || ULims(2,2) < ui(2))
        
                rF = 1;
                ui = [0 0];
            end
            
          
            
            
            err = norm(ui - uprev) / norm(uprev) * 100 ;

            NL = NL+1;
            
        end 

    end
    

    
end




end

