function [XP] = gp3174_SolidLattice (Ncp, gShell, pa)

% RETURNS:

% XP(n,m,3): coord's of control points defining a region bounded by 6 NURBS surfaces

% INPUTS:
% XPi(n,m,3): initial control points coord's. Include only perimetral cp's
% coords, the other are '0'.
% n,m: No. cp's in each direction

% E / Ed: young modulus of bars (horizt. and vertical) / diagonals

% creates a rectangular truss (nodes = control points) with diagonals 
% and prescibes displacements at contours to achieve final shape. Internal
% control points will follow the deformation


% Re-orders the system: 'a' unknown, 'b' known

% fa  =  Kaa  Kab  = ua  (uses first matrix row)
% fb     Kba  Kbb    ub

% fa  = Kaa ua + Kab ub => {fa - Kab ub} = Kaa ua

% ua = [Kaa]^-1  {fa - Kab ub}


% 1.- EXTRACTS SURFACES-HULL CONTROL POINTS:

xdim = 3 ;

XPi = zeros(Ncp(1),Ncp(2),Ncp(3),xdim);

conta = 0 ;
for j = 1:1:Ncp(2)
for k = 1:1:Ncp(3)
    
    conta = conta + 1 ;
    for di = 1:1:xdim
        
        XPi(1,j,k,di)      = gShell(pa).face(3).fSurface.P(conta,di);
        XPi(Ncp(1),j,k,di) = gShell(pa).face(6).fSurface.P(conta,di);
    end
end
end

conta = 0 ;
for i = 1:1:Ncp(1)
for k = 1:1:Ncp(3)
    
    conta = conta + 1 ;
    for di = 1:1:xdim
        
        XPi(i,1,k,di)      = gShell(pa).face(2).fSurface.P(conta,di);
        XPi(i,Ncp(2),k,di) = gShell(pa).face(5).fSurface.P(conta,di);
    end
end
end

conta = 0 ;
for i = 1:1:Ncp(1)
for j = 1:1:Ncp(2)
    
    conta = conta + 1 ;
    for di = 1:1:xdim
        
        XPi(i,j,1,di)      = gShell(pa).face(1).fSurface.P(conta,di);
        XPi(i,j,Ncp(3),di) = gShell(pa).face(4).fSurface.P(conta,di);
    end
end
end



% 2.- COMPUTES No. BARS:

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
Ncp = zeros(3,1);

Ncp(1) = 3 ;
Ncp(2) = 4 ;
Ncp(3) = 5 ;
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


NB = zeros(3,1); % No. bars in each param. direction

NB(1) = (Ncp(1)-1)*(Ncp(2)*Ncp(3));
NB(2) = (Ncp(2)-1)*(Ncp(3)*Ncp(1));
NB(3) = (Ncp(3)-1)*(Ncp(1)*Ncp(2));

ND = zeros(3,1); % No. of pairs of diagonals (in plane perp. to each direction)

ND(1) = Ncp(1)*(Ncp(2)-1)*(Ncp(3)-1);
ND(2) = Ncp(2)*(Ncp(3)-1)*(Ncp(1)-1);
ND(3) = Ncp(3)*(Ncp(1)-1)*(Ncp(2)-1);

TNB = sum(NB); % total No. bars
TN = sum(NB) + 2*sum(ND); % total No. bars + diags.

% No. nodes, elems & DoF:

nNod = Ncp(1)*Ncp(2)*Ncp(3);
DoF = nNod*xdim ;
NE = (Ncp(1)-1)*(Ncp(2)-1)*(Ncp(3)-1);


% 3.- BARS-VTX CONNECTIVITY MATRIX, R.ELEM-VTX CONN. MATRIX & EXTERNAL BAR FLAG:

BVcon = zeros(TN,2);

RVcon = zeros(NE,8);

flagEB = zeros(TN,1); % turns to 1 if is external bar, i.e. at least one node is permitral, except diagonals


% 3.1.- Bars & diags:

contaN = 0 ;

b1 = 0; % bars counter
b2 = 0;
b3 = 0;

d1 = 0 ; % diagonals counter
d2 = 0 ;
d3 = 0 ;



for i = 1:1:Ncp(1)
for j = 1:1:Ncp(2)
for k = 1:1:Ncp(3)
    
    contaN = contaN + 1;
    
    % bars:
    
    if i < Ncp(1)
        
        b1 = b1 + 1;
        BVcon(b1,1) = contaN;
        BVcon(b1,2) = contaN + Ncp(2)*Ncp(3);
        
        if i==1 || i==Ncp(1)-1
            flagEB(b1) = 1 ;
        elseif j==1 || j==Ncp(2)
            flagEB(b1) = 1 ;
        elseif k==1 || k==Ncp(3)
            flagEB(b1) = 1 ;
        end

    end
    
    if j < Ncp(2)
        
        b2 = b2 + 1 ;
        
        r = NB(1) + b2 ;
        
        BVcon(r,1) = contaN ;
        BVcon(r,2) = contaN + Ncp(3);
        
       	if i==1 || i==Ncp(1)
            flagEB(r) = 1 ;
        elseif j==1 || j==Ncp(2)-1
            flagEB(r) = 1 ;
        elseif k==1 || k==Ncp(3)
            flagEB(r) = 1 ;
        end
        
    end
    
    if k < Ncp(3)
        
        b3 = b3 + 1 ;
        
        r = NB(1)+ NB(2) + b3 ;
        
        BVcon(r,1) = contaN ;
        BVcon(r,2) = contaN + 1;
        
    	if i==1 || i==Ncp(1)
            flagEB(r) = 1 ;
        elseif j==1 || j==Ncp(2)
            flagEB(r) = 1 ;
        elseif k==1 || k==Ncp(3)-1
            flagEB(r) = 1 ;
        end
        
    end
    
    
    % diagonals:
    
    if j < Ncp(2) && k < Ncp(3)
        
        d1 = d1 + 1 ;
        BVcon(TNB + d1,1) = contaN ;
        BVcon(TNB + d1,2) = contaN + Ncp(3)+1 ;
        
      	BVcon(TNB + ND(1) + d1,1) = contaN+1 ;
        BVcon(TNB + ND(1) + d1,2) = contaN + Ncp(3) ;
    end
    
    if i < Ncp(1) && k < Ncp(3)
        
        d2 = d2 + 1 ;
        BVcon(TNB + 2*ND(1) + d2,1) = contaN ;
        BVcon(TNB + 2*ND(1) + d2,2) = contaN + Ncp(3)*Ncp(2) + 1 ;
        
      	BVcon(TNB + 2*ND(1) + ND(2) + d2,1) = contaN+1 ;
        BVcon(TNB + 2*ND(1) + ND(2) + d2,2) = contaN + Ncp(3)*Ncp(2) ;
    end
    
    if i < Ncp(1) && j < Ncp(2)
        
        d3 = d3 + 1 ;
        BVcon(TNB + 2*(ND(1)+ND(2)) + d3,1) = contaN ;
        BVcon(TNB + 2*(ND(1)+ND(2)) + d3,2) = contaN + Ncp(2)*Ncp(3) + Ncp(3) ;
        
      	BVcon(TNB + 2*(ND(1)+ND(2)) + ND(3) + d3,2) = contaN + Ncp(3) ;
        BVcon(TNB + 2*(ND(1)+ND(2)) + ND(3) + d3,1) = contaN + Ncp(2)*Ncp(3) ;
    end
end
end
end



% 3.2.- Elements: ordered hierarchically

contaE = 0;
for i = 1:1:Ncp(1)-1
for j = 1:1:Ncp(2)-1
for k = 1:1:Ncp(3)-1
    
    contaE = contaE+1;

    RVcon(contaE,1) = Ncp(2)*Ncp(3)*(i-1) + Ncp(3)*(j-1) + k  ;
    RVcon(contaE,2) = RVcon(contaE,1)  + 1 ;
    
    RVcon(contaE,3) = RVcon(contaE,1) + Ncp(3) ;
    RVcon(contaE,4) = RVcon(contaE,3) + 1 ;
    
   	RVcon(contaE,5) = RVcon(contaE,1) + Ncp(2)*Ncp(3) ;
    RVcon(contaE,6) = RVcon(contaE,5) + 1 ;
    
    RVcon(contaE,7) = RVcon(contaE,5) + Ncp(3) ;
    RVcon(contaE,8) = RVcon(contaE,7) + 1 ;
    
end
end
end




% 4.- SHELL ENVELOPE PLANES:

% 4.1.- Fit to each face a plane by LSM, that passes through its average point

A = zeros(6,1); % the 6 planes constants
B = zeros(6,1);
C = zeros(6,1);
D = zeros(6,1);

for f = 1:1:6
    
    % sample points
    
    sX = gShell(pa).face(f).fSurface.P; 
    m = size(sX,1);
    
    
    % average of sX
    
    sXa = zeros(xdim,1); 
    for di = 1:1:xdim
        sXa(di) = sum(sX(:,di))/m;
    end
    

    % finds normal to fitted plane (LSM)
    
    [Np] = go2519_planeFit (sXa, sX, m, xdim);
    
    
    % allocates plane coefficients:
    
    A(f) = Np(1);
    B(f) = Np(2);
    C(f) = Np(3);
    
    for di = 1:1:xdim
        D(f) = D(f)-Np(di)*sXa(di);
    end

end


% 4.2.- Computes the 8 vertexes of the cuboid (in hierarchical order):

% vertex VS plane ref:
VPcon = [1 2 3 ; 2 3 4 ; 1 3 5 ; 3 4 5 ; 1 2 6 ; 2 4 6 ; 1 5 6 ; 4 5 6 ];

vX = zeros(8,xdim);

for v = 1:1:8
    
    % matrix of coefficients:
    M = zeros(3,3);
    
    % vector LHS:
    b = zeros(xdim,1);
    
    % planes refs:
    r = zeros(3,1);
    r(1) = VPcon(v,1);
    r(2) = VPcon(v,2);
    r(3) = VPcon(v,3);
    
    % assemblies:
    for fil = 1:1:3
        M(fil,1) = A(r(fil)) ; M(fil,2) = B(r(fil)) ; M(fil,3) = C(r(fil)) ;
        b(fil) = -D(r(fil));
    end
    
    % solves and allocates:
    x = M\b;
    for di=1:1:xdim
        vX(v,di) = x(di);
    end
end


% edge VS vtx reference:

EVcon = [1 5 ; 2 6 ; 3 7 ; 4 8 ; ...
         1 3 ; 2 4 ; 5 7 ; 6 8 ; ...
         1 2 ; 3 4 ; 5 6 ; 7 8 ] ;

%{
figure(12)
for e = 1:1:12
    
    r = zeros(2,1);
    
    r(1) = EVcon(e,1);
    r(2) = EVcon(e,2);
    
    plot3( [vX(r(1),1) vX(r(2),1)], [vX(r(1),2) vX(r(2),2)], [vX(r(1),3) vX(r(2),3)], ...
            'k', 'LineWidth' ,2 ); hold on
    
end
axis equal
%}

         
% divides each edge:

Ns = [Ncp(1), Ncp(1), Ncp(1), Ncp(1) , ...
      Ncp(2), Ncp(2), Ncp(2), Ncp(2) , ...
      Ncp(3), Ncp(3), Ncp(3), Ncp(3) ];

eL = zeros(12,2, xdim); % to store nodes in each segment
  
for e = 1:1:12
    
    r = zeros(2,1); % vx refs
    r(1) = EVcon(e,1);
    r(2) = EVcon(e,2);
    
    n = Ns(e); % No nodes in that line
    
    % lenght of each segment:
    
    L = zeros(xdim,1);
    for di = 1:1:xdim
        L(di) = (vX(r(2),di) - vX(r(1),di)) / (n-1);
    end
    
    
    % computes nodes at each edge:
    
    acc = zeros(xdim,1);
    acc(:,1) = vX(r(1),:);
    
    for i = 1:1:n
        
        for di = 1:1:xdim
            eL(e,i,di) = acc(di);
        end
        acc = acc + L;
    end

end
         
% creates grid of nodes in btm / top planes:

% FEcon = [ 1 3 5 7 ; 1 2 9 11 ; 5 6 9 10 ; 2 4 6 8 ; 3 4 10 12 ; 7 8 11 12 ]; % Face VS edge connection 

FEcon2 = [ 1 3 5 7 ; ...
           2 4 6 8  ]; % Face VS edge connection (for faces 1 and 4)
      
fg = zeros(2,1,1,xdim); % 1 and 4 face-grid of nodes
      
for f = 1:1:2
    
    e1 = FEcon2(f,1); % 1st and 2nd edge ref's
    e2 = FEcon2(f,2);
    
    % divides lines between e1 and e2
    
    for i = 1:1:Ncp(1)
        
        % initial and final points of the line
        
        p1 = zeros(xdim,1);
        p2 = zeros(xdim,1);
        
        for di=1:1:xdim
            p1(di,1) = eL(e1,i,di);
            p2(di,1) = eL(e2,i,di);
        end
        
        %{
      	figure(12)
      	scatter3(p1(1),p1(2),p1(3),'r'); hold on
        scatter3(p2(1),p2(2),p2(3),'r'); hold on
       	axis equal
        %}
        
        % divides p1-p2 and allocates:
        
     	L = zeros(xdim,1);
        for di = 1:1:xdim
            L(di) = (p2(di) - p1(di)) / (Ncp(2)-1);
        end

        acc = p1;
        for j = 1:1:Ncp(2)
            
            for di = 1:1:xdim
                fg(f,i,j,di) = acc(di);
            end

            %{
            figure(12)
            scatter3(acc(1),acc(2),acc(3),'r'); hold on
            axis equal
            %}
            
            acc = acc + L;
        end
        
    end
    
end
         
      
% joints nodes from both grids and allocates node number:

% initial lattice nodes coordinates:

eXo = zeros(Ncp(1), Ncp(2), Ncp(3), xdim); % full expanded form
nXo = zeros(nNod, xdim); % node by node matrix form
Xo = zeros(DoF,1); % vector form

ur = zeros(DoF, 1); % increment required for achieving final shape (only applicable to contour points!)

contaNa = 0 ;
contaNb = 0 ;

for i = 1:1:Ncp(1)
for j = 1:1:Ncp(2)
    
    for di = 1:1:xdim
        p1(di) = fg(1,i,j,di);
        p2(di) = fg(2,i,j,di);
    end
    
    L = zeros(xdim,1);
  	for di = 1:1:xdim
        L(di) = (p2(di) - p1(di)) / (Ncp(3)-1);
    end
    
    acc = p1;
    for k = 1:1:Ncp(3)
        
        contaNa = contaNa+xdim;
        contaNb = contaNb+1;
        
        for di = 1:1:xdim
            
          	r = contaNa-xdim+di ;
            
            eXo(i,j,k,di) = acc(di);
            nXo(contaNb,di) = acc(di);
            
            Xo(r,1) = acc(di);
            
            ur(r,1) = XPi(i,j,k,di) - Xo(r,1); % note, it will use only the boundary surface
            
        end
        
        %{
            figure(12)
            scatter3(acc(1),acc(2),acc(3),'r'); hold on
            axis equal
            %}
        
        acc = acc + L;
    end
    
end
end

%{

% represents initial lattice:


figure (12)
for b = 1:1:TNB
    
    r1 = conM(b,1);
    r2 = conM(b,2);
    plot3( [nXo(r1,1) nXo(r2,1)] , [nXo(r1,2) nXo(r2,2)] , [nXo(r1,3) nXo(r2,3)] , 'b' ); hold on
end


for d = TNB+2*(ND(1)+ND(2)):1:TN
    
   	r1 = conM(d,1);
    r2 = conM(d,2);
    plot3( [nXo(r1,1) nXo(r2,1)] , [nXo(r1,2) nXo(r2,2)] , [nXo(r1,3) nXo(r2,3)] , 'r' ); hold on
    
end

scatter3(nXo(:,1), nXo(:,2), nXo(:,3), 6, 'r', 'filled'); hold on
axis equal

%}


% 5.- COMPUTES STIFFNESS MATRIX (sub-K MATRICES):

N = 2 ;             % No. nodes per bar
nr = zeros(N,1);    % nodes references

DX = zeros(xdim,1); % increment of Xcoord's vector

K1 = zeros(DoF,DoF); % bars perimeter (with at least one perimeter node)
K2 = zeros(DoF,DoF); % bars internal (with no perimeter node)
Kd = zeros(DoF,DoF); % diagonals

K = zeros(DoF,DoF);
A = 1 ;

Ebe = ones(3,1)*100;
Ebi = ones(3,1)*100;
Ed = ones(3,1)*1;


% bars and diag's (note E changhes acc/to bar type and direction)

for b = 1 : 1 : TN

    % nodes refs
    nr(1) = BVcon(b,1);
    nr(2) = BVcon(b,2);
    
    % length and orientation:
    DX(:,1) = nXo(nr(2),:) - nXo(nr(1),:) ;
    L = norm(DX);
    u = DX/L;
    
    % selects properties acc/to bar direction
    if 0 < b && b <= NB(1)
        E = Ebe(1);
    elseif NB(1) < b && b <= NB(1)+NB(2)
        E = Ebe(2);
    elseif NB(1)+NB(2) < b && b <= TNB
        E = Ebe(3);
	elseif TNB < b && b <= TNB + 2*ND(1)
        E = Ed(1);
 	elseif TNB+ 2*ND(1) < b && b <= TNB + 2*ND(1)+ 2*ND(2)
        E = Ed(2);
   	elseif TNB + 2*ND(1)+ 2*ND(2) < b && b <= TN
        E = Ed(3);
    end
    
    [kL] = ms3201_KGrod3D (E,A,L, u);
    
    % allocates kb to K
    [K] = a3211_insertinK (K, kL, nr, N, xdim);
    
end


% 6.- COMPUTES Kaa, Kab AND ub:

% 6.1.- Nodes references for 'ub'(contour-known) and 'ua' (internals-unknown):

nk = 2 * ( Ncp(1)*Ncp(2) + Ncp(1)*(Ncp(3)-2) + (Ncp(2)-2)*(Ncp(3)-2) );
nu = nNod-nk;

% stores addresses of known and un-known nodes
dkr = zeros (nk , 1); 
dur = zeros (nu , 1);

contaR = 0;
contaK = 0;
contaU = 0;

for i = 1:1:Ncp(1)
for j = 1:1:Ncp(2)
for k = 1:1:Ncp(3)
    
    contaR = contaR + 1 ;
    
    if i == 1 || i == Ncp(1)

        contaK = contaK + 1 ;
        dkr(contaK) = contaR ;
        
    elseif j == 1 || j == Ncp(2)
        
        contaK = contaK + 1 ;
        dkr(contaK) = contaR ;
        
    elseif k == 1 || k == Ncp(3)
        
        contaK = contaK + 1 ;
        dkr(contaK) = contaR ;
        
    else
        contaU = contaU + 1 ;
        dur(contaU) = contaR ;
        
    end
end
end
end



% 6.2.- Prepares unknown-'a' and known-'b' vectors and matrices:

ub = a3212_extractU (ur, nk, dkr, xdim);

mnr = [dur dur] ;
Kaa = a3213_extractK (K, [nu,nu]', mnr, xdim);

% nodes ref's matrix
if nk == nu
    
    mnr = [dur dkr];
    
elseif nk > nu
    
    mnr = [ [dur; zeros(nk-nu,1)] dkr];
    
elseif nk < nu
    
    mnr = [ dur [dkr; zeros(nu-nk,1)]];
end

Kab = a3213_extractK (K, [nu,nk]', mnr, xdim);


% 7.- SOLVES DISPLACEMENTS ua (for internals):

fa = -Kab*ub;
ua = Kaa\fa ;


% extends with prescribed displacements:

u = zeros(DoF,1);

contaK = 1 ;
contaU = 1 ;

for i = 1:1:nNod
    
    % prescribed displacement:
    if contaK <= nk
    
        if i==dkr(contaK)
            for di = 1:1:xdim
                
                r = i*xdim-(di-1);
                u(r) = ur(r);
                
            end
            contaK = contaK + 1 ;
        end
        
    end
    
    % computed displacement:
    
    if contaU <= nu
    
        if i==dur(contaU)
            for di = 1:1:xdim
                
                r1 = i*xdim-(di-1);
                r2 = contaU*xdim-(di-1);
                u(r1) = ua(r2);
                
            end
            contaU = contaU + 1 ;
        end
        
    end 
    
end

 
 

% 8.- ALLOCATES NEW POSITIONS TO internal CP'S BY ADDING ua to Xo:
% Allocates to XP:

XP = zeros(nNod,xdim);

for i = 1:1:nNod
    for di = 1:1:xdim
        
       	r = i*xdim - (xdim) + di;
        
        XP(i,di) = Xo(r)+u(r);
                
    end
end




end

