function [XP] = sf3156_fillPlane (XPi)

% RETURNS:

% XP(n,m,2): coord's of control points defining a region bounded by four
% co-planar NURBS curves

% INPUTS:
% XPi(n,m,2): initial control points coord's. Include only perimetral cp's
% coords, the other are '0'.
% n,m: No. cp's in each direction

xdim = 2 ;
XP = XPi;
n = size(XP,1);
m = size(XP,2);


% 1.- INITIATION OF INTERNAL POINTS: interpolates using previous alingment
% distances in 4 directions, and works out the average

% 1.1.- Upwards:
XPu = XP;
for b = 2:1:m
    
    L = 0 ; % total length between cp's
    l = zeros(n-1,1); % length in each stretch
    
    % coord's of first and second points of stretch
    f = zeros(xdim,1);
    s = zeros(xdim,1);
    
    for a=1:1:n-1 
        for di=1:1:xdim
        f(di,1) = XPu(a,  b-1,di);
        s(di,1) = XPu(a+1,b-1,di);
        end
        l(a) = norm(s-f);
        L = L + l(a) ;
    end

    % computes increments (interpolating from extreme 
    % increments) and sum to prev. points:
    
    D1 = zeros(xdim,1); % increment at first position
    Dn = zeros(xdim,1); % incremetn at last position
    Dab = zeros(xdim,1); % inperpolated increment at location 'a'
    
    for di=1:1:xdim
        D1(di) = XP(1,b,di) - XP(1,b-1,di);
        Dn(di) = XP(n,b,di) - XP(n,b-1,di);
    end
    
    for a = 2:1:n-1
        
        % ponderation factor:
        f = 0; 
        for i=a:1:n-1
           f = f + l(i)/L; 
        end
        
        % interpolated increment:
        Dab = f*D1 + (1-f)*Dn;
        
        % coordinate of a,b point:
        for di=1:1:xdim
            XPu(a,b,di) = XPu(a,b-1,di) + Dab(di);
        end
    end
end




% 2.- CORRECTS:
% move last point of each alineation to match the original position and
% accomodates the others proportionally.

for a = 2:1:n-1
    
    Dt = zeros(xdim,1); % total last point increment between original and estimated
    
    for di = 1:1:xdim
        Dt(di) = XPu(a,m,di) - XP(a,m,di);
    end
    
    % total L and streches l:
    [L,l,~] = sf3157_stretchesL (m-1, XPu, 2, a);
    
    % amends intermediate points positions:
    for b = 2:1:m-1
        
        % factor:
        f = 0 ;
        for i=1:1:b-1
            f = f + l(i)/L;
        end
        
        % coordinates:
        for di=1:1:xdim   
            XPu(a,b,di) = XPu(a,b,di) - Dt(di)*f;
        end
    end
    
    % amends last point position:
    for di = 1:1:xdim
     	XPu(a,m,di)= XP(a,m,di);
    end
end

for i=2:1:n
for j=2:1:m
    for di=1:1:xdim
    XP(i,j,di) = XPu(i,j,di);
    end
end
end


% 3.- ACHIEVES SAME DISTANCE IN xi AND ita DIRECTIONS ALTERNATIVELLY:

for NI = 1:1:500

    for a = 2:1:n-1

        % total L, streches l and average l:
        [L, l, avL] = sf3157_stretchesL (m-1, XP, 2, a);

        % stretches slopes, vertical line indicator, increment of coordin. in each stretch:
        [s, vert, D] = sf3158_stretchesS (m-1, XP, 2, a);

        for b = 1:1:m-2
           for di=1:1:xdim
                XP(a,b+1,di) = XP(a,b,di) + avL/l(b) * D(b,di);
           end
        end
    end

    for b = 2:1:m-1

        % total L, streches l and average l:
        [L, l, avL] = sf3157_stretchesL (n-1, XP, 1, b);

        % stretches slopes, vertical line indicator, increment of coordin. in each stretch:
        [s, vert, D] = sf3158_stretchesS (m-1, XP, 1, b);

        for a = 1:1:m-2
           for di=1:1:xdim
                XP(a+1,b,di) = XP(a,b,di) + avL/l(a) * D(a,di);
           end
        end
    end
    
end




%%%%%%%%%%%%%%%%%% represent

% dots:

xc = zeros(n*m,2);

conta = 0 ;
for i=1:1:n
for j=1:1:m
   
    conta = conta + 1 ;
    
    xc(conta,1) = XPi(i,j,1);
    xc(conta,2) = XPi(i,j,2);
    
end 
end 

fR = abs(round(XPi(1,1,1),0))+1;
figure(fR);

scatter(xc(:,1), xc(:,2), 'r'); hold on

% lines 1:

for i=1:1:n
    
    x =  zeros(m,1);
    y =  zeros(m,1);
    for j=1:1:m
        
        x(j) = XPi(i,j,1);
        y(j) = XPi(i,j,2);
    end
    figure(fR);
    %plot(x,y,'--r'); hold on
end



% lines 2:

for j=1:1:m
    
    x =  zeros(n,1);
    y =  zeros(n,1);
    for i=1:1:n
        
        x(i) = XPi(i,j,1);
        y(i) = XPi(i,j,2);
    end
    figure(fR);
    %plot(x,y,'--r'); hold on
end


axis equal






end

