function [eLs] = rs3343_EstimatesL ...
    (ua,ub, NGP, ULims, NE,NEt, ULimEr, ULimEpr, fpeR,fceR, conM,conMp,conMc, grEp, vDirp, nUcoor, dSu,dSv, xdim, nd)

% RETURNS:
% eLs: estimated length of segment in NURBS surface physical space between ua and ub, that forms tita
% angle w.r.t. horizontal in parameter space. Note that if ua or ub lies
% outside surface param domain still estimates it

uar = round(ua,nd);
ubr = round(ub,nd);


% 1.- EVALUATES IF ua & ub ARE INSIDE THE DOMAIN:

IN = zeros(2,1); % change to 1 if IN

if  (ULims(1,1) <= ua(1) && ua(1) <= ULims(1,2)) && ...
  	(ULims(2,1) <= ua(2) && ua(2) <= ULims(2,2)) 
    IN(1) = 1 ;
end

if  (ULims(1,1) <= ub(1) && ub(1) <= ULims(1,2)) && ...
  	(ULims(2,1) <= ub(2) && ub(2) <= ULims(2,2)) 
    IN(2) = 1 ;
end



% 2.- CASE BOTH POINTS ARE IN:

if sum(IN)==2
    
    [eLs] = rs3344_LsegmTitaWithGP ...
    (uar,ubr, NGP, ULims, NE,NEt, ULimEr, ULimEpr, fpeR,fceR, conM,conMp,conMc, grEp,vDirp, nUcoor, dSu, dSv, xdim);
    return
end


% 3.- CASE BOTH POINTS ARE OUT:

if sum(IN)==0

    [eLs] = rs3345_LsegmTitaWithdCu ...
  	(uar,ubr, ULims, NE, NEt, ULimEr, ULimEpr,fpeR,fceR,conM,conMp,conMc, grEp, vDirp, nUcoor, dSu,dSv, xdim, nd);


    return
end


% 4.- CASE ua or ub IS OUT:

if sum(IN)==1
    
    % 4.1.- Finds intersection with parameter edge:
    
    rc = [1 1 1 2 ; 2 1 2 2 ; 1 1 2 1 ; 1 2 2 2];
    
    for lado = 1:1:4

        nua = [ULims(1,rc(lado,1)) ULims(2,rc(lado,2))]; % nodes coordinates
        nub = [ULims(1,rc(lado,3)) ULims(2,rc(lado,4))];

        [IP,iF,pF] = go2526_TwoLinIntersec (ua,ub, nua, nub, 'closed', nd);

        if pF == 1
            [IP, iF] = go2517_ParalelSegmInters2D (ua, ub, nua, nub, nd);
        end

        if iF == 1
            u2 = round(IP,nd) ;
            break
        end  
    end
    
    % 4.2.- Measures both distances separately and sums up:
    
    if IN(1)==1
        u1 = uar ;
        u3 = ubr ;
    elseif IN(2)==1
        u1 = ubr ;
        u3 = uar ;
    end
    
 	[eLs1] = rs3344_LsegmTitaWithGP ...
    (u1,u2, NGP, ULims, NE,NEt, ULimEr, ULimEpr, fpeR,fceR, conM,conMp,conMc, grEp,vDirp, nUcoor, dSu,dSv, xdim);

    [eLs2] = rs3345_LsegmTitaWithdCu ...
  	(u2,u3, ULims, NE, NEt, ULimEr, ULimEpr,fpeR,fceR,conM,conMp,conMc, grEp,vDirp, nUcoor, dSu,dSv, xdim, nd); 

    eLs = eLs1 + eLs2;
end

end

