function [u] = rs3350_DuForDx ...
        (R,tita, ua, NE, NEt, ULimEr,ULimEpr, fpeR,fceR, conM,conMp,conMc, grEp, vDirp, nUcoorr, dSu,dSv, Dumax, ULims, xdim, nd)


% RETURNS:
% u(2): Estimated Ucoor for achieving distance R in physical space at tita angle in parameter space
% rF: rejected flag: 0 / 1  for  non-rejected / rejected, at the moment always is non-refected (rF = 0)
% tF: track back flag = 1 if the estimation lied outsude limits and has been pulled back (otherwise = 0)

% refines mesh if error is larger than tolerance

% INPUTS:
% R: physical space distance to achieve from position in ua
% tita: angle (rads) w.r.t. horizontal in parameter space used to measure R
% ua(2): param. space starting position 
% Du(2): param. space increment in tita direction to estimate dS field
% NE,ULimE,conM: param. mesh discretization variables
% dSu, dSv(nodes, xdim): derivatives at mesh nodes
% ust(2): step of P-elems
% Dumax: maximum increment of u (assumed half patch diagonal)
% nd: No. digits for rounding
% tolR: tolerance in percentage to assume distance is correct

% Note: we refer as P-elem a elements in the parameter space of the
% surface, used for discretizing derivatives fields


% 1.- PRELIMINARY OPERATIONS:

u  = zeros(1,2);
NGP = 5 ;

uar = round(ua,nd);

% limits tita to be between 0 - 360 (but in radians)

if tita > 6.283185
    tita = tita - 6.283185;
end

if tita < 0
    tita = 6.283185 + tita;
end


% 2.- ESTIMATES POSITION OF SECOND POINT:

% 2.1.- First trial:

Du = norm(Dumax)/10;
ub = [ua(1)+Du*cos(tita) , ua(2)+Du*sin(tita)];

% 2.2.- Estimates X-length from ua to ub, then compares with R to   
%       adjust Du (or ub)

[Ls] = rs3343_EstimatesL ...
(ua,ub, NGP, ULims, NE,NEt, ULimEr, ULimEpr, fpeR,fceR, conM,conMp,conMc, grEp, vDirp, nUcoorr, dSu,dSv, xdim, nd);

Du = Du * R/Ls;

ub = [ua(1)+Du*cos(tita) , ua(2)+Du*sin(tita)];
ubr = round(ub,nd);


% 3.- DETERMINES:

% WHICH P-elems THE LOCATIONS ua & ub BELONGS TO
% NO. ELEMENTS (u CAN LIE IN MULTIPLE IF IT LIES IN THE BORDER)
% SAME ELEMENT FLAG (eF = 1 if ua and ub lie in the same element) 

[nEa, eRa, nEb, eRb, seF, nEs, eRs] = rs3357_elemsInfo ...
    (ULims, NE, NEt, grEp, ULimEr, ULimEpr, fpeR, uar, ubr);



% 4.- ASSEMBLIES 'SEGMENT ARRAYS' WITH INFO OF KEY u LOCATIONS, DERIVATIVES AND ELEMENTS:

% NU:        No. u key locations (include start, intersections with elems, and end point)
% uSg(NU,2): coordinates of each of NU
% mSg(NU):   deriv. module of each uSg
% ieR(NS): elements where segments pass through.
% accU(NS), accX(NS): acc. length at the end of each segment.

% NS = NU - 1

if seF ==1

    % 4.1.- For same element => computes and allocates directly:

    mSa = rs3366_BiLmdCtita (uar, tita, nEs, eRs, fpeR, fceR, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, xdim);
    mSb = rs3366_BiLmdCtita (ubr, tita, nEs, eRs, fpeR, fceR, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, xdim);
    
    NU = 2 ;
    uSg = [ua;ub];
    mSg = [mSa ; mSb];
    
    NieR = nEs;
    ieR = zeros(1,nEs);
    ieR(1,:) = eRs(:);
    
  
else
    
    % 4.2.- For diff. elements => uses specific function.
    %       Determines param. coordinates and derivatives modules at each intersection:

    [NU, uSg, mSg, NieR, ieR] = rs3359_intersecDuEls ...
    (tita, ua, ub, nEa, nEb, eRa, eRb, ULims, NE, NEt, ULimEr, ULimEpr,fpeR,fceR,conM,conMp,conMc, grEp, vDirp,nUcoorr, dSu,dSv, nd);

end




% 4.3.- acc. lengths at the end of each segment:

NS = NU-1;
accX = zeros(NS,1) ;
accU = zeros(NS,1) ;

for sg = 1:1:NS

    u1 = [uSg(sg,1)   uSg(sg,2)  ];
    u2 = [uSg(sg+1,1) uSg(sg+1,2)];
    
    Lx = rs3358_distXinSgU (u1, u2, mSg(sg), mSg(sg+1));
    Du = norm(u1-u2);
    
    if sg==1
        accX(sg) = Lx;
        accU(sg) = Du;
    else
     	accX(sg) = accX(sg-1) + Lx;
        accU(sg) = accU(sg-1) + Du;
    end
end


% 5.- FINDS SEGMENT THAT CONTAINS R IN ITS accX. CALCS LINE COEFF'S. 
%     stores segent in 'sgR', in r1/r2 stores correspondant u nodes references

% 5.1.- Finds segment:

for sgR = 1:1:NS
    if R <= accX(sgR)
        break
    end 
end

r1 = sgR;
r2 = sgR+1;


% 5.2.- Extreme u values of the segment and acc x-distance up to end of prev. segment:

u1 = [ uSg(r1,1)   uSg(r1,2) ];
u2 = [ uSg(r2,1)   uSg(r2,2) ];

if sgR == 1
    xaccp = 0;
else
    xaccp = accX(sgR-1);
end


% 5.3.- 'Current elements' ref. (where final u is expected to lie in):

ceR = zeros(1,NieR(sgR));
NceR = NieR(sgR) ;

for e = 1:1:NceR
    ceR(e) = ieR(sgR,e);
end



% 6.- IF IS NOT THE LATEST ELEMENT ESTIMATES u WITHIN THAT SEGMENT:

if sgR < NS
    
	Reff = (R - xaccp);
	[u] = rs3365_estimatesUinSg (u1, u2, tita, mSg(r1), mSg(r2), Reff, Dumax);
    u = round(u,nd);
	return
    
end


%     ONLY CARRIES ON IN CASE IS THE LATEST SEGMENT:

% 7.- CHECKS MAXIMUM Rmax ACHIEVABLE IN THE CURRENT ELEMNT 'ceR' > R
%     if Rmax < R searches next element
%     (only happens if we are in the latest segment: sgR = NS)


if sgR == NS
    
    Ttol = 5 * 180 / 3.15159 ; % tolerance for perim. & corner elements to detemine Rmax=infinite direction
    
    % 6.1.- Checks if extension of segment goes to infinite direction:
    
    [riF] = rs3362_RmaxInf (u2, tita, ULims, nd, Ttol);

    
    % if is direction with infinte Rmax => set Rmax as +1 greater than R
    if riF == 1
        Rmax = R + 1 ;
    end
    
    % if not => find next element intersection with segment and computes
    % Rmax acounting for this next element
    if riF == 0

      	% - intesect'n btw'n last segment projection and current elem.(ceR) edge: u3
        % - intesect'n btw'n last segment projection and next    elem.(neR) edge: u4
        % - next element 'neR'
        
        [u3, u4, NneR, neR, riF] = rs3363_intersecNextEl ...
            (u2, tita, NceR, ceR, fpeR,fceR, ULims, NE, NEt, grEp, vDirp, ULimEr,ULimEpr, conM,conMp,conMc, nUcoorr, nd, Ttol);
            
        if riF == 1
            Rmax = R + 1 ;
        else
            
            % 6.2.- Computes max. achievable distance (Rmax): from uSg(1) to u3:

            % computes derivative at u3:

            mSgu3 = rs3366_BiLmdCtita (u3, tita, NceR, ceR, fpeR, fceR, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, xdim);

            % distance from uSg(NU) to u3: D3x
            [D3x] = rs3358_distXinSgU (u2, u3, mSg(NU), mSgu3);

            % add to accX(NS):
            Rmax = accX(NS) + D3x;
            
        end
    end


    % 6.2.- If max. achievable distance (Rmax) is more than target R => just estimates u:
    
    if R < Rmax
        
        Reff = (R - xaccp);
        [u] = rs3365_estimatesUinSg (u1, u2, tita, mSg(r1), mSg(r2), Reff, Dumax);
        u = round(u,nd);
        return
        
    end
    
    
    % 6.3.- If not => jumps to next element until Rmax > R
    
    xaccp = accX(NS);
    
    while Rmax < R
        
        % increases xaccp to begining of next element:
        
        mSgu2 = rs3366_BiLmdCtita (u2, tita, NceR, ceR, fpeR, fceR, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, xdim);
        mSgu3 = rs3366_BiLmdCtita (u3, tita, NceR, ceR, fpeR, fceR, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, xdim);
        
        [Lx] = rs3358_distXinSgU (u2, u3, mSgu2, mSgu3);
        xaccp = xaccp + Lx ;
        
        
        % updates Rmax:

        mSgu4 = rs3366_BiLmdCtita (u4, tita, NneR, neR, fpeR, fceR, conM, conMp, conMc, nUcoorr, vDirp, dSu, dSv, xdim);
        
        [D4x]  = rs3358_distXinSgU (u3, u4, mSgu3, mSgu4);
        Rmax   = xaccp + D4x;

        
        % updates loop vbles. (only in case of not reaching R with Rmax)
        
        if Rmax < R
            
            NceR = NneR ;
            ceR  = neR ;
            u2 = u3 ;
            [u3, u4, NneR, neR, riF] = rs3363_intersecNextEl ...
                (u2, tita, NceR, ceR, fpeR,fceR, ULims, NE, NEt, grEp, vDirp, ULimEr,ULimEpr, conM,conMp,conMc, nUcoorr, nd, Ttol);
                
            if riF == 1
                Rmax = R + 1 ;
            end
        end
        
    end
    
    
    % computes u using the proper segment:
    
    Reff = (R - xaccp);
	[u] = rs3365_estimatesUinSg (u3, u4, tita, mSgu3, mSgu4, Reff, Dumax);
    u = round(u,nd);
    
end

         

end

