function [dSui, dSvi] = rs3354_BiLinInterpR (u, eR, fpeR, fceR, conM, conMp, conMc, vDirp, nUcoor, dSu, dSv, xdim)

% RETURNS:
% dSui, dSvi: interpolated vectors using rectangles discretization, either
% for 'internal', 'perimeter' or 'corner' elements

% INPUTS:
% u(2): location where estimate the derivatives
% eR:   element reference
% fpeR: first perimeter element reference
% fceR: firs corner element reference
% conM: connectivity matrices for internal, perimeter and corner element
% vDirp: varaible direction in case of perimeter elements

if eR < fpeR

% 1.- CASE 1: element is internal:
    

    % finds element parameter limits:
    
    nR1 = conM(eR,1);
    u1 = nUcoor(nR1,1);     v1 = nUcoor(nR1,2); 
    
 	nR2 = conM(eR,4);
    u2 = nUcoor(nR2,1);     v2 = nUcoor(nR2,2); 
    
    
    % interpolates derivatives:
    [dSui, dSvi] = rs3355_BiLinInterpInE (u1, u2, v1, v2, u, conM,eR, dSu, dSv, xdim);
    
    
elseif fpeR <=eR && eR<fceR
    
% 2.- CASE 2: element is perimeter:

    eReff = eR - (fpeR-1);

    % find element param. limts (in direction that varies):
    
    vd = vDirp(eReff,1); % vble direction
    
    nR1 = conMp(eReff,1);
    u1 = nUcoor(nR1,vd);     
    
 	nR2 = conMp(eReff,2);
    u2 = nUcoor(nR2,vd);     

    
    % interpolates derivatives:
    
    [dSui, dSvi] = rs3356_BiLinInterpPeE (u1,u2, u(vd), conMp, eReff, dSu, dSv, xdim);
    
    
elseif fceR<=eR
    
    
% 3.- CASE 3: element is at corner-perimeter:

    % derivatives are just copied from correspondant node:
    
    dSui = zeros(xdim,1);
    dSvi = zeros(xdim,1);
    
    eReff = eR - (fceR-1);
    nR = conMc(eReff,1);
    
    for di=1:1:xdim
        dSui(di) = dSu(nR,di) ;
        dSvi(di) = dSv(nR,di) ;
    end
    
end



end

