function [NU, uSg, mSg, NieR, ieR] = rs3359_intersecDuEls ...
         (tita, ua,ub, nEa, nEb, eRa, eRb, ULims, NE, NEt, ULimEr, ULimEpr,fpeR,fceR,conM,conMp,conMc, grEp, vDirp,nUcoor, dSu, dSv, nd)

% RETURNS:
% uSg(NU,2): points coordinates of piecewise segment. Ordered ascending from ua to ub
% mSg(NU,1): directional derivative module at each uSg
% ieR(NU-1,1); intersected elements references

% Note: it is assumed that ua and ub lie in different elements

% INPUTS:

udim = 2 ;
xdim = 3 ;

% 1.-ALLOCATES FIRST ELEM. REF. (where ua lies in) AND STORES IN 'feR':

NieRt = zeros(1,1);
ieRt  = zeros(1,1); % temporary arrays
NieRt(1,1) = nEa;

NfeR = 0 ;
feR = zeros(1,1);

for e = 1:1:nEa
    
    % 1.1.- Intersections between first elements and segment ua-ub:
    
    ueR = eRa(e);
    
    ieRt(1,e) = ueR;
 	[iE, NiE] = rs3360_intersecLadosElem ...
                (ua, ub, ueR, conM,conMp,conMc, nUcoor, fpeR, fceR, grEp, vDirp, nd);
                 
            
    % 1.2.- conditions to be first elements: 
    
    % - If is one single element in eRa array
    % - If more than single elem., 
    %       - if NiE (No. inters. points with the elem.) = 2
    %       - if NiE = 1 and is different from ua
    
    if nEa==1
        
        NfeR = 1 ;
        feR(NfeR) = ueR;
        
    elseif NiE == 2
        
        NfeR = NfeR+1;
        feR(1,NfeR) = ueR;
        
    elseif NiE == 1 && (round(ua(1),nd)~=round(iE(1,1),nd) || round(ua(2),nd)~=round(iE(1,2),nd))
        
        NfeR = NfeR+1;
        feR(1,NfeR) = ueR;
        
    end
end




% Note: feR is to have maximum 2 No. elements (that happens when segment is
% coincident with both elements party edge). OJO! they do not have to have
% the same edge length


% 2.- COMPUTES INTERSECTIONS WITH ELEMS. CONTOURS:
% stores in uSg:

contaU = 0;
uSg = zeros(1,udim);

u1 = ua ;
u2 = ub ;

NueR = NfeR;
ueR = feR ; % ueR are elements to find intersections with segment ua-ub

contaR = 1 ;

% Loops while does not find the end:
eF = 0 ; % end searching flag, turn to 1 if the unique inters.point is the same as u1 in the element


while eF == 0

    % 2.0.- In case of 2 elements does tow checks to potentially eliminate one of them:


    if NueR > 1
        
        % direction of segment is orthogonal? (if so, stores in sd direction to measure):
        
        sd = 0;
      	if (round(tita,3)==0 || round(tita,3)==3.142)  || round(tita,3)==6.283
            sd = 1 ;
        elseif round(tita,3)==1.571 || round(tita,3)==4.712
            sd = 2 ;
        end
        
        % - CASE A: it is a orthogonal segment and goes between 2 elements
        % with different lengths, or ,  Checks that ueR elements have same length, if not takes the shortest

        if sd > 0
        
            % measures shared edge of the elements
            DL = zeros(NueR,1);
            for e = 1:1:NueR

               rf = ueR(e); 

               if rf < fpeR

                   DL(e) = nUcoor(conM(rf,4),sd)-nUcoor(conM(rf,1),sd);

               elseif fpeR <= rf && rf < fceR

                   rfeff = rf - (fpeR-1);

                   if vDirp(rfeff) == sd
                       DL(e) = nUcoor(conMp(rfeff,2),sd)-nUcoor(conMp(rfeff,1),sd);
                   else
                       DL(e) = 1 ;
                   end

               elseif fceR <= rf

                   DL(e) = 1;

               end

            end


            % selects elements in case lengths are different:

            if round(DL(1),6) ~= round(DL(2),6)

                minDL = min(DL);

                NueR = 1 ;
                ueRt = ueR ;
                ueR = zeros(1,1);
                ueR(1,1) = ueRt(1);

                if DL(2) == minDL
                    ueR(1,1) = ueRt(2);
                end
            end
        end
        
      	% - CASE B: checks if u1 is right at the corner and side of both
      	% elements (corner for smaller and side for larger). In that case
      	% selects the larger element (if u draw the situation will realize
      	% it is the only posibility):

        if sd == 0
        
            % finds which element corners are not coincident with u1:
            
            corF = zeros(NueR,1); % turns to 1 if u1 lies at one corner
            
            for e = 1:1:NueR
                
                rf = ueR(e);
                
                if rf < fpeR
                    
                    for nod=1:1:4
                    if nUcoor(conM(rf,nod),1) == u1(1) && nUcoor(conM(rf,nod),2) == u1(2)
                            
                        corF(e,1) = 1 ;
                        break
                        
                    end
                    end
                    
                elseif rf < fceR
                    
                    rfeff = rf - (fpeR-1);
                    for nod=1:1:2
                    if nUcoor(conMp(rfeff,nod),1) == u1(1) && nUcoor(conMp(rfeff,nod),2) == u1(2)
                            
                        corF(e,1) = 1 ;
                        break
                        
                    end 
                    end
                    
                end
                
            end
            
            % discriminates the elemnt that is intersected at the corner:
            
            if 0 < sum (corF) && sum (corF) < NueR
            
                NueRt = 0 ;
                ueRt = zeros(1,1);
                
                for e = 1:1:NueR
             	if corF(e)==0 
                    NueRt = NueRt + 1 ;
                 	ueRt(NueRt,1) = ueR(e);
             	end
                end
                
                NueR = NueRt ;
                ueR =ueRt;
                
            end
                
        end
        
    end
    


    % 2.1.- Find intersections with ueR element edges:

    NiE = 0 ;
    iE = zeros(1,2);
       
    for e = 1:1:NueR
        
        [iEe, NiEe] = rs3360_intersecLadosElem ...
      	(u1, u2, ueR(e), conM,conMp,conMc, nUcoor, fpeR, fceR, grEp, vDirp, nd);
        
        for i = 1:1:NiEe
            iE(NiE+i,1) = iEe(i,1);
            iE(NiE+i,2) = iEe(i,2);
        end
        NiE = NiE + NiEe ;

    end
    
    [iE,~] = GEN_16_RemoveRepeatedRows (iE, 2, 10^(-nd));
    NiE = size(iE,1);
    

    % 2.2.- Select insersection point: 
    % - in case of 1 inters. point => changes 'end flag' to 1 if the unique intersection point is u1
    % - in case of 2 inters. points => selects the one different from ua

    if NiE == 1

        ui = [iE(1,1) iE(1,2)];
        
        if round(ui(1),nd)== round(u1(1),nd) && round(ui(2),nd)== round(u1(2),nd) % ends intersection search
            eF = 1;
        end

    elseif NiE > 1

        for i=1:1:NiE
  
            if round(iE(i,1),nd) ~= round(u1(1),nd) || round(iE(i,2),nd) ~= round(u1(2),nd)
                ui = [iE(i,1) iE(i,2)];
                break
            end
        end
    end
    

    % (next 3 tasks are done only if is not the end)

    if eF == 0
        
        % 2.3.- Stores intersection point:
        
        contaU = contaU + 1 ;
        uSg(contaU,1) = ui(1) ;
        uSg(contaU,2) = ui(2) ;
 

        % 2.4.- Finds other element where ui belongs to (different from  
        % previous ueR) and updates ueR.
    
        uir = round(ui,nd);

        
        [nE,eR] = rs3346_FindRele ...
      	(ULims, NE, NEt, grEp, ULimEr, ULimEpr, fpeR, uir);
    
        NueRt = 0;
        ueRt = zeros(1,1);
        
        for i = 1:1:nE
            
            addF = 1 ;
            for j = 1:1:NueR
                if eR(i) == ueR(j)
                    addF = 0 ;
                    break
                end
            end
            
            if addF==1
                NueRt = NueRt+1;
                ueRt(1,NueRt) = eR(i);
            end  
        end
        
        NueR = NueRt ;
        ueR  = ueRt ;
        
        % 2.5.- Stores intersected elements ref's:
    
        contaR = contaR + 1 ;
        NieRt(contaR,1) = nE ;

        for e = 1:1:nE
            ieRt(contaR,e) = eR(e);
        end
    

        % 2.6.- Updates u1 and u2 (note u2 is always last segment point 'ub'):
    
        u1 = ui ;
        u2 = ub ;
        
    end
  
end


% 3.- ADD INITIAL & FINAL POINTS:
% checks that they are different

if round(ua(1),4)~=round(uSg(1,1),4) || round(ua(2),4)~=round(uSg(1,2),4)
    uSg = [ ua ; uSg];
end

NUt = size(uSg,1);

if round(ub(1),4)~=round(uSg(NUt,1),4) || round(ub(2),4)~=round(uSg(NUt,2),4)
    uSg = [ uSg ; ub];
end



% 4.- COMPUTES ELEMENTS:

% uses temporary arrays, 
% select only if elements are repeated in consecutive rows

% 5.1.- Add last row of elements for latest point:

contaR = contaR+1;

NieRt(contaR) = nEb;

for e = 1:1:nEb
    ieRt(contaR,e) = eRb(e);
end

% 4.2.- Detects rep'd elems btw'n consecutive rows of ieRt and stores in ieR:

nme = max(NieRt);

NieR = zeros(contaR-1,1);
ieR  = zeros(contaR-1,1);

for r = 1:1:contaR-1
    
    contaE = 0 ;
    
    for e0 = 1:1:nme
    for e1 = 1:1:nme    
        if (ieRt(r+0,e0) == ieRt(r+1,e1)) && ieRt(r+0,e0)~=0
            contaE = contaE+1;
            ieR(r,contaE) = ieRt(r+0,e0);   
        end
    end
    end
    
    NieR(r) = contaE ;
end


% 5.- COMPUTES ARRAY mSg:

NU =  size(uSg,1);
mSg = zeros(NU,1);

for s = 1:1:NU
    
    ut = [ uSg(s,1) uSg(s,2) ];

    %{
    [dSsg] = rs3352_dSdir ...
    (ut, tita, ULims, NE, NEt, ULimEr, ULimEpr, fpeR,fceR, conM,conMp,conMc, grEp,vDirp, nUcoor, dSu,dSv, xdim);
    
    mSg(s) = norm(dSsg); 
    %}
    
	[NeR, eR] = rs3346_FindRele (ULims, NE, NEt, grEp, ULimEr, ULimEpr, fpeR, ut);
    
    %{
    NeR = 1 ;
    if s==NU
        eR = ieR(NU-1,1);
    else
        eR = ieR(s,1);
    end
    %}
 
    mSg(s) = rs3366_BiLmdCtita (ut, tita, NeR, eR, fpeR, fceR, conM, conMp, conMc, nUcoor,vDirp, dSu, dSv, xdim);
    

    
end



   


end

