function [DT, cf] = de3501_DisTp (deR, Tnod, Rthr, Ep, connT, THC)

% RETURNS:
% DT: type of distorsion in case exists, if not it is "0"

% "NE": needle
% "WE": wedge
% "SP": splindle
% "CA": cap
% "SL": sliver
% "UN": undefined

% cf: correction features: struct with members that depend on the type of distortion.


% INPUTS:
% Rthr: regulatity thresold (0 to 1), 1 is regular tet. The lower R the worse.
%       If R < Rthr it is considered a distorted one
% Ep:   percentage of bave to consider small edge

% pt1362_pTet (Tnod, 3, 500, 'k', 1, 1, 0)

DT = "0";
cf = 0 ;
pudim = 3 ;


% 1.- REGULARITY, EDGES LENGTHS AND AVERAGE EDGE LENGTH:

[R, bave, b] = th1319_tetR (Tnod);

if R > Rthr
    return
end


% 2.- No. SMALL EDGES:

Nse = 0 ;
for i = 1:1:6
    if b(i) < bave * Ep/100
        Nse = Nse + 1 ;
    end
end


% 3.- ESTABLISHES THE TYPE:

if Nse == 3     % NEedle
    
    
    DT = "NE";
    
    
elseif Nse == 1 % WEdge or SPindle
    
    
    DT = "WE";
    % DT = "SP";
    
    
elseif Nse == 0 % CAp or SLiver
    

 	% 3.3 a.- Detection of lines candidate to remove:
    % These are a couple opposite lines that have the least distance between them

    olr = [ 1 2 3 4 ; 2 3 1 4 ; 1 3 2 4]; % opposite lines refs

    A = zeros(pudim,1);
    B = zeros(pudim,1);
    x = zeros(pudim,1);

    dmin = 9999;
    ps = 0 ;

    for p = 1:1:3

        % nodes refs:

        a1 = connT.EV(deR, olr(p,1));
        a2 = connT.EV(deR, olr(p,2));

        b1 = connT.EV(deR, olr(p,3));
        b2 = connT.EV(deR, olr(p,4));


        % lines points/vects:
        
        A(:,1) = THC.Av(a1,:);
        x(:,1) = THC.Av(a2,:);
        u = x-A;

        B(:,1) = THC.Av(b1,:);
        x(:,1) = THC.Av(b2,:);
        v = x-B;

        h = go2547_dBtwn2Lines (A,u, B,v);

        if h < dmin
            dmin = h ;
            ps = p;
        end

    end

    % 3.3 b.- Local and global refs of selected nodes and lines:

    r = olr(ps,:); 

    gN = [  connT.EV(deR,r(1)) , connT.EV(deR,r(2)) ;
            connT.EV(deR,r(3)) , connT.EV(deR,r(4)) ] ;


    gL = zeros(2,1);

    for l = 1:1:6

        r = connT.EL(deR,l);

        n1 = connT.LV(r,1);
        n2 = connT.LV(r,2);

        if (n1 == gN(1,1) && n2 == gN(1,2)) || (n2 == gN(1,1) && n1 == gN(1,2))
            gL(1) = r ;
        elseif (n1 == gN(2,1) && n2 == gN(2,2)) || (n2 == gN(2,1) && n1 == gN(2,2))
            gL(2) = r ;
        end
    end



    % 3.3 c.- Closest point to both lines:

    N = 2 ;

    a = zeros(N,pudim); % points of lines
    v = zeros(N,pudim); % vectors of lines

    a(1,:) = THC.Av(gN(1,1),:);
    a(2,:) = THC.Av(gN(2,1),:);

    v(1,:) = THC.Av(gN(1,2),:) - THC.Av(gN(1,1),:);
    v(2,:) = THC.Av(gN(2,2),:) - THC.Av(gN(2,1),:);

    [CP] = go2523_P_to_nLines (a, v, N, pudim);


    
    % 3.3. d.- Projection of CP on both lines and its position w.r.t. ends of segment:

    CPp = zeros(2,pudim);
    ON  = zeros(2,1); % projection ON segment or not
    pD = zeros(2,1); % percejage of distance to end of segment, w.r.t. segment length

    tolL = 0.5 ;

    for r = 1:1:2

        A(:,1) = THC.Av(gN(r,1),:);
        B(:,1) = THC.Av(gN(r,2),:);
        x(:,1) = v(r,:);

        [~, d] = go2527_distPtoRecta (CP, A, x, pudim);

        tmp = CP + d ;

        CPp(r,:) = tmp(:,1);

        ON(r,1) = go2520_PonSegment (tmp, A, B, pudim, tolL);

        dmin = min(norm(tmp-A), norm(tmp-B));

        pD(r,1) = 100*dmin / norm(B-A);

    end



    % 3.4 e.- Determines if is cap or sliver:

    % If both projections CPp are within the segment and not more than 20% far
    % away from ends, then it is is SLIVER, otherwise it is CAP.

    pDLim = 20 ;

    if sum(ON)==2 && (pD(1) > pDLim && pD(2) > pDLim)
        
        DT = "SL";
        cf = struct( 'gN', [],  'gL', [], 'CPp', [] );
        
        cf.gN = gN ;
        cf.gL = gL ;
        cf.CPp = CPp ;
        
    else
        
        DT = "CA";
        
    end


    
end



end
