classdef GenElem  < handle
    
    % general element
    
    
    properties 

        nSe ;           % No. sub-elements (for integration purpose in T-splines influenced eleents)
        
        mTrim;          % trimmed / not : 1 / 0
        mNBsk;          % No. attached trimming boundary skins (in case if trimmed element)
        mBskR;          % references of trimming bound. skins
        
        mNicp;          % No. influential cp's
        mRicpA;         % Abs. ref's of icps (Nicp)
        
        % only for rect elements:
        mIcoLi ;        % indexes limits of the element (udim,2)
        mPcoLi ;        % parameter limits of the element (udim,2) 
        mVspan;         % indicates void span (0) or not (1) in each direction , (udim)
        
        % integration edge (with two gen points)
        
        nIed ;      % No. de edges
        mIed = IEdge; % each object possess mP
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        % these are referred to paramenter mesh element ('filled elements)
        
        Ntnod ;         % T-mesh nodes affected by that element
        mTnod ;         % T-mesh affected nodes references
        
        Ntele ;         % T-mesh elems affected by that element
        mTele ;         % T-mesh affected elems references
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        % only for mTyp = 1 - analysis elements
        
        NGP;           	% No. Gauss points
        mGr ;           % GP's references
        
        % mGP = GenPoint; % Gauss points array
        
        ON ;            % initially 1, if out of domain becomes 0
        
        dJ2; 
        
        mAe;            % area of the element in physical space

        conW ;          % connection matrix element-wires (3) , 3 wires
        

        
    end
    
    methods 
        
 
        %function obj = GenElem
            
            % constructor:
           
            
        %end
        
        function cor =  cornersIcoor (obj, udim)
            
            % RETURNS: cor(Ncor,udim)
            % corner index coordinates of element
            % arranged according to hierarchy of udim directions
            
            Ncor = 2^udim;

            cor = zeros(Ncor,udim);

            if udim==2
            
                Nc=1;
                for i=1:1:2
                for j=1:1:2  
                    
                    cor(Nc,1) = obj.mIcoLi(1,i);
                    cor(Nc,2) = obj.mIcoLi(2,j);
                    
                    Nc = Nc+1;
                end
                end
            
            
            elseif udim ==3
                
                Nc=1;
                for i=1:1:2
                for j=1:1:2  
                for k=1:1:2 
                    
                    cor(Nc,1) = obj.mIcoLi(1,i);
                    cor(Nc,2) = obj.mIcoLi(2,j);
                    cor(Nc,3) = obj.mIcoLi(3,k);
                    
                    Nc = Nc+1;
                end
                end
                end

            end
            
        end

        
          
        function cor =  cornersPcoor (obj, udim)
            
            % RETURNS: cor(Ncor,udim)
            % corner param coordinates of element
            % arranged according to hierarchy of udim directions
            
            Ncor = 2^udim;

            cor = zeros(Ncor,udim);

            if udim==2
            
                Nc=1;
                for i=1:1:2
                for j=1:1:2  
                    
                    cor(Nc,1) = obj.mPcoLi(1,i);
                    cor(Nc,2) = obj.mPcoLi(2,j);
                    
                    Nc = Nc+1;
                end
                end
            
            
            elseif udim ==3
                
                Nc=1;
                for i=1:1:2
                for j=1:1:2  
                for k=1:1:2 
                    
                    cor(Nc,1) = obj.mPcoLi(1,i);
                    cor(Nc,2) = obj.mPcoLi(2,j);
                    cor(Nc,3) = obj.mPcoLi(3,k);
                    
                    Nc = Nc+1;
                end
                end
                end

            end
            
        end

        
        
        function kL = LocalStiffnessSubK (obj,a,b, xdim,udim, NGPe, mGP,thk, dfract, addNL)

            % calculates sub-matrix (xdim,xdim relating cp's a-b)

            % INPUTS:
            % a,b:  basis functions references, correspondant to dRgp_ph references
            % xdim:
            % NGPe: No. Gauss points
            % we:   weight of Gp's (array)
            % thk:  thickness, for 3D cases assume = 1
            % addNL:    if 'yes' adds KNL stiffness matrix
            
            % 1.- Sizes matrices and const. matrix of the element:
            
            if xdim==2
                Bfil =  3 ;
            elseif xdim==3
                Bfil =  6 ;
            end


            kL  = zeros(xdim,xdim); % to answer
            kLg = zeros(xdim,xdim); % for storing kL at each GP and add to kL
            kLgNL = zeros(xdim,xdim);
            
            % deformation matrices for each cp:
            
            Ba = zeros(Bfil,xdim); % standards
            Bb = zeros(Bfil,xdim);
            
            if strcmp(addNL,'yes')==true
                BaNL = zeros( 2*xdim, xdim ); % with NL
                BbNL = zeros( 2*xdim, xdim );
            end
            
            % 3.- Loop on GP's to carry out the summation:
            
            dRx = zeros(obj.mNicp,xdim);
            
            for g=1:1:NGPe
                
                d = mGP(g).mVsc(3);
                
                if d > dfract
                    
                    kLg = zeros(xdim,xdim);
                    cte = 0 ;
                    
                else
                
                    % extract dRx for that Gp:
                    dRx(:,:)= mGP(g).dR_fi(:,:);

                    detJ1 = mGP(g).dJ1;
                    detJ2 = mGP(g).dJ2;
                    
                    % 3.1.- B matrix for cp a and b:
                    
                    Nia=1;
                    
                    ia(1) = a;
                    ib(1) = b;
                    
                    Ba = GEN_1001_Bmatrix( Nia, ia , dRx, xdim);
                    Bb = GEN_1001_Bmatrix( Nia, ib , dRx, xdim);
                    
                    if strcmp(addNL,'yes')==true
                        BaNL = GEN_1010_BNLmatrix( Nia, ia , dRx, xdim);
                        BbNL = GEN_1010_BNLmatrix( Nia, ib , dRx, xdim);
                    end
                    
                    % 3.2.- Computes kLg for that GP:
                    
                    pD = mGP(g).mD;

                    if udim==2
                        
                        D = zeros(3,3) ;
                        for i=1:1:2
                        for j=1:1:2
                            D(i,j) = pD(i,j);
                        end
                        end
                        
                        D(1,3) = pD(1,4); D(2,3) = pD(2,4); D(3,3) = pD(4,4); 
                        D(3,1) = pD(4,1); D(3,2) = pD(4,2); 
                        
                    elseif udim==3
                        
                        D = pD ;
                        
                    end

                    kLg = Ba' * D ;
                    kLg = kLg * Bb ;

                    
                    % 3.3.- Coputes kLgNL for that GP:
                    
                    if strcmp(addNL,'yes')==true
                        
                        tm = GEN_1011_tMatrix( mGP(g), xdim );

                        kLgNL = BaNL' * tm ;
                        kLgNL = kLgNL * BbNL ;
                        
                    end
                    
                    % 3.4.- constant, depending on the GP:

                    cte = (detJ1 * detJ2 * mGP(g).mw * thk) ;
                
                end
                
                % .- Add to kL:
                kL = kL + (kLg + kLgNL).* cte  ;
                
            end % loop on GP's

        end
        
        
        
        
    end
    
end

