classdef GenLinSolver_DEL < handle

    % solves linear problems
    
    
    properties
        
        % input variables:
        mPatches    = GenPatch();
        mBoCurves   = GenBoEntity();            % boundary curves (object)

        
        % intermediate computed variables:
        mK ;        % stiffness matrix (total considering all ACTIVE cp's of all patches)

        mFextTOTAL; % vector of total external force
        mPatchPos;  % array with 1st position of each patch in K
        mN ;        % size of stiffness matrix with no constraints
        mNup ;      % size of stiffness matrix updated with constraints (can change when adding constraints)
        mBCk; mBCf; % matrix and vector containing coupling and Dirichlet Bc's
        
    end
    
    
    methods
        

 
     	function ComputesBCs (obj, DirBC,CouBC, NGPe, pxdim,eudim,thk, nop, nTs, insR, DoF)
            
            % Computes rows to add to K and F due to BC's
            % and stores in mBCk and mBCf matrix and vector respectively
            
            % INPUTS:
            % NGPe:     No. GP's for the entity integration in each direction
            % nop:      No. patches
            % nTs:      No. time steps
            % insR:     instance reference where take the info from
            
            % 0.- Starts mBCk and mBCf and provides weights of Gauss points for boundary:
            
            obj.mBCk = zeros(2,2);
            obj.mBCf = zeros(2,1);
            [~, Wk] = ip2301_GP_1D (NGPe); % Wk(g): Gp's weights for numerical integration
            
            
            
            % 1.- Loop on patches:
            
            for pa=1:1:nop
                
                % patch object pass to Patch for confortability
                Patch = obj.mPatches(pa);
                
                % 2.- Loop on boundary curves of the patch
                %     to detect which one has BC's:
                
                Nbc = size(Patch.BoCu,1);

                
                for bc = 1:1:Nbc

                    % entity object is transferred to bo_curve for confortability
                    entiRef = Patch.BoCu(bc,1);
                    bo_entity = obj.mBoCurves(entiRef);
                    
                    % curve Jacobian 2, one per element: J2(Nee)
                    dJ2 = bo_entity.dJ2;
                    
                    % No. elements:
                    nEbc = bo_entity.Nee;
                    
                    % conditions attached to that curve, if =0 no condition
                    DBC = bo_entity.mDirCon;
                    COU = bo_entity.mCoup  ;

                    % 3.- Dirichlet BC:
                    if DBC ~= 0  
                        IncDirKF(obj,Patch,pa, bc,nEbc, DirBC, DBC, dJ2, Wk, NGPe, thk, pxdim, eudim, insR, DoF);
                    end 

                    
                    % 5.- Coupling:
                    if COU ~= 0  

                        % patches list for that coupling condition:
                        patCou = CouBC(COU).mPatches;
                        NoPatCou = size(patCou,1);
                        
                        % check that is the master patch (= 1st in the BC_cou().mSurface() array
                                                
                        if patCou(1) == pa % 1st surface ref = patch ref. => carries on

                            IncCouKF(obj,obj.mPatches,pa, bc,nEbc, obj.mBoCurves,...
                                CouBC, COU, NoPatCou,dJ2, Wk, NGPe, thk, nTs, pxdim, eudim, insR, DoF);
                        end
                        
                    end

                    
                end % loop on boundary curves

            end % loop on patches
            

        end

        function ConstructKfromP (obj, nop, xdim, udim, insR, Anty, yLims,OrtElas, dfract, thk)
          
            % construct stiffness matrix K but looping on Gp's, not in
            % elements
            
            % INPUTS:
            % nop:      No. patches
            % xdim:     No. dims in physical space
            % udim:     No. dims in parameter space
            % insR:     instance reference
            % Anty:     analysis type. (1) p. stress, (2) p. strain, for 2D
            % En,nu:    elastic constants
            % thk:      thickness
            
            
            wb = waitbar(0,'calculating K with no constraints');
            set(wb,'Name','K computation');
            wbPos = [10 250 300 70]; % left bottom wide height
            set(wb,'position', wbPos);

            
            % 0.- Extracts data from patch:
            
            Instances(nop) = GenInstance;
            Meshes(nop) = Tmesh;
            
            for pa=1:1:nop
            Instances(pa) = obj.mPatches(pa).mRefin(insR);
            Meshes(pa) = obj.mPatches(pa).mRefin(insR).mTM;
            end
            
            
            % 1.- Initiates temporal variable K:
            %     (we will work with K just for comfortability)
            
            N = obj.mN ;
            obj.mK = zeros(N,N);
            K      = zeros(N,N);
            
            
 
            % 2.- Calculates each gp contribution to each cp and stores in K:
            
            for pa=1:1:nop % Loop on patches
                
                nGP  = Meshes(pa).nGP ; % No. Gauss points in that patch
                mGP  = Meshes(pa).mGP ; % Gp's object 
                
                
                % 2.1.- Computes constit. matrix for that patch => NOW IS IN EACH ELEMENT:
                
                %obj.mPatches(pa).ConstitutiveMatrix(xdim,udim, Anty, Ee, nu);

                
                % 2.2.- Loop on  Gauss points:
                
                wbE = waitbar(0, strcat ('calculating K with no constraints, patch ',num2str(pa)));
                set(wbE,'Name','elements integration');
                wbEPos = [10 150 300 70]; % left bottom wide height
                set(wbE,'position', wbEPos);
            
                for g = 1:1:nGP

                    Nicp = mGP(g).mNicp;

                    for a = 1:1:Nicp % loop on icp's of the point

                        % absolute position of cp:
                        cpPosa = mGP(g).mRicpA(a);
                            
                        % references from cpRef matrix, this will provide 
                        % the position in the global K matrix
                        cpR1 = Instances(pa).mcpRef(cpPosa);

                        % local positions in the patch stiffness matrix:
                        fil = GEN_11_LocalPosition_Ref(cpR1, xdim);

                        % 1st submatrix component positions in the global stiffness matrix:
                        fil = fil + obj.mPatchPos(pa) - 1 ;

                        
                    for b = 1:1:Nicp % (same steps as cp "a")

                        cpPosb = mGP(g).mRicpA(b);
                        cpR2 = Instances(pa).mcpRef(cpPosb);
                        col = GEN_11_LocalPosition_Ref(cpR2, xdim);
                        col = col + obj.mPatchPos(pa) - 1 ;

                        % computes Elem local stiffness sub-matrix for cp's a-b: 
                        addNL = 'no' ; % 'yes' ; % considers geom. non-linearities, otherwise 'no'
                        
                        kL = mGP(g).LocalStiffnessSubK (a,b, Anty, xdim,udim, thk, dfract, addNL);
                        
                        % stores kL values with in K:
                        for i=0:1:xdim-1
                        for j=0:1:xdim-1

                           K(fil+i,col+j)= K(fil+i,col+j) + kL(1+i,1+j);

                        end
                        end
                        
                    end
                    end

                    % progress bar for elements
                    wbEPos = [10 150 300 70]; % left bottom wide height
                    set(wbE,'position', wbEPos);
                    waitbar(g/nGP,wbE, strcat('integration point = ',num2str(g+1))) ; 

                    clearvars Elem ;

                end  % loop on Gauss points
                
                delete (wbE);
                
                % progress bar for patches
                waitbar(pa/nop,wb, strcat('patch= ',num2str(pa+1))) ; 
            
                
                clearvars nGP mGP ;
                
            end % loop on patches
            

            
            % 3.- Transfers to the object:
            
            obj.mK = K ;
            
            delete (wb);
            
            
            
            

            
        end
        

        %%% specific BC's functions: %%%%%%%%%%%%%%%%%
        
        function IncDirKF(obj,Patch,pa, bc, nEbc,DirBC, DBC,dJ2, Wk, NGPe, thk, pxdim, eudim, insR, DoF)
           
            % This function extend mBCk matrix and mBCf vector considering
            % Dirichlet BC's.
            % mBCk and mBCf are object attributes, and can be accessed from
            % here directly
            
            % INPUTS:
            % Patch:    where the Bo_entities are
            % NGPe:     No. gp's of the bo_entity
            % pxdim:    patch physical No. coord's
            % eudim:    bo_entity parameter No. coord's
            % insR:     instance ref. where extract info from
            
            % Instance to work with:
            Refin = Patch.mRefin(insR);
            
            % Computes Constrained DoF:
            cDoF=0;
            for di=1:1:pxdim
                if isnan(DirBC(DBC).md(di))==0 % Dmd
                    cDoF = cDoF + 1 ;
                end
            end
            

            % No. rows to add to K : na = Constrained DoF x Nicp on that curve
            Nicp = Refin.NCurCp(bc); % bc = curve local reference
            na = cDoF * Nicp;
                        
            % prepares matrix and vector to add to K and F
            DirF = zeros ( na , 1 );
            DirK = zeros ( na , DoF); % note: mNup is initiated as mN in constructK method
            
  
            % aggregates in a single WkG(totalNGP) all the Gp's weigths
            WkG = GEN_19_agregatesWgp (Wk, NGPe, eudim);
            
            
            % loops on icp's of the curve (Lr = local ref. of the cp)
            for Lr1 = 1:1:Nicp
                
                % DirF components:
                
                % integrates DirF matrix factor:
                Acc=0;
                for e=1:1:nEbc
                    J2 = dJ2(e);
                    for g = 1:1:NGPe^eudim
                        
                        J1 = Refin.dJ1Egp(bc,e,g); 
                        R1 = Refin.RatEgp(bc,e,g,Lr1);
                        
                        Acc = Acc + WkG(g) * J2 * J1 * R1 ;
                    end
                end


                % rows ref's to add
                filR = zeros(pxdim,1) ; 
                diResta=1;
                for di=1:1:pxdim
                if isnan(DirBC(DBC).md(di))==0  % Dmd(di)
                        filR(di) = cDoF*Lr1+(diResta-cDoF);
                        diResta=diResta+1;
                end
                end
                
                
                % stores factor at the correspondant DirF vector position:
                for di=1:1:pxdim
                if isnan(DirBC(DBC).md(di))==0 % Dmd(di)
                   DirF (filR(di) , 1) = Acc * thk * DirBC(DBC).md(di) ;   % Dmd(di)  
                end
                end
            
                %for di=1:1:cDoF
                    %DirF (cDoF*Lr1+(di-cDoF) , 1) = Acc * thk * DirBC(DBC).Dmd(di) ;
                %end
                

                for Lr2 = 1:1:Nicp 
                    
                    cpAbsRef = Refin.CurCpA(bc,Lr2);   % cp absolute reference
                    cpPos = Refin.mcpRef(cpAbsRef);     % position of active cp's in the patch

                    col = GEN_11_LocalPosition_Ref(cpPos, pxdim); % col. position within the patch
                    col = col + obj.mPatchPos(pa) - 1 ; % col. position with in the whole system (K)

                    % integrates DirK matrix factor:
                    Acc=0;
                    for e=1:1:nEbc
                        J2 = dJ2(e);
                        for g = 1:1:NGPe^eudim

                            J1 = Refin.dJ1Egp(bc,e,g); 
                            R1 = Refin.RatEgp(bc,e,g,Lr1);
                            R2 = Refin.RatEgp(bc,e,g,Lr2);

                            Acc = Acc + WkG(g) * J2 * J1 * R1 * R2 ;
                        end
                    end
            


                    % stores factor at the correspondant DirK matrix position:

                    for di=1:1:pxdim
                    if isnan(DirBC(DBC).md(di))==0 % Dmd(di) 
                        DirK( filR(di) , (col + (di-1)) ) = Acc * thk ;
                    end
                    end
                    
                    %for di=1:1:pxdim
                    %DirK(pxdim * Lr1 + (di-pxdim) , (col + (di-1)) ) = Acc * thk ;
                    %end

                end  
                
            end % loop on icp's
            

            
            % updates mBCk and mBCf:
            updatesBC(obj, DirK, DirF, na);
            

        end
        
        function IncCouKF(obj,Patches,maPaRef, bc,nEbc, B_curves,CouBC, COU, NoPat,dJ2, Wk, NGPe, thk, ns, pxdim, eudim, insR, DoF)
           
            % This function extend mBCk matrix and mBCf vector considering
            % coupling conditions
            % mBCk and mBCf are object attributes, and can be accessed from
            % here directly
            
            % INPUTS:
            % Patches:  all patches (needed coz they are related to each other)
            % maPaRef:  master patch reference
            % NoPat:    total No. patches attached by the coupled constraint
            
            % No. rows to add to K : na = xdim x Nicp on that bo_entity from master patch
     
                        
            % Master instance to work with:
            maRefin =  Patches(maPaRef).mRefin(insR);
            
            Nicp = maRefin.NCurCp(bc);
            na = pxdim * Nicp;

            
            % loop on No. slave patches = No. conditions
            for spat = 2:1:NoPat
            
            % prepares matrix and vector to add to K and F
            CouK = zeros ( na , DoF); % note: mNup is initiated as mN in constructK method
            CouF = zeros ( na , 1 ); % "b", which is equal to Integral(-RRx_A+RRx_B), that is , positions of icp

            
            % aggregates in a single WkG(totalNGP) all the Gp's weigths
            WkG = GEN_19_agregatesWgp( Wk, NGPe , eudim ); 

            
            % some aux. arrays
            x = zeros(pxdim,1);
            
            %%%%% MASTER PATCH %%%
             
            %Mpatch = Patches(maPaRef);
            
            
            % loops on curves's icp's from master patch (Lr = local ref. of the cp)
            for Lr1 = 1:1:Nicp
            for Lr2 = 1:1:Nicp 
                    
                cpAbsRef = maRefin.CurCpA(bc,Lr2);  % cp absolute reference
                cpPos = maRefin.mcpRef(cpAbsRef);     % position of active cp's in the patch

                for di=1:1:pxdim
                    x(di) = maRefin.mA(cpAbsRef).mXcoor(di); % master icp physical coordinates
                end
                

                col = GEN_11_LocalPosition_Ref(cpPos, pxdim); % col. position within the patch
                col = col + obj.mPatchPos(maPaRef) - 1 ; % col. position with in the whole system (K)
         
                % integrates CouK matrix factor:
                Acc=0;
                for e=1:1:nEbc
                    J2 = dJ2(e);
                    for g = 1:1:NGPe^eudim
                        
                        J1 = maRefin.dJ1Egp(bc,e,g); 
                        R1 = maRefin.RatEgp(bc,e,g,Lr1);
                        R2 = maRefin.RatEgp(bc,e,g,Lr2);
                        
                        Acc = Acc + WkG(g) * J2 * J1 * R1 * R2 ;

                    end
                end

                                                     

                % stores factor at the correspondant CouK matrix & CouF vector position:
                
                for di=1:1:pxdim
                    
                    CouK(pxdim * Lr1 + (di-pxdim) , (col + (di-1)) ) = Acc * thk ;
                    
                    %CouF(pxdim * Lr1 + (di-pxdim),1) = CouF(pxdim * Lr1 + (di-pxdim),1) - Acc * thk * x(di); %  large displ.
                    
                    %CouF(pxdim*Lr1+(di-pxdim),1) = 0 ; 
                end
                                
            end  
            end % loop on icp's
            
          
            
            %%%%% SLAVE PATCH %%%
            
            svPaRef = CouBC(COU).mPatches(spat);  % slave patch ref
            Spatch = Patches(svPaRef) ;
            svRefin = Spatch.mRefin(insR);
            

            % finds the curve of the slave patch with the coupling BC:
            
            Nbc = size(Spatch.BoCu,1); % No. b_curves of slave patch
            
            for bcS=1:1:Nbc
                bcRef = Spatch.BoCu(bcS); % bo_cur ref.
                if B_curves(bcRef).mCoup == COU % that b_curve has the Coup.
                    break % bcRef / bcS is the curve to work with
                end
            end
            

            % icp's from slave patch on that bcRef bo_curve
            NicpS = svRefin.NCurCp(bcS);

            % loops on curves's icp's from slave patch (Lr = local ref. of the cp)
            for Lr1 = 1:1:Nicp 
            for Lr2 = 1:1:NicpS 
                
                cpAbsRef = svRefin.CurCpA(bcS,Lr2);  % cp absolute reference
                cpPos = svRefin.mcpRef(cpAbsRef);     % position of active cp's in the patch

                for di=1:1:pxdim
                   x(di) = svRefin.mA(cpAbsRef). mXcoor(di); % slave icp coordinates 
                end
                
                col = GEN_11_LocalPosition_Ref(cpPos, pxdim); % col. position within the patch
                col = col + obj.mPatchPos(svPaRef) - 1 ; % col. position with in the whole system (K)
                
                % integrates CouK matrix factor:

                e=1; % revise THIS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                R2t = zeros(NGPe^eudim,1);
                for g = 1:1:NGPe^eudim
 
                    R2t(g) = svRefin.RatEgp(bcS,e,g,Lr2);
                end

                if eudim==1
                
                    % R2t vector needs to be inverted in the slave to
                    % coincide with master at Gp's
                    R2t = R2t(end:-1:1);
                    
                elseif eudim==2
                    
                    %R2t(:) = Spatch.RatEgp(bc,e,:,Lr2); % slave bf's
                    %matrix needs rearrangement, see general criteria
                    [R2t] = GEN_23_transposeVector( R2t , NGPe );
                    
                end
                
                Acc=0;
                for e=1:1:nEbc
                    J2 = dJ2(e);
                    for g = 1:1:NGPe^eudim
                        
                        J1 = maRefin.dJ1Egp(bc,e,g); 
                        R1 = maRefin.RatEgp(bc,e,g,Lr1);
                        R2 = R2t(g);  %%  rearranged 

                        Acc = Acc - WkG(g) * J2 * J1 * R1 * R2 ;
                    end
                end
 
                
                % stores factor at the correspondant CouK matrix & CouF vector position:
                % for CouF: add to "b" the 2nd force vector components:
                for di=1:1:pxdim
                    
                    CouK(pxdim * Lr1 + (di-pxdim) , (col + (di-1)) ) = Acc * thk ;
                    %CouF(pxdim * Lr1 + (di-pxdim),1) = CouF(pxdim * Lr1 + (di-pxdim),1) - Acc * thk * x(di);
                    %CouF(pxdim*Lr1+(di-pxdim),1) = 0 ;
                end
                
            end  
            end % loop on icp's
            
     
            
            % updates mBCk and mBCf:
            updatesBC(obj,CouK, CouF, na);
            
           
            end  % loop on slaves surfaces ( = No. conditions)
            
        end
       
        function updatesBC (obj, bcK, bcF, na)
            
         	% used for updating mBCk and mBCf, in case of coupling and Dirichlet BC's:
  
            c = size(obj.mBCk,2);
            f = size(obj.mBCf,1);
            
            if c == 2 % first condition => initiates the matrix
                
                c = size(bcK,2);
                f = na ;
                
                obj.mBCk = bcK ;
                obj.mBCf = bcF ;
                
            else  % there are conditions already stores => stores after them
                
                obj.mBCk = [ obj.mBCk ;
                             bcK    ];
                         
                obj.mBCf = [ obj.mBCf ;
                             bcF    ];
            end
            
            
        end
        
        
    end % end of methods
    
    
    
end

