classdef Rmesh < Mesh

% Stores mesh data of one patch
    
properties
    
    nNOi;  	% No. nodes of the mesh (index space)
    nELi;    % No. elements in total, i/ void. = No. elem's in index space
    FtoIe;  % (nELf) matrix that stores index elements ref. to a given filled elem't ref.
    FtoIn;  % (nNO ) matrix that stores index nodes ref. to a given iparam. node ref.
    
    
    mNi = GenGeomPoint();   % (nNOi), nodes in index space
    
    % wires for index mesh:
    nWIi;             
    mWi = GenWire();  
    
   	% wires connectivity matrices
    conEWi; % (nELi, wires per element)
    conWEi; % (nWIi, elements per wire)
    conWNi; % (nWIi, 2) nodes from mNi at the end of the wire
    

    % conn. matrices for index space
    conENi; 
    conENSi;


    
    
    % effective knot vector: if 0 no edge, if number, the number itself is
    % the knot value. It is thought for 2D index space Scott algorithm
    effKVI = ILine();
    % e.g.: 1 2 0 4 5 0 7, there are no edges at 3rd and 6th positions
    

    
end
    
methods
    
    function obj = Rmesh()
    % constructor by default 
       
        %obj.mW = WIRE(); 
      
    end
    
    
    
    
    function allocates_nodCoor(obj,Nonod,nodesP, uD,xD, KnotV, w,B, Ncp, Degree,Usize )
        % provides nodes coordinates matrices
        
        % INPUTS:
        % Nonod:    No. nodes
        % nodesP:   matrix with nodes param. coordinates
        % uD,xD:    dims in parameter and physical space
        % uD,... :  patch features
        
        
        % 1.- Provides sizes:
        obj.nodU = zeros (Nonod,uD);
        obj.nodX = zeros (Nonod,xD);
        P = zeros(uD,1);
        
        
        % 2.- Expand cp's coord's:
        [Ew,EB] = GEN_12_ExpandBw( w,B,xD, Ncp );
        
        
        % 3.- Populates:
        for node = 1:1:Nonod
            
            for di=1:1:uD
                obj.nodU (node,di) = nodesP(node,di);
                P(di) = nodesP(node,di);
            end
            
            [X] = GEN_03_01_Solid_Surf_phys_coord( P, KnotV, Ncp,Degree,Usize, Ew,EB, uD,xD);
            
            for di=1:1:xD
                obj.nodX (node,di) = X(di);
            end  
        end
        
        
        
    end
    
    
    function allocates_conM(obj,Nne,nE, conn)
       % allocates connectivity matrix (nE, nodesRef) 
        
       % INPUTS:
       % Nne:   No. nodes per element
       % nE:    No. elements
       % conn:  connectivity matrix
       
       % 1.- Provides size:
       obj.conM = zeros(nE,Nne);
       
       % 2.- Populates matrix:
       obj.conM = conn ;
       
    end
    
    
    function allocates_elems(obj)
        % inizializtes element matrix and values for them
        obj.mE(obj.nEL) = TriangleTwoDimsElement();
    end
    
end
    
end

