function [rEskin, NcvE, cLUvE, cLXvE] = st0303_iniTriang ...
    (Spatches, Eskin, agCurvE, Nbe, pRt, sRt, CPcon, nloE, SCconE, ncuE, tolRc, tolRs, Xtol, DUtol, ndig, tolD, DerC)

% RETURNS:
% rEskin(Nbe,1); triangulation with initial Rt, prepared to be refined to quadratic approximation

pudim = 3 ;
eudim = 2 ;
cudim = 1 ;
xdim = 3 ;

TNcuE = size(agCurvE,1);

cLXvE = zeros(TNcuE,2,xdim); 
cLUvE = zeros(TNcuE,2,cudim);  
%cAvE  = zeros(TNcuE,2,pudim); 
NcvE  = zeros(TNcuE,1);


curveError = 'no';
tolC = 0 ;




% 1.- CONTOUR CURVES DISCRETIZATION:

Nvx = 1 ; % sets min. number of divisions in the curves to one

for c = 1:1:TNcuE
 
	% 1.1.- Extracts NURBS features of curve and patch:

    cNcp    = agCurvE(c,1).Ncp ;
	cDegree = agCurvE(c,1).Degree ;
    cUsize  = agCurvE(c,1).Usize ;
  	cKnotV  = agCurvE(c,1).KnotV ;
  	cB      = agCurvE(c,1).P ;
 	cw      = agCurvE(c,1).w ;

  	pR = CPcon(c,1);
    R  = pRt(pR,1);
    
	pNcp     = Spatches(pR,1).INS(1).mNcp ;
    pDegree  = Spatches(pR,1).INS(1).mDegree ;
    pUsize   = Spatches(pR,1).INS(1).mUsize ;
    pKnotV   = Spatches(pR,1).INS(1).mKnotV ;
    pP       = Spatches(pR,1).INS(1).mP ;
    pw       = Spatches(pR,1).INS(1).mw ;
    pCD      = [ 0 0 0 ];
    
	% knot vector has to be row vector:
    if size(cKnotV,2)==1
        cKnotV = cKnotV';
    end
    
	% 1.2.- Physical, param. (curve space) and param. (patch space) coord's:
       
    [cU,cX,~,~] = ts3315_oneCvtxs (R, Nvx, cNcp, cDegree, cUsize, cKnotV, cB, cw, xdim, cudim, tolRc);

    ccv = size(cX,1); % No. vtxs of the contour curve
    

	%[cA] = v2861b_PPvol (cX, ccv, pDegree, pNcp, pKnotV, pUsize, pP, pw, pCD, xdim, Xtol, DUtol);


    % 1.3.- Allocates to response:
    
    NcvE(c,1) = ccv ;
    
    for v = 1:1:ccv 
        cLXvE(c,v,:) = cX(v,:);
        cLUvE(c,v,:) = cU(v);
        %cAvE(c,v,:) = cA(v,:);
    end
 
end




% 2.- SURFACE DISCRETIZATION: TRIANGULATION


rEskin(Nbe,1) = cEskin ;

for b = 1:1:Nbe
    rEskin(b,1) = Eskin(b,1);
end



for s = 1:1:Nbe

    % 3.1.- Surface features:
    
    R = sRt(s,1);
    
    eswP = 30 ;
    esw = eswP/100 * R ;
    
    sNcp    = Eskin(s,1).mNcp;
    sDegree = Eskin(s,1).mDegree;
    sUsize  = Eskin(s,1).mUsize;
    sKnotV  = Eskin(s,1).mKnotV;
    CS     = Eskin(s,1).mCS;
    sP     = Eskin(s,1).mP;
    sw     = Eskin(s,1).mw;
    [sEw,sEP] = GEN_12_ExpandBw (sw, sP,xdim, eudim, sNcp);

    sULims = [ sKnotV(1,1) sKnotV(1,sUsize(1))  ;  sKnotV(2,1) sKnotV(2,sUsize(2))];
    
    NL = nloE(s); % No. loops
    
	% 3.2.- Contours coord's in terms of surface: Note only selects curves
    % relevant to that surface
    
    % Surface contour edges: starts by 0,0 and moves counter-clock-wise:
    % they are numbered as 1 to 4 edges with that order

    [edCurv] = ts3313_EdgesContours (sNcp, sDegree, sUsize, sP,sw, sKnotV);

    % edCurv(4) = struct ('Ncp',0,  'Degree',0, 'Usize',0, 'KnotV',[], 'w',[], 'P',[], 'conF', 0, 'CD', 0);

    % 2.- LOOPS ON SURFACE LOOP TO COMPUTE CONTOUR VERTEXES
    
    [cXv, cUv, Nlv, Nlcv] = rs3336b_loops ...
    (edCurv, cLXvE, cLUvE, NcvE, SCconE, ncuE, nloE, agCurvE, s, sDegree, sNcp, sKnotV, sUsize, sP, sw, sEP, sEw, CS, eudim, xdim, Xtol, DUtol);



    % 3.- CLOSED SURFACE CHECK:

    if sum(CS) > 0
        [cUv] = su2914_CSuLims (cUv, sKnotV, sUsize, nloE(s), Nlv, CS, eudim);
    end

    
	% 3.1.- Edge contour loop for that surface alone: 'unique surface' = usf
    
    usf = struct('Ncp',[], 'Degree',[], 'Usize',[], 'KnotV',[], 'EP',[], 'Ew',[], 'P',[],'w',[], 'CS',[0 0]); % un-trimmed surface feat.
    
	usf.Ncp     = sNcp;
    usf.Degree  = sDegree;
    usf.KnotV   = sKnotV;
    usf.Usize   = sUsize;
    usf.P       = sP;
    usf.w       = sw;
	usf.EP      = sEP;
    usf.Ew      = sEw;
    usf.CS      = CS;

    [ecCurv, eTNcu, eSCcon, encu, enlo, ~, ~] = ts3311_ECcurves (1, usf, xdim);
    
    [ecLXv, ecLUv, eNcv, ecRc] = ts3312_Cvertexes (ecCurv, eTNcu, R, Nvx, tolRc, xdim, tolC, curveError);
    
    [ecXv, ecUv, eNlv, eNlcv] = rs3336_cUvSurface (ecLXv, ecLUv, eNcv, eSCcon, encu, enlo, ecCurv, 1, usf, xdim);
    

	% 3.2.- Detects if it is strip or not:
    
    pt = 15 ; % percentage of deviation of var. coeff. to consider strip
    [strF, Ld] = sv3346_Strip (sNcp, sEP, R, pt, xdim, eudim);
    
    %{
    if s == 1
        
        figure(s)
        scatter3(sP(:,1), sP(:,2),sP(:,3),'r'); hold on
        
        for i = 1:1:prod(sNcp)
            text(sP(i,1), sP(i,2),sP(i,3),num2str(i)); hold on
        end

        axis equal
        
        if gg==0
        end
        
    end
    %}
    

	% 3.4.- Surface vertexes for un-trimmed surface:
    % separates vertexes inside surf. param. limits (sUv) from a bigger group of all of them (sUvt):

    [NCT, cnR, sRvt, sUvt, sXvt, inF, coF, dSmesh, sRv, sUv, sXv, Nv] = sv3340_Svertexes ...
    (s, strF, Ld, pt , sDegree, sNcp, sKnotV, sUsize, sP,sw, sEw,sEP, sULims, R, esw, xdim, eudim, ndig, tolRs,tolD, ecUv, eNlv, eNlcv, DerC);

   
    % 3.5.- Estimates pattern coordinates of contours:
    % cRv = zeros(NL,max(Nlv),udim) ;

    [cRv] = rs3337_coRcoor (sRvt, sUvt, coF, cUv, NL, Nlv, eudim, s);
    

    % 3.6.- Selects only surface coordinates that are within the loops
    % from sUv selectes them and stores in sUvi

    [sRvi, sUvi, sXvi, Nvi] = sv3344_SvforT (s, eswP, sRv, sUv, sXv, Nv, cUv, cXv, NL, Nlv, xdim, eudim);
    

    % 3.7.- Merge all coordinates and triangulates:
    
    [NE, TVcon, VTcon, LVcon, VLcon, TLcon, Rv, Uv, Xv, XNe, XNv, Ae] = rs3334_Rtriang ...
    (cRv, sRvi, cUv, sUvi, cXv, sXvi, Nvi, NL, Nlv, Nlcv, ncuE, SCconE, s, xdim);


    % 3.8.- Allocates to response:

    rEskin(s,1).mNE = NE ;
    rEskin(s,1).mTVcon = TVcon ;
    rEskin(s,1).mVTcon = VTcon ;
    rEskin(s,1).mLVcon = LVcon ;
    rEskin(s,1).mVLcon = VLcon ;
    rEskin(s,1).mTLcon = TLcon ;
    rEskin(s,1).mRv = Rv ;
    rEskin(s,1).mUv = Uv ;
    rEskin(s,1).mXv = Xv ;
    
    rEskin(s,1).mXNe = XNe ;
    rEskin(s,1).mXNv = XNv ;
    rEskin(s,1).mArea = Ae ;

    
    rEskin(s,1).mNV  = size(Xv,1);
    rEskin(s,1).mNlv = Nlv ;
    
    
    %{
    %if s == 2
    fr=1 ;
    
    NLI = size(LVcon,1);
    
    NV = size(Xv,1);
    
    pt1366_pTriLinMesh_nD ...
    (Xv, NLI, LVcon, TVcon, 3, fr, 'k', 0, NV, 0, 0, NE, 0, 0, 0, 0, 0, 1)
    
    %end
    %}
    
end





end

