function [qlF, Xmc] = th1338_quadrLine (agCurvE, Eskin, n1,n2, tNV, tNC, tNS, tCR, tSR, tUc, tUs, Xv, xdim, Xtol, DUtol)

% RETURNS:
% qlF = 1 if line is quadratic (lies on trimming surface)
% Xmc(xdim,1): mid-point computed, considering edge curve or surface

% INPUTS:
% n1,n2: nodes refs
% tNV: No. vertexes that are within the TTR

qlF = 0 ;
Xmc = 0 ;    


% If both nodes lie within the TTR, i.e. lie at rEskin, then both refs
%  are equal or less than number of TTR nodes.

if n1 <= tNV && n2 <= tNV 

    % 1.- DETECTS IF IT IS A CURVE OR SURFACE LINE:
    

	SR = 0 ; % curve and surface references, = 0 if not found
	CR = 0 ;

    
	% 1.1.- Curve line:
    
    for a = 1:1:tNC(n1)
        for b = 1:1:tNC(n2)

            if abs(tCR(n1,a)) == abs(tCR(n2,b))

                % curve ref:
                CR = abs(tCR(n1,a));
                
                % computes param space curve mid-point:
                umid = zeros(1,1);
                umid(:,1) = (tUc(n1,CR,1) + tUc(n2,CR,1))/2;
                
                break
            end  

        end
        
        if CR>0
     	break 
        end
    end


	% 1.2.- Surface line:
    
    if CR == 0 % curve has priority

        for a = 1:1:tNS(n1)
            for b = 1:1:tNS(n2)

                if tSR(n1,a) == tSR(n2,b)

                    % surface ref:
                    SR = tSR(n1,a);
                    
                  	% computes param space surf mid-point:
                    umid = zeros(1,2);
                    umid(1,:) = (tUs(n1,SR,:)+tUs(n2,SR,:))/2;
                    
                    break
                end

            end
            
            if SR>0
            break 
            end
            
        end
 
    end
        
    
    % 2.- ADDS MID-NODE:
	% Using curve or surface, whichever applies
    
    
	% average of physical coordinates of both end nodes:
    
	Xm = zeros(xdim,1);
	Xm(:,1) = (Xv(n1,:) + Xv(n2,:))/2;

    if CR > 0
          
        % 2.1.- Curve:
        
        qlF = 1 ;
        
        p = agCurvE(CR,1).Degree;
    	n = agCurvE(CR,1).Ncp;
    	U = agCurvE(CR,1).KnotV;
     	P = agCurvE(CR,1).P;
      	w = agCurvE(CR,1).w;
        cULims = [0 1];
      	cCD = agCurvE(CR,1).CD;
            
     	[u, ~] = c2815b_onePoint (Xm, umid, p, n, U, P ,w, cULims, cCD, xdim, Xtol, DUtol);
     	Xmc = b2601_BNURBS_Xcoord (u, U, n, p, n+p+1, w, P, 1, xdim);
            
            
	elseif SR > 0
            
        % 2.2.- Surface:
        
        qlF = 1 ;
        
        bDegree = Eskin(SR,1).mDegree;
        bNcp    = Eskin(SR,1).mNcp;
        bKnotV  = Eskin(SR,1).mKnotV;
        bUsize  = Eskin(SR,1).mUsize;
        bP      = Eskin(SR,1).mP;
        bw      = Eskin(SR,1).mw;
        bCD     = Eskin(SR,1).mCS;
        bULims = [ 0 1 ; 0 1 ];
            
        [bEw,bEP] = GEN_12_ExpandBw (bw, bP,xdim, 2, bNcp);

        [u, ~] = s2835b_onePoint (Xm, umid, bDegree, bNcp, bKnotV, bUsize, bP ,bw, bEP ,bEw, bCD, bULims, xdim, Xtol, DUtol);
        Xmc = b2601_BNURBS_Xcoord (u, bKnotV, bNcp,bDegree,bUsize, bEw,bEP, 2,xdim);
            
    end

end



end

